package ext {
import ext.mixin.Observable;

[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Native("Ext.ElementLoader", require)]
/**
 * Fires before a load request is made to the server.
 * Returning false from an event listener can prevent the load
 * from occurring.
 * @eventType ext.events.ElementLoader_optionsEvent.BEFORE_LOAD
 */
[Event(name = 'onBeforeLoad', type = 'ext.events.ElementLoader_optionsEvent')]
/**
 * Fires after an unsuccessful load.
 * @eventType ext.events.ElementLoader_response_optionsEvent.EXCEPTION
 */
[Event(name = 'onException', type = 'ext.events.ElementLoader_response_optionsEvent')]
/**
 * Fires after a successful load.
 * @eventType ext.events.ElementLoader_response_optionsEvent.LOAD
 */
[Event(name = 'onLoad', type = 'ext.events.ElementLoader_response_optionsEvent')]
/**
 * A class used to load remote content to an Element. Sample usage:
 *
 * <pre><code><a href="#!/api/Ext-method-get" rel="Ext-method-get" class="docClass">Ext.get</a>('el').load({
 *     url: 'myPage.php',
 *     scripts: true,
 *     params: {
 *         id: 1
 *     }
 * });
 * </code></pre>
 *
 * <p>In general this class will not be instanced directly, rather the <a href="#!/api/Ext.dom.Element-method-load" rel="Ext.dom.Element-method-load" class="docClass">Ext.dom.Element.load</a> method
 * will be used.</p>
 */
public class ElementLoader extends ext.mixin.Observable {
  /**
   * @param config @inheritDoc
   */
  public function ElementLoader(config:ext.ElementLoader = null) {
    super();
  }

  /**
   * <code>true</code> in this class to identify an object as an instantiated ElementLoader, or subclass thereof.
   * @default true
   */
  public native function get isLoader():Boolean;

  /**
   * @private
   */
  public native function set isLoader(value:Boolean):void;

  [ExtConfig]
  /**
   * Any additional options to be passed to the request, for example timeout or headers.
   */
  public native function get ajaxOptions():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set ajaxOptions(value:Object):void;

  [ExtConfig]
  /**
   * <code>true</code> to have the loader make a request as soon as it is created.
   * This argument can also be a set of options that will be passed to <a href="#!/api/Ext.ElementLoader-method-load" rel="Ext.ElementLoader-method-load" class="docClass">load</a> when it is called.
   * @default false
   */
  public native function get autoLoad():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set autoLoad(value:*):void;

  [ExtConfig]
  /**
   * Params that will be attached to every request. These parameters
   * will not be overridden by any params in the load options.
   */
  public native function get baseParams():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set baseParams(value:Object):void;

  [ExtConfig]
  /**
   * A function to be called when a load request finishes.
   * Will be called with the following config parameters:
   *
   * <ul>
   * <li>this - The ElementLoader instance.</li>
   * <li>success - True if successful request.</li>
   * <li>response - The response object.</li>
   * <li>options - Ajax options.</li>
   * </ul>
   *
   *
   * <p>&#64;controllable</p>
   */
  public native function get callback():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set callback(value:*):void;

  [ExtConfig]
  /**
   * A function to be called when a load request fails.
   * Will be called with the following config parameters:
   *
   * <ul>
   * <li>this - The ElementLoader instance.</li>
   * <li>response - The response object.</li>
   * <li>options - Ajax options.</li>
   * </ul>
   *
   *
   * <p>&#64;controllable</p>
   */
  public native function get failure():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set failure(value:*):void;

  [ExtConfig]
  /**
   * True or a string to show when the element is loading.
   * @default false
   */
  public native function get loadMask():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set loadMask(value:*):void;

  [ExtConfig]
  /**
   * Any params to be attached to the Ajax request. These parameters will
   * be overridden by any params in the load options.
   */
  public native function get params():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set params(value:Object):void;

  [ExtConfig]
  /**
   * A custom function to render the content to the element. The function should
   * return false if the renderer could not be applied. The passed parameters are:
   *
   * <ul>
   * <li>The loader</li>
   * <li>The response</li>
   * <li>The active request</li>
   * </ul>
   */
  public native function get renderer():Function;

  [ExtConfig]
  /**
   * @private
   */
  public native function set renderer(value:Function):void;

  [ExtConfig]
  /**
   * The scope to execute the <a href="#!/api/Ext.ElementLoader-cfg-renderer" rel="Ext.ElementLoader-cfg-renderer" class="docClass">renderer</a> function in.
   */
  public native function get rendererScope():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set rendererScope(value:Object):void;

  [ExtConfig]
  /**
   * The scope to execute the <a href="#!/api/Ext.ElementLoader-cfg-success" rel="Ext.ElementLoader-cfg-success" class="docClass">success</a> and <a href="#!/api/Ext.ElementLoader-cfg-failure" rel="Ext.ElementLoader-cfg-failure" class="docClass">failure</a> functions in.
   */
  public native function get scope():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set scope(value:Object):void;

  [ExtConfig]
  /**
   * True to parse any inline script tags in the response.
   * @default false
   */
  public native function get scripts():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set scripts(value:Boolean):void;

  [ExtConfig]
  /**
   * A function to be called when a load request is successful.
   * Will be called with the following config parameters:
   *
   * <ul>
   * <li>this - The ElementLoader instance.</li>
   * <li>response - The response object.</li>
   * <li>options - Ajax options.</li>
   * </ul>
   *
   *
   * <p>&#64;controllable</p>
   */
  public native function get success():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set success(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The target element for the loader. It can be the DOM element, the id or an <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a>.
   * @see #getTarget()
   * @see #setTarget()
   */
  public native function get target():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set target(value:*):void;

  [ExtConfig]
  /**
   * The url to retrieve the content from.
   */
  public native function get url():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set url(value:String):void;

  /**
   * Destroys the loader. Any active requests will be aborted.
   */
  override public native function destroy(...params):void;

  /**
   * Aborts the active load request
   */
  public native function abort():void;

  /**
   * Returns the target of this loader.
   * @return The target or null if none exists.
   */
  public native function getTarget():ext.Component;

  /**
   * Checks whether the loader is automatically refreshing. See <a href="#!/api/Ext.ElementLoader-method-startAutoRefresh" rel="Ext.ElementLoader-method-startAutoRefresh" class="docClass">startAutoRefresh</a>.
   * @return True if the loader is automatically refreshing
   */
  public native function isAutoRefreshing():Boolean;

  /**
   * Loads new data from the server.
   * @param options The options for the request. They can be any configuration option that can be specified for
   * the class, with the exception of the target option. Note that any options passed to the method will override any
   * class defaults.
   */
  public native function load(options:Object):void;

  /**
   * Sets an <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> as the target of this loader.
   * Note that if the target is changed, any active requests will be aborted.
   * @param target The element or its ID.
   */
  public native function setTarget(target:*):void;

  /**
   * Automatically refreshes the content over a specified period.
   * @param interval The interval to refresh in ms.
   * @param options The options to pass to the load method. See <a href="#!/api/Ext.ElementLoader-method-load" rel="Ext.ElementLoader-method-load" class="docClass">load</a>
   */
  public native function startAutoRefresh(interval:Number, options:Object = null):void;

  /**
   * Clears any auto refresh. See <a href="#!/api/Ext.ElementLoader-method-startAutoRefresh" rel="Ext.ElementLoader-method-startAutoRefresh" class="docClass">startAutoRefresh</a>.
   */
  public native function stopAutoRefresh():void;
}
}