package ext {
import ext.data.AbstractStore;
import ext.data.Store;
import ext.util.IStoreHolder;

[Uses("ext.Component")]
[Uses("ext.util.StoreHolder")]
[Uses("ext.util.Positionable")]
[Uses("ext.Base")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Animate")]
[Uses("ext.util.Floating")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.state.Stateful")]
[Native("Ext.LoadMask", require)]
/**
 * A modal, floating Component which may be shown above a specified <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Component</a> while loading data.
 * When shown, the configured owning Component will be covered with a modality mask, and the LoadMask's <a href="#!/api/Ext.LoadMask-cfg-msg" rel="Ext.LoadMask-cfg-msg" class="docClass">msg</a> will be
 * displayed centered, accompanied by a spinner image.
 *
 * <p>If the <a href="#!/api/Ext.LoadMask-cfg-store" rel="Ext.LoadMask-cfg-store" class="docClass">store</a> config option is specified, the masking will be automatically shown and then hidden synchronized with
 * the Store's loading process.</p>
 *
 * <p>Because this is a floating Component, its z-index will be managed by the global <a href="#!/api/Ext.WindowManager" rel="Ext.WindowManager" class="docClass">ZIndexManager</a>
 * object, and upon show, it will place itsef at the top of the hierarchy.</p>
 *
 * <p>Example usage:</p>
 *
 * <pre class='inline-example '><code>var myPanel = new <a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a>({
 *     renderTo : document.body,
 *     height   : 100,
 *     width    : 200,
 *     title    : 'Foo'
 * });
 *
 * var myMask = new <a href="#!/api/Ext.LoadMask" rel="Ext.LoadMask" class="docClass">Ext.LoadMask</a>({
 *     msg    : 'Please wait...',
 *     target : myPanel
 * });
 *
 * myMask.show();
 * </code></pre>
 */
public class LoadMask extends ext.Component implements ext.util.IStoreHolder {
  /**
   * Creates new LoadMask.
   * @param config The config object.
   */
  public function LoadMask(config:ext.LoadMask = null) {
    super();
  }

  [ExtConfig]
  /**
   * <code>true</code> to enable an iframe shim for this LoadMask to keep
   * windowed objects from showing through.
   */
  override public native function get shim():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set shim(value:Boolean):void;

  /**
   * This property allows the object
   * to destroy bound stores that have #Ext.data.AbstractStore.autoDestroy
   * option set to <code>true</code>.
   * @default false
   */
  public native function get autoDestroyBoundStore():Boolean;

  /**
   * @private
   */
  public native function set autoDestroyBoundStore(value:Boolean):void;

  /**
   * <code>true</code> in this class to identify an object as an instantiated LoadMask, or subclass thereof.
   * @default true
   */
  public native function get isLoadMask():Boolean;

  /**
   * @private
   */
  public native function set isLoadMask(value:Boolean):void;

  [ExtConfig]
  /**
   * The text to display in a centered loading message box.
   * @default "Loading..."
   */
  public native function get msg():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set msg(value:String):void;

  [ExtConfig]
  /**
   * Optional Store to which the mask is bound. The mask is displayed when a load request is issued, and
   * hidden on either load success, or load fail.
   * @see #setStore()
   */
  public native function get store():ext.data.Store;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set store(value:ext.data.Store):void;

  [ExtConfig]
  /**
   * The Component you wish to mask. The the mask will be automatically sized
   * upon Component resize, and the message box will be kept centered.
   */
  public native function get target():ext.Component;

  [ExtConfig]
  /**
   * @private
   */
  public native function set target(value:ext.Component):void;

  [ExtConfig]
  /**
   * Whether or not to use a loading message class or simply mask the bound element.
   * @default true
   */
  public native function get useMsg():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set useMsg(value:Boolean):void;

  [ExtConfig]
  /**
   * True to mask the <a href="#!/api/Ext.Component-method-getTargetEl" rel="Ext.Component-method-getTargetEl" class="docClass">targetEl</a> of the bound Component. By default,
   * the <a href="#!/api/Ext.Component-method-getEl" rel="Ext.Component-method-getEl" class="docClass">el</a> will be masked.
   * @default false
   */
  public native function get useTargetEl():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set useTargetEl(value:Boolean):void;

  /**
   * Changes the data store bound to this LoadMask.
   * @param store The store to bind or ID of the store.
   * When no store given (or when <code>null</code> or <code>undefined</code> passed), unbinds the existing store.
   */
  public native function bindStore(store:ext.data.Store = undefined):void;

  /**
   * Gets the current store instance.
   * @return The store, null if one does not exist.
   */
  public native function getStore():ext.data.AbstractStore;

  /**
   * Sets the store to the specified store.
   * @since 5.0.0
   */
  public native function setStore(store:Object):void;

  /**
   * Binds listeners for this component to the store. By default it will add
   * anything bound by the getStoreListeners method, however it can be overridden
   * in a subclass to provide any more complicated handling.
   * @param store The store to bind to
   */
  protected native function bindStoreListeners(store:ext.data.AbstractStore):void;

  /**
   * Template method, it is called when a new store is bound
   * to the current instance.
   * @param store The store being bound
   * @param initial True if this store is being bound as initialization of the instance.
   */
  protected native function onBindStore(store:ext.data.AbstractStore, initial:Boolean):void;

  /**
   * Template method, it is called when an existing store is unbound
   * from the current instance.
   * @param store The store being unbound
   * @param initial True if this store is being bound as initialization of the instance.
   */
  protected native function onUnbindStore(store:ext.data.AbstractStore, initial:Boolean):void;

  /**
   * Unbinds listeners from this component to the store. By default it will remove
   * anything bound by the bindStoreListeners method, however it can be overridden
   * in a subclass to provide any more complicated handling.
   * @param store The store to unbind from
   */
  protected native function unbindStoreListeners(store:ext.data.AbstractStore):void;
}
}