package ext {
import ext.data.Connection;

[Uses("ext.data.Connection")]
[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Native]
/**
 * A singleton instance of an <code><a href="#!/api/Ext.data.Connection" rel="Ext.data.Connection" class="docClass">Ext.data.Connection</a></code>. This class is used to
 * communicate with your server side code. It can be used as follows:
 *
 * <pre><code> <a href="#!/api/Ext.Ajax-method-request" rel="Ext.Ajax-method-request" class="docClass">Ext.Ajax.request</a>({
 *      url: 'ajax_demo/sample.json',
 *
 *      success: function(response, opts) {
 *          var obj = <a href="#!/api/Ext-method-decode" rel="Ext-method-decode" class="docClass">Ext.decode</a>(response.responseText);
 *          console.dir(obj);
 *      },
 *
 *      failure: function(response, opts) {
 *          console.log('server-side failure with status code ' + response.status);
 *      }
 *  });
 * </code></pre>
 *
 * <p>Default options for all requests can be set by changing a property on the <a href="#!/api/Ext.Ajax" rel="Ext.Ajax" class="docClass">Ext.Ajax</a> class:</p>
 *
 * <pre><code> <a href="#!/api/Ext.Ajax-method-setTimeout" rel="Ext.Ajax-method-setTimeout" class="docClass">Ext.Ajax.setTimeout</a>(60000); // 60 seconds
 * </code></pre>
 *
 * <p>Any options specified in the request method for the Ajax request will override any
 * defaults set on the <code><a href="#!/api/Ext.Ajax" rel="Ext.Ajax" class="docClass">Ext.Ajax</a></code> singleton. In the code sample below, the timeout for the
 * request will be 60 seconds.</p>
 *
 * <pre><code> <a href="#!/api/Ext.Ajax-method-setTimeout" rel="Ext.Ajax-method-setTimeout" class="docClass">Ext.Ajax.setTimeout</a>(120000); // 120 seconds
 *
 *  <a href="#!/api/Ext.Ajax-method-request" rel="Ext.Ajax-method-request" class="docClass">Ext.Ajax.request</a>({
 *      url: 'page.aspx',
 *      timeout: 60000
 *  });
 * </code></pre>
 *
 * <p>In general, this class will be used for all Ajax requests in your application. The main
 * reason for creating a separate <code><a href="#!/api/Ext.data.Connection" rel="Ext.data.Connection" class="docClass">Ext.data.Connection</a></code> is for a series of
 * requests that share common settings that are different to all other requests in the
 * application.</p>
 * <p>Type of singleton Ajax.</p>
 * @see ext.#Ajax ext.Ajax
 */
public class SAjax extends ext.data.Connection {
  /**
   * @inheritDoc
   */
  public function SAjax(config:ext.data.Connection = null) {
    super();
  }

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * An object containing properties which are used as extra parameters to each request made
   * by this object. Session information and other data that you need
   * to pass with each request are commonly put here.
   */
  override public native function get extraParams():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  override public native function set extraParams(value:Object):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The default HTTP method to be used for requests. Note that this is case-sensitive and
   * should be all caps (if not set but params are present will use <code>POST</code>, otherwise will
   * use <code>GET</code>.)
   */
  override public native function get method():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  override public native function set method(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The default URL to be used for requests to the server.
   * If the server receives all requests through one URL, setting this once is easier than
   * entering it on every request.
   */
  override public native function get url():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  override public native function set url(value:String):void;
}
}