package ext {
import js.Event;
import js.HTMLElement;

[Uses("ext.Base")]
[Deprecated(since = "5.0.0")]
[Native]
/**
 * Registers event handlers on DOM elements.
 *
 * <p>This class is deprecated.  Please use the <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> api to attach listeners to
 * DOM Elements.  For example:</p>
 *
 * <pre><code>var element = <a href="#!/api/Ext-method-get" rel="Ext-method-get" class="docClass">Ext.get</a>('myId');
 *
 * element.on('click', function(e) {
 *     // event handling logic here
 * });
 * </code></pre>
 * <p>Type of singleton EventManager.</p>
 * @see ext.#EventManager ext.EventManager
 */
public class SEventManager extends ext.Base {
  /**
   * Appends an event handler to an element.  The shorthand version <a href="#!/api/Ext.EventManager-method-on" rel="Ext.EventManager-method-on" class="docClass">on</a> is equivalent.
   * Typically you will use <a href="#!/api/Ext.dom.Element-method-addListener" rel="Ext.dom.Element-method-addListener" class="docClass">Ext.dom.Element.addListener</a> directly on an Element in favor of
   * calling this version.
   *
   * <p><a href="#!/api/Ext.EventManager-method-on" rel="Ext.EventManager-method-on" class="docClass">on</a> is an alias for <a href="#!/api/Ext.EventManager-method-addListener" rel="Ext.EventManager-method-addListener" class="docClass">addListener</a>.</p>
   * @param el The html element or id to assign the event handler to.
   * @param eventName The name of the event to listen for.
   * May also be an object who's property names are event names.
   * @param handler The handler function the event invokes. A String parameter
   * is assumed to be method name in <code>scope</code> object, or Element object if no scope is provided.
   * @param {ext.event.Event} handler.event The <a href="#!/api/Ext.event.Event" rel="Ext.event.Event" class="docClass">EventObject</a> describing the event.
   * @param {ext.dom.Element} handler.target The Element which was the target of the event.
   * Note that this may be filtered by using the <code>delegate</code> option.
   * @param {Object} handler.options The options object from the addListener call.
   * @param scope The scope (<code>this</code> reference) in which the handler function is executed.
   * Defaults to the Element.
   * @param options An object containing handler configuration properties.
   * This may contain any of the following properties (See <a href="#!/api/Ext.dom.Element-method-addListener" rel="Ext.dom.Element-method-addListener" class="docClass">Ext.dom.Element.addListener</a>
   * for examples of how to use these options.):
   * @param {Object} options.scope The scope (<code>this</code> reference) in which the handler function is executed. Defaults to the Element.
   * @param {String} options.delegate A simple selector to filter the target or look for a descendant of the target. See <a href="#!/api/Ext.dom.Query" rel="Ext.dom.Query" class="docClass">Ext.dom.Query</a> for
   * information about simple selectors.
   * @param {Boolean} options.stopEvent True to stop the event. That is stop propagation, and prevent the default action.
   * @param {Boolean} options.preventDefault True to prevent the default action
   * @param {Boolean} options.stopPropagation True to prevent event propagation
   * @param {Boolean} options.normalized False to pass a browser event to the handler function instead of an <a href="#!/api/Ext.event.Event" rel="Ext.event.Event" class="docClass">Ext.event.Event</a>
   * @param {Number} options.delay The number of milliseconds to delay the invocation of the handler after te event fires.
   * @param {Boolean} options.single True to add a handler to handle just the next firing of the event, and then remove itself.
   * @param {Number} options.buffer Causes the handler to be scheduled to run in an <a href="#!/api/Ext.util.DelayedTask" rel="Ext.util.DelayedTask" class="docClass">Ext.util.DelayedTask</a> delayed
   * by the specified number of milliseconds. If the event fires again within that time, the original
   * handler is <em>not</em> invoked, but the new handler is scheduled in its place.
   * @param {ext.dom.Element} options.target Only call the handler if the event was fired on the target Element,
   * <em>not</em> if the event was bubbled up from a child node.
   * @param {Boolean} options.capture <code>true</code> to initiate capture which will fire the listeners on the target Element <em>before</em> any descendant Elements.
   * Normal events start with the target element and propagate upward to ancestor elements, whereas captured events propagate from the top of the DOM
   * downward to descendant elements. This option is the same as the useCapture parameter in the javascript addEventListener method.
   */
  public native function addListener(el:*, eventName:String, handler:Function, scope:Object = null, options:Object = null):void;

  [Deprecated(since = "5.0.0")]
  /**
   * Get the id of the element. If one has not been assigned, automatically assign it.
   * @param element The element to get the id for.
   * @return id
   */
  public native function getId(element:*):String;

  [Deprecated(since = "5.0.0")]
  /**
   * Gets the x coordinate from the event
   * @param event The event
   * @return The x coordinate
   */
  public native function getPageX(event:Object):Number;

  [Deprecated(since = "5.0.0")]
  /**
   * Gets the x &amp; y coordinate from the event
   * @param event The event
   * @return The x/y coordinate
   */
  public native function getPageXY(event:Object):Array;

  [Deprecated(since = "5.0.0")]
  /**
   * Gets the y coordinate from the event
   * @param event The event
   * @return The y coordinate
   */
  public native function getPageY(event:Object):Number;

  [Deprecated(since = "5.0.0")]
  /**
   * Gets the related target from the event.
   * @param event The event
   * @return The related target.
   */
  public native function getRelatedTarget(event:Object):js.HTMLElement;

  [Deprecated(since = "5.0.0")]
  /**
   * Gets the target of the event.
   * @param event The event
   * @return target
   */
  public native function getTarget(event:Object):js.HTMLElement;

  /**
   * Appends an event handler to an element.  The shorthand version <a href="#!/api/Ext.EventManager-method-on" rel="Ext.EventManager-method-on" class="docClass">on</a> is equivalent.
   * Typically you will use <a href="#!/api/Ext.dom.Element-method-addListener" rel="Ext.dom.Element-method-addListener" class="docClass">Ext.dom.Element.addListener</a> directly on an Element in favor of
   * calling this version.
   *
   * <p><a href="#!/api/Ext.EventManager-method-on" rel="Ext.EventManager-method-on" class="docClass">on</a> is an alias for <a href="#!/api/Ext.EventManager-method-addListener" rel="Ext.EventManager-method-addListener" class="docClass">addListener</a>.</p>
   * @param el The html element or id to assign the event handler to.
   * @param eventName The name of the event to listen for.
   * May also be an object who's property names are event names.
   * @param handler The handler function the event invokes. A String parameter
   * is assumed to be method name in <code>scope</code> object, or Element object if no scope is provided.
   * @param {ext.event.Event} handler.event The <a href="#!/api/Ext.event.Event" rel="Ext.event.Event" class="docClass">EventObject</a> describing the event.
   * @param {ext.dom.Element} handler.target The Element which was the target of the event.
   * Note that this may be filtered by using the <code>delegate</code> option.
   * @param {Object} handler.options The options object from the addListener call.
   * @param scope The scope (<code>this</code> reference) in which the handler function is executed.
   * Defaults to the Element.
   * @param options An object containing handler configuration properties.
   * This may contain any of the following properties (See <a href="#!/api/Ext.dom.Element-method-addListener" rel="Ext.dom.Element-method-addListener" class="docClass">Ext.dom.Element.addListener</a>
   * for examples of how to use these options.):
   * @param {Object} options.scope The scope (<code>this</code> reference) in which the handler function is executed. Defaults to the Element.
   * @param {String} options.delegate A simple selector to filter the target or look for a descendant of the target. See <a href="#!/api/Ext.dom.Query" rel="Ext.dom.Query" class="docClass">Ext.dom.Query</a> for
   * information about simple selectors.
   * @param {Boolean} options.stopEvent True to stop the event. That is stop propagation, and prevent the default action.
   * @param {Boolean} options.preventDefault True to prevent the default action
   * @param {Boolean} options.stopPropagation True to prevent event propagation
   * @param {Boolean} options.normalized False to pass a browser event to the handler function instead of an <a href="#!/api/Ext.event.Event" rel="Ext.event.Event" class="docClass">Ext.event.Event</a>
   * @param {Number} options.delay The number of milliseconds to delay the invocation of the handler after te event fires.
   * @param {Boolean} options.single True to add a handler to handle just the next firing of the event, and then remove itself.
   * @param {Number} options.buffer Causes the handler to be scheduled to run in an <a href="#!/api/Ext.util.DelayedTask" rel="Ext.util.DelayedTask" class="docClass">Ext.util.DelayedTask</a> delayed
   * by the specified number of milliseconds. If the event fires again within that time, the original
   * handler is <em>not</em> invoked, but the new handler is scheduled in its place.
   * @param {ext.dom.Element} options.target Only call the handler if the event was fired on the target Element,
   * <em>not</em> if the event was bubbled up from a child node.
   * @param {Boolean} options.capture <code>true</code> to initiate capture which will fire the listeners on the target Element <em>before</em> any descendant Elements.
   * Normal events start with the target element and propagate upward to ancestor elements, whereas captured events propagate from the top of the DOM
   * downward to descendant elements. This option is the same as the useCapture parameter in the javascript addEventListener method.
   */
  public native function on(el:*, eventName:String, handler:* = undefined, scope:Object = null, options:Object = null):void;

  [Deprecated(replacement = "on", since = "5.0.0")]
  /**
   * Adds a listener to be notified when the browser window is resized and provides resize event buffering (100 milliseconds),
   * passes new viewport width and height to handlers.
   * @param fn The handler function the window resize event invokes.
   * @param scope The scope (<code>this</code> reference) in which the handler function executes. Defaults to the browser window.
   * @param options Options object as passed to <a href="#!/api/Ext.dom.Element-method-addListener" rel="Ext.dom.Element-method-addListener" class="docClass">Ext.dom.Element.addListener</a>
   */
  public native function onWindowResize(fn:Function, scope:Object = null, options:Object = null):void;

  [Deprecated(since = "5.0.0")]
  /**
   * Adds a listener to be notified when the browser window is unloaded.
   * @param fn The handler function the window unload event invokes.
   * @param scope The scope (<code>this</code> reference) in which the handler function executes. Defaults to the browser window.
   * @param options Options object as passed to <a href="#!/api/Ext.dom.Element-method-addListener" rel="Ext.dom.Element-method-addListener" class="docClass">Ext.dom.Element.addListener</a>
   */
  public native function onWindowUnload(fn:Function, scope:Object = null, options:Object = null):void;

  [Deprecated(since = "5.0.0")]
  /**
   * Prevents the browsers default handling of the event.
   * @param event The event to prevent the default
   */
  public native function preventDefault(event:js.Event):void;

  [Deprecated(since = "5.0.0")]
  /**
   * Recursively removes all previous added listeners from an element and its children.
   * Typically you will use <a href="#!/api/Ext.dom.Element-method-clearListeners" rel="Ext.dom.Element-method-clearListeners" class="docClass">Ext.dom.Element.clearListeners</a> directly on an Element
   * in favor of calling this method.
   * @param el The id or html element from which
   * to remove all event handlers.
   * @param eventName The name of the event.
   */
  public native function purgeElement(el:*, eventName:String = null):void;

  [Deprecated(since = "5.0.0")]
  /**
   * Removes all event handers from an element.  Typically you will use <a href="#!/api/Ext.dom.Element-method-clearListeners" rel="Ext.dom.Element-method-clearListeners" class="docClass">Ext.dom.Element.clearListeners</a> directly on an Element in favor of calling this method.
   * @param el The id or html element from which
   * to remove all event handlers.
   */
  public native function removeAll(el:*):void;

  /**
   * Removes an event handler from an element.  The shorthand version <a href="#!/api/Ext.EventManager-method-un" rel="Ext.EventManager-method-un" class="docClass">un</a> is equivalent.  Typically
   * you will use <a href="#!/api/Ext.dom.Element-method-removeListener" rel="Ext.dom.Element-method-removeListener" class="docClass">Ext.dom.Element.removeListener</a> directly on an Element in favor of calling this version.
   *
   * <p><a href="#!/api/Ext.EventManager-method-on" rel="Ext.EventManager-method-on" class="docClass">on</a> is an alias for <a href="#!/api/Ext.EventManager-method-addListener" rel="Ext.EventManager-method-addListener" class="docClass">addListener</a>.</p>
   * @param el The id or html element from which to remove the listener.
   * @param eventName The name of the event.
   * @param fn The handler function to remove. <strong>This must be a reference to the function passed
   * into the <a href="#!/api/Ext.EventManager-method-addListener" rel="Ext.EventManager-method-addListener" class="docClass">addListener</a> call.</strong>
   * @param scope If a scope (<code>this</code> reference) was specified when the listener was added,
   * then this must refer to the same object.
   */
  public native function removeListener(el:*, eventName:String, fn:Function, scope:Object = null):void;

  [Deprecated(since = "5.0.0")]
  /**
   * Removes the passed window resize listener.
   * @param fn The method the event invokes
   * @param scope The scope of handler
   */
  public native function removeResizeListener(fn:Function, scope:Object = null):void;

  [Deprecated(since = "5.0.0")]
  /**
   * Removes the passed window unload listener.
   * @param fn The method the event invokes
   * @param scope The scope of handler
   */
  public native function removeUnloadListener(fn:Function, scope:Object = null):void;

  [Deprecated(since = "5.0.0")]
  /**
   * Stop the event (preventDefault and stopPropagation)
   * @param event The event to stop
   */
  public native function stopEvent(event:js.Event):void;

  [Deprecated(since = "5.0.0")]
  /**
   * Cancels bubbling of the event.
   * @param event The event to stop bubbling.
   */
  public native function stopPropagation(event:js.Event):void;

  /**
   * Removes an event handler from an element.  The shorthand version <a href="#!/api/Ext.EventManager-method-un" rel="Ext.EventManager-method-un" class="docClass">un</a> is equivalent.  Typically
   * you will use <a href="#!/api/Ext.dom.Element-method-removeListener" rel="Ext.dom.Element-method-removeListener" class="docClass">Ext.dom.Element.removeListener</a> directly on an Element in favor of calling this version.
   *
   * <p><a href="#!/api/Ext.EventManager-method-on" rel="Ext.EventManager-method-on" class="docClass">on</a> is an alias for <a href="#!/api/Ext.EventManager-method-addListener" rel="Ext.EventManager-method-addListener" class="docClass">addListener</a>.</p>
   * @param el The id or html element from which to remove the listener.
   * @param eventName The name of the event.
   * @param fn The handler function to remove. <strong>This must be a reference to the function passed
   * into the <a href="#!/api/Ext.EventManager-method-addListener" rel="Ext.EventManager-method-addListener" class="docClass">addListener</a> call.</strong>
   * @param scope If a scope (<code>this</code> reference) was specified when the listener was added,
   * then this must refer to the same object.
   */
  public native function un(el:*, eventName:String, fn:Function, scope:Object):void;
}
}