package ext {
import ext.container.Container;
import ext.data.Store;
import ext.dom.Element;
import js.HTMLElement;

[Native]
/**
 * The Ext namespace (global object) encapsulates all classes, singletons, and
 * utility methods provided by Sencha's libraries.
 *
 * <p>Most user interface Components are at a lower level of nesting in the namespace,
 * but many common utility functions are provided as direct properties of the Ext namespace.</p>
 *
 * <p>Also many frequently used methods from other classes are provided as shortcuts
 * within the Ext namespace. For example <a href="#!/api/Ext-method-getCmp" rel="Ext-method-getCmp" class="docClass">Ext.getCmp</a> aliases
 * <a href="#!/api/Ext.ComponentManager-method-get" rel="Ext.ComponentManager-method-get" class="docClass">Ext.ComponentManager.get</a>.</p>
 *
 * <p>Many applications are initiated with Ext.application which is
 * called once the DOM is ready. This ensures all scripts have been loaded, preventing
 * dependency issues. For example:</p>
 *
 * <pre><code> Ext.application({
 *      name: 'MyApp',
 *
 *      launch: function () {
 *          <a href="#!/api/Ext.MessageBox-method-alert" rel="Ext.MessageBox-method-alert" class="docClass">Ext.Msg.alert</a>(this.name, 'Ready to go!');
 *      }
 *  });
 * </code></pre>
 *
 * <p><b><a href="http://www.sencha.com/products/sencha-cmd/">Sencha Cmd</a></b> is a free tool
 * for helping you generate and build Ext JS (and Sencha Touch) applications. See
 * <code><a href="#!/api/Ext.app.Application" rel="Ext.app.Application" class="docClass">Application</a></code> for more information about creating an app.</p>
 *
 * <p>A lower-level technique that does not use the <code><a href="#!/api/Ext.app.Application" rel="Ext.app.Application" class="docClass">Ext.app.Application</a></code> architecture is
 * <a href="#!/api/Ext-method-onReady" rel="Ext-method-onReady" class="docClass">Ext.onReady</a>.</p>
 *
 * <p>For more information about how to use the Ext classes, see:</p>
 *
 * <ul>
 * <li><a href="http://www.sencha.com/learn/">The Learning Center</a></li>
 * <li><a href="http://www.sencha.com/learn/Ext_FAQ">The FAQ</a></li>
 * <li><a href="http://www.sencha.com/forum/">The forums</a></li>
 * </ul>
 * <p>Type of singleton Ext.</p>
 * @see ext.#Ext ext.Ext
 */
public class SExt {
  /**
   * URL to a 1x1 transparent gif image used by Ext to create inline icons with
   * CSS background images.
   * @default 'data:image/gif;base64,R0lGODlhAQABAID/AMDAwAAAACH5BAEAAAAALAAAAAABAAEAAAICRAEAOw=='
   */
  public native function get BLANK_IMAGE_URL():String;

  /**
   * @private
   */
  public native function set BLANK_IMAGE_URL(value:String):void;

  /**
   * URL to a blank file used by Ext when in secure mode for iframe src and onReady src
   * to prevent the IE insecure content warning (<code>'about:blank'</code>, except for IE
   * in secure mode, which is <code>'javascript:""'</code>).
   */
  public native function get SSL_SECURE_URL():String;

  /**
   * @private
   */
  public native function set SSL_SECURE_URL(value:String):void;

  /**
   * Indicates whether to use native browser parsing for JSON methods.
   * This option is ignored if the browser does not support native JSON methods.
   *
   * <p><strong>Note:</strong> Native JSON methods will not work with objects that have functions.
   * Also, property names must be quoted, otherwise the data will not parse.</p>
   * @default false
   */
  public native function get USE_NATIVE_JSON():Boolean;

  /**
   * @private
   */
  public native function set USE_NATIVE_JSON(value:Boolean):void;

  /**
   * The base prefix to use for all <code>Ext</code> components. To configure this property, you should use the
   * Ext.buildSettings object before the framework is loaded:
   *
   * <pre><code>Ext.buildSettings = {
   *     baseCSSPrefix : 'abc-'
   * };
   * </code></pre>
   *
   * <p>or you can change it before any components are rendered:</p>
   *
   * <pre><code><a href="#!/api/Ext-property-baseCSSPrefix" rel="Ext-property-baseCSSPrefix" class="docClass">Ext.baseCSSPrefix</a> = Ext.buildSettings.baseCSSPrefix = 'abc-';
   * </code></pre>
   *
   * <p>This will change what CSS classes components will use and you should
   * then recompile the SASS changing the <code>$prefix</code> SASS variable to match.</p>
   * @default 'x-'
   */
  public native function get baseCSSPrefix():String;

  /**
   * @private
   */
  public native function set baseCSSPrefix(value:String):void;

  /**
   * The current version of Chrome (0 if the browser is not Chrome).
   */
  public native function get chromeVersion():Number;

  /**
   * A reusable empty function.
   */
  public native function get emptyFn():Function;

  /**
   * @private
   */
  public native function set emptyFn(value:Function):void;

  /**
   * A zero length string which will pass a truth test. Useful for passing to methods
   * which use a truth test to reject <i>falsy</i> values where a string value must be cleared.
   */
  public native function get emptyString():Object;

  /**
   * @private
   */
  public native function set emptyString(value:Object):void;

  [Deprecated(since = "6.0.2")]
  /**
   * This property is provided for backward
   * compatibility with previous versions of Ext JS. Accessibility is always enabled
   * in Ext JS 6.0+.
   *
   * <p>This property is deprecated. To disable WAI-ARIA compatibility warnings,
   * override <code>Ext.ariaWarn</code> function in your application startup code:</p>
   *
   * <pre><code> Ext.application({
   *      launch: function() {
   *          Ext.ariaWarn = <a href="#!/api/Ext-property-emptyFn" rel="Ext-property-emptyFn" class="docClass">Ext.emptyFn</a>;
   *      }
   *  });
   * </code></pre>
   *
   * <p>For stricter compatibility with WAI-ARIA requirements, replace <code>Ext.ariaWarn</code>
   * with a function that will raise an error instead:</p>
   *
   * <pre><code> Ext.application({
   *      launch: function() {
   *          Ext.ariaWarn = function(target, msg) {
   *              <a href="#!/api/Ext-method-raise" rel="Ext-method-raise" class="docClass">Ext.raise</a>({
   *                  msg: msg,
   *                  component: target
   *              });
   *          };
   *      }
   *  });
   * </code></pre>
   * @default true
   * @since 6.0.0
   */
  public native function get enableAria():Boolean;

  /**
   * @private
   */
  public native function set enableAria(value:Boolean):void;

  /**
   * True if the <a href="#!/api/Ext.fx.Anim" rel="Ext.fx.Anim" class="docClass">Ext.fx.Anim</a> Class is available.
   * @default true
   */
  public native function get enableFx():Boolean;

  /**
   * @private
   */
  public native function set enableFx(value:Boolean):void;

  /**
   * <code>true</code> to automatically uncache orphaned Ext.Elements periodically. If set to
   * <code>false</code>, the application will be required to clean up orphaned Ext.Elements and
   * it's listeners as to not cause memory leakage.
   * @default true
   */
  public native function get enableGarbageCollector():Boolean;

  /**
   * @private
   */
  public native function set enableGarbageCollector(value:Boolean):void;

  /**
   * True to automatically purge event listeners during garbageCollection.
   * @default true
   */
  public native function get enableListenerCollection():Boolean;

  /**
   * @private
   */
  public native function set enableListenerCollection(value:Boolean):void;

  /**
   * An array containing extra enumerables for old browsers
   */
  public native function get enumerables():Array;

  /**
   * @private
   */
  public native function set enumerables(value:Array):void;

  /**
   * The current version of Firefox (0 if the browser is not Firefox).
   */
  public native function get firefoxVersion():Number;

  /**
   * This indicate the start timestamp of current cycle.
   * It is only reliable during dom-event-initiated cycles and
   * Ext.draw.Animator initiated cycles.
   */
  public native function get frameStartTime():Object;

  /**
   * @private
   */
  public native function set frameStartTime(value:Object):void;

  /**
   * A reusable identity function that simply returns its first argument.
   */
  public native function get identityFn():Function;

  /**
   * @private
   */
  public native function set identityFn(value:Function):void;

  /**
   * The current version of IE (0 if the browser is not IE). This does not account
   * for the documentMode of the current page, which is factored into <a href="#!/api/Ext-property-isIE8" rel="Ext-property-isIE8" class="docClass">isIE8</a>,
   * and <a href="#!/api/Ext-property-isIE9" rel="Ext-property-isIE9" class="docClass">isIE9</a>. Thus this is not always true:
   *
   * <pre><code><a href="#!/api/Ext-property-isIE8" rel="Ext-property-isIE8" class="docClass">Ext.isIE8</a> == (<a href="#!/api/Ext-property-ieVersion" rel="Ext-property-ieVersion" class="docClass">Ext.ieVersion</a> == 8)
   * </code></pre>
   */
  public native function get ieVersion():Number;

  /**
   * True if the detected browser is Chrome.
   */
  public native function get isChrome():Boolean;

  /**
   * <code>true</code> when the document body is ready for use.
   */
  public native function get isDomReady():Boolean;

  /**
   * True if the detected browser is Edge.
   */
  public native function get isEdge():Boolean;

  /**
   * True if the detected browser uses the Gecko layout engine (e.g. Mozilla, Firefox).
   */
  public native function get isGecko():Boolean;

  /**
   * True if the detected browser is Internet Explorer.
   */
  public native function get isIE():Boolean;

  /**
   * True if the detected browser is Internet Explorer 10.x.
   */
  public native function get isIE10():Boolean;

  /**
   * True if the detected browser is Internet Explorer 10.x or lower.
   */
  public native function get isIE10m():Boolean;

  /**
   * True if the detected browser is Internet Explorer 10.x or higher.
   */
  public native function get isIE10p():Boolean;

  /**
   * True if the detected browser is Internet Explorer 11.x.
   */
  public native function get isIE11():Boolean;

  /**
   * True if the detected browser is Internet Explorer 11.x or lower.
   */
  public native function get isIE11m():Boolean;

  /**
   * True if the detected browser is Internet Explorer 11.x or higher.
   */
  public native function get isIE11p():Boolean;

  /**
   * True if the detected browser is Internet Explorer 8.x.
   */
  public native function get isIE8():Boolean;

  /**
   * True if the detected browser is Internet Explorer 8.x or lower.
   */
  public native function get isIE8m():Boolean;

  /**
   * True if the detected browser is Internet Explorer 8.x or higher.
   */
  public native function get isIE8p():Boolean;

  /**
   * True if the detected browser is Internet Explorer 9.x.
   */
  public native function get isIE9():Boolean;

  /**
   * True if the detected browser is Internet Explorer 9.x or lower.
   */
  public native function get isIE9m():Boolean;

  /**
   * True if the detected browser is Internet Explorer 9.x or higher.
   */
  public native function get isIE9p():Boolean;

  /**
   * True if the detected platform is Linux.
   */
  public native function get isLinux():Boolean;

  /**
   * True if the detected platform is Mac OS.
   */
  public native function get isMac():Boolean;

  /**
   * True if the detected browser is Opera.
   */
  public native function get isOpera():Boolean;

  /**
   * <code>true</code> when <code>isDomReady</code> is true and the Framework is ready for use.
   */
  public native function get isReady():Boolean;

  /**
   * True if the detected browser is Safari.
   */
  public native function get isSafari():Boolean;

  /**
   * True if the page is running over SSL
   */
  public native function get isSecure():Boolean;

  /**
   * True if the detected browser uses WebKit.
   */
  public native function get isWebKit():Boolean;

  /**
   * True if the detected platform is Windows.
   */
  public native function get isWindows():Boolean;

  /**
   * The name of the property in the global namespace (The <code>window</code> in browser environments) which refers to the current instance of Ext.
   *
   *
   * <p>This is usually <code>"Ext"</code>, but if a sandboxed build of ExtJS is being used, this will be an alternative name.</p>
   *
   *
   * <p>If code is being generated for use by <code>eval</code> or to create a <code>new Function</code>, and the global instance
   * of Ext must be referenced, this is the name that should be built into the code.</p>
   * @default 'Ext'
   */
  public native function get name():String;

  /**
   * @private
   */
  public native function set name(value:String):void;

  /**
   * The current version of Opera (0 if the browser is not Opera).
   */
  public native function get operaVersion():Number;

  /**
   * This object contains properties that describe the current device or platform. These
   * values can be used in <code><a href="#!/api/Ext.Class-cfg-platformConfig" rel="Ext.Class-cfg-platformConfig" class="docClass">platformConfig</a></code> as well as
   * <code><a href="#!/api/Ext.mixin.Responsive-cfg-responsiveConfig" rel="Ext.mixin.Responsive-cfg-responsiveConfig" class="docClass">responsiveConfig</a></code> statements.
   *
   * <p>This object can be modified to include tags that are useful for the application. To
   * add custom properties, it is advisable to use a sub-object. For example:</p>
   *
   * <pre><code> Ext.platformTags.app = {
   *      mobile: true
   *  };
   * </code></pre>
   * <ul>
   *   <li><code>phone</code> : Boolean</li>
   *   <li><code>tablet</code> : Boolean</li>
   *   <li><code>desktop</code> : Boolean</li>
   *   <li><code>touch</code> : Boolean
   * Indicates touch inputs are available.
   *   </li>
   *   <li><code>safari</code> : Boolean</li>
   *   <li><code>chrome</code> : Boolean</li>
   *   <li><code>windows</code> : Boolean</li>
   *   <li><code>firefox</code> : Boolean</li>
   *   <li><code>ios</code> : Boolean
   * True for iPad, iPhone and iPod.
   *   </li>
   *   <li><code>android</code> : Boolean</li>
   *   <li><code>blackberry</code> : Boolean</li>
   *   <li><code>tizen</code> : Boolean</li>
   * </ul>
   */
  public native function get platformTags():Object;

  /**
   * @private
   */
  public native function set platformTags(value:Object):void;

  /**
   * A reusable empty function for use as <code>privates</code> members.
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyClass', {
   *      nothing: <a href="#!/api/Ext-property-emptyFn" rel="Ext-property-emptyFn" class="docClass">Ext.emptyFn</a>,
   *
   *      privates: {
   *          privateNothing: <a href="#!/api/Ext-property-privateFn" rel="Ext-property-privateFn" class="docClass">Ext.privateFn</a>
   *      }
   *  });
   * </code></pre>
   */
  public native function get privateFn():Function;

  /**
   * @private
   */
  public native function set privateFn(value:Function):void;

  /**
   * The current version of Safari (0 if the browser is not Safari).
   */
  public native function get safariVersion():Number;

  /**
   * Set this to true before onReady to prevent any styling from being added to
   * the body element.  By default a few styles such as font-family, and color
   * are added to the body element via a "x-body" class.  When this is set to
   * <code>true</code> the "x-body" class is not added to the body element, but is added
   * to the elements of root-level containers instead.
   */
  public native function get scopeCss():Boolean;

  /**
   * @private
   */
  public native function set scopeCss(value:Boolean):void;

  /**
   * Set to <code>true</code> to use a <a href="#!/api/Ext.util.Floating-cfg-shim" rel="Ext.util.Floating-cfg-shim" class="docClass">shim</a> on all floating Components
   * and <a href="#!/api/Ext.LoadMask" rel="Ext.LoadMask" class="docClass">LoadMasks</a>
   * @default false
   */
  public native function get useShims():Boolean;

  /**
   * @private
   */
  public native function set useShims(value:Boolean):void;

  /**
   * The current version of WebKit (0 if the browser does not use WebKit).
   */
  public native function get webKitVersion():Number;

  [ExtConfig]
  /**
   * This object is used to enable or disable debugging for classes or namespaces. The
   * default instance looks like this:
   *
   * <pre><code> <a href="#!/api/Ext-cfg-debugConfig" rel="Ext-cfg-debugConfig" class="docClass">Ext.debugConfig</a> = {
   *      hooks: {
   *          '*': true
   *      }
   *  };
   * </code></pre>
   *
   * <p>Typically applications will set this in their <code>"app.json"</code> like so:</p>
   *
   * <pre><code> {
   *      "debug": {
   *          "hooks": {
   *              // Default for all namespaces:
   *              '*': true,
   *
   *              // Except for Ext namespace which is disabled
   *              'Ext': false,
   *
   *              // Except for Ext.layout namespace which is enabled
   *              'Ext.layout': true
   *          }
   *      }
   *  }
   * </code></pre>
   *
   * <p>Alternatively, because this property is consumed very early in the load process of
   * the framework, this can be set in a <code>script</code> tag that is defined prior to loading
   * the framework itself.</p>
   *
   * <p>For example, to enable debugging for the <code>Ext.layout</code> namespace only:</p>
   *
   * <pre><code> var Ext = Ext || {};
   *  <a href="#!/api/Ext-cfg-debugConfig" rel="Ext-cfg-debugConfig" class="docClass">Ext.debugConfig</a> = {
   *      hooks: {
   *          //...
   *      }
   *  };
   * </code></pre>
   *
   * <p>For any class declared, the longest matching namespace specified determines if its
   * <code>debugHooks</code> will be enabled. The default setting is specified by the '*' property.</p>
   *
   * <p><strong>NOTE:</strong> This option only applies to debug builds. All debugging is disabled in
   * production builds.</p>
   */
  public native function get debugConfig():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set debugConfig(value:Object):void;

  [ExtConfig]
  /**
   * This object is initialized prior to loading the framework (Ext JS or Sencha
   * Touch) and contains settings and other information describing the application.
   *
   * <p>For applications built using Sencha Cmd, this is produced from the <code>"app.json"</code>
   * file with information extracted from all of the required packages' <code>"package.json"</code>
   * files. This can be set to a string when your application is using the
   * (microloader)[#/guide/microloader]. In this case, the string of "foo" will be
   * requested as <code>"foo.json"</code> and the object in that JSON file will parsed and set
   * as this object.</p>
   * @since 5.0.0
   * <ul>
   *   <li><code>compatibility</code>
   * An object keyed by package name with
   * the value being to desired compatibility level as a version number. If this is
   * just a string, this version is assumed to apply to the framework ('ext' or
   * 'touch'). Setting this value to less than 5 for 'ext' will enable the compatibility
   * layer to assist in the application upgrade process. For details on the upgrade
   * process, see the (Upgrade Guide)[#/guides/upgrade_50].
   *   </li>
   *   <li><code>debug</code> : Object
   * An object configuring the debugging characteristics
   * of the framework. See <code><a href="#!/api/Ext-cfg-debugConfig" rel="Ext-cfg-debugConfig" class="docClass">Ext.debugConfig</a></code> which is set to this value.
   *   </li>
   *   <li><code>packages</code> : Object
   * An object keyed by package name with the value
   * being a subset of the package's <code>"package.json"</code> descriptor.
   *   </li>
   * </ul>
   */
  public native function get manifest():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set manifest(value:*):void;

  /**
   * Applies event listeners to elements by selectors when the document is ready.
   * The event name is specified with an <code>&#64;</code> suffix.
   *
   * <pre><code><a href="#!/api/Ext-method-addBehaviors" rel="Ext-method-addBehaviors" class="docClass">Ext.addBehaviors</a>({
   *     // add a listener for click on all anchors in element with id foo
   *     '#foo a&#64;click': function(e, t){
   *         // do something
   *     },
   *
   *     // add the same listener to multiple selectors (separated by comma BEFORE the @)
   *     '#foo a, #bar span.some-class&#64;mouseover': function(){
   *         // do something
   *     }
   * });
   * </code></pre>
   * @param obj The list of behaviors to apply
   */
  public native function addBehaviors(obj:Object):void;

  /**
   * Same as <a href="#!/api/Ext.ComponentQuery-method-query" rel="Ext.ComponentQuery-method-query" class="docClass">Ext.ComponentQuery.query</a>.
   * @param selector The selector string to filter returned Components.
   * @param root The Container within which to perform the query.
   * If omitted, all Components within the document are included in the search.
   * <p>This parameter may also be an array of Components to filter according to the selector.</p>
   * @return The matched Components.
   */
  public native function all(selector:String, root:ext.container.Container = null):Array;

  /**
   * Copies all the properties of <code>config</code> to the specified <code>object</code>. There are two levels
   * of defaulting supported:
   *
   * <pre><code> <a href="#!/api/Ext-method-apply" rel="Ext-method-apply" class="docClass">Ext.apply</a>(obj, { a: 1 }, { a: 2 });
   *  //obj.a === 1
   *
   *  <a href="#!/api/Ext-method-apply" rel="Ext-method-apply" class="docClass">Ext.apply</a>(obj, {  }, { a: 2 });
   *  //obj.a === 2
   * </code></pre>
   *
   * <p>Note that if recursive merging and cloning without referencing the original objects
   * or arrays is needed, use <a href="#!/api/Ext.Object-method-merge" rel="Ext.Object-method-merge" class="docClass">Ext.Object.merge</a> instead.</p>
   * @param object The receiver of the properties.
   * @param config The primary source of the properties.
   * @param defaults An object that will also be applied for default values.
   * @return returns <code>object</code>.
   */
  public native function apply(object:Object, config:Object, defaults:Object = null):Object;

  /**
   * Copies all the properties of config to object if they don't already exist.
   * @param object The receiver of the properties
   * @param config The source of the properties
   * @return returns obj
   */
  public native function applyIf(object:Object, config:Object):Object;

  /**
   * Schedules the specified callback function to be executed on the next turn of the
   * event loop. Where available, this method uses the browser's <code>setImmediate</code> API. If
   * not available, this method substitutes <code>setTimeout(0)</code>. Though not a perfect
   * replacement for <code>setImmediate</code> it is sufficient for many use cases.
   *
   * <p>For more details see <a href="https://developer.mozilla.org/en-US/docs/Web/API/Window/setImmediate">MDN</a>.</p>
   * @param fn Callback function.
   * @param scope The scope for the callback (<code>this</code> pointer).
   * @param parameters Additional parameters to pass to <code>fn</code>.
   * @return A cancelation id for <code><a href="#!/api/Ext-method-asapCancel" rel="Ext-method-asapCancel" class="docClass">asapCancel</a></code>.
   */
  public native function asap(fn:Function, scope:Object = null, parameters:Array = null):Number;

  /**
   * Cancels a previously scheduled call to <code><a href="#!/api/Ext-method-asap" rel="Ext-method-asap" class="docClass">asap</a></code>.
   *
   * <pre><code> var asapId = <a href="#!/api/Ext-method-asap" rel="Ext-method-asap" class="docClass">Ext.asap</a>(me.method, me);
   *  ...
   *
   *  if (nevermind) {
   *      Ext.apasCancel(asapId);
   *  }
   * </code></pre>
   * @param id The id returned by <code><a href="#!/api/Ext-method-asap" rel="Ext-method-asap" class="docClass">asap</a></code>.
   */
  public native function asapCancel(id:Number):void;

  /**
   * Utility wrapper that suspends layouts of all components for the duration of a given
   * function.
   * @param fn The function to execute.
   * @param scope The scope (<code>this</code> reference) in which the specified function
   * is executed.
   */
  public native function batchLayouts(fn:Function, scope:Object = null):void;

  /**
   * Create a new function from the provided <code>fn</code>, change <code>this</code> to the provided scope,
   * optionally overrides arguments for the call. Defaults to the arguments passed by
   * the caller.
   *
   * <p><a href="#!/api/Ext-method-bind" rel="Ext-method-bind" class="docClass">Ext.bind</a> is alias for <a href="#!/api/Ext.Function-method-bind" rel="Ext.Function-method-bind" class="docClass">Ext.Function.bind</a></p>
   *
   * <p><strong>NOTE:</strong> This method is deprecated. Use the standard <code>bind</code> method of JavaScript
   * <code>Function</code> instead:</p>
   *
   * <pre><code> function foo () {
   *      ...
   *  }
   *
   *  var fn = foo.bind(this);
   * </code></pre>
   *
   * <p>This method is unavailable natively on IE8 and IE/Quirks but Ext JS provides a
   * "polyfill" to emulate the important features of the standard <code>bind</code> method. In
   * particular, the polyfill only provides binding of "this" and optional arguments.</p>
   * @param fn The function to delegate.
   * @param scope The scope (<code>this</code> reference) in which the function is executed.
   * <strong>If omitted, defaults to the default global environment object (usually the browser window).</strong>
   * @param args Overrides arguments for the call. (Defaults to the arguments passed by the caller)
   * @param appendArgs if True args are appended to call args instead of overriding,
   * if a number the args are inserted at the specified position.
   * @return The new function.
   */
  public native function bind(fn:Function, scope:Object = null, args:Array = null, appendArgs:* = undefined):Function;

  /**
   * Execute a callback function in a particular scope. If <code>callback</code> argument is a
   * function reference, that is called. If it is a string, the string is assumed to
   * be the name of a method on the given <code>scope</code>. If no function is passed the call
   * is ignored.
   *
   * <p>For example, these calls are equivalent:</p>
   *
   * <pre><code> var myFunc = this.myFunc;
   *
   *  <a href="#!/api/Ext-method-callback" rel="Ext-method-callback" class="docClass">Ext.callback</a>('myFunc', this, [arg1, arg2]);
   *  <a href="#!/api/Ext-method-callback" rel="Ext-method-callback" class="docClass">Ext.callback</a>(myFunc, this, [arg1, arg2]);
   *
   *  <a href="#!/api/Ext-method-isFunction" rel="Ext-method-isFunction" class="docClass">Ext.isFunction</a>(myFunc) &amp;&amp; this.myFunc(arg1, arg2);
   * </code></pre>
   * @param callback The callback function to execute or the name of
   * the callback method on the provided <code>scope</code>.
   * @param scope The scope in which <code>callback</code> should be invoked. If <code>callback</code>
   * is a string this object provides the method by that name. If this is <code>null</code> then
   * the <code>caller</code> is used to resolve the scope to a <code>ViewController</code> or the proper
   * <code>defaultListenerScope</code>.
   * @param args The arguments to pass to the function.
   * @param delay Pass a number to delay the call by a number of milliseconds.
   * @param caller The object calling the callback. This is used to resolve
   * named methods when no explicit <code>scope</code> is provided.
   * @param defaultScope (Default caller) The default scope to return if none is found.
   * @default caller
   * @return The value returned by the callback or <code>undefined</code> (if there is a <code>delay</code>
   * or if the <code>callback</code> is not a function).
   */
  public native function callback(callback:*, scope:Object = null, args:Array = null, delay:Number = NaN, caller:Object = null, defaultScope:Object = null):Object;

  /**
   * This method checks the registered package versions against the provided version
   * <code>specs</code>. A <code>spec</code> is either a string or an object indicating a boolean operator.
   * This method accepts either form or an array of these as the first argument. The
   * second argument applies only when the first is an array and indicates whether
   * all <code>specs</code> must match or just one.
   *
   * <h2>Package Version Specifications</h2>
   *
   * <p>The string form of a <code>spec</code> is used to indicate a version or range of versions
   * for a particular package. This form of <code>spec</code> consists of three (3) parts:</p>
   *
   * <ul>
   * <li>Package name followed by "&#64;". If not provided, the framework is assumed.</li>
   * <li>Minimum version.</li>
   * <li>Maximum version.</li>
   * </ul>
   *
   *
   * <p>At least one version number must be provided. If both minimum and maximum are
   * provided, these must be separated by a "-".</p>
   *
   * <p>Some examples of package version specifications:</p>
   *
   * <pre><code> 4.2.2           (exactly version 4.2.2 of the framework)
   *  4.2.2+          (version 4.2.2 or higher of the framework)
   *  4.2.2-          (version 4.2.2 or higher of the framework)
   *  4.2.1 - 4.2.3   (versions from 4.2.1 up to 4.2.3 of the framework)
   *  - 4.2.2         (any version up to version 4.2.1 of the framework)
   *
   *  foo&#64;1.0         (exactly version 1.0 of package "foo")
   *  foo&#64;1.0-1.3     (versions 1.0 up to 1.3 of package "foo")
   * </code></pre>
   *
   * <p><strong>NOTE:</strong> This syntax is the same as that used in Sencha Cmd's package
   * requirements declarations.</p>
   *
   * <h2>Boolean Operator Specifications</h2>
   *
   * <p>Instead of a string, an object can be used to describe a boolean operation to
   * perform on one or more <code>specs</code>. The operator is either <strong><code>and</code></strong> or <strong><code>or</code></strong>
   * and can contain an optional <strong><code>not</code></strong>.</p>
   *
   * <p>For example:</p>
   *
   * <pre><code> {
   *      not: true,  // negates boolean result
   *      and: [
   *          '4.2.2',
   *          'foo&#64;1.0.1 - 2.0.1'
   *      ]
   *  }
   * </code></pre>
   *
   * <p>Each element of the array can in turn be a string or object spec. In other
   * words, the value is passed to this method (recursively) as the first argument
   * so these two calls are equivalent:</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-checkVersion" rel="Ext-method-checkVersion" class="docClass">Ext.checkVersion</a>({
   *      not: true,  // negates boolean result
   *      and: [
   *          '4.2.2',
   *          'foo&#64;1.0.1 - 2.0.1'
   *      ]
   *  });
   *
   *  !<a href="#!/api/Ext-method-checkVersion" rel="Ext-method-checkVersion" class="docClass">Ext.checkVersion</a>([
   *          '4.2.2',
   *          'foo&#64;1.0.1 - 2.0.1'
   *      ], true);
   * </code></pre>
   *
   * <h2>Examples</h2>
   *
   * <pre><code> // A specific framework version
   *  <a href="#!/api/Ext-method-checkVersion" rel="Ext-method-checkVersion" class="docClass">Ext.checkVersion</a>('4.2.2');
   *
   *  // A range of framework versions:
   *  <a href="#!/api/Ext-method-checkVersion" rel="Ext-method-checkVersion" class="docClass">Ext.checkVersion</a>('4.2.1-4.2.3');
   *
   *  // A specific version of a package:
   *  <a href="#!/api/Ext-method-checkVersion" rel="Ext-method-checkVersion" class="docClass">Ext.checkVersion</a>('foo&#64;1.0.1');
   *
   *  // A single spec that requires both a framework version and package
   *  // version range to match:
   *  <a href="#!/api/Ext-method-checkVersion" rel="Ext-method-checkVersion" class="docClass">Ext.checkVersion</a>({
   *      and: [
   *          '4.2.2',
   *          'foo&#64;1.0.1-1.0.2'
   *      ]
   *  });
   *
   *  // These checks can be nested:
   *  <a href="#!/api/Ext-method-checkVersion" rel="Ext-method-checkVersion" class="docClass">Ext.checkVersion</a>({
   *      and: [
   *          '4.2.2',  // exactly version 4.2.2 of the framework *AND*
   *          {
   *              // either (or both) of these package specs:
   *              or: [
   *                  'foo&#64;1.0.1-1.0.2',
   *                  'bar&#64;3.0+'
   *              ]
   *          }
   *      ]
   *  });
   * </code></pre>
   *
   * <h2>Version Comparisons</h2>
   *
   * <p>Version comparsions are assumed to be "prefix" based. That is to say, <code>"foo&#64;1.2"</code>
   * matches any version of "foo" that has a major version 1 and a minor version of 2.</p>
   *
   * <p>This also applies to ranges. For example <code>"foo&#64;1.2-2.2"</code> matches all versions
   * of "foo" from 1.2 up to 2.2 regardless of the specific patch and build.</p>
   *
   * <h2>Use in Overrides</h2>
   *
   * <p>This methods primary use is in support of conditional overrides on an
   * <code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a></code> declaration.</p>
   * @param specs A version specification string, an object
   * containing <code>or</code> or <code>and</code> with a value that is equivalent to <code>specs</code> or an array
   * of either of these.
   * @param matchAll Pass <code>true</code> to require all specs to match.
   * @default false
   * @return True if <code>specs</code> matches the registered package versions.
   */
  public native function checkVersion(specs:*, matchAll:Boolean = false):Boolean;

  [Deprecated(replacement = "clean", since = "4.0.0")]
  /**
   * Old alias to <a href="#!/api/Ext.Array-method-clean" rel="Ext.Array-method-clean" class="docClass">Ext.Array.clean</a>
   * @return results
   */
  public native function clean(array:Array):Array;

  /**
   * Clone simple variables including array, {}-like objects, DOM nodes and Date without keeping the old reference.
   * A reference for the object itself is returned if it's not a direct descendant of Object. For model cloning,
   * see <a href="#!/api/Ext.data.Model-method-copy" rel="Ext.data.Model-method-copy" class="docClass">Model.copy</a>.
   * @param item The variable to clone
   * @param cloneDom <code>true</code> to clone DOM nodes.
   * @default true
   * @return clone
   */
  public native function clone(item:Object, cloneDom:Boolean = true):Object;

  /**
   * Coerces the first value if possible so that it is comparable to the second value.
   *
   * <p>Coercion only works between the basic atomic data types String, Boolean, Number, Date, null and undefined.</p>
   *
   * <p>Numbers and numeric strings are coerced to Dates using the value as the millisecond era value.</p>
   *
   * <p>Strings are coerced to Dates by parsing using the <a href="#!/api/Ext.Date-property-defaultFormat" rel="Ext.Date-property-defaultFormat" class="docClass">defaultFormat</a>.</p>
   *
   * <p>For example</p>
   *
   * <pre><code><a href="#!/api/Ext-method-coerce" rel="Ext-method-coerce" class="docClass">Ext.coerce</a>('false', true);
   * </code></pre>
   *
   * <p>returns the boolean value <code>false</code> because the second parameter is of type <code>Boolean</code>.</p>
   * @param from The value to coerce
   * @param to The value it must be compared against
   * @return The coerced value.
   */
  public native function coerce(from:*, to:*):Object;

  /**
   * Copies a set of named properties fom the source object to the destination object.
   *
   * <p>Example:</p>
   *
   * <pre><code>var foo = { a: 1, b: 2, c: 3 };
   *
   * var bar = <a href="#!/api/Ext-method-copy" rel="Ext-method-copy" class="docClass">Ext.copy</a>({}, foo, 'a,c');
   * // bar = { a: 1, c: 3 };
   * </code></pre>
   *
   * <p>Important note: To borrow class prototype methods, use <a href="#!/api/Ext.Base-static-method-borrow" rel="Ext.Base-static-method-borrow" class="docClass">Ext.Base.borrow</a> instead.</p>
   * @param dest The destination object.
   * @param source The source object.
   * @param names Either an Array of property names, or a comma-delimited list
   * of property names to copy.
   * @param usePrototypeKeys Pass <code>true</code> to copy keys off of the
   * prototype as well as the instance.
   * @default false
   * @return The <code>dest</code> object.
   */
  public native function copy(dest:Object, source:Object, names:*, usePrototypeKeys:Boolean = false):Object;

  /**
   * Copies a set of named properties fom the source object to the destination object
   * if the destination object does not already have them.
   *
   * <p>Example:</p>
   *
   * <pre><code>var foo = { a: 1, b: 2, c: 3 };
   *
   * var bar = <a href="#!/api/Ext-method-copyIf" rel="Ext-method-copyIf" class="docClass">Ext.copyIf</a>({ a:42 }, foo, 'a,c');
   * // bar = { a: 42, c: 3 };
   * </code></pre>
   * @param destination The destination object.
   * @param source The source object.
   * @param names Either an Array of property names, or a single string
   * with a list of property names separated by ",", ";" or spaces.
   * @return The <code>dest</code> object.
   */
  public native function copyIf(destination:Object, source:Object, names:*):Object;

  [Deprecated(replacement = "copy", since = "6.0.1")]
  /**
   * Copies a set of named properties fom the source object to the destination object.
   *
   * <p>Example:</p>
   *
   * <pre><code>var foo = { a: 1, b: 2, c: 3 };
   *
   * var bar = <a href="#!/api/Ext-method-copyTo" rel="Ext-method-copyTo" class="docClass">Ext.copyTo</a>({}, foo, 'a,c');
   * // bar = { a: 1, c: 3 };
   * </code></pre>
   *
   * <p>Important note: To borrow class prototype methods, use <a href="#!/api/Ext.Base-static-method-borrow" rel="Ext.Base-static-method-borrow" class="docClass">Ext.Base.borrow</a> instead.</p>
   * @param dest The destination object.
   * @param source The source object.
   * @param names Either an Array of property names, or a comma-delimited list
   * of property names to copy.
   * @param usePrototypeKeys Pass <code>true</code> to copy keys off of the
   * prototype as well as the instance.
   * @default false
   * @return The <code>dest</code> object.
   */
  public native function copyTo(dest:Object, source:Object, names:*, usePrototypeKeys:Boolean = false):Object;

  [Deprecated(replacement = "copyIf", since = "6.0.1")]
  /**
   * Copies a set of named properties fom the source object to the destination object
   * if the destination object does not already have them.
   *
   * <p>Example:</p>
   *
   * <pre><code>var foo = { a: 1, b: 2, c: 3 };
   *
   * var bar = <a href="#!/api/Ext-method-copyToIf" rel="Ext-method-copyToIf" class="docClass">Ext.copyToIf</a>({ a:42 }, foo, 'a,c');
   * // bar = { a: 42, c: 3 };
   * </code></pre>
   * @param destination The destination object.
   * @param source The source object.
   * @param names Either an Array of property names, or a single string
   * with a list of property names separated by ",", ";" or spaces.
   * @return The <code>dest</code> object.
   */
  public native function copyToIf(destination:Object, source:Object, names:*):Object;

  /**
   * Instantiate a class by either full name, alias or alternate name.
   *
   * <p>If <a href="#!/api/Ext.Loader" rel="Ext.Loader" class="docClass">Ext.Loader</a> is <a href="#!/api/Ext.Loader-method-setConfig" rel="Ext.Loader-method-setConfig" class="docClass">enabled</a> and the class has
   * not been defined yet, it will attempt to load the class via synchronous loading.</p>
   *
   * <p>For example, all these three lines return the same result:</p>
   *
   * <pre><code> // xtype
   *  var window = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>({
   *      xtype: 'window',
   *      width: 600,
   *      height: 800,
   *      ...
   *  });
   *
   *  // alias
   *  var window = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('widget.window', {
   *      width: 600,
   *      height: 800,
   *      ...
   *  });
   *
   *  // alternate name
   *  var window = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.window.Window" rel="Ext.window.Window" class="docClass">Ext.Window</a>', {
   *      width: 600,
   *      height: 800,
   *      ...
   *  });
   *
   *  // full class name
   *  var window = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.window.Window" rel="Ext.window.Window" class="docClass">Ext.window.Window</a>', {
   *      width: 600,
   *      height: 800,
   *      ...
   *  });
   *
   *  // single object with xclass property:
   *  var window = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>({
   *      xclass: '<a href="#!/api/Ext.window.Window" rel="Ext.window.Window" class="docClass">Ext.window.Window</a>', // any valid value for 'name' (above)
   *      width: 600,
   *      height: 800,
   *      ...
   *  });
   * </code></pre>
   * @param nameOrAliasOrConfig The class name or alias. Can be specified as <code>xclass</code>
   * property if only one object parameter is specified.
   * @param args Additional arguments after the name will be passed to
   * the class' constructor.
   * @return instance
   */
  public native function create(nameOrAliasOrConfig:Object, ...args):*;

  /**
   * Instantiate a class by its alias. This is usually invoked by the
   * shorthand <a href="#!/api/Ext-method-createByAlias" rel="Ext-method-createByAlias" class="docClass">createByAlias</a>.
   *
   * <p>If <a href="#!/api/Ext.Loader" rel="Ext.Loader" class="docClass">Ext.Loader</a> is <a href="#!/api/Ext.Loader-method-setConfig" rel="Ext.Loader-method-setConfig" class="docClass">enabled</a> and the class
   * has not been defined yet, it will attempt to load the class via synchronous
   * loading.</p>
   *
   * <pre><code>var window = <a href="#!/api/Ext-method-createByAlias" rel="Ext-method-createByAlias" class="docClass">Ext.createByAlias</a>('widget.window', { width: 600, height: 800 });
   * </code></pre>
   * @param args Additional arguments after the alias will be passed to the
   * class constructor.
   * @return instance
   */
  public native function createByAlias(alias:String, ...args):*;

  /**
   * Shorthand for <a href="#!/api/Ext.JSON-method-decode" rel="Ext.JSON-method-decode" class="docClass">Ext.JSON.decode</a>
   * @param json The JSON string.
   * @param safe <code>true</code> to return null, otherwise throw an exception
   * if the JSON is invalid.
   * @default false
   * @return The resulting object.
   */
  public native function decode(json:String, safe:Boolean = false):Object;

  /**
   * Calls this function after the number of milliseconds specified, optionally in a specific scope. Example usage:
   *
   * <pre><code>var sayHi = function(name){
   *     alert('Hi, ' + name);
   * }
   *
   * // executes immediately:
   * sayHi('Fred');
   *
   * // executes after 2 seconds:
   * <a href="#!/api/Ext.Function-method-defer" rel="Ext.Function-method-defer" class="docClass">Ext.Function.defer</a>(sayHi, 2000, this, ['Fred']);
   *
   * // this syntax is sometimes useful for deferring
   * // execution of an anonymous function:
   * <a href="#!/api/Ext.Function-method-defer" rel="Ext.Function-method-defer" class="docClass">Ext.Function.defer</a>(function(){
   *     alert('Anonymous');
   * }, 100);
   * </code></pre>
   *
   * <p><a href="#!/api/Ext-method-defer" rel="Ext-method-defer" class="docClass">Ext.defer</a> is alias for <a href="#!/api/Ext.Function-method-defer" rel="Ext.Function-method-defer" class="docClass">Ext.Function.defer</a></p>
   * @param fn The function to defer.
   * @param millis The number of milliseconds for the <code>setTimeout</code> call
   * (if less than or equal to 0 the function is executed immediately).
   * @param scope The scope (<code>this</code> reference) in which the function is executed.
   * <strong>If omitted, defaults to the browser window.</strong>
   * @param args Overrides arguments for the call. Defaults to the arguments passed by the caller.
   * @param appendArgs If <code>true</code> args are appended to call args instead of overriding,
   * or, if a number, then the args are inserted at the specified position.
   * @default false
   * @return The timeout id that can be used with <code>clearTimeout</code>.
   */
  public native function defer(fn:Function, millis:Number, scope:Object = null, args:Array = null, appendArgs:* = false):Number;

  /**
   * Defines a class or override. A basic class is defined like this:
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('My.awesome.Class', {
   *      someProperty: 'something',
   *
   *      someMethod: function(s) {
   *          alert(s + this.someProperty);
   *      }
   *
   *      ...
   *  });
   *
   *  var obj = new My.awesome.Class();
   *
   *  obj.someMethod('Say '); // alerts 'Say something'
   * </code></pre>
   *
   * <p>To create an anonymous class, pass <code>null</code> for the <code>className</code>:</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>(null, {
   *      constructor: function () {
   *          // ...
   *      }
   *  });
   * </code></pre>
   *
   * <p>In some cases, it is helpful to create a nested scope to contain some private
   * properties. The best way to do this is to pass a function instead of an object
   * as the second parameter. This function will be called to produce the class
   * body:</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.foo.Bar', function () {
   *      var id = 0;
   *
   *      return {
   *          nextId: function () {
   *              return ++id;
   *          }
   *      };
   *  });
   * </code></pre>
   *
   * <p><em>Note</em> that when using override, the above syntax will not override successfully, because
   * the passed function would need to be executed first to determine whether or not the result
   * is an override or defining a new object. As such, an alternative syntax that immediately
   * invokes the function can be used:</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.override.BaseOverride', function () {
   *      var counter = 0;
   *
   *      return {
   *          override: '<a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>',
   *          logId: function () {
   *              console.log(++counter, this.id);
   *          }
   *      };
   *  }());
   * </code></pre>
   *
   * <p>When using this form of <code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a></code>, the function is passed a reference to its
   * class. This can be used as an efficient way to access any static properties you
   * may have:</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.foo.Bar', function (Bar) {
   *      return {
   *          statics: {
   *              staticMethod: function () {
   *                  // ...
   *              }
   *          },
   *
   *          method: function () {
   *              return Bar.staticMethod();
   *          }
   *      };
   *  });
   * </code></pre>
   *
   * <p>To define an override, include the <code>override</code> property. The content of an
   * override is aggregated with the specified class in order to extend or modify
   * that class. This can be as simple as setting default property values or it can
   * extend and/or replace methods. This can also extend the statics of the class.</p>
   *
   * <p>One use for an override is to break a large class into manageable pieces.</p>
   *
   * <pre><code> // File: /src/app/Panel.js
   *
   *  <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('My.app.Panel', {
   *      extend: '<a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a>',
   *      requires: [
   *          'My.app.PanelPart2',
   *          'My.app.PanelPart3'
   *      ]
   *
   *      constructor: function (config) {
   *          this.callParent(arguments); // calls <a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a>'s constructor
   *          //...
   *      },
   *
   *      statics: {
   *          method: function () {
   *              return 'abc';
   *          }
   *      }
   *  });
   *
   *  // File: /src/app/PanelPart2.js
   *  <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('My.app.PanelPart2', {
   *      override: 'My.app.Panel',
   *
   *      constructor: function (config) {
   *          this.callParent(arguments); // calls My.app.Panel's constructor
   *          //...
   *      }
   *  });
   * </code></pre>
   *
   * <p>Another use of overrides is to provide optional parts of classes that can be
   * independently required. In this case, the class may even be unaware of the
   * override altogether.</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('My.ux.CoolTip', {
   *      override: '<a href="#!/api/Ext.tip.ToolTip" rel="Ext.tip.ToolTip" class="docClass">Ext.tip.ToolTip</a>',
   *
   *      constructor: function (config) {
   *          this.callParent(arguments); // calls <a href="#!/api/Ext.tip.ToolTip" rel="Ext.tip.ToolTip" class="docClass">Ext.tip.ToolTip</a>'s constructor
   *          //...
   *      }
   *  });
   * </code></pre>
   *
   * <p>The above override can now be required as normal.</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('My.app.App', {
   *      requires: [
   *          'My.ux.CoolTip'
   *      ]
   *  });
   * </code></pre>
   *
   * <p>Overrides can also contain statics, inheritableStatics, or privates:</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('My.app.BarMod', {
   *      override: 'Ext.foo.Bar',
   *
   *      statics: {
   *          method: function (x) {
   *              return this.callParent([x * 2]); // call Ext.foo.Bar.method
   *          }
   *      }
   *  });
   * </code></pre>
   *
   * <p>Starting in version 4.2.2, overrides can declare their <code>compatibility</code> based
   * on the framework version or on versions of other packages. For details on the
   * syntax and options for these checks, see <code><a href="#!/api/Ext-method-checkVersion" rel="Ext-method-checkVersion" class="docClass">Ext.checkVersion</a></code>.</p>
   *
   * <p>The simplest use case is to test framework version for compatibility:</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('App.overrides.grid.Panel', {
   *      override: '<a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>',
   *
   *      compatibility: '4.2.2', // only if framework version is 4.2.2
   *
   *      //...
   *  });
   * </code></pre>
   *
   * <p>An array is treated as an OR, so if any specs match, the override is
   * compatible.</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('App.overrides.some.Thing', {
   *      override: 'Foo.some.Thing',
   *
   *      compatibility: [
   *          '4.2.2',
   *          'foo&#64;1.0.1-1.0.2'
   *      ],
   *
   *      //...
   *  });
   * </code></pre>
   *
   * <p>To require that all specifications match, an object can be provided:</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('App.overrides.some.Thing', {
   *      override: 'Foo.some.Thing',
   *
   *      compatibility: {
   *          and: [
   *              '4.2.2',
   *              'foo&#64;1.0.1-1.0.2'
   *          ]
   *      },
   *
   *      //...
   *  });
   * </code></pre>
   *
   * <p>Because the object form is just a recursive check, these can be nested:</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('App.overrides.some.Thing', {
   *      override: 'Foo.some.Thing',
   *
   *      compatibility: {
   *          and: [
   *              '4.2.2',  // exactly version 4.2.2 of the framework *AND*
   *              {
   *                  // either (or both) of these package specs:
   *                  or: [
   *                      'foo&#64;1.0.1-1.0.2',
   *                      'bar&#64;3.0+'
   *                  ]
   *              }
   *          ]
   *      },
   *
   *      //...
   *  });
   * </code></pre>
   *
   * <p>IMPORTANT: An override is only included in a build if the class it overrides is
   * required. Otherwise, the override, like the target class, is not included. In
   * Sencha Cmd v4, the <code>compatibility</code> declaration can likewise be used to remove
   * incompatible overrides from a build.</p>
   * @param className The class name to create in string dot-namespaced format, for example:
   * 'My.very.awesome.Class', 'FeedViewer.plugin.CoolPager'
   * It is highly recommended to follow this simple convention:
   *  - The root and the class name are 'CamelCased'
   *  - Everything else is lower-cased
   * Pass <code>null</code> to create an anonymous class.
   * @param data The key - value pairs of properties to apply to this class. Property names can be of any valid
   * strings, except those in the reserved listed below:
   * <ul>
   * <li><a href="#!/api/Ext.Class-cfg-alias" rel="Ext.Class-cfg-alias" class="docClass">alias</a></li>
   * <li><a href="#!/api/Ext.Class-cfg-alternateClassName" rel="Ext.Class-cfg-alternateClassName" class="docClass">alternateClassName</a></li>
   * <li><a href="#!/api/Ext.Class-cfg-cachedConfig" rel="Ext.Class-cfg-cachedConfig" class="docClass">cachedConfig</a></li>
   * <li><a href="#!/api/Ext.Class-cfg-config" rel="Ext.Class-cfg-config" class="docClass">config</a></li>
   * <li><a href="#!/api/Ext.Class-cfg-extend" rel="Ext.Class-cfg-extend" class="docClass">extend</a></li>
   * <li><a href="#!/api/Ext.Class-cfg-inheritableStatics" rel="Ext.Class-cfg-inheritableStatics" class="docClass">inheritableStatics</a></li>
   * <li><a href="#!/api/Ext.Class-cfg-mixins" rel="Ext.Class-cfg-mixins" class="docClass">mixins</a></li>
   * <li><a href="#!/api/Ext.Class-cfg-override" rel="Ext.Class-cfg-override" class="docClass">override</a></li>
   * <li><a href="#!/api/Ext.Class-cfg-platformConfig" rel="Ext.Class-cfg-platformConfig" class="docClass">platformConfig</a></li>
   * <li><a href="#!/api/Ext.Class-cfg-privates" rel="Ext.Class-cfg-privates" class="docClass">privates</a></li>
   * <li><a href="#!/api/Ext.Class-cfg-requires" rel="Ext.Class-cfg-requires" class="docClass">requires</a></li>
   * <li><code>self</code></li>
   * <li><a href="#!/api/Ext.Class-cfg-singleton" rel="Ext.Class-cfg-singleton" class="docClass">singleton</a></li>
   * <li><a href="#!/api/Ext.Class-cfg-statics" rel="Ext.Class-cfg-statics" class="docClass">statics</a></li>
   * <li><a href="#!/api/Ext.Class-cfg-uses" rel="Ext.Class-cfg-uses" class="docClass">uses</a></li>
   * <li><a href="#!/api/Ext.Class-cfg-xtype" rel="Ext.Class-cfg-xtype" class="docClass">xtype</a> (for <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Components</a> only)</li>
   * </ul>
   * @param createdFn Callback to execute after the class is created, the execution scope of which
   * (<code>this</code>) will be the newly created class itself.
   */
  public native function define(className:String, data:Object, createdFn:Function = null):ext.Base;

  /**
   * Destroys all of the given objects. If arrays are passed, the elements of these
   * are destroyed recursively.
   *
   * <p>What it means to "destroy" an object depends on the type of object.</p>
   *
   * <ul>
   * <li><code>Array</code>: Each element of the array is destroyed recursively.</li>
   * <li><code>Object</code>: Any object with a <code>destroy</code> method will have that method called.</li>
   * </ul>
   * @param args Any number of objects or arrays.
   */
  public native function destroy(...args):void;

  /**
   * Destroys the specified named members of the given object using <code><a href="#!/api/Ext-method-destroy" rel="Ext-method-destroy" class="docClass">Ext.destroy</a></code>. These
   * properties will be set to <code>null</code>.
   * @param object The object who's properties you wish to destroy.
   * @param args One or more names of the properties to destroy and remove from the object.
   */
  public native function destroyMembers(object:Object, ...args):void;

  /**
   * Iterates an array or an iterable value and invoke the given callback function for each item.
   *
   * <pre><code>var countries = ['Vietnam', 'Singapore', 'United States', 'Russia'];
   *
   * <a href="#!/api/Ext.Array-method-each" rel="Ext.Array-method-each" class="docClass">Ext.Array.each</a>(countries, function(name, index, countriesItSelf) {
   *     console.log(name);
   * });
   *
   * var sum = function() {
   *     var sum = 0;
   *
   *     <a href="#!/api/Ext.Array-method-each" rel="Ext.Array-method-each" class="docClass">Ext.Array.each</a>(arguments, function(value) {
   *         sum += value;
   *     });
   *
   *     return sum;
   * };
   *
   * sum(1, 2, 3); // returns 6
   * </code></pre>
   *
   * <p>The iteration can be stopped by returning <code>false</code> from the callback function.<br/>
   * Returning <code>undefined</code> (i.e <code>return;</code>) will only exit the callback function and
   * proceed with the next iteration of the loop.</p>
   *
   * <pre><code><a href="#!/api/Ext.Array-method-each" rel="Ext.Array-method-each" class="docClass">Ext.Array.each</a>(countries, function(name, index, countriesItSelf) {
   *     if (name === 'Singapore') {
   *         return false; // break here
   *     }
   * });
   * </code></pre>
   *
   * <p><a href="#!/api/Ext-method-each" rel="Ext-method-each" class="docClass">Ext.each</a> is alias for <a href="#!/api/Ext.Array-method-each" rel="Ext.Array-method-each" class="docClass">Ext.Array.each</a></p>
   * @param iterable The value to be iterated. If this
   * argument is not iterable, the callback function is called once.
   * @param fn The callback function. If it returns <code>false</code>, the iteration
   * stops and this method returns the current <code>index</code>. Returning <code>undefined</code> (i.e
   * <code>return;</code>) will only exit the callback function and proceed with the next iteration
   * in the loop.
   * @param {Object} fn.item The item at the current <code>index</code> in the passed <code>array</code>
   * @param {Number} fn.index The current <code>index</code> within the <code>array</code>
   * @param {Array} fn.allItems The <code>array</code> itself which was passed as the first argument
   * @param {Boolean} fn.return Return <code>false</code> to stop iteration.
   * @param scope The scope (<code>this</code> reference) in which the specified function is executed.
   * @param reverse Reverse the iteration order (loop from the end to the beginning).
   * @default false
   * @return See description for the <code>fn</code> parameter.
   */
  public native function each(iterable:*, fn:Function, scope:Object = null, reverse:Boolean = false):Boolean;

  /**
   * Shorthand for <a href="#!/api/Ext.JSON-method-encode" rel="Ext.JSON-method-encode" class="docClass">Ext.JSON.encode</a>
   * @param o The variable to encode.
   * @return The JSON string.
   */
  public native function encode(o:Object):String;

  /**
   * Explicitly exclude files from being loaded. Useful when used in conjunction with a
   * broad include expression. Can be chained with more <code>require</code> and <code>exclude</code> methods,
   * for example:
   *
   * <pre><code><a href="#!/api/Ext-method-exclude" rel="Ext-method-exclude" class="docClass">Ext.exclude</a>('Ext.data.*').require('*');
   *
   * <a href="#!/api/Ext-method-exclude" rel="Ext-method-exclude" class="docClass">Ext.exclude</a>('widget.button*').require('widget.*');
   * </code></pre>
   * @return Contains <code>exclude</code>, <code>require</code> and <code>syncRequire</code> methods for chaining.
   */
  public native function exclude(excludes:*):Object;

  [Deprecated(replacement = "define", since = "4.0.0")]
  /**
   * This method deprecated. Use <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a> instead.
   * @return The subclass constructor from the <tt>overrides</tt> parameter, or a generated one if not provided.
   */
  public native function extend(superclass:Function, overrides:Object):Function;

  /**
   * A global factory method to instantiate a class from a config object. For example,
   * these two calls are equivalent:
   *
   * <pre><code><a href="#!/api/Ext-method-factory" rel="Ext-method-factory" class="docClass">Ext.factory</a>({ text: 'My Button' }, '<a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">Ext.Button</a>');
   * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">Ext.Button</a>', { text: 'My Button' });
   * </code></pre>
   *
   * <p>If an existing instance is also specified, it will be updated with the supplied config object. This is useful
   * if you need to either create or update an object, depending on if an instance already exists. For example:</p>
   *
   * <pre><code>var button;
   * button = <a href="#!/api/Ext-method-factory" rel="Ext-method-factory" class="docClass">Ext.factory</a>({ text: 'New Button' }, '<a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">Ext.Button</a>', button);     // Button created
   * button = <a href="#!/api/Ext-method-factory" rel="Ext-method-factory" class="docClass">Ext.factory</a>({ text: 'Updated Button' }, '<a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">Ext.Button</a>', button); // Button updated
   * </code></pre>
   * @param config The config object to instantiate or update an instance with.
   * @param classReference The class to instantiate from (if there is a default).
   * @param instance The instance to update.
   */
  public native function factory(config:Object, classReference:String = null, instance:Object = null, aliasNamespace:Object = null):void;

  /**
   * Shorthand for <a href="#!/api/Ext.GlobalEvents-method-fireEvent" rel="Ext.GlobalEvents-method-fireEvent" class="docClass">Ext.GlobalEvents.fireEvent</a>.
   * @since 6.2.0
   * @param eventName The name of the event to fire.
   * @param args Variable number of parameters are passed to handlers.
   * @return returns false if any of the handlers return false otherwise it returns true.
   */
  public native function fireEvent(eventName:String, ...args):Boolean;

  /**
   * Returns the first match to the given component query.
   * See <a href="#!/api/Ext.ComponentQuery-method-query" rel="Ext.ComponentQuery-method-query" class="docClass">Ext.ComponentQuery.query</a>.
   * @param selector The selector string to filter returned Component.
   * @param root The Container within which to perform the query.
   * If omitted, all Components within the document are included in the search.
   * <p>This parameter may also be an array of Components to filter according to the selector.</p>
   * @return The first matched Component or <code>null</code>.
   */
  public native function first(selector:String, root:ext.container.Container = null):ext.Component;

  [Deprecated(replacement = "flatten", since = "4.0.0")]
  /**
   * Old alias to <a href="#!/api/Ext.Array-method-flatten" rel="Ext.Array-method-flatten" class="docClass">Ext.Array.flatten</a>
   * @param array The array to flatten
   * @return The 1-d array.
   */
  public native function flatten(array:Array):Array;

  /**
   * Gets the globally shared flyweight Element, with the passed node as the active
   * element. Do not store a reference to this element - the dom node can be overwritten
   * by other code. <a href="#!/api/Ext-method-fly" rel="Ext-method-fly" class="docClass">fly</a> is alias for <a href="#!/api/Ext.dom.Element-static-method-fly" rel="Ext.dom.Element-static-method-fly" class="docClass">Ext.dom.Element.fly</a>.
   *
   * <p>Use this to make one-time references to DOM elements which are not going to be
   * accessed again either by application code, or by Ext's classes. If accessing an
   * element which will be processed regularly, then <a href="#!/api/Ext-method-get" rel="Ext-method-get" class="docClass">Ext.get</a> will be
   * more appropriate to take advantage of the caching provided by the
   * <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> class.</p>
   *
   * <p>If this method is called with and id or element that has already been cached by
   * a previous call to <a href="#!/api/Ext-method-get" rel="Ext-method-get" class="docClass">Ext.get</a>() it will return the cached Element instead of the
   * flyweight instance.</p>
   * @param dom The DOM node or <code>id</code>.
   * @param named Allows for creation of named reusable flyweights to prevent
   * conflicts (e.g. internally Ext uses "_global").
   * @return The shared Element object (or <code>null</code> if no matching
   * element was found).
   */
  public native function fly(dom:*, named:String = null):ext.dom.Element;

  /**
   * Retrieves <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> objects. <a href="#!/api/Ext-method-get" rel="Ext-method-get" class="docClass">get</a> is alias for <a href="#!/api/Ext.dom.Element-static-method-get" rel="Ext.dom.Element-static-method-get" class="docClass">Ext.dom.Element.get</a>.
   *
   * <p><strong>This method does not retrieve <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Component</a>s.</strong> This method retrieves <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a>
   * objects which encapsulate DOM elements. To retrieve a Component by its ID, use <a href="#!/api/Ext.ComponentManager-method-get" rel="Ext.ComponentManager-method-get" class="docClass">Ext.ComponentManager.get</a>.</p>
   *
   * <p>When passing an id, it should not include the <code>#</code> character that is used for a css selector.</p>
   *
   * <pre><code>// For an element with id 'foo'
   * <a href="#!/api/Ext-method-get" rel="Ext-method-get" class="docClass">Ext.get</a>('foo'); // Correct
   * <a href="#!/api/Ext-method-get" rel="Ext-method-get" class="docClass">Ext.get</a>('#foo'); // Incorrect
   * </code></pre>
   *
   * <p>Uses simple caching to consistently return the same object. Automatically fixes if an object was recreated with
   * the same id via AJAX or DOM.</p>
   * @param element The <code>id</code> of the node, a DOM Node or an existing Element.
   * @return The Element object (or <code>null</code> if no matching element was found).
   */
  public native function get(element:*):ext.dom.Element;

  /**
   * Returns the current document body as an <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a>.
   * @return The document body.
   */
  public native function getBody():ext.dom.Element;

  /**
   * Get the class of the provided object; returns null if it's not an instance
   * of any class created with <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>. This is usually invoked by the
   * shorthand <a href="#!/api/Ext-method-getClass" rel="Ext-method-getClass" class="docClass">getClass</a>.
   *
   * <pre><code>var component = new <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>();
   *
   * <a href="#!/api/Ext-method-getClass" rel="Ext-method-getClass" class="docClass">Ext.getClass</a>(component); // returns <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a>
   * </code></pre>
   * @return class
   */
  public native function getClass(object:Object):ext.ExtClass;

  /**
   * Get the name of the class by its reference or its instance. This is
   * usually invoked by the shorthand <a href="#!/api/Ext-method-getClassName" rel="Ext-method-getClassName" class="docClass">getClassName</a>.
   *
   * <pre><code><a href="#!/api/Ext.ClassManager-method-getName" rel="Ext.ClassManager-method-getName" class="docClass">Ext.ClassManager.getName</a>(<a href="#!/api/Ext.Action" rel="Ext.Action" class="docClass">Ext.Action</a>); // returns "<a href="#!/api/Ext.Action" rel="Ext.Action" class="docClass">Ext.Action</a>"
   * </code></pre>
   * @return className
   */
  public native function getClassName(object:*):String;

  /**
   * This is shorthand reference to <a href="#!/api/Ext.ComponentManager-method-get" rel="Ext.ComponentManager-method-get" class="docClass">Ext.ComponentManager.get</a>.
   * Looks up an existing <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Component</a> by <a href="#!/api/Ext.Component-cfg-id" rel="Ext.Component-cfg-id" class="docClass">id</a>
   * @param id The component <a href="#!/api/Ext.Component-cfg-id" rel="Ext.Component-cfg-id" class="docClass">id</a>
   * @return The Component, <code>undefined</code> if not found, or <code>null</code> if a
   * Class was found.
   */
  public native function getCmp(id:String):ext.Component;

  /**
   * Returns the current HTML document object as an <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a>.
   * Typically used for attaching event listeners to the document.  Note: since
   * the document object is not an HTMLElement many of the <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> methods
   * are not applicable and may throw errors if called on the returned
   * Element instance.
   * @return The document.
   */
  public native function getDoc():ext.dom.Element;

  /**
   * Return the dom node for the passed String (id), dom node, or <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.Element</a>.
   * Here are some examples:
   *
   * <pre><code>// gets dom node based on id
   * var elDom = <a href="#!/api/Ext-method-getDom" rel="Ext-method-getDom" class="docClass">Ext.getDom</a>('elId');
   *
   * // gets dom node based on the dom node
   * var elDom1 = <a href="#!/api/Ext-method-getDom" rel="Ext-method-getDom" class="docClass">Ext.getDom</a>(elDom);
   *
   * // If we don't know if we are working with an
   * // <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.Element</a> or a dom node use <a href="#!/api/Ext-method-getDom" rel="Ext-method-getDom" class="docClass">Ext.getDom</a>
   * function(el){
   *     var dom = <a href="#!/api/Ext-method-getDom" rel="Ext-method-getDom" class="docClass">Ext.getDom</a>(el);
   *     // do something with the dom node
   * }
   * </code></pre>
   *
   * <p><strong>Note:</strong> the dom node to be found actually needs to exist (be rendered, etc)
   * when this method is called to be successful.</p>
   */
  public native function getDom(el:*):js.HTMLElement;

  /**
   * Returns the current document head as an <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a>.
   * @return The document head.
   */
  public native function getHead():ext.dom.Element;

  /**
   * @return Namespace prefix if it's known, otherwise undefined
   */
  public native function getNamespace(className:String):String;

  /**
   * Returns the size of the browser scrollbars. This can differ depending on
   * operating system settings, such as the theme or font size.
   * @param force true to force a recalculation of the value.
   * @return An object containing scrollbar sizes.
   * <ul>
   *   <li><code>width</code> : Number
   * The width of the vertical scrollbar.
   *   </li>
   *   <li><code>height</code> : Number
   * The height of the horizontal scrollbar.
   *   </li>
   * </ul>
   */
  public native function getScrollbarSize(force:Boolean = false):Object;

  /**
   * Shortcut to <a href="#!/api/Ext.data.StoreManager-method-lookup" rel="Ext.data.StoreManager-method-lookup" class="docClass">Ext.data.StoreManager.lookup</a>.
   * @param store The id of the Store, or a Store instance, or a store configuration
   * @param defaultType The store type to create when used with store configuration and there
   * is no type specified on the config.
   */
  public native function getStore(store:*, defaultType:String = null):ext.data.Store;

  /**
   * Get the version number of the supplied package name; will return the version of
   * the framework.
   * @param packageName The package name, e.g., 'core', 'touch', 'ext'.
   * @return The version.
   */
  public native function getVersion(packageName:String = null):ext.Version;

  /**
   * Returns the current window object as an <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a>.
   * Typically used for attaching event listeners to the window.  Note: since
   * the window object is not an HTMLElement many of the <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> methods
   * are not applicable and may throw errors if called on the returned
   * Element instance.
   * @return The window.
   */
  public native function getWin():ext.dom.Element;

  [Deprecated(replacement = "htmlDecode")]
  /**
   * Old alias to <a href="#!/api/Ext.String-method-htmlDecode" rel="Ext.String-method-htmlDecode" class="docClass">Ext.String.htmlDecode</a>
   * @param value The string to decode.
   * @return The decoded text.
   */
  public native function htmlDecode(value:String):String;

  [Deprecated(replacement = "htmlEncode")]
  /**
   * Old alias to <a href="#!/api/Ext.String-method-htmlEncode" rel="Ext.String-method-htmlEncode" class="docClass">Ext.String.htmlEncode</a>
   * @param value The string to encode.
   * @return The encoded text.
   */
  public native function htmlEncode(value:String):String;

  /**
   * Generates unique ids. If the object/element is passes and it already has an <code>id</code>, it is unchanged.
   * @param o The object to generate an id for.
   * @param prefix The <code>id</code> prefix.
   * @default ext-gen
   * @return The generated <code>id</code>.
   */
  public native function id(o:Object = null, prefix:String = "ext-gen"):String;

  /**
   * Calls this function repeatedly at a given interval, optionally in a specific scope.
   *
   * <p><a href="#!/api/Ext-method-defer" rel="Ext-method-defer" class="docClass">Ext.defer</a> is alias for <a href="#!/api/Ext.Function-method-defer" rel="Ext.Function-method-defer" class="docClass">Ext.Function.defer</a></p>
   * @param fn The function to defer.
   * @param millis The number of milliseconds for the <code>setInterval</code> call
   * @param scope The scope (<code>this</code> reference) in which the function is executed.
   * <strong>If omitted, defaults to the browser window.</strong>
   * @param args Overrides arguments for the call. Defaults to the arguments passed by the caller.
   * @param appendArgs If <code>true</code> args are appended to call args instead of overriding,
   * or, if a number, then the args are inserted at the specified position.
   * @default false
   * @return The interval id that can be used with <code>clearInterval</code>.
   */
  public native function interval(fn:Function, millis:Number, scope:Object = null, args:Array = null, appendArgs:* = false):Number;

  /**
   * Returns <code>true</code> if the passed value is a JavaScript Array, <code>false</code> otherwise.
   * @param target The target to test.
   */
  public native function isArray(target:Object):Boolean;

  /**
   * Returns <code>true</code> if the passed value is a boolean.
   * @param value The value to test.
   */
  public native function isBoolean(value:Object):Boolean;

  /**
   * Returns <code>true</code> if the passed value is a JavaScript Date object, <code>false</code> otherwise.
   * @param object The object to test.
   */
  public native function isDate(object:Object):Boolean;

  /**
   * This method returns <code>true</code> if debug is enabled for the specified class. This is
   * done by checking the <code>Ext.debugConfig.hooks</code> config for the closest match to the
   * given <code>className</code>.
   * @param className The name of the class.
   * @return <code>true</code> if debug is enabled for the specified class.
   */
  public native function isDebugEnabled(className:String):Boolean;

  /**
   * Returns <code>true</code> if the passed value is defined.
   * @param value The value to test.
   */
  public native function isDefined(value:Object):Boolean;

  /**
   * Returns <code>true</code> if the passed value is an HTMLElement
   * @param value The value to test.
   */
  public native function isElement(value:Object):Boolean;

  /**
   * Returns true if the passed value is empty, false otherwise. The value is deemed to be empty if it is either:
   *
   * <ul>
   * <li><code>null</code></li>
   * <li><code>undefined</code></li>
   * <li>a zero-length array</li>
   * <li>a zero-length string (Unless the <code>allowEmptyString</code> parameter is set to <code>true</code>)</li>
   * </ul>
   * @param value The value to test.
   * @param allowEmptyString <code>true</code> to allow empty strings.
   * @default false
   */
  public native function isEmpty(value:Object, allowEmptyString:Boolean = false):Boolean;

  /**
   * Returns <code>true</code> if the passed value is a JavaScript Function, <code>false</code> otherwise.
   * @param value The value to test.
   */
  public native function isFunction(value:Object):Boolean;

  /**
   * Returns <code>true</code> if the passed value is iterable, that is, if elements of it are addressable using array
   * notation with numeric indices, <code>false</code> otherwise.
   *
   * <p>Arrays and function <code>arguments</code> objects are iterable. Also HTML collections such as <code>NodeList</code> and `HTMLCollection'
   * are iterable.</p>
   * @param value The value to test
   */
  public native function isIterable(value:Object):Boolean;

  /**
   * Returns 'true' if the passed value is a String that matches the MS Date JSON
   * encoding format.
   * @param value The string to test.
   */
  public native function isMSDate(value:String):Boolean;

  /**
   * Returns <code>true</code> if the passed value is a number. Returns <code>false</code> for non-finite numbers.
   * @param value The value to test.
   */
  public native function isNumber(value:Object):Boolean;

  /**
   * Validates that a value is numeric.
   * @param value Examples: 1, '1', '2.34'
   * @return True if numeric, false otherwise
   */
  public native function isNumeric(value:Object):Boolean;

  /**
   * Returns <code>true</code> if the passed value is a JavaScript Object, <code>false</code> otherwise.
   * @param value The value to test.
   */
  public native function isObject(value:Object):Boolean;

  /**
   * Returns <code>true</code> if the passed value is a JavaScript 'primitive', a string, number
   * or boolean.
   * @param value The value to test.
   */
  public native function isPrimitive(value:Object):Boolean;

  /**
   * Returns <code>true</code>if the passed value is a string.
   * @param value The value to test.
   */
  public native function isString(value:Object):Boolean;

  /**
   * Returns <code>true</code> if the passed value is a TextNode
   * @param value The value to test.
   */
  public native function isTextNode(value:Object):Boolean;

  /**
   * Iterates either an array or an object. This method delegates to
   * <a href="#!/api/Ext.Array-method-each" rel="Ext.Array-method-each" class="docClass">Ext.Array.each</a> if the given value is iterable, and <a href="#!/api/Ext.Object-method-each" rel="Ext.Object-method-each" class="docClass">Ext.Object.each</a> otherwise.
   * @param object The object or array to be iterated.
   * @param fn The function to be called for each iteration. See and <a href="#!/api/Ext.Array-method-each" rel="Ext.Array-method-each" class="docClass">Ext.Array.each</a> and
   * <a href="#!/api/Ext.Object-method-each" rel="Ext.Object-method-each" class="docClass">Ext.Object.each</a> for detailed lists of arguments passed to this function depending on the given object
   * type that is being iterated.
   * @param scope The scope (<code>this</code> reference) in which the specified function is executed.
   * Defaults to the object being iterated itself.
   */
  public native function iterate(object:*, fn:Function, scope:Object = null):void;

  /**
   * Logs a message. If a console is present it will be used. On Opera, the method
   * "opera.postError" is called. In other cases, the message is logged to an array
   * "Ext.log.out". An attached debugger can watch this array and view the log. The
   * log buffer is limited to a maximum of "Ext.log.max" entries (defaults to 250).
   *
   * <p>If additional parameters are passed, they are joined and appended to the message.
   * A technique for tracing entry and exit of a function is this:</p>
   *
   * <pre><code>function foo () {
   *     <a href="#!/api/Ext-method-log" rel="Ext-method-log" class="docClass">Ext.log</a>({ indent: 1 }, '&gt;&gt; foo');
   *
   *     // log statements in here or methods called from here will be indented
   *     // by one step
   *
   *     <a href="#!/api/Ext-method-log" rel="Ext-method-log" class="docClass">Ext.log</a>({ outdent: 1 }, '&lt;&lt; foo');
   * }
   * </code></pre>
   *
   * <p>This method does nothing in a release build.</p>
   * @param options The message to log or an options object with any
   * of the following properties:
   * <ul>
   * <li><code>msg</code>: The message to log (required).</li>
   * <li><code>level</code>: One of: "error", "warn", "info" or "log" (the default is "log").</li>
   * <li><code>dump</code>: An object to dump to the log as part of the message.</li>
   * <li><code>stack</code>: True to include a stack trace in the log.</li>
   * <li><code>indent</code>: Cause subsequent log statements to be indented one step.</li>
   * <li><code>outdent</code>: Cause this and following statements to be one step less indented.</li>
   * </ul>
   * @param message The message to log (required unless specified in
   * options object).
   */
  public native function log(options:* = undefined, ...message):void;

  [Deprecated(replacement = "max", since = "4.0.0")]
  /**
   * Old alias to <a href="#!/api/Ext.Array-method-max" rel="Ext.Array-method-max" class="docClass">Ext.Array.max</a>
   * @param array The Array from which to select the maximum value.
   * @param comparisonFn a function to perform the comparison which determines maximization.
   * If omitted the ">" operator will be used.
   * <strong>Note:</strong> gt = 1; eq = 0; lt = -1
   * @param comparisonFn.max Current maximum value.
   * @param comparisonFn.item The value to compare with the current maximum.
   * @return maxValue The maximum value.
   */
  public native function max(array:*, comparisonFn:Function = null):Object;

  [Deprecated(replacement = "mean", since = "4.0.0")]
  /**
   * Old alias to <a href="#!/api/Ext.Array-method-mean" rel="Ext.Array-method-mean" class="docClass">Ext.Array.mean</a>
   * @param array The Array to calculate the mean value of.
   * @return The mean.
   */
  public native function mean(array:Array):Number;

  /**
   * A convenient alias method for <a href="#!/api/Ext.Object-method-merge" rel="Ext.Object-method-merge" class="docClass">Ext.Object.merge</a>.
   * @param destination The object into which all subsequent objects are merged.
   * @param object Any number of objects to merge into the destination.
   * @return merged The destination object with all passed objects merged in.
   */
  public native function merge(destination:Object, ...object):Object;

  [Deprecated(replacement = "min", since = "4.0.0")]
  /**
   * Old alias to <a href="#!/api/Ext.Array-method-min" rel="Ext.Array-method-min" class="docClass">Ext.Array.min</a>
   * @param array The Array from which to select the minimum value.
   * @param comparisonFn a function to perform the comparison which determines minimization.
   * If omitted the "&lt;" operator will be used.
   * <strong>Note:</strong> gt = 1; eq = 0; lt = -1
   * @param comparisonFn.min Current minimum value.
   * @param comparisonFn.item The value to compare with the current minimum.
   * @return minValue The minimum value.
   */
  public native function min(array:*, comparisonFn:Function = null):Object;

  /**
   * Creates namespaces to be used for scoping variables and classes so that they are not global.
   * Specifying the last node of a namespace implicitly creates all other nodes. Usage:
   *
   * <pre><code><a href="#!/api/Ext-method-namespace" rel="Ext-method-namespace" class="docClass">Ext.namespace</a>('Company', 'Company.data');
   *
   * // equivalent and preferable to the above syntax
   * <a href="#!/api/Ext-method-ns" rel="Ext-method-ns" class="docClass">Ext.ns</a>('Company.data');
   *
   * Company.Widget = function() { ... };
   *
   * Company.data.CustomStore = function(config) { ... };
   * </code></pre>
   * @return The (last) namespace object created.
   */
  public native function namespace(...namespaces):Object;

  /**
   * Returns the current timestamp.
   * @return Milliseconds since UNIX epoch.
   */
  public native function now():Number;

  /**
   * Convenient alias for <a href="#!/api/Ext-method-namespace" rel="Ext-method-namespace" class="docClass">Ext.namespace</a>.
   * @return The (last) namespace object created.
   */
  public native function ns(...namespaces):Object;

  [Deprecated(replacement = "from", since = "4.0.0")]
  /**
   * Validate that a value is numeric and convert it to a number if necessary. Returns the specified default value if
   * it is not.
   *
   * <pre><code><a href="#!/api/Ext.Number-method-from" rel="Ext.Number-method-from" class="docClass">Ext.Number.from</a>('1.23', 1); // returns 1.23
   * <a href="#!/api/Ext.Number-method-from" rel="Ext.Number-method-from" class="docClass">Ext.Number.from</a>('abc', 1); // returns 1
   * </code></pre>
   * @param defaultValue The value to return if the original value is non-numeric
   * @return value, if numeric, defaultValue otherwise
   */
  public native function num(value:Object, defaultValue:Number):Number;

  /**
   * Shorthand for <a href="#!/api/Ext.GlobalEvents-method-addListener" rel="Ext.GlobalEvents-method-addListener" class="docClass">Ext.GlobalEvents.addListener</a>.
   * @param eventName The name of the event to listen for.
   * May also be an object who's property names are event names.
   * @param fn The method the event invokes or the <em>name</em> of
   * the method within the specified <code>scope</code>.  Will be called with arguments
   * given to <a href="#!/api/Ext.util.Observable-method-fireEvent" rel="Ext.util.Observable-method-fireEvent" class="docClass">Ext.util.Observable.fireEvent</a> plus the <code>options</code> parameter described
   * below.
   * @param scope The scope (<code>this</code> reference) in which the handler function is
   * executed. <strong>If omitted, defaults to the object which fired the event.</strong>
   * @param options An object containing handler configuration.
   * <p><strong>Note:</strong> The options object will also be passed as the last argument to every
   * event handler.</p>
   * <p>This object may contain any of the following properties:</p>
   * @param {Object} options.scope The scope (<code>this</code> reference) in which the handler function is executed. <strong>If omitted,
   *   defaults to the object which fired the event.</strong>
   * @param {Number} options.delay The number of milliseconds to delay the invocation of the handler after the event
   *   fires.
   * @param {Boolean} options.single True to add a handler to handle just the next firing of the event, and then remove
   *   itself.
   * @param {Number} options.buffer Causes the handler to be scheduled to run in an <a href="#!/api/Ext.util.DelayedTask" rel="Ext.util.DelayedTask" class="docClass">Ext.util.DelayedTask</a> delayed
   *   by the specified number of milliseconds. If the event fires again within that time,
   *   the original handler is <em>not</em> invoked, but the new handler is scheduled in its place.
   * @param {Number} options.onFrame Causes the handler to be scheduled to run at the next
   *   <a href="#!/api/Ext.Function-method-requestAnimationFrame" rel="Ext.Function-method-requestAnimationFrame" class="docClass">animation frame event</a>. If the
   *   event fires again before that time, the handler is not rescheduled - the handler
   *   will only be called once when the next animation frame is fired, with the last set
   *   of arguments passed.
   * @param {ext.util.Observable} options.target Only call the handler if the event was fired on the target Observable, <em>not</em> if the
   *   event was bubbled up from a child Observable.
   * @param {String} options.element <strong>This option is only valid for listeners bound to <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Components</a>.</strong>
   *   The name of a Component property which references an <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">element</a>
   *   to add a listener to.
   * <p>  This option is useful during Component construction to add DOM event listeners to
   *   elements of <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Components</a> which will exist only after the
   *   Component is rendered.</p>
   * <p>  For example, to add a click listener to a Panel's body:</p>
   * <pre><code>  var panel = new <a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a>({
   *       title: 'The title',
   *       listeners: {
   *           click: this.handlePanelClick,
   *           element: 'body'
   *       }
   *   });
   * </code></pre>
   * <p>In order to remove listeners attached using the element, you'll need to reference
   * the element itself as seen below.</p>
   * <pre><code> panel.body.un(...)
   * </code></pre>
   * @param {String} [options.delegate] A simple selector to filter the event target or look for a descendant of the target.
   * <p>  The "delegate" option is only available on <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> instances (or
   *   when attaching a listener to a <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> via a Component using the
   *   element option).</p>
   * <p>  See the <em>delegate</em> example below.</p>
   * @param {Boolean} [options.capture] When set to <code>true</code>, the listener is fired in the capture phase of the event propagation
   *  sequence, instead of the default bubble phase.
   * <p>  The <code>capture</code> option is only available on <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> instances (or
   *   when attaching a listener to a <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> via a Component using the
   *   element option).</p>
   * @param {Boolean} [options.stopPropagation] <strong>This option is only valid for listeners bound to <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Elements</a>.</strong>
   *   <code>true</code> to call <a href="#!/api/Ext.event.Event-method-stopPropagation" rel="Ext.event.Event-method-stopPropagation" class="docClass">stopPropagation</a> on the event object
   *   before firing the handler.
   * @param {Boolean} [options.preventDefault] <strong>This option is only valid for listeners bound to <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Elements</a>.</strong>
   *   <code>true</code> to call <a href="#!/api/Ext.event.Event-method-preventDefault" rel="Ext.event.Event-method-preventDefault" class="docClass">preventDefault</a> on the event object
   *   before firing the handler.
   * @param {Boolean} [options.stopEvent] <strong>This option is only valid for listeners bound to <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Elements</a>.</strong>
   *   <code>true</code> to call <a href="#!/api/Ext.event.Event-method-stopEvent" rel="Ext.event.Event-method-stopEvent" class="docClass">stopEvent</a> on the event object
   *   before firing the handler.
   * @param {Array} [options.args] Optional arguments to pass to the handler function. Any additional arguments
   *   passed to <a href="#!/api/Ext.util.Observable-method-fireEvent" rel="Ext.util.Observable-method-fireEvent" class="docClass">fireEvent</a> will be appended
   *   to these arguments.
   * @param {Boolean} [options.destroyable] When specified as <code>true</code>, the function returns a <code>destroyable</code> object. An object
   *   which implements the <code>destroy</code> method which removes all listeners added in this call.
   *   This syntax can be a helpful shortcut to using <a href="#!/api/Ext-method-un" rel="Ext-method-un" class="docClass">un</a>; particularly when
   *   removing multiple listeners.  <em>NOTE</em> - not compatible when using the <em>element</em>
   *   option.  See <a href="#!/api/Ext-method-un" rel="Ext-method-un" class="docClass">un</a> for the proper syntax for removing listeners added using the
   *   <em>element</em> config.
   * @default false
   * @param {Number} [options.priority] An optional numeric priority that determines the order in which event handlers
   *   are run. Event handlers with no priority will be run as if they had a priority
   *   of 0. Handlers with a higher priority will be prioritized to run sooner than
   *   those with a lower priority.  Negative numbers can be used to set a priority
   *   lower than the default. Internally, the framework uses a range of 1000 or
   *   greater, and -1000 or lesser for handlers that are intended to run before or
   *   after all others, so it is recommended to stay within the range of -999 to 999
   *   when setting the priority of event handlers in application-level code.
   *   A priority must be an integer to be valid.  Fractional values are reserved for
   *   internal framework use.
   * @param {String} [options.order] A legacy option that is provided for backward compatibility.
   *   It is recommended to use the <code>priority</code> option instead.  Available options are:
   * <ul>
   * <li><code>'before'</code>: equal to a priority of <code>100</code></li>
   * <li><code>'current'</code>: equal to a priority of <code>0</code> or default priority</li>
   * <li><code>'after'</code>: equal to a priority of <code>-100</code></li>
   * </ul>
   * @default 'current'
   * @param order A shortcut for the <code>order</code> event option.  Provided for backward compatibility.
   *   Please use the <code>priority</code> event option instead.
   * <p><strong>Combining Options</strong></p>
   * <p>Using the options argument, it is possible to combine different types of listeners:</p>
   * <p>A delayed, one-time listener.</p>
   * <pre><code>myPanel.on('hide', this.handleClick, this, {
   *     single: true,
   *     delay: 100
   * });
   * </code></pre>
   * <p><strong>Attaching multiple handlers in 1 call</strong></p>
   * <p>The method also allows for a single argument to be passed which is a config object
   * containing properties which specify multiple handlers and handler configs.</p>
   * <pre><code>grid.on({
   *     itemclick: 'onItemClick',
   *     itemcontextmenu: grid.onItemContextmenu,
   *     destroy: {
   *         fn: function () {
   *             // function called within the 'altCmp' scope instead of grid
   *         },
   *         scope: altCmp // unique scope for the destroy handler
   *     },
   *     scope: grid       // default scope - provided for example clarity
   * });
   * </code></pre>
   * <p><strong>Delegate</strong></p>
   * <p>This is a configuration option that you can pass along when registering a handler for
   * an event to assist with event delegation. By setting this configuration option
   * to a simple selector, the target element will be filtered to look for a
   * descendant of the target. For example:</p>
   * <pre><code>var panel = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>({
   *     xtype: 'panel',
   *     renderTo: document.body,
   *     title: 'Delegate Handler Example',
   *     frame: true,
   *     height: 220,
   *     width: 220,
   *     html: '&lt;h1 class="myTitle"&gt;BODY TITLE&lt;/h1&gt;Body content'
   * });
   * // The click handler will only be called when the click occurs on the
   * // delegate: h1.myTitle ("h1" tag with class "myTitle")
   * panel.on({
   *     click: function (e) {
   *         console.log(e.getTarget().innerHTML);
   *     },
   *     element: 'body',
   *     delegate: 'h1.myTitle'
   *  });
   * </code></pre>
   * @default 'current'
   * @return <strong>Only when the <code>destroyable</code> option is specified. </strong>
   * <p> A <code>Destroyable</code> object. An object which implements the <code>destroy</code> method which removes
   *  all listeners added in this call. For example:</p>
   * <pre><code>this.btnListeners =  = myButton.on({
   *     destroyable: true
   *     mouseover:   function() { console.log('mouseover'); },
   *     mouseout:    function() { console.log('mouseout'); },
   *     click:       function() { console.log('click'); }
   * });
   * </code></pre>
   * <p>And when those listeners need to be removed:</p>
   * <pre><code><a href="#!/api/Ext-method-destroy" rel="Ext-method-destroy" class="docClass">Ext.destroy</a>(this.btnListeners);
   * </code></pre>
   * <p>or</p>
   * <pre><code>this.btnListeners.destroy();
   * </code></pre>
   */
  public native function on(eventName:*, fn:* = undefined, scope:Object = null, options:Object = null, order:String = 'current'):Object;

  /**
   * Adds a listener to be notified when the document is ready (before onload and before
   * images are loaded).
   * @param fn The method to call.
   * @param scope The scope (<code>this</code> reference) in which the handler function
   * executes. Defaults to the browser window.
   * @param options An object with extra options.
   * @param {Number} [options.delay] A number of milliseconds to delay.
   * @default 0
   * @param {Number} [options.priority] Relative priority of this callback. A larger
   * number will result in the callback being sorted before the others.  Priorities
   * 1000 or greater and -1000 or lesser are reserved for internal framework use only.
   * @default 0
   * @param {Boolean} [options.dom] Pass <code>true</code> to only wait for DOM ready, <code>false</code>
   * means full Framework and DOM readiness.
   * numbers are reserved.
   * @default false
   */
  public native function onReady(fn:Function, scope:Object = null, options:Object = null):void;

  /**
   * Overrides members of the specified <code>target</code> with the given values.
   *
   * <p>If the <code>target</code> is a class declared using <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>, the
   * <code>override</code> method of that class is called (see <a href="#!/api/Ext.Base-static-method-override" rel="Ext.Base-static-method-override" class="docClass">Ext.Base.override</a>) given
   * the <code>overrides</code>.</p>
   *
   * <p>If the <code>target</code> is a function, it is assumed to be a constructor and the contents
   * of <code>overrides</code> are applied to its <code>prototype</code> using <a href="#!/api/Ext-method-apply" rel="Ext-method-apply" class="docClass">Ext.apply</a>.</p>
   *
   * <p>If the <code>target</code> is an instance of a class declared using <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>,
   * the <code>overrides</code> are applied to only that instance. In this case, methods are
   * specially processed to allow them to use <a href="#!/api/Ext.Base-method-callParent" rel="Ext.Base-method-callParent" class="docClass">Ext.Base.callParent</a>.</p>
   *
   * <pre><code> var panel = new <a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.Panel</a>({ ... });
   *
   *  <a href="#!/api/Ext-method-override" rel="Ext-method-override" class="docClass">Ext.override</a>(panel, {
   *      initComponent: function () {
   *          // extra processing...
   *
   *          this.callParent();
   *      }
   *  });
   * </code></pre>
   *
   * <p>If the <code>target</code> is none of these, the <code>overrides</code> are applied to the <code>target</code>
   * using <a href="#!/api/Ext-method-apply" rel="Ext-method-apply" class="docClass">Ext.apply</a>.</p>
   *
   * <p>Please refer to <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a> and <a href="#!/api/Ext.Base-static-method-override" rel="Ext.Base-static-method-override" class="docClass">Ext.Base.override</a> for
   * further details.</p>
   * @param target The target to override.
   * @param overrides The properties to add or replace on <code>target</code>.
   */
  public native function override_(target:Object, overrides:Object):void;

  /**
   * Create a new function from the provided <code>fn</code>, the arguments of which are pre-set to <code>args</code>.
   * New arguments passed to the newly created callback when it's invoked are appended after the pre-set ones.
   * This is especially useful when creating callbacks.
   *
   * <p>For example:</p>
   *
   * <pre><code>var originalFunction = function(){
   *     alert(<a href="#!/api/Ext.Array-method-from" rel="Ext.Array-method-from" class="docClass">Ext.Array.from</a>(arguments).join(' '));
   * };
   *
   * var callback = <a href="#!/api/Ext.Function-method-pass" rel="Ext.Function-method-pass" class="docClass">Ext.Function.pass</a>(originalFunction, ['Hello', 'World']);
   *
   * callback(); // alerts 'Hello World'
   * callback('by Me'); // alerts 'Hello World by Me'
   * </code></pre>
   *
   * <p><a href="#!/api/Ext-method-pass" rel="Ext-method-pass" class="docClass">Ext.pass</a> is alias for <a href="#!/api/Ext.Function-method-pass" rel="Ext.Function-method-pass" class="docClass">Ext.Function.pass</a></p>
   * @param fn The original function.
   * @param args The arguments to pass to new callback.
   * @param scope The scope (<code>this</code> reference) in which the function is executed.
   * @return The new callback function.
   */
  public native function pass(fn:Function, args:Array, scope:Object = null):Function;

  [Deprecated(replacement = "pluck", since = "4.0.0")]
  /**
   * Old alias to <a href="#!/api/Ext.Array-method-pluck" rel="Ext.Array-method-pluck" class="docClass">Ext.Array.pluck</a>
   * @param array The Array of items to pluck the value from.
   * @param propertyName The property name to pluck from each element.
   * @return The value from each item in the Array.
   */
  public native function pluck(array:*, propertyName:String):Array;

  /**
   * Selects child nodes of a given root based on the passed CSS selector.
   * @param selector The CSS selector.
   * @param asDom <code>false</code> to return an array of <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a>
   * @default true
   * @param root The root element of the query or id of
   * the root
   * @return An Array of elements that match
   * the selector.  If there are no matches, an empty Array is returned.
   */
  public native function query(selector:String, asDom:Boolean = true, root:* = undefined):*;

  /**
   * Raise an error that can include additional data and supports automatic console logging
   * if available. You can pass a string error message or an object with the <code>msg</code> attribute
   * which will be used as the error message. The object can contain any other name-value
   * attributes (or objects) to be logged along with the error.
   *
   * <p>Note that after displaying the error message a JavaScript error will ultimately be
   * thrown so that execution will halt.</p>
   *
   * <p>Example usage:</p>
   *
   * <pre><code><a href="#!/api/Ext-method-raise" rel="Ext-method-raise" class="docClass">Ext.raise</a>('A simple string error message');
   *
   * // or...
   *
   * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Ext.Foo', {
   *     doSomething: function(option){
   *         if (someCondition === false) {
   *             <a href="#!/api/Ext-method-raise" rel="Ext-method-raise" class="docClass">Ext.raise</a>({
   *                 msg: 'You cannot do that!',
   *                 option: option,   // whatever was passed into the method
   *                 code: 100 // other arbitrary info
   *             });
   *         }
   *     }
   * });
   * </code></pre>
   * @param err The error message string, or an object containing the
   * attribute "msg" that will be used as the error message. Any other data included in the
   * object will also be logged to the browser console, if available.
   */
  public native function raise(err:*):void;

  /**
   * Creates a new store for the given id and config, then registers it with the <a href="#!/api/Ext.data.StoreManager" rel="Ext.data.StoreManager" class="docClass">Store Manager</a>.
   * Sample usage:
   *
   * <pre><code><a href="#!/api/Ext-method-regStore" rel="Ext-method-regStore" class="docClass">Ext.regStore</a>('AllUsers', {
   *     model: 'User'
   * });
   *
   * // the store can now easily be used throughout the application
   * new Ext.List({
   *     store: 'AllUsers',
   *     ... other config
   * });
   * </code></pre>
   * @param id The id to set on the new store
   * @param config The store config
   */
  public native function regStore(id:String, config:Object):void;

  /**
   * Removes an HTMLElement from the document.  If the HTMLElement was previously
   * cached by a call to <a href="#!/api/Ext-method-get" rel="Ext-method-get" class="docClass">Ext.get</a>(), removeNode will call the <a href="#!/api/Ext.dom.Element-method-destroy" rel="Ext.dom.Element-method-destroy" class="docClass">destroy</a> method of the <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> instance, which removes all DOM
   * event listeners, and deletes the cache reference.
   * @param node The node to remove
   */
  public native function removeNode(node:js.HTMLElement):void;

  /**
   * Loads all classes by the given names and all their direct dependencies; optionally
   * executes the given callback function when finishes, within the optional scope.
   * @param expressions The class, classes or wildcards to load.
   * @param fn The callback function.
   * @param scope The execution scope (<code>this</code>) of the callback function.
   */
  public native function require(expressions:*, fn:Function = null, scope:Object = null):void;

  /**
   * Resolves a resource URL that may contain a resource pool identifier token at the
   * front. The tokens are formatted as HTML tags "&lt;poolName&#64;packageName&gt;" followed
   * by a normal relative path. This token is only processed if present at the first
   * character of the given string.
   *
   * <p>These tokens are parsed and the pieces are then passed to the
   * getResourcePath method.</p>
   *
   * <p>For example:</p>
   *
   * <pre><code> [{
   *      xtype: 'image',
   *      src: '&lt;shared&gt;images/foo.png'
   *  },{
   *      xtype: 'image',
   *      src: '&lt;&#64;package&gt;images/foo.png'
   *  },{
   *      xtype: 'image',
   *      src: '&lt;shared&#64;package&gt;images/foo.png'
   *  }]
   * </code></pre>
   *
   * <p>In the above example, "shared" is the name of a Sencha Cmd resource pool and
   * "package" is the name of a Sencha Cmd package.</p>
   * @since 6.0.1
   * @param url The URL that may contain a resource pool token at the front.
   */
  public native function resolveResource(url:String):String;

  /**
   * Resumes layout activity in the whole framework.
   *
   * <p><a href="#!/api/Ext-method-suspendLayouts" rel="Ext-method-suspendLayouts" class="docClass">suspendLayouts</a> is alias of <a href="#!/api/Ext.Component-method-suspendLayouts" rel="Ext.Component-method-suspendLayouts" class="docClass">Ext.Component.suspendLayouts</a>.</p>
   * @param flush <code>true</code> to perform all the pending layouts. This can also be
   * achieved by calling <a href="#!/api/Ext.Component-static-method-flushLayouts" rel="Ext.Component-static-method-flushLayouts" class="docClass">flushLayouts</a> directly.
   * @default false
   */
  public native function resumeLayouts(flush:Boolean = false):void;

  /**
   * A reusable function which returns the value of <code>getId()</code> called upon a single passed parameter.
   * Useful when creating a <a href="#!/api/Ext.util.MixedCollection" rel="Ext.util.MixedCollection" class="docClass">Ext.util.MixedCollection</a> of objects keyed by an identifier returned from a <code>getId</code> method.
   */
  public native function returnId(o:Object):void;

  /**
   * A reusable function which returns <code>true</code>.
   */
  public native function returnTrue():void;

  /**
   * Selects elements based on the passed CSS selector to enable
   * <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Element</a> methods to be applied to many related
   * elements in one statement through the returned
   * <a href="#!/api/Ext.dom.CompositeElementLite" rel="Ext.dom.CompositeElementLite" class="docClass">CompositeElementLite</a> object.
   * @param selector The CSS selector or an array of
   * elements
   * @param composite Return a CompositeElement as opposed to
   * a CompositeElementLite. Defaults to false.
   * @default false
   * @param root The root element of the query or id of
   * the root
   */
  public native function select(selector:*, composite:Boolean = false, root:* = undefined):*;

  /**
   * Sets the default font-family to use for components that support a <code>glyph</code> config.
   * @param fontFamily The name of the font-family
   */
  public native function setGlyphFontFamily(fontFamily:String):void;

  /**
   * Set version number for the given package name.
   * @param packageName The package name, e.g. 'core', 'touch', 'ext'.
   * @param version The version, e.g. '1.2.3alpha', '2.4.0-dev'.
   */
  public native function setVersion(packageName:String, version:*):ext.SExt;

  [Deprecated(replacement = "sum", since = "4.0.0")]
  /**
   * Old alias to <a href="#!/api/Ext.Array-method-sum" rel="Ext.Array-method-sum" class="docClass">Ext.Array.sum</a>
   * @param array The Array to calculate the sum value of.
   * @return The sum.
   */
  public native function sum(array:Array):Number;

  /**
   * Stops layouts from happening in the whole framework.
   *
   * <p>It's useful to suspend the layout activity while updating multiple components and
   * containers:</p>
   *
   * <pre><code><a href="#!/api/Ext-method-suspendLayouts" rel="Ext-method-suspendLayouts" class="docClass">Ext.suspendLayouts</a>();
   * // batch of updates...
   * <a href="#!/api/Ext-method-resumeLayouts" rel="Ext-method-resumeLayouts" class="docClass">Ext.resumeLayouts</a>(true);
   * </code></pre>
   *
   * <p><a href="#!/api/Ext-method-suspendLayouts" rel="Ext-method-suspendLayouts" class="docClass">suspendLayouts</a> is alias of <a href="#!/api/Ext.Component-method-suspendLayouts" rel="Ext.Component-method-suspendLayouts" class="docClass">Ext.Component.suspendLayouts</a>.</p>
   *
   * <p>See also <a href="#!/api/Ext-method-batchLayouts" rel="Ext-method-batchLayouts" class="docClass">batchLayouts</a> for more abstract way of doing this.</p>
   */
  public native function suspendLayouts():void;

  /**
   * Synchronously loads all classes by the given names and all their direct dependencies; optionally
   * executes the given callback function when finishes, within the optional scope.
   * @param expressions The class, classes or wildcards to load.
   * @param fn The callback function.
   * @param scope The execution scope (<code>this</code>) of the callback function.
   */
  public native function syncRequire(expressions:*, fn:Function = null, scope:Object = null):void;

  /**
   * Returns the current high-resolution timestamp.
   * @since 6.0.1
   * @return Milliseconds ellapsed since arbitrary epoch.
   */
  public native function ticks():Number;

  /**
   * Converts any iterable (numeric indices and a length property) into a true array.
   *
   * <pre><code>function test() {
   *     var args = <a href="#!/api/Ext.Array-method-toArray" rel="Ext.Array-method-toArray" class="docClass">Ext.Array.toArray</a>(arguments),
   *         fromSecondToLastArgs = <a href="#!/api/Ext.Array-method-toArray" rel="Ext.Array-method-toArray" class="docClass">Ext.Array.toArray</a>(arguments, 1);
   *
   *     alert(args.join(' '));
   *     alert(fromSecondToLastArgs.join(' '));
   * }
   *
   * test('just', 'testing', 'here'); // alerts 'just testing here';
   *                                  // alerts 'testing here';
   *
   * <a href="#!/api/Ext.Array-method-toArray" rel="Ext.Array-method-toArray" class="docClass">Ext.Array.toArray</a>(document.getElementsByTagName('div')); // will convert the NodeList into an array
   * <a href="#!/api/Ext.Array-method-toArray" rel="Ext.Array-method-toArray" class="docClass">Ext.Array.toArray</a>('splitted'); // returns ['s', 'p', 'l', 'i', 't', 't', 'e', 'd']
   * <a href="#!/api/Ext.Array-method-toArray" rel="Ext.Array-method-toArray" class="docClass">Ext.Array.toArray</a>('splitted', 0, 3); // returns ['s', 'p', 'l']
   * </code></pre>
   *
   * <p><a href="#!/api/Ext-method-toArray" rel="Ext-method-toArray" class="docClass">Ext.toArray</a> is alias for <a href="#!/api/Ext.Array-method-toArray" rel="Ext.Array-method-toArray" class="docClass">Ext.Array.toArray</a></p>
   * @param iterable the iterable object to be turned into a true Array.
   * @param start a zero-based index that specifies the start of extraction.
   * @default 0
   * @param end a 1-based index that specifies the end of extraction.
   * @default -1
   */
  public native function toArray(iterable:Object, start:Number = 0, end:Number = -1):Array;

  /**
   * Returns the type of the given variable in string format. List of possible values are:
   *
   * <ul>
   * <li><code>undefined</code>: If the given value is <code>undefined</code></li>
   * <li><code>null</code>: If the given value is <code>null</code></li>
   * <li><code>string</code>: If the given value is a string</li>
   * <li><code>number</code>: If the given value is a number</li>
   * <li><code>boolean</code>: If the given value is a boolean value</li>
   * <li><code>date</code>: If the given value is a <code>Date</code> object</li>
   * <li><code>function</code>: If the given value is a function reference</li>
   * <li><code>object</code>: If the given value is an object</li>
   * <li><code>array</code>: If the given value is an array</li>
   * <li><code>regexp</code>: If the given value is a regular expression</li>
   * <li><code>element</code>: If the given value is a DOM Element</li>
   * <li><code>textnode</code>: If the given value is a DOM text node and contains something other than whitespace</li>
   * <li><code>whitespace</code>: If the given value is a DOM text node and contains only whitespace</li>
   * </ul>
   */
  public native function typeOf(value:Object):String;

  /**
   * Shorthand for <a href="#!/api/Ext.GlobalEvents-method-removeListener" rel="Ext.GlobalEvents-method-removeListener" class="docClass">Ext.GlobalEvents.removeListener</a>.
   * @param eventName The type of event the handler was associated with.
   * @param fn The handler to remove. <strong>This must be a reference to the function
   * passed into the
   * <a href="#!/api/Ext.util.Observable-method-addListener" rel="Ext.util.Observable-method-addListener" class="docClass">addListener</a> call.</strong>
   * @param scope The scope originally specified for the handler. It
   * must be the same as the scope argument specified in the original call to
   * <a href="#!/api/Ext.util.Observable-method-addListener" rel="Ext.util.Observable-method-addListener" class="docClass">Ext.util.Observable.addListener</a> or the listener will not be removed.
   * <p><strong>Convenience Syntax</strong></p>
   * <p>You can use the <a href="#!/api/Ext.util.Observable-method-addListener" rel="Ext.util.Observable-method-addListener" class="docClass">addListener</a>
   * <code>destroyable: true</code> config option in place of calling un().  For example:</p>
   * <pre><code>var listeners = cmp.on({
   *     scope: cmp,
   *     afterrender: cmp.onAfterrender,
   *     beforehide: cmp.onBeforeHide,
   *     destroyable: true
   * });
   * // Remove listeners
   * listeners.destroy();
   * // or
   * cmp.un(
   *     scope: cmp,
   *     afterrender: cmp.onAfterrender,
   *     beforehide: cmp.onBeforeHide
   * );
   * </code></pre>
   * <p><strong>Exception - DOM event handlers using the element config option</strong></p>
   * <p>You must go directly through the element to detach an event handler attached using
   * the <a href="#!/api/Ext.util.Observable-method-addListener" rel="Ext.util.Observable-method-addListener" class="docClass">addListener</a> <em>element</em> option.</p>
   * <pre><code>panel.on({
   *     element: 'body',
   *     click: 'onBodyCLick'
   * });
   * panel.body.un({
   *     click: 'onBodyCLick'
   * });
   * </code></pre>
   */
  public native function un(eventName:String, fn:Function, scope:Object = null):void;

  [Deprecated(replacement = "unique", since = "4.0.0")]
  /**
   * Old alias to <a href="#!/api/Ext.Array-method-unique" rel="Ext.Array-method-unique" class="docClass">Ext.Array.unique</a>
   * @return results
   */
  public native function unique(array:Array):Array;

  [Deprecated(replacement = "urlAppend")]
  /**
   * Old alias to <a href="#!/api/Ext.String-method-urlAppend" rel="Ext.String-method-urlAppend" class="docClass">Ext.String.urlAppend</a>
   * @param url The URL to append to.
   * @param string The content to append to the URL.
   * @return The resulting URL
   */
  public native function urlAppend(url:String, string:String):String;

  [Deprecated(replacement = "fromQueryString", since = "4.0.0")]
  /**
   * Alias for <a href="#!/api/Ext.Object-method-fromQueryString" rel="Ext.Object-method-fromQueryString" class="docClass">Ext.Object.fromQueryString</a>.
   * @param queryString The query string to decode
   * @param recursive Whether or not to recursively decode the string. This format is supported by
   * PHP / Ruby on Rails servers and similar.
   * @default false
   */
  public native function urlDecode(queryString:String, recursive:Boolean = false):Object;

  [Deprecated(replacement = "toQueryString", since = "4.0.0")]
  /**
   * Takes an object and converts it to an encoded query string.
   *
   * <p>Non-recursive:</p>
   *
   * <pre><code><a href="#!/api/Ext.Object-method-toQueryString" rel="Ext.Object-method-toQueryString" class="docClass">Ext.Object.toQueryString</a>({foo: 1, bar: 2}); // returns "foo=1&amp;bar=2"
   * <a href="#!/api/Ext.Object-method-toQueryString" rel="Ext.Object-method-toQueryString" class="docClass">Ext.Object.toQueryString</a>({foo: null, bar: 2}); // returns "foo=&amp;bar=2"
   * <a href="#!/api/Ext.Object-method-toQueryString" rel="Ext.Object-method-toQueryString" class="docClass">Ext.Object.toQueryString</a>({'some price': '$300'}); // returns "some%20price=%24300"
   * <a href="#!/api/Ext.Object-method-toQueryString" rel="Ext.Object-method-toQueryString" class="docClass">Ext.Object.toQueryString</a>({date: new Date(2011, 0, 1)}); // returns "date=%222011-01-01T00%3A00%3A00%22"
   * <a href="#!/api/Ext.Object-method-toQueryString" rel="Ext.Object-method-toQueryString" class="docClass">Ext.Object.toQueryString</a>({colors: ['red', 'green', 'blue']}); // returns "colors=red&amp;colors=green&amp;colors=blue"
   * </code></pre>
   *
   * <p>Recursive:</p>
   *
   * <pre><code><a href="#!/api/Ext.Object-method-toQueryString" rel="Ext.Object-method-toQueryString" class="docClass">Ext.Object.toQueryString</a>({
   *     username: 'Jacky',
   *     dateOfBirth: {
   *         day: 1,
   *         month: 2,
   *         year: 1911
   *     },
   *     hobbies: ['coding', 'eating', 'sleeping', ['nested', 'stuff']]
   * }, true); // returns the following string (broken down and url-decoded for ease of reading purpose):
   * // username=Jacky
   * //    &amp;dateOfBirth[day]=1&amp;dateOfBirth[month]=2&amp;dateOfBirth[year]=1911
   * //    &amp;hobbies[0]=coding&amp;hobbies[1]=eating&amp;hobbies[2]=sleeping&amp;hobbies[3][0]=nested&amp;hobbies[3][1]=stuff
   * </code></pre>
   * @param object The object to encode
   * @param recursive Whether or not to interpret the object in recursive format.
   * (PHP / Ruby on Rails servers and similar).
   * @default false
   * @return queryString
   */
  public native function urlEncode(object:Object, recursive:Boolean = false):String;

  /**
   * Returns the given value itself if it's not empty, as described in <a href="#!/api/Ext-method-isEmpty" rel="Ext-method-isEmpty" class="docClass">isEmpty</a>; returns the default
   * value (second argument) otherwise.
   * @param value The value to test.
   * @param defaultValue The value to return if the original value is empty.
   * @param allowBlank <code>true</code> to allow zero length strings to qualify as non-empty.
   * @default false
   * @return value, if non-empty, else defaultValue.
   */
  public native function valueFrom(value:Object, defaultValue:Object, allowBlank:Boolean = false):Object;

  /**
   * Convenient shorthand to create a widget by its xtype or a config object.
   *
   * <pre><code> var button = <a href="#!/api/Ext-method-widget" rel="Ext-method-widget" class="docClass">Ext.widget</a>('button'); // Equivalent to <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('widget.button');
   *
   *  var panel = <a href="#!/api/Ext-method-widget" rel="Ext-method-widget" class="docClass">Ext.widget</a>('panel', { // Equivalent to <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('widget.panel')
   *      title: 'Panel'
   *  });
   *
   *  var grid = <a href="#!/api/Ext-method-widget" rel="Ext-method-widget" class="docClass">Ext.widget</a>({
   *      xtype: 'grid',
   *      ...
   *  });
   * </code></pre>
   *
   * <p>If a <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">component</a> instance is passed, it is simply returned.</p>
   * @param name The xtype of the widget to create.
   * @param config The configuration object for the widget constructor.
   * @return The widget instance
   */
  public native function widget(name:String = null, config:Object = null):Object;

  /**
   * This function returns the path to a resource loaded from the resources folder.
   *
   * @param path specifies the relative path to the resource inside the resources folder
   * @param poolName specifies the resources pool name, which is either "path", "shared" or null. Important for universal apps. If in doubt use null.
   * @param packageName The Sencha package name the resource is used from.
   * @return The path to the resource
   */
  public native function getResourcePath(path:String, poolName:String = null, packageName:String = null):String;
}
}