package ext {
import ext.mixin.Observable;

[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Native]
/**
 * Fires when a Component is added to a Container.
 * @eventType ext.events.GlobalEvents_componentEvent.ADDED
 */
[Event(name = 'onAdded', type = 'ext.events.GlobalEvents_componentEvent')]
/**
 * Fires before <a href="#!/api/Ext.mixin.Responsive" rel="Ext.mixin.Responsive" class="docClass">Ext.mixin.Responsive</a> perform any updates in response to
 * dynamic changes. This is prior to refreshing <code>responsiveFormulas</code>.
 * @since 5.0.1
 * @eventType ext.events.GlobalEvents_contextEvent.BEFORE_RESPONSIVE_UPDATE
 */
[Event(name = 'onBeforeResponsiveUpdate', type = 'ext.events.GlobalEvents_contextEvent')]
/**
 * Fires when <a href="#!/api/Ext.mixin.Responsive" rel="Ext.mixin.Responsive" class="docClass">Ext.mixin.Responsive</a> is about to perform updates in response to
 * dynamic changes. At this point all <code>responsiveFormulas</code> have been refreshed.
 * @since 5.0.1
 * @eventType ext.events.GlobalEvents_contextEvent.BEGIN_RESPONSIVE_UPDATE
 */
[Event(name = 'onBeginResponsiveUpdate', type = 'ext.events.GlobalEvents_contextEvent')]
/**
 * Fires when a Component is collapsed (e.g., a panel).
 * @eventType ext.events.GlobalEvents_componentEvent.COLLAPSE
 */
[Event(name = 'onCollapse', type = 'ext.events.GlobalEvents_componentEvent')]
/**
 * Fires when a Component is expanded (e.g., a panel).
 * @eventType ext.events.GlobalEvents_componentEvent.EXPAND
 */
[Event(name = 'onExpand', type = 'ext.events.GlobalEvents_componentEvent')]
/**
 * Fires when a Component is hidden.
 * @eventType ext.events.GlobalEvents_componentEvent.HIDE
 */
[Event(name = 'onHide', type = 'ext.events.GlobalEvents_componentEvent')]
/**
 * Fires when an event handler finishes its run, just before returning to
 * browser control.
 *
 * <p>This includes DOM event handlers, Ajax (including JSONP) event handlers,
 * and <a href="#!/api/Ext.util.TaskRunner" rel="Ext.util.TaskRunner" class="docClass">TaskRunners</a></p>
 *
 * <p>When called at the tail of a DOM event, the event object is passed as the
 * sole parameter.</p>
 *
 * <p>This can be useful for performing cleanup, or update tasks which need to
 * happen only after all code in an event handler has been run, but which
 * should not be executed in a timer due to the intervening browser
 * reflow/repaint which would take place.</p>
 * @eventType ext.events.GlobalEventsEvent.IDLE
 */
[Event(name = 'onIdle', type = 'ext.events.GlobalEventsEvent')]
/**
 * A mousedown listener on the document that is immune to stopPropagation()
 * used in cases where we need to know if a mousedown event occurred on the
 * document regardless of whether some other handler tried to stop it.  An
 * example where this is useful is a menu that needs to be hidden whenever
 * there is a mousedown event on the document.
 * @eventType ext.events.GlobalEvents_eEvent.MOUSE_DOWN
 */
[Event(name = 'onMouseDown', type = 'ext.events.GlobalEvents_eEvent')]
/**
 * Fires when a Component is removed from a Container.
 * @eventType ext.events.GlobalEvents_componentEvent.REMOVED
 */
[Event(name = 'onRemoved', type = 'ext.events.GlobalEvents_componentEvent')]
/**
 * Fires when the browser window is resized.  To avoid running resize handlers
 * too often resulting in sluggish window resizing, resize events use a buffer
 * of 100 milliseconds.
 * @eventType ext.events.GlobalEvents_width_heightEvent.RESIZE
 */
[Event(name = 'onResize', type = 'ext.events.GlobalEvents_width_heightEvent')]
/**
 * Fires after <a href="#!/api/Ext.mixin.Responsive" rel="Ext.mixin.Responsive" class="docClass">Ext.mixin.Responsive</a> has performed updates in response to
 * dynamic changes.
 * @since 5.0.1
 * @eventType ext.events.GlobalEvents_contextEvent.RESPONSIVE_UPDATE
 */
[Event(name = 'onResponsiveUpdate', type = 'ext.events.GlobalEvents_contextEvent')]
/**
 * Fires after global layout processing has been resumed in <a href="#!/api/Ext.Component-method-resumeLayouts" rel="Ext.Component-method-resumeLayouts" class="docClass">Ext.Component.resumeLayouts</a>.
 * @eventType ext.events.GlobalEventsEvent.RESUME_LAYOUTS
 */
[Event(name = 'onResumeLayouts', type = 'ext.events.GlobalEventsEvent')]
/**
 * Fires when a Component is shown.
 * @eventType ext.events.GlobalEvents_componentEvent.SHOW
 */
[Event(name = 'onShow', type = 'ext.events.GlobalEvents_componentEvent')]
/**
 * An <code><a href="#!/api/Ext.mixin.Observable" rel="Ext.mixin.Observable" class="docClass">Observable</a></code> through which Ext fires global events.
 *
 * <p><a href="#!/api/Ext-method-on" rel="Ext-method-on" class="docClass">Ext.on</a>() and <a href="#!/api/Ext-method-un" rel="Ext-method-un" class="docClass">Ext.un</a>() are shorthand for <a href="#!/api/Ext.GlobalEvents-method-addListener" rel="Ext.GlobalEvents-method-addListener" class="docClass">addListener</a> and <a href="#!/api/Ext.GlobalEvents-method-removeListener" rel="Ext.GlobalEvents-method-removeListener" class="docClass">removeListener</a>
 * on this Observable.  For example, to listen for the idle event:</p>
 *
 * <pre><code><a href="#!/api/Ext-method-on" rel="Ext-method-on" class="docClass">Ext.on</a>('idle', function() {
 *     // do something
 * });
 * </code></pre>
 * <p>Type of singleton GlobalEvents.</p>
 * @see ext.#GlobalEvents ext.GlobalEvents
 */
public class SGlobalEvents extends ext.mixin.Observable {
  /**
   * @inheritDoc
   */
  public function SGlobalEvents(config:ext.mixin.Observable = null) {
    super();
  }
}
}