package ext {

[Native]
/**
 * Modified version of <a href="http://www.json.org/js.html">Douglas Crockford's JSON.js</a> that doesn't
 * mess with the Object prototype.
 * <p>Type of singleton JSON.</p>
 * @see ext.#JSON ext.JSON
 */
public class SJSON {
  /**
   * Decodes (parses) a JSON string to an object. If the JSON is invalid, this function throws
   * a SyntaxError unless the safe option is set.
   * @param json The JSON string.
   * @param safe <code>true</code> to return null, otherwise throw an exception
   * if the JSON is invalid.
   * @default false
   * @return The resulting object.
   */
  public native function decode(json:String, safe:Boolean = false):Object;

  /**
   * Encodes an Object, Array or other value.
   *
   * <p>If the environment's native JSON encoding is not being used (<a href="#!/api/Ext-property-USE_NATIVE_JSON" rel="Ext-property-USE_NATIVE_JSON" class="docClass">Ext.USE_NATIVE_JSON</a> is not set,
   * or the environment does not support it), then ExtJS's encoding will be used. This allows the developer
   * to add a <code>toJSON</code> method to their classes which need serializing to return a valid JSON representation
   * of the object.</p>
   * @param o The variable to encode.
   * @return The JSON string.
   */
  public native function encode(o:Object):String;

  /**
   * Encodes a Date. This returns the actual string which is inserted into the JSON string as the literal
   * expression. <strong>The returned value includes enclosing double quotation marks.</strong>
   *
   * <p>The default return format is <code>"yyyy-mm-ddThh:mm:ss"</code>.</p>
   *
   * <p>To override this:</p>
   *
   * <pre><code><a href="#!/api/Ext.JSON-method-encodeDate" rel="Ext.JSON-method-encodeDate" class="docClass">Ext.JSON.encodeDate</a> = function(d) {
   *     return <a href="#!/api/Ext.Date-method-format" rel="Ext.Date-method-format" class="docClass">Ext.Date.format</a>(d, '"Y-m-d"');
   * };
   * </code></pre>
   * @param d The Date to encode
   * @return The string literal to use in a JSON string.
   */
  public native function encodeDate(d:Date):String;

  /**
   * Encodes a String. This returns the actual string which is inserted into the JSON string as the literal
   * expression. <strong>The returned value includes enclosing double quotation marks.</strong>
   *
   * <p>To override this:</p>
   *
   * <pre><code><a href="#!/api/Ext.JSON-method-encodeString" rel="Ext.JSON-method-encodeString" class="docClass">Ext.JSON.encodeString</a> = function(s) {
   *     return 'Foo' + s;
   * };
   * </code></pre>
   * @param s The String to encode
   * @return The string literal to use in a JSON string.
   */
  public native function encodeString(s:String):String;

  /**
   * The function which <a href="#!/api/Ext.JSON-method-encode" rel="Ext.JSON-method-encode" class="docClass">encode</a> uses to encode all javascript values to their JSON representations
   * when <a href="#!/api/Ext-property-USE_NATIVE_JSON" rel="Ext-property-USE_NATIVE_JSON" class="docClass">Ext.USE_NATIVE_JSON</a> is <code>false</code>.
   *
   * <p>This is made public so that it can be replaced with a custom implementation.</p>
   * @param o Any javascript value to be converted to its JSON representation
   * @return The JSON representation of the passed value.
   */
  public native function encodeValue(o:Object):String;
}
}