package ext {

[Native]
/**
 * Contains information about features supported in the current environment as well
 * as bugs detected.
 * <p>Type of singleton supports.</p>
 * @see ext.#supports ext.supports
 */
public class Ssupports {
  /**
   * <code>true</code> if the browser fires focus events (focus, blur, focusin, focusout)
   * asynchronously, i.e. in a separate event loop invocation. This is only true
   * for all versions Internet Explorer; Microsoft Edge and other browsers fire
   * focus events synchronously.
   */
  public native function get AsyncFocusEvents():Boolean;

  /**
   * @private
   */
  public native function set AsyncFocusEvents(value:Boolean):void;

  /**
   * <code>true</code> if the device supports the HTML5 <code>audio</code> tag.
   *
   * <p>This property is available at application boot time, before document ready.</p>
   */
  public native function get Audio():Boolean;

  /**
   * @private
   */
  public native function set Audio(value:Boolean):void;

  [Deprecated(message = "Use &lt;code>Audio&lt;/code> instead.\n")]
  /**
   * <code>true</code> if the device supports the HTML5 <code>audio</code> tag.
   */
  public native function get AudioTag():Boolean;

  /**
   * @private
   */
  public native function set AudioTag(value:Boolean):void;

  /**
   * <code>true</code> if the browser supports the getBoundingClientRect method on elements
   * This property is available at application boot time, before document ready.
   */
  public native function get BoundingClientRect():Boolean;

  /**
   * @private
   */
  public native function set BoundingClientRect(value:Boolean):void;

  /**
   * <code>true</code> if the device supports CSS3 border radius.
   * This property is <em>NOT</em> available at application boot time. Only after the document ready event.
   */
  public native function get CSS3BorderRadius():Boolean;

  /**
   * @private
   */
  public native function set CSS3BorderRadius(value:Boolean):void;

  /**
   * <code>true</code> if document environment supports the CSS3
   * box-shadow style.
   *
   * <p>This property is available at application boot time, before document ready.</p>
   */
  public native function get CSS3BoxShadow():Boolean;

  /**
   * @private
   */
  public native function set CSS3BoxShadow(value:Boolean):void;

  /**
   * <code>true</code> if the device supports CSS3 linear gradients.
   * This property is available at application boot time, before document ready.
   */
  public native function get CSS3LinearGradient():Boolean;

  /**
   * @private
   */
  public native function set CSS3LinearGradient(value:Boolean):void;

  /**
   * <code>true</code> if document environment supports the CSS3
   * pointer-events style.
   *
   * <p>This property is available at application boot time, before document ready.</p>
   */
  public native function get CSSPointerEvents():Boolean;

  /**
   * @private
   */
  public native function set CSSPointerEvents(value:Boolean):void;

  /**
   * <code>true</code> if the device supports Canvas.
   *
   * <p>This property is available at application boot time, before document ready.</p>
   */
  public native function get Canvas():Boolean;

  /**
   * @private
   */
  public native function set Canvas(value:Boolean):void;

  /**
   * <code>true</code> if created child elements
   * lose their innerHTML when modifying the innerHTML of the parent element.
   * This property is <em>NOT</em> available at application boot time. Only after the document ready event.
   */
  public native function get ChildContentClearedWhenSettingInnerHTML():Boolean;

  /**
   * @private
   */
  public native function set ChildContentClearedWhenSettingInnerHTML(value:Boolean):void;

  /**
   * <code>true</code> if document environment supports the HTML5
   * classList API.
   *
   * <p>This property is available at application boot time, before document ready.</p>
   */
  public native function get ClassList():Boolean;

  /**
   * @private
   */
  public native function set ClassList(value:Boolean):void;

  /**
   * <code>true</code> if the native DOM cloneNode method copies
   * expando properties to the newly cloned node.
   *
   * <p>This property is available at application boot time, before document ready.</p>
   */
  public native function get CloneNodeCopiesExpando():Boolean;

  /**
   * @private
   */
  public native function set CloneNodeCopiesExpando(value:Boolean):void;

  /**
   * <code>true</code> if the browser supports document.defaultView.getComputedStyle().
   * This property is <em>NOT</em> available at application boot time. Only after the document ready event.
   */
  public native function get ComputedStyle():Boolean;

  /**
   * @private
   */
  public native function set ComputedStyle(value:Boolean):void;

  /**
   * <code>true</code> if browser support CreateContextualFragment
   * range native methods.
   * See https://developer.mozilla.org/en/DOM/range.createContextualFragment
   *
   * <p>This property is available at application boot time, before document ready.</p>
   */
  public native function get CreateContextualFragment():Boolean;

  /**
   * @private
   */
  public native function set CreateContextualFragment(value:Boolean):void;

  /**
   * <code>true</code> if the device supports CSS3DTransform.
   * This property is available at application boot time, before document ready.
   */
  public native function get Css3DTransforms():Boolean;

  /**
   * @private
   */
  public native function set Css3DTransforms(value:Boolean):void;

  /**
   * <code>true</code> if the device supports device motion (acceleration
   * and rotation rate).
   *
   * <p>This property is available at application boot time, before document ready.</p>
   */
  public native function get DeviceMotion():Boolean;

  /**
   * @private
   */
  public native function set DeviceMotion(value:Boolean):void;

  /**
   * <code>true</code> if when asking for an element's dimension via offsetWidth or offsetHeight,
   * getBoundingClientRect, etc. the browser returns the subpixel width rounded to the nearest pixel.
   *
   * <p>This property is available at application boot time, before document ready.</p>
   */
  public native function get Direct2DBug():Boolean;

  /**
   * @private
   */
  public native function set Direct2DBug(value:Boolean):void;

  /**
   * <code>true</code> if INPUT elements lose their
   * selection when their display style is changed. Essentially, if a text input
   * has focus and its display style is changed, the I-beam disappears.
   *
   * <p>This bug is encountered due to the work around in place for the <a href="#!/api/Ext.supports-property-RightMargin" rel="Ext.supports-property-RightMargin" class="docClass">RightMargin</a>
   * bug. This has been observed in Safari 4.0.4 and older, and appears to be fixed
   * in Safari 5. It's not clear if Safari 4.1 has the bug, but it has the same WebKit
   * version number as Safari 5 (according to http://unixpapa.com/js/gecko.html).</p>
   *
   * <p>This property is available at application boot time, before document ready.</p>
   */
  public native function get DisplayChangeInputSelectionBug():Object;

  /**
   * @private
   */
  public native function set DisplayChangeInputSelectionBug(value:Object):void;

  /**
   * <code>true</code> if TEXTAREA elements lose their
   * selection when their display style is changed. Essentially, if a text area has
   * focus and its display style is changed, the I-beam disappears.
   *
   * <p>This bug is encountered due to the work around in place for the <a href="#!/api/Ext.supports-property-RightMargin" rel="Ext.supports-property-RightMargin" class="docClass">RightMargin</a>
   * bug. This has been observed in Chrome 10 and Safari 5 and older, and appears to
   * be fixed in Chrome 11.</p>
   *
   * <p>This property is available at application boot time, before document ready.</p>
   */
  public native function get DisplayChangeTextAreaSelectionBug():Object;

  /**
   * @private
   */
  public native function set DisplayChangeTextAreaSelectionBug(value:Object):void;

  /**
   * True if the browser emulates a mouseover event on tap (mobile safari)
   *
   * <p>This property is available at application boot time, before document ready.</p>
   */
  public native function get EmulatedMouseOver():Boolean;

  /**
   * @private
   */
  public native function set EmulatedMouseOver(value:Boolean):void;

  /**
   * <code>true</code> if the device supports CSS float.
   * This property is available at application boot time, before document ready.
   */
  public native function get Float():Boolean;

  /**
   * @private
   */
  public native function set Float(value:Boolean):void;

  [Deprecated(message = "Use &lt;code>Geolocation&lt;/code> instead (notice the lower-casing of &#39;L&#39;).\n")]
  /**
   * <code>true</code> if the device supports Geo-location.
   */
  public native function get GeoLocation():Boolean;

  /**
   * @private
   */
  public native function set GeoLocation(value:Boolean):void;

  /**
   * <code>true</code> if the device supports GeoLocation.
   * This property is available at application boot time, before document ready.
   */
  public native function get Geolocation():Boolean;

  /**
   * @private
   */
  public native function set Geolocation(value:Boolean):void;

  /**
   * <code>true</code> if the browser will return the left/top/right/bottom
   * position as a percentage when explicitly set as a percentage value.
   *
   * <p>This property is <em>NOT</em> available at application boot time. Only after the document ready event.</p>
   */
  public native function get GetPositionPercentage():Boolean;

  /**
   * @private
   */
  public native function set GetPositionPercentage(value:Boolean):void;

  /**
   * True if the user agent supports the hashchange event
   *
   * <p>This property is available at application boot time, before document ready.</p>
   */
  public native function get Hashchange():Boolean;

  /**
   * @private
   */
  public native function set Hashchange(value:Boolean):void;

  /**
   * <code>true</code> if the browser is currently
   * running in Windows High Contrast accessibility mode.
   */
  public native function get HighContrastMode():Boolean;

  /**
   * @private
   */
  public native function set HighContrastMode(value:Boolean):void;

  /**
   * <code>true</code> if the device supports HTML5 history. See
   * https://developer.mozilla.org/en/DOM/Manipulating_the_browser_history
   *
   * <p>This property is available at application boot time, before document ready.</p>
   */
  public native function get History():Boolean;

  /**
   * @private
   */
  public native function set History(value:Boolean):void;

  /**
   * <code>true</code> if localStorage is supported.
   *
   * <p>This property is available at application boot time, before document ready.</p>
   */
  public native function get LocalStorage():Boolean;

  /**
   * @private
   */
  public native function set LocalStorage(value:Boolean):void;

  /**
   * <code>true</code> if the browser supports mouseenter and mouseleave events
   * This property is available at application boot time, before document ready.
   */
  public native function get MouseEnterLeave():Boolean;

  /**
   * @private
   */
  public native function set MouseEnterLeave(value:Boolean):void;

  /**
   * <code>true</code> if the browser supports the mousewheel event
   * This property is available at application boot time, before document ready.
   */
  public native function get MouseWheel():Boolean;

  /**
   * @private
   */
  public native function set MouseWheel(value:Boolean):void;

  /**
   * <code>true</code> if the browser supports placeholders
   * on numeric input fields
   *
   * <p>This property is available at application boot time, before document ready.</p>
   */
  public native function get NumericInputPlaceHolder():Boolean;

  /**
   * @private
   */
  public native function set NumericInputPlaceHolder(value:Boolean):void;

  /**
   * <code>true</code> if the browser supports normal css opacity
   * This property is available at application boot time, before document ready.
   */
  public native function get Opacity():Boolean;

  /**
   * @private
   */
  public native function set Opacity(value:Boolean):void;

  /**
   * <code>true</code> if the device supports different orientations.
   * This property is available at application boot time, before document ready.
   */
  public native function get Orientation():Boolean;

  /**
   * @private
   */
  public native function set Orientation(value:Boolean):void;

  /**
   * <code>true</code> if the device supports the <code>orientationchange</code>
   * event.
   *
   * <p>This property is available at application boot time, before document ready.</p>
   */
  public native function get OrientationChange():Boolean;

  /**
   * @private
   */
  public native function set OrientationChange(value:Boolean):void;

  /**
   * <code>true</code> if the browser supports the HTML5 placeholder attribute on inputs
   * This property is available at application boot time, before document ready.
   */
  public native function get Placeholder():Boolean;

  /**
   * @private
   */
  public native function set Placeholder(value:Boolean):void;

  /**
   * <code>true</code> if browser support document.createRange native method.
   * See https://developer.mozilla.org/en/DOM/range.
   *
   * <p>This property is available at application boot time, before document ready.</p>
   */
  public native function get Range():Boolean;

  /**
   * @private
   */
  public native function set Range(value:Boolean):void;

  /**
   * <code>true</code> if the device supports right margin.
   * See https://bugs.webkit.org/show_bug.cgi?id=13343 for why this is needed.
   *
   * <p>This property is <em>NOT</em> available at application boot time. Only after the document ready event.</p>
   */
  public native function get RightMargin():Boolean;

  /**
   * @private
   */
  public native function set RightMargin(value:Boolean):void;

  /**
   * <code>true</code> if the BoundingClientRect is
   * rotated when the element is rotated using a CSS transform.
   * This property is <em>NOT</em> available at application boot time. Only after the document ready event.
   */
  public native function get RotatedBoundingClientRect():Boolean;

  /**
   * @private
   */
  public native function set RotatedBoundingClientRect(value:Boolean):void;

  /**
   * In some browsers the right padding of an overflowing element is not accounted
   * for in its scrollWidth.  The result can vary depending on whether or not
   * The element contains block-level children.  This method tests the effect
   * of padding on scrollWidth when there are no block-level children inside the
   * overflowing element.
   *
   * <p>This method returns true if the browser is affected by this bug.</p>
   *
   * <p>This property is <em>NOT</em> available at application boot time. Only after the document ready event.</p>
   */
  public native function get ScrollWidthInlinePaddingBug():Boolean;

  /**
   * @private
   */
  public native function set ScrollWidthInlinePaddingBug(value:Boolean):void;

  /**
   * <code>true</code> if the browser supports the CSS selector API level 2.
   * https://dev.w3.org/2006/webapi/selectors-api2/
   */
  public native function get Selectors2():Boolean;

  /**
   * @private
   */
  public native function set Selectors2(value:Boolean):void;

  /**
   * True if the browser fires the keydown event on specialkey autorepeat
   *
   * <p>note 1: IE fires ONLY the keydown event on specialkey autorepeat
   * note 2: Safari &lt; 3.1, Gecko (Mac/Linux) &amp; Opera fire only the keypress event on
   * specialkey autorepeat (research done by Jan Wolter at
   * http://unixpapa.com/js/key.html)
   * note 3: Opera 12 behaves like other modern browsers so this workaround does not
   * work anymore</p>
   *
   * <p>This property is available at application boot time, before document ready.</p>
   */
  public native function get SpecialKeyDownRepeat():Boolean;

  /**
   * @private
   */
  public native function set SpecialKeyDownRepeat(value:Boolean):void;

  /**
   * <code>true</code> if the device supports SVG.
   *
   * <p>This property is available at application boot time, before document ready.</p>
   */
  public native function get Svg():Boolean;

  /**
   * @private
   */
  public native function set Svg(value:Boolean):void;

  /**
   * <code>true</code> if the browser supports maxlength on textareas.
   * This property is available at application boot time, before document ready.
   */
  public native function get TextAreaMaxLength():Boolean;

  /**
   * @private
   */
  public native function set TextAreaMaxLength(value:Boolean):void;

  /**
   * <code>true</code> if the browser supports touch input.
   *
   * <p>This property is available at application boot time, before document ready.</p>
   */
  public native function get Touch():Boolean;

  /**
   * @private
   */
  public native function set Touch(value:Boolean):void;

  /**
   * <code>true</code> if the device supports touch events (<code>touchstart</code>,
   * <code>touchmove</code>, <code>touchend</code>).
   *
   * <p>This property is available at application boot time, before document ready.</p>
   */
  public native function get TouchEvents():Boolean;

  /**
   * @private
   */
  public native function set TouchEvents(value:Boolean):void;

  /**
   * <code>true</code> if the device supports CSS3 Transitions.
   *
   * <p>This property is available at application boot time, before document ready.</p>
   */
  public native function get Transitions():Boolean;

  /**
   * @private
   */
  public native function set Transitions(value:Boolean):void;

  /**
   * <code>true</code> if the device supports transparent color.
   * This property is <em>NOT</em> available at application boot time. Only after the document ready event.
   */
  public native function get TransparentColor():Boolean;

  /**
   * @private
   */
  public native function set TransparentColor(value:Boolean):void;

  /**
   * <code>true</code> if the device supports the HTML5 <code>video</code> tag.
   *
   * <p>This property is available at application boot time, before document ready.</p>
   */
  public native function get Video():Boolean;

  /**
   * @private
   */
  public native function set Video(value:Boolean):void;

  /**
   * <code>true</code> if the device supports ViewportUnits.
   */
  public native function get ViewportUnits():Boolean;

  /**
   * @private
   */
  public native function set ViewportUnits(value:Boolean):void;

  /**
   * <code>true</code> if the device supports VML.
   * This property is available at application boot time, before document ready.
   */
  public native function get Vml():Boolean;

  /**
   * @private
   */
  public native function set Vml(value:Boolean):void;

  /**
   * <code>true</code> if the browser supports XMLHttpRequest
   *
   * <p>This property is available at application boot time, before document ready.</p>
   */
  public native function get XHR2():Boolean;

  /**
   * <code>true</code> if the browser supports XMLHttpRequest
   * upload progress info
   *
   * <p>This property is available at application boot time, before document ready.</p>
   */
  public native function get XHRUploadProgress():Boolean;

  /**
   * @private
   */
  public native function set XHRUploadProgress(value:Boolean):void;

  /**
   * The method name which matches an element against a selector if implemented in this environment.
   *
   * <p>This property is available at application boot time, before document ready.</p>
   */
  public native function get matchesSelector():String;

  /**
   * @private
   */
  public native function set matchesSelector(value:String):void;
}
}