package ext {
import ext.app.ViewController;
import ext.app.ViewModel;
import ext.data.Session;
import ext.mixin.IBindable;
import ext.mixin.IComponentDelegation;
import ext.mixin.IInheritable;
import ext.mixin.IPluggable;
import ext.mixin.Observable;
import ext.plugin.AbstractPlugin;

[Uses("ext.Evented")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Pluggable")]
[Uses("ext.Mixin")]
[Uses("ext.Base")]
[Uses("ext.mixin.Observable")]
[Native("Ext.Widget", require)]
/**
 * Fires when the <a href="#!/api/Ext.Widget-cfg-height" rel="Ext.Widget-cfg-height" class="docClass">height</a> configuration is changed by <a href="#!/api/Ext.Widget-method-setHeight" rel="Ext.Widget-method-setHeight" class="docClass">setHeight</a>.
 * @eventType ext.events.Widget_value_oldValueEvent.HEIGHT_CHANGE
 */
[Event(name = 'onHeightChange', type = 'ext.events.Widget_value_oldValueEvent')]
/**
 * Fires when the <a href="#!/api/Ext.Widget-cfg-hidden" rel="Ext.Widget-cfg-hidden" class="docClass">hidden</a> configuration is changed by <a href="#!/api/Ext.Widget-method-setHidden" rel="Ext.Widget-method-setHidden" class="docClass">setHidden</a>.
 * @eventType ext.events.Widget_value_oldValueEvent.HIDDEN_CHANGE
 */
[Event(name = 'onHiddenChange', type = 'ext.events.Widget_value_oldValueEvent')]
/**
 * Fires when the <a href="#!/api/Ext.Widget-cfg-width" rel="Ext.Widget-cfg-width" class="docClass">width</a> configuration is changed by <a href="#!/api/Ext.Widget-method-setWidth" rel="Ext.Widget-method-setWidth" class="docClass">setWidth</a>.
 * @eventType ext.events.Widget_value_oldValueEvent.WIDTH_CHANGE
 */
[Event(name = 'onWidthChange', type = 'ext.events.Widget_value_oldValueEvent')]
/**
 * <a href="#!/api/Ext.Widget" rel="Ext.Widget" class="docClass">Ext.Widget</a> is a light-weight Component that consists of nothing more than a template
 * Element that can be cloned to quickly and efficiently replicate many instances.
 * <a href="#!/api/Ext.Widget" rel="Ext.Widget" class="docClass">Ext.Widget</a> is typically not instantiated directly, because the default template is
 * just a single element with no listeners. Instead <a href="#!/api/Ext.Widget" rel="Ext.Widget" class="docClass">Ext.Widget</a> should be extended to
 * create Widgets that have a useful markup structure and event listeners.
 *
 * <p>For example:</p>
 *
 * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyWidget', {
 *      extend: '<a href="#!/api/Ext.Widget" rel="Ext.Widget" class="docClass">Ext.Widget</a>',
 *
 *      // The element template passed to <a href="#!/api/Ext.dom.Element-static-method-create" rel="Ext.dom.Element-static-method-create" class="docClass">Ext.Element.create</a>()
 *      element: {
 *          reference: 'element',
 *          listeners: {
 *              click: 'onClick'
 *          },
 *          children: [{
 *              reference: 'innerElement',
 *              listeners: {
 *                  click: 'onInnerClick'
 *              }
 *          }]
 *      },
 *
 *      constructor: function(config) {
 *          // It is important to remember to call the Widget superclass constructor
 *          // when overriding the constructor in a derived class. This ensures that
 *          // the element is initialized from the template, and that initConfig() is
 *          // is called.
 *          this.callParent([config]);
 *
 *          // After calling the superclass constructor, the Element is available and
 *          // can safely be manipulated. Reference Elements are instances of
 *          // <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.Element</a>, and are cached on each Widget instance by reference name.
 *          <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>().appendChild(this.element);
 *      },
 *
 *      onClick: function() {
 *          // listeners use this Widget instance as their scope
 *          console.log('element clicked', this);
 *      },
 *
 *      onInnerClick: function() {
 *          // access the innerElement reference by name
 *          console.log('inner element clicked', this.innerElement);
 *      }
 *  });
 * </code></pre>
 */
public class Widget extends ext.Evented implements ext.mixin.IInheritable, ext.mixin.IBindable, ext.mixin.IComponentDelegation, ext.mixin.IPluggable {
  /**
   * @param config @inheritDoc
   */
  public function Widget(config:ext.Widget = null) {
    super();
  }

  /**
   * A CSS class to apply to the main element that will be inherited down the class
   * hierarchy.  Subclasses may override this property on their prototype to add their
   * own CSS class in addition to the CSS classes inherited from ancestor classes via
   * the prototype chain.  For example
   *
   * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Foo', {
   *     extend: '<a href="#!/api/Ext.Widget" rel="Ext.Widget" class="docClass">Ext.Widget</a>',
   *     classCls: 'foo'
   * });
   *
   * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Bar', {
   *     extend: 'Foo',
   *     classCls: 'bar'
   * });
   *
   * var bar = new Bar();
   *
   * console.log(bar.element.className); // outputs 'foo bar'
   * </code></pre>
   */
  public native function get classCls():Object;

  /**
   * @private
   */
  public native function set classCls(value:Object):void;

  /**
   * When set to <code>true</code> during widget class definition, that class will be the "root" for
   * <a href="#!/api/Ext.Widget-property-classCls" rel="Ext.Widget-property-classCls" class="docClass">classCls</a> inheritance. Derived classes may set this to <code>true</code> to avoid
   * inheriting a <a href="#!/api/Ext.Widget-property-classCls" rel="Ext.Widget-property-classCls" class="docClass">classCls</a> from their superclass.
   * @default true
   */
  public native function get classClsRoot():Boolean;

  /**
   * @private
   */
  public native function set classClsRoot(value:Boolean):void;

  /**
   * This property is used to determine the property of a <code>bind</code> config that is just
   * the value. For example, if <code>defaultBindProperty="value"</code>, then this shorthand
   * <code>bind</code> config:
   *
   * <pre><code> bind: '{name}'
   * </code></pre>
   *
   * <p>Is equivalent to this object form:</p>
   *
   * <pre><code> bind: {
   *      value: '{name}'
   *  }
   * </code></pre>
   *
   * <p>The <code>defaultBindProperty</code> is set to "value" for form fields and to "store" for
   * grids and trees.</p>
   */
  public native function get defaultBindProperty():String;

  /**
   * @private
   */
  public native function set defaultBindProperty(value:String):void;

  /**
   * A configuration object for <a href="#!/api/Ext.dom.Element-static-method-create" rel="Ext.dom.Element-static-method-create" class="docClass">Ext.Element.create</a>() that is used to create the Element
   * template.  Supports all the standard options of a <a href="#!/api/Ext.dom.Element-static-method-create" rel="Ext.dom.Element-static-method-create" class="docClass">Ext.Element.create</a>() config and
   * adds 2 additional options:
   *
   * <ol>
   * <li><p><code>reference</code> - this option specifies a name for Element references.  These
   * references names become properties of the Widget instance and refer to <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.Element</a>
   * instances that were created using the template:</p>
   *
   * <p>  element: {
   *       reference: 'element',
   *       children: [{
   *           reference: 'innerElement'
   *       }]
   *   }</p></li>
   * </ol>
   *
   *
   * <p>After construction of a widget the reference elements are accessible as follows:</p>
   *
   * <pre><code>var foo = new FooWidget(),
   *     innerEl = foo.innerElement; // an <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.Element</a> that wraps the innerElement
   * </code></pre>
   *
   * <p>The reference attribute is optional, but all Widgets must have a <code>'element'</code>
   * reference on some element within the template (usually the outermost one).</p>
   *
   * <ol>
   * <li><p><code>listeners</code> - a standard listeners object as specified by <a href="#!/api/Ext.mixin.Observable" rel="Ext.mixin.Observable" class="docClass">Ext.mixin.Observable</a>.</p>
   *
   * <p> element: {
   *      reference: 'element',
   *      listeners: {
   *          click: 'onClick'
   *      },
   *      children: [{
   *          reference: 'innerElement',
   *          listeners: {
   *              click: 'onInnerClick'
   *          }
   *      }]
   *  }</p></li>
   * </ol>
   *
   *
   * <p>Since listeners cannot be attached without an <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.Element</a> reference the <code>reference</code>
   * property MUST be specified in order to use <code>listeners</code>.</p>
   *
   * <p>The Widget instance is used as the scope for all listeners specified in this way,
   * so it is invalid to use the <code>scope</code> option in the <code>listeners</code> config since it will
   * always be overwritten using <code>this</code>.</p>
   * @default {reference: 'element'}
   */
  public native function get element():Object;

  /**
   * @private
   */
  public native function set element(value:Object):void;

  /**
   * An array of child elements to use as the children of the main element in the {&#64;link
   *
   * <h1>element} template.  Only used if "children" are not specified explicitly in the</h1>
   *
   * <p><a href="#!/api/Ext.Widget-property-element" rel="Ext.Widget-property-element" class="docClass">element</a> template.</p>
   * @default []
   */
  public native function get template():Array;

  /**
   * @private
   */
  public native function set template(value:Array):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The base CSS class to apply to this widget's element.
   * Used as the prefix for <a href="#!/api/Ext.Widget-cfg-ui" rel="Ext.Widget-cfg-ui" class="docClass">ui</a>-specific class names.
   * When set to <code>true</code> the <a href="#!/api/Ext.Widget-property-classCls" rel="Ext.Widget-property-classCls" class="docClass">classCls</a> will be used as the baseCls
   * @default true
   * @see #getBaseCls()
   * @see #setBaseCls()
   */
  public native function get baseCls():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set baseCls(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Setting this config option adds or removes data bindings for other configs.
   * For example, to bind the <code>title</code> config:
   *
   * <pre><code> var panel = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>({
   *      xtype: 'panel',
   *      bind: {
   *          title: 'Hello {user.name}'
   *      }
   *  });
   * </code></pre>
   *
   * <p>To dynamically add bindings:</p>
   *
   * <pre><code> panel.setBind({
   *      title: 'Greetings {user.name}!'
   *  });
   * </code></pre>
   *
   * <p>To remove bindings:</p>
   *
   * <pre><code> panel.setBind({
   *      title: null
   *  });
   * </code></pre>
   *
   * <p>The bind expressions are presented to <code><a href="#!/api/Ext.app.ViewModel-method-bind" rel="Ext.app.ViewModel-method-bind" class="docClass">Ext.app.ViewModel.bind</a></code>. The
   * <code>ViewModel</code> instance is determined by <code>lookupViewModel</code>.</p>
   * @default {$value: null, lazy: true}
   * @see #getBind()
   * @see #setBind()
   */
  public native function get bind():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set bind(value:Object):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Enables or disables bordering on this component.
   * The following values are accepted:
   *
   * <ul>
   * <li><code>null</code> or `true (default): Do nothing and allow the border to be specified by the theme.</li>
   * <li><code>false</code>: suppress the default border provided by the theme.</li>
   * </ul>
   *
   *
   * <p>Please note that enabling bordering via this config will not add a <code>border-color</code>
   * or <code>border-style</code> CSS property to the component; you provide the <code>border-color</code>
   * and <code>border-style</code> via CSS rule or <a href="#!/api/Ext.Widget-cfg-style" rel="Ext.Widget-cfg-style" class="docClass">style</a> configuration
   * (if not already provide by the theme).</p>
   *
   * <h2>Using <a href="#!/api/Ext.Widget-cfg-style" rel="Ext.Widget-cfg-style" class="docClass">style</a>:</h2>
   *
   * <pre><code><a href="#!/api/Ext.container.Viewport-event-add" rel="Ext.container.Viewport-event-add" class="docClass">Ext.Viewport.add</a>({
   *     centered: true,
   *     width: 100,
   *     height: 100,
   *
   *     style: 'border: 1px solid blue;'
   *     // ...
   * });
   * </code></pre>
   *
   * <h2>Using CSS:</h2>
   *
   * <pre><code><a href="#!/api/Ext.container.Viewport-event-add" rel="Ext.container.Viewport-event-add" class="docClass">Ext.Viewport.add</a>({
   *     centered: true,
   *     width: 100,
   *     height: 100,
   *
   *     cls: 'my-component'
   *     // ...
   * });
   * </code></pre>
   *
   * <p>And your CSS file:</p>
   *
   * <pre><code>.my-component {
   *     border: 1px solid red;
   * }
   * </code></pre>
   * @see #getBorder()
   * @see #setBorder()
   */
  public native function get border():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set border(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The CSS class to add to this widget's element, in
   * addition to the <a href="#!/api/Ext.Widget-cfg-baseCls" rel="Ext.Widget-cfg-baseCls" class="docClass">baseCls</a>. In many cases, this property will be specified
   * by the derived widget class. See <a href="#!/api/Ext.Widget-cfg-userCls" rel="Ext.Widget-cfg-userCls" class="docClass">userCls</a> for adding additional CSS
   * classes to widget instances (such as items in a <a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Ext.Container</a>).
   * @see #getCls()
   * @see #setCls()
   */
  public native function get cls():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set cls(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * A string alias, a configuration object or an instance of a <code>ViewController</code> for
   * this container. Sample usage:
   *
   * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.UserController', {
   *     alias: 'controller.user'
   * });
   *
   * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('UserContainer', {
   *     extend: 'Ext.container.container',
   *     controller: 'user'
   * });
   * // Or
   * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('UserContainer', {
   *     extend: 'Ext.container.container',
   *     controller: {
   *         type: 'user',
   *         someConfig: true
   *     }
   * });
   *
   * // Can also instance at runtime
   * var ctrl = new MyApp.UserController();
   * var view = new UserContainer({
   *     controller: ctrl
   * });
   * </code></pre>
   * @see #getController()
   * @see #setController()
   */
  public native function get controller():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set controller(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * If <code>true</code>, this component will be the default scope (this pointer) for events
   * specified with string names so that the scope can be dynamically resolved. The
   * component will automatically become the defaultListenerScope if a
   * <a href="#!/api/Ext.mixin.Bindable-cfg-controller" rel="Ext.mixin.Bindable-cfg-controller" class="docClass">controller</a> is specified.
   *
   * <p>See the introductory docs for <a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Ext.container.Container</a> for some sample
   * usages.</p>
   *
   * <p><strong>NOTE</strong>: This value can only be reliably set at construction time. Setting it
   * after that time may not correctly rewire all of the potentially effected
   * listeners.</p>
   * @default false
   * @see #getDefaultListenerScope()
   * @see #setDefaultListenerScope()
   */
  public native function get defaultListenerScope():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set defaultListenerScope(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The height of this Component; must be a valid CSS length value, e.g: <code>300</code>, <code>100px</code>, <code>30%</code>, etc.
   * By default, if this is not explicitly set, this Component's element will simply have its own natural size.
   * If set to <code>auto</code>, it will set the width to <code>null</code> meaning it will have its own natural size.
   * @see #getHeight()
   * @see #setHeight()
   */
  public native function get height():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set height(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Whether or not this Component is hidden (its CSS <code>display</code> property is set to <code>none</code>).
   *
   * <p>Defaults to <code>true</code> for floated Components.</p>
   * @see #getHidden()
   * @see #setHidden()
   */
  public native function get hidden():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set hidden(value:Boolean):void;

  [ExtConfig]
  /**
   * An object or array of objects that will provide custom functionality for this
   * component. If a string is provided or a string is one of the elements of the
   * array, that string is treated as the <code>type</code> alias. For example, "listpaging"
   * is the type alias for <code>Ext.plugin.ListPaging</code>. The full alias includes the
   * "plugin." prefix (i.e., 'plugin.listpaging').
   *
   * <p>Plugins should derive from <code><a href="#!/api/Ext.plugin.Abstract" rel="Ext.plugin.Abstract" class="docClass">Ext.plugin.Abstract</a></code> but this is not required. The
   * only requirement for a valid plugin is that it contain an <code>init()</code> method that
   * accepts a reference to the owning component.</p>
   *
   * <p>When a component is created, if any plugins are available, the component will
   * call the <code><a href="#!/api/Ext.plugin.Abstract-method-init" rel="Ext.plugin.Abstract-method-init" class="docClass">init</a></code> method on each plugin,
   * passing a reference to itself. Each plugin can then call methods or respond to
   * events on the component as needed to provide its functionality.</p>
   *
   * <h2>Example code</h2>
   *
   * <p>A plugin by alias:</p>
   *
   * <pre><code> var list = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>({
   *      xtype: 'list',
   *      itemTpl: '&lt;div class="item"&gt;{title}&lt;/div&gt;',
   *      store: 'Items',
   *
   *      plugins: 'listpaging'
   *  });
   * </code></pre>
   *
   * <p>Multiple plugins by alias:</p>
   *
   * <pre><code> var list = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>({
   *      xtype: 'list',
   *      itemTpl: '&lt;div class="item"&gt;{title}&lt;/div&gt;',
   *      store: 'Items',
   *
   *      plugins: ['listpaging', 'pullrefresh']
   *  });
   * </code></pre>
   *
   * <p>Single plugin by class name with config options:</p>
   *
   * <pre><code> var list = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>({
   *      xtype: 'list',
   *      itemTpl: '&lt;div class="item"&gt;{title}&lt;/div&gt;',
   *      store: 'Items',
   *
   *      plugins: {
   *          type: 'listpaging',
   *          autoPaging: true
   *      }
   *  });
   * </code></pre>
   *
   * <p>Multiple plugins by type and class name with config options:</p>
   *
   * <pre><code> var list = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>({
   *      xtype: 'list',
   *      itemTpl: '&lt;div class="item"&gt;{title}&lt;/div&gt;',
   *      store: 'Items',
   *
   *      plugins: [{
   *          xclass: 'Ext.plugin.PullRefresh',
   *          pullRefreshText: 'Pull to refresh...'
   *      }, {
   *          type: 'listpaging',
   *          autoPaging: true
   *      }]
   *  });
   * </code></pre>
   */
  public native function get plugins():Array;

  [ExtConfig]
  /**
   * @private
   */
  public native function set plugins(value:Array):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * One or more names of config properties that this component should publish
   * to its ViewModel. Generally speaking, only properties defined in a class config
   * block (including ancestor config blocks and mixins) are eligible for publishing
   * to the viewModel. Some components override this and publish their most useful
   * configs by default.
   *
   * <p><strong>Note:</strong> We'll discuss publishing properties <strong>not</strong> found in the config block below.</p>
   *
   * <p>Values determined to be invalid by component (often form fields and model validations)
   * will not be published to the ViewModel.</p>
   *
   * <p>This config uses the <code><a href="#!/api/Ext.mixin.Bindable-cfg-reference" rel="Ext.mixin.Bindable-cfg-reference" class="docClass">reference</a></code> to determine the name of the data
   * object to place in the <code>ViewModel</code>. If <code>reference</code> is not set then this config
   * is ignored.</p>
   *
   * <p>By using this config and <code><a href="#!/api/Ext.mixin.Bindable-cfg-reference" rel="Ext.mixin.Bindable-cfg-reference" class="docClass">reference</a></code> you can bind configs between
   * components. For example:</p>
   *
   * <pre><code> ...
   *      items: [{
   *          xtype: 'textfield',
   *          reference: 'somefield',  // component's name in the ViewModel
   *          publishes: 'value' // value is not published by default
   *      },{
   *          ...
   *      },{
   *          xtype: 'displayfield',
   *          bind: 'You have entered "{somefield.value}"'
   *      }]
   *  ...
   * </code></pre>
   *
   * <p>Classes must provide this config as an Object:</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('App.foo.Bar', {
   *      publishes: {
   *          foo: true,
   *          bar: true
   *      }
   *  });
   * </code></pre>
   *
   * <p>This is required for the config system to properly merge values from derived
   * classes.</p>
   *
   * <p>For instances this value can be specified as a value as show above or an array
   * or object as follows:</p>
   *
   * <pre><code> {
   *      xtype: 'textfield',
   *      reference: 'somefield',
   *      publishes: [
   *          'value',
   *          'rawValue',
   *          'dirty'
   *      ]
   *  }
   *
   *  // This achieves the same result as the above array form.
   *  {
   *      xtype: 'textfield',
   *      reference: 'somefield',
   *      publishes: {
   *          value: true,
   *          rawValue: true,
   *          dirty: true
   *      }
   *  }
   * </code></pre>
   *
   * <p>In some cases, users may want to publish a property to the viewModel that is not found in a class
   * config block. In these situations, you may utilize <a href="#!/api/Ext.mixin.Bindable-method-publishState" rel="Ext.mixin.Bindable-method-publishState" class="docClass">publishState</a> if the property has a
   * setter method.  Let's use <a href="#!/api/Ext.form.Labelable-method-setFieldLabel" rel="Ext.form.Labelable-method-setFieldLabel" class="docClass">setFieldLabel</a> as an example:</p>
   *
   * <pre><code>  setFieldLabel: function(fieldLabel) {
   *       this.callParent(arguments);
   *       this.publishState('fieldLabel', fieldLabel);
   *   }
   * </code></pre>
   *
   * <p>With the above chunk of code, fieldLabel may now be published to the viewModel.</p>
   * @since 5.0.0
   * @see #getPublishes()
   * @see #setPublishes()
   */
  public native function get publishes():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set publishes(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Specifies a name for this component inside its component hierarchy. This name
   * must be unique within its <a href="#!/api/Ext.container.Container-cfg-referenceHolder" rel="Ext.container.Container-cfg-referenceHolder" class="docClass">view</a>
   * or its <a href="#!/api/Ext.app.ViewController" rel="Ext.app.ViewController" class="docClass">ViewController</a>. See the documentation in
   * <a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Ext.container.Container</a> for more information about references.
   *
   * <p><strong>Note</strong>: Valid identifiers start with a letter or underscore and are followed
   * by zero or more additional letters, underscores or digits. References are case
   * sensitive.</p>
   * @see #getReference()
   * @see #setReference()
   */
  public native function get reference():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set reference(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * If provided this creates a new <code>Session</code> instance for this component. If this
   * is a <code>Container</code>, this will then be inherited by all child components.
   *
   * <p>To create a new session you can specify <code>true</code>:</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>({
   *      xtype: 'viewport',
   *      session: true,
   *
   *      items: [{
   *          ...
   *      }]
   *  });
   * </code></pre>
   *
   * <p>Alternatively, a config object can be provided:</p>
   *
   * <pre><code> <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>({
   *      xtype: 'viewport',
   *      session: {
   *          ...
   *      },
   *
   *      items: [{
   *          ...
   *      }]
   *  });
   * </code></pre>
   * @default null
   * @see #getSession()
   * @see #setSession()
   */
  public native function get session():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set session(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Additional CSS styles that will be rendered into an inline style attribute when
   * the widget is rendered.
   *
   * <p>You can pass either a string syntax:</p>
   *
   * <pre><code>style: 'background:red'
   * </code></pre>
   *
   * <p>Or by using an object:</p>
   *
   * <pre><code>style: {
   *     background: 'red'
   * }
   * </code></pre>
   *
   * <p>When using the object syntax, you can define CSS Properties by using a string:</p>
   *
   * <pre><code>style: {
   *     'border-left': '1px solid red'
   * }
   * </code></pre>
   *
   * <p>Although the object syntax is much easier to read, we suggest you to use the
   * string syntax for better performance.</p>
   * @see #getStyle()
   * @see #setStyle()
   */
  public native function get style():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set style(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Emulates the behavior of the CSS
   * touch-action
   * property in a cross-browser compatible manner.
   *
   * <p>Keys in this object are touch action names, and values are <code>false</code> to disable
   * a touch action or <code>true</code> to enable it.  Accepted keys are:</p>
   *
   * <ul>
   * <li><code>panX</code></li>
   * <li><code>panY</code></li>
   * <li><code>pinchZoom</code></li>
   * <li><code>doubleTapZoom</code></li>
   * </ul>
   *
   *
   * <p>All touch actions are enabled (<code>true</code>) by default, so it is usually only necessary
   * to specify which touch actions to disable.  For example, the following disables
   * only horizontal scrolling and pinch-to-zoom on the component's main element:</p>
   *
   * <pre><code>touchAction: {
   *     panX: false,
   *     pinchZoom: false
   * }
   * </code></pre>
   *
   * <p>Touch actions can be specified on reference elements using the reference element
   * name, for example:</p>
   *
   * <pre><code>// disables horizontal scrolling on the main element, and double-tap-zoom
   * // on the child element named "body"
   * touchAction: {
   *     panY: false
   *     body: {
   *         doubleTapZoom: false
   *     }
   * }
   * </code></pre>
   *
   * <p>The primary motivation for setting the touch-action of an element is to prevent
   * the browser's default handling of a gesture such as pinch-to-zoom, or
   * drag-to-scroll, so that the application can implement its own handling of that
   * gesture on the element.  Suppose, for example, a component has a custom drag
   * handler on its element and wishes to prevent horizontal scrolling of its container
   * while it is being dragged:</p>
   *
   * <pre><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.Widget" rel="Ext.Widget" class="docClass">Ext.Widget</a>', {
   *     touchAction: {
   *         panX: false
   *     },
   *     listeners: {
   *         drag: function(e) {
   *             // implement drag logic
   *         }
   *     }
   * });
   * </code></pre>
   * @see #getTouchAction()
   * @see #setTouchAction()
   */
  public native function get touchAction():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set touchAction(value:Object):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * This object holds a map of <code>config</code> properties that will update their binding
   * as they are modified. For example, <code>value</code> is a key added by form fields. The
   * form of this config is the same as <code><a href="#!/api/Ext.mixin.Bindable-cfg-publishes" rel="Ext.mixin.Bindable-cfg-publishes" class="docClass">publishes</a></code>.
   *
   * <p>This config is defined so that updaters are not created and added for all
   * bound properties since most cannot be modified by the end-user and hence are
   * not appropriate for two-way binding.</p>
   * @see #getTwoWayBindable()
   * @see #setTwoWayBindable()
   */
  public native function get twoWayBindable():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set twoWayBindable(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The ui or uis to be used on this Component
   *
   * <p>When a ui is configured, CSS class names are added to the <a href="#!/api/Ext.Widget-property-element" rel="Ext.Widget-property-element" class="docClass">element</a>, created
   * by appending the ui name(s) to each <a href="#!/api/Ext.Widget-property-classCls" rel="Ext.Widget-property-classCls" class="docClass">classCls</a> and/or <a href="#!/api/Ext.Widget-cfg-baseCls" rel="Ext.Widget-cfg-baseCls" class="docClass">baseCls</a>.</p>
   * @see #getUi()
   * @see #setUi()
   */
  public native function get ui():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set ui(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * One or more CSS classes to add to the component's primary element. This config
   * is intended solely for use by the component instantiator (the "user"), not by
   * derived classes.
   *
   * <p>For example:</p>
   *
   * <pre><code> items: [{
   *      xtype: 'button',
   *      userCls: 'my-button'
   *  ...
   *  }]
   * </code></pre>
   * @see #getUserCls()
   * @see #setUserCls()
   */
  public native function get userCls():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set userCls(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The <code>ViewModel</code> is a data provider for this component and its children. The
   * data contained in the <code>ViewModel</code> is typically used by adding <code>bind</code> configs
   * to the components that want present or edit this data.
   *
   * <p>When set, the <code>ViewModel</code> is created and links to any inherited <code>viewModel</code>
   * instance from an ancestor container as the "parent". The <code>ViewModel</code> hierarchy,
   * once established, only supports creation or destruction of children. The
   * parent of a <code>ViewModel</code> cannot be changed on the fly.</p>
   *
   * <p>If this is a root-level <code>ViewModel</code>, the data model connection is made to this
   * component's associated <code><a href="#!/api/Ext.data.Session" rel="Ext.data.Session" class="docClass">Data Session</a></code>. This is
   * determined by calling <code>getInheritedSession</code>.</p>
   * @default {$value: null, lazy: true}
   * @see #getViewModel()
   * @see #setViewModel()
   */
  public native function get viewModel():ext.app.ViewModel;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set viewModel(value:ext.app.ViewModel):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The width of this Component; must be a valid CSS length value, e.g: <code>300</code>, <code>100px</code>, <code>30%</code>, etc.
   * By default, if this is not explicitly set, this Component's element will simply have its own natural size.
   * If set to <code>auto</code>, it will set the width to <code>null</code> meaning it will have its own natural size.
   * @see #getWidth()
   * @see #setWidth()
   */
  public native function get width():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set width(value:*):void;

  /**
   * Adds a plugin. For example:
   *
   * <pre><code> list.addPlugin('pullrefresh');
   * </code></pre>
   *
   * <p>Or:</p>
   *
   * <pre><code> list.addPlugin({
   *      type: 'pullrefresh',
   *      pullRefreshText: 'Pull to refresh...'
   *  });
   * </code></pre>
   * @since 6.2.0
   * @param plugin The plugin or config object or
   * alias to add.
   */
  public native function addPlugin(plugin:*):void;

  /**
   * Removes and destroys a plugin.
   *
   * <p><strong>Note:</strong> Not all plugins are designed to be removable. Consult the documentation
   * for the specific plugin in question to be sure.</p>
   * @since 6.2.0
   * @param plugin The plugin or its <code>id</code> to remove.
   * @return plugin instance or <code>null</code> if not found.
   */
  public native function destroyPlugin(plugin:*):ext.plugin.AbstractPlugin;

  /**
   * Perform the actual destruction sequence. This is the method to override in your
   * subclasses to add steps specific to the destruction of custom Component or Widget.
   *
   * <p>As a rule of thumb, subclasses should destroy their child Components, Elements,
   * and/or other objects before calling parent method. Any object references will be
   * nulled after this method has finished, to prevent the possibility of memory leaks.</p>
   * @since 6.2.0
   */
  public native function doDestroy():void;

  /**
   * Retrieves plugin by its <code>type</code> alias. For example:
   *
   * <pre><code> var list = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>({
   *      xtype: 'list',
   *      itemTpl: '&lt;div class="item"&gt;{title}&lt;/div&gt;',
   *      store: 'Items',
   *
   *      plugins: ['listpaging', 'pullrefresh']
   *  });
   *
   *  list.findPlugin('pullrefresh').setPullRefreshText('Pull to refresh...');
   * </code></pre>
   *
   * <p><strong>Note:</strong> See also <a href="#!/api/Ext.mixin.Pluggable-method-getPlugin" rel="Ext.mixin.Pluggable-method-getPlugin" class="docClass">getPlugin</a>.</p>
   * @since 6.2.0
   * @param type The Plugin's <code>type</code> as specified by the class's
   * <a href="#!/api/Ext.Class-cfg-alias" rel="Ext.Class-cfg-alias" class="docClass">alias</a> configuration.
   * @return plugin instance or <code>null</code> if not found.
   */
  public native function findPlugin(type:String):ext.plugin.AbstractPlugin;

  /**
   * Returns the value of <a href="#!/api/Ext.mixin.Bindable-cfg-bind" rel="Ext.mixin.Bindable-cfg-bind" class="docClass">bind</a>.
   */
  public native function getBind():Object;

  /**
   * Returns the value of <a href="#!/api/Ext.Widget-cfg-border" rel="Ext.Widget-cfg-border" class="docClass">border</a>.
   */
  public native function getBorder():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.Widget-cfg-cls" rel="Ext.Widget-cfg-cls" class="docClass">cls</a>.
   */
  public native function getCls():*;

  /**
   * Returns the <a href="#!/api/Ext.app.ViewController" rel="Ext.app.ViewController" class="docClass">Ext.app.ViewController</a> instance associated with this
   * component via the <a href="#!/api/Ext.mixin.Bindable-cfg-controller" rel="Ext.mixin.Bindable-cfg-controller" class="docClass">controller</a> config or <a href="#!/api/Ext.mixin.Bindable-method-setController" rel="Ext.mixin.Bindable-method-setController" class="docClass">setController</a> method.
   * @return Returns this component's ViewController or
   * null if one was not configured
   */
  public native function getController():ext.app.ViewController;

  /**
   * Returns the value of <a href="#!/api/Ext.mixin.Bindable-cfg-defaultListenerScope" rel="Ext.mixin.Bindable-cfg-defaultListenerScope" class="docClass">defaultListenerScope</a>.
   */
  public native function getDefaultListenerScope():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.Widget-cfg-height" rel="Ext.Widget-cfg-height" class="docClass">height</a>.
   */
  public native function getHeight():*;

  /**
   * Returns the value of <a href="#!/api/Ext.Widget-cfg-hidden" rel="Ext.Widget-cfg-hidden" class="docClass">hidden</a>.
   */
  public native function getHidden():Boolean;

  /**
   * This method returns an object containing the inherited properties for this instance.
   * @since 5.0.0
   * @param inner Pass <code>true</code> to return <code>inheritedStateInner</code> instead
   * of the normal <code>inheritedState</code> object. This is only needed internally and should
   * not be passed by user code.
   * @default false
   * @return The <code>inheritedState</code> object containing inherited properties.
   */
  public native function getInherited(inner:Boolean = false):Object;

  /**
   * This method returns the value of a config property that may be inherited from some
   * ancestor.
   *
   * <p>In some cases, a config may be explicitly set on a component with the intent of
   * <em>only</em> being presented to its children while that component should act upon the
   * inherited value (see <code>referenceHolder</code> for example). In these cases the <code>skipThis</code>
   * parameter should be specified as <code>true</code>.</p>
   * @since 5.0.0
   * @param property The name of the config property to return.
   * @param skipThis Pass <code>true</code> if the property should be ignored if
   * found on this instance. In other words, <code>true</code> means the property must be inherited
   * and not explicitly set on this instance.
   * @default false
   * @return The value of the requested <code>property</code>.
   */
  public native function getInheritedConfig(property:String, skipThis:Boolean = false):*;

  /**
   * Retrieves a plugin by its <code>id</code>.
   *
   * <pre><code> var list = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>({
   *      xtype: 'list',
   *      itemTpl: '&lt;div class="item"&gt;{title}&lt;/div&gt;',
   *      store: 'Items',
   *
   *      plugins: {
   *          xclass: 'Ext.plugin.PullRefresh',
   *          id: 'foo'
   *      }
   *  });
   *
   *  list.getPlugin('foo').setPullRefreshText('Pull to refresh...');
   * </code></pre>
   *
   * <p><strong>Note:</strong> See also <a href="#!/api/Ext.mixin.Pluggable-method-findPlugin" rel="Ext.mixin.Pluggable-method-findPlugin" class="docClass">findPlugin</a>.</p>
   * @since 6.2.0
   * @param id The <code>id</code> of the plugin.
   * @return plugin instance or <code>null</code> if not found.
   */
  public native function getPlugin(id:String):ext.plugin.AbstractPlugin;

  /**
   * Returns the value of <a href="#!/api/Ext.mixin.Bindable-cfg-publishes" rel="Ext.mixin.Bindable-cfg-publishes" class="docClass">publishes</a>.
   * @since 5.0.0
   */
  public native function getPublishes():*;

  /**
   * Returns the value of <a href="#!/api/Ext.mixin.Bindable-cfg-reference" rel="Ext.mixin.Bindable-cfg-reference" class="docClass">reference</a>.
   */
  public native function getReference():String;

  /**
   * Returns the value of <a href="#!/api/Ext.mixin.Bindable-cfg-session" rel="Ext.mixin.Bindable-cfg-session" class="docClass">session</a>.
   */
  public native function getSession():*;

  /**
   * Returns the height and width of the Component.
   * @return The current <code>height</code> and <code>width</code> of the Component.
   * <ul>
   *   <li><code>width</code> : Number</li>
   *   <li><code>height</code> : Number</li>
   * </ul>
   */
  public native function getSize():Object;

  /**
   * Returns the value of <a href="#!/api/Ext.Widget-cfg-style" rel="Ext.Widget-cfg-style" class="docClass">style</a>.
   */
  public native function getStyle():*;

  /**
   * Returns the value of <a href="#!/api/Ext.Widget-cfg-touchAction" rel="Ext.Widget-cfg-touchAction" class="docClass">touchAction</a>.
   */
  public native function getTouchAction():Object;

  /**
   * Returns the value of <a href="#!/api/Ext.mixin.Bindable-cfg-twoWayBindable" rel="Ext.mixin.Bindable-cfg-twoWayBindable" class="docClass">twoWayBindable</a>.
   */
  public native function getTwoWayBindable():*;

  /**
   * Returns the value of <a href="#!/api/Ext.Widget-cfg-ui" rel="Ext.Widget-cfg-ui" class="docClass">ui</a>.
   */
  public native function getUi():*;

  /**
   * Returns the value of <a href="#!/api/Ext.Widget-cfg-userCls" rel="Ext.Widget-cfg-userCls" class="docClass">userCls</a>.
   */
  public native function getUserCls():*;

  /**
   * Returns the value of <a href="#!/api/Ext.mixin.Bindable-cfg-viewModel" rel="Ext.mixin.Bindable-cfg-viewModel" class="docClass">viewModel</a>.
   */
  public native function getViewModel():*;

  /**
   * Returns the value of <a href="#!/api/Ext.Widget-cfg-width" rel="Ext.Widget-cfg-width" class="docClass">width</a>.
   */
  public native function getWidth():*;

  /**
   * Tests whether this Widget matches a <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">ComponentQuery</a>
   * selector string.
   * @param selector The selector string to test against.
   * @return <code>true</code> if this Widget matches the selector.
   */
  public native function matches(selector:String):Boolean;

  /**
   * Returns <code>true</code> if this Component is currently hidden.
   * @param deep <code>true</code> to check if this component
   * is hidden because a parent container is hidden. Alternatively, a reference to the
   * top-most parent at which to stop climbing.
   * @default false
   * @return <code>true</code> if currently hidden.
   */
  public native function isHidden(deep:* = false):Boolean;

  /**
   * Returns <code>true</code> if this Component is currently visible.
   * @param deep <code>true</code> to check if this component
   * is visible and all parents are also visible.
   * @default false
   * @return <code>true</code> if currently visible.
   */
  public native function isVisible(deep:Boolean = false):Boolean;

  /**
   * Tests whether or not this Component is of a specific xtype. This can test whether this Component is descended
   * from the xtype (default) or whether it is directly of the xtype specified (<code>shallow = true</code>).
   * *If using your own subclasses, be aware that a Component must register its own xtype
   * to participate in determination of inherited xtypes.__
   *
   * <p>For a list of all available xtypes, see the <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Ext.Component</a> header.</p>
   *
   * <p>Example usage:</p>
   *
   * <pre><code>var t = new Ext.field.Text();
   * var isText = t.isXType('textfield'); // true
   * var isBoxSubclass = t.isXType('field'); // true, descended from Ext.field.Field
   * var isBoxInstance = t.isXType('field', true); // false, not a direct Ext.field.Field instance
   * </code></pre>
   * @param xtype The xtype to check for this Component.
   * @param shallow <code>false</code> to check whether this Component is descended from the xtype (this is
   * the default), or <code>true</code> to check whether this Component is directly of the specified xtype.
   * @return <code>true</code> if this component descends from the specified xtype, <code>false</code> otherwise.
   */
  public native function isXType(xtype:String, shallow:Boolean = false):Boolean;

  /**
   * Gets the controller that controls this view. May be a controller that belongs
   * to a view higher in the hierarchy.
   * @since 5.0.1
   * @param skipThis <code>true</code> to not consider the controller directly attached
   * to this view (if it exists).
   * @default false
   * @return The controller. <code>null</code> if no controller is found.
   */
  public native function lookupController(skipThis:Boolean = false):ext.app.ViewController;

  /**
   * Returns the <code><a href="#!/api/Ext.data.Session" rel="Ext.data.Session" class="docClass">Ext.data.Session</a></code> for this instance. This property may come
   * from this instance's <code><a href="#!/api/Ext.mixin.Bindable-cfg-session" rel="Ext.mixin.Bindable-cfg-session" class="docClass">session</a></code> or be inherited from this object's parent.
   * @since 5.0.0
   * @param skipThis Pass <code>true</code> to ignore a <code>session</code> configured on
   * this instance and only consider an inherited session.
   * @default false
   */
  public native function lookupSession(skipThis:Boolean = false):ext.data.Session;

  /**
   * Gets a named template instance for this class. See <a href="#!/api/Ext.XTemplate-static-method-getTpl" rel="Ext.XTemplate-static-method-getTpl" class="docClass">Ext.XTemplate.getTpl</a>.
   * @since 6.2.0
   * @param name The name of the property that holds the template.
   * @return The template, <code>null</code> if not found.
   */
  public native function lookupTpl(name:String):ext.XTemplate;

  /**
   * Returns the <code><a href="#!/api/Ext.app.ViewModel" rel="Ext.app.ViewModel" class="docClass">Ext.app.ViewModel</a></code> for this instance. This property may come from this
   * this instance's <code><a href="#!/api/Ext.mixin.Bindable-cfg-viewModel" rel="Ext.mixin.Bindable-cfg-viewModel" class="docClass">viewModel</a></code> or be inherited from this object's parent.
   * @since 5.0.0
   * @param skipThis Pass <code>true</code> to ignore a <code>viewModel</code> configured on
   * this instance and only consider an inherited view model.
   * @default false
   */
  public native function lookupViewModel(skipThis:Boolean = false):ext.app.ViewModel;

  /**
   * Called when this Inheritable is added to a parent
   */
  public native function onInheritedAdd(instanced:Boolean):void;

  /**
   * Called when this inheritable is removed from a parent
   * @param destroying <code>true</code> if this item will be destroyed by it's container
   */
  public native function onInheritedRemove(destroying:Boolean):void;

  /**
   * Removes and (optionally) destroys a plugin.
   *
   * <p><strong>Note:</strong> Not all plugins are designed to be removable. Consult the documentation
   * for the specific plugin in question to be sure.</p>
   * @since 6.2.0
   * @param plugin The plugin or its <code>id</code> to remove.
   * @param destroy Pass <code>true</code> to not call <code>destroy()</code> on the plugin.
   * @return plugin instance or <code>null</code> if not found.
   */
  public native function removePlugin(plugin:*, destroy:Boolean = false):ext.plugin.AbstractPlugin;

  /**
   * Sets the value of <a href="#!/api/Ext.mixin.Bindable-cfg-bind" rel="Ext.mixin.Bindable-cfg-bind" class="docClass">bind</a>.
   * @param bind The new value.
   */
  public native function setBind(bind:Object):void;

  /**
   * Sets the value of <a href="#!/api/Ext.Widget-cfg-border" rel="Ext.Widget-cfg-border" class="docClass">border</a>.
   * @param border The new value.
   */
  public native function setBorder(border:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.Widget-cfg-cls" rel="Ext.Widget-cfg-cls" class="docClass">cls</a>.
   * @param cls The new value.
   */
  public native function setCls(cls:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.mixin.Bindable-cfg-controller" rel="Ext.mixin.Bindable-cfg-controller" class="docClass">controller</a>.
   * @param controller The new value.
   */
  public native function setController(controller:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.mixin.Bindable-cfg-defaultListenerScope" rel="Ext.mixin.Bindable-cfg-defaultListenerScope" class="docClass">defaultListenerScope</a>.
   * @param defaultListenerScope The new value.
   */
  public native function setDefaultListenerScope(defaultListenerScope:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.Widget-cfg-height" rel="Ext.Widget-cfg-height" class="docClass">height</a>.
   * @param height The new value.
   */
  public native function setHeight(height:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.Widget-cfg-hidden" rel="Ext.Widget-cfg-hidden" class="docClass">hidden</a>.
   * @param hidden The new value.
   */
  public native function setHidden(hidden:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.mixin.Bindable-cfg-publishes" rel="Ext.mixin.Bindable-cfg-publishes" class="docClass">publishes</a>.
   * @since 5.0.0
   * @param publishes The new value.
   */
  public native function setPublishes(publishes:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.mixin.Bindable-cfg-reference" rel="Ext.mixin.Bindable-cfg-reference" class="docClass">reference</a>.
   * @param reference The new value.
   */
  public native function setReference(reference:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.mixin.Bindable-cfg-session" rel="Ext.mixin.Bindable-cfg-session" class="docClass">session</a>.
   * @param session The new value.
   */
  public native function setSession(session:*):void;

  /**
   * Sets the size of the Component.
   * @param width The new width for the Component.
   * @param height The new height for the Component.
   */
  public native function setSize(width:Number, height:Number):void;

  /**
   * Sets the value of <a href="#!/api/Ext.Widget-cfg-style" rel="Ext.Widget-cfg-style" class="docClass">style</a>.
   * @param style The new value.
   */
  public native function setStyle(style:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.Widget-cfg-touchAction" rel="Ext.Widget-cfg-touchAction" class="docClass">touchAction</a>.
   * @param touchAction The new value.
   */
  public native function setTouchAction(touchAction:Object):void;

  /**
   * Sets the value of <a href="#!/api/Ext.mixin.Bindable-cfg-twoWayBindable" rel="Ext.mixin.Bindable-cfg-twoWayBindable" class="docClass">twoWayBindable</a>.
   * @param twoWayBindable The new value.
   */
  public native function setTwoWayBindable(twoWayBindable:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.Widget-cfg-ui" rel="Ext.Widget-cfg-ui" class="docClass">ui</a>.
   * @param ui The new value.
   */
  public native function setUi(ui:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.Widget-cfg-userCls" rel="Ext.Widget-cfg-userCls" class="docClass">userCls</a>.
   * @param userCls The new value.
   */
  public native function setUserCls(userCls:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.mixin.Bindable-cfg-viewModel" rel="Ext.mixin.Bindable-cfg-viewModel" class="docClass">viewModel</a>.
   * @param viewModel The new value.
   */
  public native function setViewModel(viewModel:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.Widget-cfg-width" rel="Ext.Widget-cfg-width" class="docClass">width</a>.
   * @param width The new value.
   */
  public native function setWidth(width:*):void;

  /**
   * Toggles the specified CSS class on this element (removes it if it already exists,
   * otherwise adds it).
   * @param className The CSS class to toggle.
   * @param state If specified as <code>true</code>, causes the class to be added. If
   * specified as <code>false</code>, causes the class to be removed.
   */
  public native function toggleCls(className:String, state:Boolean = false):void;

  protected native function applyStyle(style:Object, oldStyle:Object):void;

  /**
   * Returns the value of <a href="#!/api/Ext.Widget-cfg-baseCls" rel="Ext.Widget-cfg-baseCls" class="docClass">baseCls</a>.
   */
  protected native function getBaseCls():*;

  /**
   * A template method for modifying the <a href="#!/api/Ext.Widget-property-element" rel="Ext.Widget-property-element" class="docClass">element</a> config before it is processed.
   * By default adds the result of <code>this.getTemplate()</code> as the <code>children</code> array of
   * <a href="#!/api/Ext.Widget-property-element" rel="Ext.Widget-property-element" class="docClass">element</a> if <code>children</code> were not specified in the original
   * <a href="#!/api/Ext.Widget-property-element" rel="Ext.Widget-property-element" class="docClass">element</a> config.  Typically this method should not need to be implemented
   * in subclasses.  Instead the <a href="#!/api/Ext.Widget-property-element" rel="Ext.Widget-property-element" class="docClass">element</a> property should be use to configure
   * the element template for a given Widget subclass.
   *
   * <p>This method is called once when the first instance of each Widget subclass is
   * created.  The element config object that is returned is cached and used as the template
   * for all successive instances.  The scope object for this method is the class prototype,
   * not the instance.</p>
   * @return the element config object
   */
  protected native function getElementConfig():Object;

  /**
   * Used by <a href="#!/api/Ext.ComponentQuery" rel="Ext.ComponentQuery" class="docClass">ComponentQuery</a>, and the <a href="#!/api/Ext.Component-method-up" rel="Ext.Component-method-up" class="docClass">up</a>
   * method to find the owning Component in the linkage hierarchy.
   *
   * <p>By default this returns the Container which contains this Component.</p>
   *
   * <p>This may be overridden by Component authors who implement ownership hierarchies
   * which are not based upon ownerCt, such as BoundLists being owned by Fields or Menus
   * being owned by Buttons.</p>
   */
  protected native function getRefOwner():ext.Component;

  /**
   * Initializes the Element for this Widget instance.  If this is the first time a
   * Widget of this type has been instantiated the <a href="#!/api/Ext.Widget-property-element" rel="Ext.Widget-property-element" class="docClass">element</a> config will be
   * processed to create an Element.  This Element is then cached on the prototype (see
   * afterCachedConfig) so that future instances can obtain their element by simply
   * cloning the Element that was cached by the first instance.
   */
  protected native function initElement():void;

  /**
   * Called by <code>getInherited</code> to initialize the inheritedState the first time it is
   * requested.
   */
  protected native function initInheritedState(inheritedState:Object):void;

  /**
   * Checks if a particular binding is synchronizing the value.
   * @param name The name of the property being bound to.
   * @return <code>true</code> if the binding is syncing.
   */
  protected native function isSyncing(name:String):Boolean;

  /**
   * Publish this components state to the <code>ViewModel</code>. If no arguments are given (or if
   * this is the first call), the entire state is published. This state is determined by
   * the <code>publishes</code> property.
   *
   * <p>This method is called only by component authors.</p>
   * @since 5.0.0
   * @param property The name of the property to update.
   * @param value The value of <code>property</code>. Only needed if <code>property</code> is given.
   */
  protected native function publishState(property:String = null, value:Object = null):void;

  /**
   * Returns the default listener scope for a "satellite" of this component.
   * Used for resolving scope for observable objects that are not part of the normal
   * Container/Component hierarchy (for example, plugins)
   * @since 5.1.1
   * @return The listener scope
   */
  protected native function resolveSatelliteListenerScope(satellite:ext.mixin.Observable, defaultScope:Object = null):Object;

  /**
   * Sets the value of <a href="#!/api/Ext.Widget-cfg-baseCls" rel="Ext.Widget-cfg-baseCls" class="docClass">baseCls</a>.
   * @param baseCls The new value.
   */
  protected native function setBaseCls(baseCls:*):void;

  protected native function updateHeight(height:Object):void;

  protected native function updateStyle(style:Object):void;

  protected native function updateWidth(width:Object):void;
}
}