package ext.button {
import ext.menu.CheckItem;
import ext.menu.Menu;

[Uses("ext.button.SplitButton")]
[Uses("ext.button.Button")]
[Uses("ext.Component")]
[Uses("ext.util.Positionable")]
[Uses("ext.Base")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Animate")]
[Uses("ext.util.Floating")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.state.Stateful")]
[Uses("ext.mixin.Queryable")]
[Native("Ext.button.Cycle", require)]
/**
 * Fires after the button's active menu item has changed. Note that if a <a href="#!/api/Ext.button.Cycle-cfg-changeHandler" rel="Ext.button.Cycle-cfg-changeHandler" class="docClass">changeHandler</a> function is
 * set on this CycleButton, it will be called instead on active item change and this change event will not
 * be fired.
 * @eventType ext.button.events.CycleButton_itemEvent.CHANGE
 */
[Event(name = 'onChange', type = 'ext.button.events.CycleButton_itemEvent')]
/**
 * A specialized SplitButton that contains a menu of <a href="#!/api/Ext.menu.CheckItem" rel="Ext.menu.CheckItem" class="docClass">Ext.menu.CheckItem</a> elements. The button automatically
 * cycles through each menu item on click, raising the button's <a href="#!/api/Ext.button.Cycle-event-change" rel="Ext.button.Cycle-event-change" class="docClass">change</a> event (or calling the button's
 * <a href="#!/api/Ext.button.Cycle-cfg-changeHandler" rel="Ext.button.Cycle-cfg-changeHandler" class="docClass">changeHandler</a> function, if supplied) for the active menu item. Clicking on the arrow section of the
 * button displays the dropdown menu just like a normal SplitButton.  Example usage:
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.button.Cycle" rel="Ext.button.Cycle" class="docClass">Ext.button.Cycle</a>', {
 *     showText: true,
 *     prependText: 'View as ',
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     menu: {
 *         id: 'view-type-menu',
 *         items: [{
 *             text: 'text only',
 *             iconCls: 'view-text',
 *             checked: true
 *         },{
 *             text: 'HTML',
 *             iconCls: 'view-html'
 *         }]
 *     },
 *     changeHandler: function(cycleBtn, activeItem) {
 *         <a href="#!/api/Ext.MessageBox-method-alert" rel="Ext.MessageBox-method-alert" class="docClass">Ext.Msg.alert</a>('Change View', activeItem.text);
 *     }
 * });
 * </code></pre>
 */
public class CycleButton extends ext.button.SplitButton {
  /**
   * @param config @inheritDoc
   */
  public function CycleButton(config:ext.button.CycleButton = null) {
    super();
  }

  /**
   * The <a href="#!/api/Ext.menu.Menu" rel="Ext.menu.Menu" class="docClass">Menu</a> object used to display the <a href="#!/api/Ext.menu.CheckItem" rel="Ext.menu.CheckItem" class="docClass">CheckItems</a> representing the
   * available choices.
   */
  override public native function get menu():ext.menu.Menu;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set menu(value:ext.menu.Menu):void;

  [ExtConfig]
  /**
   * A callback function that will be invoked each time the active menu item in the button's menu has changed. If this
   * callback is not supplied, the SplitButton will instead fire the <a href="#!/api/Ext.button.Cycle-event-change" rel="Ext.button.Cycle-event-change" class="docClass">change</a> event on active item change. The
   * changeHandler function will be called with the following argument list: (SplitButton this, <a href="#!/api/Ext.menu.CheckItem" rel="Ext.menu.CheckItem" class="docClass">Ext.menu.CheckItem</a>
   * item)
   * &#64;controllable
   * @default undefined
   */
  public native function get changeHandler():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set changeHandler(value:*):void;

  [ExtConfig]
  /**
   * The charCode to be used as the static icon for this button.  This icon will always be
   * displayed regardless of which item is selected in the dropdown list. This override
   * the default behavior of changing the button's icon to match the selected item's icon
   * on change. This property expects a format consistent with that of <a href="#!/api/Ext.button.Cycle-cfg-glyph" rel="Ext.button.Cycle-cfg-glyph" class="docClass">glyph</a>
   */
  public native function get forceGlyph():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set forceGlyph(value:*):void;

  [ExtConfig]
  /**
   * A css class which sets an image to be used as the static icon for this button. This icon will always be displayed
   * regardless of which item is selected in the dropdown list. This overrides the default behavior of changing the
   * button's icon to match the selected item's icon on change.
   */
  public native function get forceIcon():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set forceIcon(value:String):void;

  [Deprecated(replacement = "menu", since = "4.0")]
  [ExtConfig]
  /**
   * An array of <a href="#!/api/Ext.menu.CheckItem" rel="Ext.menu.CheckItem" class="docClass">Ext.menu.CheckItem</a> <strong>config</strong> objects to be used when creating the button's menu items (e.g.,
   * <code>{text:'Foo', iconCls:'foo-icon'}</code>)
   */
  public native function get items():Array;

  [ExtConfig]
  /**
   * @private
   */
  public native function set items(value:Array):void;

  [ExtConfig]
  /**
   * A static string to prepend before the active item's text when displayed as the button's text (only applies when
   * showText = true).
   * @default ''
   */
  public native function get prependText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set prependText(value:String):void;

  [ExtConfig]
  /**
   * True to display the active item's text as the button text. The Button will show its
   * configured <a href="#!/api/Ext.button.Cycle-cfg-text" rel="Ext.button.Cycle-cfg-text" class="docClass">text</a> if this config is omitted.
   * @default false
   */
  public native function get showText():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set showText(value:Boolean):void;

  /**
   * Gets the currently active menu item.
   * @return The active item
   */
  public native function getActiveItem():ext.menu.CheckItem;

  /**
   * Sets the button's active menu item.
   * @param item The item to activate
   * @param suppressEvent True to prevent the <a href="#!/api/Ext.button.Cycle-event-change" rel="Ext.button.Cycle-event-change" class="docClass">change</a> event and <a href="#!/api/Ext.button.Cycle-cfg-changeHandler" rel="Ext.button.Cycle-cfg-changeHandler" class="docClass">changeHandler</a> from firing.
   * @default false
   */
  public native function setActiveItem(item:ext.menu.CheckItem, suppressEvent:Boolean = false):void;

  /**
   * This is normally called internally on button click, but can be called externally to advance the button's active
   * item programmatically to the next one in the menu. If the current item is the last one in the menu the active
   * item will be set to the first item in the menu.
   */
  public native function toggleSelected():void;
}
}