package ext.button {
import ext.container.Container;

[Uses("ext.container.Container")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.Base")]
[Uses("ext.mixin.Container")]
[Uses("ext.Mixin")]
[Uses("ext.Component")]
[Uses("ext.util.Positionable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Animate")]
[Uses("ext.util.Floating")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.state.Stateful")]
[Native("Ext.button.Segmented", require)]
/**
 * Fires when any child button's pressed state has changed and caused the value to change.
 * @eventType ext.button.events.SegmentedButton_newValue_oldValueEvent.CHANGE
 */
[Event(name = 'onChange', type = 'ext.button.events.SegmentedButton_newValue_oldValueEvent')]
/**
 * Fires when any child button's pressed state has changed.
 * @eventType ext.button.events.SegmentedButton_button_isPressedEvent.TOGGLE
 */
[Event(name = 'onToggle', type = 'ext.button.events.SegmentedButton_button_isPressedEvent')]
/**
 * SegmentedButton is a container for a group of <a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">Button</a>s.  You
 * may populate the segmented button's children by adding buttons to the items config.  The segmented
 * button's children enjoy the same customizations as regular buttons, such as
 * menu, tooltip, etc.  You can see usages of the various configuration
 * possibilities in the example below.
 *
 * <pre class='inline-example @preview'><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.button.Segmented" rel="Ext.button.Segmented" class="docClass">Ext.button.Segmented</a>', {
 *      renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *      allowMultiple: true,
 *      items: [{
 *           text: 'Segment Item 1',
 *           menu: [{
 *               text: 'Menu Item 1'
 *           }]
 *      },{
 *           text: 'Segment Item 2',
 *           tooltip: 'My custom tooltip'
 *      },{
 *           text: 'Segment Item 3'
 *      }],
 *      listeners: {
 *           toggle: function(container, button, pressed) {
 *                console.log("User toggled the '" + button.text + "' button: " + (pressed ? 'on' : 'off'));
 *           }
 *      }
 * });
 * </code></pre>
 */
public class SegmentedButton extends ext.container.Container {
  /**
   * @param config @inheritDoc
   */
  public function SegmentedButton(config:ext.button.SegmentedButton = null) {
    super();
  }

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Allow toggling the pressed state of each button.
   * Only applicable when <a href="#!/api/Ext.button.Segmented-cfg-allowMultiple" rel="Ext.button.Segmented-cfg-allowMultiple" class="docClass">allowMultiple</a> is <code>false</code>.
   * @default false
   * @see #getAllowDepress()
   * @see #setAllowDepress()
   */
  public native function get allowDepress():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set allowDepress(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Allow multiple pressed buttons.
   * @default false
   * @see #getAllowMultiple()
   * @see #setAllowMultiple()
   */
  public native function get allowMultiple():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set allowMultiple(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * True to enable pressed/not pressed toggling.
   * @default true
   * @see #getAllowToggle()
   * @see #setAllowToggle()
   */
  public native function get allowToggle():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set allowToggle(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Default <a href="#!/api/Ext.Component-cfg-ui" rel="Ext.Component-cfg-ui" class="docClass">ui</a> to use for buttons in this segmented button.
   * Buttons can override this default by specifying their own UI
   * @default 'default'
   * @see #getDefaultUI()
   * @see #setDefaultUI()
   */
  public native function get defaultUI():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set defaultUI(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * If <a href="#!/api/Ext.button.Segmented-cfg-allowMultiple" rel="Ext.button.Segmented-cfg-allowMultiple" class="docClass">allowMultiple</a> is <code>true</code>, this config may be set to <code>true</code> to indicate that at least
   * one button in the set must remain pressed at all times.
   *
   * <p>If no <a href="#!/api/Ext.button.Segmented-cfg-value" rel="Ext.button.Segmented-cfg-value" class="docClass">value</a> is configured, and no child buttons are configured <code>pressed</code>, the first child
   * button is set <code>pressed: true</code></p>
   * @default false
   * @see #getForceSelection()
   * @see #setForceSelection()
   */
  public native function get forceSelection():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set forceSelection(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The value of this button.  When <a href="#!/api/Ext.button.Segmented-cfg-allowMultiple" rel="Ext.button.Segmented-cfg-allowMultiple" class="docClass">allowMultiple</a> is <code>false</code>, value is a
   * String or Number.  When <a href="#!/api/Ext.button.Segmented-cfg-allowMultiple" rel="Ext.button.Segmented-cfg-allowMultiple" class="docClass">is &lt;code&gt;true&lt;/code&gt;, value is an array
   * of values.  A value corresponds to a child button's {&#64;link Ext.button.Button#value
   * value</a>, or its index if no child button values match the given value.
   *
   * <p>Using the <code>value</code> config of the child buttons with single toggle:</p>
   *
   * <pre class='inline-example '><code>var button = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.button.Segmented" rel="Ext.button.Segmented" class="docClass">Ext.button.Segmented</a>', {
   *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
   *     value: 'optTwo', // begin with "Option Two" selected
   *     items: [{
   *         text: 'Option One',
   *         value: 'optOne'
   *     }, {
   *         text: 'Option Two',
   *         value: 'optTwo'
   *     }, {
   *         text: 'Option Three',
   *         value:  'optThree'
   *     }]
   * });
   *
   * console.log(button.getValue()); // optTwo
   *
   * // Sets the value to optOne, and sets the pressed state of the "Option One" button
   * button.setValue('optOne');
   *
   * console.log(button.getValue()); // optOne
   * </code></pre>
   *
   * <p>Using multiple toggle, and index-based values:</p>
   *
   * <pre class='inline-example '><code>var button = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.button.Segmented" rel="Ext.button.Segmented" class="docClass">Ext.button.Segmented</a>', {
   *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
   *     allowMultiple: true
   *     value: [1, 2], // begin with "Option Two" and "Option Three" selected
   *     items: [{
   *         text: 'Option One'
   *     }, {
   *         text: 'Option Two'
   *     }, {
   *         text: 'Option Three'
   *     }]
   * });
   *
   * // Sets value to [0, 2], and sets pressed state of "Option One" and "Option Three"
   * button.setValue([0, 2]);
   *
   * console.log(button.getValue()); // [0, 2]
   *
   * // Remove all pressed buttons, and set value to null
   * button.setValue(null);
   * </code></pre>
   * @see #getValue()
   * @see #setValue()
   */
  public native function get value():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set value(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * True to align the buttons vertically
   * @default false
   * @see #getVertical()
   * @see #setVertical()
   */
  public native function get vertical():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set vertical(value:Boolean):void;

  /**
   * Returns the value of <a href="#!/api/Ext.button.Segmented-cfg-allowDepress" rel="Ext.button.Segmented-cfg-allowDepress" class="docClass">allowDepress</a>.
   */
  public native function getAllowDepress():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.button.Segmented-cfg-allowMultiple" rel="Ext.button.Segmented-cfg-allowMultiple" class="docClass">allowMultiple</a>.
   */
  public native function getAllowMultiple():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.button.Segmented-cfg-allowToggle" rel="Ext.button.Segmented-cfg-allowToggle" class="docClass">allowToggle</a>.
   */
  public native function getAllowToggle():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.button.Segmented-cfg-defaultUI" rel="Ext.button.Segmented-cfg-defaultUI" class="docClass">defaultUI</a>.
   */
  public native function getDefaultUI():String;

  /**
   * Returns the value of <a href="#!/api/Ext.button.Segmented-cfg-forceSelection" rel="Ext.button.Segmented-cfg-forceSelection" class="docClass">forceSelection</a>.
   */
  public native function getForceSelection():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.button.Segmented-cfg-value" rel="Ext.button.Segmented-cfg-value" class="docClass">value</a>.
   */
  public native function getValue():*;

  /**
   * Returns the value of <a href="#!/api/Ext.button.Segmented-cfg-vertical" rel="Ext.button.Segmented-cfg-vertical" class="docClass">vertical</a>.
   */
  public native function getVertical():Boolean;

  /**
   * Sets the value of <a href="#!/api/Ext.button.Segmented-cfg-allowDepress" rel="Ext.button.Segmented-cfg-allowDepress" class="docClass">allowDepress</a>.
   * @param allowDepress The new value.
   */
  public native function setAllowDepress(allowDepress:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.button.Segmented-cfg-allowMultiple" rel="Ext.button.Segmented-cfg-allowMultiple" class="docClass">allowMultiple</a>.
   * @param allowMultiple The new value.
   */
  public native function setAllowMultiple(allowMultiple:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.button.Segmented-cfg-allowToggle" rel="Ext.button.Segmented-cfg-allowToggle" class="docClass">allowToggle</a>.
   * @param allowToggle The new value.
   */
  public native function setAllowToggle(allowToggle:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.button.Segmented-cfg-defaultUI" rel="Ext.button.Segmented-cfg-defaultUI" class="docClass">defaultUI</a>.
   * @param defaultUI The new value.
   */
  public native function setDefaultUI(defaultUI:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.button.Segmented-cfg-forceSelection" rel="Ext.button.Segmented-cfg-forceSelection" class="docClass">forceSelection</a>.
   * @param forceSelection The new value.
   */
  public native function setForceSelection(forceSelection:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.button.Segmented-cfg-value" rel="Ext.button.Segmented-cfg-value" class="docClass">value</a>.
   * @param value The new value.
   */
  public native function setValue(value:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.button.Segmented-cfg-vertical" rel="Ext.button.Segmented-cfg-vertical" class="docClass">vertical</a>.
   * @param vertical The new value.
   */
  public native function setVertical(vertical:Boolean):void;
}
}