package ext.container {
import ext.panel.Panel;
import ext.util.IFocusableContainer;

[Uses("ext.panel.Panel")]
[Uses("ext.util.FocusableContainer")]
[Uses("ext.Mixin")]
[Uses("ext.Base")]
[Uses("ext.container.DockingContainer")]
[Uses("ext.container.Container")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.mixin.Container")]
[Uses("ext.Component")]
[Uses("ext.util.Positionable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Animate")]
[Uses("ext.util.Floating")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.state.Stateful")]
[Native("Ext.container.ButtonGroup", require)]
/**
 * Provides a container for arranging a group of related Buttons in a tabular manner.
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a>', {
 *     title: 'Panel with ButtonGroup',
 *     width: 300,
 *     height:200,
 *     renderTo: document.body,
 *     bodyPadding: 10,
 *     html: 'HTML Panel Content',
 *     tbar: [{
 *         xtype: 'buttongroup',
 *         columns: 3,
 *         title: 'Clipboard',
 *         items: [{
 *             text: 'Paste',
 *             scale: 'large',
 *             rowspan: 3,
 *             iconCls: 'add',
 *             iconAlign: 'top',
 *             cls: 'btn-as-arrow'
 *         },{
 *             xtype:'splitbutton',
 *             text: 'Menu Button',
 *             scale: 'large',
 *             rowspan: 3,
 *             iconCls: 'add',
 *             iconAlign: 'top',
 *             arrowAlign:'bottom',
 *             menu: [{ text: 'Menu Item 1' }]
 *         },{
 *             xtype:'splitbutton', text: 'Cut', iconCls: 'add16', menu: [{text: 'Cut Menu Item'}]
 *         },{
 *             text: 'Copy', iconCls: 'add16'
 *         },{
 *             text: 'Format', iconCls: 'add16'
 *         }]
 *     }]
 * });
 * </code></pre>
 */
public class ButtonGroup extends ext.panel.Panel implements ext.util.IFocusableContainer {
  /**
   * @param config @inheritDoc
   */
  public function ButtonGroup(config:ext.container.ButtonGroup = null) {
    super();
  }

  [ExtConfig]
  /**
   * The base CSS class to apply to this component's element. This will also be prepended to elements within this
   * component like Panel's body will get a class <code>x-panel-body</code>. This means that if you create a subclass of Panel, and
   * you want it to get all the Panels styling for the element and the body, you leave the <code>baseCls</code> <code>x-panel</code> and use
   * <code>componentCls</code> to add specific styling for this component.
   * @default Ext.baseCSSPrefix + 'btn-group'
   */
  override public native function get baseCls():String;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set baseCls(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The alignment of the title text within the available space between the icon and the tools.
   * @default 'center'
   * @see #getTitleAlign()
   * @see #setTitleAlign()
   */
  override public native function get titleAlign():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  override public native function set titleAlign(value:String):void;

  [ExtConfig]
  /**
   * DOM tabIndex attribute to set on the
   * active Focusable child of this container when using the "Roaming tabindex"
   * technique.
   * @default 0
   */
  public native function get activeChildTabIndex():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set activeChildTabIndex(value:Number):void;

  [ExtConfig]
  /**
   * Set this to <code>true</code> to enable focusing
   * disabled children via keyboard.
   */
  public native function get allowFocusingDisabledChildren():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set allowFocusingDisabledChildren(value:Boolean):void;

  [ExtConfig]
  /**
   * The <code>columns</code> configuration property passed to the <a href="#!/api/Ext.container.ButtonGroup-cfg-layout" rel="Ext.container.ButtonGroup-cfg-layout" class="docClass">configured layout manager</a>.
   * See <a href="#!/api/Ext.layout.container.Table-cfg-columns" rel="Ext.layout.container.Table-cfg-columns" class="docClass">Ext.layout.container.Table.columns</a>.
   */
  public native function get columns():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set columns(value:Number):void;

  [ExtConfig]
  /**
   * A default <a href="#!/api/Ext.Component-cfg-ui" rel="Ext.Component-cfg-ui" class="docClass">ui</a> to use for <a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">Button</a> items
   */
  public native function get defaultButtonUI():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set defaultButtonUI(value:String):void;

  [ExtConfig]
  /**
   * Enable or disable
   * navigation with arrow keys for this FocusableContainer. This option may
   * be useful with nested FocusableContainers such as Grid column headers,
   * when only the root container should handle keyboard events.
   * @default true
   */
  public native function get enableFocusableContainer():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set enableFocusableContainer(value:Boolean):void;

  [ExtConfig]
  /**
   * DOM tabIndex attribute to set on
   * inactive Focusable children of this container when using the "Roaming tabindex"
   * technique. This value rarely needs to be changed from its default.
   * @default -1
   */
  public native function get inactiveChildTabIndex():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set inactiveChildTabIndex(value:Number):void;

  /**
   * @inheritDoc
   */
  public native function get focusableContainerEl():*;

  /**
   * @inheritDoc
   */
  public native function set focusableContainerEl(value:*):void;
}
}