package ext.dashboard {
import ext.panel.Panel;

[Uses("ext.panel.Panel")]
[Uses("ext.container.DockingContainer")]
[Uses("ext.Base")]
[Uses("ext.container.Container")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.mixin.Container")]
[Uses("ext.Mixin")]
[Uses("ext.Component")]
[Uses("ext.util.Positionable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Animate")]
[Uses("ext.util.Floating")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.state.Stateful")]
[Native("Ext.dashboard.Dashboard", require)]
/**
 * @eventType ext.dashboard.events.DashboardEvent.BEFORE_DRAG_OVER
 */
[Event(name = 'onBeforeDragOver', type = 'ext.dashboard.events.DashboardEvent')]
/**
 * @eventType ext.dashboard.events.DashboardEvent.BEFORE_DROP
 */
[Event(name = 'onBeforeDrop', type = 'ext.dashboard.events.DashboardEvent')]
/**
 * @eventType ext.dashboard.events.DashboardEvent.DRAG_OVER
 */
[Event(name = 'onDragOver', type = 'ext.dashboard.events.DashboardEvent')]
/**
 * @eventType ext.dashboard.events.DashboardEvent.DROP
 */
[Event(name = 'onDrop', type = 'ext.dashboard.events.DashboardEvent')]
/**
 * @eventType ext.dashboard.events.DashboardEvent.VALIDATE_DROP
 */
[Event(name = 'onValidateDrop', type = 'ext.dashboard.events.DashboardEvent')]
/**
 * This class manages a drag-drop Dashboard similar to the legacy Ext JS Portal example.
 * The user-directed layout of the Dashboard is preserved the Ext JS <code>stateful</code> mechanism
 * to preserve potentially dynamic user sizing and collapsed states as well as order of
 * items in their columns.
 */
public class Dashboard extends ext.panel.Panel {
  /**
   * @param config @inheritDoc
   */
  public function Dashboard(config:ext.dashboard.Dashboard = null) {
    super();
  }

  [ExtConfig]
  /**
   * An array designating the width of columns in your dashboard's default state as described
   * by the <a href="#!/api/Ext.dashboard.Dashboard-cfg-defaultContent" rel="Ext.dashboard.Dashboard-cfg-defaultContent" class="docClass">defaultContent</a> property. For example:
   *
   * <p>   columnWidths: [
   *       0.35,
   *       0.40,
   *       0.25
   *    ]</p>
   *
   * <p>As you can see, this array contains the default widths for the 3 columns in the dashboard's
   * initial view. The column widths should total to an integer value, typically 1 as shown
   * above. When column widths exceed 1, they will be wrapped effectively creating "rows". This
   * means that if your column widths add up to more than 1, you would still want the first few
   * to total 1 to ensure that the first row fills the dashboard space. This applies whenever
   * the column widths extend past an integer value.</p>
   *
   * <p><strong>Note:</strong> columnWidths will not be utilized if there is stateful information that
   * dictates different user saved column widths.</p>
   */
  public native function get columnWidths():Array;

  [ExtConfig]
  /**
   * @private
   */
  public native function set columnWidths(value:Array):void;

  [ExtConfig]
  /**
   * An array of <a href="#!/api/Ext.dashboard.Part" rel="Ext.dashboard.Part" class="docClass">part</a> configuration objects that define your dashboard's
   * default state. These should not be confused with component configurations.
   *
   * <p>Each config object should also include:</p>
   *
   * <ul>
   * <li><code>type</code> - The type of <a href="#!/api/Ext.dashboard.Part" rel="Ext.dashboard.Part" class="docClass">part</a> that you want to be generated.</li>
   * <li><code>columnIndex</code> - The column position in which the <a href="#!/api/Ext.dashboard.Part" rel="Ext.dashboard.Part" class="docClass">part</a> should reside.</li>
   * <li><code>height</code> - The desired height of the <a href="#!/api/Ext.dashboard.Part" rel="Ext.dashboard.Part" class="docClass">part</a> to be generated.</li>
   * </ul>
   *
   *
   * <p>The remaining properties are specific to your part's config object. For example:</p>
   *
   * <pre><code>  defaultContent: [{
   *       type: 'rss',
   *       columnIndex: 0,
   *       height: 500,
   *       feedUrl: 'http://feeds.feedburner.com/extblog'
   *   }, {
   *       type: 'stockTicker',
   *       columnIndex: 1,
   *       height: 300
   *   }, {
   *       type: 'stocks',
   *       columnIndex: 1,
   *       height: 300
   *   }, {
   *       type: 'rss',
   *       columnIndex: 2,
   *       height: 350,
   *       feedUrl: 'http://rss.cnn.com/rss/edition.rss'
   *   }]
   * </code></pre>
   *
   * <p>Default column widths are defined by <a href="#!/api/Ext.dashboard.Dashboard-cfg-columnWidths" rel="Ext.dashboard.Dashboard-cfg-columnWidths" class="docClass">columnWidths</a> and not in these
   * part config objects.</p>
   *
   * <p><strong>Note:</strong> defaultContent will not be utilized if there is stateful information that
   * dictates different user saved positioning and componentry.</p>
   */
  public native function get defaultContent():Array;

  [ExtConfig]
  /**
   * @private
   */
  public native function set defaultContent(value:Array):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The maximum number of visible columns.
   * @default 4
   * @see #getMaxColumns()
   * @see #setMaxColumns()
   */
  public native function get maxColumns():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set maxColumns(value:Number):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * An object keyed by <code>id</code> for the parts that can be created for this <code>Dashboard</code>.
   * @see #getParts()
   * @see #setParts()
   */
  public native function get parts():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set parts(value:Object):void;

  /**
   * Returns the value of <a href="#!/api/Ext.dashboard.Dashboard-cfg-maxColumns" rel="Ext.dashboard.Dashboard-cfg-maxColumns" class="docClass">maxColumns</a>.
   */
  public native function getMaxColumns():Number;

  /**
   * Returns the value of <a href="#!/api/Ext.dashboard.Dashboard-cfg-parts" rel="Ext.dashboard.Dashboard-cfg-parts" class="docClass">parts</a>.
   */
  public native function getParts():Object;

  /**
   * Sets the value of <a href="#!/api/Ext.dashboard.Dashboard-cfg-maxColumns" rel="Ext.dashboard.Dashboard-cfg-maxColumns" class="docClass">maxColumns</a>.
   * @param maxColumns The new value.
   */
  public native function setMaxColumns(maxColumns:Number):void;

  /**
   * Sets the value of <a href="#!/api/Ext.dashboard.Dashboard-cfg-parts" rel="Ext.dashboard.Dashboard-cfg-parts" class="docClass">parts</a>.
   * @param parts The new value.
   */
  public native function setParts(parts:Object):void;
}
}