package ext.data {
import ext.mixin.IFactoryable;
import ext.mixin.Observable;
import ext.util.Collection;

[Uses("ext.mixin.Observable")]
[Uses("ext.mixin.Factoryable")]
[Uses("ext.Base")]
[Native("Ext.data.AbstractStore", require)]
/**
 * Fired when a Model instance has been added to this Store.
 * @since 1.1.0
 * @eventType ext.data.events.AbstractStore_store_records_indexEvent.ADD
 */
[Event(name = 'onAdd', type = 'ext.data.events.AbstractStore_store_records_indexEvent')]
/**
 * Fires before a store is sorted.
 *
 * <p>For <a href="#!/api/Ext.data.AbstractStore-cfg-remoteSort" rel="Ext.data.AbstractStore-cfg-remoteSort" class="docClass">remotely sorted</a> stores, this will be just before the load operation triggered by changing the
 * store's sorters.</p>
 *
 * <p>For locally sorted stores, this will be just before the data items in the store's backing collection are sorted.</p>
 * @eventType ext.data.events.AbstractStore_store_sortersEvent.BEFORE_SORT
 */
[Event(name = 'onBeforeSort', type = 'ext.data.events.AbstractStore_store_sortersEvent')]
/**
 * Fires when the <a href="#!/api/Ext.data.AbstractStore-method-beginUpdate" rel="Ext.data.AbstractStore-method-beginUpdate" class="docClass">beginUpdate</a> method is called. Automatic synchronization as configured
 * by the <a href="#!/api/Ext.data.ProxyStore-cfg-autoSync" rel="Ext.data.ProxyStore-cfg-autoSync" class="docClass">autoSync</a> flag is deferred until the <a href="#!/api/Ext.data.AbstractStore-method-endUpdate" rel="Ext.data.AbstractStore-method-endUpdate" class="docClass">endUpdate</a> method is called, so multiple
 * mutations can be coalesced into one synchronization operation.
 * @eventType ext.data.events.AbstractStoreEvent.BEGIN_UPDATE
 */
[Event(name = 'onBeginUpdate', type = 'ext.data.events.AbstractStoreEvent')]
/**
 * Fired after the <a href="#!/api/Ext.data.Store-method-removeAll" rel="Ext.data.Store-method-removeAll" class="docClass">removeAll</a> method is called.
 * @since 1.1.0
 * @eventType ext.data.events.AbstractStoreEvent.CLEAR
 */
[Event(name = 'onClear', type = 'ext.data.events.AbstractStoreEvent')]
/**
 * Fires whenever records are added to or removed from the Store.
 *
 * <p>To hook into modifications of records in this Store use the <a href="#!/api/Ext.data.AbstractStore-event-update" rel="Ext.data.AbstractStore-event-update" class="docClass">update</a> event.</p>
 * @since 1.1.0
 * @eventType ext.data.events.AbstractStoreEvent.DATA_CHANGED
 */
[Event(name = 'onDataChanged', type = 'ext.data.events.AbstractStoreEvent')]
/**
 * Fires when the <a href="#!/api/Ext.data.AbstractStore-method-endUpdate" rel="Ext.data.AbstractStore-method-endUpdate" class="docClass">endUpdate</a> method is called. Automatic synchronization as configured
 * by the <a href="#!/api/Ext.data.ProxyStore-cfg-autoSync" rel="Ext.data.ProxyStore-cfg-autoSync" class="docClass">autoSync</a> flag is deferred until the <a href="#!/api/Ext.data.AbstractStore-method-endUpdate" rel="Ext.data.AbstractStore-method-endUpdate" class="docClass">endUpdate</a> method is called, so multiple
 * mutations can be coalesced into one synchronization operation.
 * @eventType ext.data.events.AbstractStoreEvent.END_UPDATE
 */
[Event(name = 'onEndUpdate', type = 'ext.data.events.AbstractStoreEvent')]
/**
 * Fires when the data cache has changed in a bulk manner (e.g., it has been sorted, filtered, etc.) and a
 * widget that is using this Store as a Record cache should refresh its view.
 * @eventType ext.data.events.AbstractStoreEvent.REFRESH
 */
[Event(name = 'onRefresh', type = 'ext.data.events.AbstractStoreEvent')]
/**
 * Fired when one or more records have been removed from this Store.
 *
 * <p><strong>The signature for this event has changed in 5.0: </strong></p>
 * @since 5.0.0
 * @eventType ext.data.events.AbstractStore_store_records_index_isMoveEvent.REMOVE
 */
[Event(name = 'onRemove', type = 'ext.data.events.AbstractStore_store_records_index_isMoveEvent')]
/**
 * Fires after a store is sorted.
 *
 * <p>For <a href="#!/api/Ext.data.AbstractStore-cfg-remoteSort" rel="Ext.data.AbstractStore-cfg-remoteSort" class="docClass">remotely sorted</a> stores, this will be upon the success of a load operation triggered by
 * changing the store's sorters.</p>
 *
 * <p>For locally sorted stores, this will be just after the data items in the store's backing collection are sorted.</p>
 * @eventType ext.data.events.AbstractStore_storeEvent.SORT
 */
[Event(name = 'onSort', type = 'ext.data.events.AbstractStore_storeEvent')]
/**
 * Fires when a Model instance has been updated.
 * @since 1.1.0
 * @eventType ext.data.events.AbstractStore_record_operation_modifiedFieldNames_detailsEvent.UPDATE
 */
[Event(name = 'onUpdate', type = 'ext.data.events.AbstractStore_record_operation_modifiedFieldNames_detailsEvent')]
/**
 * AbstractStore is a superclass of <a href="#!/api/Ext.data.ProxyStore" rel="Ext.data.ProxyStore" class="docClass">Ext.data.ProxyStore</a> and <a href="#!/api/Ext.data.ChainedStore" rel="Ext.data.ChainedStore" class="docClass">Ext.data.ChainedStore</a>. It's never used directly,
 * but offers a set of methods used by both of those subclasses.
 *
 * <p>We've left it here in the docs for reference purposes, but unless you need to make a whole new type of Store, what
 * you're probably looking for is <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>.</p>
 */
public class AbstractStore extends ext.mixin.Observable implements ext.mixin.IFactoryable {
  /**
   * @param config @inheritDoc
   */
  public function AbstractStore(config:ext.data.AbstractStore = null) {
    super();
  }

  /**
   * The page that the Store has most recently loaded (see <a href="#!/api/Ext.data.Store-method-loadPage" rel="Ext.data.Store-method-loadPage" class="docClass">loadPage</a>)
   * @default 1
   */
  public native function get currentPage():Number;

  /**
   * @private
   */
  public native function set currentPage(value:Number):void;

  /**
   * <code>true</code> in this class to identify an object as an instantiated Store, or subclass thereof.
   * @default true
   */
  public native function get isStore():Boolean;

  /**
   * @private
   */
  public native function set isStore(value:Boolean):void;

  /**
   * A counter that is increased by <code>beginUpdate</code> and decreased by <code>endUpdate</code>. When
   * this transitions from 0 to 1 the <code><a href="#!/api/Ext.data.AbstractStore-event-beginupdate" rel="Ext.data.AbstractStore-event-beginupdate" class="docClass">beginupdate</a></code> event is
   * fired. When it transitions back from 1 to 0 the <code><a href="#!/api/Ext.data.AbstractStore-event-endupdate" rel="Ext.data.AbstractStore-event-endupdate" class="docClass">endupdate</a></code>
   * event is fired.
   * @default 0
   * @since 5.0.0
   */
  public native function get updating():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * When a Store is used by only one <a href="#!/api/Ext.view.View" rel="Ext.view.View" class="docClass">DataView</a>, and should only exist for the lifetime of that view, then
   * configure the autoDestroy flag as <code>true</code>. This causes the destruction of the view to trigger the destruction of its Store.
   * @see #getAutoDestroy()
   * @see #setAutoDestroy()
   */
  public native function get autoDestroy():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set autoDestroy(value:Boolean):void;

  [ExtConfig]
  /**
   * Array of <a href="#!/api/Ext.util.Filter" rel="Ext.util.Filter" class="docClass">Filters</a> for this store. Can also be passed array of
   * functions which will be used as the <a href="#!/api/Ext.util.Filter-cfg-filterFn" rel="Ext.util.Filter-cfg-filterFn" class="docClass">filterFn</a> config
   * for filters:
   *
   * <pre><code>filters: [
   *     function(item) {
   *         return item.weight &gt; 0;
   *     }
   * ]
   * </code></pre>
   *
   * <p>To filter after the grid is loaded use the <a href="#!/api/Ext.data.Store-method-filterBy" rel="Ext.data.Store-method-filterBy" class="docClass">filterBy</a> function.</p>
   * @see #setFilters()
   */
  public native function get filters():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set filters(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The direction in which sorting should be applied when grouping. Supported values are "ASC" and "DESC".
   * @default 'ASC'
   * @see #getGroupDir()
   * @see #setGroupDir()
   */
  public native function get groupDir():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set groupDir(value:String):void;

  [ExtConfig]
  /**
   * The field by which to group data in the store. Internally, grouping is very similar to sorting - the
   * groupField and <a href="#!/api/Ext.data.AbstractStore-cfg-groupDir" rel="Ext.data.AbstractStore-cfg-groupDir" class="docClass">groupDir</a> are injected as the first sorter (see <a href="#!/api/Ext.data.AbstractStore-method-sort" rel="Ext.data.AbstractStore-method-sort" class="docClass">sort</a>). Stores support a single
   * level of grouping, and groups can be fetched via the <a href="#!/api/Ext.data.AbstractStore-method-getGroups" rel="Ext.data.AbstractStore-method-getGroups" class="docClass">getGroups</a> method.
   * @see #setGroupField()
   */
  public native function get groupField():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set groupField(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The grouper by which to group the data store. May also be specified by the <a href="#!/api/Ext.data.AbstractStore-cfg-groupField" rel="Ext.data.AbstractStore-cfg-groupField" class="docClass">groupField</a> config, however
   * they should not be used together.
   * @see #getGrouper()
   * @see #setGrouper()
   */
  public native function get grouper():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set grouper(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The number of records considered to form a 'page'. This is used to power the built-in
   * paging using the nextPage and previousPage functions when the grid is paged using a
   * <a href="#!/api/Ext.toolbar.Paging" rel="Ext.toolbar.Paging" class="docClass">PagingToolbar</a> Defaults to 25.
   *
   * <p>To disable paging, set the pageSize to <code>0</code>.</p>
   * @default 25
   * @see #getPageSize()
   * @see #setPageSize()
   */
  public native function get pageSize():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set pageSize(value:Number):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * <code>true</code> to defer any filtering operation to the server. If <code>false</code>, filtering is done locally on the client.
   * @default false
   * @see #getRemoteFilter()
   * @see #setRemoteFilter()
   */
  public native function get remoteFilter():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set remoteFilter(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * <code>true</code> if the sorting should be performed on the server side, false if it is local only.
   * @default false
   * @see #getRemoteSort()
   * @see #setRemoteSort()
   */
  public native function get remoteSort():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set remoteSort(value:Boolean):void;

  [ExtConfig]
  /**
   * The initial set of <a href="#!/api/Ext.util.Sorter" rel="Ext.util.Sorter" class="docClass">Sorters</a>
   * @see #setSorters()
   */
  public native function get sorters():Array;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  [ArrayElementType("ext.util.Sorter")]
  /**
   * @private
   */
  public native function set sorters(value:Array):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Configure as <code>true</code> to have the filters saved when a client <a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">grid</a> saves its state.
   * @default false
   * @see #getStatefulFilters()
   * @see #setStatefulFilters()
   */
  public native function get statefulFilters():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set statefulFilters(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Unique identifier for this store. If present, this Store will be registered with the <a href="#!/api/Ext.data.StoreManager" rel="Ext.data.StoreManager" class="docClass">Ext.data.StoreManager</a>,
   * making it easy to reuse elsewhere.
   *
   * <p>Note that when a store is instantiated by a Controller, the storeId will default
   * to the name of the store if not specified in the class.</p>
   * @see #getStoreId()
   * @see #setStoreId()
   */
  public native function get storeId():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set storeId(value:String):void;

  /**
   * @inheritDoc
   */
  public native function get factoryConfig():Object;

  /**
   * @inheritDoc
   */
  public native function set factoryConfig(value:Object):void;

  /**
   * Adds a new Filter to this Store's <a href="#!/api/Ext.data.AbstractStore-cfg-filters" rel="Ext.data.AbstractStore-cfg-filters" class="docClass">filter set</a> and
   * by default, applies the updated filter set to the Store's unfiltered dataset.
   * @param filters The set of filters to add to the current <a href="#!/api/Ext.data.AbstractStore-cfg-filters" rel="Ext.data.AbstractStore-cfg-filters" class="docClass">filter set</a>.
   * @param suppressEvent If <code>true</code> the filter is cleared silently.
   */
  public native function addFilter(filters:*, suppressEvent:Boolean = false):void;

  /**
   * This method may be called to indicate the start of multiple changes to the store.
   *
   * <p>Automatic synchronization as configured by the <a href="#!/api/Ext.data.ProxyStore-cfg-autoSync" rel="Ext.data.ProxyStore-cfg-autoSync" class="docClass">autoSync</a> flag is deferred
   * until the <a href="#!/api/Ext.data.AbstractStore-method-endUpdate" rel="Ext.data.AbstractStore-method-endUpdate" class="docClass">endUpdate</a> method is called, so multiple mutations can be coalesced
   * into one synchronization operation.</p>
   *
   * <p>Internally this method increments a counter that is decremented by <code>endUpdate</code>. It
   * is important, therefore, that if you call <code>beginUpdate</code> directly you match that
   * call with a call to <code>endUpdate</code> or you will prevent the collection from updating
   * properly.</p>
   *
   * <p>For example:</p>
   *
   * <pre><code> var store = <a href="#!/api/Ext.data.StoreManager-method-lookup" rel="Ext.data.StoreManager-method-lookup" class="docClass">Ext.StoreManager.lookup</a>({
   *      //...
   *      autoSync: true
   *  });
   *
   *  store.beginUpdate();
   *
   *  record.set('fieldName', 'newValue');
   *
   *  store.add(item);
   *  // ...
   *
   *  store.insert(index, otherItem);
   *  //...
   *
   *  // Interested parties will listen for the endupdate event
   *  store.endUpdate();
   * </code></pre>
   * @since 5.0.0
   */
  public native function beginUpdate():void;

  /**
   * Reverts to a view of the Record cache with no filtering applied.
   * @param suppressEvent If <code>true</code> the filter is cleared silently.
   * <p>For a locally filtered Store, this means that the filter collection is cleared without firing the
   * <a href="#!/api/Ext.data.AbstractStore-event-datachanged" rel="Ext.data.AbstractStore-event-datachanged" class="docClass">datachanged</a> event.</p>
   * <p>For a remotely filtered Store, this means that the filter collection is cleared, but the store
   * is not reloaded from the server.</p>
   */
  public native function clearFilter(suppressEvent:Boolean = false):void;

  /**
   * Clear the store grouping
   */
  public native function clearGrouping():void;

  /**
   * Checks if a record is in the current active data set.
   * @param record The record
   * @return <code>true</code> if the record is in the current active data set.
   */
  public native function contains(record:ext.data.Model):Boolean;

  /**
   * Perform the Store destroying sequence. Override this method to add destruction
   * behaviors to your custom Stores.
   */
  protected native function doDestroy():void;

  /**
   * This method is called after modifications are complete on a store. For details
   * see <code><a href="#!/api/Ext.data.AbstractStore-method-beginUpdate" rel="Ext.data.AbstractStore-method-beginUpdate" class="docClass">beginUpdate</a></code>.
   * @since 5.0.0
   */
  public native function endUpdate():void;

  /**
   * Filters the data in the Store by one or more fields. Example usage:
   *
   * <pre><code>//filter with a single field
   * myStore.filter('firstName', 'Don');
   *
   * //filtering with multiple filters
   * myStore.filter([
   *     {
   *         property : 'firstName',
   *         value    : 'Don'
   *     },
   *     {
   *         property : 'lastName',
   *         value    : 'Griffin'
   *     }
   * ]);
   * </code></pre>
   *
   * <p>Internally, Store converts the passed arguments into an array of <a href="#!/api/Ext.util.Filter" rel="Ext.util.Filter" class="docClass">Ext.util.Filter</a> instances, and delegates
   * the actual filtering to its internal <a href="#!/api/Ext.util.MixedCollection" rel="Ext.util.MixedCollection" class="docClass">Ext.util.MixedCollection</a>.</p>
   * @param filters Either a string name of one of the fields in this Store's configured
   * <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Model</a>, or an array of filter configurations.
   * @param value The property value by which to filter. Only applicable if <code>filters</code> is a string.
   */
  public native function filter(filters:* = undefined, value:String = null):void;

  /**
   * Filters by a function. The specified function will be called for each
   * Record in this Store. If the function returns <code>true</code> the Record is included,
   * otherwise it is filtered out.
   *
   * <p>When store is filtered, most of the methods for accessing store data will be working only
   * within the set of filtered records. The notable exception is <a href="#!/api/Ext.data.AbstractStore-method-getById" rel="Ext.data.AbstractStore-method-getById" class="docClass">getById</a>.</p>
   * @param fn The function to be called. It will be passed the following parameters:
   * @param {ext.data.Model} fn.record The record to test for filtering. Access field values
   *  using <a href="#!/api/Ext.data.Model-method-get" rel="Ext.data.Model-method-get" class="docClass">Ext.data.Model.get</a>.
   * @param scope The scope (this reference) in which the function is executed.
   * Defaults to this Store.
   */
  public native function filterBy(fn:Function, scope:Object = null):void;

  /**
   * Finds the index of the first matching Record in this store by a specific field value.
   *
   * <p>When store is filtered, finds records only within filter.</p>
   *
   * <p>IMPORTANT</p>
   *
   * <p>If this store is <a href="#!/api/Ext.data.BufferedStore" rel="Ext.data.BufferedStore" class="docClass">Buffered</a>, this can ONLY find records which happen to be cached in the page cache.
   * This will be parts of the dataset around the currently visible zone, or recently visited zones if the pages
   * have not yet been purged from the cache.</p>
   * @param property The name of the Record field to test.
   * @param value Either a string that the field value
   * should begin with, or a RegExp to test against the field.
   * @param startIndex The index to start searching at
   * @default 0
   * @param anyMatch True to match any part of the string, not just the
   * beginning.
   * @default false
   * @param caseSensitive True for case sensitive comparison
   * @default false
   * @param exactMatch True to force exact match (^ and $ characters
   * added to the regex). Ignored if <code>anyMatch</code> is <code>true</code>.
   * @default false
   * @return The matched index or -1
   */
  public native function find(property:String, value:*, startIndex:Number = 0, anyMatch:Boolean = false, caseSensitive:Boolean = false, exactMatch:Boolean = false):Number;

  /**
   * Find the index of the first matching Record in this Store by a function.
   * If the function returns <code>true</code> it is considered a match.
   *
   * <p>When store is filtered, finds records only within filter.</p>
   *
   * <p>IMPORTANT</p>
   *
   * <p>If this store is <a href="#!/api/Ext.data.BufferedStore" rel="Ext.data.BufferedStore" class="docClass">Buffered</a>, this can ONLY find records which happen to be cached in the page cache.
   * This will be parts of the dataset around the currently visible zone, or recently visited zones if the pages
   * have not yet been purged from the cache.</p>
   * @param fn The function to be called. It will be passed the following parameters:
   * @param {ext.data.Model} fn.record The record to test for filtering. Access field values
   *  using <a href="#!/api/Ext.data.Model-method-get" rel="Ext.data.Model-method-get" class="docClass">Ext.data.Model.get</a>.
   * @param {Object} fn.id The ID of the Record passed.
   * @param scope The scope (this reference) in which the function is executed.
   * Defaults to this Store.
   * @param startIndex The index to start searching at
   * @default 0
   * @return The matched index or -1
   */
  public native function findBy(fn:Function, scope:Object = null, startIndex:Number = 0):Number;

  /**
   * Finds the index of the first matching Record in this store by a specific field value.
   *
   * <p>When store is filtered, finds records only within filter.</p>
   *
   * <p>IMPORTANT</p>
   *
   * <p>If this store is <a href="#!/api/Ext.data.BufferedStore" rel="Ext.data.BufferedStore" class="docClass">Buffered</a>, this can ONLY find records which happen to be cached in the page cache.
   * This will be parts of the dataset around the currently visible zone, or recently visited zones if the pages
   * have not yet been purged from the cache.</p>
   * @param fieldName The name of the Record field to test.
   * @param value The value to match the field against.
   * @param startIndex The index to start searching at
   * @default 0
   * @return The matched index or -1
   */
  public native function findExact(fieldName:String, value:*, startIndex:Number = 0):Number;

  /**
   * Finds the first matching Record in this store by a specific field value.
   *
   * <p>When store is filtered, finds records only within filter.</p>
   *
   * <p>IMPORTANT</p>
   *
   * <p>If this store is <a href="#!/api/Ext.data.BufferedStore" rel="Ext.data.BufferedStore" class="docClass">Buffered</a>, this can ONLY find records which happen to be cached in the page cache.
   * This will be parts of the dataset around the currently visible zone, or recently visited zones if the pages
   * have not yet been purged from the cache.</p>
   * @param fieldName The name of the Record field to test.
   * @param value Either a string that the field value
   * should begin with, or a RegExp to test against the field.
   * @param startIndex The index to start searching at
   * @default 0
   * @param anyMatch True to match any part of the string, not just the
   * beginning.
   * @default false
   * @param caseSensitive True for case sensitive comparison
   * @default false
   * @param exactMatch True to force exact match (^ and $ characters
   * added to the regex). Ignored if <code>anyMatch</code> is <code>true</code>.
   * @default false
   * @return The matched record or null
   */
  public native function findRecord(fieldName:String, value:*, startIndex:Number = 0, anyMatch:Boolean = false, caseSensitive:Boolean = false, exactMatch:Boolean = false):ext.data.Model;

  /**
   * Get the Record at the specified index.
   *
   * <p>The index is effected by filtering.</p>
   * @param index The index of the Record to find.
   * @return The Record at the passed index. Returns null if not found.
   */
  public native function getAt(index:Number):ext.data.Model;

  /**
   * Returns the value of <a href="#!/api/Ext.data.AbstractStore-cfg-autoDestroy" rel="Ext.data.AbstractStore-cfg-autoDestroy" class="docClass">autoDestroy</a>.
   */
  public native function getAutoDestroy():Boolean;

  /**
   * Get the Record with the specified id.
   *
   * <p>This method is not affected by filtering, lookup will be performed from all records
   * inside the store, filtered or not.</p>
   * @param id The id of the Record to find.
   * @return The Record with the passed id. Returns null if not found.
   */
  public native function getById(id:String):ext.data.Model;

  /**
   * Gets the number of records in store.
   *
   * <p>If using paging, this may not be the total size of the dataset. If the data object
   * used by the Reader contains the dataset size, then the <a href="#!/api/Ext.data.ProxyStore-method-getTotalCount" rel="Ext.data.ProxyStore-method-getTotalCount" class="docClass">Ext.data.ProxyStore.getTotalCount</a> function returns
   * the dataset size.  <strong>Note</strong>: see the Important note in <a href="#!/api/Ext.data.ProxyStore-method-load" rel="Ext.data.ProxyStore-method-load" class="docClass">Ext.data.ProxyStore.load</a>.</p>
   *
   * <p>When store is filtered, it's the number of records matching the filter.</p>
   * @return The number of Records in the Store.
   */
  public native function getCount():Number;

  /**
   * Gets the filters for this store.
   * @return The filters
   */
  public native function getFilters(autoCreate:Boolean = false):ext.util.Collection;

  /**
   * Returns the value of <a href="#!/api/Ext.data.AbstractStore-cfg-groupDir" rel="Ext.data.AbstractStore-cfg-groupDir" class="docClass">groupDir</a>.
   */
  public native function getGroupDir():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.AbstractStore-cfg-grouper" rel="Ext.data.AbstractStore-cfg-grouper" class="docClass">grouper</a>.
   */
  public native function getGrouper():*;

  /**
   * Returns a collection of readonly sub-collections of your store's records
   * with grouping applied. These sub-collections are maintained internally by
   * the collection.
   *
   * <p>See <a href="#!/api/Ext.data.AbstractStore-cfg-groupField" rel="Ext.data.AbstractStore-cfg-groupField" class="docClass">groupField</a>, <a href="#!/api/Ext.data.AbstractStore-cfg-groupDir" rel="Ext.data.AbstractStore-cfg-groupDir" class="docClass">groupDir</a>. Example for a store
   * containing records with a color field:</p>
   *
   * <pre><code>var myStore = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
   *     groupField: 'color',
   *     groupDir  : 'DESC'
   * });
   *
   * myStore.getGroups();
   * </code></pre>
   *
   * <p>The above should result in the following format:</p>
   *
   * <pre><code>[
   *     {
   *         name: 'yellow',
   *         children: [
   *             // all records where the color field is 'yellow'
   *         ]
   *     },
   *     {
   *         name: 'red',
   *         children: [
   *             // all records where the color field is 'red'
   *         ]
   *     }
   * ]
   * </code></pre>
   *
   * <p>Group contents are affected by filtering.</p>
   * @return The grouped data
   */
  public native function getGroups():ext.util.Collection;

  /**
   * Returns the value of <a href="#!/api/Ext.data.AbstractStore-cfg-pageSize" rel="Ext.data.AbstractStore-cfg-pageSize" class="docClass">pageSize</a>.
   */
  public native function getPageSize():Number;

  [ArrayElementType("ext.data.Model")]
  /**
   * Gathers a range of Records between specified indices.
   *
   * <p>This method is affected by filtering.</p>
   * @param start The starting index. Defaults to zero.
   * @param end The ending index. Defaults to the last record. The end index <strong>is included</strong>.
   * @return An array of records.
   */
  public native function getRange(start:Number = 0, end:Number = NaN):Array;

  /**
   * Returns the value of <a href="#!/api/Ext.data.AbstractStore-cfg-remoteFilter" rel="Ext.data.AbstractStore-cfg-remoteFilter" class="docClass">remoteFilter</a>.
   */
  public native function getRemoteFilter():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.data.AbstractStore-cfg-remoteSort" rel="Ext.data.AbstractStore-cfg-remoteSort" class="docClass">remoteSort</a>.
   */
  public native function getRemoteSort():Boolean;

  /**
   * Gets the sorters for this store.
   * @return The sorters
   */
  public native function getSorters(autoCreate:Boolean = false):ext.util.Collection;

  /**
   * Returns the value of <a href="#!/api/Ext.data.AbstractStore-cfg-statefulFilters" rel="Ext.data.AbstractStore-cfg-statefulFilters" class="docClass">statefulFilters</a>.
   */
  public native function getStatefulFilters():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.data.AbstractStore-cfg-storeId" rel="Ext.data.AbstractStore-cfg-storeId" class="docClass">storeId</a>.
   */
  public native function getStoreId():String;

  /**
   * Groups data inside the store.
   * @param grouper Either a string name of one of the fields in this Store's
   * configured <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Model</a>, or an object, or a <a href="#!/api/Ext.util.Grouper" rel="Ext.util.Grouper" class="docClass">grouper</a> configuration object.
   * @param direction The overall direction to group the data by. Defaults to the value of <a href="#!/api/Ext.data.AbstractStore-cfg-groupDir" rel="Ext.data.AbstractStore-cfg-groupDir" class="docClass">groupDir</a>.
   */
  public native function group(grouper:*, direction:String = null):void;

  /**
   * Tests whether the store currently has any active filters.
   * @return <code>true</code> if the store is filtered.
   */
  public native function isFiltered():Boolean;

  /**
   * Tests whether the store currently has an active grouper.
   * @return <code>true</code> if the store is grouped.
   */
  public native function isGrouped():Boolean;

  /**
   * Returns <code>true</code> if the Store has been loaded.
   * @return <code>true</code> if the Store has been loaded.
   */
  public native function isLoaded():Boolean;

  /**
   * Returns <code>true</code> if the Store is currently performing a load operation.
   * @return <code>true</code> if the Store is currently loading.
   */
  public native function isLoading():Boolean;

  /**
   * Tests whether the store currently has any active sorters.
   * @return <code>true</code> if the store is sorted.
   */
  public native function isSorted():Boolean;

  /**
   * Removes an individual Filter from the current <a href="#!/api/Ext.data.AbstractStore-cfg-filters" rel="Ext.data.AbstractStore-cfg-filters" class="docClass">filter set</a> using the passed Filter/Filter id and
   * by default, applies the updated filter set to the Store's unfiltered dataset.
   * @param toRemove The id of a Filter to remove from the filter set, or a Filter instance to remove.
   * @param suppressEvent If <code>true</code> the filter is cleared silently.
   */
  public native function removeFilter(toRemove:*, suppressEvent:Boolean = false):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.AbstractStore-cfg-autoDestroy" rel="Ext.data.AbstractStore-cfg-autoDestroy" class="docClass">autoDestroy</a>.
   * @param autoDestroy The new value.
   */
  public native function setAutoDestroy(autoDestroy:Boolean):void;

  public native function setFilters():void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.AbstractStore-cfg-groupDir" rel="Ext.data.AbstractStore-cfg-groupDir" class="docClass">groupDir</a>.
   * @param groupDir The new value.
   */
  public native function setGroupDir(groupDir:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.AbstractStore-cfg-groupField" rel="Ext.data.AbstractStore-cfg-groupField" class="docClass">groupField</a>.
   * @param groupField The new value.
   */
  public native function setGroupField(groupField:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.AbstractStore-cfg-grouper" rel="Ext.data.AbstractStore-cfg-grouper" class="docClass">grouper</a>.
   * @param grouper The new value.
   */
  public native function setGrouper(grouper:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.AbstractStore-cfg-pageSize" rel="Ext.data.AbstractStore-cfg-pageSize" class="docClass">pageSize</a>.
   * @param pageSize The new value.
   */
  public native function setPageSize(pageSize:Number):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.AbstractStore-cfg-remoteFilter" rel="Ext.data.AbstractStore-cfg-remoteFilter" class="docClass">remoteFilter</a>.
   * @param remoteFilter The new value.
   */
  public native function setRemoteFilter(remoteFilter:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.AbstractStore-cfg-remoteSort" rel="Ext.data.AbstractStore-cfg-remoteSort" class="docClass">remoteSort</a>.
   * @param remoteSort The new value.
   */
  public native function setRemoteSort(remoteSort:Boolean):void;

  public native function setSorters():void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.AbstractStore-cfg-statefulFilters" rel="Ext.data.AbstractStore-cfg-statefulFilters" class="docClass">statefulFilters</a>.
   * @param statefulFilters The new value.
   */
  public native function setStatefulFilters(statefulFilters:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.AbstractStore-cfg-storeId" rel="Ext.data.AbstractStore-cfg-storeId" class="docClass">storeId</a>.
   * @param storeId The new value.
   */
  public native function setStoreId(storeId:String):void;

  /**
   * Sorts the data in the Store by one or more of its properties. Example usage:
   *
   * <pre><code>//sort by a single field
   * myStore.sort('myField', 'DESC');
   *
   * //sorting by multiple fields
   * myStore.sort([
   *     {
   *         property : 'age',
   *         direction: 'ASC'
   *     },
   *     {
   *         property : 'name',
   *         direction: 'DESC'
   *     }
   * ]);
   * </code></pre>
   *
   * <p>Internally, Store converts the passed arguments into an array of <a href="#!/api/Ext.util.Sorter" rel="Ext.util.Sorter" class="docClass">Ext.util.Sorter</a> instances, and delegates
   * the actual sorting to its internal <a href="#!/api/Ext.util.MixedCollection" rel="Ext.util.MixedCollection" class="docClass">Ext.util.MixedCollection</a>.</p>
   *
   * <p>When passing a single string argument to sort, Store maintains a ASC/DESC toggler per field, so this code:</p>
   *
   * <pre><code>store.sort('myField');
   * store.sort('myField');
   * </code></pre>
   *
   * <p>Is equivalent to this code, because Store handles the toggling automatically:</p>
   *
   * <pre><code>store.sort('myField', 'ASC');
   * store.sort('myField', 'DESC');
   * </code></pre>
   * @param sorters Either a string name of one of the fields in this Store's configured
   * <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Model</a>, or an array of sorter configurations.
   * @param direction The overall direction to sort the data by.
   * @default "ASC"
   */
  public native function sort(sorters:* = undefined, direction:String = "ASC"):void;
}
}