package ext.data {
import ext.data.request.BaseRequest;
import ext.mixin.Observable;

[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Native("Ext.data.Connection", require)]
/**
 * Fires before a network request is made to retrieve a data object.
 * @eventType ext.data.events.Connection_optionsEvent.BEFORE_REQUEST
 */
[Event(name = 'onBeforeRequest', type = 'ext.data.events.Connection_optionsEvent')]
/**
 * Fires if the request was successfully completed.
 * @eventType ext.data.events.Connection_response_optionsEvent.REQUEST_COMPLETE
 */
[Event(name = 'onRequestComplete', type = 'ext.data.events.Connection_response_optionsEvent')]
/**
 * Fires if an error HTTP status was returned from the server. This event may also
 * be listened to in the event that a request has timed out or has been aborted.
 * See <a href="http://www.w3.org/Protocols/rfc2616/rfc2616-sec10.html">HTTP Status Code Definitions</a>
 * for details of HTTP status codes.
 * @eventType ext.data.events.Connection_response_optionsEvent.REQUEST_EXCEPTION
 */
[Event(name = 'onRequestException', type = 'ext.data.events.Connection_response_optionsEvent')]
/**
 * The Connection class encapsulates a connection to the page's originating domain, allowing requests to be made either
 * to a configured URL, or to a URL specified at request time.
 *
 * <p>Requests made by this class are asynchronous, and will return immediately. No data from the server will be available
 * to the statement immediately following the <a href="#!/api/Ext.data.Connection-method-request" rel="Ext.data.Connection-method-request" class="docClass">request</a> call. To process returned data, use a success callback
 * in the request options object, or an <a href="#!/api/Ext.data.Connection-event-requestcomplete" rel="Ext.data.Connection-event-requestcomplete" class="docClass">event listener</a>.</p>
 *
 * <h1>File Uploads</h1>
 *
 * <p>File uploads are not performed using normal "Ajax" techniques, that is they are not performed using XMLHttpRequests.
 * Instead the form is submitted in the standard manner with the DOM &lt;form&gt; element temporarily modified to have its
 * target set to refer to a dynamically generated, hidden &lt;iframe&gt; which is inserted into the document but removed
 * after the return data has been gathered.</p>
 *
 * <p>The server response is parsed by the browser to create the document for the IFRAME. If the server is using JSON to
 * send the return object, then the Content-Type header must be set to "text/html" in order to tell the browser to
 * insert the text unchanged into the document body.</p>
 *
 * <p>Characters which are significant to an HTML parser must be sent as HTML entities, so encode <code>&lt;</code> as <code>&amp;lt;</code>, <code>&amp;</code> as
 * <code>&amp;amp;</code> etc.</p>
 *
 * <p>The response text is retrieved from the document, and a fake XMLHttpRequest object is created containing a
 * responseText property in order to conform to the requirements of event handlers and callbacks.</p>
 *
 * <p>Be aware that file upload packets are sent with the content type multipart/form and some server technologies
 * (notably JEE) may require some custom processing in order to retrieve parameter names and parameter values from the
 * packet content.</p>
 *
 * <p>Also note that it's not possible to check the response code of the hidden iframe, so the success handler will ALWAYS fire.</p>
 *
 * <h1>Binary Posts</h1>
 *
 * <p>The class supports posting binary data to the server by using native browser capabilities, or a flash polyfill plugin in browsers that do not support native binary posting (e.g. Internet Explorer version 9 or less). A number of limitations exist when the polyfill is used:</p>
 *
 * <ul>
 * <li>Only asynchronous connections are supported.</li>
 * <li>Only the POST method can be used.</li>
 * <li>The return data can only be binary for now. Set the <a href="#!/api/Ext.data.Connection-cfg-binary" rel="Ext.data.Connection-cfg-binary" class="docClass">binary</a> parameter to <tt>true</tt>.</li>
 * <li>Only the 0, 1 and 4 (complete) readyState values will be reported to listeners.</li>
 * <li>The flash object will be injected at the bottom of the document and should be invisible.</li>
 * <li>Important: See note about packaing the flash plugin with the app in the documenetation of <a href="#!/api/Ext.data.flash.BinaryXhr" rel="Ext.data.flash.BinaryXhr" class="docClass">BinaryXhr</a>.</li>
 * </ul>
 */
public class Connection extends ext.mixin.Observable {
  /**
   * @param config @inheritDoc
   */
  public function Connection(config:ext.data.Connection = null) {
    super();
  }

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * <code>true</code> if this request should run asynchronously. Setting this to <code>false</code> should generally
   * be avoided, since it will cause the UI to be blocked, the user won't be able to interact
   * with the browser until the request completes.
   * @default true
   * @see #getAsync()
   * @see #setAsync()
   */
  public native function get async():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set async(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Whether this request should abort any pending requests.
   * @default false
   * @see #getAutoAbort()
   * @see #setAutoAbort()
   */
  public native function get autoAbort():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set autoAbort(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * True if the response should be treated as binary data.  If true, the binary
   * data will be accessible as a "responseBytes" property on the response object.
   * @default false
   * @see #getBinary()
   * @see #setBinary()
   */
  public native function get binary():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set binary(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * True to enable CORS support on the XHR object. Currently the only effect of this option
   * is to use the XDomainRequest object instead of XMLHttpRequest if the browser is IE8 or above.
   * @default false
   * @see #getCors()
   * @see #setCors()
   */
  public native function get cors():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set cors(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * An object containing request headers which are added to each request made by this object.
   * @see #getDefaultHeaders()
   * @see #setDefaultHeaders()
   */
  public native function get defaultHeaders():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set defaultHeaders(value:Object):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The default header to be sent out with any post request.
   * @default 'application/x-www-form-urlencoded; charset=UTF-8'
   * @see #getDefaultPostHeader()
   * @see #setDefaultPostHeader()
   */
  public native function get defaultPostHeader():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set defaultPostHeader(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The header to send with Ajax requests. Also see <a href="#!/api/Ext.data.Connection-cfg-useDefaultXhrHeader" rel="Ext.data.Connection-cfg-useDefaultXhrHeader" class="docClass">useDefaultXhrHeader</a>.
   * @default 'XMLHttpRequest'
   * @see #getDefaultXhrHeader()
   * @see #setDefaultXhrHeader()
   */
  public native function get defaultXhrHeader():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set defaultXhrHeader(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * True to add a unique cache-buster param to GET requests.
   * @default true
   * @see #getDisableCaching()
   * @see #setDisableCaching()
   */
  public native function get disableCaching():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set disableCaching(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Change the parameter which is sent went disabling caching through a cache buster.
   * @default '_dc'
   * @see #getDisableCachingParam()
   * @see #setDisableCachingParam()
   */
  public native function get disableCachingParam():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set disableCachingParam(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Any parameters to be appended to the request.
   * @see #getExtraParams()
   * @see #setExtraParams()
   */
  public native function get extraParams():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set extraParams(value:Object):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The default HTTP method to be used for requests.
   *
   * <p>If not set, but <a href="#!/api/Ext.data.Connection-method-request" rel="Ext.data.Connection-method-request" class="docClass">request</a> params are present, POST will be used;
   * otherwise, GET will be used.</p>
   * @see #getMethod()
   * @see #setMethod()
   */
  public native function get method():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set method(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The password to pass when using <a href="#!/api/Ext.data.Connection-cfg-withCredentials" rel="Ext.data.Connection-cfg-withCredentials" class="docClass">withCredentials</a>.
   * @default ''
   * @see #getPassword()
   * @see #setPassword()
   */
  public native function get password():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set password(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The timeout in milliseconds to be used for
   * requests.<br/>
   * Defaults to 30000 milliseconds (30 seconds).
   *
   * <p>When a request fails due to timeout the XMLHttpRequest response object will
   * contain:</p>
   *
   * <pre><code>timedout: true
   * </code></pre>
   * @default 30000
   * @see #getTimeout()
   * @see #setTimeout()
   */
  public native function get timeout():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set timeout(value:Number):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The URL for this connection.
   * @see #getUrl()
   * @see #setUrl()
   */
  public native function get url():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set url(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * <code>true</code> to send the <a href="#!/api/Ext.data.Connection-cfg-defaultXhrHeader" rel="Ext.data.Connection-cfg-defaultXhrHeader" class="docClass">defaultXhrHeader</a> along with any request.
   * @default true
   * @see #getUseDefaultXhrHeader()
   * @see #setUseDefaultXhrHeader()
   */
  public native function get useDefaultXhrHeader():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set useDefaultXhrHeader(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The username to pass when using <a href="#!/api/Ext.data.Connection-cfg-withCredentials" rel="Ext.data.Connection-cfg-withCredentials" class="docClass">withCredentials</a>.
   * @default ''
   * @see #getUsername()
   * @see #setUsername()
   */
  public native function get username():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set username(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * True to set <code>withCredentials = true</code> on the XHR object
   * @default false
   * @see #getWithCredentials()
   * @see #setWithCredentials()
   */
  public native function get withCredentials():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set withCredentials(value:Boolean):void;

  /**
   * Aborts an active request.
   * @param request Defaults to the last request
   */
  public native function abort(request:Object = null):void;

  /**
   * Aborts all active requests
   */
  public native function abortAll():void;

  /**
   * Returns the value of <a href="#!/api/Ext.data.Connection-cfg-async" rel="Ext.data.Connection-cfg-async" class="docClass">async</a>.
   */
  public native function getAsync():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.data.Connection-cfg-autoAbort" rel="Ext.data.Connection-cfg-autoAbort" class="docClass">autoAbort</a>.
   */
  public native function getAutoAbort():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.data.Connection-cfg-binary" rel="Ext.data.Connection-cfg-binary" class="docClass">binary</a>.
   */
  public native function getBinary():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.data.Connection-cfg-cors" rel="Ext.data.Connection-cfg-cors" class="docClass">cors</a>.
   */
  public native function getCors():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.data.Connection-cfg-defaultHeaders" rel="Ext.data.Connection-cfg-defaultHeaders" class="docClass">defaultHeaders</a>.
   */
  public native function getDefaultHeaders():Object;

  /**
   * Returns the value of <a href="#!/api/Ext.data.Connection-cfg-defaultPostHeader" rel="Ext.data.Connection-cfg-defaultPostHeader" class="docClass">defaultPostHeader</a>.
   */
  public native function getDefaultPostHeader():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.Connection-cfg-defaultXhrHeader" rel="Ext.data.Connection-cfg-defaultXhrHeader" class="docClass">defaultXhrHeader</a>.
   */
  public native function getDefaultXhrHeader():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.Connection-cfg-disableCaching" rel="Ext.data.Connection-cfg-disableCaching" class="docClass">disableCaching</a>.
   */
  public native function getDisableCaching():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.data.Connection-cfg-disableCachingParam" rel="Ext.data.Connection-cfg-disableCachingParam" class="docClass">disableCachingParam</a>.
   */
  public native function getDisableCachingParam():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.Connection-cfg-extraParams" rel="Ext.data.Connection-cfg-extraParams" class="docClass">extraParams</a>.
   */
  public native function getExtraParams():Object;

  /**
   * Returns the value of <a href="#!/api/Ext.data.Connection-cfg-method" rel="Ext.data.Connection-cfg-method" class="docClass">method</a>.
   */
  public native function getMethod():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.Connection-cfg-password" rel="Ext.data.Connection-cfg-password" class="docClass">password</a>.
   */
  public native function getPassword():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.Connection-cfg-timeout" rel="Ext.data.Connection-cfg-timeout" class="docClass">timeout</a>.
   */
  public native function getTimeout():Number;

  /**
   * Returns the value of <a href="#!/api/Ext.data.Connection-cfg-url" rel="Ext.data.Connection-cfg-url" class="docClass">url</a>.
   */
  public native function getUrl():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.Connection-cfg-useDefaultXhrHeader" rel="Ext.data.Connection-cfg-useDefaultXhrHeader" class="docClass">useDefaultXhrHeader</a>.
   */
  public native function getUseDefaultXhrHeader():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.data.Connection-cfg-username" rel="Ext.data.Connection-cfg-username" class="docClass">username</a>.
   */
  public native function getUsername():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.Connection-cfg-withCredentials" rel="Ext.data.Connection-cfg-withCredentials" class="docClass">withCredentials</a>.
   */
  public native function getWithCredentials():Boolean;

  /**
   * Determines whether this object has a request outstanding.
   * @param request Defaults to the last transaction
   * @return True if there is an outstanding request.
   */
  public native function isLoading(request:Object = null):Boolean;

  /**
   * Sends an HTTP (Ajax) request to a remote server.
   *
   * <p><strong>Important:</strong> Ajax server requests are asynchronous, and this call will
   * return before the response has been received.</p>
   *
   * <p>Instead, process any returned data using a promise:</p>
   *
   * <pre><code> <a href="#!/api/Ext.Ajax-method-request" rel="Ext.Ajax-method-request" class="docClass">Ext.Ajax.request</a>({
   *      url: 'ajax_demo/sample.json'
   *  }).then(function(response, opts) {
   *      var obj = <a href="#!/api/Ext-method-decode" rel="Ext-method-decode" class="docClass">Ext.decode</a>(response.responseText);
   *      console.dir(obj);
   *  },
   *  function(response, opts) {
   *      console.log('server-side failure with status code ' + response.status);
   *  });
   * </code></pre>
   *
   * <p>Or in callback functions:</p>
   *
   * <pre><code> <a href="#!/api/Ext.Ajax-method-request" rel="Ext.Ajax-method-request" class="docClass">Ext.Ajax.request</a>({
   *      url: 'ajax_demo/sample.json',
   *
   *      success: function(response, opts) {
   *          var obj = <a href="#!/api/Ext-method-decode" rel="Ext-method-decode" class="docClass">Ext.decode</a>(response.responseText);
   *          console.dir(obj);
   *      },
   *
   *      failure: function(response, opts) {
   *          console.log('server-side failure with status code ' + response.status);
   *      }
   *  });
   * </code></pre>
   *
   * <p>To execute a callback function in the correct scope, use the <code>scope</code> option.</p>
   * @param options An object which may contain the following properties:
   * <p>(The options object may also contain any other property which might be needed to perform
   * postprocessing in a callback because it is passed to callback functions.)</p>
   * @param options.url The URL to which to send the request, or a function
   * to call which returns a URL string. The scope of the function is specified by the <code>scope</code> option.
   * Defaults to the configured <code>url</code>.
   * @param {Boolean} options.async <code>true</code> if this request should run asynchronously.
   * Setting this to <code>false</code> should generally be avoided, since it will cause the UI to be
   * blocked, the user won't be able to interact with the browser until the request completes.
   * Defaults to <code>true</code>.
   * @param options.params An object containing properties which are
   * used as parameters to the request, a url encoded string or a function to call to get either. The scope
   * of the function is specified by the <code>scope</code> option.
   * @param {String} options.method The HTTP method to use
   * for the request. Defaults to the configured method, or if no method was configured,
   * "GET" if no parameters are being sent, and "POST" if parameters are being sent.  Note that
   * the method name is case-sensitive and should be all caps.
   * @param {Function} options.callback The function to be called upon receipt of the HTTP response.
   * The callback is called regardless of success or failure and is passed the following parameters:
   * @param {Object} options.callback.options The parameter to the request call.
   * @param {Boolean} options.callback.success True if the request succeeded.
   * @param {Object} options.callback.response The XMLHttpRequest object containing the response data.
   * See <a href="http://www.w3.org/TR/XMLHttpRequest/">www.w3.org/TR/XMLHttpRequest/</a> for details about
   * accessing elements of the response.
   * @param {Function} options.success The function to be called upon success of the request.
   * The callback is passed the following parameters:
   * @param {Object} options.success.response The XMLHttpRequest object containing the response data.
   * @param {Object} options.success.options The parameter to the request call.
   * @param {Function} options.failure The function to be called upon failure of the request.
   * The callback is passed the following parameters:
   * @param {Object} options.failure.response The XMLHttpRequest object containing the response data.
   * @param {Object} options.failure.options The parameter to the request call.
   * @param {Object} options.scope The scope in which to execute the callbacks: The "this" object for
   * the callback function. If the <code>url</code>, or <code>params</code> options were specified as functions from which to
   * draw values, then this also serves as the scope for those function calls. Defaults to the browser
   * window.
   * @param {Number} options.timeout The timeout in milliseconds to be used for this
   * request.<br/>
   * Defaults to 30000 milliseconds (30 seconds).
   * <p>When a request fails due to timeout the XMLHttpRequest response object will
   * contain:</p>
   * <pre><code>timedout: true
   * </code></pre>
   * @param options.form The <code>&lt;form&gt;</code> Element or the id of the <code>&lt;form&gt;</code>
   * to pull parameters from.
   * @param {Boolean} options.isUpload <strong>Only meaningful when used with the <code>form</code> option.</strong>
   * <p>True if the form object is a file upload (will be set automatically if the form was configured
   * with <strong><code>enctype</code></strong> <code>"multipart/form-data"</code>).</p>
   * <p>File uploads are not performed using normal "Ajax" techniques, that is they are <strong>not</strong>
   * performed using XMLHttpRequests. Instead the form is submitted in the standard manner with the
   * DOM <code>&lt;form&gt;</code> element temporarily modified to have its <a href="http://www.w3.org/TR/REC-html40/present/frames.html#adef-target">target</a> set to refer to a dynamically
   * generated, hidden <code>&lt;iframe&gt;</code> which is inserted into the document but removed after the return data
   * has been gathered.</p>
   * <p>The server response is parsed by the browser to create the document for the IFRAME. If the
   * server is using JSON to send the return object, then the <a href="http://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.17">Content-Type</a> header must be set to
   * "text/html" in order to tell the browser to insert the text unchanged into the document body.</p>
   * <p>The response text is retrieved from the document, and a fake XMLHttpRequest object is created
   * containing a <code>responseText</code> property in order to conform to the requirements of event handlers
   * and callbacks.</p>
   * <p>Be aware that file upload packets are sent with the content type <a href="http://www.faqs.org/rfcs/rfc2388.html">multipart/form</a> and some server
   * technologies (notably JEE) may require some custom processing in order to retrieve parameter names
   * and parameter values from the packet content.</p>
   * @param {Object} options.headers Request headers to set for the request.
   * The XHR will attempt to set an appropriate Content-Type based on the params/data passed
   * to the request. To prevent this, setting the Content-Type header to <code>null</code> or <code>undefined</code>
   * will not attempt to set any Content-Type and it will be left to the browser.
   * @param {Object} options.xmlData XML document to use for the post. Note: This will be used instead
   * of params for the post data. Any params will be appended to the URL.
   * @param options.jsonData JSON data to use as the post. Note: This will be used
   * instead of params for the post data. Any params will be appended to the URL.
   * @param {String} options.rawData A raw string to use as the post. Note: This will be used
   * instead of params for the post data. Any params will be appended to the URL.
   * @param {Array} options.binaryData An array of bytes to submit in binary form. Any params will be appended to the URL. If binaryData is present, you must set <a href="#!/api/Ext.data.Connection-cfg-binary" rel="Ext.data.Connection-cfg-binary" class="docClass">binary</a> to <tt>true</tt> and options.method to <tt>POST</tt>.
   * @param {Boolean} options.disableCaching True to add a unique cache-buster param to GET requests.
   * @param {Boolean} options.withCredentials True to add the withCredentials property to the XHR object
   * @param {String} options.username The username to pass when using <code>withCredentials</code>.
   * @param {String} options.password The password to pass when using <code>withCredentials</code>.
   * @param {Boolean} options.binary True if the response should be treated as binary data.  If true, the binary
   * data will be accessible as a "responseBytes" property on the response object.
   * @return The request object. This may be used to abort the
   * request.
   */
  public native function request(options:Object):ext.data.request.BaseRequest;

  /**
   * Sets the value of <a href="#!/api/Ext.data.Connection-cfg-async" rel="Ext.data.Connection-cfg-async" class="docClass">async</a>.
   * @param async The new value.
   */
  public native function setAsync(async:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.Connection-cfg-autoAbort" rel="Ext.data.Connection-cfg-autoAbort" class="docClass">autoAbort</a>.
   * @param autoAbort The new value.
   */
  public native function setAutoAbort(autoAbort:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.Connection-cfg-binary" rel="Ext.data.Connection-cfg-binary" class="docClass">binary</a>.
   * @param binary The new value.
   */
  public native function setBinary(binary:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.Connection-cfg-cors" rel="Ext.data.Connection-cfg-cors" class="docClass">cors</a>.
   * @param cors The new value.
   */
  public native function setCors(cors:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.Connection-cfg-defaultHeaders" rel="Ext.data.Connection-cfg-defaultHeaders" class="docClass">defaultHeaders</a>.
   * @param defaultHeaders The new value.
   */
  public native function setDefaultHeaders(defaultHeaders:Object):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.Connection-cfg-defaultPostHeader" rel="Ext.data.Connection-cfg-defaultPostHeader" class="docClass">defaultPostHeader</a>.
   * @param defaultPostHeader The new value.
   */
  public native function setDefaultPostHeader(defaultPostHeader:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.Connection-cfg-defaultXhrHeader" rel="Ext.data.Connection-cfg-defaultXhrHeader" class="docClass">defaultXhrHeader</a>.
   * @param defaultXhrHeader The new value.
   */
  public native function setDefaultXhrHeader(defaultXhrHeader:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.Connection-cfg-disableCaching" rel="Ext.data.Connection-cfg-disableCaching" class="docClass">disableCaching</a>.
   * @param disableCaching The new value.
   */
  public native function setDisableCaching(disableCaching:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.Connection-cfg-disableCachingParam" rel="Ext.data.Connection-cfg-disableCachingParam" class="docClass">disableCachingParam</a>.
   * @param disableCachingParam The new value.
   */
  public native function setDisableCachingParam(disableCachingParam:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.Connection-cfg-extraParams" rel="Ext.data.Connection-cfg-extraParams" class="docClass">extraParams</a>.
   * @param extraParams The new value.
   */
  public native function setExtraParams(extraParams:Object):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.Connection-cfg-method" rel="Ext.data.Connection-cfg-method" class="docClass">method</a>.
   * @param method The new value.
   */
  public native function setMethod(method:String):void;

  /**
   * Sets various options such as the url, params for the request
   * @param options The initial options
   * @param scope The scope to execute in
   * @return The params for the request
   */
  public native function setOptions(options:Object, scope:Object):Object;

  /**
   * Sets the value of <a href="#!/api/Ext.data.Connection-cfg-password" rel="Ext.data.Connection-cfg-password" class="docClass">password</a>.
   * @param password The new value.
   */
  public native function setPassword(password:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.Connection-cfg-timeout" rel="Ext.data.Connection-cfg-timeout" class="docClass">timeout</a>.
   * @param timeout The new value.
   */
  public native function setTimeout(timeout:Number):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.Connection-cfg-url" rel="Ext.data.Connection-cfg-url" class="docClass">url</a>.
   * @param url The new value.
   */
  public native function setUrl(url:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.Connection-cfg-useDefaultXhrHeader" rel="Ext.data.Connection-cfg-useDefaultXhrHeader" class="docClass">useDefaultXhrHeader</a>.
   * @param useDefaultXhrHeader The new value.
   */
  public native function setUseDefaultXhrHeader(useDefaultXhrHeader:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.Connection-cfg-username" rel="Ext.data.Connection-cfg-username" class="docClass">username</a>.
   * @param username The new value.
   */
  public native function setUsername(username:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.Connection-cfg-withCredentials" rel="Ext.data.Connection-cfg-withCredentials" class="docClass">withCredentials</a>.
   * @param withCredentials The new value.
   */
  public native function setWithCredentials(withCredentials:Boolean):void;
}
}