package ext.data {
import ext.util.MixedCollection;

[Uses("ext.util.MixedCollection")]
[Uses("ext.util.AbstractMixedCollection")]
[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Uses("ext.util.Sortable")]
[Native("Ext.data.ErrorCollection", require)]
/**
 * Wraps a collection of validation error responses and provides convenient functions for
 * accessing and errors for specific fields.
 *
 * <p>Usually this class does not need to be instantiated directly - instances are instead
 * created automatically when <a href="#!/api/Ext.data.Model-method-validate" rel="Ext.data.Model-method-validate" class="docClass">validate</a> on a model instance:</p>
 *
 * <pre><code> // Validate some existing model instance - in this case it returned 2 failures
 *  // messages
 *
 *  var errors = myModel.validate();
 *  errors.isValid(); //false
 *
 *  errors.length; //2
 *  errors.getByField('name');  // [{field: 'name',  message: 'must be present'}]
 *  errors.getByField('title'); // [{field: 'title', message: 'is too short'}]
 * </code></pre>
 */
public class DataErrorCollection extends ext.util.MixedCollection {
  /**
   * @inheritDoc
   */
  public function DataErrorCollection(config:ext.util.MixedCollection = null) {
    super();
  }

  /**
   * Returns all of the errors for the given field
   * @param fieldName The field to get errors for
   * @return All errors for the given field
   */
  public native function getByField(fieldName:String):Array;

  /**
   * Returns true if there are no errors in the collection
   */
  public native function isValid():Boolean;
}
}