package ext.data {

[Uses("ext.data.Store")]
[Uses("ext.data.ProxyStore")]
[Uses("ext.data.AbstractStore")]
[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Uses("ext.mixin.Factoryable")]
[Uses("ext.data.LocalStore")]
[Uses("ext.Mixin")]
[Native("Ext.data.JsonStore", require)]
/**
 * Small helper class to make creating <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>s from JSON data easier.
 * A JsonStore will be automatically configured with a <a href="#!/api/Ext.data.reader.Json" rel="Ext.data.reader.Json" class="docClass">Ext.data.reader.Json</a>.
 *
 * <p>A store configuration would be something like:</p>
 *
 * <pre><code>var store = new <a href="#!/api/Ext.data.JsonStore" rel="Ext.data.JsonStore" class="docClass">Ext.data.JsonStore</a>({
 *     // store configs
 *     storeId: 'myStore',
 *
 *     proxy: {
 *         type: 'ajax',
 *         url: 'get-images.php',
 *         reader: {
 *             type: 'json',
 *             rootProperty: 'images'
 *         }
 *     },
 *
 *     //alternatively, a Model name can be given (see <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a> for an example)
 *     fields: ['name', 'url', {name:'size', type: 'float'}, {name:'lastmod', type:'date'}]
 * });
 * </code></pre>
 *
 * <p>This store is configured to consume a returned object of the form:</p>
 *
 * <pre><code>{
 *     images: [
 *         {name: 'Image one', url:'/GetImage.php?id=1', size:46.5, lastmod: new Date(2007, 10, 29)},
 *         {name: 'Image Two', url:'/GetImage.php?id=2', size:43.2, lastmod: new Date(2007, 10, 30)}
 *     ]
 * }
 * </code></pre>
 *
 * <p>An object literal of this form could also be used as the <a href="#!/api/Ext.data.JsonStore-cfg-data" rel="Ext.data.JsonStore-cfg-data" class="docClass">data</a> config option.</p>
 */
public class JsonStore extends ext.data.Store {
  /**
   * @param config @inheritDoc
   */
  public function JsonStore(config:ext.data.JsonStore = null) {
    super();
  }
}
}