package ext.data {
import ext.ExtClass;
import ext.mixin.IDirty;
import ext.mixin.Observable;

[Uses("ext.mixin.Observable")]
[Uses("ext.mixin.Dirty")]
[Uses("ext.Base")]
[Native("Ext.data.Session", require)]
/**
 * Fires when a change in the object's <a href="#!/api/Ext.mixin.Dirty-cfg-dirty" rel="Ext.mixin.Dirty-cfg-dirty" class="docClass">dirty</a> state is detected.
 *
 * <p><strong>Note:</strong> In order for this event to fire, the class that mixes in this mixin
 * must be <code><a href="#!/api/Ext.mixin.Observable" rel="Ext.mixin.Observable" class="docClass">Observable</a></code>.</p>
 * @eventType ext.data.events.Session_dirtyEvent.DIRTY_CHANGE
 */
[Event(name = 'onDirtyChange', type = 'ext.data.events.Session_dirtyEvent')]
/**
 * This class manages models and their associations. Instances of <code>Session</code> are typically
 * associated with some <code>Component</code> (perhaps the Viewport or a Window) and then used by
 * their <code><a href="#!/api/Ext.app.ViewModel" rel="Ext.app.ViewModel" class="docClass">view models</a></code> to enable data binding.
 *
 * <p>The primary job of a Session is to manage a collection of records of many different
 * types and their associations. This often starts by loading records when requested (via
 * bind - see below) and culminates when it is time to save to the server.</p>
 *
 * <p>Because the Session tracks all records it loads, it ensures that for any given type of
 * model, only one record exists with a given <code>id</code>. This means that all edits of that
 * record are properly targeted at that one instance.</p>
 *
 * <p>Similarly, when associations are loaded, the <code><a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a></code> created to hold the
 * associated records is tracked by the Session. So all requests for the "OrderItems" of
 * a particular Order id will result in the same Store. Adding and removing items from
 * that Order then is sure to remain consistent.</p>
 *
 * <h1>Data</h1>
 *
 * <p>Since the Session is managing all this data, there are several methods it provides
 * to give convenient access to that data. The most important of these is <code>update</code> and
 * <code>getChanges</code>.</p>
 *
 * <p>The <code>update</code> and <code>getChanges</code> methods both operate on object that contains a summary
 * of records and associations and different CRUD operations.</p>
 *
 * <h2>Saving</h2>
 *
 * <p>There are two basic ways to save the contents of a Session: <code>getChanges</code> and
 * <code>getSaveBatch</code>. We've already seen <code>getChanges</code>. The data contained in the CRUD object
 * can be translated into whatever shape is needed by the server.</p>
 *
 * <p>To leverage the <code><a href="#!/api/Ext.data.Model-cfg-proxy" rel="Ext.data.Model-cfg-proxy" class="docClass">proxy</a></code> facilities defined by each Model
 * class, there is the <code>getSaveBatch</code> method. That method returns an <code><a href="#!/api/Ext.data.Batch" rel="Ext.data.Batch" class="docClass">Ext.data.Batch</a></code>
 * object populated with the necessary <code>create</code>, <code>update</code> and <code>destory</code> operations to
 * save all of the changes in the Session.</p>
 */
public class Session extends ext.mixin.Observable implements ext.mixin.IDirty {
  /**
   * @param config @inheritDoc
   */
  public function Session(config:ext.data.Session = null) {
    super();
  }

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * <code>true</code> to automatically destroy this session when a component it is attached
   * to is destroyed. This should be set to false if the session is intended to be
   * used across multiple root level components.
   * @default true
   * @since 5.0.1
   * @see #getAutoDestroy()
   * @see #setAutoDestroy()
   */
  public native function get autoDestroy():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set autoDestroy(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * This config property describes the modified state of this object. In most
   * cases this config's value is maintained by the object and should be considered
   * readonly. The class implementor should be the only one to call the setter.
   * @see #getDirty()
   * @see #setDirty()
   */
  public native function get dirty():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set dirty(value:Boolean):void;

  [ExtConfig]
  /**
   * This config property indicates that the <code>dirty</code> state of this object should be
   * ignored. Because this capability is mixed in at a class level, this config can
   * be helpful when some instances do not participate in dirty state tracking.
   *
   * <p>This option should be set at construction time. When set to <code>true</code>, the object
   * will always have <code>dirty</code> value of <code>false</code>.</p>
   * @default false
   */
  public native function get ignoreDirty():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set ignoreDirty(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The parent session for this session.
   * @see #getParent()
   * @see #setParent()
   */
  public native function get parent():ext.data.Session;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set parent(value:ext.data.Session):void;

  [ExtConfig]
  /**
   * Set this config at construction time (or on the class body) to automatically set
   * the <code>dirty</code> state based on the records passed to <code>trackRecordState</code>.
   *
   * <p>This config defaults to <code>true</code> but only has an effect when the record tracking
   * methods are called (<code>trackRecordState</code>, <code>untrackRecordState</code> and <code>clearRecordStates</code>).</p>
   * @default true
   */
  public native function get recordStateIsDirtyState():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set recordStateIsDirtyState(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @default 'default'
   * @see #getSchema()
   * @see #setSchema()
   */
  public native function get schema():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set schema(value:*):void;

  /**
   * Adds an existing record instance to the session. The record
   * may not belong to another session. The record cannot be a phantom record, instead
   * use <a href="#!/api/Ext.data.Session-method-createRecord" rel="Ext.data.Session-method-createRecord" class="docClass">createRecord</a>.
   * @param record The record to adopt.
   */
  public native function adopt(record:ext.data.Model):void;

  /**
   * Marks the session as "clean" by calling <a href="#!/api/Ext.data.Model-method-commit" rel="Ext.data.Model-method-commit" class="docClass">Ext.data.Model.commit</a> on each record
   * that is known to the session.
   *
   * <ul>
   * <li>Phantom records will no longer be phantom.</li>
   * <li>Modified records will no longer be dirty.</li>
   * <li>Dropped records will be erased.</li>
   * </ul>
   * @since 5.1.0
   */
  public native function commit():void;

  /**
   * Creates a new record and tracks it in this session.
   * @param type The <code>entityName</code> or the actual class of record to create.
   * @param data The data for the record.
   * @return The new record.
   */
  public native function createRecord(type:*, data:Object = null):ext.data.Model;

  /**
   * Returns the value of <a href="#!/api/Ext.data.Session-cfg-autoDestroy" rel="Ext.data.Session-cfg-autoDestroy" class="docClass">autoDestroy</a>.
   * @since 5.0.1
   */
  public native function getAutoDestroy():Boolean;

  /**
   * Returns an object describing all of the modified fields, created or dropped records
   * and many-to-many association changes maintained by this session.
   * @return An object in the CRUD format (see the intro docs). <code>null</code> if there are no changes.
   */
  public native function getChanges():Object;

  /**
   * Returns the value of <a href="#!/api/Ext.mixin.Dirty-cfg-dirty" rel="Ext.mixin.Dirty-cfg-dirty" class="docClass">dirty</a>.
   */
  public native function getDirty():Boolean;

  /**
   * Transforms a list of ids into a list of records for a particular type.
   * @param entityType The entity type.
   * @param ids The ids to transform.
   * @return The models corresponding to the ids.
   */
  public native function getEntityList(entityType:ext.ExtClass, ids:Array):Array;

  /**
   * Gets a user friendly identifier for a Model.
   * @param entityType The entity type.
   * @param id The id of the entity.
   * @return The identifier.
   */
  public native function getModelIdentifier(entityType:ext.ExtClass, id:Object):String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.Session-cfg-parent" rel="Ext.data.Session-cfg-parent" class="docClass">parent</a>.
   */
  public native function getParent():ext.data.Session;

  /**
   * Get a cached record from the session. If the record does not exist, it will
   * be created. If the <code>autoLoad</code> parameter is not set to <code>false</code>, the record will
   * be loaded via the <a href="#!/api/Ext.data.Model-cfg-proxy" rel="Ext.data.Model-cfg-proxy" class="docClass">proxy</a> of the Model.
   *
   * <p>If this session is configured with a <code><a href="#!/api/Ext.data.Session-cfg-parent" rel="Ext.data.Session-cfg-parent" class="docClass">parent</a></code> session, a <em>copy</em> of any existing record
   * in the <code>parent</code> will be adopted into this session. If the <code>parent</code> does not contain the record,
   * the record will be created and <em>not</em> inserted into the parent.</p>
   *
   * <p>See also <a href="#!/api/Ext.data.Session-method-peekRecord" rel="Ext.data.Session-method-peekRecord" class="docClass">peekRecord</a>.</p>
   * @param type The <code>entityName</code> or the actual class of record to create.
   * This may also be a record instance, where the type and id will be inferred from the record. If the record is
   * not attached to a session, it will be adopted. If it exists in a parent, an appropriate copy will be made as
   * described.
   * @param id The id of the record.
   * @param autoLoad <code>false</code> to prevent the record from being loaded if
   * it does not exist. If this parameter is an object, it will be passed to the <a href="#!/api/Ext.data.Model-method-load" rel="Ext.data.Model-method-load" class="docClass">Ext.data.Model.load</a> call.
   * @default true
   * @return The record.
   */
  public native function getRecord(type:*, id:Object, autoLoad:* = true):ext.data.Model;

  /**
   * Returns an <code><a href="#!/api/Ext.data.Batch" rel="Ext.data.Batch" class="docClass">Ext.data.Batch</a></code> containing the <code><a href="#!/api/Ext.data.operation.Operation" rel="Ext.data.operation.Operation" class="docClass">Ext.data.operation.Operation</a></code> instances
   * that are needed to save all of the changes in this session. This sorting is based
   * on operation type, associations and foreign keys. Generally speaking the operations
   * in the batch can be committed to a server sequentially and the server will never be
   * sent a request with an invalid (client-generated) id in a foreign key field.
   * @param sort Pass <code>false</code> to disable the batch operation sort.
   * @default true
   */
  public native function getSaveBatch(sort:Boolean = true):ext.data.Batch;

  /**
   * Returns the value of <a href="#!/api/Ext.data.Session-cfg-schema" rel="Ext.data.Session-cfg-schema" class="docClass">schema</a>.
   */
  public native function getSchema():*;

  /**
   * Returns <code>true</code> if this object is <code>dirty</code>.
   */
  public native function isDirty():void;

  /**
   * Gets an existing record from the session. The record will <em>not</em> be created if it does
   * not exist.
   *
   * <p>See also: <a href="#!/api/Ext.data.Session-method-getRecord" rel="Ext.data.Session-method-getRecord" class="docClass">getRecord</a>.</p>
   * @param type The <code>entityName</code> or the actual class of record to create.
   * @param id The id of the record.
   * @param deep <code>true</code> to consult
   * @default false
   * @return The record, <code>null</code> if it does not exist.
   */
  public native function peekRecord(type:*, id:Object, deep:Boolean = false):ext.data.Model;

  /**
   * Save any changes in this session to a <a href="#!/api/Ext.data.Session-cfg-parent" rel="Ext.data.Session-cfg-parent" class="docClass">parent</a> session.
   */
  public native function save():void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.Session-cfg-autoDestroy" rel="Ext.data.Session-cfg-autoDestroy" class="docClass">autoDestroy</a>.
   * @since 5.0.1
   * @param autoDestroy The new value.
   */
  public native function setAutoDestroy(autoDestroy:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.mixin.Dirty-cfg-dirty" rel="Ext.mixin.Dirty-cfg-dirty" class="docClass">dirty</a>.
   * @param dirty The new value.
   */
  public native function setDirty(dirty:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.Session-cfg-parent" rel="Ext.data.Session-cfg-parent" class="docClass">parent</a>.
   * @param parent The new value.
   */
  public native function setParent(parent:ext.data.Session):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.Session-cfg-schema" rel="Ext.data.Session-cfg-schema" class="docClass">schema</a>.
   * @param schema The new value.
   */
  public native function setSchema(schema:*):void;

  /**
   * Create a child session with this session as the <a href="#!/api/Ext.data.Session-cfg-parent" rel="Ext.data.Session-cfg-parent" class="docClass">parent</a>.
   * @return The copied session.
   */
  public native function spawn():ext.data.Session;

  /**
   * Complete a bulk update for this session.
   * @param data Data in the CRUD format (see the intro docs).
   */
  public native function update(data:Object):void;

  /**
   * Walks the internal data tracked by this session and calls methods on the provided
   * <code>visitor</code> object. The visitor can then accumulate whatever data it finds important.
   * The visitor object can provide a number of methods, but all are optional.
   *
   * <p>This method does not enumerate associations since these can be traversed given the
   * records that are enumerated. For many-to-many associations, however, this method
   * does enumerate the changes because these changes are not "owned" by either side of
   * such associations.</p>
   * @param visitor
   * @param {Function} [visitor.onCleanRecord] This method is called to describe a record
   * that is known but unchanged.
   * @param {ext.data.Model} visitor.onCleanRecord.record The unmodified record.
   * @param {Function} [visitor.onDirtyRecord] This method is called to describe a record
   * that has either been created, dropped or modified.
   * @param {ext.data.Model} visitor.onDirtyRecord.record The modified record.
   * @param {Function} [visitor.onMatrixChange] This method is called to describe a
   * change in a many-to-many association (a "matrix").
   * @param {ext.data.schema.Association} visitor.onMatrixChange.association The object
   * describing the many-to-many ("matrix") association.
   * @param visitor.onMatrixChange.leftId The <code>idProperty</code> of the record on the
   * "left" of the association.
   * @param visitor.onMatrixChange.rightId The <code>idProperty</code> of the record on the
   * "right" of the association.
   * @param {Number} visitor.onMatrixChange.state A negative number if the two records
   * are being disassociated or a positive number if they are being associated. For
   * example, when adding User 10 to Group 20, this would be 1. When removing the User
   * this argument would be -1.
   * @return The visitor instance
   */
  public native function visitData(visitor:Object):Object;

  /**
   * Template method, will be called by Model after a record is committed.
   * @since 6.2.0
   * @param record The record.
   */
  protected native function afterCommit(record:ext.data.Model):void;

  /**
   * Template method, will be called by Model after a record is dropped.
   * @since 6.2.0
   * @param record The record.
   */
  protected native function afterDrop(record:ext.data.Model):void;

  /**
   * Template method, will be called by Model after a record is edited.
   * @since 6.2.0
   * @param record The record.
   */
  protected native function afterEdit(record:ext.data.Model):void;

  /**
   * Template method, will be called by Model after a record is erased (a drop
   * that is committed).
   * @param record The record.
   */
  protected native function afterErase(record:ext.data.Model):void;

  /**
   * Clears all record state tracking. This state is maintained by <code>trackRecordState</code>
   * and <code>untrackRecordState</code>.
   */
  protected native function clearRecordStates():void;

  /**
   * The same functionality as <a href="#!/api/Ext.data.Session-method-getChanges" rel="Ext.data.Session-method-getChanges" class="docClass">getChanges</a>, however we also take into account our
   * parent session.
   * @return An object in the CRUD format (see the intro docs). <code>null</code> if there are no changes.
   */
  protected native function getChangesForParent():Object;

  /**
   * Triggered when an associated item from <a href="#!/api/Ext.data.Session-method-update" rel="Ext.data.Session-method-update" class="docClass">update</a> references a record
   * that does not exist in the session.
   * @param entityType The entity type.
   * @param id The id of the model.
   */
  protected native function onInvalidAssociationEntity(entityType:ext.ExtClass, id:Object):void;

  /**
   * Triggered when an drop block from <a href="#!/api/Ext.data.Session-method-update" rel="Ext.data.Session-method-update" class="docClass">update</a> tries to create a record
   * that already exists.
   * @param entityType The entity type.
   * @param id The id of the model.
   */
  protected native function onInvalidEntityCreate(entityType:ext.ExtClass, id:Object):void;

  /**
   * Triggered when an drop block from <a href="#!/api/Ext.data.Session-method-update" rel="Ext.data.Session-method-update" class="docClass">update</a> references a record
   * that does not exist in the session.
   * @param entityType The entity type.
   * @param id The id of the model.
   */
  protected native function onInvalidEntityDrop(entityType:ext.ExtClass, id:Object):void;

  /**
   * Triggered when an drop block from <a href="#!/api/Ext.data.Session-method-update" rel="Ext.data.Session-method-update" class="docClass">update</a> tries to create a record
   * that already exists.
   * @param entityType The entity type.
   * @param id The id of the model.
   */
  protected native function onInvalidEntityRead(entityType:ext.ExtClass, id:Object):void;

  /**
   * Triggered when an update block from <a href="#!/api/Ext.data.Session-method-update" rel="Ext.data.Session-method-update" class="docClass">update</a> references a record
   * that does not exist in the session.
   * @param entityType The entity type.
   * @param id The id of the model.
   * @param dropped <code>true</code> if the record was dropped.
   */
  protected native function onInvalidEntityUpdate(entityType:ext.ExtClass, id:Object, dropped:Boolean):void;

  /**
   * This method is called to track a given record in the total number of dirty records
   * (modified, created or dropped). See <code>untrackRecordState</code> and <code>clearRecordStates</code>.
   * @param record The record to track.
   * @param initial Pass <code>true</code> the first time a record is introduced.
   * @return Returns <code>true</code> if the state of dirty records has changed.
   */
  protected native function trackRecordState(record:ext.data.Model, initial:Boolean):Boolean;

  /**
   * This method is called to remove the tracking of a given record from the total number
   * of dirty records (modified, created or dropped). The record passed to this method
   * must have been previously passed to <code>trackRecordState</code>.
   * @param record The record to stop tracking.
   * @return Returns <code>true</code> if the state of dirty records has changed.
   */
  protected native function untrackRecordState(record:ext.data.Model):Boolean;
}
}