package ext.data.proxy {
import ext.data.Request;
import ext.data.operation.Operation;

[Uses("ext.data.proxy.DataProxy")]
[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Uses("ext.mixin.Factoryable")]
[Native("Ext.data.proxy.Server", require)]
/**
 * Fires when the server returns an exception. This event may also be listened
 * to in the event that a request has timed out or has been aborted.
 * @eventType ext.data.proxy.events.ServerProxy_request_operationEvent.EXCEPTION
 */
[Event(name = 'onException', type = 'ext.data.proxy.events.ServerProxy_request_operationEvent')]
/**
 * ServerProxy is a superclass of <a href="#!/api/Ext.data.proxy.JsonP" rel="Ext.data.proxy.JsonP" class="docClass">JsonPProxy</a> and <a href="#!/api/Ext.data.proxy.Ajax" rel="Ext.data.proxy.Ajax" class="docClass">AjaxProxy</a>, and
 * would not usually be used directly.
 */
public class ServerProxy extends ext.data.proxy.DataProxy {
  /**
   * @param config @inheritDoc
   */
  public function ServerProxy(config:ext.data.proxy.ServerProxy = null) {
    super();
  }

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Specific urls to call on CRUD action methods "create", "read", "update" and "destroy". Defaults to:
   *
   * <pre><code>api: {
   *     create  : undefined,
   *     read    : undefined,
   *     update  : undefined,
   *     destroy : undefined
   * }
   * </code></pre>
   *
   * <p>The url is built based upon the action being executed [create|read|update|destroy] using the commensurate
   * <a href="#!/api/Ext.data.proxy.Server-cfg-api" rel="Ext.data.proxy.Server-cfg-api" class="docClass">api</a> property, or if undefined default to the configured
   * <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>.<a href="#!/api/Ext.data.proxy.Server-cfg-url" rel="Ext.data.proxy.Server-cfg-url" class="docClass">url</a>.</p>
   *
   * <p>For example:</p>
   *
   * <pre><code>api: {
   *     create  : '/controller/new',
   *     read    : '/controller/load',
   *     update  : '/controller/update',
   *     destroy : '/controller/destroy_action'
   * }
   * </code></pre>
   *
   * <p>If the specific URL for a given CRUD action is undefined, the CRUD action request will be directed to the
   * configured <a href="#!/api/Ext.data.proxy.Server-cfg-url" rel="Ext.data.proxy.Server-cfg-url" class="docClass">url</a>.</p>
   * @see #getApi()
   * @see #setApi()
   */
  public native function get api():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set api(value:Object):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The name of the cache param added to the url when using noCache. Defaults to "_dc".
   * @default "_dc"
   * @see #getCacheString()
   * @see #setCacheString()
   */
  public native function get cacheString():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set cacheString(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The name of the direction parameter to send in a request. <strong>This is only used when simpleSortMode is set to
   * true.</strong>
   * @default "dir"
   * @see #getDirectionParam()
   * @see #setDirectionParam()
   */
  public native function get directionParam():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set directionParam(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Extra parameters that will be included on every request. Individual requests with params of the same name
   * will override these params when they are in conflict.
   * @default {}
   * @see #getExtraParams()
   * @see #setExtraParams()
   */
  public native function get extraParams():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set extraParams(value:Object):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The name of the 'filter' parameter to send in a request. Defaults to 'filter'. Set this to <code>''</code> if you don't
   * want to send a filter parameter.
   * @default "filter"
   * @see #getFilterParam()
   * @see #setFilterParam()
   */
  public native function get filterParam():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set filterParam(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The name of the direction parameter to send in a request. <strong>This is only used when simpleGroupMode is set to
   * true.</strong>
   * @default "groupDir"
   * @see #getGroupDirectionParam()
   * @see #setGroupDirectionParam()
   */
  public native function get groupDirectionParam():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set groupDirectionParam(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The name of the 'group' parameter to send in a request. Defaults to 'group'. Set this to <code>''</code> if you don't
   * want to send a group parameter.
   * @default "group"
   * @see #getGroupParam()
   * @see #setGroupParam()
   */
  public native function get groupParam():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set groupParam(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The name of the parameter which carries the id of the entity being operated upon.
   * @default "id"
   * @see #getIdParam()
   * @see #setIdParam()
   */
  public native function get idParam():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set idParam(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The name of the 'limit' parameter to send in a request. Defaults to 'limit'. Set this to <code>''</code> if you don't
   * want to send a limit parameter.
   * @default "limit"
   * @see #getLimitParam()
   * @see #setLimitParam()
   */
  public native function get limitParam():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set limitParam(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Disable caching by adding a unique parameter name to the request. Set to false to allow caching. Defaults to true.
   * @default true
   * @see #getNoCache()
   * @see #setNoCache()
   */
  public native function get noCache():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set noCache(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The name of the 'page' parameter to send in a request. Defaults to 'page'. Set this to <code>''</code> if you don't
   * want to send a page parameter.
   * @default "page"
   * @see #getPageParam()
   * @see #setPageParam()
   */
  public native function get pageParam():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set pageParam(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Enabling simpleGroupMode in conjunction with remoteGroup will only send one group property and a direction when a
   * remote group is requested. The <a href="#!/api/Ext.data.proxy.Server-cfg-groupDirectionParam" rel="Ext.data.proxy.Server-cfg-groupDirectionParam" class="docClass">groupDirectionParam</a> and <a href="#!/api/Ext.data.proxy.Server-cfg-groupParam" rel="Ext.data.proxy.Server-cfg-groupParam" class="docClass">groupParam</a> will be sent with the property name and either 'ASC'
   * or 'DESC'.
   * @default false
   * @see #getSimpleGroupMode()
   * @see #setSimpleGroupMode()
   */
  public native function get simpleGroupMode():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set simpleGroupMode(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Enabling simpleSortMode in conjunction with remoteSort will only send one sort property and a direction when a
   * remote sort is requested. The <a href="#!/api/Ext.data.proxy.Server-cfg-directionParam" rel="Ext.data.proxy.Server-cfg-directionParam" class="docClass">directionParam</a> and <a href="#!/api/Ext.data.proxy.Server-cfg-sortParam" rel="Ext.data.proxy.Server-cfg-sortParam" class="docClass">sortParam</a> will be sent with the property name
   * and either 'ASC' or 'DESC'.
   * @default false
   * @see #getSimpleSortMode()
   * @see #setSimpleSortMode()
   */
  public native function get simpleSortMode():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set simpleSortMode(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The name of the 'sort' parameter to send in a request. Defaults to 'sort'. Set this to <code>''</code> if you don't
   * want to send a sort parameter.
   * @default "sort"
   * @see #getSortParam()
   * @see #setSortParam()
   */
  public native function get sortParam():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set sortParam(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The name of the 'start' parameter to send in a request. Defaults to 'start'. Set this to <code>''</code> if you don't
   * want to send a start parameter.
   * @default "start"
   * @see #getStartParam()
   * @see #setStartParam()
   */
  public native function get startParam():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set startParam(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The number of milliseconds to wait for a response. Defaults to 30000 milliseconds (30 seconds).
   * @default 30000
   * @see #getTimeout()
   * @see #setTimeout()
   */
  public native function get timeout():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set timeout(value:Number):void;

  [ExtConfig]
  /**
   * The URL from which to request the data object.
   * @default ''
   * @see #setUrl()
   */
  public native function get url():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set url(value:String):void;

  /**
   * Creates an <a href="#!/api/Ext.data.Request" rel="Ext.data.Request" class="docClass">Request</a> object from <a href="#!/api/Ext.data.operation.Operation" rel="Ext.data.operation.Operation" class="docClass">Operation</a>.
   *
   * <p>This gets called from doRequest methods in subclasses of Server proxy.</p>
   * @param operation The operation to execute
   * @return The request object
   */
  public native function buildRequest(operation:ext.data.operation.Operation):ext.data.Request;

  /**
   * Generates a url based on a given <a href="#!/api/Ext.data.Request" rel="Ext.data.Request" class="docClass">Ext.data.Request</a> object. By default, ServerProxy's buildUrl will add the
   * cache-buster param to the end of the url. Subclasses may need to perform additional modifications to the url.
   * @param request The request object
   * @return The url
   */
  public native function buildUrl(request:ext.data.Request):String;

  /**
   * In ServerProxy subclasses, the <a href="#!/api/Ext.data.proxy.Server-method-create" rel="Ext.data.proxy.Server-method-create" class="docClass">create</a>, <a href="#!/api/Ext.data.proxy.Server-method-read" rel="Ext.data.proxy.Server-method-read" class="docClass">read</a>, <a href="#!/api/Ext.data.proxy.Server-method-update" rel="Ext.data.proxy.Server-method-update" class="docClass">update</a> and <a href="#!/api/Ext.data.proxy.Server-method-erase" rel="Ext.data.proxy.Server-method-erase" class="docClass">erase</a> methods all
   * pass through to doRequest. Each ServerProxy subclass must implement the doRequest method - see <a href="#!/api/Ext.data.proxy.JsonP" rel="Ext.data.proxy.JsonP" class="docClass">Ext.data.proxy.JsonP</a> and <a href="#!/api/Ext.data.proxy.Ajax" rel="Ext.data.proxy.Ajax" class="docClass">Ext.data.proxy.Ajax</a> for examples. This method carries the same signature as
   * each of the methods that delegate to it.
   * @param operation The <a href="#!/api/Ext.data.operation.Operation" rel="Ext.data.operation.Operation" class="docClass">Ext.data.operation.Operation</a> object
   * @param callback The callback function to call when the Operation has completed
   * @param scope The scope in which to execute the callback
   */
  public native function doRequest(operation:ext.data.operation.Operation, callback:Function, scope:Object):void;

  /**
   * Encodes the array of <a href="#!/api/Ext.util.Filter" rel="Ext.util.Filter" class="docClass">Ext.util.Filter</a> objects into a string to be sent in the request url. By default,
   * this simply JSON-encodes the filter data
   * @param filters The array of <a href="#!/api/Ext.util.Filter" rel="Ext.util.Filter" class="docClass">Filter</a> objects
   * @return The encoded filters
   */
  public native function encodeFilters(filters:Array):String;

  /**
   * Encodes the array of <a href="#!/api/Ext.util.Sorter" rel="Ext.util.Sorter" class="docClass">Ext.util.Sorter</a> objects into a string to be sent in the request url. By default,
   * this simply JSON-encodes the sorter data
   * @param sorters The array of <a href="#!/api/Ext.util.Sorter" rel="Ext.util.Sorter" class="docClass">Sorter</a> objects
   * @param preventArray Prevents the items from being output as an array.
   * @default false
   * @return The encoded sorters
   */
  public native function encodeSorters(sorters:Array, preventArray:Boolean = false):String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.proxy.Server-cfg-api" rel="Ext.data.proxy.Server-cfg-api" class="docClass">api</a>.
   */
  public native function getApi():Object;

  /**
   * Returns the value of <a href="#!/api/Ext.data.proxy.Server-cfg-cacheString" rel="Ext.data.proxy.Server-cfg-cacheString" class="docClass">cacheString</a>.
   */
  public native function getCacheString():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.proxy.Server-cfg-directionParam" rel="Ext.data.proxy.Server-cfg-directionParam" class="docClass">directionParam</a>.
   */
  public native function getDirectionParam():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.proxy.Server-cfg-extraParams" rel="Ext.data.proxy.Server-cfg-extraParams" class="docClass">extraParams</a>.
   */
  public native function getExtraParams():Object;

  /**
   * Returns the value of <a href="#!/api/Ext.data.proxy.Server-cfg-filterParam" rel="Ext.data.proxy.Server-cfg-filterParam" class="docClass">filterParam</a>.
   */
  public native function getFilterParam():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.proxy.Server-cfg-groupDirectionParam" rel="Ext.data.proxy.Server-cfg-groupDirectionParam" class="docClass">groupDirectionParam</a>.
   */
  public native function getGroupDirectionParam():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.proxy.Server-cfg-groupParam" rel="Ext.data.proxy.Server-cfg-groupParam" class="docClass">groupParam</a>.
   */
  public native function getGroupParam():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.proxy.Server-cfg-idParam" rel="Ext.data.proxy.Server-cfg-idParam" class="docClass">idParam</a>.
   */
  public native function getIdParam():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.proxy.Server-cfg-limitParam" rel="Ext.data.proxy.Server-cfg-limitParam" class="docClass">limitParam</a>.
   */
  public native function getLimitParam():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.proxy.Server-cfg-noCache" rel="Ext.data.proxy.Server-cfg-noCache" class="docClass">noCache</a>.
   */
  public native function getNoCache():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.data.proxy.Server-cfg-pageParam" rel="Ext.data.proxy.Server-cfg-pageParam" class="docClass">pageParam</a>.
   */
  public native function getPageParam():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.proxy.Server-cfg-simpleGroupMode" rel="Ext.data.proxy.Server-cfg-simpleGroupMode" class="docClass">simpleGroupMode</a>.
   */
  public native function getSimpleGroupMode():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.data.proxy.Server-cfg-simpleSortMode" rel="Ext.data.proxy.Server-cfg-simpleSortMode" class="docClass">simpleSortMode</a>.
   */
  public native function getSimpleSortMode():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.data.proxy.Server-cfg-sortParam" rel="Ext.data.proxy.Server-cfg-sortParam" class="docClass">sortParam</a>.
   */
  public native function getSortParam():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.proxy.Server-cfg-startParam" rel="Ext.data.proxy.Server-cfg-startParam" class="docClass">startParam</a>.
   */
  public native function getStartParam():String;

  /**
   * Returns the value of <a href="#!/api/Ext.data.proxy.Server-cfg-timeout" rel="Ext.data.proxy.Server-cfg-timeout" class="docClass">timeout</a>.
   */
  public native function getTimeout():Number;

  /**
   * Sets the value of <a href="#!/api/Ext.data.proxy.Server-cfg-api" rel="Ext.data.proxy.Server-cfg-api" class="docClass">api</a>.
   * @param api The new value.
   */
  public native function setApi(api:Object):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.proxy.Server-cfg-cacheString" rel="Ext.data.proxy.Server-cfg-cacheString" class="docClass">cacheString</a>.
   * @param cacheString The new value.
   */
  public native function setCacheString(cacheString:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.proxy.Server-cfg-directionParam" rel="Ext.data.proxy.Server-cfg-directionParam" class="docClass">directionParam</a>.
   * @param directionParam The new value.
   */
  public native function setDirectionParam(directionParam:String):void;

  /**
   * Sets a value in the underlying <a href="#!/api/Ext.data.proxy.Server-cfg-extraParams" rel="Ext.data.proxy.Server-cfg-extraParams" class="docClass">extraParams</a>.
   * @param name The key for the new value
   * @param value The value
   */
  public native function setExtraParam(name:String, value:Object):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.proxy.Server-cfg-extraParams" rel="Ext.data.proxy.Server-cfg-extraParams" class="docClass">extraParams</a>.
   * @param extraParams The new value.
   */
  public native function setExtraParams(extraParams:Object):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.proxy.Server-cfg-filterParam" rel="Ext.data.proxy.Server-cfg-filterParam" class="docClass">filterParam</a>.
   * @param filterParam The new value.
   */
  public native function setFilterParam(filterParam:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.proxy.Server-cfg-groupDirectionParam" rel="Ext.data.proxy.Server-cfg-groupDirectionParam" class="docClass">groupDirectionParam</a>.
   * @param groupDirectionParam The new value.
   */
  public native function setGroupDirectionParam(groupDirectionParam:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.proxy.Server-cfg-groupParam" rel="Ext.data.proxy.Server-cfg-groupParam" class="docClass">groupParam</a>.
   * @param groupParam The new value.
   */
  public native function setGroupParam(groupParam:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.proxy.Server-cfg-idParam" rel="Ext.data.proxy.Server-cfg-idParam" class="docClass">idParam</a>.
   * @param idParam The new value.
   */
  public native function setIdParam(idParam:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.proxy.Server-cfg-limitParam" rel="Ext.data.proxy.Server-cfg-limitParam" class="docClass">limitParam</a>.
   * @param limitParam The new value.
   */
  public native function setLimitParam(limitParam:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.proxy.Server-cfg-noCache" rel="Ext.data.proxy.Server-cfg-noCache" class="docClass">noCache</a>.
   * @param noCache The new value.
   */
  public native function setNoCache(noCache:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.proxy.Server-cfg-pageParam" rel="Ext.data.proxy.Server-cfg-pageParam" class="docClass">pageParam</a>.
   * @param pageParam The new value.
   */
  public native function setPageParam(pageParam:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.proxy.Server-cfg-simpleGroupMode" rel="Ext.data.proxy.Server-cfg-simpleGroupMode" class="docClass">simpleGroupMode</a>.
   * @param simpleGroupMode The new value.
   */
  public native function setSimpleGroupMode(simpleGroupMode:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.proxy.Server-cfg-simpleSortMode" rel="Ext.data.proxy.Server-cfg-simpleSortMode" class="docClass">simpleSortMode</a>.
   * @param simpleSortMode The new value.
   */
  public native function setSimpleSortMode(simpleSortMode:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.proxy.Server-cfg-sortParam" rel="Ext.data.proxy.Server-cfg-sortParam" class="docClass">sortParam</a>.
   * @param sortParam The new value.
   */
  public native function setSortParam(sortParam:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.proxy.Server-cfg-startParam" rel="Ext.data.proxy.Server-cfg-startParam" class="docClass">startParam</a>.
   * @param startParam The new value.
   */
  public native function setStartParam(startParam:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.proxy.Server-cfg-timeout" rel="Ext.data.proxy.Server-cfg-timeout" class="docClass">timeout</a>.
   * @param timeout The new value.
   */
  public native function setTimeout(timeout:Number):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.proxy.Server-cfg-url" rel="Ext.data.proxy.Server-cfg-url" class="docClass">url</a>.
   * @param url The new value.
   */
  public native function setUrl(url:String):void;

  /**
   * Optional callback function which can be used to clean up after a request has been completed.
   * @param request The Request object
   * @param success True if the request was successful
   */
  protected native function afterRequest(request:ext.data.Request, success:Boolean):void;

  /**
   * Encode any values being sent to the server. Can be overridden in subclasses.
   * @param value An array of sorters/filters.
   * @return The encoded value
   */
  protected native function applyEncoding(value:Array):Object;

  /**
   * Processes response, which may involve updating or committing records, each of which
   * will inform the owning stores and their interested views. Finally, we may perform
   * an additional layout if the data shape has changed.
   */
  protected native function processResponse(success:Object, operation:Object, request:Object, response:Object):void;
}
}