package ext.data.reader {

[Uses("ext.data.reader.JsonReader")]
[Uses("ext.data.reader.DataReader")]
[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Uses("ext.mixin.Factoryable")]
[Native("Ext.data.reader.Array", require)]
/**
 * Data reader class to create an Array of <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a> objects from an Array.
 * Each element of that Array represents a row of data fields. The
 * fields are pulled into a Record object using as a subscript, the <code>mapping</code> property
 * of the field definition if it exists, or the field's ordinal position in the definition.
 *
 *
 *
 *
 * <p><u>Example code:</u></p>
 *
 *
 *
 *
 * <pre><code>Employee = <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Employee', {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     fields: [
 *         'id',
 *         {name: 'name', mapping: 1},         // "mapping" only needed if an "id" field is present which
 *         {name: 'occupation', mapping: 2}    // precludes using the ordinal position as the index.
 *     ]
 * });
 *
 * var myReader = new <a href="#!/api/Ext.data.reader.Array" rel="Ext.data.reader.Array" class="docClass">Ext.data.reader.Array</a>({
 *     model: 'Employee'
 * }, Employee);
 * </code></pre>
 *
 *
 *
 *
 * <p>This would consume an Array like this:</p>
 *
 *
 *
 *
 * <pre><code>[ [1, 'Bill', 'Gardener'], [2, 'Ben', 'Horticulturalist'] ]
 * </code></pre>
 */
public class ArrayReader extends ext.data.reader.JsonReader {
  /**
   * Create a new ArrayReader
   * @param meta Metadata configuration options.
   */
  public function ArrayReader(meta:Object) {
    super();
  }
}
}