package ext.data.schema {

[Uses("ext.data.schema.Association")]
[Uses("ext.Base")]
[Native("Ext.data.schema.ManyToMany", require)]
/**
 * This relationship describes the case where any one entity of one type may relate to any
 * number of entities of another type, and also in the reverse.
 *
 * <p>This form of association cannot store id's in the related entities since that would
 * limit the number of related entities to one for the entity with the foreign key. Instead,
 * these relationships are typically implemented using a so-called "matrix" table. This
 * table typically has two columns to hold the id's of a pair of related entities. This
 * pair of id's is unique in the matrix table.</p>
 *
 * <h1>Declaration Forms</h1>
 *
 * <pre><code> // Fully spelled out - all properties are their defaults:
 *
 *  <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('App.models.Group', {
 *      extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *
 *      manyToMany: {
 *          UserGroups: {
 *              type: 'User',
 *              role: 'users',
 *              field: 'userId',
 *              right: {
 *                  field: 'groupId',
 *                  role: 'groups'
 *              }
 *          }
 *      }
 *  });
 *
 *  // Eliminate "right" object and use boolean to indicate Group is on the
 *  // right. By default, left/right is determined by alphabetic order.
 *
 *  <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('App.models.Group', {
 *      extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *
 *      manyToMany: {
 *          UserGroups: {
 *              type: 'User',
 *              role: 'users',
 *              field: 'userId',
 *              right: true
 *          }
 *      }
 *  });
 *
 *  // Eliminate object completely and rely on string to name the other type. Still
 *  // keep Group on the "right".
 *
 *  <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('App.models.Group', {
 *      extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *
 *      manyToMany: {
 *          UserGroups: 'User#'   // '#' is on the side (left or right) of Group
 *      }
 *  });
 *
 *  // Remove explicit matrix name and keep Group on the "right". Generated matrixName
 *  // remains "UserGroups".
 *
 *  <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('App.models.Group', {
 *      extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *
 *      manyToMany: [
 *          'User#'
 *      ]
 *  });
 *
 *  // Minimal definition but now Group is on the "left" since "Group" sorts before
 *  // "User". Generated matrixName is now "GroupUsers".
 *
 *  <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('App.models.Group', {
 *      extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *
 *      manyToMany: [
 *          'User'
 *      ]
 *  });
 * </code></pre>
 */
public class ManyToMany extends ext.data.schema.Association {
  /**
   * @inheritDoc
   */
  public function ManyToMany(config:ext.data.schema.Association = null) {
    super();
  }
}
}