package ext.data.schema {

[Uses("ext.data.schema.Association")]
[Uses("ext.Base")]
[Native("Ext.data.schema.OneToOne", require)]
/**
 * <strong>This class is never created directly. It should be constructed through associations in <code><a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a></code>.</strong>
 *
 * <p>This is a specialized version of <code><a href="#!/api/Ext.data.schema.ManyToOne" rel="Ext.data.schema.ManyToOne" class="docClass">Ext.data.schema.ManyToOne</a></code> that declares a relationship between a single
 * entity type and a single related entities. The relationship can be declared as a keyed or keyless relationship.</p>
 *
 * <pre><code>// Keyed
 * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('User', {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     fields: ['id', 'name', {
 *         name: 'userInfoId',
 *         reference: {
 *             type: 'UserInfo',
 *             unique: true
 *         }
 *     }]
 * });
 *
 * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('UserInfo', {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     fields: ['id', 'secretKey']
 * });
 *
 * // Keyless
 * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('User', {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     fields: ['id', 'name'],
 *     hasOne: 'UserInfo'
 * });
 *
 * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Ticket', {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     fields: ['id', 'secretKey']
 * });
 *
 * // Generated methods
 * var user = new User();
 * user.getUserInfo();
 * user.setUserInfo();
 *
 * var info = new UserInfo();
 * info.getUser();
 * info.setUser();
 *
 *
 * var ticket = new Ticket();
 * ticket.setCustomer(customer);
 * console.log(ticket.getCustomer()); // The customer object
 * </code></pre>
 *
 * <p>By declaring a keyed relationship, extra functionality is gained that maintains
 * the key field in the model as changes are made to the association.</p>
 *
 * <p>For available configuration options, see <a href="#!/api/Ext.data.schema.Reference" rel="Ext.data.schema.Reference" class="docClass">Ext.data.schema.Reference</a>.
 * Each record type will have a <a href="#!/api/Ext.data.schema.Association-method-recordGetter" rel="Ext.data.schema.Association-method-recordGetter" class="docClass">getter</a> and <a href="#!/api/Ext.data.schema.Association-method-recordSetter" rel="Ext.data.schema.Association-method-recordSetter" class="docClass">setter</a>.</p>
 */
public class OneToOne extends ext.data.schema.Association {
  /**
   * @inheritDoc
   */
  public function OneToOne(config:ext.data.schema.Association = null) {
    super();
  }
}
}