package ext.data.writer {

[Uses("ext.data.writer.DataWriter")]
[Uses("ext.mixin.Factoryable")]
[Uses("ext.Base")]
[Native("Ext.data.writer.Json", require)]
/**
 * This class is used to write <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a> data to the server in a JSON format.
 * The <a href="#!/api/Ext.data.writer.Json-cfg-allowSingle" rel="Ext.data.writer.Json-cfg-allowSingle" class="docClass">allowSingle</a> configuration can be set to false to force the records to always
 * be encoded in an array, even if there is only a single record being sent.
 */
public class JsonWriter extends ext.data.writer.DataWriter {
  /**
   * @param config @inheritDoc
   */
  public function JsonWriter(config:ext.data.writer.JsonWriter = null) {
    super();
  }

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Configure with <code>false</code> to ensure that records are always wrapped in an array, even if there is only
   * one record being sent. When there is more than one record, they will always be encoded into an array.
   * @default true
   * @see #getAllowSingle()
   * @see #setAllowSingle()
   */
  public native function get allowSingle():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set allowSingle(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Configure <code>true</code> to send record data (all record fields if <a href="#!/api/Ext.data.writer.Json-cfg-writeAllFields" rel="Ext.data.writer.Json-cfg-writeAllFields" class="docClass">writeAllFields</a> is <code>true</code>)
   * as a JSON encoded HTTP parameter named by the <a href="#!/api/Ext.data.writer.Json-cfg-rootProperty" rel="Ext.data.writer.Json-cfg-rootProperty" class="docClass">rootProperty</a> configuration.
   *
   * <p>The encode option should only be set to true when a <a href="#!/api/Ext.data.writer.Json-cfg-rootProperty" rel="Ext.data.writer.Json-cfg-rootProperty" class="docClass">rootProperty</a> is defined, because the values will be
   * sent as part of the request parameters as opposed to a raw post. The root will be the name of the parameter
   * sent to the server.</p>
   * @default false
   * @see #getEncode()
   * @see #setEncode()
   */
  public native function get encode():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set encode(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * By default, when dot-delimited field <a href="#!/api/Ext.data.writer.Json-cfg-nameProperty" rel="Ext.data.writer.Json-cfg-nameProperty" class="docClass">mappings</a> are
   * used (e.g. <code>name: 'myProperty', mapping: 'my.nested.property'</code>) the writer will simply output a flat data
   * object containing the mapping string literal as the property name (e.g. <code>{ 'my.nested.property': 'foo' }</code>).
   *
   * <p>Mappings are used to map incoming nested JSON to flat Ext models. In many case, the data output by the
   * writer should preferrably match the original nested data format. Setting this config to <code>true</code> will ensure
   * that the output will instead look like <code>{ my: { nested: { property: 'foo' }}}</code>. The output is generated
   * by <a href="#!/api/Ext.data.writer.Json-method-getExpandedData" rel="Ext.data.writer.Json-method-getExpandedData" class="docClass">getExpandedData</a>, which can optionally be overridden to apply more customized logic.</p>
   * @default false
   * @see #getExpandData()
   * @see #setExpandData()
   */
  public native function get expandData():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set expandData(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The HTTP parameter name by which JSON encoded records will be passed to the server if the
   * <a href="#!/api/Ext.data.writer.Json-cfg-encode" rel="Ext.data.writer.Json-cfg-encode" class="docClass">encode</a> option is <code>true</code>.
   * @see #getRootProperty()
   * @see #setRootProperty()
   */
  public native function get rootProperty():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set rootProperty(value:String):void;

  /**
   * Returns the value of <a href="#!/api/Ext.data.writer.Json-cfg-allowSingle" rel="Ext.data.writer.Json-cfg-allowSingle" class="docClass">allowSingle</a>.
   */
  public native function getAllowSingle():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.data.writer.Json-cfg-encode" rel="Ext.data.writer.Json-cfg-encode" class="docClass">encode</a>.
   */
  public native function getEncode():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.data.writer.Json-cfg-expandData" rel="Ext.data.writer.Json-cfg-expandData" class="docClass">expandData</a>.
   */
  public native function getExpandData():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.data.writer.Json-cfg-rootProperty" rel="Ext.data.writer.Json-cfg-rootProperty" class="docClass">rootProperty</a>.
   */
  public native function getRootProperty():String;

  /**
   * Sets the value of <a href="#!/api/Ext.data.writer.Json-cfg-allowSingle" rel="Ext.data.writer.Json-cfg-allowSingle" class="docClass">allowSingle</a>.
   * @param allowSingle The new value.
   */
  public native function setAllowSingle(allowSingle:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.writer.Json-cfg-encode" rel="Ext.data.writer.Json-cfg-encode" class="docClass">encode</a>.
   * @param encode The new value.
   */
  public native function setEncode(encode:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.writer.Json-cfg-expandData" rel="Ext.data.writer.Json-cfg-expandData" class="docClass">expandData</a>.
   * @param expandData The new value.
   */
  public native function setExpandData(expandData:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.data.writer.Json-cfg-rootProperty" rel="Ext.data.writer.Json-cfg-rootProperty" class="docClass">rootProperty</a>.
   * @param rootProperty The new value.
   */
  public native function setRootProperty(rootProperty:String):void;

  /**
   * The Reader classes support dot-delimited data mappings for extracting nested raw data into fields, so the
   * writer must support converting the flat <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a> structure back into the original nested data
   * format. Using the same mappings when available, the Writer will simply split each delimiter into a nested
   * object in the output, which should exactly match the input format. For example, record data like this:
   *
   * <pre><code>my.nested.property: 'foo',
   * my.nested.another: 'bar',
   * my.somethingElse: 123
   * </code></pre>
   *
   * <p>should write out as...</p>
   *
   * <pre><code>my: {
   *     nested: {
   *         property: 'foo',
   *         another: 'bar
   *     },
   *     somethingElse: 123
   * }
   * </code></pre>
   *
   * <p>This behavior is governed by the <a href="#!/api/Ext.data.writer.Json-cfg-expandData" rel="Ext.data.writer.Json-cfg-expandData" class="docClass">expandData</a> config. By default, this option is <code>false</code> for
   * compatibility reasons, and will output a flat structure matching the flat record format. Setting this config
   * to <code>true</code> will enable the expanded mapping behavior as shown here. This method could also be overridden
   * to provide an even more customized output data structure.</p>
   */
  protected native function getExpandedData(data:Object):Object;
}
}