package ext.dd {
import ext.event.Event;

[Uses("ext.dd.DDProxy")]
[Uses("ext.dd.DD")]
[Uses("ext.dd.DragDrop")]
[Uses("ext.Base")]
[Native("Ext.dd.DragSource", require)]
/**
 * A simple class that provides the basic implementation needed to make any element draggable.
 */
public class DragSource extends ext.dd.DDProxy {
  /**
   * Creates new drag-source.
   * @param el The container element or ID of it.
   * @param config Config object.
   */
  public function DragSource(el:*, config:ext.dd.DragSource = null) {
    super(null, null, null);
  }

  /**
   * This property contains the data representing the dragged object. This data is set up by the implementation of the
   * <a href="#!/api/Ext.dd.DragSource-method-getDragData" rel="Ext.dd.DragSource-method-getDragData" class="docClass">getDragData</a> method. It must contain a ddel property, but can contain any other data according to the
   * application's needs.
   */
  public native function get dragData():Object;

  /**
   * @private
   */
  public native function set dragData(value:Object):void;

  [ExtConfig]
  /**
   * If true, animates the proxy element back to the position of the handle element used to trigger the drag.
   * @default true
   */
  public native function get animRepair():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set animRepair(value:Boolean):void;

  [ExtConfig]
  /**
   * A named drag drop group to which this object belongs.  If a group is specified, then this object will only
   * interact with other drag drop objects in the same group.
   */
  public native function get ddGroup():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set ddGroup(value:String):void;

  [ExtConfig]
  /**
   * The CSS class returned to the drag source when drop is allowed.
   * @default Ext.baseCSSPrefix + 'dd-drop-ok'
   */
  public native function get dropAllowed():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set dropAllowed(value:String):void;

  [ExtConfig]
  /**
   * The CSS class returned to the drag source when drop is not allowed.
   * @default Ext.baseCSSPrefix + 'dd-drop-nodrop'
   */
  public native function get dropNotAllowed():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set dropNotAllowed(value:String):void;

  [ExtConfig]
  /**
   * The color to use when visually highlighting the drag source in the afterRepair
   * method after a failed drop (defaults to light blue). The color must be a 6 digit hex value, without
   * a preceding '#'.
   * @default 'c3daf9'
   */
  public native function get repairHighlightColor():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set repairHighlightColor(value:String):void;

  /**
   * An empty function by default, but provided so that you can perform a custom action
   * after a valid drag drop has occurred by providing an implementation.
   * @param target The drop target
   * @param e The event object
   * @param id The id of the dropped element
   */
  public native function afterDragDrop(target:ext.dd.DragDrop, e:ext.event.Event, id:String):void;

  /**
   * An empty function by default, but provided so that you can perform a custom action
   * when the dragged item enters the drop target by providing an implementation.
   * @param target The drop target
   * @param e The event object
   * @param id The id of the dragged element
   */
  public native function afterDragEnter(target:ext.dd.DragDrop, e:ext.event.Event, id:String):void;

  /**
   * An empty function by default, but provided so that you can perform a custom action
   * after the dragged item is dragged out of the target without dropping.
   * @param target The drop target
   * @param e The event object
   * @param id The id of the dragged element
   */
  public native function afterDragOut(target:ext.dd.DragDrop, e:ext.event.Event, id:String):void;

  /**
   * An empty function by default, but provided so that you can perform a custom action
   * while the dragged item is over the drop target by providing an implementation.
   * @param target The drop target
   * @param e The event object
   * @param id The id of the dragged element
   */
  public native function afterDragOver(target:ext.dd.DragDrop, e:ext.event.Event, id:String):void;

  /**
   * An empty function by default, but provided so that you can perform a custom action
   * after an invalid drop has occurred by providing an implementation.
   * @param e The event object
   * @param id The id of the dropped element
   */
  public native function afterInvalidDrop(e:ext.event.Event, id:String):void;

  /**
   * An empty function by default, but provided so that you can perform a custom action
   * after a valid drop has occurred by providing an implementation.
   * @param target The target DD
   * @param e The event object
   * @param id The id of the dropped element
   */
  public native function afterValidDrop(target:Object, e:ext.event.Event, id:String):void;

  /**
   * An empty function by default, but provided so that you can perform a custom action before the dragged
   * item is dropped onto the target and optionally cancel the onDragDrop.
   * @param target The drop target
   * @param e The event object
   * @param id The id of the dragged element
   * @return isValid True if the drag drop event is valid, else false to cancel
   */
  public native function beforeDragDrop(target:ext.dd.DragDrop, e:ext.event.Event, id:String):Boolean;

  /**
   * An empty function by default, but provided so that you can perform a custom action
   * before the dragged item enters the drop target and optionally cancel the onDragEnter.
   * @param target The drop target
   * @param e The event object
   * @param id The id of the dragged element
   * @return isValid True if the drag event is valid, else false to cancel
   */
  public native function beforeDragEnter(target:ext.dd.DragDrop, e:ext.event.Event, id:String):Boolean;

  /**
   * An empty function by default, but provided so that you can perform a custom action before the dragged
   * item is dragged out of the target without dropping, and optionally cancel the onDragOut.
   * @param target The drop target
   * @param e The event object
   * @param id The id of the dragged element
   * @return isValid True if the drag event is valid, else false to cancel
   */
  public native function beforeDragOut(target:ext.dd.DragDrop, e:ext.event.Event, id:String):Boolean;

  /**
   * An empty function by default, but provided so that you can perform a custom action
   * while the dragged item is over the drop target and optionally cancel the onDragOver.
   * @param target The drop target
   * @param e The event object
   * @param id The id of the dragged element
   * @return isValid True if the drag event is valid, else false to cancel
   */
  public native function beforeDragOver(target:ext.dd.DragDrop, e:ext.event.Event, id:String):Boolean;

  /**
   * An empty function by default, but provided so that you can perform a custom action after an invalid
   * drop has occurred.
   * @param target The drop target
   * @param e The event object
   * @param id The id of the dragged element
   * @return isValid True if the invalid drop should proceed, else false to cancel
   */
  public native function beforeInvalidDrop(target:ext.dd.DragDrop, e:ext.event.Event, id:String):Boolean;

  /**
   * Returns the data object associated with this drag source
   * @return data An object containing arbitrary data
   */
  public native function getDragData(e:ext.event.Event):Object;

  /**
   * Returns the drag source's underlying <a href="#!/api/Ext.dd.StatusProxy" rel="Ext.dd.StatusProxy" class="docClass">Ext.dd.StatusProxy</a>
   * @return proxy The StatusProxy
   */
  public native function getProxy():ext.dd.StatusProxy;

  /**
   * Hides the drag source's <a href="#!/api/Ext.dd.StatusProxy" rel="Ext.dd.StatusProxy" class="docClass">Ext.dd.StatusProxy</a>
   */
  public native function hideProxy():void;

  /**
   * An empty function by default, but provided so that you can perform a custom action before the initial
   * drag event begins and optionally cancel it.
   * @param data An object containing arbitrary data to be shared with drop targets
   * @param e The event object
   * @return isValid True if the drag event is valid, else false to cancel
   */
  public native function onBeforeDrag(data:Object, e:ext.event.Event):Boolean;

  /**
   * An empty function by default, but provided so that you can perform a custom action once the initial
   * drag event has begun.  The drag cannot be canceled from this function.
   * @param x The x position of the click on the dragged object
   * @param y The y position of the click on the dragged object
   */
  public native function onStartDrag(x:Number, y:Number):void;
}
}