package ext.dd {
import ext.dom.Element;
import ext.event.Event;
import ext.mixin.Observable;
import js.HTMLElement;

[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Native("Ext.dd.DragTracker", require)]
/**
 * @eventType ext.dd.events.DragTracker_eEvent.BEFORE_START
 */
[Event(name = 'onBeforeStart', type = 'ext.dd.events.DragTracker_eEvent')]
/**
 * @eventType ext.dd.events.DragTracker_eEvent.DRAG
 */
[Event(name = 'onDrag', type = 'ext.dd.events.DragTracker_eEvent')]
/**
 * @eventType ext.dd.events.DragTracker_eEvent.DRAG_END
 */
[Event(name = 'onDragEnd', type = 'ext.dd.events.DragTracker_eEvent')]
/**
 * @eventType ext.dd.events.DragTracker_eEvent.DRAG_START
 */
[Event(name = 'onDragStart', type = 'ext.dd.events.DragTracker_eEvent')]
/**
 * Fires when the mouse button is pressed down, but before a drag operation begins. The
 * drag operation begins after either the mouse has been moved by <a href="#!/api/Ext.dd.DragTracker-cfg-tolerance" rel="Ext.dd.DragTracker-cfg-tolerance" class="docClass">tolerance</a> pixels,
 * or after the <a href="#!/api/Ext.dd.DragTracker-cfg-autoStart" rel="Ext.dd.DragTracker-cfg-autoStart" class="docClass">autoStart</a> timer fires.
 *
 * <p>Return <code>false</code> to veto the drag operation.</p>
 * @eventType ext.dd.events.DragTracker_eEvent.MOUSE_DOWN
 */
[Event(name = 'onMouseDown', type = 'ext.dd.events.DragTracker_eEvent')]
/**
 * Fired when the mouse is moved. Returning false cancels the drag operation.
 * @eventType ext.dd.events.DragTracker_eEvent.MOUSE_MOVE
 */
[Event(name = 'onMouseMove', type = 'ext.dd.events.DragTracker_eEvent')]
/**
 * Fires when the mouse exits the DragTracker's target element (or if <a href="#!/api/Ext.dd.DragTracker-cfg-delegate" rel="Ext.dd.DragTracker-cfg-delegate" class="docClass">delegate</a> is
 * used, when the mouse exits a delegate element).
 *
 * <p><strong>Only available when <a href="#!/api/Ext.dd.DragTracker-cfg-trackOver" rel="Ext.dd.DragTracker-cfg-trackOver" class="docClass">trackOver</a> is <code>true</code></strong></p>
 * @eventType ext.dd.events.DragTracker_eEvent.MOUSE_OUT
 */
[Event(name = 'onMouseOut', type = 'ext.dd.events.DragTracker_eEvent')]
/**
 * Fires when the mouse enters the DragTracker's target element (or if <a href="#!/api/Ext.dd.DragTracker-cfg-delegate" rel="Ext.dd.DragTracker-cfg-delegate" class="docClass">delegate</a> is
 * used, when the mouse enters a delegate element).
 *
 * <p><strong>Only available when <a href="#!/api/Ext.dd.DragTracker-cfg-trackOver" rel="Ext.dd.DragTracker-cfg-trackOver" class="docClass">trackOver</a> is <code>true</code></strong></p>
 * @eventType ext.dd.events.DragTracker_e_targetEvent.MOUSE_OVER
 */
[Event(name = 'onMouseOver', type = 'ext.dd.events.DragTracker_e_targetEvent')]
/**
 * @eventType ext.dd.events.DragTracker_eEvent.MOUSE_UP
 */
[Event(name = 'onMouseUp', type = 'ext.dd.events.DragTracker_eEvent')]
/**
 * A DragTracker listens for drag events on an Element and fires events at the start and end of the drag,
 * as well as during the drag. This is useful for components such as <a href="#!/api/Ext.slider.Multi" rel="Ext.slider.Multi" class="docClass">Ext.slider.Multi</a>, where there is
 * an element that can be dragged around to change the Slider's value.
 *
 * <p>DragTracker provides a series of template methods that should be overridden to provide functionality
 * in response to detected drag operations. These are onBeforeStart, onStart, onDrag, onCancel and onEnd.
 * See <a href="#!/api/Ext.slider.Multi" rel="Ext.slider.Multi" class="docClass">Ext.slider.Multi</a>'s initEvents function for an example implementation.</p>
 */
public class DragTracker extends ext.mixin.Observable {
  /**
   * @param config @inheritDoc
   */
  public function DragTracker(config:ext.dd.DragTracker = null) {
    super();
  }

  /**
   * Indicates whether the user is currently dragging this tracker.
   * @default false
   */
  public native function get active():Boolean;

  /**
   * The element being dragged.
   *
   * <p>Only valid during drag operations.</p>
   *
   * <p>If the <a href="#!/api/Ext.dd.DragTracker-cfg-delegate" rel="Ext.dd.DragTracker-cfg-delegate" class="docClass">delegate</a> option is used, this will be the delegate element which was mousedowned.</p>
   */
  public native function get dragTarget():js.HTMLElement;

  [ExtConfig]
  /**
   * Specify <code>true</code> to defer trigger start by 1000 ms.
   * Specify a Number for the number of milliseconds to defer trigger start.
   * @default false
   */
  public native function get autoStart():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set autoStart(value:*):void;

  [ExtConfig]
  /**
   * A <a href="#!/api/Ext.util.Region" rel="Ext.util.Region" class="docClass">Region</a> (Or an element from which a Region measurement will be read)
   * which is used to constrain the result of the <a href="#!/api/Ext.dd.DragTracker-method-getOffset" rel="Ext.dd.DragTracker-method-getOffset" class="docClass">getOffset</a> call.
   *
   * <p>This may be set any time during the DragTracker's lifecycle to set a dynamic constraining region.</p>
   */
  public native function get constrainTo():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set constrainTo(value:*):void;

  [ExtConfig]
  /**
   * A CSS selector which identifies child elements within the DragTracker's encapsulating
   * Element which are the tracked elements. This limits tracking to only begin when the matching elements are mousedowned.
   *
   * <p>This may also be a specific child element within the DragTracker's encapsulating element to use as the tracked element.</p>
   */
  public native function get delegate():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set delegate(value:String):void;

  [ExtConfig]
  /**
   * The target element or ID of the element on which the DragTracker will be initialized.
   */
  public native function get el():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set el(value:*):void;

  [ExtConfig]
  /**
   * A CSS class to add to the DragTracker's target element when the element (or, if the <a href="#!/api/Ext.dd.DragTracker-cfg-delegate" rel="Ext.dd.DragTracker-cfg-delegate" class="docClass">delegate</a>
   * option is used, when a delegate element) is mouseovered.
   *
   * <p>If the <a href="#!/api/Ext.dd.DragTracker-cfg-delegate" rel="Ext.dd.DragTracker-cfg-delegate" class="docClass">delegate</a> option is used, these events fire only when a delegate element is entered of left.</p>
   */
  public native function get overCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set overCls(value:String):void;

  [ExtConfig]
  /**
   * Specify <code>false</code> to enable default actions on onMouseDown events.
   * @default true
   */
  public native function get preventDefault():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set preventDefault(value:Boolean):void;

  [ExtConfig]
  /**
   * Specify <code>true</code> to stop the <code>mousedown</code> event from bubbling to outer listeners from the target element (or its delegates).
   * @default false
   */
  public native function get stopEvent():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set stopEvent(value:Boolean):void;

  [ExtConfig]
  /**
   * Number of pixels the drag target must be moved before dragging is
   * considered to have started.
   * @default 5
   */
  public native function get tolerance():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set tolerance(value:Number):void;

  [ExtConfig]
  /**
   * Set to true to fire mouseover and mouseout events when the mouse enters or leaves the target element.
   *
   * <p>This is implicitly set when an <a href="#!/api/Ext.dd.DragTracker-cfg-overCls" rel="Ext.dd.DragTracker-cfg-overCls" class="docClass">overCls</a> is specified.</p>
   *
   * <p>If the <a href="#!/api/Ext.dd.DragTracker-cfg-delegate" rel="Ext.dd.DragTracker-cfg-delegate" class="docClass">delegate</a> option is used, these events fire only when a delegate element is entered of left.</p>
   * @default false
   */
  public native function get trackOver():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set trackOver(value:Boolean):void;

  /**
   * Returns the drag target. This is usually the DragTracker's encapsulating element.
   *
   * <p>If the <a href="#!/api/Ext.dd.DragTracker-cfg-delegate" rel="Ext.dd.DragTracker-cfg-delegate" class="docClass">delegate</a> option is being used, this may be a child element which matches the
   * <a href="#!/api/Ext.dd.DragTracker-cfg-delegate" rel="Ext.dd.DragTracker-cfg-delegate" class="docClass">delegate</a> selector.</p>
   * @return The element currently being tracked.
   */
  public native function getDragTarget():ext.dom.Element;

  /**
   * Returns the X, Y offset of the current mouse position from the mousedown point.
   *
   * <p>This method may optionally constrain the real offset values, and returns a point coerced in one
   * of two modes:</p>
   *
   * <ul>
   * <li><code>point</code>
   * The current mouse position is coerced into the constrainRegion and the resulting position is returned.</li>
   * <li><code>dragTarget</code>
   * The new <a href="#!/api/Ext.util.Region" rel="Ext.util.Region" class="docClass">Region</a> of the <a href="#!/api/Ext.dd.DragTracker-method-getDragTarget" rel="Ext.dd.DragTracker-method-getDragTarget" class="docClass">dragTarget</a> is calculated
   * based upon the current mouse position, and then coerced into the constrainRegion. The returned
   * mouse position is then adjusted by the same delta as was used to coerce the region.</li>
   * </ul>
   * @param constrainMode If omitted the true mouse position is returned. May be passed
   * as <code>point</code> or <code>dragTarget</code>. See above.
   * @return The <code>X, Y</code> offset from the mousedown point, optionally constrained.
   */
  public native function getOffset(constrainMode:String = null):Array;

  /**
   * Initializes the DragTracker on a given element.
   * @param el The element or element ID
   */
  public native function initEl(el:*):void;

  /**
   * Template method which should be overridden by each DragTracker instance. Called when the user first clicks and
   * holds the mouse button down. Return false to disallow the drag
   * @param e The event object
   */
  public native function onBeforeStart(e:ext.event.Event):void;

  /**
   * Template method which mey be overridden by each DragTracker instance. Called when a mouseup gesture is detected
   * but the onStart has not yet been reached. To clear things up that may have been set up on <a href="#!/api/Ext.dd.DragTracker-method-onBeforeStart" rel="Ext.dd.DragTracker-method-onBeforeStart" class="docClass">onBeforeStart</a>.
   * @param e The event object
   */
  public native function onCancel(e:ext.event.Event):void;

  /**
   * Template method which should be overridden by each DragTracker instance. Called whenever a drag has been detected.
   * @param e The event object
   */
  public native function onDrag(e:ext.event.Event):void;

  /**
   * Template method which should be overridden by each DragTracker instance. Called when a drag operation has been completed
   * (e.g. the user clicked and held the mouse down, dragged the element and then released the mouse button)
   * @param e The event object
   */
  public native function onEnd(e:ext.event.Event):void;

  /**
   * Template method which should be overridden by each DragTracker instance. Called when a drag operation starts
   * (e.g. the user has moved the tracked element beyond the specified tolerance)
   * @param e The event object
   */
  public native function onStart(e:ext.event.Event):void;
}
}