package ext.dd {
import ext.event.Event;

[Uses("ext.dd.DDTarget")]
[Uses("ext.dd.DragDrop")]
[Uses("ext.Base")]
[Native("Ext.dd.DropTarget", require)]
/**
 * A simple class that provides the basic implementation needed to make any element a drop target that can have
 * draggable items dropped onto it.  The drop has no effect until an implementation of notifyDrop is provided.
 */
public class DropTarget extends ext.dd.DDTarget {
  /**
   * Creates new DropTarget.
   * @param el The container element or ID of it.
   */
  public function DropTarget(el:*, config:ext.dd.DropTarget = null) {
    super(null, null, null);
  }

  [ExtConfig]
  /**
   * True to register this container with the ScrollManager for auto scrolling during
   * drag operations.
   * @default false
   */
  public native function get containerScroll():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set containerScroll(value:Boolean):void;

  [ExtConfig]
  /**
   * A named drag drop group to which this object belongs.  If a group is specified, then this object will only
   * interact with other drag drop objects in the same group.
   */
  public native function get ddGroup():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set ddGroup(value:String):void;

  [ExtConfig]
  /**
   * The CSS class returned to the drag source when drop is allowed.
   * @default Ext.baseCSSPrefix + 'dd-drop-ok'
   */
  public native function get dropAllowed():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set dropAllowed(value:String):void;

  [ExtConfig]
  /**
   * The CSS class returned to the drag source when drop is not allowed.
   * @default Ext.baseCSSPrefix + 'dd-drop-nodrop'
   */
  public native function get dropNotAllowed():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set dropNotAllowed(value:String):void;

  [ExtConfig]
  /**
   * The CSS class applied to the drop target element while the drag source is over it.
   * @default ""
   */
  public native function get overClass():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set overClass(value:String):void;

  /**
   * The function a <a href="#!/api/Ext.dd.DragSource" rel="Ext.dd.DragSource" class="docClass">Ext.dd.DragSource</a> calls once to notify this drop target that the dragged item has
   * been dropped on it. This method removes any <code>overClass</code> and returns false, so you must provide an
   * implementation that does something to process the drop event and returns true so that the drag source's
   * repair action does not run.
   *
   * <p>You should <code>callParent</code> from an override of this method to ensure proper cleanup is
   * performed.</p>
   * @param source The drag source that was dragged over this drop target
   * @param e The event
   * @param data An object containing arbitrary data supplied by the drag source
   * @return False if the drop was invalid.
   */
  public native function notifyDrop(source:ext.dd.DragSource, e:ext.event.Event, data:Object):Boolean;

  /**
   * The function a <a href="#!/api/Ext.dd.DragSource" rel="Ext.dd.DragSource" class="docClass">Ext.dd.DragSource</a> calls once to notify this drop target that the source is now over the
   * target.  This default implementation adds the CSS class specified by overClass (if any) to the drop element
   * and returns the dropAllowed config value.  This method should be overridden if drop validation is required.
   * @param source The drag source that was dragged over this drop target
   * @param e The event
   * @param data An object containing arbitrary data supplied by the drag source
   * @return status The CSS class that communicates the drop status back to the source so that the
   * underlying <a href="#!/api/Ext.dd.StatusProxy" rel="Ext.dd.StatusProxy" class="docClass">Ext.dd.StatusProxy</a> can be updated
   */
  public native function notifyEnter(source:ext.dd.DragSource, e:ext.event.Event, data:Object):String;

  /**
   * The function a <a href="#!/api/Ext.dd.DragSource" rel="Ext.dd.DragSource" class="docClass">Ext.dd.DragSource</a> calls once to notify this drop target that the source has been dragged
   * out of the target without dropping. This default implementation simply removes the CSS class specified by
   * <code>overClass</code> (if any) from the drop element.
   * @param source The drag source that was dragged over this drop target
   * @param e The event
   * @param data An object containing arbitrary data supplied by the drag source
   */
  public native function notifyOut(source:ext.dd.DragSource, e:ext.event.Event, data:Object):void;

  /**
   * The function a <a href="#!/api/Ext.dd.DragSource" rel="Ext.dd.DragSource" class="docClass">Ext.dd.DragSource</a> calls continuously while it is being dragged over the target.
   * This method will be called on every mouse movement while the drag source is over the drop target.
   * This default implementation simply returns the dropAllowed config value.
   * @param source The drag source that was dragged over this drop target
   * @param e The event
   * @param data An object containing arbitrary data supplied by the drag source
   * @return status The CSS class that communicates the drop status back to the source so that the
   * underlying <a href="#!/api/Ext.dd.StatusProxy" rel="Ext.dd.StatusProxy" class="docClass">Ext.dd.StatusProxy</a> can be updated
   */
  public native function notifyOver(source:ext.dd.DragSource, e:ext.event.Event, data:Object):String;
}
}