package ext.direct {

[Uses("ext.direct.SDirectManager")]
[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Native("Ext.direct.Manager", require)]
/**
 * Ext Direct aims to streamline communication between the client and server
 * by providing a single interface that reduces the amount of common code
 * typically required to validate data and handle returned data packets
 * (reading data, error conditions, etc).
 *
 * <p>The <code>Ext.direct</code> namespace includes several classes for a closer integration
 * with the server-side. The Ext.data namespace also includes classes for working
 * with Ext.data.Stores which are backed by data from an Ext Direct method.</p>
 *
 * <h1>Specification</h1>
 *
 * <p>For additional information consult the <a href="http://sencha.com/products/extjs/extdirect">Ext Direct Specification</a>.</p>
 *
 * <h1>Providers</h1>
 *
 * <p>Ext Direct uses a provider architecture, where one or more providers are used
 * to transport data to and from the server. There are several providers that exist
 * in the core at the moment:</p>
 *
 * <ul>
 * <li><a href="#!/api/Ext.direct.JsonProvider" rel="Ext.direct.JsonProvider" class="docClass">JsonProvider</a> for simple JSON operations</li>
 * <li><a href="#!/api/Ext.direct.PollingProvider" rel="Ext.direct.PollingProvider" class="docClass">PollingProvider</a> for repeated requests</li>
 * <li><a href="#!/api/Ext.direct.RemotingProvider" rel="Ext.direct.RemotingProvider" class="docClass">RemotingProvider</a> exposes server side to the client.</li>
 * </ul>
 *
 *
 * <p>A provider does not need to be invoked directly, providers are added via
 * <a href="#!/api/Ext.direct.Manager" rel="Ext.direct.Manager" class="docClass">#addProvider</a>. RemotingProviders' API declarations
 * can also be loaded with <a href="#!/api/Ext.direct.Manager" rel="Ext.direct.Manager" class="docClass">#loadProvider</a>, with
 * Provider instance created automatically after successful retrieval.</p>
 *
 * <h1>Router</h1>
 *
 * <p>Ext Direct RemotingProviders utilize a "router" on the server to direct
 * requests from the client to the appropriate server-side method. Because
 * the Ext Direct API is platform-agnostic, you could completely swap out
 * a Java based server solution and replace it with one that uses C#
 * without changing the client side JavaScript at all, or vice versa.</p>
 *
 * <h1>Server side events</h1>
 *
 * <p>Custom events from the server may be handled by the client by adding listeners, for example:</p>
 *
 * <pre><code>{"type":"event","name":"message","data":"Successfully polled at: 11:19:30 am"}
 *
 * // add a handler for a 'message' event sent by the server
 * <a href="#!/api/Ext.direct.Manager-method-on" rel="Ext.direct.Manager-method-on" class="docClass">Ext.direct.Manager.on</a>('message', function(e){
 *     out.append(String.format('&lt;p&gt;&lt;i&gt;{0}&lt;/i&gt;&lt;/p&gt;', e.data));
 *     out.el.scrollTo('t', 100000, true);
 * });
 * </code></pre>
 */
public const DirectManager:ext.direct.SDirectManager;
}