package ext.direct {

[Uses("ext.direct.JsonProvider")]
[Uses("ext.direct.Provider")]
[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Native("Ext.direct.PollingProvider", require)]
/**
 * Fired immediately before a poll takes place.
 * @eventType ext.direct.events.PollingProviderEvent.BEFORE_POLL
 */
[Event(name = 'onBeforePoll', type = 'ext.direct.events.PollingProviderEvent')]
/**
 * Fired immediately after a poll takes place.
 * @eventType ext.direct.events.PollingProviderEvent.POLL
 */
[Event(name = 'onPoll', type = 'ext.direct.events.PollingProviderEvent')]
/**
 * Provides for repetitive polling of the server at distinct <a href="#!/api/Ext.direct.PollingProvider-cfg-interval" rel="Ext.direct.PollingProvider-cfg-interval" class="docClass">intervals</a>.
 * The initial request for data originates from the client, and then is responded to by the
 * server.
 *
 * <p>Configuration for the PollingProvider can be generated by the server-side
 * API portion of the Ext Direct stack.</p>
 *
 * <p>An instance of PollingProvider may be created directly via the new keyword or by simply
 * specifying <code>type = 'polling'</code>. For example:</p>
 *
 * <pre><code> var pollA = new <a href="#!/api/Ext.direct.PollingProvider" rel="Ext.direct.PollingProvider" class="docClass">Ext.direct.PollingProvider</a>({
 *      type:'polling',
 *      url: 'php/pollA.php',
 *  });
 *  <a href="#!/api/Ext.direct.Manager-method-addProvider" rel="Ext.direct.Manager-method-addProvider" class="docClass">Ext.direct.Manager.addProvider</a>(pollA);
 *  pollA.disconnect();
 *
 *  <a href="#!/api/Ext.direct.Manager-method-addProvider" rel="Ext.direct.Manager-method-addProvider" class="docClass">Ext.direct.Manager.addProvider</a>({
 *      type:'polling',
 *      url: 'php/pollB.php',
 *      id: 'pollB-provider'
 *  });
 *  var pollB = <a href="#!/api/Ext.direct.Manager-method-getProvider" rel="Ext.direct.Manager-method-getProvider" class="docClass">Ext.direct.Manager.getProvider</a>('pollB-provider');
 * </code></pre>
 */
public class PollingProvider extends ext.direct.JsonProvider {
  /**
   * @param config @inheritDoc
   */
  public function PollingProvider(config:ext.direct.PollingProvider = null) {
    super();
  }

  [ExtConfig]
  /**
   * An object containing properties which are to be sent as parameters on every
   * polling request. Note that if baseParams are set and <a href="#!/api/Ext.direct.PollingProvider-cfg-url" rel="Ext.direct.PollingProvider-cfg-url" class="docClass">url</a> parameter
   * is an URL string, poll requests will use POST method instead of default GET.
   */
  public native function get baseParams():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set baseParams(value:Object):void;

  [ExtConfig]
  /**
   * How often to poll the server-side in milliseconds. Defaults to every 3 seconds.
   * @default 3000
   */
  public native function get interval():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set interval(value:Number):void;

  [ExtConfig]
  /**
   * Ext Direct method to use for polling. If a method name is provided as a string,
   * the actual function will not be resolved until the first time this provider
   * is connected.
   *
   * <p>The method should accept named arguments and will be passed <a href="#!/api/Ext.direct.PollingProvider-cfg-baseParams" rel="Ext.direct.PollingProvider-cfg-baseParams" class="docClass">baseParams</a>
   * if set.</p>
   */
  public native function get pollFn():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set pollFn(value:*):void;

  [ExtConfig]
  /**
   * The timeout to use for each request.
   */
  public native function get timeout():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set timeout(value:Number):void;

  [Deprecated(since = "5.1.0")]
  [ExtConfig]
  /**
   * The url which the PollingProvider should contact with each request. This can also be
   * an imported Ext Direct method which will be passed baseParams as named arguments.
   *
   * <p><em>Note</em> that using Function <code>url</code> is deprecated, use <a href="#!/api/Ext.direct.PollingProvider-cfg-pollFn" rel="Ext.direct.PollingProvider-cfg-pollFn" class="docClass">pollFn</a> instead.</p>
   */
  public native function get url():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set url(value:*):void;
}
}