package ext.direct {
import ext.mixin.Observable;

[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Native("Ext.direct.Provider", require)]
/**
 * Fires when the Provider connects to the server-side
 * @eventType ext.direct.events.ProviderEvent.CONNECT
 */
[Event(name = 'onConnect', type = 'ext.direct.events.ProviderEvent')]
/**
 * Fires when the Provider receives data from the server-side. This event is fired
 * for valid responses as well as for exceptions.
 * @eventType ext.direct.events.Provider_eEvent.DATA
 */
[Event(name = 'onData', type = 'ext.direct.events.Provider_eEvent')]
/**
 * Fires when the Provider disconnects from the server-side
 * @eventType ext.direct.events.ProviderEvent.DISCONNECT
 */
[Event(name = 'onDisconnect', type = 'ext.direct.events.ProviderEvent')]
/**
 * Fires when the Provider receives an exception from the server-side. This event is <em>not</em>
 * fired for valid responses.
 * @eventType ext.direct.events.Provider_eEvent.EXCEPTION
 */
[Event(name = 'onException', type = 'ext.direct.events.Provider_eEvent')]
/**
 * <a href="#!/api/Ext.direct.Provider" rel="Ext.direct.Provider" class="docClass">Ext.direct.Provider</a> is an abstract class meant to be extended.
 *
 * <p>For example Ext JS implements the following subclasses:</p>
 *
 * <pre><code>Provider
 * |
 * +---JsonProvider
 *     |
 *     +---PollingProvider
 *     |
 *     +---RemotingProvider
 * </code></pre>
 */
public class Provider extends ext.mixin.Observable {
  /**
   * Check if the passed configuration object contains enough
   * information to construct a Provider.
   * @return <code>true</code> if config is sufficient, <code>false</code> otherwise.
   */
  public static native function checkConfig(config:Object):Boolean;

  /**
   * @param config @inheritDoc
   */
  public function Provider(config:ext.direct.Provider = null) {
    super();
  }

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * An object containing default headers for every Ajax request made by this Provider.
   * @see #getHeaders()
   * @see #setHeaders()
   */
  public native function get headers():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set headers(value:Object):void;

  [ExtConfig]
  /**
   * The unique id of the provider (defaults to an <a href="#!/api/Ext-method-id" rel="Ext-method-id" class="docClass">auto-assigned id</a>).
   * You should assign an id if you need to be able to access the provider later and you do
   * not have an object reference available, for example:
   *
   * <pre><code> <a href="#!/api/Ext.direct.Manager-method-addProvider" rel="Ext.direct.Manager-method-addProvider" class="docClass">Ext.direct.Manager.addProvider</a>({
   *      type: 'polling',
   *      url:  'php/poll.php',
   *      id:   'poll-provider'
   *  });
   *  var p = <a href="#!/api/Ext.direct.Manager" rel="Ext.direct.Manager" class="docClass">Ext.direct.Manager</a>.<a href="#!/api/Ext.direct.Manager-method-getProvider" rel="Ext.direct.Manager-method-getProvider" class="docClass">getProvider</a>('poll-provider');
   * p.disconnect();
   * </code></pre>
   */
  public native function get id():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set id(value:String):void;

  [ExtConfig]
  /**
   * List of Provider events that should be relayed by <a href="#!/api/Ext.direct.Manager" rel="Ext.direct.Manager" class="docClass">Ext.direct.Manager</a>.
   * 'data' event is always relayed.
   */
  public native function get relayedEvents():Array;

  [ExtConfig]
  /**
   * @private
   */
  public native function set relayedEvents(value:Array):void;

  /**
   * Connect the provider and start its service.
   * Provider will fire <code>connect</code> event upon successful connection.
   */
  public native function connect():void;

  /**
   * Disconnect the provider and stop its service.
   * Provider will fire <code>disconnect</code> event upon successful disconnect.
   */
  public native function disconnect(force:Object):void;

  /**
   * Returns the value of <a href="#!/api/Ext.direct.Provider-cfg-headers" rel="Ext.direct.Provider-cfg-headers" class="docClass">headers</a>.
   */
  public native function getHeaders():Object;

  /**
   * Returns whether or not the server-side is currently connected.
   */
  public native function isConnected():void;

  /**
   * Sets the value of <a href="#!/api/Ext.direct.Provider-cfg-headers" rel="Ext.direct.Provider-cfg-headers" class="docClass">headers</a>.
   * @param headers The new value.
   */
  public native function setHeaders(headers:Object):void;

  /**
   * Do connection setup. This is a template method.
   */
  protected native function doConnect():void;

  /**
   * Do connection teardown. This is a template method.
   */
  protected native function doDisconnect():void;
}
}