package ext.drag {
import ext.Base;
import ext.mixin.IFactoryable;
import ext.util.Region;

[Uses("ext.Base")]
[Uses("ext.mixin.Factoryable")]
[Native("Ext.drag.Constraint", require)]
/**
 * Provides constraining behavior for a <a href="#!/api/Ext.drag.Source" rel="Ext.drag.Source" class="docClass">Ext.drag.Source</a>.
 */
public class DragConstraint extends ext.Base implements ext.mixin.IFactoryable {
  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The element to constrain to:
   * - <code>true</code> to constrain to the parent of the <a href="#!/api/Ext.drag.Source-cfg-element" rel="Ext.drag.Source-cfg-element" class="docClass">Ext.drag.Source.element</a>.
   * - The id, DOM element or <a href="#!/api/Ext.dom.Element" rel="Ext.dom.Element" class="docClass">Ext.dom.Element</a> to constrain to.
   * @see #getElement()
   * @see #setElement()
   */
  public native function get element():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set element(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * <code>true</code> to limit dragging to the horizontal axis.
   * @default false
   * @see #getHorizontal()
   * @see #setHorizontal()
   */
  public native function get horizontal():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set horizontal(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The region to constrain to.
   * @see #getRegion()
   * @see #setRegion()
   */
  public native function get region():ext.util.Region;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set region(value:ext.util.Region):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The interval to move this drag target during a drag in both dimensions.
   * - <code>{x: 30}</code>, snap only x
   * - <code>{y: 30}</code>, snap only y
   * - <code>{x: 30, y: 40}</code>, snap both
   * - <code>40</code>, snap both to <code>40</code>.
   *
   * <p>The snap may also be a function to calculate the snap value on each tick.</p>
   *
   * <pre><code> snap: {
   *      x: function(info, x) {
   *          return x &lt; 300 ? 150 : 500;
   *      }
   *  }
   * </code></pre>
   * @see #getSnap()
   * @see #setSnap()
   */
  public native function get snap():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set snap(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The <a href="#!/api/Ext.drag.Source" rel="Ext.drag.Source" class="docClass">source</a> for the constraint. This will be
   * set automatically when constructed via the source.
   * @see #getSource()
   * @see #setSource()
   */
  public native function get source():ext.drag.DragSource;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set source(value:ext.drag.DragSource):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * <code>true</code> to limit dragging to the vertical axis.
   * @default false
   * @see #getVertical()
   * @see #setVertical()
   */
  public native function get vertical():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set vertical(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The minimum and maximum x position. Use <code>null</code> to
   * not set a constraint:
   * - <code>[100, null]</code>, constrain only the minimum
   * - <code>[null, 100]</code>, constrain only the maximum
   * - <code>[200, 200]</code>, constrain both.
   * @see #getX()
   * @see #setX()
   */
  public native function get x():Array;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set x(value:Array):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The minimum and maximum y position. Use <code>null</code> to
   * not set a constraint:
   * - <code>[100, null]</code>, constrain only the minimum
   * - <code>[null, 100]</code>, constrain only the maximum
   * - <code>[200, 200]</code>, constrain both.
   * @see #getY()
   * @see #setY()
   */
  public native function get y():Array;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set y(value:Array):void;

  /**
   * @inheritDoc
   */
  public native function get factoryConfig():Object;

  /**
   * @inheritDoc
   */
  public native function set factoryConfig(value:Object):void;

  /**
   * Constrain the position of the drag proxy using the configured rules.
   * @param xy The position.
   * @param info The drag information.
   * @return The xy position.
   */
  public native function constrain(xy:Array, info:ext.drag.DragInfo):Array;

  /**
   * Returns the value of <a href="#!/api/Ext.drag.Constraint-cfg-element" rel="Ext.drag.Constraint-cfg-element" class="docClass">element</a>.
   */
  public native function getElement():*;

  /**
   * Returns the value of <a href="#!/api/Ext.drag.Constraint-cfg-horizontal" rel="Ext.drag.Constraint-cfg-horizontal" class="docClass">horizontal</a>.
   */
  public native function getHorizontal():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.drag.Constraint-cfg-region" rel="Ext.drag.Constraint-cfg-region" class="docClass">region</a>.
   */
  public native function getRegion():ext.util.Region;

  /**
   * Returns the value of <a href="#!/api/Ext.drag.Constraint-cfg-snap" rel="Ext.drag.Constraint-cfg-snap" class="docClass">snap</a>.
   */
  public native function getSnap():*;

  /**
   * Returns the value of <a href="#!/api/Ext.drag.Constraint-cfg-source" rel="Ext.drag.Constraint-cfg-source" class="docClass">source</a>.
   */
  public native function getSource():ext.drag.DragSource;

  /**
   * Returns the value of <a href="#!/api/Ext.drag.Constraint-cfg-vertical" rel="Ext.drag.Constraint-cfg-vertical" class="docClass">vertical</a>.
   */
  public native function getVertical():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.drag.Constraint-cfg-x" rel="Ext.drag.Constraint-cfg-x" class="docClass">x</a>.
   */
  public native function getX():Array;

  /**
   * Returns the value of <a href="#!/api/Ext.drag.Constraint-cfg-y" rel="Ext.drag.Constraint-cfg-y" class="docClass">y</a>.
   */
  public native function getY():Array;

  /**
   * Sets the value of <a href="#!/api/Ext.drag.Constraint-cfg-element" rel="Ext.drag.Constraint-cfg-element" class="docClass">element</a>.
   * @param element The new value.
   */
  public native function setElement(element:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.drag.Constraint-cfg-horizontal" rel="Ext.drag.Constraint-cfg-horizontal" class="docClass">horizontal</a>.
   * @param horizontal The new value.
   */
  public native function setHorizontal(horizontal:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.drag.Constraint-cfg-region" rel="Ext.drag.Constraint-cfg-region" class="docClass">region</a>.
   * @param region The new value.
   */
  public native function setRegion(region:ext.util.Region):void;

  /**
   * Sets the value of <a href="#!/api/Ext.drag.Constraint-cfg-snap" rel="Ext.drag.Constraint-cfg-snap" class="docClass">snap</a>.
   * @param snap The new value.
   */
  public native function setSnap(snap:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.drag.Constraint-cfg-source" rel="Ext.drag.Constraint-cfg-source" class="docClass">source</a>.
   * @param source The new value.
   */
  public native function setSource(source:ext.drag.DragSource):void;

  /**
   * Sets the value of <a href="#!/api/Ext.drag.Constraint-cfg-vertical" rel="Ext.drag.Constraint-cfg-vertical" class="docClass">vertical</a>.
   * @param vertical The new value.
   */
  public native function setVertical(vertical:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.drag.Constraint-cfg-x" rel="Ext.drag.Constraint-cfg-x" class="docClass">x</a>.
   * @param x The new value.
   */
  public native function setX(x:Array):void;

  /**
   * Sets the value of <a href="#!/api/Ext.drag.Constraint-cfg-y" rel="Ext.drag.Constraint-cfg-y" class="docClass">y</a>.
   * @param y The new value.
   */
  public native function setY(y:Array):void;
}
}