package ext.drag {
import ext.Base;
import ext.Promise;
import js.HTMLElement;

[Uses("ext.Base")]
[Native("Ext.drag.Info", require)]
/**
 * This class is used to unify information for a specific drag instance.
 * This object is passed to template methods and events to obtain
 * details about the current operation.
 *
 * <p>It is not expected that this class will be created by user code.</p>
 */
public class DragInfo extends ext.Base {
  /**
   * Information about the cursor position. Not available when
   * <a href="#!/api/Ext.drag.Info-property-isNative" rel="Ext.drag.Info-property-isNative" class="docClass">isNative</a> is <code>true</code>.
   * <ul>
   *   <li><code>current</code> : Object
   * The current cursor position.
   * <ul>
   *   <li><code>x</code> : Number
   * The current x position.
   *   </li>
   *   <li><code>y</code> : Number
   * The current y position.
   *   </li>
   * </ul>
   *   </li>
   *   <li><code>delta</code> : Object
   * The change in cursor position.
   * <ul>
   *   <li><code>x</code> : Number
   * The change in x position.
   *   </li>
   *   <li><code>y</code> : Number
   * The change in y position.
   *   </li>
   * </ul>
   *   </li>
   *   <li><code>initial</code> : Object
   * The intial cursor position.
   * <ul>
   *   <li><code>x</code> : Number
   * The initial x position.
   *   </li>
   *   <li><code>y</code> : Number
   * The initial y position.
   *   </li>
   * </ul>
   *   </li>
   *   <li><code>offset</code> : Object
   * The offset from the cursor to the top/left of
   * the <a href="#!/api/Ext.drag.Source-cfg-element" rel="Ext.drag.Source-cfg-element" class="docClass">element</a>.
   * <ul>
   *   <li><code>x</code> : Number
   * The x offset.
   *   </li>
   *   <li><code>y</code> : Number
   * The y offset.
   *   </li>
   * </ul>
   *   </li>
   * </ul>
   */
  public native function get cursor():Object;

  /**
   * @private
   */
  public native function set cursor(value:Object):void;

  /**
   * Information about the <a href="#!/api/Ext.drag.Source-cfg-element" rel="Ext.drag.Source-cfg-element" class="docClass">Ext.drag.Source.element</a> position.
   * Not available when <a href="#!/api/Ext.drag.Info-property-isNative" rel="Ext.drag.Info-property-isNative" class="docClass">isNative</a> is <code>true</code>.
   * <ul>
   *   <li><code>current</code> : Object
   * The current element position.
   * <ul>
   *   <li><code>x</code> : Number
   * The current x position.
   *   </li>
   *   <li><code>y</code> : Number
   * The current y position.
   *   </li>
   * </ul>
   *   </li>
   *   <li><code>delta</code> : Object
   * The change in element position.
   * <ul>
   *   <li><code>x</code> : Number
   * The change in x position.
   *   </li>
   *   <li><code>y</code> : Number
   * The change in y position.
   *   </li>
   * </ul>
   *   </li>
   *   <li><code>initial</code> : Object
   * The intial element position.
   * <ul>
   *   <li><code>x</code> : Number
   * The initial x position.
   *   </li>
   *   <li><code>y</code> : Number
   * The initial y position.
   *   </li>
   * </ul>
   *   </li>
   * </ul>
   */
  public native function get element():Object;

  /**
   * @private
   */
  public native function set element(value:Object):void;

  /**
   * The event target that the drag started on.
   *
   * <p>Not available when <a href="#!/api/Ext.drag.Info-property-isNative" rel="Ext.drag.Info-property-isNative" class="docClass">isNative</a> is <code>true</code>.</p>
   */
  public native function get eventTarget():js.HTMLElement;

  /**
   * @private
   */
  public native function set eventTarget(value:js.HTMLElement):void;

  /**
   * A list of files included for this drag. See:
   * https://developer.mozilla.org/en/docs/Web/API/FileList
   *
   * <p>Only available when <a href="#!/api/Ext.drag.Info-property-isNative" rel="Ext.drag.Info-property-isNative" class="docClass">isNative</a> is <code>true</code>.</p>
   */
  public native function get files():Object;

  /**
   * @private
   */
  public native function set files(value:Object):void;

  /**
   * <code>true</code> if the drag is a native drag event, for example
   * a file draggedi nto the browser.
   * @default false
   */
  public native function get isNative():Boolean;

  /**
   * @private
   */
  public native function set isNative(value:Boolean):void;

  /**
   * Information about the <a href="#!/api/Ext.drag.Source-cfg-proxy" rel="Ext.drag.Source-cfg-proxy" class="docClass">Ext.drag.Source.proxy</a> position.
   * This may be the actual <a href="#!/api/Ext.drag.Source-cfg-element" rel="Ext.drag.Source-cfg-element" class="docClass">Ext.drag.Source.element</a>.
   * Not available when <a href="#!/api/Ext.drag.Info-property-isNative" rel="Ext.drag.Info-property-isNative" class="docClass">isNative</a> is <code>true</code>.
   * <ul>
   *   <li><code>current</code> : Object
   * The current proxy position.
   * <ul>
   *   <li><code>x</code> : Number
   * The current x position.
   *   </li>
   *   <li><code>y</code> : Number
   * The current y position.
   *   </li>
   * </ul>
   *   </li>
   *   <li><code>delta</code> : Object
   * The change in proxy position.
   * <ul>
   *   <li><code>x</code> : Number
   * The change in x position.
   *   </li>
   *   <li><code>y</code> : Number
   * The change in y position.
   *   </li>
   * </ul>
   *   </li>
   *   <li><code>initial</code> : Object
   * The intial proxy position.
   * <ul>
   *   <li><code>x</code> : Number
   * The initial x position.
   *   </li>
   *   <li><code>y</code> : Number
   * The initial y position.
   *   </li>
   * </ul>
   *   </li>
   *   <li><code>element</code> : ext.dom.Element
   * The proxy element.
   *   </li>
   *   <li><code>isElement</code> : Boolean
   * <code>true</code> if the proxy is the <a href="#!/api/Ext.drag.Source-cfg-element" rel="Ext.drag.Source-cfg-element" class="docClass">Ext.drag.Source.element</a>.
   *   </li>
   *   <li><code>isUnderCursor</code> : Boolean
   * <code>true</code> if the alignment causes the proxy to be under the cursor.
   *   </li>
   * </ul>
   */
  public native function get proxy():Object;

  /**
   * @private
   */
  public native function set proxy(value:Object):void;

  /**
   * The drag source. Not available when <a href="#!/api/Ext.drag.Info-property-isNative" rel="Ext.drag.Info-property-isNative" class="docClass">isNative</a> is <code>true</code>.
   */
  public native function get source():ext.drag.DragSource;

  /**
   * @private
   */
  public native function set source(value:ext.drag.DragSource):void;

  /**
   * The active target. <code>null</code> if not over a target.
   */
  public native function get target():ext.drag.DragTarget;

  /**
   * @private
   */
  public native function set target(value:ext.drag.DragTarget):void;

  /**
   * The data types this drag provides. Added via <a href="#!/api/Ext.drag.Info-method-setData" rel="Ext.drag.Info-method-setData" class="docClass">setData</a>.
   */
  public native function get types():Array;

  /**
   * @private
   */
  public native function set types(value:Array):void;

  /**
   * <code>true</code> if the <a href="#!/api/Ext.drag.Info-property-target" rel="Ext.drag.Info-property-target" class="docClass">target</a> is valid. See <a href="#!/api/Ext.drag.Target" rel="Ext.drag.Target" class="docClass">Ext.drag.Target</a> for
   * information about validity. <code>false</code> if there is no target.
   * @default false
   */
  public native function get valid():Boolean;

  /**
   * @private
   */
  public native function set valid(value:Boolean):void;

  /**
   * Clear the data for a particular type.
   * @param type The type.
   */
  public native function clearData(type:String):void;

  /**
   * Create a copy of this object with the current state.
   * @return A copy of this object.
   */
  public native function clone():ext.drag.DragInfo;

  /**
   * Get data for this drag. This method may only be called once the drop completes.
   * @param type The type of data to retrieve. Must be in the <a href="#!/api/Ext.drag.Info-property-types" rel="Ext.drag.Info-property-types" class="docClass">types</a>.
   * See also <a href="#!/api/Ext.drag.Info-method-setData" rel="Ext.drag.Info-method-setData" class="docClass">setData</a>.
   * @return The data. If the produced data is not a <a href="#!/api/Ext.Promise" rel="Ext.Promise" class="docClass">Ext.Promise</a>,
   * it will be wrapped in one.
   */
  public native function getData(type:String):ext.Promise;

  /**
   * Set data for this drag. Multiple types may be registered. Each type will be
   * added to <a href="#!/api/Ext.drag.Info-property-types" rel="Ext.drag.Info-property-types" class="docClass">types</a>.
   * @param type The type of data being registered.
   * @param value The value being registered. If a function
   * is provided it will be evaluated if requested when the drop completes. The
   * function should return a value or a <a href="#!/api/Ext.Promise" rel="Ext.Promise" class="docClass">Ext.Promise</a> that will produce a value.
   */
  public native function setData(type:String, value:*):void;
}
}