package ext.drag {
import ext.mixin.IIdentifiable;
import ext.mixin.Observable;

[Uses("ext.mixin.Observable")]
[Uses("ext.mixin.Identifiable")]
[Uses("ext.Base")]
[Native("Ext.drag.Item", require)]
/**
 * A base class for draggable and droppable items that wrap a DOM element.
 */
public class DragItem extends ext.mixin.Observable implements ext.mixin.IIdentifiable {
  /**
   * @inheritDoc
   */
  public function DragItem(config:ext.mixin.Observable = null) {
    super();
  }

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * <code>true</code> to destroy the <a href="#!/api/Ext.drag.Item-cfg-element" rel="Ext.drag.Item-cfg-element" class="docClass">element</a> when this item is destroyed.
   * @default true
   * @see #getAutoDestroy()
   * @see #setAutoDestroy()
   */
  public native function get autoDestroy():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set autoDestroy(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The id, dom or Element reference for this item.
   * @see #getElement()
   * @see #setElement()
   */
  public native function get element():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set element(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * A group controls which <a href="#!/api/Ext.drag.Source" rel="Ext.drag.Source" class="docClass">sources</a> and <a href="#!/api/Ext.drag.Target" rel="Ext.drag.Target" class="docClass">Ext.drag.Target</a> targets
   * can interact with each other. Only items that have the same (or intersecting) groups will
   * react to each other. Items with no groups will be in the default pool.
   * @see #getGroups()
   * @see #setGroups()
   */
  public native function get groups():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set groups(value:*):void;

  /**
   * Disable the current item to disallow it from participating
   * in drag/drop operations.
   */
  public native function disable():void;

  /**
   * Enable the current item to allow it to participate in
   * drag/drop operations.
   */
  public native function enable():void;

  /**
   * Returns the value of <a href="#!/api/Ext.drag.Item-cfg-autoDestroy" rel="Ext.drag.Item-cfg-autoDestroy" class="docClass">autoDestroy</a>.
   */
  public native function getAutoDestroy():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.drag.Item-cfg-element" rel="Ext.drag.Item-cfg-element" class="docClass">element</a>.
   */
  public native function getElement():*;

  /**
   * Returns the value of <a href="#!/api/Ext.drag.Item-cfg-groups" rel="Ext.drag.Item-cfg-groups" class="docClass">groups</a>.
   */
  public native function getGroups():*;

  /**
   * Retrieves the id of this component. Will autogenerate an id if one has not already been set.
   * @return id
   */
  override public native function getId():String;

  /**
   * Checks whether this item is currently disabled.
   * @return <code>true</code> if this item is disabled.
   */
  public native function isDisabled():Boolean;

  /**
   * Sets the value of <a href="#!/api/Ext.drag.Item-cfg-autoDestroy" rel="Ext.drag.Item-cfg-autoDestroy" class="docClass">autoDestroy</a>.
   * @param autoDestroy The new value.
   */
  public native function setAutoDestroy(autoDestroy:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.drag.Item-cfg-element" rel="Ext.drag.Item-cfg-element" class="docClass">element</a>.
   * @param element The new value.
   */
  public native function setElement(element:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.drag.Item-cfg-groups" rel="Ext.drag.Item-cfg-groups" class="docClass">groups</a>.
   * @param groups The new value.
   */
  public native function setGroups(groups:*):void;
}
}