package ext.event.gesture {
import ext.Base;
import ext.mixin.IIdentifiable;

[Uses("ext.Base")]
[Uses("ext.mixin.Identifiable")]
[Native("Ext.event.gesture.Recognizer", require)]
[ExcludeClass]
/**
 * A base class for all gesture recognizers.
 *
 * <p>The following gestures are enabled by default in both Ext JS and Sencha Touch:</p>
 *
 * <ul>
 * <li><a href="#!/api/Ext.event.gesture.Tap" rel="Ext.event.gesture.Tap" class="docClass">Ext.event.gesture.Tap</a></li>
 * <li><a href="#!/api/Ext.event.gesture.DoubleTap" rel="Ext.event.gesture.DoubleTap" class="docClass">Ext.event.gesture.DoubleTap</a></li>
 * <li><a href="#!/api/Ext.event.gesture.LongPress" rel="Ext.event.gesture.LongPress" class="docClass">Ext.event.gesture.LongPress</a></li>
 * <li><a href="#!/api/Ext.event.gesture.Drag" rel="Ext.event.gesture.Drag" class="docClass">Ext.event.gesture.Drag</a></li>
 * <li><a href="#!/api/Ext.event.gesture.Swipe" rel="Ext.event.gesture.Swipe" class="docClass">Ext.event.gesture.Swipe</a></li>
 * <li><a href="#!/api/Ext.event.gesture.Pinch" rel="Ext.event.gesture.Pinch" class="docClass">Ext.event.gesture.Pinch</a></li>
 * <li><a href="#!/api/Ext.event.gesture.Rotate" rel="Ext.event.gesture.Rotate" class="docClass">Ext.event.gesture.Rotate</a></li>
 * <li><a href="#!/api/Ext.event.gesture.EdgeSwipe" rel="Ext.event.gesture.EdgeSwipe" class="docClass">Ext.event.gesture.EdgeSwipe</a></li>
 * </ul>
 */
public class GestureRecognizer extends ext.Base implements ext.mixin.IIdentifiable {
  /**
   * @param config @inheritDoc
   */
  public function GestureRecognizer(config:ext.event.gesture.GestureRecognizer = null) {
    super();
  }

  /**
   * The priority of the recognizer. Determines the order in which it recognizes gestures
   * relative to other recognizers.  The default recognizers use the following priorities:
   *
   * <ul>
   * <li><a href="#!/api/Ext.event.gesture.Drag" rel="Ext.event.gesture.Drag" class="docClass">Ext.event.gesture.Drag</a>: 100</li>
   * <li><a href="#!/api/Ext.event.gesture.Tap" rel="Ext.event.gesture.Tap" class="docClass">Ext.event.gesture.Tap</a>: 200</li>
   * <li><a href="#!/api/Ext.event.gesture.DoubleTap" rel="Ext.event.gesture.DoubleTap" class="docClass">Ext.event.gesture.DoubleTap</a>: 300</li>
   * <li><a href="#!/api/Ext.event.gesture.LongPress" rel="Ext.event.gesture.LongPress" class="docClass">Ext.event.gesture.LongPress</a>: 400</li>
   * <li><a href="#!/api/Ext.event.gesture.EdgeSwipe" rel="Ext.event.gesture.EdgeSwipe" class="docClass">Ext.event.gesture.EdgeSwipe</a>: 500</li>
   * <li><a href="#!/api/Ext.event.gesture.Swipe" rel="Ext.event.gesture.Swipe" class="docClass">Ext.event.gesture.Swipe</a>: 600</li>
   * <li><a href="#!/api/Ext.event.gesture.Pinch" rel="Ext.event.gesture.Pinch" class="docClass">Ext.event.gesture.Pinch</a>: 700</li>
   * <li><a href="#!/api/Ext.event.gesture.Rotate" rel="Ext.event.gesture.Rotate" class="docClass">Ext.event.gesture.Rotate</a>: 800</li>
   * </ul>
   * @default 0
   */
  public native function get priority():Number;

  /**
   * @private
   */
  public native function set priority(value:Number):void;

  /**
   * Retrieves the id of this component. Will autogenerate an id if one has not already been set.
   * @return id
   */
  public native function getId():String;
}
}