package ext.flash {
import ext.Component;
import ext.dom.Element;

[Uses("ext.Component")]
[Uses("ext.util.Positionable")]
[Uses("ext.Base")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Animate")]
[Uses("ext.util.Floating")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.state.Stateful")]
[Native("Ext.flash.Component", require)]
/**
 * Fired when the Flash movie embedding fails
 * @eventType ext.flash.events.FlashComponentEvent.FAILURE
 */
[Event(name = 'onFailure', type = 'ext.flash.events.FlashComponentEvent')]
/**
 * Fired when the Flash movie has been successfully embedded
 * @eventType ext.flash.events.FlashComponentEvent.SUCCESS
 */
[Event(name = 'onSuccess', type = 'ext.flash.events.FlashComponentEvent')]
/**
 * A simple Component for displaying an Adobe Flash SWF movie. The movie will be sized and can participate
 * in layout like any other Component.
 *
 * <p>This component requires the third-party SWFObject library version 2.2 or above. It is not included within
 * the ExtJS distribution, so you will have to include it into your page manually in order to use this component.
 * The SWFObject library can be downloaded from the <a href="http://code.google.com/p/swfobject">SWFObject project page</a>
 * and then simply import it into the head of your HTML document:</p>
 *
 * <pre><code>&lt;script type="text/javascript" src="path/to/local/swfobject.js"&gt;&lt;/script&gt;
 * </code></pre>
 *
 * <h2>Configuration</h2>
 *
 * <p>This component allows several options for configuring how the target Flash movie is embedded. The most
 * important is the required <a href="#!/api/Ext.flash.Component-cfg-url" rel="Ext.flash.Component-cfg-url" class="docClass">url</a> which points to the location of the Flash movie to load. Other
 * configurations include:</p>
 *
 * <ul>
 * <li><a href="#!/api/Ext.flash.Component-cfg-backgroundColor" rel="Ext.flash.Component-cfg-backgroundColor" class="docClass">backgroundColor</a></li>
 * <li><a href="#!/api/Ext.flash.Component-cfg-wmode" rel="Ext.flash.Component-cfg-wmode" class="docClass">wmode</a></li>
 * <li><a href="#!/api/Ext.flash.Component-cfg-flashVars" rel="Ext.flash.Component-cfg-flashVars" class="docClass">flashVars</a></li>
 * <li><a href="#!/api/Ext.flash.Component-cfg-flashParams" rel="Ext.flash.Component-cfg-flashParams" class="docClass">flashParams</a></li>
 * <li><a href="#!/api/Ext.flash.Component-cfg-flashAttributes" rel="Ext.flash.Component-cfg-flashAttributes" class="docClass">flashAttributes</a></li>
 * </ul>
 *
 *
 * <h2>Example usage:</h2>
 *
 * <pre><code>var win = <a href="#!/api/Ext-method-widget" rel="Ext-method-widget" class="docClass">Ext.widget</a>('window', {
 *     title: "It's a tiger!",
 *     layout: 'fit',
 *     width: 300,
 *     height: 300,
 *     x: 20,
 *     y: 20,
 *     resizable: true,
 *     items: {
 *         xtype: 'flash',
 *         url: 'tiger.swf'
 *     }
 * });
 * win.show();
 * </code></pre>
 *
 * <h2>Express Install</h2>
 *
 * <p>Adobe provides a tool called <a href="http://www.adobe.com/devnet/flashplayer/articles/express_install.html">Express Install</a>
 * that offers users an easy way to upgrade their Flash player. If you wish to make use of this, you should set
 * the static EXPRESS_INSTALL_URL property to the location of your Express Install SWF file:</p>
 *
 * <pre><code><a href="#!/api/Ext.flash.Component-static-property-EXPRESS_INSTALL_URL" rel="Ext.flash.Component-static-property-EXPRESS_INSTALL_URL" class="docClass">Ext.flash.Component.EXPRESS_INSTALL_URL</a> = 'path/to/local/expressInstall.swf';
 * </code></pre>
 */
public class FlashComponent extends ext.Component {
  /**
   * The url for installing flash if it doesn't exist. This should be set to a local resource.
   * See <a href="http://get.adobe.com/flashplayer/">http://get.adobe.com/flashplayer/</a> for details.
   * @default 'http:/' + '/swfobject.googlecode.com/svn/trunk/swfobject/expressInstall.swf'
   */
  public static native function get EXPRESS_INSTALL_URL():String;

  /**
   * @param config @inheritDoc
   */
  public function FlashComponent(config:ext.flash.FlashComponent = null) {
    super();
  }

  /**
   * A reference to the object or embed element into which the SWF file is loaded. Only
   * populated after the component is rendered and the SWF has been successfully embedded.
   */
  public native function get swf():ext.dom.Element;

  /**
   * @private
   */
  public native function set swf(value:ext.dom.Element):void;

  [ExtConfig]
  /**
   * The background color of the SWF movie.
   * @default "#ffffff"
   */
  public native function get backgroundColor():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set backgroundColor(value:String):void;

  [ExtConfig]
  /**
   * True to prompt the user to install flash if not installed. Note that this uses
   * <a href="#!/api/Ext.flash.Component-static-property-EXPRESS_INSTALL_URL" rel="Ext.flash.Component-static-property-EXPRESS_INSTALL_URL" class="docClass">Ext.FlashComponent.EXPRESS_INSTALL_URL</a>, which should be set to the local resource.
   * @default false
   */
  public native function get expressInstall():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set expressInstall(value:Boolean):void;

  [ExtConfig]
  /**
   * A set of key value pairs to be passed to the flash object as attributes.
   */
  public native function get flashAttributes():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set flashAttributes(value:Object):void;

  [ExtConfig]
  /**
   * A set of key value pairs to be passed to the flash object as parameters. Possible parameters can be found here:
   * http://kb2.adobe.com/cps/127/tn_12701.html
   */
  public native function get flashParams():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set flashParams(value:Object):void;

  [ExtConfig]
  /**
   * A set of key value pairs to be passed to the flash object as flash variables.
   */
  public native function get flashVars():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set flashVars(value:Object):void;

  [ExtConfig]
  /**
   * Indicates the version the flash content was published for.
   * @default "9.0.115"
   */
  public native function get flashVersion():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set flashVersion(value:String):void;

  [ExtConfig]
  /**
   * The height of the embedded SWF movie inside the component.
   *
   * <p>Defaults to "100%" so that the movie matches the height of the component.</p>
   * @default "100%"
   */
  public native function get swfHeight():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set swfHeight(value:*):void;

  [ExtConfig]
  /**
   * The width of the embedded SWF movie inside the component.
   *
   * <p>Defaults to "100%" so that the movie matches the width of the component.</p>
   * @default "100%"
   */
  public native function get swfWidth():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set swfWidth(value:*):void;

  [ExtConfig]
  /**
   * The URL of the SWF file to include.
   */
  public native function get url():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set url(value:String):void;

  [ExtConfig]
  /**
   * The wmode of the flash object. This can be used to control layering.
   * Set to 'transparent' to ignore the <a href="#!/api/Ext.flash.Component-cfg-backgroundColor" rel="Ext.flash.Component-cfg-backgroundColor" class="docClass">backgroundColor</a> and make the background of the Flash
   * movie transparent.
   * @default "opaque"
   */
  public native function get wmode():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set wmode(value:String):void;

  /**
   * Retrieves the id of the SWF object/embed element.
   */
  public native function getSwfId():String;
}
}