package ext.form {
import ext.Component;
import ext.form.field.IField;
import js.HTMLElement;

[Uses("ext.form.FieldContainer")]
[Uses("ext.form.field.Field")]
[Uses("ext.Base")]
[Uses("ext.container.Container")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.mixin.Container")]
[Uses("ext.Mixin")]
[Uses("ext.Component")]
[Uses("ext.util.Positionable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Animate")]
[Uses("ext.util.Floating")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.state.Stateful")]
[Uses("ext.form.Labelable")]
[Uses("ext.form.FieldAncestor")]
[Native("Ext.form.CheckboxGroup", require)]
/**
 * Fires when the value of a field is changed. The value of a field is
 * checked for changes when the field's <a href="#!/api/Ext.form.field.Field-method-setValue" rel="Ext.form.field.Field-method-setValue" class="docClass">setValue</a> method
 * is called and when any of the events listed in
 * <a href="#!/api/Ext.form.field.Base-cfg-checkChangeEvents" rel="Ext.form.field.Base-cfg-checkChangeEvents" class="docClass">checkChangeEvents</a> are fired.
 * @eventType ext.form.events.CheckboxGroup_newValue_oldValueEvent.CHANGE
 */
[Event(name = 'onChange', type = 'ext.form.events.CheckboxGroup_newValue_oldValueEvent')]
/**
 * Fires when a change in the field's <a href="#!/api/Ext.form.field.Field-method-isDirty" rel="Ext.form.field.Field-method-isDirty" class="docClass">isDirty</a> state is detected.
 * @eventType ext.form.events.CheckboxGroup_isDirtyEvent.DIRTY_CHANGE
 */
[Event(name = 'onDirtyChange', type = 'ext.form.events.CheckboxGroup_isDirtyEvent')]
/**
 * Fires when a change in the field's validity is detected.
 * @eventType ext.form.events.CheckboxGroup_isValidEvent.VALIDITY_CHANGE
 */
[Event(name = 'onValidityChange', type = 'ext.form.events.CheckboxGroup_isValidEvent')]
/**
 * A <a href="#!/api/Ext.form.FieldContainer" rel="Ext.form.FieldContainer" class="docClass">field container</a> which has a specialized layout for arranging
 * <a href="#!/api/Ext.form.field.Checkbox" rel="Ext.form.field.Checkbox" class="docClass">Ext.form.field.Checkbox</a> controls into columns, and provides convenience
 * <a href="#!/api/Ext.form.field.Field" rel="Ext.form.field.Field" class="docClass">Ext.form.field.Field</a> methods for <a href="#!/api/Ext.form.CheckboxGroup-method-getValue" rel="Ext.form.CheckboxGroup-method-getValue" class="docClass">getting</a>, <a href="#!/api/Ext.form.CheckboxGroup-method-setValue" rel="Ext.form.CheckboxGroup-method-setValue" class="docClass">setting</a>,
 * and <a href="#!/api/Ext.form.CheckboxGroup-method-validate" rel="Ext.form.CheckboxGroup-method-validate" class="docClass">validating</a> the group of checkboxes as a whole.
 *
 * <h1>Validation</h1>
 *
 * <p>Individual checkbox fields themselves have no default validation behavior, but
 * sometimes you want to require a user to select at least one of a group of checkboxes. CheckboxGroup
 * allows this by setting the config <code><a href="#!/api/Ext.form.CheckboxGroup-cfg-allowBlank" rel="Ext.form.CheckboxGroup-cfg-allowBlank" class="docClass">allowBlank</a>:false</code>; when the user does not check at
 * least one of the checkboxes, the entire group will be highlighted as invalid and the
 * <a href="#!/api/Ext.form.CheckboxGroup-cfg-blankText" rel="Ext.form.CheckboxGroup-cfg-blankText" class="docClass">error message</a> will be displayed according to the <a href="#!/api/Ext.form.CheckboxGroup-cfg-msgTarget" rel="Ext.form.CheckboxGroup-cfg-msgTarget" class="docClass">msgTarget</a> config.</p>
 *
 * <h1>Layout</h1>
 *
 * <p>The default layout for CheckboxGroup makes it easy to arrange the checkboxes into
 * columns; see the <a href="#!/api/Ext.form.CheckboxGroup-cfg-columns" rel="Ext.form.CheckboxGroup-cfg-columns" class="docClass">columns</a> and <a href="#!/api/Ext.form.CheckboxGroup-cfg-vertical" rel="Ext.form.CheckboxGroup-cfg-vertical" class="docClass">vertical</a> config documentation for details. You may also
 * use a completely different layout by setting the <a href="#!/api/Ext.form.CheckboxGroup-cfg-layout" rel="Ext.form.CheckboxGroup-cfg-layout" class="docClass">layout</a> to one of the
 * other supported layout types; for instance you may wish to use a custom arrangement
 * of hbox and vbox containers. In that case the checkbox components at any depth will
 * still be managed by the CheckboxGroup's validation.</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a>', {
 *     title: 'Checkbox Group',
 *     width: 300,
 *     height: 125,
 *     bodyPadding: 10,
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     items:[{
 *         xtype: 'checkboxgroup',
 *         fieldLabel: 'Two Columns',
 *         // Arrange checkboxes into two columns, distributed vertically
 *         columns: 2,
 *         vertical: true,
 *         items: [
 *             { boxLabel: 'Item 1', name: 'rb', inputValue: '1' },
 *             { boxLabel: 'Item 2', name: 'rb', inputValue: '2', checked: true },
 *             { boxLabel: 'Item 3', name: 'rb', inputValue: '3' },
 *             { boxLabel: 'Item 4', name: 'rb', inputValue: '4' },
 *             { boxLabel: 'Item 5', name: 'rb', inputValue: '5' },
 *             { boxLabel: 'Item 6', name: 'rb', inputValue: '6' }
 *         ]
 *     }]
 * });
 * </code></pre>
 */
public class CheckboxGroup extends ext.form.FieldContainer implements ext.form.field.IField {
  /**
   * @param config @inheritDoc
   */
  public function CheckboxGroup(config:ext.form.CheckboxGroup = null) {
    super();
  }

  [ExtConfig]
  /**
   * True to disable the field. Disabled Fields will not be <a href="#!/api/Ext.form.Basic-method-submit" rel="Ext.form.Basic-method-submit" class="docClass">submitted</a>.
   * @default false
   * @see #setDisabled()
   */
  override public native function get disabled():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  override public native function set disabled(value:Boolean):void;

  [ExtConfig]
  /**
   * An Array of <a href="#!/api/Ext.form.field.Checkbox" rel="Ext.form.field.Checkbox" class="docClass">Checkbox</a>es or Checkbox config objects to arrange in the group.
   */
  override public native function get items():Array;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set items(value:Array):void;

  /**
   * The value <code>true</code> to identify an object as an instance of this or derived class.
   * @default true
   * @since 6.2.0
   */
  public native function get isCheckboxGroup():Boolean;

  /**
   * Flag denoting that this component is a Field. Always true.
   * @default true
   */
  public native function get isFormField():Boolean;

  /**
   * @private
   */
  public native function set isFormField(value:Boolean):void;

  /**
   * The original value of the field as configured in the <a href="#!/api/Ext.form.field.Field-cfg-value" rel="Ext.form.field.Field-cfg-value" class="docClass">value</a> configuration, or as loaded by the last
   * form load operation if the form's <a href="#!/api/Ext.form.Basic-cfg-trackResetOnLoad" rel="Ext.form.Basic-cfg-trackResetOnLoad" class="docClass">trackResetOnLoad</a> setting is <code>true</code>.
   */
  public native function get originalValue():*;

  /**
   * @private
   */
  public native function set originalValue(value:*):void;

  [ExtConfig]
  /**
   * False to validate that at least one item in the group is checked. If no items are selected at
   * validation time, <a href="#!/api/Ext.form.CheckboxGroup-cfg-blankText" rel="Ext.form.CheckboxGroup-cfg-blankText" class="docClass">blankText</a> will be used as the error text.
   * @default true
   */
  public native function get allowBlank():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set allowBlank(value:Boolean):void;

  [ExtConfig]
  /**
   * Error text to display if the <a href="#!/api/Ext.form.CheckboxGroup-cfg-allowBlank" rel="Ext.form.CheckboxGroup-cfg-allowBlank" class="docClass">allowBlank</a> validation fails
   * @default "You must select at least one item in this group"
   */
  public native function get blankText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set blankText(value:String):void;

  [ExtConfig]
  /**
   * Specifies the number of columns to use when displaying grouped checkbox/radio controls using automatic layout.
   * This config can take several types of values:
   *
   * <ul>
   * <li>'auto' - The controls will be rendered one per column on one row and the width of each column will be evenly
   * distributed based on the width of the overall field container. This is the default.</li>
   * <li>Number - If you specific a number (e.g., 3) that number of columns will be created and the contained controls
   * will be automatically distributed based on the value of <a href="#!/api/Ext.form.CheckboxGroup-cfg-vertical" rel="Ext.form.CheckboxGroup-cfg-vertical" class="docClass">vertical</a>.</li>
   * <li>Array - You can also specify an array of column widths, mixing integer (fixed width) and float (percentage
   * width) values as needed (e.g., [100, .25, .75]). Any integer values will be rendered first, then any float
   * values will be calculated as a percentage of the remaining space. Float values do not have to add up to 1
   * (100%) although if you want the controls to take up the entire field container you should do so.</li>
   * </ul>
   * @default 'auto'
   */
  public native function get columns():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set columns(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The value of the <code>name</code> attribute of the input elements
   * belonging to this Group. If not set, Group's <code>id</code> will be used.
   * @see #getName()
   */
  public native function get name():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set name(value:String):void;

  [ExtConfig]
  /**
   * Setting this to false will prevent the field from being <a href="#!/api/Ext.form.Basic-method-submit" rel="Ext.form.Basic-method-submit" class="docClass">submitted</a> even when it is
   * not disabled.
   * @default true
   */
  public native function get submitValue():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set submitValue(value:Boolean):void;

  [ExtConfig]
  /**
   * Specifies whether this field should be validated immediately whenever a change in its value is detected.
   * If the validation results in a change in the field's validity, a <a href="#!/api/Ext.form.field.Field-event-validitychange" rel="Ext.form.field.Field-event-validitychange" class="docClass">validitychange</a> event will be
   * fired. This allows the field to show feedback about the validity of its contents immediately as the user is
   * typing.
   *
   * <p>When set to false, feedback will not be immediate. However the form will still be validated before submitting if
   * the clientValidation option to <a href="#!/api/Ext.form.Basic-method-doAction" rel="Ext.form.Basic-method-doAction" class="docClass">Ext.form.Basic.doAction</a> is enabled, or if the field or form are validated
   * manually.</p>
   *
   * <p>See also <a href="#!/api/Ext.form.field.Base-cfg-checkChangeEvents" rel="Ext.form.field.Base-cfg-checkChangeEvents" class="docClass">Ext.form.field.Base.checkChangeEvents</a> for controlling how changes to the field's value are
   * detected.</p>
   * @default true
   */
  public native function get validateOnChange():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set validateOnChange(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * This property, when a <code>String</code>, contributes its value to the error state of this
   * instance as reported by <code>getErrors</code>.
   * @see #getValidation()
   * @see #setValidation()
   */
  public native function get validation():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set validation(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * A value to initialize this field with.
   * @see #getValue()
   * @see #setValue()
   */
  public native function get value():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set value(value:Object):void;

  [ExtConfig]
  /**
   * The event name(s) to use to publish the <a href="#!/api/Ext.form.field.Field-cfg-value" rel="Ext.form.field.Field-cfg-value" class="docClass">value</a>
   * <a href="#!/api/Ext.form.field.Base-cfg-bind" rel="Ext.form.field.Base-cfg-bind" class="docClass">Ext.form.field.Base.bind</a> for this field.
   * @default 'change'
   * @since 5.0.1
   */
  public native function get valuePublishEvent():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set valuePublishEvent(value:*):void;

  [ExtConfig]
  /**
   * True to distribute contained controls across columns, completely filling each column top to bottom before
   * starting on the next column. The number of controls in each column will be automatically calculated to keep
   * columns as even as possible. The default value is false, so that controls will be added to columns one at a time,
   * completely filling each row left to right before starting on the next row.
   * @default false
   */
  public native function get vertical():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set vertical(value:Boolean):void;

  /**
   * When a checkbox is added to the group, monitor it for changes
   */
  override protected native function onAdd(component:ext.Component, position:Number):void;

  /**
   * A utility for grouping a set of modifications which may trigger value changes into a single transaction, to
   * prevent excessive firing of <a href="#!/api/Ext.form.field.Field-event-change" rel="Ext.form.field.Field-event-change" class="docClass">change</a> events. This is useful for instance if the field has sub-fields which
   * are being updated as a group; you don't want the container field to check its own changed state for each subfield
   * change.
   * @param fn The function to call with change checks suspended.
   */
  public native function batchChanges(fn:Function):void;

  /**
   * Checks whether the value of the field has changed since the last time it was checked.
   * If the value has changed, it:
   *
   * <ol>
   * <li>Fires the <a href="#!/api/Ext.form.field.Field-event-change" rel="Ext.form.field.Field-event-change" class="docClass">change event</a>,</li>
   * <li>Performs validation if the <a href="#!/api/Ext.form.field.Field-cfg-validateOnChange" rel="Ext.form.field.Field-cfg-validateOnChange" class="docClass">validateOnChange</a> config is enabled, firing the
   * <a href="#!/api/Ext.form.field.Field-event-validitychange" rel="Ext.form.field.Field-event-validitychange" class="docClass">validitychange event</a> if the validity has changed, and</li>
   * <li>Checks the <a href="#!/api/Ext.form.field.Field-method-isDirty" rel="Ext.form.field.Field-method-isDirty" class="docClass">dirty state</a> of the field and fires the <a href="#!/api/Ext.form.field.Field-event-dirtychange" rel="Ext.form.field.Field-event-dirtychange" class="docClass">dirtychange event</a>
   * if it has changed.</li>
   * </ol>
   */
  public native function checkChange():void;

  /**
   * Checks the <a href="#!/api/Ext.form.field.Field-method-isDirty" rel="Ext.form.field.Field-method-isDirty" class="docClass">isDirty</a> state of the field and if it has changed since the last time it was checked,
   * fires the <a href="#!/api/Ext.form.field.Field-event-dirtychange" rel="Ext.form.field.Field-event-dirtychange" class="docClass">dirtychange</a> event.
   */
  public native function checkDirty():void;

  /**
   * Cleans up values initialized by this Field mixin on the current instance.
   * Components using this mixin should call this method before being destroyed.
   */
  public native function cleanupField():void;

  /**
   * Clear any invalid styles/messages for this field. Components using this mixin should implement this method to
   * update the components rendering to clear any existing messages.
   *
   * <p><strong>Note</strong>: this method does not cause the Field's <a href="#!/api/Ext.form.field.Field-method-validate" rel="Ext.form.field.Field-method-validate" class="docClass">validate</a> or <a href="#!/api/Ext.form.field.Field-method-isValid" rel="Ext.form.field.Field-method-isValid" class="docClass">isValid</a> methods to return <code>true</code>
   * if the value does not <em>pass</em> validation. So simply clearing a field's errors will not necessarily allow
   * submission of forms submitted with the <a href="#!/api/Ext.form.action.Submit-cfg-clientValidation" rel="Ext.form.action.Submit-cfg-clientValidation" class="docClass">Ext.form.action.Submit.clientValidation</a> option set.</p>
   */
  public native function clearInvalid():void;

  /**
   * Only relevant if the instance's <a href="#!/api/Ext.form.field.Field-method-isFileUpload" rel="Ext.form.field.Field-method-isFileUpload" class="docClass">isFileUpload</a> method returns true. Returns a reference to the file input
   * DOM element holding the user's selected file. The input will be appended into the submission form and will not be
   * returned, so this method should also create a replacement.
   */
  public native function extractFileInput():js.HTMLElement;

  /**
   * Returns an Array of all checkboxes in the container which are currently checked
   * @return Array of <a href="#!/api/Ext.form.field.Checkbox" rel="Ext.form.field.Checkbox" class="docClass">Ext.form.field.Checkbox</a> components
   */
  public native function getChecked():Array;

  /**
   * Runs CheckboxGroup's validations and returns an array of any errors. The only error by default is if allowBlank
   * is set to false and no items are checked.
   * @param value The value to get errors for (defaults to the current field value)
   * @return Array of all validation errors
   */
  public native function getErrors(value:*):Array;

  /**
   * Returns the <a href="#!/api/Ext.form.field.Field-cfg-name" rel="Ext.form.field.Field-cfg-name" class="docClass">name</a> attribute of the field. This is used as the parameter name
   * when including the field value in a <a href="#!/api/Ext.form.Basic-method-submit" rel="Ext.form.Basic-method-submit" class="docClass">form submit()</a>.
   * @return name The field <a href="#!/api/Ext.form.field.Field-cfg-name" rel="Ext.form.field.Field-cfg-name" class="docClass">name</a>
   */
  public native function getName():String;

  /**
   * Returns the value of <a href="#!/api/Ext.form.field.Field-cfg-validation" rel="Ext.form.field.Field-cfg-validation" class="docClass">validation</a>.
   */
  public native function getValidation():*;

  /**
   * Returns an object containing the values of all checked checkboxes within the group. Each key-value pair in the
   * object corresponds to a checkbox <a href="#!/api/Ext.form.field.Checkbox-cfg-name" rel="Ext.form.field.Checkbox-cfg-name" class="docClass">name</a>. If there is only one checked checkbox
   * with a particular name, the value of that pair will be the String <a href="#!/api/Ext.form.field.Checkbox-cfg-inputValue" rel="Ext.form.field.Checkbox-cfg-inputValue" class="docClass">inputValue</a> of that checkbox. If there are multiple checked checkboxes with that name, the value of that pair
   * will be an Array of the selected inputValues.
   *
   * <p>The object format returned from this method can also be passed directly to the <a href="#!/api/Ext.form.CheckboxGroup-method-setValue" rel="Ext.form.CheckboxGroup-method-setValue" class="docClass">setValue</a> method.</p>
   *
   * <p>NOTE: In Ext 3, this method returned an array of Checkbox components; this was changed to make it more consistent
   * with other field components and with the <a href="#!/api/Ext.form.CheckboxGroup-method-setValue" rel="Ext.form.CheckboxGroup-method-setValue" class="docClass">setValue</a> argument signature. If you need the old behavior in
   * Ext 4+, use the <a href="#!/api/Ext.form.CheckboxGroup-method-getChecked" rel="Ext.form.CheckboxGroup-method-getChecked" class="docClass">getChecked</a> method instead.</p>
   */
  public native function getValue():*;

  /**
   * Initializes this Field mixin on the current instance. Components using this mixin should call this method during
   * their own initialization process.
   */
  public native function initField():void;

  /**
   * Returns whether this Field is a file upload field; if it returns true, forms will use special techniques for
   * <a href="#!/api/Ext.form.Basic-method-submit" rel="Ext.form.Basic-method-submit" class="docClass">submitting the form</a> via AJAX. See <a href="#!/api/Ext.form.Basic-method-hasUpload" rel="Ext.form.Basic-method-hasUpload" class="docClass">Ext.form.Basic.hasUpload</a> for details. If
   * this returns true, the <a href="#!/api/Ext.form.field.Field-method-extractFileInput" rel="Ext.form.field.Field-method-extractFileInput" class="docClass">extractFileInput</a> method must also be implemented to return the corresponding file
   * input element.
   */
  public native function isFileUpload():Boolean;

  /**
   * Returns whether or not the field value is currently valid by <a href="#!/api/Ext.form.field.Field-method-getErrors" rel="Ext.form.field.Field-method-getErrors" class="docClass">validating</a> the field's current
   * value. The <a href="#!/api/Ext.form.field.Field-event-validitychange" rel="Ext.form.field.Field-event-validitychange" class="docClass">validitychange</a> event will not be fired; use <a href="#!/api/Ext.form.field.Field-method-validate" rel="Ext.form.field.Field-method-validate" class="docClass">validate</a> instead if you want the event
   * to fire. <strong>Note</strong>: <a href="#!/api/Ext.form.field.Field-cfg-disabled" rel="Ext.form.field.Field-cfg-disabled" class="docClass">disabled</a> fields are always treated as valid.
   *
   * <p>Implementations are encouraged to ensure that this method does not have side-effects such as triggering error
   * message display.</p>
   * @return True if the value is valid, else false
   */
  public native function isValid():Boolean;

  /**
   * Display one or more error messages associated with this field, using
   * <a href="#!/api/Ext.form.Labelable-cfg-msgTarget" rel="Ext.form.Labelable-cfg-msgTarget" class="docClass">Ext.form.Labelable.msgTarget</a> to determine how to display the messages and
   * applying <a href="#!/api/Ext.form.Labelable-cfg-invalidCls" rel="Ext.form.Labelable-cfg-invalidCls" class="docClass">Ext.form.Labelable.invalidCls</a> to the field's UI element.
   *
   * <pre><code>var formPanel = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a>', {
   *     title: 'Contact Info',
   *     width: 300,
   *     bodyPadding: 10,
   *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
   *     items: [{
   *         xtype: 'textfield',
   *         name: 'name',
   *         id: 'nameId',
   *         fieldLabel: 'Name'
   *     }],
   *     bbar: [{
   *         text: 'Mark both fields invalid',
   *         handler: function() {
   *             var nameField = formPanel.getForm().findField('name');
   *             nameField.markInvalid('Name invalid message');
   *
   *             // multiple error string syntax
   *             // nameField.markInvalid(['First message', 'Second message']);
   *         }
   *     }]
   * });
   * </code></pre>
   *
   * <p><strong>Note</strong>: this method does not cause the Field's <a href="#!/api/Ext.form.field.Field-method-validate" rel="Ext.form.field.Field-method-validate" class="docClass">validate</a> or
   * <a href="#!/api/Ext.form.field.Field-method-isValid" rel="Ext.form.field.Field-method-isValid" class="docClass">isValid</a> methods to return <code>false</code> if the value does <em>pass</em> validation.
   * So simply marking a Field as invalid will not prevent submission of forms
   * submitted with the <a href="#!/api/Ext.form.action.Submit-cfg-clientValidation" rel="Ext.form.action.Submit-cfg-clientValidation" class="docClass">Ext.form.action.Submit.clientValidation</a> option set.</p>
   * @param errors The validation message(s) to display.
   */
  public native function markInvalid(errors:*):void;

  /**
   * Resets the checked state of all <a href="#!/api/Ext.form.field.Checkbox" rel="Ext.form.field.Checkbox" class="docClass">checkboxes</a> in the group to their originally
   * loaded values and clears any validation messages.
   * See <a href="#!/api/Ext.form.Basic" rel="Ext.form.Basic" class="docClass">Ext.form.Basic</a>.<a href="#!/api/Ext.form.Basic-cfg-trackResetOnLoad" rel="Ext.form.Basic-cfg-trackResetOnLoad" class="docClass">trackResetOnLoad</a>
   */
  public native function reset():void;

  /**
   * Sets the value of <a href="#!/api/Ext.form.field.Field-cfg-validation" rel="Ext.form.field.Field-cfg-validation" class="docClass">validation</a>.
   * @param validation The new value.
   */
  public native function setValidation(validation:*):void;

  /**
   * Sets the value(s) of all checkboxes in the group. The expected format is an Object of name-value pairs
   * corresponding to the names of the checkboxes in the group. Each pair can have either a single or multiple values:
   *
   * <ul>
   * <li>A single Boolean or String value will be passed to the <code>setValue</code> method of the checkbox with that name.
   * See the rules in <a href="#!/api/Ext.form.field.Checkbox-method-setValue" rel="Ext.form.field.Checkbox-method-setValue" class="docClass">Ext.form.field.Checkbox.setValue</a> for accepted values.</li>
   * <li>An Array of String values will be matched against the <a href="#!/api/Ext.form.field.Checkbox-cfg-inputValue" rel="Ext.form.field.Checkbox-cfg-inputValue" class="docClass">inputValue</a>
   * of checkboxes in the group with that name; those checkboxes whose inputValue exists in the array will be
   * checked and others will be unchecked.</li>
   * </ul>
   *
   *
   * <p>If a checkbox's name is not in the mapping at all, it will be unchecked.</p>
   *
   * <p>An example:</p>
   *
   * <pre><code>var myCheckboxGroup = new <a href="#!/api/Ext.form.CheckboxGroup" rel="Ext.form.CheckboxGroup" class="docClass">Ext.form.CheckboxGroup</a>({
   *     columns: 3,
   *     items: [{
   *         name: 'cb1',
   *         boxLabel: 'Single 1'
   *     }, {
   *         name: 'cb2',
   *         boxLabel: 'Single 2'
   *     }, {
   *         name: 'cb3',
   *         boxLabel: 'Single 3'
   *     }, {
   *         name: 'cbGroup',
   *         boxLabel: 'Grouped 1'
   *         inputValue: 'value1'
   *     }, {
   *         name: 'cbGroup',
   *         boxLabel: 'Grouped 2'
   *         inputValue: 'value2'
   *     }, {
   *         name: 'cbGroup',
   *         boxLabel: 'Grouped 3'
   *         inputValue: 'value3'
   *     }]
   * });
   *
   * myCheckboxGroup.setValue({
   *     cb1: true,
   *     cb3: false,
   *     cbGroup: ['value1', 'value3']
   * });
   * </code></pre>
   *
   * <p>The above code will cause the checkbox named 'cb1' to be checked, as well as the first and third checkboxes named
   * 'cbGroup'. The other three checkboxes will be unchecked.</p>
   * @param value The value to set
   * @return this
   */
  public native function setValue(value:*):ext.form.field.IField;

  /**
   * @inheritDoc
   */
  public native function getModelData(includeEmptyText:Object, isSubmitting:Object):Object;

  /**
   * @inheritDoc
   */
  public native function getSubmitData():Object;

  /**
   * @inheritDoc
   */
  public native function isDirty():Boolean;

  /**
   * @inheritDoc
   */
  public native function isEqual(value1:Object, value2:Object):Boolean;

  /**
   * @inheritDoc
   */
  public native function resetOriginalValue():void;

  /**
   * @inheritDoc
   */
  public native function validate():Boolean;

  /**
   * Initializes the field's value based on the initial config. If the <a href="#!/api/Ext.form.CheckboxGroup-cfg-value" rel="Ext.form.CheckboxGroup-cfg-value" class="docClass">value</a> config is specified then we use
   * that to set the value; otherwise we initialize the originalValue by querying the values of all sub-checkboxes
   * after they have been initialized.
   */
  public native function initValue():void;

  /**
   * Template method before a field is reset.
   */
  protected native function beforeReset():void;
}
}