package ext.form {
import ext.data.Model;
import ext.form.field.Field;
import ext.panel.Panel;

[Uses("ext.panel.Panel")]
[Uses("ext.form.FieldAncestor")]
[Uses("ext.Mixin")]
[Uses("ext.Base")]
[Uses("ext.container.DockingContainer")]
[Uses("ext.container.Container")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.mixin.Container")]
[Uses("ext.Component")]
[Uses("ext.util.Positionable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Animate")]
[Uses("ext.util.Floating")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.state.Stateful")]
[Native("Ext.form.Panel", require)]
/**
 * Fires when an action is completed.
 * @eventType ext.form.events.FormPanel_actionEvent.ACTION_COMPLETE
 */
[Event(name = 'onActionComplete', type = 'ext.form.events.FormPanel_actionEvent')]
/**
 * Fires when an action fails.
 * @eventType ext.form.events.FormPanel_actionEvent.ACTION_FAILED
 */
[Event(name = 'onActionFailed', type = 'ext.form.events.FormPanel_actionEvent')]
/**
 * Fires before any action is performed. Return false to cancel the action.
 * @eventType ext.form.events.FormPanel_actionEvent.BEFORE_ACTION
 */
[Event(name = 'onBeforeAction', type = 'ext.form.events.FormPanel_actionEvent')]
/**
 * Fires when the dirty state of the entire form changes.
 * @eventType ext.form.events.FormPanel_dirtyEvent.DIRTY_CHANGE
 */
[Event(name = 'onDirtyChange', type = 'ext.form.events.FormPanel_dirtyEvent')]
/**
 * Fires when the active error message is changed for any one of the <a href="#!/api/Ext.form.Labelable" rel="Ext.form.Labelable" class="docClass">Ext.form.Labelable</a> instances
 * within this container.
 * @eventType ext.form.events.FormPanel_field_errorEvent.FIELD_ERROR_CHANGE
 */
[Event(name = 'onFieldErrorChange', type = 'ext.form.events.FormPanel_field_errorEvent')]
/**
 * Fires when the validity state of any one of the <a href="#!/api/Ext.form.field.Field" rel="Ext.form.field.Field" class="docClass">Ext.form.field.Field</a> instances within this
 * container changes.
 * @eventType ext.form.events.FormPanel_field_isValidEvent.FIELD_VALIDITY_CHANGE
 */
[Event(name = 'onFieldValidityChange', type = 'ext.form.events.FormPanel_field_isValidEvent')]
/**
 * Fires when the validity of the entire form changes.
 * @eventType ext.form.events.FormPanel_validEvent.VALIDITY_CHANGE
 */
[Event(name = 'onValidityChange', type = 'ext.form.events.FormPanel_validEvent')]
/**
 * FormPanel provides a standard container for forms. It is essentially a standard <a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a> which
 * automatically creates a <a href="#!/api/Ext.form.Basic" rel="Ext.form.Basic" class="docClass">BasicForm</a> for managing any <a href="#!/api/Ext.form.field.Field" rel="Ext.form.field.Field" class="docClass">Ext.form.field.Field</a>
 * objects that are added as descendants of the panel. It also includes conveniences for configuring and
 * working with the BasicForm and the collection of Fields.
 *
 * <h1>Layout</h1>
 *
 * <p>By default, FormPanel is configured with <code><a href="#!/api/Ext.layout.container.Anchor" rel="Ext.layout.container.Anchor" class="docClass">layout:'anchor'</a></code> for
 * the layout of its immediate child items. This can be changed to any of the supported container layouts.
 * The layout of sub-containers is configured in <a href="#!/api/Ext.container.Container-cfg-layout" rel="Ext.container.Container-cfg-layout" class="docClass">the standard way</a>.</p>
 *
 * <h1>BasicForm</h1>
 *
 * <p>FormPanel class accepts all
 * of the config options supported by the <a href="#!/api/Ext.form.Basic" rel="Ext.form.Basic" class="docClass">Ext.form.Basic</a> class, and will pass them along to
 * the internal BasicForm when it is created.</p>
 *
 * <p>The following events fired by the BasicForm will be re-fired by the FormPanel and can therefore be
 * listened for on the FormPanel itself:</p>
 *
 * <ul>
 * <li><a href="#!/api/Ext.form.Basic-event-beforeaction" rel="Ext.form.Basic-event-beforeaction" class="docClass">beforeaction</a></li>
 * <li><a href="#!/api/Ext.form.Basic-event-actionfailed" rel="Ext.form.Basic-event-actionfailed" class="docClass">actionfailed</a></li>
 * <li><a href="#!/api/Ext.form.Basic-event-actioncomplete" rel="Ext.form.Basic-event-actioncomplete" class="docClass">actioncomplete</a></li>
 * <li><a href="#!/api/Ext.form.Basic-event-validitychange" rel="Ext.form.Basic-event-validitychange" class="docClass">validitychange</a></li>
 * <li><a href="#!/api/Ext.form.Basic-event-dirtychange" rel="Ext.form.Basic-event-dirtychange" class="docClass">dirtychange</a></li>
 * </ul>
 *
 *
 * <h1>Field Defaults</h1>
 *
 * <p>The <a href="#!/api/Ext.form.Panel-cfg-fieldDefaults" rel="Ext.form.Panel-cfg-fieldDefaults" class="docClass">fieldDefaults</a> config option conveniently allows centralized configuration of default values
 * for all fields added as descendants of the FormPanel. Any config option recognized by implementations
 * of <a href="#!/api/Ext.form.Labelable" rel="Ext.form.Labelable" class="docClass">Ext.form.Labelable</a> may be included in this object. See the <a href="#!/api/Ext.form.Panel-cfg-fieldDefaults" rel="Ext.form.Panel-cfg-fieldDefaults" class="docClass">fieldDefaults</a> documentation
 * for details of how the defaults are applied.</p>
 *
 * <h1>Form Validation</h1>
 *
 * <p>With the default configuration, form fields are validated on-the-fly while the user edits their values.
 * This can be controlled on a per-field basis (or via the <a href="#!/api/Ext.form.Panel-cfg-fieldDefaults" rel="Ext.form.Panel-cfg-fieldDefaults" class="docClass">fieldDefaults</a> config) with the field
 * config properties <a href="#!/api/Ext.form.field.Field-cfg-validateOnChange" rel="Ext.form.field.Field-cfg-validateOnChange" class="docClass">Ext.form.field.Field.validateOnChange</a> and <a href="#!/api/Ext.form.field.Base-cfg-checkChangeEvents" rel="Ext.form.field.Base-cfg-checkChangeEvents" class="docClass">Ext.form.field.Base.checkChangeEvents</a>,
 * and the FormPanel's config properties <a href="#!/api/Ext.form.Panel-cfg-pollForChanges" rel="Ext.form.Panel-cfg-pollForChanges" class="docClass">pollForChanges</a> and <a href="#!/api/Ext.form.Panel-cfg-pollInterval" rel="Ext.form.Panel-cfg-pollInterval" class="docClass">pollInterval</a>.</p>
 *
 * <p>Any component within the FormPanel can be configured with <code>formBind: true</code>. This will cause that
 * component to be automatically disabled when the form is invalid, and enabled when it is valid. This is most
 * commonly used for Button components to prevent submitting the form in an invalid state, but can be used on
 * any component type.</p>
 *
 * <p>For more information on form validation see the following:</p>
 *
 * <ul>
 * <li><a href="#!/api/Ext.form.field.Field-cfg-validateOnChange" rel="Ext.form.field.Field-cfg-validateOnChange" class="docClass">Ext.form.field.Field.validateOnChange</a></li>
 * <li><a href="#!/api/Ext.form.Panel-cfg-pollForChanges" rel="Ext.form.Panel-cfg-pollForChanges" class="docClass">pollForChanges</a> and <a href="#!/api/Ext.form.Panel-cfg-pollInterval" rel="Ext.form.Panel-cfg-pollInterval" class="docClass">pollInterval</a></li>
 * <li><a href="#!/api/Ext.form.field.VTypes" rel="Ext.form.field.VTypes" class="docClass">Ext.form.field.VTypes</a></li>
 * <li><a href="#!/api/Ext.form.Basic-method-doAction" rel="Ext.form.Basic-method-doAction" class="docClass">BasicForm.doAction clientValidation notes</a></li>
 * </ul>
 *
 *
 * <h1>Form Submission</h1>
 *
 * <p>By default, Ext Forms are submitted through Ajax, using <a href="#!/api/Ext.form.action.Action" rel="Ext.form.action.Action" class="docClass">Ext.form.action.Action</a>. See the documentation for
 * <a href="#!/api/Ext.form.Basic" rel="Ext.form.Basic" class="docClass">Ext.form.Basic</a> for details.</p>
 *
 * <h1>Example usage</h1>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a>', {
 *     title: 'Simple Form',
 *     bodyPadding: 5,
 *     width: 350,
 *
 *     // The form will submit an AJAX request to this URL when submitted
 *     url: 'save-form.php',
 *
 *     // Fields will be arranged vertically, stretched to full width
 *     layout: 'anchor',
 *     defaults: {
 *         anchor: '100%'
 *     },
 *
 *     // The fields
 *     defaultType: 'textfield',
 *     items: [{
 *         fieldLabel: 'First Name',
 *         name: 'first',
 *         allowBlank: false
 *     },{
 *         fieldLabel: 'Last Name',
 *         name: 'last',
 *         allowBlank: false
 *     }],
 *
 *     // Reset and Submit buttons
 *     buttons: [{
 *         text: 'Reset',
 *         handler: function() {
 *             this.up('form').getForm().reset();
 *         }
 *     }, {
 *         text: 'Submit',
 *         formBind: true, //only enabled once the form is valid
 *         disabled: true,
 *         handler: function() {
 *             var form = this.up('form').getForm();
 *             if (form.isValid()) {
 *                 form.submit({
 *                     success: function(form, action) {
 *                        <a href="#!/api/Ext.MessageBox-method-alert" rel="Ext.MessageBox-method-alert" class="docClass">Ext.Msg.alert</a>('Success', action.result.msg);
 *                     },
 *                     failure: function(form, action) {
 *                         <a href="#!/api/Ext.MessageBox-method-alert" rel="Ext.MessageBox-method-alert" class="docClass">Ext.Msg.alert</a>('Failed', action.result.msg);
 *                     }
 *                 });
 *             }
 *         }
 *     }],
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 * });
 * </code></pre>
 */
public class FormPanel extends ext.panel.Panel implements ext.form.IFieldAncestor {
  /**
   * @param config @inheritDoc
   */
  public function FormPanel(config:ext.form.FormPanel = null) {
    super();
  }

  [ExtConfig]
  /**
   * The <a href="#!/api/Ext.container.Container-cfg-layout" rel="Ext.container.Container-cfg-layout" class="docClass">Ext.container.Container.layout</a> for the form panel's immediate child items.
   * @default 'anchor'
   * @see #setLayout()
   */
  override public native function get layout():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  override public native function set layout(value:*):void;

  [ExtConfig]
  /**
   * If specified, load and submit actions will be handled with <a href="#!/api/Ext.form.action.DirectLoad" rel="Ext.form.action.DirectLoad" class="docClass">DirectLoad</a>
   * and <a href="#!/api/Ext.form.action.DirectSubmit" rel="Ext.form.action.DirectSubmit" class="docClass">DirectSubmit</a>.  Methods which have been imported by
   * <a href="#!/api/Ext.direct.Manager" rel="Ext.direct.Manager" class="docClass">Ext.direct.Manager</a> can be specified here to load and submit forms. API methods may also be
   * specified as strings. See <a href="#!/api/Ext.data.proxy.Direct-cfg-directFn" rel="Ext.data.proxy.Direct-cfg-directFn" class="docClass">Ext.data.proxy.Direct.directFn</a>.  Such as the following:
   *
   * <pre><code>api: {
   *     load: App.ss.MyProfile.load,
   *     submit: App.ss.MyProfile.submit
   * }
   * </code></pre>
   *
   * <p>Load actions can use <a href="#!/api/Ext.form.Panel-cfg-paramOrder" rel="Ext.form.Panel-cfg-paramOrder" class="docClass">paramOrder</a> or <a href="#!/api/Ext.form.Panel-cfg-paramsAsHash" rel="Ext.form.Panel-cfg-paramsAsHash" class="docClass">paramsAsHash</a> to customize how the load method
   * is invoked.  Submit actions will always use a standard form submit. The <code>formHandler</code> configuration
   * (see Ext.direct.RemotingProvider#action) must be set on the associated server-side method which has
   * been imported by <a href="#!/api/Ext.direct.Manager" rel="Ext.direct.Manager" class="docClass">Ext.direct.Manager</a>.</p>
   */
  public native function get api():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set api(value:Object):void;

  [ExtConfig]
  /**
   * Parameters to pass with all requests. e.g. baseParams: <code>{id: '123', foo: 'bar'}</code>.
   *
   * <p>Parameters are encoded as standard HTTP parameters using <a href="#!/api/Ext.Object-method-toQueryString" rel="Ext.Object-method-toQueryString" class="docClass">Ext.Object.toQueryString</a>.</p>
   */
  public native function get baseParams():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set baseParams(value:Object):void;

  [ExtConfig]
  /**
   * An <a href="#!/api/Ext.data.reader.Reader" rel="Ext.data.reader.Reader" class="docClass">Ext.data.reader.Reader</a> (e.g. <a href="#!/api/Ext.data.reader.Xml" rel="Ext.data.reader.Xml" class="docClass">Ext.data.reader.Xml</a>) instance or
   * configuration to be used to read field error messages returned from 'submit' actions.
   * This is optional as there is built-in support for processing JSON responses.
   *
   * <p>The Records which provide messages for the invalid Fields must use the
   * Field name (or id) as the Record ID, and must contain a field called 'msg'
   * which contains the error message.</p>
   *
   * <p>The errorReader does not have to be a full-blown implementation of a
   * Reader. It simply needs to implement a <code>read(xhr)</code> function
   * which returns an Array of Records in an object with the following
   * structure:</p>
   *
   * <pre><code>{
   *     records: recordArray
   * }
   * </code></pre>
   */
  public native function get errorReader():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set errorReader(value:*):void;

  [ExtConfig]
  /**
   * If specified, the properties in this object are used as default config values for each <a href="#!/api/Ext.form.Labelable" rel="Ext.form.Labelable" class="docClass">Ext.form.Labelable</a>
   * instance (e.g. <a href="#!/api/Ext.form.field.Base" rel="Ext.form.field.Base" class="docClass">Ext.form.field.Base</a> or <a href="#!/api/Ext.form.FieldContainer" rel="Ext.form.FieldContainer" class="docClass">Ext.form.FieldContainer</a>) that is added as a descendant of
   * this container. Corresponding values specified in an individual field's own configuration, or from the <a href="#!/api/Ext.container.Container-cfg-defaults" rel="Ext.container.Container-cfg-defaults" class="docClass">defaults config</a> of its parent container, will take precedence. See the
   * documentation for <a href="#!/api/Ext.form.Labelable" rel="Ext.form.Labelable" class="docClass">Ext.form.Labelable</a> to see what config options may be specified in the fieldDefaults.
   *
   * <p>Example:</p>
   *
   * <pre><code>new <a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a>({
   *     fieldDefaults: {
   *         labelAlign: 'left',
   *         labelWidth: 100
   *     },
   *     items: [{
   *         xtype: 'fieldset',
   *         defaults: {
   *             labelAlign: 'top'
   *         },
   *         items: [{
   *             name: 'field1'
   *         }, {
   *             name: 'field2'
   *         }]
   *     }, {
   *         xtype: 'fieldset',
   *         items: [{
   *             name: 'field3',
   *             labelWidth: 150
   *         }, {
   *             name: 'field4'
   *         }]
   *     }]
   * });
   * </code></pre>
   *
   * <p>In this example, field1 and field2 will get labelAlign:'top' (from the fieldset's defaults) and labelWidth:100
   * (from fieldDefaults), field3 and field4 will both get labelAlign:'left' (from fieldDefaults and field3 will use
   * the labelWidth:150 from its own config.</p>
   */
  public native function get fieldDefaults():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set fieldDefaults(value:Object):void;

  [ExtConfig]
  /**
   * If set to true, the field values are sent as JSON in the request body.
   * All of the field values, plus any additional params configured via <a href="#!/api/Ext.form.Panel-cfg-baseParams" rel="Ext.form.Panel-cfg-baseParams" class="docClass">baseParams</a>
   * and/or the <code>options</code> to <a href="#!/api/Ext.form.Panel-method-submit" rel="Ext.form.Panel-method-submit" class="docClass">submit</a>, will be included in the values POSTed in the body of the request.
   */
  public native function get jsonSubmit():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set jsonSubmit(value:Boolean):void;

  [ExtConfig]
  /**
   * The request method to use (GET or POST) for form actions if one isn't supplied in the action options.
   */
  public native function get method():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set method(value:String):void;

  [ExtConfig]
  /**
   * A list of params to be executed server side. Only used for the <a href="#!/api/Ext.form.Panel-cfg-api" rel="Ext.form.Panel-cfg-api" class="docClass">api</a> <code>load</code>
   * configuration.
   *
   * <p>Specify the params in the order in which they must be executed on the
   * server-side as either (1) an Array of String values, or (2) a String of params
   * delimited by either whitespace, comma, or pipe. For example,
   * any of the following would be acceptable:</p>
   *
   * <pre><code>paramOrder: ['param1','param2','param3']
   * paramOrder: 'param1 param2 param3'
   * paramOrder: 'param1,param2,param3'
   * paramOrder: 'param1|param2|param'
   * </code></pre>
   */
  public native function get paramOrder():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set paramOrder(value:*):void;

  [ExtConfig]
  /**
   * Only used for the <a href="#!/api/Ext.form.Panel-cfg-api" rel="Ext.form.Panel-cfg-api" class="docClass">api</a> <code>load</code> configuration. If true, parameters will be sent as a
   * single hash collection of named arguments. Providing a <a href="#!/api/Ext.form.Panel-cfg-paramOrder" rel="Ext.form.Panel-cfg-paramOrder" class="docClass">paramOrder</a> nullifies this
   * configuration.
   */
  public native function get paramsAsHash():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set paramsAsHash(value:Boolean):void;

  [ExtConfig]
  /**
   * If set to <code>true</code>, sets up an interval task (using the <a href="#!/api/Ext.form.Panel-cfg-pollInterval" rel="Ext.form.Panel-cfg-pollInterval" class="docClass">pollInterval</a>) in which the
   * panel's fields are repeatedly checked for changes in their values. This is in addition to the normal detection
   * each field does on its own input element, and is not needed in most cases. It does, however, provide a
   * means to absolutely guarantee detection of all changes including some edge cases in some browsers which
   * do not fire native events. Defaults to <code>false</code>.
   */
  public native function get pollForChanges():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set pollForChanges(value:Boolean):void;

  [ExtConfig]
  /**
   * Interval in milliseconds at which the form's fields are checked for value changes. Only used if
   * the <a href="#!/api/Ext.form.Panel-cfg-pollForChanges" rel="Ext.form.Panel-cfg-pollForChanges" class="docClass">pollForChanges</a> option is set to <code>true</code>. Defaults to 500 milliseconds.
   */
  public native function get pollInterval():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set pollInterval(value:Number):void;

  [ExtConfig]
  /**
   * An <a href="#!/api/Ext.data.reader.Reader" rel="Ext.data.reader.Reader" class="docClass">Ext.data.reader.Reader</a> (e.g. <a href="#!/api/Ext.data.reader.Xml" rel="Ext.data.reader.Xml" class="docClass">Ext.data.reader.Xml</a>) instance or
   * configuration to be used to read data when executing 'load' actions. This
   * is optional as there is built-in support for processing JSON responses.
   */
  public native function get reader():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set reader(value:*):void;

  [ExtConfig]
  /**
   * If set to true, a standard HTML form submit is used instead of a XHR (Ajax) style form submission.
   * All of the field values, plus any additional params configured via <a href="#!/api/Ext.form.Panel-cfg-baseParams" rel="Ext.form.Panel-cfg-baseParams" class="docClass">baseParams</a>
   * and/or the <code>options</code> to <a href="#!/api/Ext.form.Panel-method-submit" rel="Ext.form.Panel-method-submit" class="docClass">submit</a>, will be included in the values submitted in the form.
   */
  public native function get standardSubmit():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set standardSubmit(value:Boolean):void;

  [ExtConfig]
  /**
   * Timeout for form actions in seconds.
   */
  public native function get timeout():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set timeout(value:Number):void;

  [ExtConfig]
  /**
   * If set to true, <a href="#!/api/Ext.form.Panel-method-reset" rel="Ext.form.Panel-method-reset" class="docClass">reset</a>() resets to the last loaded or
   * <a href="#!/api/Ext.form.Basic-method-setValues" rel="Ext.form.Basic-method-setValues" class="docClass">Ext.form.Basic.setValues</a>() data instead of when the form was first
   * created.
   */
  public native function get trackResetOnLoad():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set trackResetOnLoad(value:Boolean):void;

  [ExtConfig]
  /**
   * The URL to use for form actions if one isn't supplied in the
   * <a href="#!/api/Ext.form.Basic-method-doAction" rel="Ext.form.Basic-method-doAction" class="docClass">doAction</a> options.
   */
  public native function get url():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set url(value:String):void;

  [ExtConfig]
  /**
   * By default wait messages are displayed with <a href="#!/api/Ext.MessageBox-method-wait" rel="Ext.MessageBox-method-wait" class="docClass">Ext.MessageBox.wait</a>. You can target a specific
   * element by passing it or its id or mask the form itself by passing in true.
   */
  public native function get waitMsgTarget():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set waitMsgTarget(value:*):void;

  [ExtConfig]
  /**
   * The default title to show for the waiting message box
   */
  public native function get waitTitle():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set waitTitle(value:String):void;

  /**
   * Forces each field within the form panel to
   * <a href="#!/api/Ext.form.field.Field-method-checkChange" rel="Ext.form.field.Field-method-checkChange" class="docClass">check if its value has changed</a>.
   */
  public native function checkChange():void;

  /**
   * Provides access to the <a href="#!/api/Ext.form.Basic" rel="Ext.form.Basic" class="docClass">Form</a> which this Panel contains.
   * @return The <a href="#!/api/Ext.form.Basic" rel="Ext.form.Basic" class="docClass">Form</a> which this Panel contains.
   */
  public native function getForm():ext.form.BasicForm;

  /**
   * Returns the currently loaded <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a> instance if one was loaded via <a href="#!/api/Ext.form.Panel-method-loadRecord" rel="Ext.form.Panel-method-loadRecord" class="docClass">loadRecord</a>.
   * @return The loaded instance
   */
  public native function getRecord():ext.data.Model;

  /**
   * Convenience function for fetching the current value of each field in the form. This is the same as calling
   * <a href="#!/api/Ext.form.Basic-method-getValues" rel="Ext.form.Basic-method-getValues" class="docClass">this.getForm().getValues()</a>.
   * @param asString If true, will return the key/value collection as a single
   * URL-encoded param string.
   * @default false
   * @param dirtyOnly If true, only fields that are dirty will be included in the result.
   * @default false
   * @param includeEmptyText If true, the configured emptyText of empty fields will be used.
   * @default false
   * @param useDataValues If true, the <a href="#!/api/Ext.form.field.Field-method-getModelData" rel="Ext.form.field.Field-method-getModelData" class="docClass">getModelData</a>
   * method is used to retrieve values from fields, otherwise the <a href="#!/api/Ext.form.field.Field-method-getSubmitData" rel="Ext.form.field.Field-method-getSubmitData" class="docClass">getSubmitData</a>
   * method is used.
   * @default false
   */
  public native function getValues(asString:Boolean = false, dirtyOnly:Boolean = false, includeEmptyText:Boolean = false, useDataValues:Boolean = false):*;

  /**
   * Convenience function to check if the form has any invalid fields. This is the same as calling
   * <a href="#!/api/Ext.form.Basic-method-hasInvalidField" rel="Ext.form.Basic-method-hasInvalidField" class="docClass">this.getForm().hasInvalidField()</a>.
   */
  public native function hasInvalidField():void;

  /**
   * Convenience function to check if the form has any dirty fields. This is the same as calling
   * <a href="#!/api/Ext.form.Basic-method-isDirty" rel="Ext.form.Basic-method-isDirty" class="docClass">this.getForm().isDirty()</a>.
   */
  public native function isDirty():Boolean;

  /**
   * Convenience function to check if the form has all valid fields. This is the same as calling
   * <a href="#!/api/Ext.form.Basic-method-isValid" rel="Ext.form.Basic-method-isValid" class="docClass">this.getForm().isValid()</a>.
   */
  public native function isValid():Boolean;

  /**
   * This is a proxy for the underlying BasicForm's <a href="#!/api/Ext.form.Basic-method-load" rel="Ext.form.Basic-method-load" class="docClass">Ext.form.Basic.load</a> call.
   * @param options The options to pass to the action (see <a href="#!/api/Ext.form.Basic-method-load" rel="Ext.form.Basic-method-load" class="docClass">Ext.form.Basic.load</a> and
   * <a href="#!/api/Ext.form.Basic-method-doAction" rel="Ext.form.Basic-method-doAction" class="docClass">Ext.form.Basic.doAction</a> for details)
   */
  public native function load(options:*):ext.panel.Panel;

  /**
   * Loads an <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a> into this form (internally just calls <a href="#!/api/Ext.form.Basic-method-loadRecord" rel="Ext.form.Basic-method-loadRecord" class="docClass">Ext.form.Basic.loadRecord</a>)
   * See also <a href="#!/api/Ext.form.Basic-cfg-trackResetOnLoad" rel="Ext.form.Basic-cfg-trackResetOnLoad" class="docClass">trackResetOnLoad</a>. The fields in the model are mapped to
   * fields in the form by matching either the <a href="#!/api/Ext.form.field.Base-cfg-name" rel="Ext.form.field.Base-cfg-name" class="docClass">Ext.form.field.Base.name</a> or <a href="#!/api/Ext.Component-cfg-itemId" rel="Ext.Component-cfg-itemId" class="docClass">Ext.Component.itemId</a>.
   * @param record The record to load
   * @return The <a href="#!/api/Ext.form.Basic" rel="Ext.form.Basic" class="docClass">Ext.form.Basic</a> attached to this FormPanel
   */
  public native function loadRecord(record:ext.data.Model):ext.form.BasicForm;

  /**
   * Convenience function reset the form. This is the same as calling
   * <a href="#!/api/Ext.form.Basic-method-reset" rel="Ext.form.Basic-method-reset" class="docClass">this.getForm().reset()</a>.
   * @param resetRecord True to unbind any record set
   * by <a href="#!/api/Ext.form.Basic-method-loadRecord" rel="Ext.form.Basic-method-loadRecord" class="docClass">loadRecord</a>
   * @default false
   * @return this
   */
  public native function reset(resetRecord:Boolean = false):ext.form.BasicForm;

  /**
   * Start an interval task to continuously poll all the fields in the form for changes in their
   * values. This is normally started automatically by setting the <a href="#!/api/Ext.form.Panel-cfg-pollForChanges" rel="Ext.form.Panel-cfg-pollForChanges" class="docClass">pollForChanges</a> config.
   * @param interval The interval in milliseconds at which the check should run.
   */
  public native function startPolling(interval:Number):void;

  /**
   * Stop a running interval task that was started by <a href="#!/api/Ext.form.Panel-method-startPolling" rel="Ext.form.Panel-method-startPolling" class="docClass">startPolling</a>.
   */
  public native function stopPolling():void;

  /**
   * This is a proxy for the underlying BasicForm's <a href="#!/api/Ext.form.Basic-method-submit" rel="Ext.form.Basic-method-submit" class="docClass">Ext.form.Basic.submit</a> call.
   * @param options The options to pass to the action (see <a href="#!/api/Ext.form.Basic-method-submit" rel="Ext.form.Basic-method-submit" class="docClass">Ext.form.Basic.submit</a> and
   * <a href="#!/api/Ext.form.Basic-method-doAction" rel="Ext.form.Basic-method-doAction" class="docClass">Ext.form.Basic.doAction</a> for details)
   */
  public native function submit(options:Object):void;

  /**
   * Persists the values in this form into the passed <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a> object in a beginEdit/endEdit block.
   * If the record is not specified, it will attempt to update (if it exists) the record provided to <a href="#!/api/Ext.form.Panel-method-loadRecord" rel="Ext.form.Panel-method-loadRecord" class="docClass">loadRecord</a>.
   * @param record The record to edit
   * @return The <a href="#!/api/Ext.form.Basic" rel="Ext.form.Basic" class="docClass">Ext.form.Basic</a> attached to this FormPanel
   */
  public native function updateRecord(record:ext.data.Model = null):ext.form.BasicForm;

  /**
   * Initializes the FieldAncestor's state; this must be called from the initComponent method of any components
   * importing this mixin.
   */
  protected native function initFieldAncestor():void;

  /**
   * Fired when the error message of any field within the container changes.
   * @param field The sub-field whose active error changed
   * @param error The new active error message
   */
  protected native function onFieldErrorChange(field:ext.form.Labelable, error:String):void;

  /**
   * Fired when the validity of any field within the container changes.
   * @param field The sub-field whose validity changed
   * @param valid The new validity state
   */
  protected native function onFieldValidityChange(field:ext.form.field.Field, valid:Boolean):void;
}
}