package ext.form {
import ext.IMixin;

[Uses("ext.Mixin")]
[Uses("ext.form.FieldAncestor")]
[Uses("ext.Base")]
[Mixin("ext.form.FieldAncestor")]
/**
 * A mixin for <a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Ext.container.Container</a> components that are likely to have form fields in their
 * items subtree. Adds the following capabilities:
 *
 * <ul>
 * <li>Methods for handling the addition and removal of <a href="#!/api/Ext.form.Labelable" rel="Ext.form.Labelable" class="docClass">Ext.form.Labelable</a> and <a href="#!/api/Ext.form.field.Field" rel="Ext.form.field.Field" class="docClass">Ext.form.field.Field</a>
 * instances at any depth within the container.</li>
 * <li>Events (<a href="#!/api/Ext.form.FieldAncestor-event-fieldvaliditychange" rel="Ext.form.FieldAncestor-event-fieldvaliditychange" class="docClass">fieldvaliditychange</a> and <a href="#!/api/Ext.form.FieldAncestor-event-fielderrorchange" rel="Ext.form.FieldAncestor-event-fielderrorchange" class="docClass">fielderrorchange</a>) for handling changes to the state
 * of individual fields at the container level.</li>
 * <li>Automatic application of <a href="#!/api/Ext.form.FieldAncestor-cfg-fieldDefaults" rel="Ext.form.FieldAncestor-cfg-fieldDefaults" class="docClass">fieldDefaults</a> config properties to each field added within the
 * container, to facilitate uniform configuration of all fields.</li>
 * </ul>
 *
 *
 * <p>This mixin is primarily for internal use by <a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a> and <a href="#!/api/Ext.form.FieldContainer" rel="Ext.form.FieldContainer" class="docClass">Ext.form.FieldContainer</a>,
 * and should not normally need to be used directly.</p>
 * @see ext.form.FieldAncestor
 */
public interface IFieldAncestor extends ext.IMixin {
  [ExtConfig]
  /**
   * If specified, the properties in this object are used as default config values for each <a href="#!/api/Ext.form.Labelable" rel="Ext.form.Labelable" class="docClass">Ext.form.Labelable</a>
   * instance (e.g. <a href="#!/api/Ext.form.field.Base" rel="Ext.form.field.Base" class="docClass">Ext.form.field.Base</a> or <a href="#!/api/Ext.form.FieldContainer" rel="Ext.form.FieldContainer" class="docClass">Ext.form.FieldContainer</a>) that is added as a descendant of
   * this container. Corresponding values specified in an individual field's own configuration, or from the <a href="#!/api/Ext.container.Container-cfg-defaults" rel="Ext.container.Container-cfg-defaults" class="docClass">defaults config</a> of its parent container, will take precedence. See the
   * documentation for <a href="#!/api/Ext.form.Labelable" rel="Ext.form.Labelable" class="docClass">Ext.form.Labelable</a> to see what config options may be specified in the fieldDefaults.
   *
   * <p>Example:</p>
   *
   * <pre><code>new <a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a>({
   *     fieldDefaults: {
   *         labelAlign: 'left',
   *         labelWidth: 100
   *     },
   *     items: [{
   *         xtype: 'fieldset',
   *         defaults: {
   *             labelAlign: 'top'
   *         },
   *         items: [{
   *             name: 'field1'
   *         }, {
   *             name: 'field2'
   *         }]
   *     }, {
   *         xtype: 'fieldset',
   *         items: [{
   *             name: 'field3',
   *             labelWidth: 150
   *         }, {
   *             name: 'field4'
   *         }]
   *     }]
   * });
   * </code></pre>
   *
   * <p>In this example, field1 and field2 will get labelAlign:'top' (from the fieldset's defaults) and labelWidth:100
   * (from fieldDefaults), field3 and field4 will both get labelAlign:'left' (from fieldDefaults and field3 will use
   * the labelWidth:150 from its own config.</p>
   */
  function get fieldDefaults():Object;

  [ExtConfig]
  function set fieldDefaults(value:Object):void;
}
}