package ext.form {
import ext.form.field.IField;

[Uses("ext.form.CheckboxGroup")]
[Uses("ext.form.field.Field")]
[Uses("ext.Base")]
[Uses("ext.form.FieldContainer")]
[Uses("ext.container.Container")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.mixin.Container")]
[Uses("ext.Mixin")]
[Uses("ext.Component")]
[Uses("ext.util.Positionable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Animate")]
[Uses("ext.util.Floating")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.state.Stateful")]
[Uses("ext.form.Labelable")]
[Uses("ext.form.FieldAncestor")]
[Native("Ext.form.RadioGroup", require)]
/**
 * A <a href="#!/api/Ext.form.FieldContainer" rel="Ext.form.FieldContainer" class="docClass">field container</a> which has a specialized layout for arranging
 * <a href="#!/api/Ext.form.field.Radio" rel="Ext.form.field.Radio" class="docClass">Ext.form.field.Radio</a> controls into columns, and provides convenience <a href="#!/api/Ext.form.field.Field" rel="Ext.form.field.Field" class="docClass">Ext.form.field.Field</a>
 * methods for <a href="#!/api/Ext.form.RadioGroup-method-getValue" rel="Ext.form.RadioGroup-method-getValue" class="docClass">getting</a>, <a href="#!/api/Ext.form.RadioGroup-method-setValue" rel="Ext.form.RadioGroup-method-setValue" class="docClass">setting</a>, and <a href="#!/api/Ext.form.RadioGroup-method-validate" rel="Ext.form.RadioGroup-method-validate" class="docClass">validating</a> the
 * group of radio buttons as a whole.
 *
 * <h1>Validation</h1>
 *
 * <p>Individual radio buttons themselves have no default validation behavior, but
 * sometimes you want to require a user to select one of a group of radios. RadioGroup
 * allows this by setting the config <code><a href="#!/api/Ext.form.RadioGroup-cfg-allowBlank" rel="Ext.form.RadioGroup-cfg-allowBlank" class="docClass">allowBlank</a>:false</code>; when the user does not check at
 * one of the radio buttons, the entire group will be highlighted as invalid and the
 * <a href="#!/api/Ext.form.RadioGroup-cfg-blankText" rel="Ext.form.RadioGroup-cfg-blankText" class="docClass">error message</a> will be displayed according to the <a href="#!/api/Ext.form.RadioGroup-cfg-msgTarget" rel="Ext.form.RadioGroup-cfg-msgTarget" class="docClass">msgTarget</a> config.</p>
 *
 * <h1>Layout</h1>
 *
 * <p>The default layout for RadioGroup makes it easy to arrange the radio buttons into
 * columns; see the <a href="#!/api/Ext.form.RadioGroup-cfg-columns" rel="Ext.form.RadioGroup-cfg-columns" class="docClass">columns</a> and <a href="#!/api/Ext.form.RadioGroup-cfg-vertical" rel="Ext.form.RadioGroup-cfg-vertical" class="docClass">vertical</a> config documentation for details. You may also
 * use a completely different layout by setting the <a href="#!/api/Ext.form.RadioGroup-cfg-layout" rel="Ext.form.RadioGroup-cfg-layout" class="docClass">layout</a> to one of the
 * other supported layout types; for instance you may wish to use a custom arrangement
 * of hbox and vbox containers. In that case the Radio components at any depth will
 * still be managed by the RadioGroup's validation.</p>
 *
 * <h1>Example usage</h1>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a>', {
 *     title: 'RadioGroup Example',
 *     width: 300,
 *     height: 125,
 *     bodyPadding: 10,
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     items:[{
 *         xtype: 'radiogroup',
 *         fieldLabel: 'Two Columns',
 *         // Arrange radio buttons into two columns, distributed vertically
 *         columns: 2,
 *         vertical: true,
 *         items: [
 *             { boxLabel: 'Item 1', name: 'rb', inputValue: '1' },
 *             { boxLabel: 'Item 2', name: 'rb', inputValue: '2', checked: true},
 *             { boxLabel: 'Item 3', name: 'rb', inputValue: '3' },
 *             { boxLabel: 'Item 4', name: 'rb', inputValue: '4' },
 *             { boxLabel: 'Item 5', name: 'rb', inputValue: '5' },
 *             { boxLabel: 'Item 6', name: 'rb', inputValue: '6' }
 *         ]
 *     }]
 * });
 * </code></pre>
 */
public class RadioGroup extends ext.form.CheckboxGroup {
  /**
   * @param config @inheritDoc
   */
  public function RadioGroup(config:ext.form.RadioGroup = null) {
    super();
  }

  [ExtConfig]
  /**
   * True to allow every item in the group to be blank.
   * If allowBlank = false and no items are selected at validation time,
   * <a href="#!/api/Ext.form.RadioGroup-cfg-blankText" rel="Ext.form.RadioGroup-cfg-blankText" class="docClass">blankText</a> will be used as the error text.
   * @default true
   */
  override public native function get allowBlank():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set allowBlank(value:Boolean):void;

  [ExtConfig]
  /**
   * An Array of <a href="#!/api/Ext.form.field.Radio" rel="Ext.form.field.Radio" class="docClass">Radio</a>s or Radio config objects to arrange
   * in the group.
   */
  override public native function get items():Array;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set items(value:Array):void;

  /**
   * The value <code>true</code> to identify an object as an instance of this or derived class.
   * @default true
   * @since 6.2.0
   */
  public native function get isRadioGroup():Boolean;

  [ExtConfig]
  /**
   * By default, child <a href="#!/api/Ext.form.field.Radio" rel="Ext.form.field.Radio" class="docClass">radio</a> <code>name</code>s are scoped to the
   * encapsulating <a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">form panel</a> if any, of the document.
   *
   * <p>If you are using multiple <code>RadioGroup</code>s each of which uses the same <code>name</code>
   * configuration in child <a href="#!/api/Ext.form.field.Radio" rel="Ext.form.field.Radio" class="docClass">radio</a>s, configure this as
   * <code>true</code> to scope the names to within this <code>RadioGroup</code></p>
   * @default false
   */
  public native function get local():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set local(value:Boolean):void;

  [ExtConfig]
  /**
   * When set to <code>true</code> the <code>value</code> of this group of <code>radiofield</code> components will be
   * mapped to the <code>inputValue</code> of the checked item. This is, the <code>getValue</code> method
   * will return the <code>inputValue</code> of the checked item while <code>setValue</code> will check the
   * <code>radiofield</code> whose <code>inputValue</code> matches the given value.
   *
   * <p>This field allows the <code>radiogroup</code> to participate in binding an entire group of
   * radio buttons to a single value.</p>
   * @default false
   * @since 6.2.0
   */
  public native function get simpleValue():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set simpleValue(value:Boolean):void;

  /**
   * Sets the value of the radio group. The radio with corresponding name and value will be set.
   * This method is simpler than <a href="#!/api/Ext.form.CheckboxGroup-method-setValue" rel="Ext.form.CheckboxGroup-method-setValue" class="docClass">Ext.form.CheckboxGroup.setValue</a> because only 1 value is allowed
   * for each name. You can use the setValue method as:
   *
   * <pre><code>var form = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a>', {
   *     title       : 'RadioGroup Example',
   *     width       : 300,
   *     bodyPadding : 10,
   *     renderTo    : <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
   *     items       : [
   *         {
   *             xtype      : 'radiogroup',
   *             fieldLabel : 'Group',
   *             items      : [
   *                 { boxLabel : 'Item 1', name : 'rb', inputValue : 1 },
   *                 { boxLabel : 'Item 2', name : 'rb', inputValue : 2 }
   *             ]
   *         }
   *     ],
   *     tbar        : [
   *         {
   *             text    : 'setValue on RadioGroup',
   *             handler : function () {
   *                 form.child('radiogroup').setValue({
   *                     rb : 2
   *                 });
   *             }
   *         }
   *     ]
   * });
   * </code></pre>
   * @param value The value to set
   * @return this
   */
  override public native function setValue(value:*):ext.form.field.IField;
}
}