package ext.form.action {

[Uses("ext.form.action.LoadAction")]
[Uses("ext.form.action.DirectAction")]
[Uses("ext.Mixin")]
[Uses("ext.Base")]
[Uses("ext.form.action.Action")]
[Native("Ext.form.action.DirectLoad", require)]
/**
 * Provides Ext Direct support for loading form data.
 *
 * <p>This example illustrates usage of Ext Direct to load a form.</p>
 *
 * <pre><code>var myFormPanel = new <a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a>({
 *     // configs for FormPanel
 *     title: 'Basic Information',
 *     renderTo: document.body,
 *     width: 300, height: 160,
 *     padding: 10,
 *
 *     // configs apply to child items
 *     defaults: {anchor: '100%'},
 *     defaultType: 'textfield',
 *     items: [{
 *         fieldLabel: 'Name',
 *         name: 'name'
 *     },{
 *         fieldLabel: 'Email',
 *         name: 'email'
 *     },{
 *         fieldLabel: 'Company',
 *         name: 'company'
 *     }],
 *
 *     // configs for BasicForm
 *     api: {
 *         // The server-side method to call for load() requests
 *         load: 'Profile.getBasicInfo',
 *         // The server-side must mark the submit handler as a 'formHandler'
 *         submit: 'Profile.updateBasicInfo'
 *     },
 *     // specify the order for the passed params
 *     paramOrder: ['uid', 'foo']
 * });
 *
 * // load the form
 * myFormPanel.getForm().load({
 *     // pass 2 arguments to server side getBasicInfo method (len=2)
 *     params: {
 *         foo: 'bar',
 *         uid: 34
 *     }
 * });
 * </code></pre>
 *
 * <p>Before using DirectLoad action, make sure you set up Ext Direct remoting provider.
 * See <a href="#!/api/Ext.direct.Manager" rel="Ext.direct.Manager" class="docClass">Ext.direct.Manager</a> for more information.</p>
 *
 * <p>For corresponding submit action, see <a href="#!/api/Ext.form.action.DirectSubmit" rel="Ext.form.action.DirectSubmit" class="docClass">Ext.form.action.DirectSubmit</a>.</p>
 */
public class DirectLoadAction extends ext.form.action.LoadAction implements ext.form.action.IDirectAction {
  /**
   * @param config @inheritDoc
   */
  public function DirectLoadAction(config:ext.form.action.DirectLoadAction = null) {
    super();
  }
}
}