package ext.form.action {

[Uses("ext.form.action.Action")]
[Uses("ext.Base")]
[Native("Ext.form.action.Load", require)]
/**
 * A class which handles loading of data from a server into the Fields of an <a href="#!/api/Ext.form.Basic" rel="Ext.form.Basic" class="docClass">Ext.form.Basic</a>.
 *
 * <p>Instances of this class are only created by a <a href="#!/api/Ext.form.Basic" rel="Ext.form.Basic" class="docClass">Form</a> when <a href="#!/api/Ext.form.Basic-method-load" rel="Ext.form.Basic-method-load" class="docClass">load</a>ing.</p>
 *
 * <h2>Response Packet Criteria</h2>
 *
 * <p>A response packet <strong>must</strong> contain:</p>
 *
 * <ul>
 * <li><strong><code>success</code></strong> property : Boolean</li>
 * <li><strong><code>data</code></strong> property : Object</li>
 * </ul>
 *
 *
 * <p>The <code>data</code> property contains the values of Fields to load. The individual value object for each Field is passed to
 * the Field's <a href="#!/api/Ext.form.field.Field-method-setValue" rel="Ext.form.field.Field-method-setValue" class="docClass">setValue</a> method.</p>
 *
 * <h2>JSON Packets</h2>
 *
 * <p>By default, response packets are assumed to be JSON, so for the following form load call:</p>
 *
 * <pre><code>var myFormPanel = new <a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a>({
 *     title: 'Client and routing info',
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     defaults: {
 *         xtype: 'textfield'
 *     },
 *     items: [{
 *         fieldLabel: 'Client',
 *         name: 'clientName'
 *     }, {
 *         fieldLabel: 'Port of loading',
 *         name: 'portOfLoading'
 *     }, {
 *         fieldLabel: 'Port of discharge',
 *         name: 'portOfDischarge'
 *     }]
 * });
 * myFormPanel.getForm().load({
 *     url: '/getRoutingInfo.php',
 *     params: {
 *         consignmentRef: myConsignmentRef
 *     },
 *     failure: function(form, action) {
 *         <a href="#!/api/Ext.MessageBox-method-alert" rel="Ext.MessageBox-method-alert" class="docClass">Ext.Msg.alert</a>("Load failed", action.result.errorMessage);
 *     }
 * });
 * </code></pre>
 *
 * <p>a <strong>success response</strong> packet may look like this:</p>
 *
 * <pre><code>{
 *     success: true,
 *     data: {
 *         clientName: "Fred. Olsen Lines",
 *         portOfLoading: "FXT",
 *         portOfDischarge: "OSL"
 *     }
 * }
 * </code></pre>
 *
 * <p>while a <strong>failure response</strong> packet may look like this:</p>
 *
 * <pre><code>{
 *     success: false,
 *     errorMessage: "Consignment reference not found"
 * }
 * </code></pre>
 *
 * <p>Other data may be placed into the response for processing the <a href="#!/api/Ext.form.Basic" rel="Ext.form.Basic" class="docClass">Form</a>'s callback or event handler
 * methods. The object decoded from this JSON is available in the <a href="#!/api/Ext.form.action.Action-property-result" rel="Ext.form.action.Action-property-result" class="docClass">result</a> property.</p>
 */
public class LoadAction extends ext.form.action.Action {
  /**
   * @param config @inheritDoc
   */
  public function LoadAction(config:ext.form.action.LoadAction = null) {
    super();
  }
}
}