package ext.form.action {

[Uses("ext.form.action.Action")]
[Uses("ext.Base")]
[Native("Ext.form.action.Submit", require)]
/**
 * A class which handles submission of data from <a href="#!/api/Ext.form.Basic" rel="Ext.form.Basic" class="docClass">Form</a>s and processes the returned response.
 *
 * <p>Instances of this class are only created by a <a href="#!/api/Ext.form.Basic" rel="Ext.form.Basic" class="docClass">Form</a> when
 * <a href="#!/api/Ext.form.Basic-method-submit" rel="Ext.form.Basic-method-submit" class="docClass">submit</a>ting.</p>
 *
 * <h1>Response Packet Criteria</h1>
 *
 * <p>A response packet may contain:</p>
 *
 * <ul>
 * <li><p><strong><code>success</code></strong> property : Boolean - required.</p></li>
 * <li><p><strong><code>errors</code></strong> property : Object - optional, contains error messages for invalid fields.</p></li>
 * </ul>
 *
 *
 * <h1>JSON Packets</h1>
 *
 * <p>By default, response packets are assumed to be JSON, so a typical response packet may look like this:</p>
 *
 * <pre><code>{
 *     success: false,
 *     errors: {
 *         clientCode: "Client not found",
 *         portOfLoading: "This field must not be null"
 *     }
 * }
 * </code></pre>
 *
 * <p>Other data may be placed into the response for processing by the <a href="#!/api/Ext.form.Basic" rel="Ext.form.Basic" class="docClass">Ext.form.Basic</a>'s callback or event handler
 * methods. The object decoded from this JSON is available in the <a href="#!/api/Ext.form.action.Action-property-result" rel="Ext.form.action.Action-property-result" class="docClass">result</a> property.</p>
 *
 * <p>Alternatively, if an <a href="#!/api/Ext.form.Basic-cfg-errorReader" rel="Ext.form.Basic-cfg-errorReader" class="docClass">errorReader</a> is specified as an
 * <a href="#!/api/Ext.data.reader.Xml" rel="Ext.data.reader.Xml" class="docClass">XmlReader</a>:</p>
 *
 * <pre><code>errorReader: new <a href="#!/api/Ext.data.reader.Xml" rel="Ext.data.reader.Xml" class="docClass">Ext.data.reader.Xml</a>({
 *         record : 'field',
 *         success: '&#64;success'
 *     }, [
 *         'id', 'msg'
 *     ]
 * )
 * </code></pre>
 *
 * <p>then the results may be sent back in XML format:</p>
 *
 * <pre><code>&lt;?xml version="1.0" encoding="UTF-8"?&gt;
 * &lt;message success="false"&gt;
 * &lt;errors&gt;
 *     &lt;field&gt;
 *         &lt;id&gt;clientCode&lt;/id&gt;
 *         &lt;msg&gt;&lt;![CDATA[Code not found. &lt;br /&gt;&lt;i&gt;This is a test validation message from the server &lt;/i&gt;]]&gt;&lt;/msg&gt;
 *     &lt;/field&gt;
 *     &lt;field&gt;
 *         &lt;id&gt;portOfLoading&lt;/id&gt;
 *         &lt;msg&gt;&lt;![CDATA[Port not found. &lt;br /&gt;&lt;i&gt;This is a test validation message from the server &lt;/i&gt;]]&gt;&lt;/msg&gt;
 *     &lt;/field&gt;
 * &lt;/errors&gt;
 * &lt;/message&gt;
 * </code></pre>
 *
 * <p>Other elements may be placed into the response XML for processing by the <a href="#!/api/Ext.form.Basic" rel="Ext.form.Basic" class="docClass">Ext.form.Basic</a>'s callback or event
 * handler methods. The XML document is available in the <a href="#!/api/Ext.form.Basic-cfg-errorReader" rel="Ext.form.Basic-cfg-errorReader" class="docClass">errorReader</a>'s
 * <a href="#!/api/Ext.data.reader.Xml-property-xmlData" rel="Ext.data.reader.Xml-property-xmlData" class="docClass">xmlData</a> property.</p>
 */
public class SubmitAction extends ext.form.action.Action {
  /**
   * @param config @inheritDoc
   */
  public function SubmitAction(config:ext.form.action.SubmitAction = null) {
    super();
  }

  [ExtConfig]
  /**
   * Determines whether a Form's fields are validated in a final call to <a href="#!/api/Ext.form.Basic-method-isValid" rel="Ext.form.Basic-method-isValid" class="docClass">isValid</a> prior
   * to submission. Pass false in the Form's submit options to prevent this.
   * @default true
   */
  public native function get clientValidation():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set clientValidation(value:Boolean):void;
}
}