package ext.form.field {
import ext.Component;
import ext.dom.Element;
import ext.form.ILabelable;
import js.HTMLElement;

[Uses("ext.Component")]
[Uses("ext.form.Labelable")]
[Uses("ext.form.field.Field")]
[Uses("ext.util.Positionable")]
[Uses("ext.Base")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Animate")]
[Uses("ext.util.Floating")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.state.Stateful")]
[Native("Ext.form.field.Base", require)]
/**
 * Fires when the value of a field is changed. The value of a field is
 * checked for changes when the field's <a href="#!/api/Ext.form.field.Field-method-setValue" rel="Ext.form.field.Field-method-setValue" class="docClass">setValue</a> method
 * is called and when any of the events listed in
 * <a href="#!/api/Ext.form.field.Base-cfg-checkChangeEvents" rel="Ext.form.field.Base-cfg-checkChangeEvents" class="docClass">checkChangeEvents</a> are fired.
 * @eventType ext.form.field.events.BaseField_newValue_oldValueEvent.CHANGE
 */
[Event(name = 'onChange', type = 'ext.form.field.events.BaseField_newValue_oldValueEvent')]
/**
 * Fires when a change in the field's <a href="#!/api/Ext.form.field.Field-method-isDirty" rel="Ext.form.field.Field-method-isDirty" class="docClass">isDirty</a> state is detected.
 * @eventType ext.form.field.events.BaseField_isDirtyEvent.DIRTY_CHANGE
 */
[Event(name = 'onDirtyChange', type = 'ext.form.field.events.BaseField_isDirtyEvent')]
/**
 * Fires when the active error message is changed via <a href="#!/api/Ext.form.Labelable-method-setActiveError" rel="Ext.form.Labelable-method-setActiveError" class="docClass">setActiveError</a>.
 * @eventType ext.form.field.events.BaseField_errorEvent.ERROR_CHANGE
 */
[Event(name = 'onErrorChange', type = 'ext.form.field.events.BaseField_errorEvent')]
/**
 * Fires when any key related to navigation (arrows, tab, enter, esc, etc.) is pressed. To handle other keys
 * see <a href="#!/api/Ext.util.KeyMap" rel="Ext.util.KeyMap" class="docClass">Ext.util.KeyMap</a>. You can check <a href="#!/api/Ext.event.Event-method-getKey" rel="Ext.event.Event-method-getKey" class="docClass">Ext.event.Event.getKey</a> to determine which key was
 * pressed. For example:
 *
 * <pre><code>var form = new <a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a>({
 *     ...
 *     items: [{
 *             fieldLabel: 'Field 1',
 *             name: 'field1',
 *             allowBlank: false
 *         },{
 *             fieldLabel: 'Field 2',
 *             name: 'field2',
 *             listeners: {
 *                 specialkey: function(field, e){
 *                     // e.HOME, e.END, e.PAGE_UP, e.PAGE_DOWN,
 *                     // e.TAB, e.ESC, arrow keys: e.LEFT, e.RIGHT, e.UP, e.DOWN
 *                     if (e.getKey() == e.ENTER) {
 *                         var form = field.up('form').getForm();
 *                         form.submit();
 *                     }
 *                 }
 *             }
 *         }
 *     ],
 *     ...
 * });
 * </code></pre>
 * @eventType ext.form.field.events.BaseField_eEvent.SPECIAL_KEY
 */
[Event(name = 'onSpecialKey', type = 'ext.form.field.events.BaseField_eEvent')]
/**
 * Fires when a change in the field's validity is detected.
 * @eventType ext.form.field.events.BaseField_isValidEvent.VALIDITY_CHANGE
 */
[Event(name = 'onValidityChange', type = 'ext.form.field.events.BaseField_isValidEvent')]
/**
 * Fires when this field changes its read-only status.
 * @eventType ext.form.field.events.BaseField_ReadEvent.WRITEABLE_CHANGE
 */
[Event(name = 'onWriteableChange', type = 'ext.form.field.events.BaseField_ReadEvent')]
/**
 * Base class for form fields that provides default event handling, rendering, and other common functionality
 * needed by all form field types. Utilizes the <a href="#!/api/Ext.form.field.Field" rel="Ext.form.field.Field" class="docClass">Ext.form.field.Field</a> mixin for value handling and validation,
 * and the <a href="#!/api/Ext.form.Labelable" rel="Ext.form.Labelable" class="docClass">Ext.form.Labelable</a> mixin to provide label and error message display.
 *
 * <p>In most cases you will want to use a subclass, such as <a href="#!/api/Ext.form.field.Text" rel="Ext.form.field.Text" class="docClass">Ext.form.field.Text</a> or <a href="#!/api/Ext.form.field.Checkbox" rel="Ext.form.field.Checkbox" class="docClass">Ext.form.field.Checkbox</a>,
 * rather than creating instances of this class directly. However if you are implementing a custom form field,
 * using this as the parent class is recommended.</p>
 *
 * <h1>Values and Conversions</h1>
 *
 * <p>Because Base implements the Field mixin, it has a main value that can be initialized with the
 * <a href="#!/api/Ext.form.field.Base-cfg-value" rel="Ext.form.field.Base-cfg-value" class="docClass">value</a> config and manipulated via the <a href="#!/api/Ext.form.field.Base-method-getValue" rel="Ext.form.field.Base-method-getValue" class="docClass">getValue</a> and <a href="#!/api/Ext.form.field.Base-method-setValue" rel="Ext.form.field.Base-method-setValue" class="docClass">setValue</a> methods. This main
 * value can be one of many data types appropriate to the current field, for instance a <a href="#!/api/Ext.form.field.Date" rel="Ext.form.field.Date" class="docClass">Date</a>
 * field would use a JavaScript Date object as its value type. However, because the field is rendered as a HTML
 * input, this value data type can not always be directly used in the rendered field.</p>
 *
 * <p>Therefore Base introduces the concept of a "raw value". This is the value of the rendered HTML input field,
 * and is normally a String. The <a href="#!/api/Ext.form.field.Base-method-getRawValue" rel="Ext.form.field.Base-method-getRawValue" class="docClass">getRawValue</a> and <a href="#!/api/Ext.form.field.Base-method-setRawValue" rel="Ext.form.field.Base-method-setRawValue" class="docClass">setRawValue</a> methods can be used to directly
 * work with the raw value, though it is recommended to use getValue and setValue in most cases.</p>
 *
 * <p>Conversion back and forth between the main value and the raw value is handled by the <a href="#!/api/Ext.form.field.Base-method-valueToRaw" rel="Ext.form.field.Base-method-valueToRaw" class="docClass">valueToRaw</a> and
 * <a href="#!/api/Ext.form.field.Base-method-rawToValue" rel="Ext.form.field.Base-method-rawToValue" class="docClass">rawToValue</a> methods. If you are implementing a subclass that uses a non-String value data type, you
 * should override these methods to handle the conversion.</p>
 *
 * <h1>Rendering</h1>
 *
 * <p>The content of the field body is defined by the <a href="#!/api/Ext.form.field.Base-cfg-fieldSubTpl" rel="Ext.form.field.Base-cfg-fieldSubTpl" class="docClass">fieldSubTpl</a> XTemplate, with its argument data
 * created by the <a href="#!/api/Ext.form.field.Base-method-getSubTplData" rel="Ext.form.field.Base-method-getSubTplData" class="docClass">getSubTplData</a> method. Override this template and/or method to create custom
 * field renderings.</p>
 */
public class BaseField extends ext.Component implements ext.form.ILabelable, ext.form.field.IField {
  /**
   * @param config @inheritDoc
   */
  public function BaseField(config:ext.form.field.BaseField = null) {
    super();
  }

  [ExtConfig]
  /**
   * True to disable the field. Disabled Fields will not be <a href="#!/api/Ext.form.Basic-method-submit" rel="Ext.form.Basic-method-submit" class="docClass">submitted</a>.
   * @default false
   * @see #setDisabled()
   */
  override public native function get disabled():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  override public native function set disabled(value:Boolean):void;

  [ExtConfig]
  /**
   * The CSS class to use when the field receives focus
   * @default 'x-form-focus'
   */
  override public native function get focusCls():String;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set focusCls(value:String):void;

  [ExtConfig]
  /**
   * Sets a DOM tabIndex for this field. tabIndex may be set to <code>-1</code> in order to remove
   * the field from the tab rotation.
   *
   * <p><strong>Note:</strong> tabIndex only applies to fields that are rendered.  It does not effect
   * fields built via applyTo</p>
   * @see #getTabIndex()
   * @see #setTabIndex()
   */
  override public native function get tabIndex():Number;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set tabIndex(value:Number):void;

  /**
   * The div Element wrapping the component's contents. Only available after the component has been rendered.
   */
  public native function get bodyEl():ext.dom.Element;

  /**
   * @private
   */
  public native function set bodyEl(value:ext.dom.Element):void;

  /**
   * The div Element that will contain the component's error message(s). Note that depending on the configured
   * <a href="#!/api/Ext.form.Labelable-cfg-msgTarget" rel="Ext.form.Labelable-cfg-msgTarget" class="docClass">msgTarget</a>, this element may be hidden in favor of some other form of presentation, but will always
   * be present in the DOM for use by assistive technologies.
   */
  public native function get errorEl():ext.dom.Element;

  /**
   * @private
   */
  public native function set errorEl(value:ext.dom.Element):void;

  /**
   * The input Element for this Field. Only available after the field has been rendered.
   */
  public native function get inputEl():ext.dom.Element;

  /**
   * @private
   */
  public native function set inputEl(value:ext.dom.Element):void;

  /**
   * Flag denoting that this object is labelable as a field. Always true.
   * @default true
   */
  public native function get isFieldLabelable():Boolean;

  /**
   * @private
   */
  public native function set isFieldLabelable(value:Boolean):void;

  /**
   * Flag denoting that this component is a Field. Always true.
   * @default true
   */
  public native function get isFormField():Boolean;

  /**
   * @private
   */
  public native function set isFormField(value:Boolean):void;

  /**
   * The label Element for this component. Only available after the component has been rendered.
   */
  public native function get labelEl():ext.dom.Element;

  /**
   * @private
   */
  public native function set labelEl(value:ext.dom.Element):void;

  /**
   * The original value of the field as configured in the <a href="#!/api/Ext.form.field.Field-cfg-value" rel="Ext.form.field.Field-cfg-value" class="docClass">value</a> configuration, or as loaded by the last
   * form load operation if the form's <a href="#!/api/Ext.form.Basic-cfg-trackResetOnLoad" rel="Ext.form.Basic-cfg-trackResetOnLoad" class="docClass">trackResetOnLoad</a> setting is <code>true</code>.
   */
  public native function get originalValue():*;

  /**
   * @private
   */
  public native function set originalValue(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * If specified, then the component will be displayed with this value as its active error when first rendered. Use
   * <a href="#!/api/Ext.form.Labelable-method-setActiveError" rel="Ext.form.Labelable-method-setActiveError" class="docClass">setActiveError</a> or <a href="#!/api/Ext.form.Labelable-method-unsetActiveError" rel="Ext.form.Labelable-method-unsetActiveError" class="docClass">unsetActiveError</a> to change it after component creation.
   * @see #getActiveError()
   * @see #setActiveError()
   */
  public native function get activeError():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set activeError(value:String):void;

  [ExtConfig]
  /**
   * The template used to format the Array of error messages passed to <a href="#!/api/Ext.form.Labelable-method-setActiveErrors" rel="Ext.form.Labelable-method-setActiveErrors" class="docClass">setActiveErrors</a> into a single HTML
   * string. if the <a href="#!/api/Ext.form.Labelable-cfg-msgTarget" rel="Ext.form.Labelable-cfg-msgTarget" class="docClass">msgTarget</a> is title, it defaults to a list separated by new lines. Otherwise, it
   * renders each message as an item in an unordered list.
   */
  public native function get activeErrorsTpl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set activeErrorsTpl(value:*):void;

  [ExtConfig]
  /**
   * An optional string or <code>XTemplate</code> configuration to insert in the field markup
   * at the end of the input containing element. If an <code>XTemplate</code> is used, the component's <a href="#!/api/Ext.Component-cfg-renderData" rel="Ext.Component-cfg-renderData" class="docClass">render data</a>
   * serves as the context.
   */
  public native function get afterBodyEl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set afterBodyEl(value:*):void;

  [ExtConfig]
  /**
   * An optional string or <code>XTemplate</code> configuration to insert in the field markup
   * after the label text. If an <code>XTemplate</code> is used, the component's <a href="#!/api/Ext.Component-cfg-renderData" rel="Ext.Component-cfg-renderData" class="docClass">render data</a>
   * serves as the context.
   */
  public native function get afterLabelTextTpl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set afterLabelTextTpl(value:*):void;

  [ExtConfig]
  /**
   * An optional string or <code>XTemplate</code> configuration to insert in the field markup
   * after the label element. If an <code>XTemplate</code> is used, the component's <a href="#!/api/Ext.Component-cfg-renderData" rel="Ext.Component-cfg-renderData" class="docClass">render data</a>
   * serves as the context.
   */
  public native function get afterLabelTpl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set afterLabelTpl(value:*):void;

  [ExtConfig]
  /**
   * An optional string or <code>XTemplate</code> configuration to insert in the field markup
   * after the <a href="#!/api/Ext.form.Labelable-method-getSubTplMarkup" rel="Ext.form.Labelable-method-getSubTplMarkup" class="docClass">subTpl markup</a>. If an <code>XTemplate</code> is used, the
   * component's <a href="#!/api/Ext.Component-cfg-renderData" rel="Ext.Component-cfg-renderData" class="docClass">render data</a> serves as the context.
   */
  public native function get afterSubTpl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set afterSubTpl(value:*):void;

  [ExtConfig]
  /**
   * Localized announcement text for validation errors. This text
   * will be used by Assistive Technologies such as screen readers to alert the users when
   * field validation fails.
   *
   * <p>This config is used with Ext.String.format. '{0}' will be replaced with the actual
   * error message(s), '{1}' will be replaced with field label.</p>
   * @default 'Input error. {0}.'
   */
  public native function get ariaErrorText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set ariaErrorText(value:String):void;

  [ExtConfig]
  /**
   * Optional text description for this object. This text will be
   * announced to Assistive Technology users when the object is focused.
   */
  public native function get ariaHelp():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set ariaHelp(value:String):void;

  [ExtConfig]
  /**
   * Whether to adjust the component's body width to make room for 'side'
   * <a href="#!/api/Ext.form.Labelable-cfg-msgTarget" rel="Ext.form.Labelable-cfg-msgTarget" class="docClass">error messages</a>.
   * @default true
   */
  public native function get autoFitErrors():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set autoFitErrors(value:Boolean):void;

  [ExtConfig]
  /**
   * The CSS class to be applied to the body content element.
   * @default Ext.baseCSSPrefix + 'form-item-body'
   */
  public native function get baseBodyCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set baseBodyCls(value:String):void;

  [ExtConfig]
  /**
   * An optional string or <code>XTemplate</code> configuration to insert in the field markup
   * at the beginning of the input containing element. If an <code>XTemplate</code> is used, the component's <a href="#!/api/Ext.Component-cfg-renderData" rel="Ext.Component-cfg-renderData" class="docClass">render data</a>
   * serves as the context.
   */
  public native function get beforeBodyEl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set beforeBodyEl(value:*):void;

  [ExtConfig]
  /**
   * An optional string or <code>XTemplate</code> configuration to insert in the field markup
   * before the label text. If an <code>XTemplate</code> is used, the component's <a href="#!/api/Ext.Component-cfg-renderData" rel="Ext.Component-cfg-renderData" class="docClass">render data</a>
   * serves as the context.
   */
  public native function get beforeLabelTextTpl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set beforeLabelTextTpl(value:*):void;

  [ExtConfig]
  /**
   * An optional string or <code>XTemplate</code> configuration to insert in the field markup
   * before the label element. If an <code>XTemplate</code> is used, the component's <a href="#!/api/Ext.Component-cfg-renderData" rel="Ext.Component-cfg-renderData" class="docClass">render data</a>
   * serves as the context.
   */
  public native function get beforeLabelTpl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set beforeLabelTpl(value:*):void;

  [ExtConfig]
  /**
   * An optional string or <code>XTemplate</code> configuration to insert in the field markup
   * before the <a href="#!/api/Ext.form.Labelable-method-getSubTplMarkup" rel="Ext.form.Labelable-method-getSubTplMarkup" class="docClass">subTpl markup</a>. If an <code>XTemplate</code> is used, the
   * component's <a href="#!/api/Ext.Component-cfg-renderData" rel="Ext.Component-cfg-renderData" class="docClass">render data</a> serves as the context.
   */
  public native function get beforeSubTpl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set beforeSubTpl(value:*):void;

  [ExtConfig]
  /**
   * Defines a timeout in milliseconds for buffering <a href="#!/api/Ext.form.field.Base-cfg-checkChangeEvents" rel="Ext.form.field.Base-cfg-checkChangeEvents" class="docClass">checkChangeEvents</a> that fire in rapid succession.
   * Defaults to 50 milliseconds.
   * @default 50
   */
  public native function get checkChangeBuffer():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set checkChangeBuffer(value:Number):void;

  [ExtConfig]
  /**
   * A list of event names that will be listened for on the field's <a href="#!/api/Ext.form.field.Base-property-inputEl" rel="Ext.form.field.Base-property-inputEl" class="docClass">input element</a>, which will cause
   * the field's value to be checked for changes. If a change is detected, the <a href="#!/api/Ext.form.field.Base-event-change" rel="Ext.form.field.Base-event-change" class="docClass">change event</a> will be
   * fired, followed by validation if the <a href="#!/api/Ext.form.field.Base-cfg-validateOnChange" rel="Ext.form.field.Base-cfg-validateOnChange" class="docClass">validateOnChange</a> option is enabled.
   *
   * <p>Defaults to ['change', 'propertychange', 'keyup'] in Internet Explorer, and ['change', 'input', 'textInput', 'keyup',
   * 'dragdrop'] in other browsers. This catches all the ways that field values can be changed in most supported
   * browsers; the only known exceptions at the time of writing are:</p>
   *
   * <ul>
   * <li>Safari 3.2 and older: cut/paste in textareas via the context menu, and dragging text into textareas</li>
   * <li>Opera 10 and 11: dragging text into text fields and textareas, and cut via the context menu in text fields
   * and textareas</li>
   * <li>Opera 9: Same as Opera 10 and 11, plus paste from context menu in text fields and textareas</li>
   * </ul>
   *
   *
   * <p>If you need to guarantee on-the-fly change notifications including these edge cases, you can call the
   * <a href="#!/api/Ext.form.field.Base-method-checkChange" rel="Ext.form.field.Base-method-checkChange" class="docClass">checkChange</a> method on a repeating interval, e.g. using <a href="#!/api/Ext.util.TaskManager" rel="Ext.util.TaskManager" class="docClass">Ext.TaskManager</a>, or if the field is within
   * a <a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a>, you can use the FormPanel's <a href="#!/api/Ext.form.Panel-cfg-pollForChanges" rel="Ext.form.Panel-cfg-pollForChanges" class="docClass">Ext.form.Panel.pollForChanges</a> configuration to set up
   * such a task automatically.</p>
   */
  public native function get checkChangeEvents():Array;

  [ExtConfig]
  /**
   * @private
   */
  public native function set checkChangeEvents(value:Array):void;

  [ExtConfig]
  /**
   * The CSS class to use when the field value <a href="#!/api/Ext.form.field.Base-method-isDirty" rel="Ext.form.field.Base-method-isDirty" class="docClass">is dirty</a>.
   * @default Ext.baseCSSPrefix + 'form-dirty'
   */
  public native function get dirtyCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set dirtyCls(value:String):void;

  [ExtConfig]
  /**
   * The CSS class to be applied to the error message element.
   * @default Ext.baseCSSPrefix + 'form-error-msg'
   */
  public native function get errorMsgCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set errorMsgCls(value:String):void;

  [ExtConfig]
  /**
   * The default CSS class for the field input
   * @default 'x-form-field'
   */
  public native function get fieldCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set fieldCls(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The label for the field. It gets appended with the <a href="#!/api/Ext.form.Labelable-cfg-labelSeparator" rel="Ext.form.Labelable-cfg-labelSeparator" class="docClass">labelSeparator</a>, and its position and sizing is
   * determined by the <a href="#!/api/Ext.form.Labelable-cfg-labelAlign" rel="Ext.form.Labelable-cfg-labelAlign" class="docClass">labelAlign</a> and <a href="#!/api/Ext.form.Labelable-cfg-labelWidth" rel="Ext.form.Labelable-cfg-labelWidth" class="docClass">labelWidth</a> configs.
   * @see #getFieldLabel()
   * @see #setFieldLabel()
   */
  public native function get fieldLabel():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set fieldLabel(value:String):void;

  [ExtConfig]
  /**
   * Optional CSS style(s) to be applied to the <a href="#!/api/Ext.form.field.Base-property-inputEl" rel="Ext.form.field.Base-property-inputEl" class="docClass">field input element</a>. Should be a valid argument to
   * <a href="#!/api/Ext.dom.Element-method-applyStyles" rel="Ext.dom.Element-method-applyStyles" class="docClass">Ext.dom.Element.applyStyles</a>. Defaults to undefined. See also the <a href="#!/api/Ext.form.field.Base-method-setFieldStyle" rel="Ext.form.field.Base-method-setFieldStyle" class="docClass">setFieldStyle</a> method for changing
   * the style after initialization.
   * @see #setFieldStyle()
   */
  public native function get fieldStyle():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set fieldStyle(value:String):void;

  [ExtConfig]
  /**
   * A CSS class to be applied to the outermost element to denote that it is participating in the form field layout.
   * @default Ext.baseCSSPrefix + 'form-item'
   */
  public native function get formItemCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set formItemCls(value:String):void;

  [Deprecated(since = "6.2.0")]
  [ExtConfig]
  /**
   * Helpful text describing acceptable format for field values.
   * This text will be announced by Assistive Technologies such as screen readers when
   * the field is focused.
   *
   * <p>This option is superseded by <a href="#!/api/Ext.form.field.Base-cfg-ariaHelp" rel="Ext.form.field.Base-cfg-ariaHelp" class="docClass">ariaHelp</a>.</p>
   */
  public native function get formatText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set formatText(value:String):void;

  [ExtConfig]
  /**
   * When set to true, the label element (<a href="#!/api/Ext.form.Labelable-cfg-fieldLabel" rel="Ext.form.Labelable-cfg-fieldLabel" class="docClass">fieldLabel</a> and <a href="#!/api/Ext.form.Labelable-cfg-labelSeparator" rel="Ext.form.Labelable-cfg-labelSeparator" class="docClass">labelSeparator</a>) will be automatically
   * hidden if the <a href="#!/api/Ext.form.Labelable-cfg-fieldLabel" rel="Ext.form.Labelable-cfg-fieldLabel" class="docClass">fieldLabel</a> is empty. Setting this to false will cause the empty label element to be
   * rendered and space to be reserved for it; this is useful if you want a field without a label to line up with
   * other labeled fields in the same form.
   *
   * <p>If you wish to unconditionall hide the label even if a non-empty fieldLabel is configured, then set the
   * <a href="#!/api/Ext.form.Labelable-cfg-hideLabel" rel="Ext.form.Labelable-cfg-hideLabel" class="docClass">hideLabel</a> config to true.</p>
   * @default true
   */
  public native function get hideEmptyLabel():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set hideEmptyLabel(value:Boolean):void;

  [ExtConfig]
  /**
   * Set to true to completely hide the label element (<a href="#!/api/Ext.form.Labelable-cfg-fieldLabel" rel="Ext.form.Labelable-cfg-fieldLabel" class="docClass">fieldLabel</a> and <a href="#!/api/Ext.form.Labelable-cfg-labelSeparator" rel="Ext.form.Labelable-cfg-labelSeparator" class="docClass">labelSeparator</a>). Also see
   * <a href="#!/api/Ext.form.Labelable-cfg-hideEmptyLabel" rel="Ext.form.Labelable-cfg-hideEmptyLabel" class="docClass">hideEmptyLabel</a>, which controls whether space will be reserved for an empty fieldLabel.
   * @default false
   */
  public native function get hideLabel():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set hideLabel(value:Boolean):void;

  [ExtConfig]
  /**
   * An optional string or <code>XTemplate</code> configuration to insert in the field markup
   * inside the input element (as attributes). If an <code>XTemplate</code> is used, the component's
   * <a href="#!/api/Ext.form.field.Base-method-getSubTplData" rel="Ext.form.field.Base-method-getSubTplData" class="docClass">subTpl data</a> serves as the context.
   */
  public native function get inputAttrTpl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set inputAttrTpl(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The id that will be given to the generated input DOM element. Defaults to an automatically generated id. If you
   * configure this manually, you must make sure it is unique in the document.
   * @see #getInputId()
   */
  public native function get inputId():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set inputId(value:String):void;

  [ExtConfig]
  /**
   * The type attribute for input fields -- e.g. radio, text, password, file. The extended types
   * supported by HTML5 inputs (url, email, etc.) may also be used, though using them will cause older browsers to
   * fall back to 'text'.
   *
   * <p>The type 'password' must be used to render that field type currently -- there is no separate Ext component for
   * that. You can use <a href="#!/api/Ext.form.field.File" rel="Ext.form.field.File" class="docClass">Ext.form.field.File</a> which creates a custom-rendered file upload field, but if you want
   * a plain unstyled file input you can use a Base with inputType:'file'.</p>
   * @default 'text'
   */
  public native function get inputType():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set inputType(value:String):void;

  [ExtConfig]
  /**
   * The CSS class to use when marking the component invalid.
   * @default Ext.baseCSSPrefix + 'form-invalid'
   */
  public native function get invalidCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set invalidCls(value:String):void;

  [ExtConfig]
  /**
   * The error text to use when marking a field invalid and no message is provided
   * @default 'The value in this field is invalid'
   */
  public native function get invalidText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set invalidText(value:String):void;

  [ExtConfig]
  /**
   * Controls the position and alignment of the <a href="#!/api/Ext.form.Labelable-cfg-fieldLabel" rel="Ext.form.Labelable-cfg-fieldLabel" class="docClass">fieldLabel</a>. Valid values are:
   *
   * <ul>
   * <li>"left" (the default) - The label is positioned to the left of the field, with its text aligned to the left.
   * Its width is determined by the <a href="#!/api/Ext.form.Labelable-cfg-labelWidth" rel="Ext.form.Labelable-cfg-labelWidth" class="docClass">labelWidth</a> config.</li>
   * <li>"top" - The label is positioned above the field.</li>
   * <li>"right" - The label is positioned to the left of the field, with its text aligned to the right.
   * Its width is determined by the <a href="#!/api/Ext.form.Labelable-cfg-labelWidth" rel="Ext.form.Labelable-cfg-labelWidth" class="docClass">labelWidth</a> config.</li>
   * </ul>
   * @default 'left'
   */
  public native function get labelAlign():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set labelAlign(value:String):void;

  [ExtConfig]
  /**
   * An optional string or <code>XTemplate</code> configuration to insert in the field markup
   * inside the label element (as attributes). If an <code>XTemplate</code> is used, the component's
   * <a href="#!/api/Ext.Component-cfg-renderData" rel="Ext.Component-cfg-renderData" class="docClass">render data</a> serves as the context.
   */
  public native function get labelAttrTpl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set labelAttrTpl(value:*):void;

  [ExtConfig]
  /**
   * The CSS class to be applied to the label element. This (single) CSS class is used to formulate the renderSelector
   * and drives the field layout where it is concatenated with a hyphen ('-') and <a href="#!/api/Ext.form.Labelable-cfg-labelAlign" rel="Ext.form.Labelable-cfg-labelAlign" class="docClass">labelAlign</a>. To add
   * additional classes, use <a href="#!/api/Ext.form.Labelable-cfg-labelClsExtra" rel="Ext.form.Labelable-cfg-labelClsExtra" class="docClass">labelClsExtra</a>.
   * @default Ext.baseCSSPrefix + 'form-item-label'
   */
  public native function get labelCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set labelCls(value:String):void;

  [ExtConfig]
  /**
   * An optional string of one or more additional CSS classes to add to the label element. Defaults to empty.
   */
  public native function get labelClsExtra():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set labelClsExtra(value:String):void;

  [ExtConfig]
  /**
   * The amount of space in pixels between the <a href="#!/api/Ext.form.Labelable-cfg-fieldLabel" rel="Ext.form.Labelable-cfg-fieldLabel" class="docClass">fieldLabel</a> and the field body.
   * This defaults to <code>5</code> for compatibility with Ext JS 4, however, as of Ext JS 5
   * the space between the label and the body can optionally be determined by the theme
   * using the $form-label-horizontal-spacing (for side-aligned labels) and
   * $form-label-vertical-spacing (for top-aligned labels) SASS variables.
   * In order for the stylesheet values as to take effect, you must use a labelPad value
   * of <code>null</code>.
   * @default 5
   */
  public native function get labelPad():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set labelPad(value:Number):void;

  [ExtConfig]
  /**
   * Character(s) to be inserted at the end of the <a href="#!/api/Ext.form.Labelable-cfg-fieldLabel" rel="Ext.form.Labelable-cfg-fieldLabel" class="docClass">label text</a>.
   *
   * <p>Set to empty string to hide the separator completely.</p>
   * @default ':'
   */
  public native function get labelSeparator():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set labelSeparator(value:String):void;

  [ExtConfig]
  /**
   * A CSS style specification string to apply directly to this field's label.
   */
  public native function get labelStyle():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set labelStyle(value:String):void;

  [ExtConfig]
  /**
   * The width of the <a href="#!/api/Ext.form.Labelable-cfg-fieldLabel" rel="Ext.form.Labelable-cfg-fieldLabel" class="docClass">fieldLabel</a> in pixels. Only applicable if <a href="#!/api/Ext.form.Labelable-cfg-labelAlign" rel="Ext.form.Labelable-cfg-labelAlign" class="docClass">labelAlign</a>
   * is set to "left" or "right".
   * @default 100
   */
  public native function get labelWidth():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set labelWidth(value:Number):void;

  [ExtConfig]
  /**
   * The location where the error message text should display. Must be one of the following values:
   *
   * <ul>
   * <li><p><code>qtip</code> Display a quick tip containing the message when the user hovers over the field.
   * This is the default.</p>
   *
   * <p><strong><a href="#!/api/Ext.tip.QuickTipManager-method-init" rel="Ext.tip.QuickTipManager-method-init" class="docClass">Ext.tip.QuickTipManager.init</a> must have been called for this setting to work.</strong></p></li>
   * <li><p><code>title</code> Display the message in a default browser title attribute popup.</p></li>
   * <li><code>under</code> Add a block div beneath the field containing the error message.</li>
   * <li><code>side</code> Add an error icon to the right of the field, displaying the message in a popup on hover.</li>
   * <li><code>none</code> Don't display any error message. This might be useful if you are implementing custom error display.</li>
   * <li><code>[element id]</code> Add the error message directly to the innerHTML of the specified element.</li>
   * </ul>
   * @default 'qtip'
   */
  public native function get msgTarget():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set msgTarget(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The name of the field. This is used as the parameter name when including the field value
   * in a <a href="#!/api/Ext.form.Basic-method-submit" rel="Ext.form.Basic-method-submit" class="docClass">form submit()</a>. If no name is configured, it falls back to the <a href="#!/api/Ext.form.field.Base-cfg-inputId" rel="Ext.form.field.Base-cfg-inputId" class="docClass">inputId</a>.
   * To prevent the field from being included in the form submit, set <a href="#!/api/Ext.form.field.Base-cfg-submitValue" rel="Ext.form.field.Base-cfg-submitValue" class="docClass">submitValue</a> to false.
   * @see #getName()
   */
  public native function get name():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set name(value:String):void;

  [ExtConfig]
  /**
   * true to disable displaying any <a href="#!/api/Ext.form.Labelable-method-setActiveError" rel="Ext.form.Labelable-method-setActiveError" class="docClass">error message</a> set on this object.
   * @default false
   */
  public native function get preventMark():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set preventMark(value:Boolean):void;

  [ExtConfig]
  /**
   * true to mark the field as readOnly in HTML.
   * @default false
   * @see #setReadOnly()
   */
  public native function get readOnly():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set readOnly(value:Boolean):void;

  [ExtConfig]
  /**
   * The CSS class applied to the component's main element when it is <a href="#!/api/Ext.form.field.Base-cfg-readOnly" rel="Ext.form.field.Base-cfg-readOnly" class="docClass">readOnly</a>.
   * @default Ext.baseCSSPrefix + 'form-readonly'
   */
  public native function get readOnlyCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set readOnlyCls(value:String):void;

  [ExtConfig]
  /**
   * Setting this to false will prevent the field from being <a href="#!/api/Ext.form.Basic-method-submit" rel="Ext.form.Basic-method-submit" class="docClass">submitted</a> even when it is
   * not disabled.
   * @default true
   */
  public native function get submitValue():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set submitValue(value:Boolean):void;

  [ExtConfig]
  /**
   * Whether the field should validate when it loses focus. This will cause fields to be validated
   * as the user steps through the fields in the form regardless of whether they are making changes to those fields
   * along the way. See also <a href="#!/api/Ext.form.field.Base-cfg-validateOnChange" rel="Ext.form.field.Base-cfg-validateOnChange" class="docClass">validateOnChange</a>.
   * @default true
   */
  public native function get validateOnBlur():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set validateOnBlur(value:Boolean):void;

  [ExtConfig]
  /**
   * Specifies whether this field should be validated immediately whenever a change in its value is detected.
   * If the validation results in a change in the field's validity, a <a href="#!/api/Ext.form.field.Field-event-validitychange" rel="Ext.form.field.Field-event-validitychange" class="docClass">validitychange</a> event will be
   * fired. This allows the field to show feedback about the validity of its contents immediately as the user is
   * typing.
   *
   * <p>When set to false, feedback will not be immediate. However the form will still be validated before submitting if
   * the clientValidation option to <a href="#!/api/Ext.form.Basic-method-doAction" rel="Ext.form.Basic-method-doAction" class="docClass">Ext.form.Basic.doAction</a> is enabled, or if the field or form are validated
   * manually.</p>
   *
   * <p>See also <a href="#!/api/Ext.form.field.Base-cfg-checkChangeEvents" rel="Ext.form.field.Base-cfg-checkChangeEvents" class="docClass">Ext.form.field.Base.checkChangeEvents</a> for controlling how changes to the field's value are
   * detected.</p>
   * @default true
   */
  public native function get validateOnChange():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set validateOnChange(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * This property, when a <code>String</code>, contributes its value to the error state of this
   * instance as reported by <code>getErrors</code>.
   * @see #getValidation()
   * @see #setValidation()
   */
  public native function get validation():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set validation(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * A value to initialize this field with.
   * @see #getValue()
   * @see #setValue()
   */
  public native function get value():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set value(value:Object):void;

  [ExtConfig]
  /**
   * The event name(s) to use to publish the <a href="#!/api/Ext.form.field.Field-cfg-value" rel="Ext.form.field.Field-cfg-value" class="docClass">value</a>
   * <a href="#!/api/Ext.form.field.Base-cfg-bind" rel="Ext.form.field.Base-cfg-bind" class="docClass">Ext.form.field.Base.bind</a> for this field.
   * @default 'change'
   * @since 5.0.1
   */
  public native function get valuePublishEvent():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set valuePublishEvent(value:*):void;

  /**
   * @inheritDoc
   */
  public native function get fieldBodyCls():String;

  /**
   * @inheritDoc
   */
  public native function set fieldBodyCls(value:String):void;

  /**
   * A utility for grouping a set of modifications which may trigger value changes into a single transaction, to
   * prevent excessive firing of <a href="#!/api/Ext.form.field.Field-event-change" rel="Ext.form.field.Field-event-change" class="docClass">change</a> events. This is useful for instance if the field has sub-fields which
   * are being updated as a group; you don't want the container field to check its own changed state for each subfield
   * change.
   * @param fn The function to call with change checks suspended.
   */
  public native function batchChanges(fn:Function):void;

  /**
   * Checks whether the value of the field has changed since the last time it was checked.
   * If the value has changed, it:
   *
   * <ol>
   * <li>Fires the <a href="#!/api/Ext.form.field.Field-event-change" rel="Ext.form.field.Field-event-change" class="docClass">change event</a>,</li>
   * <li>Performs validation if the <a href="#!/api/Ext.form.field.Field-cfg-validateOnChange" rel="Ext.form.field.Field-cfg-validateOnChange" class="docClass">validateOnChange</a> config is enabled, firing the
   * <a href="#!/api/Ext.form.field.Field-event-validitychange" rel="Ext.form.field.Field-event-validitychange" class="docClass">validitychange event</a> if the validity has changed, and</li>
   * <li>Checks the <a href="#!/api/Ext.form.field.Field-method-isDirty" rel="Ext.form.field.Field-method-isDirty" class="docClass">dirty state</a> of the field and fires the <a href="#!/api/Ext.form.field.Field-event-dirtychange" rel="Ext.form.field.Field-event-dirtychange" class="docClass">dirtychange event</a>
   * if it has changed.</li>
   * </ol>
   */
  public native function checkChange():void;

  /**
   * Checks the <a href="#!/api/Ext.form.field.Field-method-isDirty" rel="Ext.form.field.Field-method-isDirty" class="docClass">isDirty</a> state of the field and if it has changed since the last time it was checked,
   * fires the <a href="#!/api/Ext.form.field.Field-event-dirtychange" rel="Ext.form.field.Field-event-dirtychange" class="docClass">dirtychange</a> event.
   */
  public native function checkDirty():void;

  /**
   * Cleans up values initialized by this Field mixin on the current instance.
   * Components using this mixin should call this method before being destroyed.
   */
  public native function cleanupField():void;

  /**
   * Clear any invalid styles/messages for this field.
   *
   * <p><strong>Note</strong>: this method does not cause the Field's <a href="#!/api/Ext.form.field.Base-method-validate" rel="Ext.form.field.Base-method-validate" class="docClass">validate</a> or <a href="#!/api/Ext.form.field.Base-method-isValid" rel="Ext.form.field.Base-method-isValid" class="docClass">isValid</a> methods to return <code>true</code>
   * if the value does not <em>pass</em> validation. So simply clearing a field's errors will not necessarily allow
   * submission of forms submitted with the <a href="#!/api/Ext.form.action.Submit-cfg-clientValidation" rel="Ext.form.action.Submit-cfg-clientValidation" class="docClass">Ext.form.action.Submit.clientValidation</a> option set.</p>
   */
  public native function clearInvalid():void;

  /**
   * Only relevant if the instance's <a href="#!/api/Ext.form.field.Field-method-isFileUpload" rel="Ext.form.field.Field-method-isFileUpload" class="docClass">isFileUpload</a> method returns true. Returns a reference to the file input
   * DOM element holding the user's selected file. The input will be appended into the submission form and will not be
   * returned, so this method should also create a replacement.
   */
  public native function extractFileInput():js.HTMLElement;

  /**
   * Gets the active error message for this component, if any. This does not trigger validation on its own, it merely
   * returns any message that the component may already hold.
   * @return The active error message on the component; if there is no error, an empty string is returned.
   */
  public native function getActiveError():String;

  /**
   * Gets an Array of any active error messages currently applied to the field. This does not trigger validation on
   * its own, it merely returns any messages that the component may already hold.
   * @return The active error messages on the component; if there are no errors, an empty Array is
   * returned.
   */
  public native function getActiveErrors():Array;

  /**
   * Runs this field's validators and returns an array of error messages for any validation failures. This is called
   * internally during validation and would not usually need to be used manually.
   *
   * <p>Each subclass should override or augment the return value to provide their own errors.</p>
   * @param value The value to get errors for (defaults to the current field value)
   * @return All error messages for this field; an empty Array if none.
   */
  public native function getErrors(value:*):Array;

  /**
   * Returns the label for the field. Defaults to simply returning the <a href="#!/api/Ext.form.Labelable-cfg-fieldLabel" rel="Ext.form.Labelable-cfg-fieldLabel" class="docClass">fieldLabel</a> config. Can be overridden
   * to provide a custom generated label.
   * @return The configured field label, or empty string if not defined
   */
  public native function getFieldLabel():String;

  /**
   * Returns the input id for this field. If none was specified via the <a href="#!/api/Ext.form.field.Base-cfg-inputId" rel="Ext.form.field.Base-cfg-inputId" class="docClass">inputId</a> config, then an id will be
   * automatically generated.
   */
  public native function getInputId():String;

  /**
   * Returns the value(s) that should be saved to the <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a> instance for this field, when <a href="#!/api/Ext.form.Basic-method-updateRecord" rel="Ext.form.Basic-method-updateRecord" class="docClass">Ext.form.Basic.updateRecord</a> is called. Typically this will be an object with a single name-value pair, the name
   * being this field's <a href="#!/api/Ext.form.field.Field-method-getName" rel="Ext.form.field.Field-method-getName" class="docClass">name</a> and the value being its current data value. More advanced field
   * implementations may return more than one name-value pair. The returned values will be saved to the corresponding
   * field names in the Model.
   *
   * <p>Note that the values returned from this method are not guaranteed to have been successfully <a href="#!/api/Ext.form.field.Field-method-validate" rel="Ext.form.field.Field-method-validate" class="docClass">validated</a>.</p>
   * @return A mapping of submit parameter names to values; each value should be a string, or an array of
   * strings if that particular name has multiple values. It can also return null if there are no parameters to be
   * submitted.
   */
  public native function getModelData(includeEmptyText:Object, isSubmitting:Object):Object;

  /**
   * Returns the <a href="#!/api/Ext.form.field.Field-cfg-name" rel="Ext.form.field.Field-cfg-name" class="docClass">name</a> attribute of the field. This is used as the parameter name
   * when including the field value in a <a href="#!/api/Ext.form.Basic-method-submit" rel="Ext.form.Basic-method-submit" class="docClass">form submit()</a>.
   * @return name The field <a href="#!/api/Ext.form.field.Field-cfg-name" rel="Ext.form.field.Field-cfg-name" class="docClass">name</a>
   */
  public native function getName():String;

  /**
   * Returns the raw value of the field, without performing any normalization, conversion, or validation. To get a
   * normalized and converted value see <a href="#!/api/Ext.form.field.Base-method-getValue" rel="Ext.form.field.Base-method-getValue" class="docClass">getValue</a>.
   * @return value The raw String value of the field
   */
  public native function getRawValue():String;

  /**
   * Creates and returns the data object to be used when rendering the <a href="#!/api/Ext.form.field.Base-cfg-fieldSubTpl" rel="Ext.form.field.Base-cfg-fieldSubTpl" class="docClass">fieldSubTpl</a>.
   * @return The template data
   */
  public native function getSubTplData(fieldData:Object):Object;

  /**
   * Returns the value that would be included in a standard form submit for this field. This will be combined with the
   * field's name to form a name=value pair in the <a href="#!/api/Ext.form.field.Base-method-getSubmitData" rel="Ext.form.field.Base-method-getSubmitData" class="docClass">submitted parameters</a>. If an empty string is
   * returned then just the name= will be submitted; if null is returned then nothing will be submitted.
   *
   * <p>Note that the value returned will have been <a href="#!/api/Ext.form.field.Base-method-processRawValue" rel="Ext.form.field.Base-method-processRawValue" class="docClass">processed</a> but may or may not have been
   * successfully <a href="#!/api/Ext.form.field.Base-method-validate" rel="Ext.form.field.Base-method-validate" class="docClass">validated</a>.</p>
   * @return The value to be submitted, or null.
   */
  public native function getSubmitValue():String;

  /**
   * Returns the value of <a href="#!/api/Ext.form.field.Field-cfg-validation" rel="Ext.form.field.Field-cfg-validation" class="docClass">validation</a>.
   */
  public native function getValidation():*;

  /**
   * Returns the current data value of the field. The type of value returned is particular to the type of the
   * particular field (e.g. a Date object for <a href="#!/api/Ext.form.field.Date" rel="Ext.form.field.Date" class="docClass">Ext.form.field.Date</a>), as the result of calling <a href="#!/api/Ext.form.field.Base-method-rawToValue" rel="Ext.form.field.Base-method-rawToValue" class="docClass">rawToValue</a> on
   * the field's <a href="#!/api/Ext.form.field.Base-method-processRawValue" rel="Ext.form.field.Base-method-processRawValue" class="docClass">processed</a> String value. To return the raw String value, see <a href="#!/api/Ext.form.field.Base-method-getRawValue" rel="Ext.form.field.Base-method-getRawValue" class="docClass">getRawValue</a>.
   * @return value The field value
   */
  public native function getValue():*;

  /**
   * Tells whether the field currently has an active error message. This does not trigger validation on its own, it
   * merely looks for any message that the component may already hold.
   */
  public native function hasActiveError():Boolean;

  /**
   * Checks if the field has a visible label
   * @return True if the field has a visible label
   */
  public native function hasVisibleLabel():Boolean;

  /**
   * Initializes this Field mixin on the current instance. Components using this mixin should call this method during
   * their own initialization process.
   */
  public native function initField():void;

  /**
   * Performs initialization of this mixin. Component classes using this mixin should call this method during their
   * own initialization.
   */
  public native function initLabelable():void;

  /**
   * Initializes the field's value based on the initial config.
   */
  public native function initValue():void;

  /**
   * Returns true if the value of this Field has been changed from its <a href="#!/api/Ext.form.field.Field-property-originalValue" rel="Ext.form.field.Field-property-originalValue" class="docClass">originalValue</a>.
   * Will always return false if the field is disabled.
   *
   * <p>Note that if the owning <a href="#!/api/Ext.form.Basic" rel="Ext.form.Basic" class="docClass">form</a> was configured with
   * <a href="#!/api/Ext.form.Basic-cfg-trackResetOnLoad" rel="Ext.form.Basic-cfg-trackResetOnLoad" class="docClass">trackResetOnLoad</a> then the <a href="#!/api/Ext.form.field.Field-property-originalValue" rel="Ext.form.field.Field-property-originalValue" class="docClass">originalValue</a> is updated when
   * the values are loaded by <a href="#!/api/Ext.form.Basic" rel="Ext.form.Basic" class="docClass">Ext.form.Basic</a>.<a href="#!/api/Ext.form.Basic-method-setValues" rel="Ext.form.Basic-method-setValues" class="docClass">setValues</a>.</p>
   * @return True if this field has been changed from its original value (and is not disabled),
   * false otherwise.
   */
  public native function isDirty():Boolean;

  /**
   * Returns whether two field <a href="#!/api/Ext.form.field.Field-method-getValue" rel="Ext.form.field.Field-method-getValue" class="docClass">values</a> are logically equal. Field implementations may override this
   * to provide custom comparison logic appropriate for the particular field's data type.
   * @param value1 The first value to compare
   * @param value2 The second value to compare
   * @return True if the values are equal, false if inequal.
   */
  public native function isEqual(value1:Object, value2:Object):Boolean;

  /**
   * Returns whether or not the field value is currently valid by <a href="#!/api/Ext.form.field.Base-method-getErrors" rel="Ext.form.field.Base-method-getErrors" class="docClass">validating</a> the
   * <a href="#!/api/Ext.form.field.Base-method-processRawValue" rel="Ext.form.field.Base-method-processRawValue" class="docClass">processed raw value</a> of the field. <strong>Note</strong>: <a href="#!/api/Ext.form.field.Base-cfg-disabled" rel="Ext.form.field.Base-cfg-disabled" class="docClass">disabled</a> fields are
   * always treated as valid.
   * @return True if the value is valid, else false
   */
  public native function isValid():Boolean;

  /**
   * Performs any necessary manipulation of a raw field value to prepare it for <a href="#!/api/Ext.form.field.Base-method-rawToValue" rel="Ext.form.field.Base-method-rawToValue" class="docClass">conversion</a> and/or
   * <a href="#!/api/Ext.form.field.Base-method-validate" rel="Ext.form.field.Base-method-validate" class="docClass">validation</a>, for instance stripping out ignored characters. In the base implementation it does
   * nothing; individual subclasses may override this as needed.
   * @param value The unprocessed string value
   * @return The processed string value
   */
  public native function processRawValue(value:Object):Object;

  /**
   * Converts a raw input field value into a mixed-type value that is suitable for this particular field type. This
   * allows controlling the normalization and conversion of user-entered values into field-type-appropriate values,
   * e.g. a Date object for <a href="#!/api/Ext.form.field.Date" rel="Ext.form.field.Date" class="docClass">Ext.form.field.Date</a>, and is invoked by <a href="#!/api/Ext.form.field.Base-method-getValue" rel="Ext.form.field.Base-method-getValue" class="docClass">getValue</a>.
   *
   * <p>It is up to individual implementations to decide how to handle raw values that cannot be successfully converted
   * to the desired object type.</p>
   *
   * <p>See <a href="#!/api/Ext.form.field.Base-method-valueToRaw" rel="Ext.form.field.Base-method-valueToRaw" class="docClass">valueToRaw</a> for the opposite conversion.</p>
   *
   * <p>The base implementation does no conversion, returning the raw value untouched.</p>
   * @return The converted value.
   */
  public native function rawToValue(rawValue:Object):Object;

  /**
   * Resets the current field value to the originally loaded value and clears any validation messages. See <a href="#!/api/Ext.form.Basic" rel="Ext.form.Basic" class="docClass">Ext.form.Basic</a>.<a href="#!/api/Ext.form.Basic-cfg-trackResetOnLoad" rel="Ext.form.Basic-cfg-trackResetOnLoad" class="docClass">trackResetOnLoad</a>
   */
  public native function reset():void;

  /**
   * Resets the field's <a href="#!/api/Ext.form.field.Field-property-originalValue" rel="Ext.form.field.Field-property-originalValue" class="docClass">originalValue</a> property so it matches the current <a href="#!/api/Ext.form.field.Field-method-getValue" rel="Ext.form.field.Field-method-getValue" class="docClass">value</a>. This is
   * called by <a href="#!/api/Ext.form.Basic" rel="Ext.form.Basic" class="docClass">Ext.form.Basic</a>.<a href="#!/api/Ext.form.Basic-method-setValues" rel="Ext.form.Basic-method-setValues" class="docClass">setValues</a> if the form's
   * <a href="#!/api/Ext.form.Basic-cfg-trackResetOnLoad" rel="Ext.form.Basic-cfg-trackResetOnLoad" class="docClass">trackResetOnLoad</a> property is set to true.
   */
  public native function resetOriginalValue():void;

  /**
   * Sets the active error message to the given string. This replaces the entire error message contents with the given
   * string. Also see <a href="#!/api/Ext.form.Labelable-method-setActiveErrors" rel="Ext.form.Labelable-method-setActiveErrors" class="docClass">setActiveErrors</a> which accepts an Array of messages and formats them according to the
   * <a href="#!/api/Ext.form.Labelable-cfg-activeErrorsTpl" rel="Ext.form.Labelable-cfg-activeErrorsTpl" class="docClass">activeErrorsTpl</a>. Note that this only updates the error message element's text and attributes, you'll
   * have to call doComponentLayout to actually update the field's layout to match. If the field extends <a href="#!/api/Ext.form.field.Base" rel="Ext.form.field.Base" class="docClass">Ext.form.field.Base</a> you should call <a href="#!/api/Ext.form.field.Base-method-markInvalid" rel="Ext.form.field.Base-method-markInvalid" class="docClass">markInvalid</a> instead.
   * @param msg The error message
   */
  public native function setActiveError(msg:String):void;

  /**
   * Set the active error message to an Array of error messages. The messages are formatted into a single message
   * string using the <a href="#!/api/Ext.form.Labelable-cfg-activeErrorsTpl" rel="Ext.form.Labelable-cfg-activeErrorsTpl" class="docClass">activeErrorsTpl</a>. Also see <a href="#!/api/Ext.form.Labelable-method-setActiveError" rel="Ext.form.Labelable-method-setActiveError" class="docClass">setActiveError</a> which allows setting the entire error
   * contents with a single string. Note that this only updates the error message element's text and attributes,
   * you'll have to call doComponentLayout to actually update the field's layout to match. If the field extends
   * <a href="#!/api/Ext.form.field.Base" rel="Ext.form.field.Base" class="docClass">Ext.form.field.Base</a> you should call <a href="#!/api/Ext.form.field.Base-method-markInvalid" rel="Ext.form.field.Base-method-markInvalid" class="docClass">markInvalid</a> instead.
   * @param errors The error messages
   */
  public native function setActiveErrors(errors:Array):void;

  /**
   * Applies a set of default configuration values to this Labelable instance. For each of the properties in the given
   * object, check if this component hasOwnProperty that config; if not then it's inheriting a default value from its
   * prototype and we should apply the default value.
   * @param defaults The defaults to apply to the object.
   */
  public native function setFieldDefaults(defaults:Object):void;

  /**
   * Set the label of this field.
   * @param label The new label. The <a href="#!/api/Ext.form.Labelable-cfg-labelSeparator" rel="Ext.form.Labelable-cfg-labelSeparator" class="docClass">labelSeparator</a> will be automatically appended to the label
   * string.
   */
  public native function setFieldLabel(label:String):void;

  /**
   * Set the <a href="#!/api/Ext.form.field.Base-cfg-fieldStyle" rel="Ext.form.field.Base-cfg-fieldStyle" class="docClass">CSS style</a> of the <a href="#!/api/Ext.form.field.Base-property-inputEl" rel="Ext.form.field.Base-property-inputEl" class="docClass">field input element</a>.
   * @param style The style(s) to apply. Should be a valid argument to <a href="#!/api/Ext.dom.Element-method-applyStyles" rel="Ext.dom.Element-method-applyStyles" class="docClass">Ext.dom.Element.applyStyles</a>.
   */
  public native function setFieldStyle(style:*):void;

  /**
   * Sets the field's raw value directly, bypassing <a href="#!/api/Ext.form.field.Base-method-valueToRaw" rel="Ext.form.field.Base-method-valueToRaw" class="docClass">value conversion</a>, change detection, and
   * validation. To set the value with these additional inspections see <a href="#!/api/Ext.form.field.Base-method-setValue" rel="Ext.form.field.Base-method-setValue" class="docClass">setValue</a>.
   * @param value The value to set
   * @return value The field value that is set
   */
  public native function setRawValue(value:Object):Object;

  /**
   * Sets the read only state of this field.
   * @param readOnly Whether the field should be read only.
   */
  public native function setReadOnly(readOnly:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.form.field.Field-cfg-validation" rel="Ext.form.field.Field-cfg-validation" class="docClass">validation</a>.
   * @param validation The new value.
   */
  public native function setValidation(validation:*):void;

  /**
   * Sets a data value into the field and runs the change detection and validation. To set the value directly
   * without these inspections see <a href="#!/api/Ext.form.field.Base-method-setRawValue" rel="Ext.form.field.Base-method-setRawValue" class="docClass">setRawValue</a>.
   * @param value The value to set
   * @return this
   */
  public native function setValue(value:*):ext.form.field.IField;

  /**
   * Returns the trimmed label by slicing off the label separator character. Can be overridden.
   * @return The trimmed field label, or empty string if not defined
   */
  public native function trimLabelSeparator():String;

  /**
   * Clears the active error message(s). Note that this only clears the error message element's text and attributes,
   * you'll have to call doComponentLayout to actually update the field's layout to match. If the field extends <a href="#!/api/Ext.form.field.Base" rel="Ext.form.field.Base" class="docClass">Ext.form.field.Base</a> you should call <a href="#!/api/Ext.form.field.Base-method-clearInvalid" rel="Ext.form.field.Base-method-clearInvalid" class="docClass">clearInvalid</a> instead.
   */
  public native function unsetActiveError():void;

  /**
   * Returns whether or not the field value is currently valid by <a href="#!/api/Ext.form.field.Field-method-getErrors" rel="Ext.form.field.Field-method-getErrors" class="docClass">validating</a> the field's current
   * value, and fires the <a href="#!/api/Ext.form.field.Field-event-validitychange" rel="Ext.form.field.Field-event-validitychange" class="docClass">validitychange</a> event if the field's validity has changed since the last validation.
   * <strong>Note</strong>: <a href="#!/api/Ext.form.field.Field-cfg-disabled" rel="Ext.form.field.Field-cfg-disabled" class="docClass">disabled</a> fields are always treated as valid.
   *
   * <p>Custom implementations of this method are allowed to have side-effects such as triggering error message display.
   * To validate without side-effects, use <a href="#!/api/Ext.form.field.Field-method-isValid" rel="Ext.form.field.Field-method-isValid" class="docClass">isValid</a>.</p>
   * @return True if the value is valid, else false
   */
  public native function validate():Boolean;

  /**
   * Uses <a href="#!/api/Ext.form.field.Base-method-getErrors" rel="Ext.form.field.Base-method-getErrors" class="docClass">getErrors</a> to build an array of validation errors. If any errors are found, they are passed to
   * <a href="#!/api/Ext.form.field.Base-method-markInvalid" rel="Ext.form.field.Base-method-markInvalid" class="docClass">markInvalid</a> and false is returned, otherwise true is returned.
   *
   * <p>Previously, subclasses were invited to provide an implementation of this to process validations - from 3.2
   * onwards <a href="#!/api/Ext.form.field.Base-method-getErrors" rel="Ext.form.field.Base-method-getErrors" class="docClass">getErrors</a> should be overridden instead.</p>
   * @param value The value to validate
   * @return True if all validations passed, false if one or more failed
   */
  public native function validateValue(value:Object):Boolean;

  /**
   * Converts a mixed-type value to a raw representation suitable for displaying in the field. This allows controlling
   * how value objects passed to <a href="#!/api/Ext.form.field.Base-method-setValue" rel="Ext.form.field.Base-method-setValue" class="docClass">setValue</a> are shown to the user, including localization. For instance, for a
   * <a href="#!/api/Ext.form.field.Date" rel="Ext.form.field.Date" class="docClass">Ext.form.field.Date</a>, this would control how a Date object passed to <a href="#!/api/Ext.form.field.Base-method-setValue" rel="Ext.form.field.Base-method-setValue" class="docClass">setValue</a> would be converted
   * to a String for display in the field.
   *
   * <p>See <a href="#!/api/Ext.form.field.Base-method-rawToValue" rel="Ext.form.field.Base-method-rawToValue" class="docClass">rawToValue</a> for the opposite conversion.</p>
   *
   * <p>The base implementation simply does a standard toString conversion, and converts <a href="#!/api/Ext-method-isEmpty" rel="Ext-method-isEmpty" class="docClass">empty values</a>
   * to an empty string.</p>
   * @param value The mixed-type value to convert to the raw representation.
   * @return The converted raw value.
   */
  public native function valueToRaw(value:Object):Object;

  /**
   * @inheritDoc
   */
  public native function getSubmitData():Object;

  /**
   * @inheritDoc
   */
  public native function isFileUpload():Boolean;

  /**
   * @inheritDoc
   */
  public native function markInvalid(errors:*):void;

  /**
   * Template method before a field is reset.
   */
  protected native function beforeReset():void;

  /**
   * Called when focus leaves this input field.
   * Used to postprocess raw values and perform conversion and validation.
   */
  protected native function completeEdit():void;

  /**
   * Generates the arguments for the field decorations <a href="#!/api/Ext.form.Labelable-cfg-labelableRenderTpl" rel="Ext.form.Labelable-cfg-labelableRenderTpl" class="docClass">rendering template</a>.
   * @param data optional object to use as the base data object.  If provided,
   * this method will add properties to the base object instead of creating a new one.
   * @return The template arguments
   */
  protected native function getLabelableRenderData(data:Object):Object;

  /**
   * Gets the markup to be inserted into the outer template's bodyEl. For fields this is the actual input element.
   */
  protected native function getSubTplMarkup(fieldData:Object):String;

  /**
   * Transform the raw value before it is set
   * @param value The value
   * @return The value to set
   */
  protected native function transformRawValue(value:Object):Object;
}
}