package ext.form.field {

[Uses("ext.form.field.BaseField")]
[Uses("ext.Component")]
[Uses("ext.util.Positionable")]
[Uses("ext.Base")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Animate")]
[Uses("ext.util.Floating")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.state.Stateful")]
[Uses("ext.form.Labelable")]
[Uses("ext.form.field.Field")]
[Native("Ext.form.field.Hidden", require)]
/**
 * A basic hidden field for storing hidden values in forms that need to be passed in the form submit.
 *
 * <p>This creates an actual input element with type="hidden" in the DOM. While its label is
 * <a href="#!/api/Ext.form.field.Hidden-cfg-hideLabel" rel="Ext.form.field.Hidden-cfg-hideLabel" class="docClass">not rendered</a> by default, it is still a real component and may be sized according
 * to its owner container's layout.</p>
 *
 * <p>Because of this, in most cases it is more convenient and less problematic to simply
 * <a href="#!/api/Ext.form.action.Action-cfg-params" rel="Ext.form.action.Action-cfg-params" class="docClass">pass hidden parameters</a> directly when
 * <a href="#!/api/Ext.form.Basic-method-submit" rel="Ext.form.Basic-method-submit" class="docClass">submitting the form</a>.</p>
 *
 * <p>Example:</p>
 *
 * <pre class='inline-example '><code>new <a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a>({
 *     title: 'My Form',
 *     items: [{
 *         xtype: 'textfield',
 *         fieldLabel: 'Text Field',
 *         name: 'text_field',
 *         value: 'value from text field'
 *     }, {
 *         xtype: 'hiddenfield',
 *         name: 'hidden_field_1',
 *         value: 'value from hidden field'
 *     }],
 *
 *     buttons: [{
 *         text: 'Submit',
 *         handler: function() {
 *             this.up('form').getForm().submit({
 *                 params: {
 *                     hidden_field_2: 'value from submit call'
 *                 }
 *             });
 *         }
 *     }]
 * });
 * </code></pre>
 *
 * <p>Submitting the above form will result in three values sent to the server:</p>
 *
 * <pre><code>text_field=value+from+text+field&amp;hidden;_field_1=value+from+hidden+field&amp;hidden_field_2=value+from+submit+call
 * </code></pre>
 */
public class Hidden extends ext.form.field.BaseField {
  /**
   * @param config @inheritDoc
   */
  public function Hidden(config:ext.form.field.Hidden = null) {
    super();
  }
}
}