package ext.form.field {
import ext.Component;

[Uses("ext.form.field.TextField")]
[Uses("ext.form.field.BaseField")]
[Uses("ext.Component")]
[Uses("ext.util.Positionable")]
[Uses("ext.Base")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Animate")]
[Uses("ext.util.Floating")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.state.Stateful")]
[Uses("ext.form.Labelable")]
[Uses("ext.form.field.Field")]
[Native("Ext.form.field.Picker", require)]
/**
 * Fires when the field's picker is collapsed.
 * @eventType ext.form.field.events.PickerEvent.COLLAPSE
 */
[Event(name = 'onCollapse', type = 'ext.form.field.events.PickerEvent')]
/**
 * Fires when the field's picker is expanded.
 * @eventType ext.form.field.events.PickerEvent.EXPAND
 */
[Event(name = 'onExpand', type = 'ext.form.field.events.PickerEvent')]
/**
 * Fires when a value is selected via the picker.
 * @eventType ext.form.field.events.Picker_valueEvent.SELECT
 */
[Event(name = 'onSelect', type = 'ext.form.field.events.Picker_valueEvent')]
/**
 * An abstract class for fields that have a single trigger which opens a "picker" popup below the field, e.g. a combobox
 * menu list or a date picker. It provides a base implementation for toggling the picker's visibility when the trigger
 * is clicked, as well as keyboard navigation and some basic events. Sizing and alignment of the picker can be
 * controlled via the <a href="#!/api/Ext.form.field.Picker-cfg-matchFieldWidth" rel="Ext.form.field.Picker-cfg-matchFieldWidth" class="docClass">matchFieldWidth</a> and <a href="#!/api/Ext.form.field.Picker-cfg-pickerAlign" rel="Ext.form.field.Picker-cfg-pickerAlign" class="docClass">pickerAlign</a>/<a href="#!/api/Ext.form.field.Picker-cfg-pickerOffset" rel="Ext.form.field.Picker-cfg-pickerOffset" class="docClass">pickerOffset</a> config properties
 * respectively.
 *
 * <p>You would not normally use this class directly, but instead use it as the parent class for a specific picker field
 * implementation. Subclasses must implement the <a href="#!/api/Ext.form.field.Picker-method-createPicker" rel="Ext.form.field.Picker-method-createPicker" class="docClass">createPicker</a> method to create a picker component appropriate
 * for the field.</p>
 */
public class Picker extends ext.form.field.TextField {
  /**
   * @param config @inheritDoc
   */
  public function Picker(config:ext.form.field.Picker = null) {
    super();
  }

  [ExtConfig]
  /**
   * False to prevent the user from typing text directly into the field; the field can only have its value set via
   * selecting a value from the picker. In this state, the picker can also be opened by clicking directly on the input
   * field itself.
   * @default true
   */
  override public native function get editable():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set editable(value:Boolean):void;

  /**
   * True if the picker is currently expanded, false if not.
   * @default false
   */
  public native function get isExpanded():Boolean;

  /**
   * @private
   */
  public native function set isExpanded(value:Boolean):void;

  /**
   * <code>true</code> in this class to identify an object as an instantiated Picker Field, or subclass thereof.
   * @default true
   */
  public native function get isPickerField():Boolean;

  /**
   * @private
   */
  public native function set isPickerField(value:Boolean):void;

  [ExtConfig]
  /**
   * Whether the picker dropdown's width should be explicitly set to match the width of the field. Defaults to true.
   * @default true
   */
  public native function get matchFieldWidth():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set matchFieldWidth(value:Boolean):void;

  [ExtConfig]
  /**
   * A class to be added to the field's <a href="#!/api/Ext.form.field.Picker-property-bodyEl" rel="Ext.form.field.Picker-property-bodyEl" class="docClass">bodyEl</a> element when the picker is opened.
   * @default 'x-pickerfield-open'
   */
  public native function get openCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set openCls(value:String):void;

  [ExtConfig]
  /**
   * The <a href="#!/api/Ext.util.Positionable-method-alignTo" rel="Ext.util.Positionable-method-alignTo" class="docClass">alignment position</a> with which to align the picker. Defaults to "tl-bl?"
   * @default 'tl-bl?'
   */
  public native function get pickerAlign():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set pickerAlign(value:String):void;

  [ExtConfig]
  /**
   * An offset [x,y] to use in addition to the <a href="#!/api/Ext.form.field.Picker-cfg-pickerAlign" rel="Ext.form.field.Picker-cfg-pickerAlign" class="docClass">pickerAlign</a> when positioning the picker.
   * Defaults to undefined.
   */
  public native function get pickerOffset():Array;

  [ExtConfig]
  /**
   * @private
   */
  public native function set pickerOffset(value:Array):void;

  [ExtConfig]
  /**
   * An additional CSS class used to style the trigger button. The trigger will always
   * get the class 'x-form-trigger' and triggerCls will be appended if specified.
   */
  public native function get triggerCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set triggerCls(value:String):void;

  /**
   * Collapses this field's picker dropdown.
   */
  public native function collapse():void;

  /**
   * Creates and returns the component to be used as this field's picker. Must be implemented by subclasses of Picker.
   */
  public native function createPicker():void;

  /**
   * Expands this field's picker dropdown.
   */
  public native function expand():void;

  /**
   * Returns a reference to the picker component for this field, creating it if necessary by
   * calling <a href="#!/api/Ext.form.field.Picker-method-createPicker" rel="Ext.form.field.Picker-method-createPicker" class="docClass">createPicker</a>.
   * @return The picker component
   */
  public native function getPicker():ext.Component;

  /**
   * Aligns the picker to the input element
   */
  protected native function alignPicker():void;

  /**
   * Handles the trigger click; by default toggles between expanding and collapsing the picker component.
   */
  protected native function onTriggerClick(e:Object):void;
}
}