package ext.form.field {

[Uses("ext.form.field.Checkbox")]
[Uses("ext.form.field.BaseField")]
[Uses("ext.Component")]
[Uses("ext.util.Positionable")]
[Uses("ext.Base")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Animate")]
[Uses("ext.util.Floating")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.state.Stateful")]
[Uses("ext.form.Labelable")]
[Uses("ext.form.field.Field")]
[Native("Ext.form.field.Radio", require)]
/**
 * Single radio field. Similar to checkbox, but automatically handles making sure only one radio is checked
 * at a time within a group of radios with the same name.
 *
 * <h1>Labeling</h1>
 *
 * <p>In addition to the <a href="#!/api/Ext.form.Labelable" rel="Ext.form.Labelable" class="docClass">standard field labeling options</a>, radio buttons
 * may be given an optional <a href="#!/api/Ext.form.field.Radio-cfg-boxLabel" rel="Ext.form.field.Radio-cfg-boxLabel" class="docClass">boxLabel</a> which will be displayed immediately to the right of the input. Also
 * see <a href="#!/api/Ext.form.RadioGroup" rel="Ext.form.RadioGroup" class="docClass">Ext.form.RadioGroup</a> for a convenient method of grouping related radio buttons.</p>
 *
 * <h1>Values</h1>
 *
 * <p>The main value of a Radio field is a boolean, indicating whether or not the radio is checked.</p>
 *
 * <p>The following values will check the radio:</p>
 *
 * <ul>
 * <li><code>true</code></li>
 * <li><code>'true'</code></li>
 * <li><code>'1'</code></li>
 * <li><code>'on'</code></li>
 * </ul>
 *
 *
 * <p>Any other value will uncheck it.</p>
 *
 * <p>In addition to the main boolean value, you may also specify a separate <a href="#!/api/Ext.form.field.Radio-cfg-inputValue" rel="Ext.form.field.Radio-cfg-inputValue" class="docClass">inputValue</a>. This will be sent
 * as the parameter value when the form is <a href="#!/api/Ext.form.Basic-method-submit" rel="Ext.form.Basic-method-submit" class="docClass">submitted</a>. You will want to set this
 * value if you have multiple radio buttons with the same <a href="#!/api/Ext.form.field.Radio-cfg-name" rel="Ext.form.field.Radio-cfg-name" class="docClass">name</a>, as is almost always the case.</p>
 *
 * <h1>Example usage</h1>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a>', {
 *     title      : 'Order Form',
 *     width      : 300,
 *     bodyPadding: 10,
 *     renderTo   : <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     items: [
 *         {
 *             xtype      : 'fieldcontainer',
 *             fieldLabel : 'Size',
 *             defaultType: 'radiofield',
 *             defaults: {
 *                 flex: 1
 *             },
 *             layout: 'hbox',
 *             items: [
 *                 {
 *                     boxLabel  : 'M',
 *                     name      : 'size',
 *                     inputValue: 'm',
 *                     id        : 'radio1'
 *                 }, {
 *                     boxLabel  : 'L',
 *                     name      : 'size',
 *                     inputValue: 'l',
 *                     id        : 'radio2'
 *                 }, {
 *                     boxLabel  : 'XL',
 *                     name      : 'size',
 *                     inputValue: 'xl',
 *                     id        : 'radio3'
 *                 }
 *             ]
 *         },
 *         {
 *             xtype      : 'fieldcontainer',
 *             fieldLabel : 'Color',
 *             defaultType: 'radiofield',
 *             defaults: {
 *                 flex: 1
 *             },
 *             layout: 'hbox',
 *             items: [
 *                 {
 *                     boxLabel  : 'Blue',
 *                     name      : 'color',
 *                     inputValue: 'blue',
 *                     id        : 'radio4'
 *                 }, {
 *                     boxLabel  : 'Grey',
 *                     name      : 'color',
 *                     inputValue: 'grey',
 *                     id        : 'radio5'
 *                 }, {
 *                     boxLabel  : 'Black',
 *                     name      : 'color',
 *                     inputValue: 'black',
 *                     id        : 'radio6'
 *                 }
 *             ]
 *         }
 *     ],
 *     bbar: [
 *         {
 *             text: 'Smaller Size',
 *             handler: function() {
 *                 var radio1 = <a href="#!/api/Ext-method-getCmp" rel="Ext-method-getCmp" class="docClass">Ext.getCmp</a>('radio1'),
 *                     radio2 = <a href="#!/api/Ext-method-getCmp" rel="Ext-method-getCmp" class="docClass">Ext.getCmp</a>('radio2'),
 *                     radio3 = <a href="#!/api/Ext-method-getCmp" rel="Ext-method-getCmp" class="docClass">Ext.getCmp</a>('radio3');
 *
 *                 //if L is selected, change to M
 *                 if (radio2.getValue()) {
 *                     radio1.setValue(true);
 *                     return;
 *                 }
 *
 *                 //if XL is selected, change to L
 *                 if (radio3.getValue()) {
 *                     radio2.setValue(true);
 *                     return;
 *                 }
 *
 *                 //if nothing is set, set size to S
 *                 radio1.setValue(true);
 *             }
 *         },
 *         {
 *             text: 'Larger Size',
 *             handler: function() {
 *                 var radio1 = <a href="#!/api/Ext-method-getCmp" rel="Ext-method-getCmp" class="docClass">Ext.getCmp</a>('radio1'),
 *                     radio2 = <a href="#!/api/Ext-method-getCmp" rel="Ext-method-getCmp" class="docClass">Ext.getCmp</a>('radio2'),
 *                     radio3 = <a href="#!/api/Ext-method-getCmp" rel="Ext-method-getCmp" class="docClass">Ext.getCmp</a>('radio3');
 *
 *                 //if M is selected, change to L
 *                 if (radio1.getValue()) {
 *                     radio2.setValue(true);
 *                     return;
 *                 }
 *
 *                 //if L is selected, change to XL
 *                 if (radio2.getValue()) {
 *                     radio3.setValue(true);
 *                     return;
 *                 }
 *
 *                 //if nothing is set, set size to XL
 *                 radio3.setValue(true);
 *             }
 *         },
 *         '-',
 *         {
 *             text: 'Select color',
 *             menu: {
 *                 indent: false,
 *                 items: [
 *                     {
 *                         text: 'Blue',
 *                         handler: function() {
 *                             var radio = <a href="#!/api/Ext-method-getCmp" rel="Ext-method-getCmp" class="docClass">Ext.getCmp</a>('radio4');
 *                             radio.setValue(true);
 *                         }
 *                     },
 *                     {
 *                         text: 'Grey',
 *                         handler: function() {
 *                             var radio = <a href="#!/api/Ext-method-getCmp" rel="Ext-method-getCmp" class="docClass">Ext.getCmp</a>('radio5');
 *                             radio.setValue(true);
 *                         }
 *                     },
 *                     {
 *                         text: 'Black',
 *                         handler: function() {
 *                             var radio = <a href="#!/api/Ext-method-getCmp" rel="Ext-method-getCmp" class="docClass">Ext.getCmp</a>('radio6');
 *                             radio.setValue(true);
 *                         }
 *                     }
 *                 ]
 *             }
 *         }
 *     ]
 * });
 * </code></pre>
 */
public class Radio extends ext.form.field.Checkbox {
  /**
   * @param config @inheritDoc
   */
  public function Radio(config:ext.form.field.Radio = null) {
    super();
  }

  /**
   * The value <code>true</code> to identify an object as an instance of this or derived class.
   * @default true
   */
  public native function get isRadio():Boolean;

  /**
   * Sets either the checked/unchecked status of this Radio, or, if a string value is passed, checks a sibling Radio
   * of the same name whose value is the value specified.
   * @param value The value to set
   * @return this
   */
  override public native function setValue(value:*):ext.form.field.IField;

  /**
   * If this radio is part of a group, it will return the selected value
   */
  public native function getGroupValue():String;
}
}