package ext.form.field {

[Uses("ext.form.field.TextField")]
[Uses("ext.form.field.BaseField")]
[Uses("ext.Component")]
[Uses("ext.util.Positionable")]
[Uses("ext.Base")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Animate")]
[Uses("ext.util.Floating")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.state.Stateful")]
[Uses("ext.form.Labelable")]
[Uses("ext.form.field.Field")]
[Native("Ext.form.field.TextArea", require)]
/**
 * This class creates a multiline text field, which can be used as a direct replacement for traditional
 * textarea fields. In addition, it supports automatically <a href="#!/api/Ext.form.field.TextArea-cfg-grow" rel="Ext.form.field.TextArea-cfg-grow" class="docClass">growing</a> the height of the textarea to
 * fit its content.
 *
 * <p>All of the configuration options from <a href="#!/api/Ext.form.field.Text" rel="Ext.form.field.Text" class="docClass">Ext.form.field.Text</a> can be used on TextArea.</p>
 *
 * <p>Example usage:</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.FormPanel</a>', {
 *     title      : 'Sample TextArea',
 *     width      : 400,
 *     bodyPadding: 10,
 *     renderTo   : <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     items: [{
 *         xtype     : 'textareafield',
 *         grow      : true,
 *         name      : 'message',
 *         fieldLabel: 'Message',
 *         anchor    : '100%'
 *     }]
 * });
 * </code></pre>
 *
 * <p>Some other useful configuration options when using <a href="#!/api/Ext.form.field.TextArea-cfg-grow" rel="Ext.form.field.TextArea-cfg-grow" class="docClass">grow</a> are <a href="#!/api/Ext.form.field.TextArea-cfg-growMin" rel="Ext.form.field.TextArea-cfg-growMin" class="docClass">growMin</a> and <a href="#!/api/Ext.form.field.TextArea-cfg-growMax" rel="Ext.form.field.TextArea-cfg-growMax" class="docClass">growMax</a>.
 * These allow you to set the minimum and maximum grow heights for the textarea.</p>
 *
 * <p><strong>NOTE:</strong> In some browsers, carriage returns ('\r', not to be confused with new lines)
 * will be automatically stripped out the value is set to the textarea. Since we cannot
 * use any reasonable method to attempt to re-insert these, they will automatically be
 * stripped out to ensure the behaviour is consistent across browser.</p>
 */
public class TextArea extends ext.form.field.TextField {
  /**
   * @param config @inheritDoc
   */
  public function TextArea(config:ext.form.field.TextArea = null) {
    super();
  }

  [ExtConfig]
  /**
   * A string that will be appended to the field's current value for the purposes of calculating the target field
   * size. Only used when the <a href="#!/api/Ext.form.field.TextArea-cfg-grow" rel="Ext.form.field.TextArea-cfg-grow" class="docClass">grow</a> config is true. Defaults to a newline for TextArea to ensure there is
   * always a space below the current line.
   * @default '\n-'
   */
  override public native function get growAppend():String;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set growAppend(value:String):void;

  [ExtConfig]
  /**
   * The maximum height to allow when <a href="#!/api/Ext.form.field.TextArea-cfg-grow" rel="Ext.form.field.TextArea-cfg-grow" class="docClass">grow</a>=true
   * @default 1000
   */
  override public native function get growMax():Number;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set growMax(value:Number):void;

  [ExtConfig]
  /**
   * The minimum height to allow when <a href="#!/api/Ext.form.field.TextArea-cfg-grow" rel="Ext.form.field.TextArea-cfg-grow" class="docClass">grow</a>=true
   * @default 60
   */
  override public native function get growMin():Number;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set growMin(value:Number):void;

  [ExtConfig]
  /**
   * True if you want the ENTER key to be classed as a special key and the <a href="#!/api/Ext.form.field.TextArea-event-specialkey" rel="Ext.form.field.TextArea-event-specialkey" class="docClass">specialkey</a> event to be fired
   * when ENTER is pressed.
   * @default false
   */
  public native function get enterIsSpecial():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set enterIsSpecial(value:Boolean):void;

  [ExtConfig]
  /**
   * true to prevent scrollbars from appearing regardless of how much text is in the field. This option is only
   * relevant when <a href="#!/api/Ext.form.field.TextArea-cfg-grow" rel="Ext.form.field.TextArea-cfg-grow" class="docClass">grow</a> is true. Equivalent to setting overflow: hidden.
   * @default false
   */
  public native function get preventScrollbars():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set preventScrollbars(value:Boolean):void;

  /**
   * Automatically grows the field to accomodate the height of the text up to the maximum
   * field height allowed. This only takes effect if <a href="#!/api/Ext.form.field.TextArea-cfg-grow" rel="Ext.form.field.TextArea-cfg-grow" class="docClass">grow</a> = true, and fires the
   * <a href="#!/api/Ext.form.field.TextArea-event-autosize" rel="Ext.form.field.TextArea-event-autosize" class="docClass">autosize</a> event if the height changes.
   */
  override public native function autoSize():void;
}
}