package ext.form.field {
import ext.dom.CompositeElement;
import ext.dom.Element;
import ext.form.trigger.Trigger;

[Uses("ext.form.field.BaseField")]
[Uses("ext.Component")]
[Uses("ext.util.Positionable")]
[Uses("ext.Base")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Animate")]
[Uses("ext.util.Floating")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.state.Stateful")]
[Uses("ext.form.Labelable")]
[Uses("ext.form.field.Field")]
[Native("Ext.form.field.Text", require)]
/**
 * Fires when the <strong><a href="#!/api/Ext.form.field.Text-method-autoSize" rel="Ext.form.field.Text-method-autoSize" class="docClass">autoSize</a></strong> function is triggered and the field is resized according to the
 * <a href="#!/api/Ext.form.field.Text-cfg-grow" rel="Ext.form.field.Text-cfg-grow" class="docClass">grow</a>/<a href="#!/api/Ext.form.field.Text-cfg-growMin" rel="Ext.form.field.Text-cfg-growMin" class="docClass">growMin</a>/<a href="#!/api/Ext.form.field.Text-cfg-growMax" rel="Ext.form.field.Text-cfg-growMax" class="docClass">growMax</a> configs as a result. This event provides a hook for the
 * developer to apply additional logic at runtime to resize the field if needed.
 * @eventType ext.form.field.events.TextField_widthEvent.AUTO_SIZE
 */
[Event(name = 'onAutoSize', type = 'ext.form.field.events.TextField_widthEvent')]
/**
 * Keydown input field event. This event only fires if <strong><a href="#!/api/Ext.form.field.Text-cfg-enableKeyEvents" rel="Ext.form.field.Text-cfg-enableKeyEvents" class="docClass">enableKeyEvents</a></strong> is set to true.
 * @eventType ext.form.field.events.TextField_eEvent.KEY_DOWN
 */
[Event(name = 'onKeyDown', type = 'ext.form.field.events.TextField_eEvent')]
/**
 * Keypress input field event. This event only fires if <strong><a href="#!/api/Ext.form.field.Text-cfg-enableKeyEvents" rel="Ext.form.field.Text-cfg-enableKeyEvents" class="docClass">enableKeyEvents</a></strong> is set to true.
 * @eventType ext.form.field.events.TextField_eEvent.KEYPRESS
 */
[Event(name = 'onKeypress', type = 'ext.form.field.events.TextField_eEvent')]
/**
 * Keyup input field event. This event only fires if <strong><a href="#!/api/Ext.form.field.Text-cfg-enableKeyEvents" rel="Ext.form.field.Text-cfg-enableKeyEvents" class="docClass">enableKeyEvents</a></strong> is set to true.
 * @eventType ext.form.field.events.TextField_eEvent.KEY_UP
 */
[Event(name = 'onKeyUp', type = 'ext.form.field.events.TextField_eEvent')]
/**
 * A basic text field.  Can be used as a direct replacement for traditional text inputs,
 * or as the base class for more sophisticated input controls (like <a href="#!/api/Ext.form.field.TextArea" rel="Ext.form.field.TextArea" class="docClass">Ext.form.field.TextArea</a>
 * and <a href="#!/api/Ext.form.field.ComboBox" rel="Ext.form.field.ComboBox" class="docClass">Ext.form.field.ComboBox</a>). Has support for empty-field placeholder values (see <a href="#!/api/Ext.form.field.Text-cfg-emptyText" rel="Ext.form.field.Text-cfg-emptyText" class="docClass">emptyText</a>).
 *
 * <h1>Validation</h1>
 *
 * <p>The Text field has a useful set of validations built in:</p>
 *
 * <ul>
 * <li><a href="#!/api/Ext.form.field.Text-cfg-allowBlank" rel="Ext.form.field.Text-cfg-allowBlank" class="docClass">allowBlank</a> for making the field required</li>
 * <li><a href="#!/api/Ext.form.field.Text-cfg-minLength" rel="Ext.form.field.Text-cfg-minLength" class="docClass">minLength</a> for requiring a minimum value length</li>
 * <li><a href="#!/api/Ext.form.field.Text-cfg-maxLength" rel="Ext.form.field.Text-cfg-maxLength" class="docClass">maxLength</a> for setting a maximum value length (with <a href="#!/api/Ext.form.field.Text-cfg-enforceMaxLength" rel="Ext.form.field.Text-cfg-enforceMaxLength" class="docClass">enforceMaxLength</a> to add it
 * as the <code>maxlength</code> attribute on the input element)</li>
 * <li><a href="#!/api/Ext.form.field.Text-cfg-regex" rel="Ext.form.field.Text-cfg-regex" class="docClass">regex</a> to specify a custom regular expression for validation</li>
 * </ul>
 *
 *
 * <p>In addition, custom validations may be added:</p>
 *
 * <ul>
 * <li><a href="#!/api/Ext.form.field.Text-cfg-vtype" rel="Ext.form.field.Text-cfg-vtype" class="docClass">vtype</a> specifies a virtual type implementation from <a href="#!/api/Ext.form.field.VTypes" rel="Ext.form.field.VTypes" class="docClass">Ext.form.field.VTypes</a> which can contain
 * custom validation logic</li>
 * <li><a href="#!/api/Ext.form.field.Text-cfg-validator" rel="Ext.form.field.Text-cfg-validator" class="docClass">validator</a> allows a custom arbitrary function to be called during validation</li>
 * </ul>
 *
 *
 * <p>The details around how and when each of these validation options get used are described in the
 * documentation for <a href="#!/api/Ext.form.field.Text-method-getErrors" rel="Ext.form.field.Text-method-getErrors" class="docClass">getErrors</a>.</p>
 *
 * <p>By default, the field value is checked for validity immediately while the user is typing in the
 * field. This can be controlled with the <a href="#!/api/Ext.form.field.Text-cfg-validateOnChange" rel="Ext.form.field.Text-cfg-validateOnChange" class="docClass">validateOnChange</a>, <a href="#!/api/Ext.form.field.Text-cfg-checkChangeEvents" rel="Ext.form.field.Text-cfg-checkChangeEvents" class="docClass">checkChangeEvents</a>, and
 * <a href="#!/api/Ext.form.field.Text-cfg-checkChangeBuffer" rel="Ext.form.field.Text-cfg-checkChangeBuffer" class="docClass">checkChangeBuffer</a> configurations. Also see the details on Form Validation in the
 * <a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a> class documentation.</p>
 *
 * <h1>Masking and Character Stripping</h1>
 *
 * <p>Text fields can be configured with custom regular expressions to be applied to entered values before
 * validation: see <a href="#!/api/Ext.form.field.Text-cfg-maskRe" rel="Ext.form.field.Text-cfg-maskRe" class="docClass">maskRe</a> and <a href="#!/api/Ext.form.field.Text-cfg-stripCharsRe" rel="Ext.form.field.Text-cfg-stripCharsRe" class="docClass">stripCharsRe</a> for details.</p>
 *
 * <h1>Example usage</h1>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a>', {
 *     title: 'Contact Info',
 *     width: 300,
 *     bodyPadding: 10,
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     items: [{
 *         xtype: 'textfield',
 *         name: 'name',
 *         fieldLabel: 'Name',
 *         allowBlank: false  // requires a non-empty value
 *     }, {
 *         xtype: 'textfield',
 *         name: 'email',
 *         fieldLabel: 'Email Address',
 *         vtype: 'email'  // requires value to be a valid email address format
 *     }]
 * });
 * </code></pre>
 *
 * <h1>Custom Subclasses</h1>
 *
 * <p>This class can be extended to provide additional functionality. The example below demonstrates creating
 * a custom search field that uses the HTML5 search input type.</p>
 *
 * <pre class='inline-example '><code>// A simple subclass of Base that creates a HTML5 search field. Redirects to the
 * // searchUrl when the Enter key is pressed.
 * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Ext.form.SearchField', {
 *     extend: '<a href="#!/api/Ext.form.field.Text" rel="Ext.form.field.Text" class="docClass">Ext.form.field.Text</a>',
 *     alias: 'widget.searchfield',
 *
 *     inputType: 'search',
 *
 *     // Config defining the search URL
 *     searchUrl: 'http://www.google.com/search?q={0}',
 *
 *     // Add specialkey listener
 *     initComponent: function() {
 *         this.callParent();
 *         this.on('specialkey', this.checkEnterKey, this);
 *     },
 *
 *     // Handle enter key presses, execute the search if the field has a value
 *     checkEnterKey: function(field, e) {
 *         var value = this.getValue();
 *         if (e.getKey() === e.ENTER &amp;&amp; !<a href="#!/api/Ext-method-isEmpty" rel="Ext-method-isEmpty" class="docClass">Ext.isEmpty</a>(value)) {
 *             location.href = <a href="#!/api/Ext.String-method-format" rel="Ext.String-method-format" class="docClass">Ext.String.format</a>(this.searchUrl, value);
 *         }
 *     }
 * });
 *
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a>', {
 *     title: 'Base Example',
 *     bodyPadding: 5,
 *     width: 250,
 *
 *     // Fields will be arranged vertically, stretched to full width
 *     layout: 'anchor',
 *     defaults: {
 *         anchor: '100%'
 *     },
 *     items: [{
 *         xtype: 'searchfield',
 *         fieldLabel: 'Search',
 *         name: 'query'
 *     }],
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 * });
 * </code></pre>
 */
public class TextField extends ext.form.field.BaseField {
  /**
   * @param config @inheritDoc
   */
  public function TextField(config:ext.form.field.TextField = null) {
    super();
  }

  [ExtConfig]
  /**
   * <code>true</code> to prevent the user from changing the field, and hide all triggers.
   * @see #setReadOnly()
   */
  override public native function get readOnly():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  override public native function set readOnly(value:Boolean):void;

  [Deprecated(replacement = "inputWrap", since = "5.0")]
  /**
   * A reference to the element that wraps the input element. Only set after the
   * field has been rendered.
   */
  public native function get inputCell():ext.dom.Element;

  /**
   * @private
   */
  public native function set inputCell(value:ext.dom.Element):void;

  /**
   * A reference to the element that wraps the input element. Only set after the
   * field has been rendered.
   */
  public native function get inputWrap():ext.dom.Element;

  /**
   * @private
   */
  public native function set inputWrap(value:ext.dom.Element):void;

  [Deprecated(message = "A composite of all the trigger button elements. Only set after the field has\nbeen rendered.\n", since = "5.0")]
  public native function get triggerEl():ext.dom.CompositeElement;

  /**
   * @private
   */
  public native function set triggerEl(value:ext.dom.CompositeElement):void;

  /**
   * A reference to the element which encapsulates the input field and all
   * trigger button(s). Only set after the field has been rendered.
   */
  public native function get triggerWrap():ext.dom.Element;

  /**
   * @private
   */
  public native function set triggerWrap(value:ext.dom.Element):void;

  [ExtConfig]
  /**
   * Specify false to validate that the value's length must be > 0. If <code>true</code>, then a blank value is <strong>always</strong> taken to be valid regardless of any <a href="#!/api/Ext.form.field.Text-cfg-vtype" rel="Ext.form.field.Text-cfg-vtype" class="docClass">vtype</a>
   * validation that may be applied.
   *
   * <p>If <a href="#!/api/Ext.form.field.Text-cfg-vtype" rel="Ext.form.field.Text-cfg-vtype" class="docClass">vtype</a> validation must still be applied to blank values, configure <a href="#!/api/Ext.form.field.Text-cfg-validateBlank" rel="Ext.form.field.Text-cfg-validateBlank" class="docClass">validateBlank</a> as <code>true</code>;</p>
   * @default true
   */
  public native function get allowBlank():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set allowBlank(value:Boolean):void;

  [ExtConfig]
  /**
   * Specify false to automatically trim the value before validating
   * the whether the value is blank. Setting this to false automatically
   * sets <a href="#!/api/Ext.form.field.Text-cfg-allowBlank" rel="Ext.form.field.Text-cfg-allowBlank" class="docClass">allowBlank</a> to false.
   * @default true
   */
  public native function get allowOnlyWhitespace():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set allowOnlyWhitespace(value:Boolean):void;

  [ExtConfig]
  /**
   * The error text to display if the <strong><a href="#!/api/Ext.form.field.Text-cfg-allowBlank" rel="Ext.form.field.Text-cfg-allowBlank" class="docClass">allowBlank</a></strong> validation fails
   * @default 'This field is required'
   */
  public native function get blankText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set blankText(value:String):void;

  [ExtConfig]
  /**
   * Specify true to disable input keystroke filtering
   * @default false
   */
  public native function get disableKeyFilter():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set disableKeyFilter(value:Boolean):void;

  [ExtConfig]
  /**
   * false to prevent the user from typing text directly into the field; the field can
   * only have its value set programmatically or via an action invoked by a trigger.
   * @default true
   */
  public native function get editable():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set editable(value:Boolean):void;

  [ExtConfig]
  /**
   * The CSS class to apply to an empty field to style the <strong><a href="#!/api/Ext.form.field.Text-cfg-emptyText" rel="Ext.form.field.Text-cfg-emptyText" class="docClass">emptyText</a></strong>.
   * This class is automatically added and removed as needed depending on the current field value.
   * @default 'x-form-empty-field'
   */
  public native function get emptyCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set emptyCls(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The default text to place into an empty field.
   *
   * <p>Note that normally this value will be submitted to the server if this field is enabled; to prevent this you can
   * set the <a href="#!/api/Ext.form.action.Action-cfg-submitEmptyText" rel="Ext.form.action.Action-cfg-submitEmptyText" class="docClass">submitEmptyText</a> option of <a href="#!/api/Ext.form.Basic-method-submit" rel="Ext.form.Basic-method-submit" class="docClass">Ext.form.Basic.submit</a> to
   * false.</p>
   *
   * <p>Also note that if you use <a href="#!/api/Ext.form.field.Text-cfg-inputType" rel="Ext.form.field.Text-cfg-inputType" class="docClass">inputType</a>:'file', <a href="#!/api/Ext.form.field.Text-cfg-emptyText" rel="Ext.form.field.Text-cfg-emptyText" class="docClass">emptyText</a> is not supported and should be
   * avoided.</p>
   *
   * <p>Note that for browsers that support it, setting this property will use the HTML 5 placeholder attribute, and for
   * older browsers that don't support the HTML 5 placeholder attribute the value will be placed directly into the input
   * element itself as the raw value. This means that older browsers will obfuscate the <a href="#!/api/Ext.form.field.Text-cfg-emptyText" rel="Ext.form.field.Text-cfg-emptyText" class="docClass">emptyText</a> value for
   * password input fields.</p>
   * @default ''
   * @see #getEmptyText()
   * @see #setEmptyText()
   */
  public native function get emptyText():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set emptyText(value:String):void;

  [ExtConfig]
  /**
   * true to enable the proxying of key events for the HTML input field
   * @default false
   */
  public native function get enableKeyEvents():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set enableKeyEvents(value:Boolean):void;

  [ExtConfig]
  /**
   * True to set the maxLength property on the underlying input field. Defaults to false
   */
  public native function get enforceMaxLength():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set enforceMaxLength(value:Boolean):void;

  [ExtConfig]
  /**
   * true if this field should automatically grow and shrink to its content
   * @default false
   */
  public native function get grow():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set grow(value:Boolean):void;

  [ExtConfig]
  /**
   * A string that will be appended to the field's current value for the purposes of calculating the target field
   * size. Only used when the <a href="#!/api/Ext.form.field.Text-cfg-grow" rel="Ext.form.field.Text-cfg-grow" class="docClass">grow</a> config is true. Defaults to a single capital "W" (the widest character in
   * common fonts) to leave enough space for the next typed character and avoid the field value shifting before the
   * width is adjusted.
   * @default 'W'
   */
  public native function get growAppend():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set growAppend(value:String):void;

  [ExtConfig]
  /**
   * The maximum width to allow when <code><a href="#!/api/Ext.form.field.Text-cfg-grow" rel="Ext.form.field.Text-cfg-grow" class="docClass">grow</a> = true</code>
   * @default 800
   */
  public native function get growMax():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set growMax(value:Number):void;

  [ExtConfig]
  /**
   * The minimum width to allow when <code><a href="#!/api/Ext.form.field.Text-cfg-grow" rel="Ext.form.field.Text-cfg-grow" class="docClass">grow</a> = true</code>
   * @default 30
   */
  public native function get growMin():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set growMin(value:Number):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * <code>true</code> to hide all triggers
   * @default false
   * @see #getHideTrigger()
   * @see #setHideTrigger()
   */
  public native function get hideTrigger():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set hideTrigger(value:Boolean):void;

  [ExtConfig]
  /**
   * The CSS class that is added to the element wrapping the input element
   * @default Ext.baseCSSPrefix + 'form-text-wrap'
   */
  public native function get inputWrapCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set inputWrapCls(value:String):void;

  [ExtConfig]
  /**
   * An input mask regular expression that will be used to filter keystrokes (character being
   * typed) that do not match.
   * Note: It does not filter characters already in the input.
   */
  public native function get maskRe():RegExp;

  [ExtConfig]
  /**
   * @private
   */
  public native function set maskRe(value:RegExp):void;

  [ExtConfig]
  /**
   * Maximum input field length allowed by validation. This behavior is intended to
   * provide instant feedback to the user by improving usability to allow pasting and editing or overtyping and back
   * tracking. To restrict the maximum number of characters that can be entered into the field use the
   * <strong><a href="#!/api/Ext.form.field.Text-cfg-enforceMaxLength" rel="Ext.form.field.Text-cfg-enforceMaxLength" class="docClass">enforceMaxLength</a></strong> option.
   *
   * <p>Defaults to Number.MAX_VALUE.</p>
   */
  public native function get maxLength():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set maxLength(value:Number):void;

  [ExtConfig]
  /**
   * Error text to display if the <strong><a href="#!/api/Ext.form.field.Text-cfg-maxLength" rel="Ext.form.field.Text-cfg-maxLength" class="docClass">maximum length</a></strong> validation fails
   * @default 'The maximum length for this field is {0}'
   */
  public native function get maxLengthText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set maxLengthText(value:String):void;

  [ExtConfig]
  /**
   * Minimum input field length required
   * @default 0
   */
  public native function get minLength():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set minLength(value:Number):void;

  [ExtConfig]
  /**
   * Error text to display if the <strong><a href="#!/api/Ext.form.field.Text-cfg-minLength" rel="Ext.form.field.Text-cfg-minLength" class="docClass">minimum length</a></strong> validation fails.
   * @default 'The minimum length for this field is {0}'
   */
  public native function get minLengthText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set minLengthText(value:String):void;

  [ExtConfig]
  /**
   * A JavaScript RegExp object to be tested against the field value during validation.
   * If the test fails, the field will be marked invalid using
   * either <strong><a href="#!/api/Ext.form.field.Text-cfg-regexText" rel="Ext.form.field.Text-cfg-regexText" class="docClass">regexText</a></strong> or <strong><a href="#!/api/Ext.form.field.Text-cfg-invalidText" rel="Ext.form.field.Text-cfg-invalidText" class="docClass">invalidText</a></strong>.
   */
  public native function get regex():RegExp;

  [ExtConfig]
  /**
   * @private
   */
  public native function set regex(value:RegExp):void;

  [ExtConfig]
  /**
   * The error text to display if <strong><a href="#!/api/Ext.form.field.Text-cfg-regex" rel="Ext.form.field.Text-cfg-regex" class="docClass">regex</a></strong> is used and the test fails during validation
   * @default ''
   */
  public native function get regexText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set regexText(value:String):void;

  [ExtConfig]
  /**
   * <code>true</code> to attach a <a href="#!/api/Ext.util.ClickRepeater" rel="Ext.util.ClickRepeater" class="docClass">click repeater</a> to the trigger(s).
   * Click repeating behavior can also be configured on the individual <a href="#!/api/Ext.form.field.Text-cfg-triggers" rel="Ext.form.field.Text-cfg-triggers" class="docClass">trigger instances using the trigger's {&#64;link {Ext.form.trigger.Trigger#repeatClick
   * repeatClick</a> config.
   * @default false
   */
  public native function get repeatTriggerClick():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set repeatTriggerClick(value:Boolean):void;

  [ExtConfig]
  /**
   * The CSS class to apply to a required field, i.e. a field where <strong><a href="#!/api/Ext.form.field.Text-cfg-allowBlank" rel="Ext.form.field.Text-cfg-allowBlank" class="docClass">allowBlank</a></strong> is false.
   * @default 'x-form-required-field'
   */
  public native function get requiredCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set requiredCls(value:String):void;

  [ExtConfig]
  /**
   * <code>true</code> to automatically select any existing field text when the field receives input
   * focus. Only applies when <a href="#!/api/Ext.form.field.Text-cfg-editable" rel="Ext.form.field.Text-cfg-editable" class="docClass">editable</a> = true
   * @default false
   */
  public native function get selectOnFocus():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set selectOnFocus(value:Boolean):void;

  [Deprecated(replacement = "width")]
  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * An initial value for the 'size' attribute on the text input element. This is only
   * used if the field has no configured <a href="#!/api/Ext.form.field.Text-cfg-width" rel="Ext.form.field.Text-cfg-width" class="docClass">width</a> and is not given a width by its
   * container's layout. Defaults to 20.
   * @see #getSize()
   */
  public native function get size():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set size(value:Number):void;

  [ExtConfig]
  /**
   * A JavaScript RegExp object used to strip unwanted content from the value
   * during input. If <code>stripCharsRe</code> is specified,
   * every <em>character sequence</em> matching <code>stripCharsRe</code> will be removed.
   */
  public native function get stripCharsRe():RegExp;

  [ExtConfig]
  /**
   * @private
   */
  public native function set stripCharsRe(value:RegExp):void;

  [ExtConfig]
  /**
   * The CSS class that is added to the div wrapping the input element and trigger button(s).
   * @default Ext.baseCSSPrefix + 'form-trigger-wrap'
   */
  public native function get triggerWrapCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set triggerWrapCls(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * <a href="#!/api/Ext.form.trigger.Trigger" rel="Ext.form.trigger.Trigger" class="docClass">Triggers</a> to use in this field.  The keys in
   * this object are unique identifiers for the triggers. The values in this object
   * are <a href="#!/api/Ext.form.trigger.Trigger" rel="Ext.form.trigger.Trigger" class="docClass">Trigger</a> configuration objects.
   *
   * <pre><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.field.Text" rel="Ext.form.field.Text" class="docClass">Ext.form.field.Text</a>', {
   *     renderTo: document.body,
   *     fieldLabel: 'My Custom Field',
   *     triggers: {
   *         foo: {
   *             cls: 'my-foo-trigger',
   *             handler: function() {
   *                 console.log('foo trigger clicked');
   *             }
   *         },
   *         bar: {
   *             cls: 'my-bar-trigger',
   *             handler: function() {
   *                 console.log('bar trigger clicked');
   *             }
   *         }
   *     }
   * });
   * </code></pre>
   *
   * <p>The weight value may be a negative value in order to position custom triggers
   * ahead of default triggers like that of ComboBox.</p>
   *
   * <pre><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.field.ComboBox" rel="Ext.form.field.ComboBox" class="docClass">Ext.form.field.ComboBox</a>', {
   *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
   *     fieldLabel: 'My Custom Field',
   *     triggers: {
   *         foo: {
   *             cls: 'my-foo-trigger',
   *             weight: -2, // negative to place before default triggers
   *             handler: function() {
   *                 console.log('foo trigger clicked');
   *             }
   *         },
   *         bar: {
   *             cls: 'my-bar-trigger',
   *             weight: -1,
   *             handler: function() {
   *                 console.log('bar trigger clicked');
   *             }
   *         }
   *     }
   * });
   * </code></pre>
   * @see #getTriggers()
   * @see #setTriggers()
   */
  public native function get triggers():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set triggers(value:Object):void;

  [ExtConfig]
  /**
   * Specify as <code>true</code> to modify the behaviour of <a href="#!/api/Ext.form.field.Text-cfg-allowBlank" rel="Ext.form.field.Text-cfg-allowBlank" class="docClass">allowBlank</a> so that blank values are not passed as valid, but are subject to any configure <a href="#!/api/Ext.form.field.Text-cfg-vtype" rel="Ext.form.field.Text-cfg-vtype" class="docClass">vtype</a> validation.
   * @default false
   */
  public native function get validateBlank():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set validateBlank(value:Boolean):void;

  [ExtConfig]
  /**
   * A custom validation function to be called during field validation (<a href="#!/api/Ext.form.field.Text-method-getErrors" rel="Ext.form.field.Text-method-getErrors" class="docClass">getErrors</a>).
   * If specified, this function will be called first, allowing the developer to override the default validation
   * process.
   *
   * <pre><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.field.Text" rel="Ext.form.field.Text" class="docClass">Ext.form.field.Text</a>', {
   *     renderTo: document.body,
   *     name: 'phone',
   *     fieldLabel: 'Phone Number',
   *     validator: function (val) {
   *         // remove non-numeric characters
   *         var tn = val.replace(/[^0-9]/g,''),
   *             errMsg = "Must be a 10 digit telephone number";
   *         // if the numeric value is not 10 digits return an error message
   *         return (tn.length === 10) ? true : errMsg;
   *     }
   * });
   * </code></pre>
   */
  public native function get validator():Function;

  [ExtConfig]
  /**
   * @private
   */
  public native function set validator(value:Function):void;

  [ExtConfig]
  /**
   * A validation type name as defined in <a href="#!/api/Ext.form.field.VTypes" rel="Ext.form.field.VTypes" class="docClass">Ext.form.field.VTypes</a>
   */
  public native function get vtype():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set vtype(value:String):void;

  [ExtConfig]
  /**
   * A custom error message to display in place of the default message provided for the <strong><code><a href="#!/api/Ext.form.field.Text-cfg-vtype" rel="Ext.form.field.Text-cfg-vtype" class="docClass">vtype</a></code></strong> currently
   * set for this field. <strong>Note</strong>: only applies if <strong><code><a href="#!/api/Ext.form.field.Text-cfg-vtype" rel="Ext.form.field.Text-cfg-vtype" class="docClass">vtype</a></code></strong> is set, else ignored.
   */
  public native function get vtypeText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set vtypeText(value:String):void;

  /**
   * Validates a value according to the field's validation rules and returns an array of errors
   * for any failing validations. Validation rules are processed in the following order:
   *
   * <ol>
   * <li><p><strong>Field specific validator</strong></p>
   *
   * <p> A validator offers a way to customize and reuse a validation specification.
   *  If a field is configured with a <code><a href="#!/api/Ext.form.field.Text-cfg-validator" rel="Ext.form.field.Text-cfg-validator" class="docClass">validator</a></code>
   *  function, it will be passed the current field value.  The <code><a href="#!/api/Ext.form.field.Text-cfg-validator" rel="Ext.form.field.Text-cfg-validator" class="docClass">validator</a></code>
   *  function is expected to return either:</p>
   *
   * <ul>
   * <li>Boolean <code>true</code>  if the value is valid (validation continues).</li>
   * <li>a String to represent the invalid message if invalid (validation halts).</li>
   * </ul>
   * </li>
   * <li><p><strong>Basic Validation</strong></p>
   *
   * <p> If the <code><a href="#!/api/Ext.form.field.Text-cfg-validator" rel="Ext.form.field.Text-cfg-validator" class="docClass">validator</a></code> has not halted validation,
   *  basic validation proceeds as follows:</p>
   *
   * <ul>
   * <li><p><code><a href="#!/api/Ext.form.field.Text-cfg-allowBlank" rel="Ext.form.field.Text-cfg-allowBlank" class="docClass">allowBlank</a></code> : (Invalid message = <code><a href="#!/api/Ext.form.field.Text-cfg-blankText" rel="Ext.form.field.Text-cfg-blankText" class="docClass">blankText</a></code>)</p>
   *
   * <p>  Depending on the configuration of <code><a href="#!/api/Ext.form.field.Text-cfg-allowBlank" rel="Ext.form.field.Text-cfg-allowBlank" class="docClass">allowBlank</a></code>, a
   *   blank field will cause validation to halt at this step and return
   *   Boolean true or false accordingly.</p></li>
   * <li><p><code><a href="#!/api/Ext.form.field.Text-cfg-minLength" rel="Ext.form.field.Text-cfg-minLength" class="docClass">minLength</a></code> : (Invalid message = <code><a href="#!/api/Ext.form.field.Text-cfg-minLengthText" rel="Ext.form.field.Text-cfg-minLengthText" class="docClass">minLengthText</a></code>)</p>
   *
   * <p>  If the passed value does not satisfy the <code><a href="#!/api/Ext.form.field.Text-cfg-minLength" rel="Ext.form.field.Text-cfg-minLength" class="docClass">minLength</a></code>
   *   specified, validation halts.</p></li>
   * <li><p><code><a href="#!/api/Ext.form.field.Text-cfg-maxLength" rel="Ext.form.field.Text-cfg-maxLength" class="docClass">maxLength</a></code> : (Invalid message = <code><a href="#!/api/Ext.form.field.Text-cfg-maxLengthText" rel="Ext.form.field.Text-cfg-maxLengthText" class="docClass">maxLengthText</a></code>)</p>
   *
   * <p> If the passed value does not satisfy the <code><a href="#!/api/Ext.form.field.Text-cfg-maxLength" rel="Ext.form.field.Text-cfg-maxLength" class="docClass">maxLength</a></code>
   *  specified, validation halts.</p></li>
   * </ul>
   * </li>
   * <li><p><strong>Preconfigured Validation Types (VTypes)</strong></p>
   *
   * <p> If none of the prior validation steps halts validation, a field
   *  configured with a <code><a href="#!/api/Ext.form.field.Text-cfg-vtype" rel="Ext.form.field.Text-cfg-vtype" class="docClass">vtype</a></code> will utilize the
   *  corresponding <a href="#!/api/Ext.form.field.VTypes" rel="Ext.form.field.VTypes" class="docClass">VTypes</a> validation function.
   *  If invalid, either the field's <code><a href="#!/api/Ext.form.field.Text-cfg-vtypeText" rel="Ext.form.field.Text-cfg-vtypeText" class="docClass">vtypeText</a></code> or
   *  the VTypes vtype Text property will be used for the invalid message.
   *  Keystrokes on the field will be filtered according to the VTypes
   *  vtype Mask property.</p></li>
   * <li><p><strong>Field specific regex test</strong></p>
   *
   * <p> If none of the prior validation steps halts validation, a field's
   *  configured <code><a href="#!/api/Ext.form.field.Text-cfg-regex" rel="Ext.form.field.Text-cfg-regex" class="docClass">regex</a></code> test will be processed.
   *  The invalid message for this test is configured with <code><a href="#!/api/Ext.form.field.Text-cfg-regexText" rel="Ext.form.field.Text-cfg-regexText" class="docClass">regexText</a></code></p></li>
   * </ol>
   * @param value The value to validate. The processed raw value will be used if nothing is passed.
   * @return Array of any validation errors
   */
  override public native function getErrors(value:*):Array;

  /**
   * Performs any necessary manipulation of a raw String value to prepare it for conversion and/or
   * <a href="#!/api/Ext.form.field.Text-method-validate" rel="Ext.form.field.Text-method-validate" class="docClass">validation</a>. For text fields this applies the configured <a href="#!/api/Ext.form.field.Text-cfg-stripCharsRe" rel="Ext.form.field.Text-cfg-stripCharsRe" class="docClass">stripCharsRe</a>
   * to the raw value.
   * @param value The unprocessed string value
   * @return The processed string value
   */
  override public native function processRawValue(value:Object):Object;

  /**
   * Sets the read-only state of this field.
   * @param readOnly True to prevent the user changing the field and explicitly
   * hide the trigger(s). Setting this to true will supersede settings editable and
   * hideTrigger. Setting this to false will defer back to <a href="#!/api/Ext.form.field.Text-cfg-editable" rel="Ext.form.field.Text-cfg-editable" class="docClass">editable</a> and <a href="#!/api/Ext.form.field.Text-cfg-hideTrigger" rel="Ext.form.field.Text-cfg-hideTrigger" class="docClass">hideTrigger</a>.
   */
  override public native function setReadOnly(readOnly:Boolean):void;

  /**
   * Automatically grows the field to accommodate the width of the text up to the maximum
   * field width allowed. This only takes effect if <a href="#!/api/Ext.form.field.Text-cfg-grow" rel="Ext.form.field.Text-cfg-grow" class="docClass">grow</a> = true, and fires the
   * <a href="#!/api/Ext.form.field.Text-event-autosize" rel="Ext.form.field.Text-event-autosize" class="docClass">autosize</a> event if the width changes.
   */
  public native function autoSize():void;

  /**
   * Returns the value of this field's <a href="#!/api/Ext.form.field.Text-cfg-emptyText" rel="Ext.form.field.Text-cfg-emptyText" class="docClass">emptyText</a>
   * @return The value of this field's emptyText
   */
  public native function getEmptyText():String;

  /**
   * Returns the value of <a href="#!/api/Ext.form.field.Text-cfg-hideTrigger" rel="Ext.form.field.Text-cfg-hideTrigger" class="docClass">hideTrigger</a>.
   */
  public native function getHideTrigger():Boolean;

  /**
   * Returns the trigger with the given id
   */
  public native function getTrigger(id:String):ext.form.trigger.Trigger;

  [Deprecated(since = "5.0")]
  /**
   * Get the total width of the trigger button area.
   * @return The total trigger width
   */
  public native function getTriggerWidth():Number;

  /**
   * Returns the value of <a href="#!/api/Ext.form.field.Text-cfg-triggers" rel="Ext.form.field.Text-cfg-triggers" class="docClass">triggers</a>.
   */
  public native function getTriggers():Object;

  /**
   * Selects text in this field
   * @param start The index where the selection should start
   * @default 0
   * @param end The index where the selection should end (defaults to the text length)
   */
  public native function selectText(start:Number = 0, end:Number = NaN):void;

  /**
   * Sets the default text to place into an empty field
   * @param value The <a href="#!/api/Ext.form.field.Text-cfg-emptyText" rel="Ext.form.field.Text-cfg-emptyText" class="docClass">emptyText</a> value for this field
   * @return this
   */
  public native function setEmptyText(value:String):ext.form.field.TextField;

  /**
   * Sets the value of <a href="#!/api/Ext.form.field.Text-cfg-hideTrigger" rel="Ext.form.field.Text-cfg-hideTrigger" class="docClass">hideTrigger</a>.
   * @param hideTrigger The new value.
   */
  public native function setHideTrigger(hideTrigger:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.form.field.Text-cfg-triggers" rel="Ext.form.field.Text-cfg-triggers" class="docClass">triggers</a>.
   * @param triggers The new value.
   */
  public native function setTriggers(triggers:Object):void;
}
}