package ext.form.field {

[Uses("ext.form.field.ComboBox")]
[Uses("ext.util.StoreHolder")]
[Uses("ext.Base")]
[Uses("ext.form.field.Picker")]
[Uses("ext.form.field.TextField")]
[Uses("ext.form.field.BaseField")]
[Uses("ext.Component")]
[Uses("ext.util.Positionable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Animate")]
[Uses("ext.util.Floating")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.state.Stateful")]
[Uses("ext.form.Labelable")]
[Uses("ext.form.field.Field")]
[Native("Ext.form.field.Time", require)]
/**
 * Provides a time input field with a time dropdown and automatic time validation.
 *
 * <p>This field recognizes and uses JavaScript Date objects as its main <a href="#!/api/Ext.form.field.Time-cfg-value" rel="Ext.form.field.Time-cfg-value" class="docClass">value</a> type (only the time portion of the
 * date is used; the month/day/year are ignored). In addition, it recognizes string values which are parsed according to
 * the <a href="#!/api/Ext.form.field.Time-cfg-format" rel="Ext.form.field.Time-cfg-format" class="docClass">format</a> and/or <a href="#!/api/Ext.form.field.Time-cfg-altFormats" rel="Ext.form.field.Time-cfg-altFormats" class="docClass">altFormats</a> configs. These may be reconfigured to use time formats appropriate for
 * the user's locale.</p>
 *
 * <p>The field may be limited to a certain range of times by using the <a href="#!/api/Ext.form.field.Time-cfg-minValue" rel="Ext.form.field.Time-cfg-minValue" class="docClass">minValue</a> and <a href="#!/api/Ext.form.field.Time-cfg-maxValue" rel="Ext.form.field.Time-cfg-maxValue" class="docClass">maxValue</a> configs,
 * and the interval between time options in the dropdown can be changed with the <a href="#!/api/Ext.form.field.Time-cfg-increment" rel="Ext.form.field.Time-cfg-increment" class="docClass">increment</a> config.</p>
 *
 * <p>Example usage:</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a>', {
 *     title: 'Time Card',
 *     width: 300,
 *     bodyPadding: 10,
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     items: [{
 *         xtype: 'timefield',
 *         name: 'in',
 *         fieldLabel: 'Time In',
 *         minValue: '6:00 AM',
 *         maxValue: '8:00 PM',
 *         increment: 30,
 *         anchor: '100%'
 *     }, {
 *         xtype: 'timefield',
 *         name: 'out',
 *         fieldLabel: 'Time Out',
 *         minValue: '6:00 AM',
 *         maxValue: '8:00 PM',
 *         increment: 30,
 *         anchor: '100%'
 *    }]
 * });
 * </code></pre>
 */
public class TimeField extends ext.form.field.ComboBox {
  /**
   * @param config @inheritDoc
   */
  public function TimeField(config:ext.form.field.TimeField = null) {
    super();
  }

  [ExtConfig]
  /**
   * The format text to be announced by screen readers when the field is focused.
   */
  override public native function get formatText():String;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set formatText(value:String):void;

  [ExtConfig]
  /**
   * The error text to display when the time in the field is invalid.
   * @default "{0} is not a valid time"
   */
  override public native function get invalidText():String;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set invalidText(value:String):void;

  [ExtConfig]
  /**
   * An additional CSS class used to style the trigger button. The trigger will always get the <a href="#!/api/Ext.form.trigger.Trigger-property-baseCls" rel="Ext.form.trigger.Trigger-property-baseCls" class="docClass">Ext.form.trigger.Trigger.baseCls</a>
   * by default and triggerCls will be <strong>appended</strong> if specified.
   * @default 'x-form-time-trigger'
   */
  override public native function get triggerCls():String;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set triggerCls(value:String):void;

  [ExtConfig]
  /**
   * Multiple date formats separated by "|" to try when parsing a user input value and it doesn't match the defined
   * format.
   * @default "g:ia|g:iA|g:i a|g:i A|h:i|g:i|H:i|ga|ha|gA|h a|g a|g A|gi|hi|gia|hia|g|H|gi a|hi a|giA|hiA|gi A|hi A"
   */
  public native function get altFormats():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set altFormats(value:String):void;

  [ExtConfig]
  /**
   * The default time format string which can be overridden for localization support.
   * The format must be valid according to <a href="#!/api/Ext.Date-method-parse" rel="Ext.Date-method-parse" class="docClass">Ext.Date.parse</a>.
   *
   * <p>Defaults to <code>'g:i A'</code>, e.g., <code>'3:15 PM'</code>. For 24-hour time format try <code>'H:i'</code> instead.</p>
   * @default undefined
   */
  public native function get format():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set format(value:String):void;

  [ExtConfig]
  /**
   * The number of minutes between each time value in the list.
   *
   * <p>Note that this only affects the <em>list of suggested times.</em></p>
   *
   * <p>To enforce that only times on the list are valid, use <a href="#!/api/Ext.form.field.Time-cfg-snapToIncrement" rel="Ext.form.field.Time-cfg-snapToIncrement" class="docClass">snapToIncrement</a>. That will coerce
   * any typed values to the nearest increment point upon blur.</p>
   * @default 15
   */
  public native function get increment():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set increment(value:Number):void;

  [ExtConfig]
  /**
   * The error text to display when the entered time is after <a href="#!/api/Ext.form.field.Time-cfg-maxValue" rel="Ext.form.field.Time-cfg-maxValue" class="docClass">maxValue</a>.
   * @default "The time in this field must be equal to or before {0}"
   */
  public native function get maxText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set maxText(value:String):void;

  [ExtConfig]
  /**
   * The maximum allowed time. Can be either a Javascript date object with a valid time value or a string time in a
   * valid format -- see <a href="#!/api/Ext.form.field.Time-cfg-format" rel="Ext.form.field.Time-cfg-format" class="docClass">format</a> and <a href="#!/api/Ext.form.field.Time-cfg-altFormats" rel="Ext.form.field.Time-cfg-altFormats" class="docClass">altFormats</a>.
   * @see #setMaxValue()
   */
  public native function get maxValue():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set maxValue(value:*):void;

  [ExtConfig]
  /**
   * The error text to display when the entered time is before <a href="#!/api/Ext.form.field.Time-cfg-minValue" rel="Ext.form.field.Time-cfg-minValue" class="docClass">minValue</a>.
   * @default "The time in this field must be equal to or after {0}"
   */
  public native function get minText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set minText(value:String):void;

  [ExtConfig]
  /**
   * The minimum allowed time. Can be either a Javascript date object with a valid time value or a string time in a
   * valid format -- see <a href="#!/api/Ext.form.field.Time-cfg-format" rel="Ext.form.field.Time-cfg-format" class="docClass">format</a> and <a href="#!/api/Ext.form.field.Time-cfg-altFormats" rel="Ext.form.field.Time-cfg-altFormats" class="docClass">altFormats</a>.
   * @see #setMinValue()
   */
  public native function get minValue():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set minValue(value:*):void;

  [ExtConfig]
  /**
   * The maximum height of the <a href="#!/api/Ext.picker.Time" rel="Ext.picker.Time" class="docClass">Ext.picker.Time</a> dropdown.
   * @default 300
   */
  public native function get pickerMaxHeight():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set pickerMaxHeight(value:Number):void;

  [ExtConfig]
  /**
   * Specify as <code>true</code> to enforce that only values on the <a href="#!/api/Ext.form.field.Time-cfg-increment" rel="Ext.form.field.Time-cfg-increment" class="docClass">increment</a> boundary are accepted.
   *
   * <p>Typed values will be coerced to the nearest <a href="#!/api/Ext.form.field.Time-cfg-increment" rel="Ext.form.field.Time-cfg-increment" class="docClass">increment</a> point on blur.</p>
   * @default false
   */
  public native function get snapToIncrement():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set snapToIncrement(value:Boolean):void;

  [ExtConfig]
  /**
   * The date format string which will be submitted to the server. The format must be valid according to
   * <a href="#!/api/Ext.Date-method-parse" rel="Ext.Date-method-parse" class="docClass">Ext.Date.parse</a>.
   *
   * <p>Defaults to <a href="#!/api/Ext.form.field.Time-cfg-format" rel="Ext.form.field.Time-cfg-format" class="docClass">format</a>.</p>
   * @default undefined
   */
  public native function get submitFormat():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set submitFormat(value:String):void;

  /**
   * Runs all of Time's validations and returns an array of any errors. Note that this first runs Text's validations,
   * so the returned array is an amalgamation of all field errors. The additional validation checks are testing that
   * the time format is valid, that the chosen time is within the <a href="#!/api/Ext.form.field.Time-cfg-minValue" rel="Ext.form.field.Time-cfg-minValue" class="docClass">minValue</a> and <a href="#!/api/Ext.form.field.Time-cfg-maxValue" rel="Ext.form.field.Time-cfg-maxValue" class="docClass">maxValue</a> constraints
   * set.
   * @param value The value to get errors for (defaults to the current field value)
   * @return All validation errors for this field
   */
  override public native function getErrors(value:*):Array;

  /**
   * Replaces any existing <a href="#!/api/Ext.form.field.Time-cfg-maxValue" rel="Ext.form.field.Time-cfg-maxValue" class="docClass">maxValue</a> with the new time and refreshes the picker's range.
   * @param value The maximum time that can be selected
   */
  public native function setMaxValue(value:*):void;

  /**
   * Replaces any existing <a href="#!/api/Ext.form.field.Time-cfg-minValue" rel="Ext.form.field.Time-cfg-minValue" class="docClass">minValue</a> with the new time and refreshes the picker's range.
   * @param value The minimum time that can be selected
   */
  public native function setMinValue(value:*):void;
}
}