package ext.grid.column {

[Uses("ext.grid.column.Column")]
[Uses("ext.grid.header.HeaderContainer")]
[Uses("ext.container.Container")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.Base")]
[Uses("ext.mixin.Container")]
[Uses("ext.Mixin")]
[Uses("ext.Component")]
[Uses("ext.util.Positionable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Animate")]
[Uses("ext.util.Floating")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.state.Stateful")]
[Uses("ext.util.FocusableContainer")]
[Native("Ext.grid.column.Check", require)]
/**
 * Fires when the UI requests a change of check status.
 * The change may be vetoed by returning <code>false</code> from a listener.
 * @eventType ext.grid.column.events.CheckColumn_rowIndex_checked_record_eEvent.BEFORE_CHECK_CHANGE
 */
[Event(name = 'onBeforeCheckChange', type = 'ext.grid.column.events.CheckColumn_rowIndex_checked_record_eEvent')]
/**
 * Fires when the header is clicked and before the mass check/uncheck takes place.
 * The change may be vetoed by returning <code>false</code> from a listener.
 * @eventType ext.grid.column.events.CheckColumn_checked_eEvent.BEFORE_HEADER_CHECK_CHANGE
 */
[Event(name = 'onBeforeHeaderCheckChange', type = 'ext.grid.column.events.CheckColumn_checked_eEvent')]
/**
 * Fires when the UI has successfully changed the checked state of a row.
 * @eventType ext.grid.column.events.CheckColumn_rowIndex_checked_record_eEvent.CHECK_CHANGE
 */
[Event(name = 'onCheckChange', type = 'ext.grid.column.events.CheckColumn_rowIndex_checked_record_eEvent')]
/**
 * Fires after the header is clicked and a mass check/uncheck operation has been completed.
 * @eventType ext.grid.column.events.CheckColumn_checked_eEvent.HEADER_CHECK_CHANGE
 */
[Event(name = 'onHeaderCheckChange', type = 'ext.grid.column.events.CheckColumn_checked_eEvent')]
/**
 * A Column subclass which renders a checkbox in each column cell which toggles the truthiness of the associated data field on click.
 *
 * <p>Example usage:</p>
 *
 * <pre class='inline-example '><code>var store = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
 *     fields: ['name', 'email', 'phone', 'active'],
 *     data: [
 *         { name: 'Lisa', email: 'lisa&#64;simpsons.com', phone: '555-111-1224', active: true },
 *         { name: 'Bart', email: 'bart&#64;simpsons.com', phone: '555-222-1234', active: true },
 *         { name: 'Homer', email: 'homer&#64;simpsons.com', phone: '555-222-1244', active: false },
 *         { name: 'Marge', email: 'marge&#64;simpsons.com', phone: '555-222-1254', active: true }
 *     ]
 * });
 *
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>', {
 *     title: 'Simpsons',
 *     height: 200,
 *     width: 400,
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     store: store,
 *     columns: [
 *         { text: 'Name', dataIndex: 'name' },
 *         { text: 'Email', dataIndex: 'email', flex: 1 },
 *         { text: 'Phone', dataIndex: 'phone' },
 *         { xtype: 'checkcolumn', text: 'Active', dataIndex: 'active' }
 *     ]
 * });
 * </code></pre>
 *
 * <p>The check column can be at any index in the columns array.</p>
 */
public class CheckColumn extends ext.grid.column.Column {
  /**
   * @param config @inheritDoc
   */
  public function CheckColumn(config:ext.grid.column.CheckColumn = null) {
    super();
  }

  [ExtConfig]
  /**
   * The tooltip text to show upon hover of a checked cell.
   */
  override public native function get tooltip():String;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set tooltip(value:String):void;

  /**
   * <code>true</code> in this class to identify an object as an instantiated Check column, or subclass thereof.
   * @default true
   */
  public native function get isCheckColumn():Boolean;

  /**
   * @private
   */
  public native function set isCheckColumn(value:Boolean):void;

  [ExtConfig]
  /**
   * The tooltip text to show upon hover of an unchecked cell.
   */
  public native function get checkedTooltip():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set checkedTooltip(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Configure as <code>true</code> to display a checkbox below the header text.
   *
   * <p>Clicking the checkbox will check/uncheck all records.</p>
   * @default false
   * @see #getHeaderCheckbox()
   * @see #setHeaderCheckbox()
   */
  public native function get headerCheckbox():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set headerCheckbox(value:Boolean):void;

  [ExtConfig]
  /**
   * Use <code>true</code> to display a check when the value is <code>false</code> instead of when the value
   * is <code>true</code>.
   * @default false
   */
  public native function get invert():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set invert(value:Boolean):void;

  [ExtConfig]
  /**
   * Prevent grid selection upon mousedown.
   * @default true
   */
  public native function get stopSelection():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set stopSelection(value:Boolean):void;

  [ExtConfig]
  /**
   * The mouse event which triggers the toggle of a single cell.
   * @default click
   */
  public native function get triggerEvent():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set triggerEvent(value:String):void;

  /**
   * Disables this CheckColumn.
   */
  override protected native function onDisable():void;

  /**
   * Enables this CheckColumn.
   */
  override protected native function onEnable():void;

  /**
   * Returns the value of <a href="#!/api/Ext.grid.column.Check-cfg-headerCheckbox" rel="Ext.grid.column.Check-cfg-headerCheckbox" class="docClass">headerCheckbox</a>.
   */
  public native function getHeaderCheckbox():Boolean;

  /**
   * Sets the value of <a href="#!/api/Ext.grid.column.Check-cfg-headerCheckbox" rel="Ext.grid.column.Check-cfg-headerCheckbox" class="docClass">headerCheckbox</a>.
   * @param headerCheckbox The new value.
   */
  public native function setHeaderCheckbox(headerCheckbox:Boolean):void;
}
}