package ext.grid.column {
import ext.dom.Element;
import ext.form.field.IField;
import ext.grid.header.HeaderContainer;

[Uses("ext.grid.header.HeaderContainer")]
[Uses("ext.container.Container")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.Base")]
[Uses("ext.mixin.Container")]
[Uses("ext.Mixin")]
[Uses("ext.Component")]
[Uses("ext.util.Positionable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Animate")]
[Uses("ext.util.Floating")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.state.Stateful")]
[Uses("ext.util.FocusableContainer")]
[Native("Ext.grid.column.Column", require)]
/**
 * This class specifies the definition for a column inside a <a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>. It encompasses
 * both the grid header configuration as well as displaying data within the grid itself. If the
 * <a href="#!/api/Ext.grid.column.Column-cfg-columns" rel="Ext.grid.column.Column-cfg-columns" class="docClass">columns</a> configuration is specified, this column will become a column group and can
 * contain other columns inside. In general, this class will not be created directly, rather
 * an array of column configurations will be passed to the grid:
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
 *     storeId:'employeeStore',
 *     fields:['firstname', 'lastname', 'seniority', 'dep', 'hired'],
 *     data:[
 *         {firstname:"Michael", lastname:"Scott", seniority:7, dep:"Management", hired:"01/10/2004"},
 *         {firstname:"Dwight", lastname:"Schrute", seniority:2, dep:"Sales", hired:"04/01/2004"},
 *         {firstname:"Jim", lastname:"Halpert", seniority:3, dep:"Sales", hired:"02/22/2006"},
 *         {firstname:"Kevin", lastname:"Malone", seniority:4, dep:"Accounting", hired:"06/10/2007"},
 *         {firstname:"Angela", lastname:"Martin", seniority:5, dep:"Accounting", hired:"10/21/2008"}
 *     ]
 * });
 *
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>', {
 *     title: 'Column Demo',
 *     store: <a href="#!/api/Ext.data.StoreManager-method-lookup" rel="Ext.data.StoreManager-method-lookup" class="docClass">Ext.data.StoreManager.lookup</a>('employeeStore'),
 *     columns: [
 *         {text: 'First Name',  dataIndex:'firstname'},
 *         {text: 'Last Name',  dataIndex:'lastname'},
 *         {text: 'Hired Month',  dataIndex:'hired', xtype:'datecolumn', format:'M'},
 *         {text: 'Department (Yrs)', xtype:'templatecolumn', tpl:'{dep} ({seniority})'}
 *     ],
 *     width: 400,
 *     forceFit: true,
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 * });
 * </code></pre>
 *
 * <h1>Convenience Subclasses</h1>
 *
 * <p>There are several column subclasses that provide default rendering for various data types</p>
 *
 * <ul>
 * <li><a href="#!/api/Ext.grid.column.Action" rel="Ext.grid.column.Action" class="docClass">Ext.grid.column.Action</a>: Renders icons that can respond to click events inline</li>
 * <li><a href="#!/api/Ext.grid.column.Boolean" rel="Ext.grid.column.Boolean" class="docClass">Ext.grid.column.Boolean</a>: Renders for boolean values</li>
 * <li><a href="#!/api/Ext.grid.column.Date" rel="Ext.grid.column.Date" class="docClass">Ext.grid.column.Date</a>: Renders for date values</li>
 * <li><a href="#!/api/Ext.grid.column.Number" rel="Ext.grid.column.Number" class="docClass">Ext.grid.column.Number</a>: Renders for numeric values</li>
 * <li><a href="#!/api/Ext.grid.column.Template" rel="Ext.grid.column.Template" class="docClass">Ext.grid.column.Template</a>: Renders a value using an <a href="#!/api/Ext.XTemplate" rel="Ext.XTemplate" class="docClass">Ext.XTemplate</a> using the record data</li>
 * </ul>
 *
 *
 * <h1>Setting Widths</h1>
 *
 * <p>The columns are laid out by a <a href="#!/api/Ext.layout.container.HBox" rel="Ext.layout.container.HBox" class="docClass">Ext.layout.container.HBox</a> layout, so a column can either
 * be given an explicit width value or a <a href="#!/api/Ext.grid.column.Column-cfg-flex" rel="Ext.grid.column.Column-cfg-flex" class="docClass">flex</a> configuration. If no width is specified the grid will
 * automatically the size the column to 100px.</p>
 *
 * <p>Group columns (columns with <a href="#!/api/Ext.grid.column.Column-cfg-columns" rel="Ext.grid.column.Column-cfg-columns" class="docClass">child columns</a>) may be sized using <a href="#!/api/Ext.grid.column.Column-cfg-flex" rel="Ext.grid.column.Column-cfg-flex" class="docClass">flex</a>,
 * in which case they will apply <code>forceFit</code> to their child columns so as not to leave blank space.</p>
 *
 * <p>If a group column is not flexed, its width is calculated by measuring the width of the
 * child columns, so a width option should not be specified in that case.</p>
 *
 * <h1>Header Options</h1>
 *
 * <ul>
 * <li><a href="#!/api/Ext.grid.column.Column-cfg-text" rel="Ext.grid.column.Column-cfg-text" class="docClass">text</a>: Sets the header text for the column</li>
 * <li><a href="#!/api/Ext.grid.column.Column-cfg-sortable" rel="Ext.grid.column.Column-cfg-sortable" class="docClass">sortable</a>: Specifies whether the column can be sorted by clicking the header or using the column menu</li>
 * <li><a href="#!/api/Ext.grid.column.Column-cfg-hideable" rel="Ext.grid.column.Column-cfg-hideable" class="docClass">hideable</a>: Specifies whether the column can be hidden using the column menu</li>
 * <li><a href="#!/api/Ext.grid.column.Column-cfg-menuDisabled" rel="Ext.grid.column.Column-cfg-menuDisabled" class="docClass">menuDisabled</a>: Disables the column header menu</li>
 * <li><a href="#!/api/Ext.grid.column.Column-cfg-draggable" rel="Ext.grid.column.Column-cfg-draggable" class="docClass">draggable</a>: Specifies whether the column header can be reordered by dragging</li>
 * <li><a href="#!/api/Ext.grid.column.Column-cfg-groupable" rel="Ext.grid.column.Column-cfg-groupable" class="docClass">groupable</a>: Specifies whether the grid can be grouped by the column dataIndex. See also <a href="#!/api/Ext.grid.feature.Grouping" rel="Ext.grid.feature.Grouping" class="docClass">Ext.grid.feature.Grouping</a></li>
 * </ul>
 *
 *
 * <h1>Data Options</h1>
 *
 * <ul>
 * <li><a href="#!/api/Ext.grid.column.Column-cfg-dataIndex" rel="Ext.grid.column.Column-cfg-dataIndex" class="docClass">dataIndex</a>: The dataIndex is the field in the underlying <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a> to use as the value for the column.</li>
 * <li><a href="#!/api/Ext.grid.column.Column-cfg-renderer" rel="Ext.grid.column.Column-cfg-renderer" class="docClass">renderer</a>: Allows the underlying store
 * value to be transformed before being displayed in the grid</li>
 * </ul>
 *
 *
 * <h2>State saving</h2>
 *
 * <p>When the owning <a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Grid</a> is configured
 * <a href="#!/api/Ext.grid.Panel-cfg-stateful" rel="Ext.grid.Panel-cfg-stateful" class="docClass">Ext.grid.Panel.stateful</a>, it will save its column state (order and width)
 * encapsulated within the default Panel state of changed width and height and
 * collapsed/expanded state.</p>
 *
 * <p>On a <code>stateful</code> grid, not only should the Grid have a
 * <a href="#!/api/Ext.grid.Panel-cfg-stateId" rel="Ext.grid.Panel-cfg-stateId" class="docClass">Ext.grid.Panel.stateId</a>, each column of the grid should also be configured
 * with a <a href="#!/api/Ext.grid.column.Column-cfg-stateId" rel="Ext.grid.column.Column-cfg-stateId" class="docClass">stateId</a> which identifies that column locally within the grid.</p>
 *
 * <p>Omitting the <code>stateId</code> config from the columns results in columns with generated
 * internal ID's.  The generated ID's are subject to change on each page load
 * making it impossible for the state manager to restore the previous state of the
 * columns.</p>
 */
public class Column extends ext.grid.header.HeaderContainer {
  /**
   * @param config @inheritDoc
   */
  public function Column(config:ext.grid.column.Column = null) {
    super();
  }

  [ExtConfig]
  /**
   * False to disable drag-drop reordering of this column.
   * @default true
   */
  override public native function get draggable():*;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set draggable(value:*):void;

  [Deprecated(replacement = "resizable", since = "4.0")]
  [ExtConfig]
  /**
   * True to prevent the column from being resizable.
   */
  override public native function get fixed():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set fixed(value:Boolean):void;

  [ExtConfig]
  /**
   * False to prevent the column from being resizable.
   * @default true
   */
  override public native function get resizable():*;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set resizable(value:*):void;

  [ExtConfig]
  /**
   * False to disable sorting of this column. Whether local/remote sorting is used is specified in
   * <code><a href="#!/api/Ext.data.Store-cfg-remoteSort" rel="Ext.data.Store-cfg-remoteSort" class="docClass">Ext.data.Store.remoteSort</a></code>.
   * @default true
   */
  override public native function get sortable():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set sortable(value:Boolean):void;

  [ExtConfig]
  /**
   * An identifier which identifies this column uniquely within the owning grid's <a href="#!/api/Ext.grid.column.Column-cfg-stateful" rel="Ext.grid.column.Column-cfg-stateful" class="docClass">state</a>.
   *
   * <p>This does not have to be <em>globally</em> unique. A column's state is not saved standalone. It is encapsulated within
   * the owning grid's state.</p>
   */
  override public native function get stateId():String;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set stateId(value:String):void;

  /**
   * Set in this class to identify, at runtime, instances which are not instances of the
   * HeaderContainer base class, but are in fact simple column headers.
   * @default true
   */
  public native function get isColumn():Boolean;

  [Deprecated(message = "see isColumn\nSet in this class to identify, at runtime, instances which are not instances of the\nHeaderContainer base class, but are in fact, the subclass: Header.\n")]
  /**
   * @default true
   */
  public native function get isHeader():Boolean;

  /**
   * @private
   */
  public native function set isHeader(value:Boolean):void;

  /**
   * Element that contains the text in column header.
   */
  public native function get textEl():ext.dom.Element;

  /**
   * @private
   */
  public native function set textEl(value:ext.dom.Element):void;

  /**
   * Element that acts as button for column header dropdown menu.
   */
  public native function get triggerEl():ext.dom.Element;

  /**
   * @private
   */
  public native function set triggerEl(value:ext.dom.Element):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Sets the alignment of the header and rendered columns.
   * Possible values are: <code>'start'</code>, <code>'center'</code>, and <code>'end'</code>.
   *
   * <p>Since 6.2.0, <code>'left'</code> and <code>'right'</code> will still work, but retain their meaning
   * even when the application is in RTL mode.</p>
   *
   * <p><code>'start'</code> and <code>'end'</code> always conform to the locale's text direction.</p>
   * @default 'start'
   * @see #getAlign()
   * @see #setAlign()
   */
  public native function get align():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set align(value:String):void;

  [ExtConfig]
  /**
   * Configure as <code>false</code> to remove all cells in this column from navigation.
   *
   * <p>This is currently used by the PivotGrid package to create columns which have
   * no semantic role, but are purely for visual indentation purposes.</p>
   * @default true
   * @since 6.2.0.
   */
  public native function get cellFocusable():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set cellFocusable(value:Boolean):void;

  [ExtConfig]
  /**
   * True to allow whitespace in this column's cells to wrap, and cause taller column height where
   * necessary.
   *
   * <p>This implicitly sets the <a href="#!/api/Ext.grid.column.Column-cfg-variableRowHeight" rel="Ext.grid.column.Column-cfg-variableRowHeight" class="docClass">variableRowHeight</a> config to <code>true</code></p>
   * @default false
   */
  public native function get cellWrap():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set cellWrap(value:Boolean):void;

  [ExtConfig]
  /**
   * An optional array of sub-column definitions. This column becomes a group, and houses the columns defined in the
   * <code>columns</code> config.
   *
   * <p>Group columns may not be sortable. But they may be hideable and moveable. And you may move headers into and out
   * of a group. Note that if all sub columns are dragged out of a group, the group is destroyed.</p>
   */
  public native function get columns():Array;

  [ExtConfig]
  /**
   * @private
   */
  public native function set columns(value:Array):void;

  [ExtConfig]
  /**
   * The name of the field in the grid's <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>'s <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a> definition from
   * which to draw the column's value. <strong>Required.</strong>
   */
  public native function get dataIndex():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set dataIndex(value:String):void;

  [ExtConfig]
  /**
   * This text will be announced by Assistive Technologies such as screen readers when
   * a cell with changed ("dirty") value is focused.
   * @default "Cell value has been edited"
   */
  public native function get dirtyText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set dirtyText(value:String):void;

  [ExtConfig]
  /**
   * A renderer to be used in conjunction with
   * <a href="#!/api/Ext.grid.plugin.RowEditing" rel="Ext.grid.plugin.RowEditing" class="docClass">RowEditing</a>. This renderer is used to display a
   * custom value for non-editable fields.
   *
   * <p><strong>Note:</strong> The editRenderer is called when the roweditor is initially shown.
   * Changes to the record during editing will not call editRenderer.</p>
   *
   * <pre><code>var store = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
   *     fields: ['name', 'email'],
   *     data: [{
   *         "name": "Finn",
   *         "email": "finn&#64;adventuretime.com"
   *     }, {
   *         "name": "Jake",
   *         "email": "jake&#64;adventuretime.com"
   *     }]
   * });
   *
   * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>', {
   *     title: 'Land Of Ooo',
   *     store: store,
   *     columns: [{
   *         text: 'Name',
   *         dataIndex: 'name',
   *         editRenderer: function(value){
   *             return '&lt;span style="color:gray;"&gt;' + value + '&lt;/span&gt;';
   *           }
   *     }, {
   *         text: 'Email',
   *         dataIndex: 'email',
   *         flex: 1,
   *         editor: {
   *             xtype: 'textfield',
   *             allowBlank: false
   *         }
   *     }],
   *     plugins: {
   *         ptype: 'rowediting',
   *         clicksToEdit: 1
   *     },
   *     height: 200,
   *     width: 400,
   *     renderTo: document.body
   * });
   * </code></pre>
   */
  public native function get editRenderer():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set editRenderer(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * An optional xtype or config object for a <a href="#!/api/Ext.form.field.Field" rel="Ext.form.field.Field" class="docClass">Field</a> to use for editing.
   * Only applicable if the grid is using an <a href="#!/api/Ext.grid.plugin.Editing" rel="Ext.grid.plugin.Editing" class="docClass">Editing</a> plugin.
   *
   * <p><strong>Note:</strong> The <a href="#!/api/Ext.form.field.HtmlEditor" rel="Ext.form.field.HtmlEditor" class="docClass">HtmlEditor</a> field is not a
   * supported editor field type.</p>
   * @see #getEditor()
   * @see #setEditor()
   */
  public native function get editor():ext.form.field.IField;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set editor(value:ext.form.field.IField):void;

  [ExtConfig]
  /**
   * The text to display in empty cells (cells with a value of <code>undefined</code>, <code>null</code>, or <code>''</code>).
   *
   * <p>Defaults to <code>&amp;#160;</code> aka <code>&amp;nbsp;</code>.</p>
   * @default undefined
   */
  public native function get emptyCellText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set emptyCellText(value:String):void;

  [ExtConfig]
  /**
   * True to enable text selection inside grid cells in this column.
   * @default false
   */
  public native function get enableTextSelection():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set enableTextSelection(value:Boolean):void;

  [ExtConfig]
  /**
   * A style definition that is used during data export via the Ext.grid.plugin.Exporter.
   * This style will be applied to the columns generated in the exported file.
   *
   * <p>You could define it as a single object that will be used by all exporters:</p>
   *
   * <pre><code> {
   *      xtype: 'numbercolumn',
   *      dataIndex: 'price',
   *      text: 'Price',
   *      exportStyle: {
   *          format: 'Currency',
   *          alignment: {
   *              horizontal: 'Right'
   *          },
   *          font: {
   *              italic: true
   *          }
   *      }
   *  }
   * </code></pre>
   *
   * <p>You could also define it as an array of objects, each object having a <code>type</code> that specifies by
   * which exporter will be used:</p>
   *
   * <pre><code> {
   *      xtype: 'numbercolumn',
   *      dataIndex: 'price',
   *      text: 'Price',
   *      exportStyle: [{
   *          type: 'html', // used by the `html` exporter
   *          format: 'Currency',
   *          alignment: {
   *              horizontal: 'Right'
   *          },
   *          font: {
   *              italic: true
   *          }
   *      },{
   *          type: 'csv', // used by the `csv` exporter
   *          format: 'General'
   *      }]
   *  }
   * </code></pre>
   *
   * <p>Or you can define it as an array of objects that has:</p>
   *
   * <ul>
   * <li>one object with no <code>type</code> key that is considered the style to use by all exporters</li>
   * <li><p>objects with the <code>type</code> key defined that are exceptions of the above rule</p>
   *
   * <p>   {
   *        xtype: 'numbercolumn',
   *        dataIndex: 'price',
   *        text: 'Price',
   *        exportStyle: [{
   *            // no type defined means this is the default
   *            format: 'Currency',
   *            alignment: {
   *                horizontal: 'Right'
   *            },
   *            font: {
   *                italic: true
   *            }
   *        },{
   *            type: 'csv', // only the CSV exporter has a special style
   *            format: 'General'
   *        }]
   *    }</p></li>
   * </ul>
   */
  public native function get exportStyle():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set exportStyle(value:*):void;

  [Deprecated(replacement = "editor", since = "4.0.5")]
  [ExtConfig]
  /**
   * Alias for <a href="#!/api/Ext.grid.column.Column-cfg-editor" rel="Ext.grid.column.Column-cfg-editor" class="docClass">editor</a>.
   */
  public native function get field():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set field(value:*):void;

  [ExtConfig]
  /**
   * This config accepts a format specification as would be used in a <code><a href="#!/api/Ext.Template" rel="Ext.Template" class="docClass">Ext.Template</a></code>
   * formatted token. For example <code>'round(2)'</code> to round numbers to 2 decimal places
   * or <code>'date("Y-m-d")'</code> to format a Date.
   *
   * <p>In previous releases the <code>renderer</code> config had limited abilities to use one
   * of the <code><a href="#!/api/Ext.util.Format" rel="Ext.util.Format" class="docClass">Ext.util.Format</a></code> methods but <code>formatter</code> now replaces that usage and
   * can also handle formatting parameters.</p>
   *
   * <p>When the value begins with <code>"this."</code> (for example, <code>"this.foo(2)"</code>), the
   * implied scope on which "foo" is found is the <code>scope</code> config for the column.</p>
   *
   * <p>If the <code>scope</code> is not given, or implied using a prefix of <code>"this"</code>, then either the
   * <a href="#!/api/Ext.grid.column.Column-method-getController" rel="Ext.grid.column.Column-method-getController" class="docClass">ViewController</a> or the closest ancestor component configured
   * as <a href="#!/api/Ext.grid.column.Column-cfg-defaultListenerScope" rel="Ext.grid.column.Column-cfg-defaultListenerScope" class="docClass">defaultListenerScope</a> is assumed to be the object with the method.</p>
   * @since 5.0.0
   */
  public native function get formatter():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set formatter(value:String):void;

  [ExtConfig]
  /**
   * If the grid uses a <a href="#!/api/Ext.grid.feature.Grouping" rel="Ext.grid.feature.Grouping" class="docClass">Ext.grid.feature.Grouping</a>, this option may be used to disable the header menu
   * item to group by the column selected. By default, the header menu group option is enabled. Set to false to
   * disable (but still show) the group option in the header menu for the column.
   */
  public native function get groupable():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set groupable(value:Boolean):void;

  [Deprecated(replacement = "text", since = "4.0")]
  [ExtConfig]
  /**
   * The header text.
   */
  public native function get header():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set header(value:String):void;

  [ExtConfig]
  /**
   * False to prevent the user from hiding this column.
   * @default true
   */
  public native function get hideable():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set hideable(value:Boolean):void;

  [ExtConfig]
  /**
   * This flag indicates that this column will be ignored when grid data is exported.
   *
   * <p>When grid data is exported you may want to export only some columns that are important
   * and not everything. Widget, check and action columns are not relevant when data is
   * exported. You can set this flag on any column that you want to be ignored during export.</p>
   *
   * <p>This is used by <a href="#!/api/Ext.grid.plugin.Clipboard" rel="Ext.grid.plugin.Clipboard" class="docClass">clipboard plugin</a> and exporter plugin.</p>
   * @default false
   */
  public native function get ignoreExport():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set ignoreExport(value:Boolean):void;

  [ExtConfig]
  /**
   * If the grid is configured with <a href="#!/api/Ext.panel.Table-cfg-enableLocking" rel="Ext.panel.Table-cfg-enableLocking" class="docClass">enableLocking</a>, or has columns which are
   * configured with a <a href="#!/api/Ext.grid.column.Column-cfg-locked" rel="Ext.grid.column.Column-cfg-locked" class="docClass">locked</a> value, this option may be used to disable user-driven locking or unlocking
   * of this column. This column will remain in the side into which its own <a href="#!/api/Ext.grid.column.Column-cfg-locked" rel="Ext.grid.column.Column-cfg-locked" class="docClass">locked</a> configuration placed it.
   */
  public native function get lockable():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set lockable(value:Boolean):void;

  [ExtConfig]
  /**
   * True to lock this column in place.  Implicitly enables locking on the grid.
   * See also <a href="#!/api/Ext.grid.Panel-cfg-enableLocking" rel="Ext.grid.Panel-cfg-enableLocking" class="docClass">Ext.grid.Panel.enableLocking</a>.
   * @default false
   */
  public native function get locked():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set locked(value:Boolean):void;

  [ExtConfig]
  /**
   * True to disable the column header menu containing sort/hide options.
   * @default false
   */
  public native function get menuDisabled():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set menuDisabled(value:Boolean):void;

  [ExtConfig]
  /**
   * The text to render in the column visibility selection menu for this column.  If not
   * specified, will default to the text value.
   */
  public native function get menuText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set menuText(value:String):void;

  [ExtConfig]
  /**
   * This flag indicates that the renderer produces HTML.
   *
   * <p>If this column is going to be updated rapidly, and the
   * <a href="#!/api/Ext.grid.column.Column-cfg-renderer" rel="Ext.grid.column.Column-cfg-renderer" class="docClass">renderer</a> or <a href="#!/api/Ext.grid.column.Column-cfg-updater" rel="Ext.grid.column.Column-cfg-updater" class="docClass">updater</a> only produces
   * text, then to avoid the expense of HTML parsing and element production during the
   * update, this property may be configured as <code>false</code>.</p>
   * @default true
   */
  public native function get producesHTML():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set producesHTML(value:Boolean):void;

  [ExtConfig]
  /**
   * A renderer is an 'interceptor' method which can be used to transform data (value,
   * appearance, etc.) before it is rendered. Example:
   *
   * <p><strong>NOTE:</strong> In previous releases, a string was treated as a method on
   * <code><a href="#!/api/Ext.util.Format" rel="Ext.util.Format" class="docClass">Ext.util.Format</a></code> but that is now handled by the <a href="#!/api/Ext.grid.column.Column-cfg-formatter" rel="Ext.grid.column.Column-cfg-formatter" class="docClass">formatter</a> config.</p>
   * @default false
   */
  public native function get renderer():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set renderer(value:*):void;

  [ExtConfig]
  /**
   * The scope to use when calling the
   * <a href="#!/api/Ext.grid.column.Column-cfg-renderer" rel="Ext.grid.column.Column-cfg-renderer" class="docClass">renderer</a> function.
   */
  public native function get scope():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set scope(value:Object):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * A Sorter, or sorter config object to apply when the standard user interface
   * sort gesture is invoked. This is usually clicking this column header, but
   * there are also menu options to sort ascending or descending.
   *
   * <p>Note that a sorter may also be specified as a function which accepts two
   * records to compare.</p>
   *
   * <p>In 6.2.0, a <code><a href="#!/api/Ext.app.ViewController" rel="Ext.app.ViewController" class="docClass">controller</a></code> method can be used
   * like so:</p>
   *
   * <pre><code> sorter: {
   *      sorterFn: 'sorterMethodName'
   *  }
   * </code></pre>
   * @since 6.0.1
   * @see #getSorter()
   * @see #setSorter()
   */
  public native function get sorter():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set sorter(value:*):void;

  [ExtConfig]
  /**
   * A renderer to be used in conjunction with the <a href="#!/api/Ext.grid.feature.Summary" rel="Ext.grid.feature.Summary" class="docClass">Summary</a> or
   * <a href="#!/api/Ext.grid.feature.GroupingSummary" rel="Ext.grid.feature.GroupingSummary" class="docClass">GroupingSummary</a> features. This renderer is used to
   * display a summary value for this column.
   * &#64;controllable
   */
  public native function get summaryRenderer():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set summaryRenderer(value:*):void;

  [ExtConfig]
  /**
   * A CSS class names to apply to the table cells for this column.
   * @default ''
   */
  public native function get tdCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set tdCls(value:String):void;

  [ExtConfig]
  /**
   * The header text to be used as innerHTML (html tags are accepted) to display in the Grid.
   * <strong>Note</strong>: to have a clickable header with no text displayed you can use the default of <code>&amp;#160;</code> aka <code>&amp;nbsp;</code>.
   * @default '&amp;#160;'
   * @see #setText()
   */
  public native function get text():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set text(value:String):void;

  [ExtConfig]
  /**
   * A tooltip to display for this column header
   */
  public native function get tooltip():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set tooltip(value:String):void;

  [ExtConfig]
  /**
   * The type of <a href="#!/api/Ext.grid.column.Column-cfg-tooltip" rel="Ext.grid.column.Column-cfg-tooltip" class="docClass">tooltip</a> to use. Either 'qtip' for QuickTips or 'title' for title attribute.
   * @default "qtip"
   */
  public native function get tooltipType():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set tooltipType(value:String):void;

  [ExtConfig]
  /**
   * An updater is a method which is used when records are updated, and an <em>existing</em> grid row needs updating.
   * The method is passed the cell element and may manipulate it in any way.
   *
   * <p><strong>Note</strong>: The updater is required to insert the <a href="#!/api/Ext.grid.column.Column-cfg-emptyCellText" rel="Ext.grid.column.Column-cfg-emptyCellText" class="docClass">emptyCellText</a> if there
   * is no value in the cell.</p>
   *
   * <pre><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>', {
   *     title: 'Grades',
   *     store: {
   *         fields: ['originalScore', 'newScore'],
   *         data: [{
   *             originalScore: 70,
   *             newScore: 70
   *         }]
   *     },
   *     columns: [{
   *         text: 'Score',
   *         dataIndex: 'newScore',
   *         editor: 'numberfield',
   *         flex: 1,
   *         updater: function (cell, value, record, view) {
   *             var inner = <a href="#!/api/Ext-method-get" rel="Ext-method-get" class="docClass">Ext.get</a>(cell).first(),
   *                 originalScore = record.get('originalScore'),
   *                 color = (value === originalScore) ? 'black' : (value &gt; originalScore) ? 'green' : 'red';
   *
   *             // set the color based on the current value relative to the originalScore value
   *             // * same   = black
   *             // * higher = green
   *             // * less   = red
   *             inner.applyStyles({
   *                 color: color
   *             });
   *             // pass the value to the cell's inner el
   *             inner.setHtml(value);
   *         }
   *     }],
   *     height: 200,
   *     width: 400,
   *     renderTo: document.body,
   *     tbar: [{
   *         xtype: 'numberfield',
   *         fieldLabel: 'New Score',
   *         value: 70,
   *         listeners: {
   *             change: function (field, newValue) {
   *                 this.up('grid').getStore().first().set('newScore', newValue);
   *             }
   *         }
   *     }]
   * });
   * </code></pre>
   *
   * <p>If a string is passed it is assumed to be the name of a method defined by the
   * <a href="#!/api/Ext.grid.column.Column-method-getController" rel="Ext.grid.column.Column-method-getController" class="docClass">ViewController</a> or an ancestor component configured as <a href="#!/api/Ext.grid.column.Column-cfg-defaultListenerScope" rel="Ext.grid.column.Column-cfg-defaultListenerScope" class="docClass">defaultListenerScope</a>.</p>
   * <ul>
   *   <li><code>cell</code> : js.HTMLElement
   * The HTML cell element to update.
   *   </li>
   *   <li><code>value</code> : Object
   * The data value for the current cell
   *   </li>
   *   <li><code>record</code> : ext.data.Model
   * The record for the current row
   *   </li>
   *   <li><code>view</code> : ext.view.DataView
   * The current view
   * <p><strong>Note</strong>: The updater is required to insert the <a href="#!/api/Ext.grid.column.Column-cfg-emptyCellText" rel="Ext.grid.column.Column-cfg-emptyCellText" class="docClass">emptyCellText</a> if there is no value in the cell.</p>
   * <p>&#64;controllable</p>
   *   </li>
   * </ul>
   */
  public native function get updater():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set updater(value:*):void;

  [ExtConfig]
  /**
   * True to indicate that data in this column may take on an unpredictable height, possibly differing from row to row.
   *
   * <p>If this is set, then View refreshes, and removal and addition of new rows will result in an ExtJS layout of the grid
   * in order to adjust for possible addition/removal of scrollbars in the case of data changing height.</p>
   *
   * <p>This config also tells the View's buffered renderer that row heights are unpredictable, and must be remeasured as the view is refreshed.</p>
   * @default false
   */
  public native function get variableRowHeight():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set variableRowHeight(value:Boolean):void;

  /**
   * Sizes this Column to fit the max content width.
   * <em>Note that group columns shrink-wrap around the size of leaf columns. Auto sizing
   * a group column auto-sizes descendant leaf columns.</em>
   */
  public native function autoSize():void;

  /**
   * Returns the value of <a href="#!/api/Ext.grid.column.Column-cfg-align" rel="Ext.grid.column.Column-cfg-align" class="docClass">align</a>.
   */
  public native function getAlign():String;

  /**
   * Retrieves the editing field for editing associated with this header.  If the
   * field has not been instantiated it will be created.
   *
   * <p><strong>Note:</strong> This method will only have an implementation if an Editing plugin has
   * been enabled on the grid (<a href="#!/api/Ext.grid.plugin.CellEditing" rel="Ext.grid.plugin.CellEditing" class="docClass">cellediting</a> /
   * <a href="#!/api/Ext.grid.plugin.RowEditing" rel="Ext.grid.plugin.RowEditing" class="docClass">rowediting</a>).</p>
   * @param record The <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Model</a> instance being edited.
   * @param defaultField An xtype or config object for a
   * <a href="#!/api/Ext.form.field.Field" rel="Ext.form.field.Field" class="docClass">Field</a> to be created as the default editor if it does
   * not already exist
   * @return The editor field associated with
   * this column.  Returns false if there is no field associated with the
   * <a href="#!/api/Ext.grid.column.Column" rel="Ext.grid.column.Column" class="docClass">Column</a>.
   */
  public native function getEditor(record:Object = null, defaultField:* = undefined):*;

  /**
   * Returns the index of this column only if this column is a base level Column. If it
   * is a group column, it returns <code>false</code>.
   */
  public native function getIndex():Number;

  /**
   * Returns the parameter to sort upon when sorting this header. By default this returns the dataIndex and will not
   * need to be overridden in most cases.
   */
  public native function getSortParam():String;

  /**
   * Returns the value of <a href="#!/api/Ext.grid.column.Column-cfg-sorter" rel="Ext.grid.column.Column-cfg-sorter" class="docClass">sorter</a>.
   * @since 6.0.1
   */
  public native function getSorter():*;

  /**
   * Returns the index of this column in the list of <em>visible</em> columns only if this column is a base level Column. If it
   * is a group column, it returns <code>false</code>.
   */
  public native function getVisibleIndex():Number;

  /**
   * Determines whether the UI should be allowed to offer an option to hide this column.
   *
   * <p>A column may <em>not</em> be hidden if to do so would leave the grid with no visible columns.</p>
   *
   * <p>This is used to determine the enabled/disabled state of header hide menu items.</p>
   */
  public native function isHideable():void;

  /**
   * Determines whether the UI should be allowed to offer an option to lock or unlock this column. Note
   * that this includes dragging a column into the opposite side of a <a href="#!/api/Ext.panel.Table-cfg-enableLocking" rel="Ext.panel.Table-cfg-enableLocking" class="docClass">lockable</a> grid.
   *
   * <p>A column may <em>not</em> be moved from one side to the other of a <a href="#!/api/Ext.panel.Table-cfg-enableLocking" rel="Ext.panel.Table-cfg-enableLocking" class="docClass">lockable</a> grid
   * if to do so would leave one side with no visible columns.</p>
   *
   * <p>This is used to determine the enabled/disabled state of the lock/unlock
   * menu item used in <a href="#!/api/Ext.panel.Table-cfg-enableLocking" rel="Ext.panel.Table-cfg-enableLocking" class="docClass">lockable</a> grids, and to determine droppabilty when dragging a header.</p>
   */
  public native function isLockable():void;

  /**
   * Determines whether this column is in the locked side of a grid. It may be a descendant node of a locked column
   * and as such will <em>not</em> have the <a href="#!/api/Ext.grid.column.Column-cfg-locked" rel="Ext.grid.column.Column-cfg-locked" class="docClass">locked</a> flag set.
   */
  public native function isLocked():void;

  /**
   * Sets the value of <a href="#!/api/Ext.grid.column.Column-cfg-align" rel="Ext.grid.column.Column-cfg-align" class="docClass">align</a>.
   * @param align The new value.
   */
  public native function setAlign(align:String):void;

  /**
   * Sets the form field to be used for editing.
   *
   * <p><strong>Note:</strong> This method will only have an implementation if an Editing plugin has
   * been enabled on the grid (<a href="#!/api/Ext.grid.plugin.CellEditing" rel="Ext.grid.plugin.CellEditing" class="docClass">cellediting</a> /
   * <a href="#!/api/Ext.grid.plugin.RowEditing" rel="Ext.grid.plugin.RowEditing" class="docClass">rowediting</a>).</p>
   * @param field An object representing a field to be created. If no xtype is specified a 'textfield' is
   * assumed.
   */
  public native function setEditor(field:ext.form.field.IField):void;

  /**
   * Sets the value of <a href="#!/api/Ext.grid.column.Column-cfg-sorter" rel="Ext.grid.column.Column-cfg-sorter" class="docClass">sorter</a>.
   * @since 6.0.1
   * @param sorter The new value.
   */
  public native function setSorter(sorter:*):void;

  /**
   * Sets the header text for this Column.
   * @param text The header to display on this Column.
   */
  public native function setText(text:String):void;

  /**
   * When defined this will take precedence over the
   * <a href="#!/api/Ext.grid.column.Column-cfg-renderer" rel="Ext.grid.column.Column-cfg-renderer" class="docClass">renderer</a> config.
   * This is meant to be defined in subclasses that wish to supply their own renderer.
   */
  protected native function defaultRenderer():void;
}
}