package ext.grid.column {

[Uses("ext.grid.column.Column")]
[Uses("ext.grid.header.HeaderContainer")]
[Uses("ext.container.Container")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.Base")]
[Uses("ext.mixin.Container")]
[Uses("ext.Mixin")]
[Uses("ext.Component")]
[Uses("ext.util.Positionable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Animate")]
[Uses("ext.util.Floating")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.state.Stateful")]
[Uses("ext.util.FocusableContainer")]
[Native("Ext.grid.column.Date", require)]
/**
 * A Column definition class which renders a passed date according to the default locale, or a configured
 * <a href="#!/api/Ext.grid.column.Date-cfg-format" rel="Ext.grid.column.Date-cfg-format" class="docClass">format</a>.
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
 *     storeId:'sampleStore',
 *     fields:[
 *         { name: 'symbol', type: 'string' },
 *         { name: 'date',   type: 'date' },
 *         { name: 'change', type: 'number' },
 *         { name: 'volume', type: 'number' },
 *         { name: 'topday', type: 'date' }
 *     ],
 *     data:[
 *         { symbol: "msft",   date: '2011/04/22', change: 2.43, volume: 61606325, topday: '04/01/2010' },
 *         { symbol: "goog",   date: '2011/04/22', change: 0.81, volume: 3053782,  topday: '04/11/2010' },
 *         { symbol: "apple",  date: '2011/04/22', change: 1.35, volume: 24484858, topday: '04/28/2010' },
 *         { symbol: "sencha", date: '2011/04/22', change: 8.85, volume: 5556351,  topday: '04/22/2010' }
 *     ]
 * });
 *
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>', {
 *     title: 'Date Column Demo',
 *     store: <a href="#!/api/Ext.data.StoreManager-method-lookup" rel="Ext.data.StoreManager-method-lookup" class="docClass">Ext.data.StoreManager.lookup</a>('sampleStore'),
 *     columns: [
 *         { text: 'Symbol',   dataIndex: 'symbol', flex: 1 },
 *         { text: 'Date',     dataIndex: 'date',   xtype: 'datecolumn',   format:'Y-m-d' },
 *         { text: 'Change',   dataIndex: 'change', xtype: 'numbercolumn', format:'0.00' },
 *         { text: 'Volume',   dataIndex: 'volume', xtype: 'numbercolumn', format:'0,000' },
 *         { text: 'Top Day',  dataIndex: 'topday', xtype: 'datecolumn',   format:'l' }
 *     ],
 *     height: 200,
 *     width: 450,
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 * });
 * </code></pre>
 */
public class DateColumn extends ext.grid.column.Column {
  /**
   * @param config @inheritDoc
   */
  public function DateColumn(config:ext.grid.column.DateColumn = null) {
    super();
  }

  [ExtConfig]
  /**
   * This flag indicates that the renderer produces HTML.
   *
   * <p>If this column is going to be updated rapidly, and the
   * <a href="#!/api/Ext.grid.column.Column-cfg-renderer" rel="Ext.grid.column.Column-cfg-renderer" class="docClass">Ext.grid.column.Column.renderer</a> or <a href="#!/api/Ext.grid.column.Date-cfg-updater" rel="Ext.grid.column.Date-cfg-updater" class="docClass">updater</a> only produces
   * text, then to avoid the expense of HTML parsing and element production during the
   * update, this property may be configured as <code>false</code>.</p>
   * @default false
   */
  override public native function get producesHTML():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set producesHTML(value:Boolean):void;

  [ExtConfig]
  /**
   * A formatting string as used by <a href="#!/api/Ext.Date-method-format" rel="Ext.Date-method-format" class="docClass">Ext.Date.format</a> to format a Date for this Column.
   *
   * <p>Defaults to the default date from <a href="#!/api/Ext.Date-property-defaultFormat" rel="Ext.Date-property-defaultFormat" class="docClass">Ext.Date.defaultFormat</a> which itself my be overridden
   * in a locale file.</p>
   */
  public native function get format():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set format(value:String):void;
}
}