package ext.grid.column {

[Uses("ext.grid.column.Column")]
[Uses("ext.grid.header.HeaderContainer")]
[Uses("ext.container.Container")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.Base")]
[Uses("ext.mixin.Container")]
[Uses("ext.Mixin")]
[Uses("ext.Component")]
[Uses("ext.util.Positionable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Animate")]
[Uses("ext.util.Floating")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.state.Stateful")]
[Uses("ext.util.FocusableContainer")]
[Native("Ext.grid.column.Number", require)]
/**
 * A Column definition class which renders a numeric data field according to a <a href="#!/api/Ext.grid.column.Number-cfg-format" rel="Ext.grid.column.Number-cfg-format" class="docClass">format</a> string.
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
 *    storeId:'sampleStore',
 *    fields:[
 *        { name: 'symbol', type: 'string' },
 *        { name: 'price',  type: 'number' },
 *        { name: 'change', type: 'number' },
 *        { name: 'volume', type: 'number' }
 *    ],
 *    data:[
 *        { symbol: "msft",   price: 25.76,  change: 2.43, volume: 61606325 },
 *        { symbol: "goog",   price: 525.73, change: 0.81, volume: 3053782  },
 *        { symbol: "apple",  price: 342.41, change: 1.35, volume: 24484858 },
 *        { symbol: "sencha", price: 142.08, change: 8.85, volume: 5556351  }
 *    ]
 * });
 *
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>', {
 *     title: 'Number Column Demo',
 *     store: <a href="#!/api/Ext.data.StoreManager-method-lookup" rel="Ext.data.StoreManager-method-lookup" class="docClass">Ext.data.StoreManager.lookup</a>('sampleStore'),
 *     columns: [
 *         { text: 'Symbol',         dataIndex: 'symbol', flex: 1 },
 *         { text: 'Current Price',  dataIndex: 'price',  renderer: <a href="#!/api/Ext.util.Format-method-usMoney" rel="Ext.util.Format-method-usMoney" class="docClass">Ext.util.Format.usMoney</a> },
 *         { text: 'Change',         dataIndex: 'change', xtype: 'numbercolumn', format:'0.00' },
 *         { text: 'Volume',         dataIndex: 'volume', xtype: 'numbercolumn', format:'0,000' }
 *     ],
 *     height: 200,
 *     width: 400,
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 * });
 * </code></pre>
 */
public class NumberColumn extends ext.grid.column.Column {
  /**
   * @param config @inheritDoc
   */
  public function NumberColumn(config:ext.grid.column.NumberColumn = null) {
    super();
  }

  [ExtConfig]
  /**
   * This flag indicates that the renderer produces HTML.
   *
   * <p>If this column is going to be updated rapidly, and the
   * <a href="#!/api/Ext.grid.column.Column-cfg-renderer" rel="Ext.grid.column.Column-cfg-renderer" class="docClass">Ext.grid.column.Column.renderer</a> or <a href="#!/api/Ext.grid.column.Number-cfg-updater" rel="Ext.grid.column.Number-cfg-updater" class="docClass">updater</a> only produces
   * text, then to avoid the expense of HTML parsing and element production during the
   * update, this property may be configured as <code>false</code>.</p>
   * @default false
   */
  override public native function get producesHTML():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set producesHTML(value:Boolean):void;

  [ExtConfig]
  /**
   * A formatting string as used by <a href="#!/api/Ext.util.Format-method-number" rel="Ext.util.Format-method-number" class="docClass">Ext.util.Format.number</a> to format a numeric value for this Column.
   * @default '0,000.00'
   */
  public native function get format():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set format(value:String):void;
}
}