package ext.grid.column {

[Uses("ext.grid.column.Column")]
[Uses("ext.grid.header.HeaderContainer")]
[Uses("ext.container.Container")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.Base")]
[Uses("ext.mixin.Container")]
[Uses("ext.Mixin")]
[Uses("ext.Component")]
[Uses("ext.util.Positionable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Animate")]
[Uses("ext.util.Floating")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.state.Stateful")]
[Uses("ext.util.FocusableContainer")]
[Native("Ext.grid.column.RowNumberer", require)]
/**
 * A special type of Grid <a href="#!/api/Ext.grid.column.Column" rel="Ext.grid.column.Column" class="docClass">Ext.grid.column.Column</a> that provides automatic
 * row numbering.
 *
 * <p>Usage:</p>
 *
 * <pre><code>columns: [
 *     {xtype: 'rownumberer'},
 *     {text: "Company", flex: 1, sortable: true, dataIndex: 'company'},
 *     {text: "Price", width: 120, sortable: true, renderer: <a href="#!/api/Ext.util.Format-method-usMoney" rel="Ext.util.Format-method-usMoney" class="docClass">Ext.util.Format.usMoney</a>, dataIndex: 'price'},
 *     {text: "Change", width: 120, sortable: true, dataIndex: 'change'},
 *     {text: "% Change", width: 120, sortable: true, dataIndex: 'pctChange'},
 *     {text: "Last Updated", width: 120, sortable: true, renderer: <a href="#!/api/Ext.util.Format-method-dateRenderer" rel="Ext.util.Format-method-dateRenderer" class="docClass">Ext.util.Format.dateRenderer</a>('m/d/Y'), dataIndex: 'lastChange'}
 * ]
 * </code></pre>
 */
public class RowNumberer extends ext.grid.column.Column {
  /**
   * @param config @inheritDoc
   */
  public function RowNumberer(config:ext.grid.column.RowNumberer = null) {
    super();
  }

  [ExtConfig]
  /**
   * This flag indicates that the renderer produces HTML.
   *
   * <p>If this column is going to be updated rapidly, and the
   * <a href="#!/api/Ext.grid.column.Column-cfg-renderer" rel="Ext.grid.column.Column-cfg-renderer" class="docClass">Ext.grid.column.Column.renderer</a> or <a href="#!/api/Ext.grid.column.RowNumberer-cfg-updater" rel="Ext.grid.column.RowNumberer-cfg-updater" class="docClass">updater</a> only produces
   * text, then to avoid the expense of HTML parsing and element production during the
   * update, this property may be configured as <code>false</code>.</p>
   * @default false
   */
  override public native function get producesHTML():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set producesHTML(value:Boolean):void;

  [ExtConfig]
  /**
   * Any valid text or HTML fragment to display in the header cell for the row number column.
   * @default "&amp;#160;"
   * @see #setText()
   */
  override public native function get text():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  override public native function set text(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The default width in pixels of the row number column.
   * @default 23
   * @see #getWidth()
   * @see #setWidth()
   */
  override public native function get width():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  override public native function set width(value:*):void;

  /**
   * <code>true</code> in this class to identify an object as an instantiated RowNumberer, or subclass thereof.
   * @default true
   */
  public native function get isRowNumberer():Boolean;

  /**
   * @private
   */
  public native function set isRowNumberer(value:Boolean):void;
}
}