package ext.grid.feature {

[Uses("ext.grid.feature.AbstractSummary")]
[Uses("ext.grid.feature.Feature")]
[Uses("ext.util.Observable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Native("Ext.grid.feature.Summary", require)]
/**
 * This feature is used to place a summary row at the bottom of the grid. If using a grouping,
 * see <a href="#!/api/Ext.grid.feature.GroupingSummary" rel="Ext.grid.feature.GroupingSummary" class="docClass">Ext.grid.feature.GroupingSummary</a>. There are 2 aspects to calculating the summaries,
 * calculation and rendering.
 *
 * <h2>Calculation</h2>
 *
 * <p>The summary value needs to be calculated for each column in the grid. This is controlled
 * by the summaryType option specified on the column. There are several built in summary types,
 * which can be specified as a string on the column configuration. These call underlying methods
 * on the store:</p>
 *
 * <ul>
 * <li><a href="#!/api/Ext.data.Store-method-count" rel="Ext.data.Store-method-count" class="docClass">count</a></li>
 * <li><a href="#!/api/Ext.data.Store-method-sum" rel="Ext.data.Store-method-sum" class="docClass">sum</a></li>
 * <li><a href="#!/api/Ext.data.Store-method-min" rel="Ext.data.Store-method-min" class="docClass">min</a></li>
 * <li><a href="#!/api/Ext.data.Store-method-max" rel="Ext.data.Store-method-max" class="docClass">max</a></li>
 * <li><a href="#!/api/Ext.data.Store-method-average" rel="Ext.data.Store-method-average" class="docClass">average</a></li>
 * </ul>
 *
 *
 * <p>Alternatively, the summaryType can be a function definition. If this is the case,
 * the function is called with an array of records to calculate the summary value.</p>
 *
 * <h2>Rendering</h2>
 *
 * <p>Similar to a column, the summary also supports a summaryRenderer function. This
 * summaryRenderer is called before displaying a value. The function is optional, if
 * not specified the default calculated value is shown. The summaryRenderer is called with:</p>
 *
 * <ul>
 * <li>value {Object} - The calculated value.</li>
 * <li>summaryData {Object} - Contains all raw summary values for the row.</li>
 * <li>field {String} - The name of the field we are calculating</li>
 * <li>metaData {Object} - A collection of metadata about the current cell; can be used or modified by the renderer.</li>
 * </ul>
 *
 *
 * <h2>Example Usage</h2>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('TestResult', {
 *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
 *     fields: ['student', {
 *         name: 'mark',
 *         type: 'int'
 *     }]
 * });
 *
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>', {
 *     width: 400,
 *     height: 200,
 *     title: 'Summary Test',
 *     style: 'padding: 20px',
 *     renderTo: document.body,
 *     features: [{
 *         ftype: 'summary'
 *     }],
 *     store: {
 *         model: 'TestResult',
 *         data: [{
 *             student: 'Student 1',
 *             mark: 84
 *         },{
 *             student: 'Student 2',
 *             mark: 72
 *         },{
 *             student: 'Student 3',
 *             mark: 96
 *         },{
 *             student: 'Student 4',
 *             mark: 68
 *         }]
 *     },
 *     columns: [{
 *         dataIndex: 'student',
 *         text: 'Name',
 *         summaryType: 'count',
 *         summaryRenderer: function(value, summaryData, dataIndex) {
 *             return <a href="#!/api/Ext.String-method-format" rel="Ext.String-method-format" class="docClass">Ext.String.format</a>('{0} student{1}', value, value !== 1 ? 's' : '');
 *         }
 *     }, {
 *         dataIndex: 'mark',
 *         text: 'Mark',
 *         summaryType: 'average'
 *     }]
 * });
 * </code></pre>
 */
public class Summary extends ext.grid.feature.AbstractSummary {
  /**
   * @param config @inheritDoc
   */
  public function Summary(config:ext.grid.feature.Summary = null) {
    super();
  }

  [ExtConfig]
  /**
   * Configure <code>'top'</code> or <code>'bottom'</code> top create a fixed summary row either above or below the scrollable table.
   */
  public native function get dock():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set dock(value:String):void;
}
}