package ext.grid.filters.filter {
import ext.Base;
import ext.menu.Menu;
import ext.mixin.IFactoryable;

[Uses("ext.Base")]
[Uses("ext.mixin.Factoryable")]
[Native("Ext.grid.filters.filter.Base", require)]
/**
 * Fires when an inactive filter becomes active
 * @eventType ext.grid.filters.filter.events.BaseFilterEvent.ACTIVATE
 */
[Event(name = 'onActivate', type = 'ext.grid.filters.filter.events.BaseFilterEvent')]
/**
 * Fires when an active filter becomes inactive
 * @eventType ext.grid.filters.filter.events.BaseFilterEvent.DEACTIVATE
 */
[Event(name = 'onDeactivate', type = 'ext.grid.filters.filter.events.BaseFilterEvent')]
/**
 * Fires when a filter configuration has changed
 * @eventType ext.grid.filters.filter.events.BaseFilterEvent.UPDATE
 */
[Event(name = 'onUpdate', type = 'ext.grid.filters.filter.events.BaseFilterEvent')]
/**
 * Abstract base class for filter implementations.
 */
public class BaseFilter extends ext.Base implements ext.mixin.IFactoryable {
  /**
   * Initializes the filter given its configuration.
   */
  public function BaseFilter(config:ext.grid.filters.filter.BaseFilter = null) {
    super();
  }

  /**
   * The filter configuration menu that will be installed into the filter submenu of a column menu.
   */
  public native function get menu():ext.menu.Menu;

  /**
   * @private
   */
  public native function set menu(value:ext.menu.Menu):void;

  /**
   * The filter type. Used by the filters.Feature class when adding filters and applying state.
   * @default 'string'
   */
  public native function get type():String;

  /**
   * @private
   */
  public native function set type(value:String):void;

  [ExtConfig]
  /**
   * True if this filter is active. Use setActive() to alter after configuration. If
   * you set a value, the filter will be actived automatically.
   */
  public native function get active():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set active(value:Boolean):void;

  [ExtConfig]
  /**
   * The <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a> dataIndex of the field this filter represents.
   * The dataIndex does not actually have to exist in the store.
   */
  public native function get dataIndex():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set dataIndex(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The default configuration options for any menu items created by this filter.
   *
   * <p>Example usage:</p>
   *
   * <pre><code> itemDefaults: {
   *      width: 150
   *  },
   * </code></pre>
   * @see #getItemDefaults()
   * @see #setItemDefaults()
   */
  public native function get itemDefaults():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set itemDefaults(value:Object):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * A function to post-process any serialization. Accepts a filter state object
   * containing <code>property</code>, <code>value</code> and <code>operator</code> properties, and may either
   * mutate it, or return a completely new representation.
   * @since 6.2.0
   * @see #getSerializer()
   * @see #setSerializer()
   */
  public native function get serializer():Function;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set serializer(value:Function):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Number of milliseconds to wait after user interaction to fire an update. Only supported
   * by filters: 'list', 'numeric', and 'string'.
   * @default 500
   * @see #getUpdateBuffer()
   * @see #setUpdateBuffer()
   */
  public native function get updateBuffer():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set updateBuffer(value:Number):void;

  /**
   * @inheritDoc
   */
  public native function get factoryConfig():Object;

  /**
   * @inheritDoc
   */
  public native function set factoryConfig(value:Object):void;

  /**
   * Destroys this filter by purging any event listeners, and removing any menus.
   */
  override public native function destroy(...params):void;

  /**
   * Returns the value of <a href="#!/api/Ext.grid.filters.filter.Base-cfg-itemDefaults" rel="Ext.grid.filters.filter.Base-cfg-itemDefaults" class="docClass">itemDefaults</a>.
   */
  public native function getItemDefaults():Object;

  /**
   * Returns the value of <a href="#!/api/Ext.grid.filters.filter.Base-cfg-serializer" rel="Ext.grid.filters.filter.Base-cfg-serializer" class="docClass">serializer</a>.
   * @since 6.2.0
   */
  public native function getSerializer():Function;

  /**
   * Returns the value of <a href="#!/api/Ext.grid.filters.filter.Base-cfg-updateBuffer" rel="Ext.grid.filters.filter.Base-cfg-updateBuffer" class="docClass">updateBuffer</a>.
   */
  public native function getUpdateBuffer():Number;

  /**
   * Sets the status of the filter and fires the appropriate events.
   * @param active The new filter state.
   * @param key The filter key for columns that support multiple filters.
   */
  public native function setActive(active:Boolean, key:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.grid.filters.filter.Base-cfg-itemDefaults" rel="Ext.grid.filters.filter.Base-cfg-itemDefaults" class="docClass">itemDefaults</a>.
   * @param itemDefaults The new value.
   */
  public native function setItemDefaults(itemDefaults:Object):void;

  /**
   * Sets the value of <a href="#!/api/Ext.grid.filters.filter.Base-cfg-serializer" rel="Ext.grid.filters.filter.Base-cfg-serializer" class="docClass">serializer</a>.
   * @since 6.2.0
   * @param serializer The new value.
   */
  public native function setSerializer(serializer:Function):void;

  /**
   * Sets the value of <a href="#!/api/Ext.grid.filters.filter.Base-cfg-updateBuffer" rel="Ext.grid.filters.filter.Base-cfg-updateBuffer" class="docClass">updateBuffer</a>.
   * @param updateBuffer The new value.
   */
  public native function setUpdateBuffer(updateBuffer:Number):void;
}
}