package ext.grid.filters.filter {
import ext.picker.DatePicker;

[Uses("ext.grid.filters.filter.TriFilter")]
[Uses("ext.grid.filters.filter.BaseFilter")]
[Uses("ext.mixin.Factoryable")]
[Uses("ext.Base")]
[Native("Ext.grid.filters.filter.Date", require)]
/**
 * The date grid filter allows you to create a filter selection that limits results
 * to values matching specific date constraints.  The filter can be set programmatically or via
 * user input with a configurable <a href="#!/api/Ext.picker.Date" rel="Ext.picker.Date" class="docClass">DatePicker menu</a> in the filter section
 * of the column header.
 *
 * <p>Example Date Filter Usage:</p>
 *
 * <pre class='inline-example '><code>var shows = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
 *     fields: ['id','show', {
 *           name: 'airDate',
 *           type: 'date',
 *           dateFormat: 'Y-m-d'
 *     }],
 *     data: [
 *         {id: 0, show: 'Battlestar Galactica', airDate: '1978-09-17'},
 *         {id: 1, show: 'Doctor Who', airDate: '1963-11-23'},
 *         {id: 2, show: 'Farscape', airDate: '1999-03-19'},
 *         {id: 3, show: 'Firefly', airDate: '2002-12-20'},
 *         {id: 4, show: 'Star Trek', airDate: '1966-09-08'},
 *         {id: 5, show: 'Star Wars: Christmas Special', airDate: '1978-11-17'}
 *     ]
 * });
 *
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>', {
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     title: 'Sci-Fi Television',
 *     height: 250,
 *     width: 375,
 *     store: shows,
 *     plugins: 'gridfilters',
 *     columns: [{
 *         dataIndex: 'id',
 *         text: 'ID',
 *         width: 50
 *     },{
 *         dataIndex: 'show',
 *         text: 'Show',
 *         flex: 1
 *     },{
 *         xtype: 'datecolumn',
 *         dataIndex: 'airDate',
 *         text: 'Original Air Date',
 *         width: 125,
 *         filter: {
 *             type: 'date',
 *
 *             // optional picker config
 *             pickerDefaults: {
 *                 // any DatePicker configs
 *             }
 *         }
 *     }]
 * });
 * </code></pre>
 */
public class DateFilter extends ext.grid.filters.filter.TriFilter {
  /**
   * @param config @inheritDoc
   */
  public function DateFilter(config:ext.grid.filters.filter.DateFilter = null) {
    super();
  }

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The date format to return when using getValue.
   * Defaults to <a href="#!/api/Ext.Date-property-defaultFormat" rel="Ext.Date-property-defaultFormat" class="docClass">Ext.Date.defaultFormat</a>.
   * @see #getDateFormat()
   * @see #setDateFormat()
   */
  public native function get dateFormat():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set dateFormat(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Configures field items individually. These properties override those defined
   * by <code><a href="#!/api/Ext.grid.filters.filter.Date-cfg-itemDefaults" rel="Ext.grid.filters.filter.Date-cfg-itemDefaults" class="docClass">itemDefaults</a></code>.
   *
   * <p>Example usage:
   *      fields: {
   *          gt: { // override fieldCfg options
   *              width: 200
   *          }
   *      },</p>
   * @default {lt: {text: 'Before'}, gt: {text: 'After'}, eq: {text: 'On'}}
   * @see #getFields()
   * @see #setFields()
   */
  public native function get fields():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set fields(value:Object):void;

  [ExtConfig]
  /**
   * Allowable date as passed to the <a href="#!/api/Ext.picker.Date" rel="Ext.picker.Date" class="docClass">Ext.DatePicker</a>
   * Defaults to undefined.
   */
  public native function get maxDate():Date;

  [ExtConfig]
  /**
   * @private
   */
  public native function set maxDate(value:Date):void;

  [ExtConfig]
  /**
   * Allowable date as passed to the <a href="#!/api/Ext.picker.Date" rel="Ext.picker.Date" class="docClass">Ext.DatePicker</a>
   * Defaults to undefined.
   */
  public native function get minDate():Date;

  [ExtConfig]
  /**
   * @private
   */
  public native function set minDate(value:Date):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Configuration options for the date picker associated with each field.
   * @default {xtype: 'datepicker', border: 0}
   * @see #getPickerDefaults()
   * @see #setPickerDefaults()
   */
  public native function get pickerDefaults():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set pickerDefaults(value:Object):void;

  /**
   * Returns the value of <a href="#!/api/Ext.grid.filters.filter.Date-cfg-dateFormat" rel="Ext.grid.filters.filter.Date-cfg-dateFormat" class="docClass">dateFormat</a>.
   */
  public native function getDateFormat():String;

  /**
   * Returns the value of <a href="#!/api/Ext.grid.filters.filter.Date-cfg-fields" rel="Ext.grid.filters.filter.Date-cfg-fields" class="docClass">fields</a>.
   */
  public native function getFields():Object;

  /**
   * Gets the menu picker associated with the passed field
   * @param item The field identifier ('lt', 'gt', 'eq')
   * @return The menu picker
   */
  public native function getPicker(item:String):Object;

  /**
   * Returns the value of <a href="#!/api/Ext.grid.filters.filter.Date-cfg-pickerDefaults" rel="Ext.grid.filters.filter.Date-cfg-pickerDefaults" class="docClass">pickerDefaults</a>.
   */
  public native function getPickerDefaults():Object;

  /**
   * Handler for when the DatePicker for a field fires the 'select' event
   */
  public native function onMenuSelect(picker:ext.picker.DatePicker, date:Object):void;

  /**
   * Sets the value of <a href="#!/api/Ext.grid.filters.filter.Date-cfg-dateFormat" rel="Ext.grid.filters.filter.Date-cfg-dateFormat" class="docClass">dateFormat</a>.
   * @param dateFormat The new value.
   */
  public native function setDateFormat(dateFormat:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.grid.filters.filter.Date-cfg-fields" rel="Ext.grid.filters.filter.Date-cfg-fields" class="docClass">fields</a>.
   * @param fields The new value.
   */
  public native function setFields(fields:Object):void;

  /**
   * Sets the value of <a href="#!/api/Ext.grid.filters.filter.Date-cfg-pickerDefaults" rel="Ext.grid.filters.filter.Date-cfg-pickerDefaults" class="docClass">pickerDefaults</a>.
   * @param pickerDefaults The new value.
   */
  public native function setPickerDefaults(pickerDefaults:Object):void;
}
}