package ext.grid.filters.filter {

[Uses("ext.grid.filters.filter.TriFilter")]
[Uses("ext.grid.filters.filter.BaseFilter")]
[Uses("ext.mixin.Factoryable")]
[Uses("ext.Base")]
[Native("Ext.grid.filters.filter.Number", require)]
/**
 * Filter type for <a href="#!/api/Ext.grid.column.Number" rel="Ext.grid.column.Number" class="docClass">number columns</a>.
 *
 * <pre class='inline-example '><code>var shows = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
 *       fields: ['id','show'],
 *       data: [
 *           {id: 0, show: 'Battlestar Galactica'},
 *           {id: 1, show: 'Doctor Who'},
 *           {id: 2, show: 'Farscape'},
 *           {id: 3, show: 'Firefly'},
 *           {id: 4, show: 'Star Trek'},
 *           {id: 5, show: 'Star Wars: Christmas Special'}
 *       ]
 *    });
 *
 *   <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>', {
 *       renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *       title: 'Sci-Fi Television',
 *       height: 250,
 *       width: 250,
 *       store: shows,
 *       plugins: 'gridfilters',
 *       columns: [{
 *           dataIndex: 'id',
 *           text: 'ID',
 *           width: 50,
 *           filter: 'number' // May also be 'numeric'
 *       },{
 *           dataIndex: 'show',
 *           text: 'Show',
 *           flex: 1
 *       }]
 *   });
 * </code></pre>
 */
public class NumberFilter extends ext.grid.filters.filter.TriFilter {
  /**
   * @param config @inheritDoc
   */
  public function NumberFilter(config:ext.grid.filters.filter.NumberFilter = null) {
    super();
  }

  [ExtConfig]
  /**
   * The empty text to show for each field.
   * @default 'Enter Number...'
   */
  public native function get emptyText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set emptyText(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Configures field items individually. These properties override those defined
   * by <code><a href="#!/api/Ext.grid.filters.filter.Number-cfg-itemDefaults" rel="Ext.grid.filters.filter.Number-cfg-itemDefaults" class="docClass">itemDefaults</a></code>.
   *
   * <p>Example usage:</p>
   *
   * <pre><code> fields: {
   *      // Override itemDefaults for one field:
   *      gt: {
   *          width: 200
   *      }
   *
   *      // "lt" and "eq" fields retain all itemDefaults
   *  },
   * </code></pre>
   * @default {gt: {iconCls: Ext.baseCSSPrefix + 'grid-filters-gt', margin: '0 0 3px 0'}, lt: {iconCls: Ext.baseCSSPrefix + 'grid-filters-lt', margin: '0 0 3px 0'}, eq: {iconCls: Ext.baseCSSPrefix + 'grid-filters-eq', margin: 0}}
   * @see #getFields()
   * @see #setFields()
   */
  public native function get fields():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set fields(value:Object):void;

  /**
   * Returns the value of <a href="#!/api/Ext.grid.filters.filter.Number-cfg-fields" rel="Ext.grid.filters.filter.Number-cfg-fields" class="docClass">fields</a>.
   */
  public native function getFields():Object;

  /**
   * Sets the value of <a href="#!/api/Ext.grid.filters.filter.Number-cfg-fields" rel="Ext.grid.filters.filter.Number-cfg-fields" class="docClass">fields</a>.
   * @param fields The new value.
   */
  public native function setFields(fields:Object):void;
}
}