package ext.grid.plugin {
import ext.XTemplate;
import ext.plugin.AbstractPlugin;

[Uses("ext.plugin.AbstractPlugin")]
[Uses("ext.Plugin")]
[Uses("ext.Base")]
[Native("Ext.grid.plugin.RowExpander", require)]
/**
 * <strong>Fired through the grid's View.</strong>
 * @eventType ext.grid.plugin.events.RowExpanderPlugin_rowNode_record_expandRowEvent.COLLAPSE_BODY
 */
[Event(name = 'onCollapseBody', type = 'ext.grid.plugin.events.RowExpanderPlugin_rowNode_record_expandRowEvent')]
/**
 * <strong>Fired through the grid's View</strong>
 * @eventType ext.grid.plugin.events.RowExpanderPlugin_rowNode_record_expandRowEvent.EXPAND_BODY
 */
[Event(name = 'onExpandBody', type = 'ext.grid.plugin.events.RowExpanderPlugin_rowNode_record_expandRowEvent')]
/**
 * Plugin (ptype = 'rowexpander') that adds the ability to have a Column in a grid which enables
 * a second row body which expands/contracts.  The expand/contract behavior is configurable to react
 * on clicking of the column, double click of the row, and/or hitting enter while a row is selected.
 *
 * <p><strong>Note:</strong> The <a href="#!/api/Ext.grid.plugin.RowExpander" rel="Ext.grid.plugin.RowExpander" class="docClass">rowexpander</a> plugin and the rowbody
 * feature are exclusive and cannot both be set on the same grid / tree.</p>
 */
public class RowExpanderPlugin extends ext.plugin.AbstractPlugin {
  /**
   * @param config @inheritDoc
   */
  public function RowExpanderPlugin(config:ext.grid.plugin.RowExpanderPlugin = null) {
    super();
  }

  [ExtConfig]
  /**
   * Configure as <code>true</code> to put the row expander body <em>before</em> the data row.
   * @default false
   */
  public native function get bodyBefore():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set bodyBefore(value:Boolean):void;

  [ExtConfig]
  /**
   * The width of the row expander column which contains the [+]/[-] icons to toggle row expansion.
   * @default 24
   */
  public native function get columnWidth():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set columnWidth(value:Number):void;

  [ExtConfig]
  /**
   * <code>true</code> to toggle a row between expanded/collapsed when double clicked
   * (defaults to <code>true</code>).
   * @default true
   */
  public native function get expandOnDblClick():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set expandOnDblClick(value:Boolean):void;

  [ExtConfig]
  /**
   * This config is no longer supported. The Enter key initiated the grid's actinoable mode.
   */
  public native function get expandOnEnter():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set expandOnEnter(value:Boolean):void;

  [ExtConfig]
  /**
   * The width of the Row Expander column header
   * @default 24
   */
  public native function get headerWidth():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set headerWidth(value:Number):void;

  [ExtConfig]
  /**
   * An XTemplate which, when passed a record data object, produces HTML for the expanded row content <em>on the locked side of a lockable grid</em>.
   */
  public native function get lockedTpl():ext.XTemplate;

  [ExtConfig]
  /**
   * @private
   */
  public native function set lockedTpl(value:ext.XTemplate):void;

  [ExtConfig]
  /**
   * An XTemplate which, when passed a record data object, produces HTML for the expanded row content.
   *
   * <p>Note that if this plugin is applied to a lockable grid, the rowBodyTpl applies to the normal (unlocked) side.
   * See <a href="#!/api/Ext.grid.plugin.RowExpander-cfg-lockedTpl" rel="Ext.grid.plugin.RowExpander-cfg-lockedTpl" class="docClass">lockedTpl</a></p>
   */
  public native function get rowBodyTpl():ext.XTemplate;

  [ExtConfig]
  /**
   * @private
   */
  public native function set rowBodyTpl(value:ext.XTemplate):void;

  [ExtConfig]
  /**
   * <code>true</code> to ensure that the full row expander body is visible when clicking on the expander icon
   * (defaults to <code>true</code>)
   * @default true
   * @since 6.2.0
   */
  public native function get scrollIntoViewOnExpand():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set scrollIntoViewOnExpand(value:Boolean):void;

  [ExtConfig]
  /**
   * <code>true</code> to select a row when clicking on the expander icon
   * (defaults to <code>false</code>).
   * @default false
   */
  public native function get selectRowOnExpand():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set selectRowOnExpand(value:Boolean):void;

  [ArrayElementType("ext.grid.feature.IFeature")]
  /**
   * @return And array of Features or Feature config objects.
   * Returns the array of Feature configurations needed to make the RowExpander work.
   * May be overridden in a subclass to modify the returned array.
   */
  protected native function getFeatureConfig(grid:Object):Array;
}
}