package ext.grid.plugin {
import ext.mixin.IIdentifiable;
import ext.mixin.IStyleCacher;

[Uses("ext.grid.plugin.RowExpanderPlugin")]
[Uses("ext.mixin.Identifiable")]
[Uses("ext.mixin.StyleCacher")]
[Uses("ext.Base")]
[Uses("ext.plugin.AbstractPlugin")]
[Uses("ext.Plugin")]
[Uses("ext.Mixin")]
[Native("Ext.grid.plugin.RowWidget", require)]
/**
 * Plugin (ptype = 'rowwidget') that adds the ability to second row body in a grid which expands/contracts.
 *
 * <p>The expand/contract behavior is configurable to react on clicking of the column, double click of the row, and/or hitting enter while a row is selected.</p>
 *
 * <p>The expansion row may contain a <a href="#!/api/Ext.grid.plugin.RowWidget-cfg-widget" rel="Ext.grid.plugin.RowWidget-cfg-widget" class="docClass">widget</a> which is primed with the record of the corresponding grid row.
 * The widget's defaultBindProperty property is set to the record.</p>
 */
public class RowWidgetPlugin extends ext.grid.plugin.RowExpanderPlugin implements ext.mixin.IIdentifiable, ext.mixin.IStyleCacher {
  /**
   * @inheritDoc
   */
  public function RowWidgetPlugin(config:ext.grid.plugin.RowExpanderPlugin = null) {
    super();
  }

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * A map of xtype to <a href="#!/api/Ext.Component-cfg-ui" rel="Ext.Component-cfg-ui" class="docClass">Ext.Component.ui</a> names to use when using Components in the expansion row.
   * @default {}
   * @see #getDefaultWidgetUI()
   * @see #setDefaultWidgetUI()
   */
  public native function get defaultWidgetUI():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set defaultWidgetUI(value:Object):void;

  [ExtConfig]
  /**
   * A config object containing an <a href="#!/api/Ext.Component-cfg-xtype" rel="Ext.Component-cfg-xtype" class="docClass">xtype</a>.
   *
   * <p>This is used to create the widgets or components which are rendered into the expansion row <em>on the locked side of a lockable grid</em>.</p>
   */
  public native function get lockedWidget():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set lockedWidget(value:Object):void;

  [ExtConfig]
  /**
   * A config object containing an <a href="#!/api/Ext.Component-cfg-xtype" rel="Ext.Component-cfg-xtype" class="docClass">xtype</a>.
   *
   * <p>This is used to create the widgets or components which are rendered into the expansion row.</p>
   *
   * <p>The associated grid row's record is used to update the widget/component's <a href="#!/api/Ext.Component-property-defaultBindProperty" rel="Ext.Component-property-defaultBindProperty" class="docClass">defaultBindProperty</a>.</p>
   *
   * <p>Note that if this plugin is applied to a lockable grid, the widget applies to the normal (unlocked) side.
   * See <a href="#!/api/Ext.grid.plugin.RowWidget-cfg-lockedWidget" rel="Ext.grid.plugin.RowWidget-cfg-lockedWidget" class="docClass">lockedWidget</a></p>
   */
  public native function get widget():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set widget(value:Object):void;

  /**
   * Returns the value of <a href="#!/api/Ext.grid.plugin.RowWidget-cfg-defaultWidgetUI" rel="Ext.grid.plugin.RowWidget-cfg-defaultWidgetUI" class="docClass">defaultWidgetUI</a>.
   */
  public native function getDefaultWidgetUI():Object;

  /**
   * Retrieves the id of this component. Will autogenerate an id if one has not already been set.
   * @return id
   */
  public native function getId():String;

  /**
   * Returns if possible the widget currently associated with the passed record within the passed view.
   *
   * <p>Note that if the record is not currently in the rendered block, <em>or</em>, it has never been expanded
   * then there will not be a widget associated with that <code>record/view</code> context.</p>
   * @param view The view for which to return the widget
   * @param record The record for which to return the widget
   */
  public native function getWidget(view:Object, record:Object):*;

  /**
   * Sets the value of <a href="#!/api/Ext.grid.plugin.RowWidget-cfg-defaultWidgetUI" rel="Ext.grid.plugin.RowWidget-cfg-defaultWidgetUI" class="docClass">defaultWidgetUI</a>.
   * @param defaultWidgetUI The new value.
   */
  public native function setDefaultWidgetUI(defaultWidgetUI:Object):void;
}
}