package ext.grid.property {
import ext.grid.GridPanel;

[Uses("ext.grid.GridPanel")]
[Uses("ext.panel.TablePanel")]
[Uses("ext.grid.locking.Lockable")]
[Uses("ext.Base")]
[Uses("ext.panel.Panel")]
[Uses("ext.container.DockingContainer")]
[Uses("ext.container.Container")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.mixin.Container")]
[Uses("ext.Mixin")]
[Uses("ext.Component")]
[Uses("ext.util.Positionable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Animate")]
[Uses("ext.util.Floating")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.state.Stateful")]
[Native("Ext.grid.property.Grid", require)]
/**
 * Fires before a property value changes.  Handlers can return false to cancel the property change
 * (this will internally call <a href="#!/api/Ext.data.Model-method-reject" rel="Ext.data.Model-method-reject" class="docClass">Ext.data.Model.reject</a> on the property's record).
 * @eventType ext.grid.property.events.PropertyGrid_source_recordId_value_oldValueEvent.BEFORE_PROPERTY_CHANGE
 */
[Event(name = 'onBeforePropertyChange', type = 'ext.grid.property.events.PropertyGrid_source_recordId_value_oldValueEvent')]
/**
 * Fires after a property value has changed.
 * @eventType ext.grid.property.events.PropertyGrid_source_recordId_value_oldValueEvent.PROPERTY_CHANGE
 */
[Event(name = 'onPropertyChange', type = 'ext.grid.property.events.PropertyGrid_source_recordId_value_oldValueEvent')]
/**
 * A specialized grid implementation intended to mimic the traditional property grid as typically seen in
 * development IDEs.  Each row in the grid represents a property of some object, and the data is stored
 * as a set of name/value pairs in <a href="#!/api/Ext.grid.property.Property" rel="Ext.grid.property.Property" class="docClass">Properties</a>. By default, the editors
 * shown are inferred from the data in the cell. More control over this can be specified by using the
 * <a href="#!/api/Ext.grid.property.Grid-cfg-sourceConfig" rel="Ext.grid.property.Grid-cfg-sourceConfig" class="docClass">sourceConfig</a> option. Example usage:
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.property.Grid" rel="Ext.grid.property.Grid" class="docClass">Ext.grid.property.Grid</a>', {
 *     title: 'Properties Grid',
 *     width: 300,
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     source: {
 *         "(name)": "My Object",
 *         "Created": <a href="#!/api/Ext.Date-method-parse" rel="Ext.Date-method-parse" class="docClass">Ext.Date.parse</a>('10/15/2006', 'm/d/Y'),
 *         "Available": false,
 *         "Version": 0.01,
 *         "Description": "A test object"
 *     }
 * });
 * </code></pre>
 */
public class PropertyGrid extends ext.grid.GridPanel {
  /**
   * @param config @inheritDoc
   */
  public function PropertyGrid(config:ext.grid.property.PropertyGrid = null) {
    super();
  }

  [Deprecated(replacement = "sourceConfig")]
  [ExtConfig]
  /**
   * An object containing name/value pairs of custom editor type definitions that allow
   * the grid to support additional types of editable fields.  By default, the grid supports strongly-typed editing
   * of strings, dates, numbers and booleans using built-in form editors, but any custom type can be supported and
   * associated with a custom input control by specifying a custom editor.  The name of the editor
   * type should correspond with the name of the property that will use the editor.  Example usage:
   *
   * <pre><code>var grid = new <a href="#!/api/Ext.grid.property.Grid" rel="Ext.grid.property.Grid" class="docClass">Ext.grid.property.Grid</a>({
   *
   *     // Custom editors for certain property names
   *     customEditors: {
   *         evtStart: <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.field.Time" rel="Ext.form.field.Time" class="docClass">Ext.form.TimeField</a>', {selectOnFocus: true})
   *     },
   *
   *     // Displayed name for property names in the source
   *     propertyNames: {
   *         evtStart: 'Start Time'
   *     },
   *
   *     // Data object containing properties to edit
   *     source: {
   *         evtStart: '10:00 AM'
   *     }
   * });
   * </code></pre>
   */
  public native function get customEditors():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set customEditors(value:Object):void;

  [Deprecated(replacement = "sourceConfig")]
  [ExtConfig]
  /**
   * An object containing name/value pairs of custom renderer type definitions that allow
   * the grid to support custom rendering of fields.  By default, the grid supports strongly-typed rendering
   * of strings, dates, numbers and booleans using built-in form editors, but any custom type can be supported and
   * associated with the type of the value.  The name of the renderer type should correspond with the name of the property
   * that it will render.  Example usage:
   *
   * <pre><code>var grid = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.property.Grid" rel="Ext.grid.property.Grid" class="docClass">Ext.grid.property.Grid</a>', {
   *     customRenderers: {
   *         Available: function(v){
   *             if (v) {
   *                 return '&lt;span style="color: green;"&gt;Yes&lt;/span&gt;';
   *             } else {
   *                 return '&lt;span style="color: red;"&gt;No&lt;/span&gt;';
   *             }
   *         }
   *     },
   *     source: {
   *         Available: true
   *     }
   * });
   * </code></pre>
   */
  public native function get customRenderers():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set customRenderers(value:Object):void;

  [ExtConfig]
  /**
   * True to automatically infer the <a href="#!/api/Ext.grid.property.Grid-cfg-sourceConfig" rel="Ext.grid.property.Grid-cfg-sourceConfig" class="docClass">type</a> based on the initial value passed
   * for each field. This ensures the editor remains the correct type even if the value is blanked
   * and becomes empty.
   * @default true
   */
  public native function get inferTypes():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set inferTypes(value:Boolean):void;

  [ExtConfig]
  /**
   * Specify the width for the name column. The value column will take any remaining space.
   * @default 115
   */
  public native function get nameColumnWidth():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set nameColumnWidth(value:*):void;

  [ExtConfig]
  /**
   * The name of the field from the property store to use as the property field name.
   * This may be useful if you do not configure the property Grid from an object, but use your own store configuration.
   * @default 'name'
   */
  public native function get nameField():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set nameField(value:String):void;

  [Deprecated(replacement = "sourceConfig")]
  [ExtConfig]
  /**
   * An object containing custom property name/display name pairs.
   * If specified, the display name will be shown in the name column instead of the property name.
   */
  public native function get propertyNames():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set propertyNames(value:Object):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * A data object to use as the data source of the grid (see <a href="#!/api/Ext.grid.property.Grid-method-setSource" rel="Ext.grid.property.Grid-method-setSource" class="docClass">setSource</a> for details).
   * @see #getSource()
   * @see #setSource()
   */
  public native function get source():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set source(value:Object):void;

  [ExtConfig]
  /**
   * This option allows various configurations to be set for each field in the property grid.
   * None of these configurations are required
   *
   * <h4>displayName</h4>
   *
   * <p>A custom name to appear as label for this field. If specified, the display name will be shown
   * in the name column instead of the property name. Example usage:</p>
   *
   * <pre><code>new <a href="#!/api/Ext.grid.property.Grid" rel="Ext.grid.property.Grid" class="docClass">Ext.grid.property.Grid</a>({
   *     source: {
   *         clientIsAvailable: true
   *     },
   *     sourceConfig: {
   *         clientIsAvailable: {
   *             // Custom name different to the field
   *             displayName: 'Available'
   *         }
   *     }
   * });
   * </code></pre>
   *
   * <h4>renderer</h4>
   *
   * <p>A function used to transform the underlying value before it is displayed in the grid.
   * By default, the grid supports strongly-typed rendering of strings, dates, numbers and booleans using built-in form editors,
   * but any custom type can be supported and associated with the type of the value. Example usage:</p>
   *
   * <pre><code>new <a href="#!/api/Ext.grid.property.Grid" rel="Ext.grid.property.Grid" class="docClass">Ext.grid.property.Grid</a>({
   *     source: {
   *         clientIsAvailable: true
   *     },
   *     sourceConfig: {
   *         clientIsAvailable: {
   *             // Custom renderer to change the color based on the value
   *             renderer: function(v){
   *                 var color = v ? 'green' : 'red';
   *                 return '&lt;span style="color: ' + color + ';"&gt;' + v + '&lt;/span&gt;';
   *             }
   *         }
   *     }
   * });
   * </code></pre>
   *
   * <h4>type</h4>
   *
   * <p>Used to explicitly specify the editor type for a particular value. By default, the type is
   * automatically inferred from the value. See <a href="#!/api/Ext.grid.property.Grid-cfg-inferTypes" rel="Ext.grid.property.Grid-cfg-inferTypes" class="docClass">inferTypes</a>. Accepted values are:</p>
   *
   * <ul>
   * <li>'date'</li>
   * <li>'boolean'</li>
   * <li>'number'</li>
   * <li>'string'</li>
   * </ul>
   *
   *
   * <p>For more advanced control an editor configuration can be passed (see the next section).
   * Example usage:</p>
   *
   * <pre><code>new <a href="#!/api/Ext.grid.property.Grid" rel="Ext.grid.property.Grid" class="docClass">Ext.grid.property.Grid</a>({
   *     source: {
   *         attending: null
   *     },
   *     sourceConfig: {
   *         attending: {
   *             // Force the type to be a numberfield, a null value would otherwise default to a textfield
   *             type: 'number'
   *         }
   *     }
   * });
   * </code></pre>
   *
   * <h4>editor</h4>
   *
   * <p>Allows the grid to support additional types of editable fields.  By default, the grid supports strongly-typed editing
   * of strings, dates, numbers and booleans using built-in form editors, but any custom type can be supported and
   * associated with a custom input control by specifying a custom editor. Example usage</p>
   *
   * <pre><code>new <a href="#!/api/Ext.grid.property.Grid" rel="Ext.grid.property.Grid" class="docClass">Ext.grid.property.Grid</a>({
   *     // Data object containing properties to edit
   *     source: {
   *         evtStart: '10:00 AM'
   *     },
   *
   *     sourceConfig: {
   *         evtStart: {
   *             editor: <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.field.Time" rel="Ext.form.field.Time" class="docClass">Ext.form.field.Time</a>', {selectOnFocus: true}),
   *             displayName: 'Start Time'
   *         }
   *     }
   * });
   * </code></pre>
   */
  public native function get sourceConfig():Object;

  [ExtConfig]
  /**
   * @private
   */
  public native function set sourceConfig(value:Object):void;

  [ExtConfig]
  /**
   * The name of the field from the property store to use as the value field name.
   * This may be useful if you do not configure the property Grid from an object, but use your own store configuration.
   * @default 'value'
   */
  public native function get valueField():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set valueField(value:String):void;

  /**
   * Gets the value of a property.
   * @since 5.1.1
   * @param prop The name of the property.
   * @return The property value. <code>null</code> if there is no value.
   */
  public native function getProperty(prop:String):Object;

  /**
   * Gets the source data object containing the property data.  See <a href="#!/api/Ext.grid.property.Grid-method-setSource" rel="Ext.grid.property.Grid-method-setSource" class="docClass">setSource</a> for details regarding the
   * format of the data object.
   * @return The data object.
   */
  public native function getSource():Object;

  /**
   * Removes a property from the grid.
   * @param prop The name of the property to remove.
   */
  public native function removeProperty(prop:String):void;

  /**
   * Sets the value of a property.
   * @param prop The name of the property to set.
   * @param value The value to test.
   * @param create <code>true</code> to create the property if it doesn't already exist.
   * @default false
   */
  public native function setProperty(prop:String, value:*, create:Boolean = false):void;

  /**
   * Sets the source data object containing the property data.  The data object can contain one or more name/value
   * pairs representing all of the properties of an object to display in the grid, and this data will automatically
   * be loaded into the grid's <a href="#!/api/Ext.grid.property.Grid-cfg-store" rel="Ext.grid.property.Grid-cfg-store" class="docClass">store</a>.  The values should be supplied in the proper data type if needed,
   * otherwise string type will be assumed.  If the grid already contains data, this method will replace any
   * existing data.  See also the <a href="#!/api/Ext.grid.property.Grid-cfg-source" rel="Ext.grid.property.Grid-cfg-source" class="docClass">source</a> config value.  Example usage:
   *
   * <pre><code>grid.setSource({
   *     "(name)": "My Object",
   *     "Created": <a href="#!/api/Ext.Date-method-parse" rel="Ext.Date-method-parse" class="docClass">Ext.Date.parse</a>('10/15/2006', 'm/d/Y'),  // date type
   *     "Available": false,  // boolean type
   *     "Version": .01,      // decimal type
   *     "Description": "A test object"
   * });
   * </code></pre>
   * @param source The data object.
   * @param sourceConfig A new <a href="#!/api/Ext.grid.property.Grid-cfg-sourceConfig" rel="Ext.grid.property.Grid-cfg-sourceConfig" class="docClass">object</a>. If this argument is not passed
   * the current configuration will be re-used. To reset the config, pass <code>null</code> or an empty object literal.
   */
  public native function setSource(source:Object, sourceConfig:Object = null):void;
}
}