package ext.layout.container {

[Uses("ext.layout.container.AnchorLayout")]
[Uses("ext.layout.container.AutoLayout")]
[Uses("ext.layout.container.ContainerLayout")]
[Uses("ext.layout.Layout")]
[Uses("ext.mixin.Factoryable")]
[Uses("ext.Base")]
[Uses("ext.util.ElementContainer")]
[Native("Ext.layout.container.Absolute", require)]
/**
 * This is a layout that inherits the anchoring of <a href="#!/api/Ext.layout.container.Anchor" rel="Ext.layout.container.Anchor" class="docClass">Ext.layout.container.Anchor</a> and adds the
 * ability for x/y positioning using the standard x and y component config options.
 *
 * <p>This class is intended to be extended or created via the <a href="#!/api/Ext.container.Container-cfg-layout" rel="Ext.container.Container-cfg-layout" class="docClass">layout</a>
 * configuration property.  See <a href="#!/api/Ext.container.Container-cfg-layout" rel="Ext.container.Container-cfg-layout" class="docClass">Ext.container.Container.layout</a> for additional details.</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.form.Panel" rel="Ext.form.Panel" class="docClass">Ext.form.Panel</a>', {
 *     title: 'Absolute Layout',
 *     width: 300,
 *     height: 275,
 *     layout: {
 *         type: 'absolute'
 *         // layout-specific configs go here
 *         //itemCls: 'x-abs-layout-item',
 *     },
 *     url:'save-form.php',
 *     defaultType: 'textfield',
 *     items: [{
 *         x: 10,
 *         y: 10,
 *         xtype:'label',
 *         text: 'Send To:'
 *     },{
 *         x: 80,
 *         y: 10,
 *         name: 'to',
 *         anchor:'90%'  // anchor width by percentage
 *     },{
 *         x: 10,
 *         y: 40,
 *         xtype:'label',
 *         text: 'Subject:'
 *     },{
 *         x: 80,
 *         y: 40,
 *         name: 'subject',
 *         anchor: '90%'  // anchor width by percentage
 *     },{
 *         x:0,
 *         y: 80,
 *         xtype: 'textareafield',
 *         name: 'msg',
 *         anchor: '100% 100%'  // anchor width and height
 *     }],
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 * });
 * </code></pre>
 */
public class AbsoluteLayout extends ext.layout.container.AnchorLayout {
  /**
   * @param config @inheritDoc
   */
  public function AbsoluteLayout(config:ext.layout.container.AbsoluteLayout = null) {
    super();
  }
}
}