package ext.layout.container {
import ext.Base;

[Uses("ext.layout.container.ContainerLayout")]
[Uses("ext.layout.Layout")]
[Uses("ext.mixin.Factoryable")]
[Uses("ext.Base")]
[Uses("ext.util.ElementContainer")]
[Native("Ext.layout.container.Auto", require)]
/**
 * The AutoLayout is the default layout manager delegated by <a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Ext.container.Container</a> to
 * render any child Components when no <code><a href="#!/api/Ext.container.Container-cfg-layout" rel="Ext.container.Container-cfg-layout" class="docClass">layout</a></code> is configured into
 * a <code><a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Container</a>.</code> AutoLayout provides only a passthrough of any layout calls
 * to any child containers.
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.Panel</a>', {
 *     width: 500,
 *     height: 280,
 *     title: 'AutoLayout Panel',
 *     layout: 'auto',
 *     renderTo: document.body,
 *     items: [{
 *         xtype: 'panel',
 *         title: 'Top Inner Panel',
 *         width: '75%',
 *         height: 90
 *     }, {
 *         xtype: 'panel',
 *         title: 'Bottom Inner Panel',
 *         width: '75%',
 *         height: 90
 *     }]
 * });
 * </code></pre>
 */
public class AutoLayout extends ext.layout.container.ContainerLayout {
  /**
   * @param config @inheritDoc
   */
  public function AutoLayout(config:ext.layout.container.AutoLayout = null) {
    super();
  }

  /**
   * true to rerun the layout if scrollbars are needed.
   * @default false
   */
  public native function get manageOverflow():Boolean;

  /**
   * @private
   */
  public native function set manageOverflow(value:Boolean):void;

  /**
   * indicates that this layout will correct cross browser padding differences when the
   * container has overflow.
   *
   * <p>In some browsers the right and/or bottom padding of a container is lost when
   * the container has overflow.  If managePadding is true the layout will apply the
   * padding to an inner wrapping element instead of the container element that has the
   * overflow so that paddding will be included in the scrollable area.
   * Note: padding will not be managed if it is configured on the container using
   * a style config or css class.  In order to be managed, padding must be added to the
   * container using the appropriate <a href="#!/api/Ext.Component-property-contentPaddingProperty" rel="Ext.Component-property-contentPaddingProperty" class="docClass">contentPaddingProperty</a>.  For <a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Panels</a> use
   * <a href="#!/api/Ext.panel.Panel-cfg-bodyPadding" rel="Ext.panel.Panel-cfg-bodyPadding" class="docClass">Ext.panel.Panel.bodyPadding</a>, and for
   * <a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Containers</a>, use
   * <a href="#!/api/Ext.Component-cfg-padding" rel="Ext.Component-cfg-padding" class="docClass">padding</a></p>
   * @default true
   */
  public native function get managePadding():Boolean;

  /**
   * @private
   */
  public native function set managePadding(value:Boolean):void;

  [ExtConfig]
  /**
   * Set to <code>true</code> to leave space for a vertical scrollbar (if the OS shows space-consuming scrollbars) regardless
   * of whether a scrollbar is needed.
   *
   * <p>This is useful if content height changes during application usage, but you do not want the calculated width
   * of child items to change when a scrollbar appears or disappears. The scrollbar will appear in the reserved space,
   * and the calculated width of child Components will not change.</p>
   *
   * <pre class='inline-example '><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('Employee', {
   *     extend: '<a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>',
   *     fields: [
   *        {name: 'rating', type: 'int'},
   *        {name: 'salary', type: 'float'},
   *        {name: 'name'}
   *     ]
   * });
   *
   * function createFakeData(count) {
   *     var firstNames   = ['Screech', 'Kelly', 'Zach', 'Jessie', 'Lisa', 'A.C.', 'Richard'],
   *         lastNames    = ['Powers', 'Kapowski', 'Morris', 'Spano', 'Turtle', 'Slater', 'Belding'],
   *         ratings      = [1, 2, 3, 4, 5],
   *         salaries     = [100, 400, 900, 1500, 1000000];
   *
   *     var data = [];
   *     for (var i = 0; i &lt; (count || 25); i++) {
   *         var ratingId    = Math.floor(Math.random() * ratings.length),
   *             salaryId    = Math.floor(Math.random() * salaries.length),
   *             firstNameId = Math.floor(Math.random() * firstNames.length),
   *             lastNameId  = Math.floor(Math.random() * lastNames.length),
   *
   *             rating      = ratings[ratingId],
   *             salary      = salaries[salaryId],
   *             name        = <a href="#!/api/Ext.String-method-format" rel="Ext.String-method-format" class="docClass">Ext.String.format</a>("{0} {1}", firstNames[firstNameId], lastNames[lastNameId]);
   *
   *         data.push({
   *             rating: rating,
   *             salary: salary,
   *             name: name
   *         });
   *     }
   *     store.loadData(data);
   * }
   *
   * // create the Data Store
   * var store = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>', {
   *     id: 'store',
   *     model: 'Employee',
   *     proxy: {
   *         type: 'memory'
   *     }
   * });
   * createFakeData(10);
   *
   * var grid = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">Ext.grid.Panel</a>', {
   *     title: 'Grid loaded with varying number of records',
   *     anchor: '100%',
   *     store: store,
   *     columns: [{
   *         xtype: 'rownumberer',
   *         width: 40,
   *         sortable: false
   *     },{
   *         text: 'Name',
   *         flex: 1,
   *         sortable: true,
   *         dataIndex: 'name'
   *     },{
   *         text: 'Rating',
   *         width: 125,
   *         sortable: true,
   *         dataIndex: 'rating'
   *     },{
   *         text: 'Salary',
   *         width: 125,
   *         sortable: true,
   *         dataIndex: 'salary',
   *         align: 'right',
   *         renderer: <a href="#!/api/Ext.util.Format-method-usMoney" rel="Ext.util.Format-method-usMoney" class="docClass">Ext.util.Format.usMoney</a>
   *     }]
   * });
   *
   * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a>', {
   *     renderTo: document.body,
   *     width: 800,
   *     height: 600,
   *     layout: {
   *         type: 'anchor',
   *         reserveScrollbar: true // There will be a gap even when there's no scrollbar
   *     },
   *     scrollable: true,
   *     items: grid,
   *     tbar: {
   *         defaults: {
   *             handler: function(b) {
   *                 createFakeData(b.count);
   *             }
   *         },
   *         items: [{
   *              text: '10 Items',
   *              count: 10
   *         },{
   *              text: '100 Items',
   *              count: 100
   *         },{
   *              text: '300 Items',
   *              count: 300
   *         },{
   *              text: '1000 Items',
   *              count: 1000
   *         },{
   *              text: '5000 Items',
   *              count: 5000
   *         }]
   *     }
   * });
   * </code></pre>
   * @default false
   */
  public native function get reserveScrollbar():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set reserveScrollbar(value:Boolean):void;

  /**
   * Returns the container size (that of the target). Only the fixed-sized dimensions can
   * be returned because the shrinkWrap dimensions are based on the contentWidth/Height
   * as determined by the container layout.
   *
   * <p>If the <a href="#!/api/Ext.layout.container.Auto-method-calculateOverflow" rel="Ext.layout.container.Auto-method-calculateOverflow" class="docClass">calculateOverflow</a> method is used and if <a href="#!/api/Ext.layout.container.Auto-property-manageOverflow" rel="Ext.layout.container.Auto-property-manageOverflow" class="docClass">manageOverflow</a> is
   * true, this will adjust the width/height by the size of scrollbars.</p>
   * @param ownerContext The owner's context item.
   * @param inDom True if the container size must be in the DOM.
   * @default false
   * @return The size
   * <ul>
   *   <li><code>width</code> : Number
   * The width
   *   </li>
   *   <li><code>height</code> : Number
   * The height
   *   </li>
   * </ul>
   */
  override protected native function getContainerSize(ownerContext:ext.Base, inDom:Boolean = false):Object;

  /**
   * Handles overflow processing for a container.  In addition to the ownerContext
   * passed to the <a href="#!/api/Ext.layout.container.Auto-method-calculate" rel="Ext.layout.container.Auto-method-calculate" class="docClass">calculate</a> method, this method also needs the containerSize
   * (the object returned by <a href="#!/api/Ext.layout.container.Auto-method-getContainerSize" rel="Ext.layout.container.Auto-method-getContainerSize" class="docClass">getContainerSize</a>).
   */
  protected native function calculateOverflow(ownerContext:ext.Base):void;

  /**
   * Returns the overflow-x style of the render target.
   * Note: If overflow is configured on a container using style or css class this method
   * will read the dom the first time it is called. It is therefore preferable for
   * performance reasons to use the {&#64;link <a href="#!/api/Ext.Component-cfg-scrollable" rel="Ext.Component-cfg-scrollable" class="docClass">Ext.Component.scrollable</a> scrollable config when
   * horizontal overflow is desired.
   */
  protected native function getOverflowXStyle(ownerContext:ext.Base):String;

  /**
   * Returns the overflow-y style of the render target.
   * Note: If overflow is configured on a container using style or css class this method
   * will read the dom the first time it is called. It is therefore preferable for
   * performance reasons to use the {&#64;link <a href="#!/api/Ext.Component-cfg-scrollable" rel="Ext.Component-cfg-scrollable" class="docClass">Ext.Component.scrollable</a> scrollable config when
   * vertical overflow is desired.
   */
  protected native function getOverflowYStyle(ownerContext:ext.Base):String;

  /**
   * This method sets the height and/or width of the outerCt/innerCt to adjust for the
   * following browser-specific issues:
   *
   * <ol>
   * <li><p>In some browsers a percentage-height element ignores the horizontal scrollbar
   * of its parent (see <a href="#!/api/Ext.supports-property-PercentageHeightOverflowBug" rel="Ext.supports-property-PercentageHeightOverflowBug" class="docClass">Ext.supports.PercentageHeightOverflowBug</a>).  If the browser is
   * affected by this bug the outerCt needs a pixel height in order to support
   * percentage-height children when not shrink-wrapping height. If the browser is not
   * affected by this bug, a height of 100% is assigned to the outerCt (see
   * beginLayoutCycle).</p></li>
   * <li><p>IE8 mode has a bug with percentage height children.  if the innerCt has
   * a height of 100%, has padding, and has a child item with a percentage height, that
   * child item will be sized as a percentage of the parent's height plus padding height.
   * In other words, a child with height:50% would have its height caclulated thusly:
   * (parentHeight + parentPaddingHeight) * 0.5
   * To fix this, we have to give the innerCt a pixel height.</p></li>
   * </ol>
   */
  protected native function setCtSizeIfNeeded(ownerContext:ext.Base, containerSize:Object):void;
}
}