package ext.layout.container {

[Uses("ext.layout.container.FitLayout")]
[Uses("ext.layout.container.ContainerLayout")]
[Uses("ext.layout.Layout")]
[Uses("ext.mixin.Factoryable")]
[Uses("ext.Base")]
[Uses("ext.util.ElementContainer")]
[Native("Ext.layout.container.Center", require)]
/**
 * This layout manager is used to center contents within a container. As a subclass of
 * <a href="#!/api/Ext.layout.container.Fit" rel="Ext.layout.container.Fit" class="docClass">fit layout</a>, CenterLayout expects to have one child
 * item; multiple items will be placed overlapping. The layout does not require any config
 * options. Items in the container can use percentage width or height rather than be fit
 * to the full size of the container.
 *
 * <p>Example usage:</p>
 *
 * <pre><code> // The content panel is centered in the container
 *
 *  var p = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.Panel</a>', {
 *      title: 'Center Layout',
 *      layout: 'center',
 *      items: [{
 *          title: 'Centered Content',
 *          width: '75%',  // assign 75% of the container width to the item
 *          html: 'Some content'
 *      }]
 *  });
 * </code></pre>
 *
 * <p>If you leave the title blank and specify no border you can create a non-visual, structural
 * container just for centering the contents.</p>
 *
 * <pre><code> var p = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Ext.Container</a>', {
 *      layout: 'center',
 *      items: [{
 *          title: 'Centered Content',
 *          width: 300,
 *          height: '90%', // assign 90% of the container height to the item
 *          html: 'Some content'
 *      }]
 *  });
 * </code></pre>
 */
public class CenterLayout extends ext.layout.container.FitLayout {
  /**
   * @param config @inheritDoc
   */
  public function CenterLayout(config:ext.layout.container.CenterLayout = null) {
    super();
  }
}
}