package ext.list {
import ext.Widget;
import ext.data.TreeModel;
import ext.dom.Element;
import ext.event.Event;

[Uses("ext.Widget")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.Mixin")]
[Uses("ext.Base")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.Pluggable")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.Evented")]
[Uses("ext.mixin.Observable")]
[Native("Ext.list.AbstractTreeItem", require)]
/**
 * @since 6.0.1
 * @eventType ext.list.events.AbstractTreeItem_sender_infoEvent.ITEM_CLICK
 */
[Event(name = 'onItemClick', type = 'ext.list.events.AbstractTreeItem_sender_infoEvent')]
/**
 * The base class for all items in the <code><a href="#!/api/Ext.list.Tree" rel="Ext.list.Tree" class="docClass">treelist</a></code>.
 */
public class AbstractTreeItem extends ext.Widget {
  /**
   * @param config @inheritDoc
   */
  public function AbstractTreeItem(config:ext.list.AbstractTreeItem = null) {
    super();
  }

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * <code>true</code> if this item is expandable. This value is taken from
   * the underlying <a href="#!/api/Ext.list.AbstractTreeItem-cfg-node" rel="Ext.list.AbstractTreeItem-cfg-node" class="docClass">node</a>.
   * @default false
   * @see #getExpandable()
   * @see #setExpandable()
   */
  public native function get expandable():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set expandable(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * <code>true</code> if this item is expanded. This value is taken from
   * the underlying <a href="#!/api/Ext.list.AbstractTreeItem-cfg-node" rel="Ext.list.AbstractTreeItem-cfg-node" class="docClass">node</a>.
   * @default false
   * @see #getExpanded()
   * @see #setExpanded()
   */
  public native function get expanded():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set expanded(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * One or more space separated CSS classes to be applied to the icon element.<br/>
   * The CSS rule(s) applied should specify a background image to be used as the
   * icon.
   *
   * <p>An example of specifying a custom icon class would be something like:</p>
   *
   * <pre><code>// specify the property in the config for the class:
   * iconCls: 'my-home-icon'
   *
   * // css rule specifying the background image to be used as the icon image:
   * .my-home-icon {
   *     background-image: url(../images/my-home-icon.gif) !important;
   * }
   * </code></pre>
   *
   * <p>In addition to specifying your own classes, you can use the font icons
   * provided in the SDK using the following syntax:</p>
   *
   * <pre><code>// using Font Awesome
   * iconCls: 'x-fa fa-home'
   *
   * // using Pictos
   * iconCls: 'pictos pictos-home'
   * </code></pre>
   *
   * <p>Depending on the theme you're using, you may need include the font icon
   * packages in your application in order to use the icons included in the
   * SDK.  For more information see:</p>
   *
   * <ul>
   * <li><a href="http://fortawesome.github.io/Font-Awesome/cheatsheet/">Font Awesome icons</a></li>
   * <li><a href="http://docs.sencha.com/extjs/6.0/core_concepts/font_ext.html">Pictos icons</a></li>
   * <li><a href="http://docs.sencha.com/extjs/6.0/core_concepts/theming.html">Theming Guide</a></li>
   * </ul>
   * @default ''
   * @see #getIconCls()
   * @see #setIconCls()
   */
  public native function get iconCls():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set iconCls(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The property from the <a href="#!/api/Ext.list.AbstractTreeItem-cfg-node" rel="Ext.list.AbstractTreeItem-cfg-node" class="docClass">node</a> to map for the <a href="#!/api/Ext.list.AbstractTreeItem-cfg-iconCls" rel="Ext.list.AbstractTreeItem-cfg-iconCls" class="docClass">iconCls</a> config.
   * @default 'iconCls'
   * @see #getIconClsProperty()
   * @see #setIconClsProperty()
   */
  public native function get iconClsProperty():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set iconClsProperty(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * <code>true</code> if this item is a leaf. This value is taken from
   * the underlying <a href="#!/api/Ext.list.AbstractTreeItem-cfg-node" rel="Ext.list.AbstractTreeItem-cfg-node" class="docClass">node</a>.
   * @default true
   * @see #getLeaf()
   * @see #setLeaf()
   */
  public native function get leaf():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set leaf(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * <code>true</code> if this item is currently loading data. This value is taken from
   * the underlying <a href="#!/api/Ext.list.AbstractTreeItem-cfg-node" rel="Ext.list.AbstractTreeItem-cfg-node" class="docClass">node</a>.
   * @default false
   * @see #getLoading()
   * @see #setLoading()
   */
  public native function get loading():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set loading(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The backing node for this item.
   * @see #getNode()
   * @see #setNode()
   */
  public native function get node():ext.data.TreeModel;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set node(value:ext.data.TreeModel):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * One of three possible values:
   *
   * <ul>
   * <li>0 if mouse is not over this item or any of its descendants.</li>
   * <li>1 if mouse is not over this item but is over one of this item's descendants.</li>
   * <li>2 if mouse is directly over this item.</li>
   * </ul>
   * @see #getOver()
   * @see #setOver()
   */
  public native function get over():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set over(value:Number):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The owning list for this container.
   * @see #getOwner()
   * @see #setOwner()
   */
  public native function get owner():ext.list.Tree;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set owner(value:ext.list.Tree):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The parent item for this item.
   * @see #getParentItem()
   * @see #setParentItem()
   */
  public native function get parentItem():ext.list.AbstractTreeItem;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set parentItem(value:ext.list.AbstractTreeItem):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * <code>true</code> if this is the selected item in the tree.
   * @default false
   * @see #getSelected()
   * @see #setSelected()
   */
  public native function get selected():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set selected(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * <code>true</code> if this item contains the <a href="#!/api/Ext.list.AbstractTreeItem-cfg-selected" rel="Ext.list.AbstractTreeItem-cfg-selected" class="docClass">selected</a> item in the tree.
   * @default false
   * @see #getSelectedParent()
   * @see #setSelectedParent()
   */
  public native function get selectedParent():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set selectedParent(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The text for this item. This value is taken from
   * the underlying <a href="#!/api/Ext.list.AbstractTreeItem-cfg-node" rel="Ext.list.AbstractTreeItem-cfg-node" class="docClass">node</a>.
   * @default {lazy: true, $value: ''}
   * @see #getText()
   * @see #setText()
   */
  public native function get text():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set text(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The property from the <a href="#!/api/Ext.list.AbstractTreeItem-cfg-node" rel="Ext.list.AbstractTreeItem-cfg-node" class="docClass">node</a> to map for the <a href="#!/api/Ext.list.AbstractTreeItem-cfg-text" rel="Ext.list.AbstractTreeItem-cfg-text" class="docClass">text</a> config.
   * @default 'text'
   * @see #getTextProperty()
   * @see #setTextProperty()
   */
  public native function get textProperty():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set textProperty(value:String):void;

  /**
   * Collapse this item. Does nothing if already collapsed.
   */
  public native function collapse():void;

  /**
   * Expand this item. Does nothing if already expanded.
   */
  public native function expand():void;

  /**
   * Returns the value of <a href="#!/api/Ext.list.AbstractTreeItem-cfg-expandable" rel="Ext.list.AbstractTreeItem-cfg-expandable" class="docClass">expandable</a>.
   */
  public native function getExpandable():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.list.AbstractTreeItem-cfg-expanded" rel="Ext.list.AbstractTreeItem-cfg-expanded" class="docClass">expanded</a>.
   */
  public native function getExpanded():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.list.AbstractTreeItem-cfg-iconCls" rel="Ext.list.AbstractTreeItem-cfg-iconCls" class="docClass">iconCls</a>.
   */
  public native function getIconCls():String;

  /**
   * Returns the value of <a href="#!/api/Ext.list.AbstractTreeItem-cfg-iconClsProperty" rel="Ext.list.AbstractTreeItem-cfg-iconClsProperty" class="docClass">iconClsProperty</a>.
   */
  public native function getIconClsProperty():String;

  /**
   * Returns the value of <a href="#!/api/Ext.list.AbstractTreeItem-cfg-leaf" rel="Ext.list.AbstractTreeItem-cfg-leaf" class="docClass">leaf</a>.
   */
  public native function getLeaf():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.list.AbstractTreeItem-cfg-loading" rel="Ext.list.AbstractTreeItem-cfg-loading" class="docClass">loading</a>.
   */
  public native function getLoading():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.list.AbstractTreeItem-cfg-node" rel="Ext.list.AbstractTreeItem-cfg-node" class="docClass">node</a>.
   */
  public native function getNode():ext.data.TreeModel;

  /**
   * Returns the value of <a href="#!/api/Ext.list.AbstractTreeItem-cfg-over" rel="Ext.list.AbstractTreeItem-cfg-over" class="docClass">over</a>.
   */
  public native function getOver():Number;

  /**
   * Returns the value of <a href="#!/api/Ext.list.AbstractTreeItem-cfg-owner" rel="Ext.list.AbstractTreeItem-cfg-owner" class="docClass">owner</a>.
   */
  public native function getOwner():ext.list.Tree;

  /**
   * Returns the value of <a href="#!/api/Ext.list.AbstractTreeItem-cfg-parentItem" rel="Ext.list.AbstractTreeItem-cfg-parentItem" class="docClass">parentItem</a>.
   */
  public native function getParentItem():ext.list.AbstractTreeItem;

  /**
   * Returns the value of <a href="#!/api/Ext.list.AbstractTreeItem-cfg-selected" rel="Ext.list.AbstractTreeItem-cfg-selected" class="docClass">selected</a>.
   */
  public native function getSelected():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.list.AbstractTreeItem-cfg-selectedParent" rel="Ext.list.AbstractTreeItem-cfg-selectedParent" class="docClass">selectedParent</a>.
   */
  public native function getSelectedParent():Boolean;

  /**
   * Returns the value of <a href="#!/api/Ext.list.AbstractTreeItem-cfg-text" rel="Ext.list.AbstractTreeItem-cfg-text" class="docClass">text</a>.
   */
  public native function getText():String;

  /**
   * Returns the value of <a href="#!/api/Ext.list.AbstractTreeItem-cfg-textProperty" rel="Ext.list.AbstractTreeItem-cfg-textProperty" class="docClass">textProperty</a>.
   */
  public native function getTextProperty():String;

  /**
   * Check if the current item is expanded.
   * @return <code>true</code> if this item is expanded.
   */
  public native function isExpanded():Boolean;

  /**
   * Sets the value of <a href="#!/api/Ext.list.AbstractTreeItem-cfg-expandable" rel="Ext.list.AbstractTreeItem-cfg-expandable" class="docClass">expandable</a>.
   * @param expandable The new value.
   */
  public native function setExpandable(expandable:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.list.AbstractTreeItem-cfg-expanded" rel="Ext.list.AbstractTreeItem-cfg-expanded" class="docClass">expanded</a>.
   * @param expanded The new value.
   */
  public native function setExpanded(expanded:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.list.AbstractTreeItem-cfg-iconCls" rel="Ext.list.AbstractTreeItem-cfg-iconCls" class="docClass">iconCls</a>.
   * @param iconCls The new value.
   */
  public native function setIconCls(iconCls:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.list.AbstractTreeItem-cfg-iconClsProperty" rel="Ext.list.AbstractTreeItem-cfg-iconClsProperty" class="docClass">iconClsProperty</a>.
   * @param iconClsProperty The new value.
   */
  public native function setIconClsProperty(iconClsProperty:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.list.AbstractTreeItem-cfg-leaf" rel="Ext.list.AbstractTreeItem-cfg-leaf" class="docClass">leaf</a>.
   * @param leaf The new value.
   */
  public native function setLeaf(leaf:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.list.AbstractTreeItem-cfg-loading" rel="Ext.list.AbstractTreeItem-cfg-loading" class="docClass">loading</a>.
   * @param loading The new value.
   */
  public native function setLoading(loading:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.list.AbstractTreeItem-cfg-node" rel="Ext.list.AbstractTreeItem-cfg-node" class="docClass">node</a>.
   * @param node The new value.
   */
  public native function setNode(node:ext.data.TreeModel):void;

  /**
   * Sets the value of <a href="#!/api/Ext.list.AbstractTreeItem-cfg-over" rel="Ext.list.AbstractTreeItem-cfg-over" class="docClass">over</a>.
   * @param over The new value.
   */
  public native function setOver(over:Number):void;

  /**
   * Sets the value of <a href="#!/api/Ext.list.AbstractTreeItem-cfg-owner" rel="Ext.list.AbstractTreeItem-cfg-owner" class="docClass">owner</a>.
   * @param owner The new value.
   */
  public native function setOwner(owner:ext.list.Tree):void;

  /**
   * Sets the value of <a href="#!/api/Ext.list.AbstractTreeItem-cfg-parentItem" rel="Ext.list.AbstractTreeItem-cfg-parentItem" class="docClass">parentItem</a>.
   * @param parentItem The new value.
   */
  public native function setParentItem(parentItem:ext.list.AbstractTreeItem):void;

  /**
   * Sets the value of <a href="#!/api/Ext.list.AbstractTreeItem-cfg-selected" rel="Ext.list.AbstractTreeItem-cfg-selected" class="docClass">selected</a>.
   * @param selected The new value.
   */
  public native function setSelected(selected:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.list.AbstractTreeItem-cfg-selectedParent" rel="Ext.list.AbstractTreeItem-cfg-selectedParent" class="docClass">selectedParent</a>.
   * @param selectedParent The new value.
   */
  public native function setSelectedParent(selectedParent:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.list.AbstractTreeItem-cfg-text" rel="Ext.list.AbstractTreeItem-cfg-text" class="docClass">text</a>.
   * @param text The new value.
   */
  public native function setText(text:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.list.AbstractTreeItem-cfg-textProperty" rel="Ext.list.AbstractTreeItem-cfg-textProperty" class="docClass">textProperty</a>.
   * @param textProperty The new value.
   */
  public native function setTextProperty(textProperty:String):void;

  /**
   * Gets the element to be used for the tree when it is in
   * <a href="#!/api/Ext.list.Tree-cfg-micro" rel="Ext.list.Tree-cfg-micro" class="docClass">micro</a> mode.
   * @return The element.
   */
  protected native function getToolElement():ext.dom.Element;

  /**
   * Append a new child item to the DOM.
   * @param item The item to insert.
   * @param refItem The item the node is to
   * be inserted before. <code>null</code> if the item is to be added to the end.
   */
  protected native function insertItem(item:ext.list.AbstractTreeItem, refItem:ext.list.AbstractTreeItem):void;

  /**
   * Checks whether the event is an event that should select this node.
   * @param e The event object.
   * @return <code>true</code> if the event should select this node.
   */
  protected native function isSelectionEvent(e:ext.event.Event):Boolean;

  /**
   * Checks whether the event is an event that should toggle the expand/collapse state.
   * @param e The event object.
   * @return <code>true</code> if the event should toggle the expand/collapsed state.
   */
  protected native function isToggleEvent(e:ext.event.Event):Boolean;

  /**
   * Handle this node being collapsed.
   * @param node The node being collapsed.
   */
  protected native function nodeCollapse(node:ext.data.TreeModel):void;

  /**
   * Handle this node being expanded.
   * @param node The node being expanded.
   */
  protected native function nodeExpand(node:ext.data.TreeModel):void;

  /**
   * Handle a node being inserted as a child of this item.
   * @param node The node being inserted.
   * @param refNode The node that is to be inserted before. <code>null</code>
   * if this operation is an append.
   */
  protected native function nodeInsert(node:ext.data.TreeModel, refNode:ext.data.TreeModel):void;

  /**
   * Handle a node being removed as a child of this item.
   * @param node The node being removed.
   */
  protected native function nodeRemove(node:ext.data.TreeModel):void;

  /**
   * Handle this node having fields changed.
   * @param node The node.
   * @param modifiedFieldNames The modified field names, if known.
   */
  protected native function nodeUpdate(node:ext.data.TreeModel, modifiedFieldNames:Array):void;

  /**
   * Remove a child item from the DOM.
   * @param item The item to remove.
   */
  protected native function removeItem(item:ext.list.AbstractTreeItem):void;
}
}