package ext.menu {
import ext.picker.ColorPicker;

[Uses("ext.menu.Menu")]
[Uses("ext.util.FocusableContainer")]
[Uses("ext.Mixin")]
[Uses("ext.Base")]
[Uses("ext.panel.Panel")]
[Uses("ext.container.DockingContainer")]
[Uses("ext.container.Container")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.mixin.Container")]
[Uses("ext.Component")]
[Uses("ext.util.Positionable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Animate")]
[Uses("ext.util.Floating")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.state.Stateful")]
[Native("Ext.menu.ColorPicker", require)]
/**
 * Fires when a color is selected
 * @eventType ext.menu.events.ColorPickerMenu_colorEvent.SELECT
 */
[Event(name = 'onSelect', type = 'ext.menu.events.ColorPickerMenu_colorEvent')]
/**
 * A menu containing a <a href="#!/api/Ext.picker.Color" rel="Ext.picker.Color" class="docClass">Ext.picker.Color</a> Component.
 *
 * <p>Notes:</p>
 *
 * <ul>
 * <li>Although not listed here, the <strong>constructor</strong> for this class accepts all of the
 * configuration options of <a href="#!/api/Ext.picker.Color" rel="Ext.picker.Color" class="docClass">Ext.picker.Color</a>.</li>
 * <li>If subclassing ColorMenu, any configuration options for the ColorPicker must be
 * applied to the <strong>initialConfig</strong> property of the ColorMenu. Applying
 * <a href="#!/api/Ext.picker.Color" rel="Ext.picker.Color" class="docClass">ColorPicker</a> configuration settings to <code>this</code> will <strong>not</strong>
 * affect the ColorPicker's configuration.</li>
 * </ul>
 *
 *
 * <p>Example:</p>
 *
 * <pre class='inline-example '><code>var colorPicker = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.menu.ColorPicker" rel="Ext.menu.ColorPicker" class="docClass">Ext.menu.ColorPicker</a>', {
 *     value: '000000'
 * });
 *
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.menu.Menu" rel="Ext.menu.Menu" class="docClass">Ext.menu.Menu</a>', {
 *     items: [{
 *         text: 'Choose a color',
 *         menu: colorPicker
 *     },{
 *         iconCls: 'add16',
 *         text: 'Icon item'
 *     },{
 *         text: 'Regular item'
 *     }]
 * }).showAt([5, 5]);
 * </code></pre>
 */
public class ColorPickerMenu extends ext.menu.Menu {
  /**
   * @param config @inheritDoc
   */
  public function ColorPickerMenu(config:ext.menu.ColorPickerMenu = null) {
    super();
  }

  /**
   * The <a href="#!/api/Ext.picker.Color" rel="Ext.picker.Color" class="docClass">Ext.picker.Color</a> instance for this ColorMenu
   */
  public native function get picker():ext.picker.ColorPicker;

  /**
   * @private
   */
  public native function set picker(value:ext.picker.ColorPicker):void;

  [ExtConfig]
  /**
   * False to continue showing the menu after a color is selected.
   * @default true
   */
  public native function get hideOnClick():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set hideOnClick(value:Boolean):void;

  [ExtConfig]
  /**
   * An id to assign to the underlying color picker.
   */
  public native function get pickerId():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set pickerId(value:String):void;
}
}