package ext.menu {
import ext.picker.DatePicker;

[Uses("ext.menu.Menu")]
[Uses("ext.util.FocusableContainer")]
[Uses("ext.Mixin")]
[Uses("ext.Base")]
[Uses("ext.panel.Panel")]
[Uses("ext.container.DockingContainer")]
[Uses("ext.container.Container")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.mixin.Container")]
[Uses("ext.Component")]
[Uses("ext.util.Positionable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Animate")]
[Uses("ext.util.Floating")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.state.Stateful")]
[Native("Ext.menu.DatePicker", require)]
/**
 * Fires when a date is selected
 * @eventType ext.menu.events.DatePickerMenu_dateEvent.SELECT
 */
[Event(name = 'onSelect', type = 'ext.menu.events.DatePickerMenu_dateEvent')]
/**
 * A menu containing an <a href="#!/api/Ext.picker.Date" rel="Ext.picker.Date" class="docClass">Ext.picker.Date</a> Component.
 *
 * <p>Notes:</p>
 *
 * <ul>
 * <li>Although not listed here, the <strong>constructor</strong> for this class accepts all of the
 * configuration options of <strong><a href="#!/api/Ext.picker.Date" rel="Ext.picker.Date" class="docClass">Ext.picker.Date</a></strong>.</li>
 * <li>If subclassing DateMenu, any configuration options for the DatePicker must be applied
 * to the <strong>initialConfig</strong> property of the DateMenu. Applying <a href="#!/api/Ext.picker.Date" rel="Ext.picker.Date" class="docClass">Date Picker</a>
 * configuration settings to <strong>this</strong> will <strong>not</strong> affect the Date Picker's configuration.</li>
 * </ul>
 *
 *
 * <p>Example:</p>
 *
 * <pre class='inline-example '><code>var dateMenu = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.menu.DatePicker" rel="Ext.menu.DatePicker" class="docClass">Ext.menu.DatePicker</a>', {
 *     handler: function(dp, date){
 *         <a href="#!/api/Ext.MessageBox-method-alert" rel="Ext.MessageBox-method-alert" class="docClass">Ext.Msg.alert</a>('Date Selected', 'You selected ' + <a href="#!/api/Ext.Date-method-format" rel="Ext.Date-method-format" class="docClass">Ext.Date.format</a>(date, 'M j, Y'));
 *     }
 * });
 *
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.menu.Menu" rel="Ext.menu.Menu" class="docClass">Ext.menu.Menu</a>', {
 *     items: [{
 *         text: 'Choose a date',
 *         menu: dateMenu
 *     },{
 *         iconCls: 'add16',
 *         text: 'Icon item'
 *     },{
 *         text: 'Regular item'
 *     }]
 * }).showAt([5, 5]);
 * </code></pre>
 */
public class DatePickerMenu extends ext.menu.Menu {
  /**
   * @param config @inheritDoc
   */
  public function DatePickerMenu(config:ext.menu.DatePickerMenu = null) {
    super();
  }

  [ExtConfig]
  /**
   * ARIA label for the Date Picker menu
   * @default 'Date picker'
   */
  override public native function get ariaLabel():String;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set ariaLabel(value:String):void;

  /**
   * The <a href="#!/api/Ext.picker.Date" rel="Ext.picker.Date" class="docClass">Ext.picker.Date</a> instance for this DateMenu
   */
  public native function get picker():ext.picker.DatePicker;

  /**
   * @private
   */
  public native function set picker(value:ext.picker.DatePicker):void;

  [ExtConfig]
  /**
   * False to continue showing the menu after a date is selected.
   * @default true
   */
  public native function get hideOnClick():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set hideOnClick(value:Boolean):void;

  [ExtConfig]
  /**
   * An id to assign to the underlying date picker.
   */
  public native function get pickerId():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set pickerId(value:String):void;
}
}