package ext.menu {
import ext.Component;
import ext.panel.Panel;
import ext.util.IFocusableContainer;

[Uses("ext.panel.Panel")]
[Uses("ext.util.FocusableContainer")]
[Uses("ext.Mixin")]
[Uses("ext.Base")]
[Uses("ext.container.DockingContainer")]
[Uses("ext.container.Container")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.mixin.Container")]
[Uses("ext.Component")]
[Uses("ext.util.Positionable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Animate")]
[Uses("ext.util.Floating")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.state.Stateful")]
[Native("Ext.menu.Menu", require)]
/**
 * Fires when this menu is clicked
 * @eventType ext.menu.events.Menu_item_eEvent.CLICK
 */
[Event(name = 'onClick', type = 'ext.menu.events.Menu_item_eEvent')]
/**
 * Fires when the mouse enters this menu
 * @eventType ext.menu.events.Menu_eEvent.MOUSE_ENTER
 */
[Event(name = 'onMouseEnter', type = 'ext.menu.events.Menu_eEvent')]
/**
 * Fires when the mouse leaves this menu
 * @eventType ext.menu.events.Menu_eEvent.MOUSE_LEAVE
 */
[Event(name = 'onMouseLeave', type = 'ext.menu.events.Menu_eEvent')]
/**
 * Fires when the mouse is hovering over this menu
 * @eventType ext.menu.events.Menu_item_eEvent.MOUSE_OVER
 */
[Event(name = 'onMouseOver', type = 'ext.menu.events.Menu_item_eEvent')]
/**
 * A menu object. This is the container to which you may add <a href="#!/api/Ext.menu.Item" rel="Ext.menu.Item" class="docClass">menu items</a>.
 *
 * <p>Menus may contain either <a href="#!/api/Ext.menu.Item" rel="Ext.menu.Item" class="docClass">menu items</a>, or general <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Components</a>.
 * Menus may also contain <a href="#!/api/Ext.panel.Panel-cfg-dockedItems" rel="Ext.panel.Panel-cfg-dockedItems" class="docClass">docked items</a> because it extends <a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a>.</p>
 *
 * <p>By default, non <a href="#!/api/Ext.menu.Item" rel="Ext.menu.Item" class="docClass">menu items</a> are indented so that they line up with the text of menu items. clearing
 * the icon column. To make a contained general <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Component</a> left aligned configure the child
 * Component with `indent: false.</p>
 *
 * <p>By default, Menus are absolutely positioned, floating Components. By configuring a
 * Menu with <code><a href="#!/api/Ext.menu.Menu-cfg-floating" rel="Ext.menu.Menu-cfg-floating" class="docClass">floating</a>: false</code>, a Menu may be used as a child of a
 * <a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Container</a>.</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.menu.Menu" rel="Ext.menu.Menu" class="docClass">Ext.menu.Menu</a>', {
 *     width: 100,
 *     margin: '0 0 10 0',
 *     floating: false,  // usually you want this set to True (default)
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),  // usually rendered by it's containing component
 *     items: [{
 *         text: 'regular item 1'
 *     },{
 *         text: 'regular item 2'
 *     },{
 *         text: 'regular item 3'
 *     }]
 * });
 *
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.menu.Menu" rel="Ext.menu.Menu" class="docClass">Ext.menu.Menu</a>', {
 *     width: 100,
 *     plain: true,
 *     floating: false,  // usually you want this set to True (default)
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),  // usually rendered by it's containing component
 *     items: [{
 *         text: 'plain item 1'
 *     },{
 *         text: 'plain item 2'
 *     },{
 *         text: 'plain item 3'
 *     }]
 * });
 * </code></pre>
 */
public class Menu extends ext.panel.Panel implements ext.util.IFocusableContainer {
  /**
   * @param config @inheritDoc
   */
  public function Menu(config:ext.menu.Menu = null) {
    super();
  }

  /**
   * @default false
   */
  override public native function get focusOnToFront():Boolean;

  /**
   * @private
   */
  override public native function set focusOnToFront(value:Boolean):void;

  [ExtConfig]
  /**
   * The default <a href="#!/api/Ext.util.Positionable-method-getAlignToXY" rel="Ext.util.Positionable-method-getAlignToXY" class="docClass">Ext.dom.Element#getAlignToXY</a> anchor position value for this menu
   * relative to its owner. Used in conjunction with <a href="#!/api/Ext.menu.Menu-method-showBy" rel="Ext.menu.Menu-method-showBy" class="docClass">showBy</a>.
   * @default "tl-bl?"
   */
  override public native function get defaultAlign():String;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set defaultAlign(value:String):void;

  [ExtConfig]
  /**
   * A Menu configured as <code>floating: true</code> (the default) will be rendered as an
   * absolutely positioned,
   * <a href="#!/api/Ext.Component-cfg-floating" rel="Ext.Component-cfg-floating" class="docClass">floating</a> <a href="#!/api/Ext.Component" rel="Ext.Component" class="docClass">Component</a>. If
   * configured as <code>floating: false</code>, the Menu may be used as a child item of another
   * <a href="#!/api/Ext.container.Container" rel="Ext.container.Container" class="docClass">Container</a>.
   * @default true
   */
  override public native function get floating():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set floating(value:Boolean):void;

  [ExtConfig]
  /**
   * True to initially render the Menu as hidden, requiring to be shown manually.
   *
   * <p>Defaults to <code>true</code> when <code>floating: true</code>, and defaults to <code>false</code> when <code>floating: false</code>.</p>
   * @default true
   * @see #setHidden()
   */
  override public native function get hidden():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  override public native function set hidden(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The minimum width of the Menu. The default minWidth only applies when the
   * <a href="#!/api/Ext.menu.Menu-cfg-floating" rel="Ext.menu.Menu-cfg-floating" class="docClass">floating</a> config is true.
   * @default 120
   * @see #getMinWidth()
   * @see #setMinWidth()
   */
  override public native function get minWidth():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  override public native function set minWidth(value:Number):void;

  /**
   * The name of the element
   * that FocusableContainer should bind its keyboard handler to. Similar to ariaEl,
   * this name is resolved to the Ext.dom.Element instance after rendering.
   * @default "el"
   */
  public native function get focusableContainerEl():*;

  /**
   * @private
   */
  public native function set focusableContainerEl(value:*):void;

  /**
   * <code>true</code> in this class to identify an object as an instantiated Menu, or subclass thereof.
   * @default true
   */
  public native function get isMenu():Boolean;

  /**
   * @private
   */
  public native function set isMenu(value:Boolean):void;

  /**
   * The parent Menu of this Menu.
   */
  public native function get parentMenu():ext.menu.Menu;

  /**
   * @private
   */
  public native function set parentMenu(value:ext.menu.Menu):void;

  [ExtConfig]
  /**
   * DOM tabIndex attribute to set on the
   * active Focusable child of this container when using the "Roaming tabindex"
   * technique.
   * @default 0
   */
  public native function get activeChildTabIndex():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set activeChildTabIndex(value:Number):void;

  [ExtConfig]
  /**
   * True to allow multiple menus to be displayed at the same time.
   * @default false
   */
  public native function get allowOtherMenus():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set allowOtherMenus(value:Boolean):void;

  [ExtConfig]
  /**
   * Enable or disable
   * navigation with arrow keys for this FocusableContainer. This option may
   * be useful with nested FocusableContainers such as Grid column headers,
   * when only the root container should handle keyboard events.
   * @default true
   */
  public native function get enableFocusableContainer():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set enableFocusableContainer(value:Boolean):void;

  [Deprecated(message = "Intra-menu key navigation is always enabled.\n", since = "5.1.0")]
  [ExtConfig]
  /**
   * @default true
   */
  public native function get enableKeyNav():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set enableKeyNav(value:Boolean):void;

  [ExtConfig]
  /**
   * True to ignore clicks on any item in this menu that is a parent item (displays a submenu)
   * so that the submenu is not dismissed when clicking the parent item.
   * @default false
   */
  public native function get ignoreParentClicks():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set ignoreParentClicks(value:Boolean):void;

  [ExtConfig]
  /**
   * DOM tabIndex attribute to set on
   * inactive Focusable children of this container when using the "Roaming tabindex"
   * technique. This value rarely needs to be changed from its default.
   * @default -1
   */
  public native function get inactiveChildTabIndex():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set inactiveChildTabIndex(value:Number):void;

  [ExtConfig]
  /**
   * The delay in ms as to how long the framework should wait before firing a mouseleave event.
   * This allows submenus not to be collapsed while hovering other menu items.
   *
   * <p>Defaults to 100</p>
   * @default 100
   */
  public native function get mouseLeaveDelay():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set mouseLeaveDelay(value:Number):void;

  [ExtConfig]
  /**
   * True to remove the incised line down the left side of the menu and to not indent general Component items.
   *
   * <p><a href="#!/api/Ext.menu.Item" rel="Ext.menu.Item" class="docClass">MenuItem</a>s will <em>always</em> have space at their start for an icon. With the <code>plain</code> setting,
   * non <a href="#!/api/Ext.menu.Item" rel="Ext.menu.Item" class="docClass">MenuItem</a> child components will not be indented to line up.</p>
   *
   * <p>Basically, <code>plain:true</code> makes a Menu behave more like a regular <a href="#!/api/Ext.layout.container.HBox" rel="Ext.layout.container.HBox" class="docClass">HBox layout</a>
   * <a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Panel</a> which just has the same background as a Menu.</p>
   *
   * <p>See also the <a href="#!/api/Ext.menu.Menu-cfg-showSeparator" rel="Ext.menu.Menu-cfg-showSeparator" class="docClass">showSeparator</a> config.</p>
   * @default false
   */
  public native function get plain():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set plain(value:Boolean):void;

  [ExtConfig]
  /**
   * True to show the icon separator.
   * @default true
   */
  public native function get showSeparator():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set showSeparator(value:Boolean):void;

  /**
   * @inheritDoc
   */
  public native function get allowFocusingDisabledChildren():Boolean;

  /**
   * @inheritDoc
   */
  public native function set allowFocusingDisabledChildren(value:Boolean):void;

  /**
   * @param inner Pass <code>true</code> to return <code>inheritedStateInner</code> instead
   * of the normal <code>inheritedState</code> object. This is only needed internally and should
   * not be passed by user code.
   * @default false
   */
  override public native function getInherited(inner:Boolean = false):Object;

  /**
   * @param item The child item to test for focusability.
   * Returns whether a menu item can be activated or not.
   * @return <code>true</code> if the passed item is focusable.
   */
  public native function canActivateItem(item:ext.Component):Boolean;

  /**
   * Deactivates the current active item on the menu, if one exists.
   */
  public native function deactivateActiveItem():void;
}
}