package ext.mixin {
import ext.Base;
import ext.data.Model;

[Uses("ext.Base")]
[Native("Ext.mixin.Dirty", require)]
/**
 * Fires when a change in the object's <a href="#!/api/Ext.mixin.Dirty-cfg-dirty" rel="Ext.mixin.Dirty-cfg-dirty" class="docClass">dirty</a> state is detected.
 *
 * <p><strong>Note:</strong> In order for this event to fire, the class that mixes in this mixin
 * must be <code><a href="#!/api/Ext.mixin.Observable" rel="Ext.mixin.Observable" class="docClass">Observable</a></code>.</p>
 * @eventType ext.mixin.events.Dirty_dirtyEvent.DIRTY_CHANGE
 */
[Event(name = 'onDirtyChange', type = 'ext.mixin.events.Dirty_dirtyEvent')]
/**
 * This mixin provides a <code>dirty</code> config that tracks the modified state of an object. If
 * the class using this mixin is <a href="#!/api/Ext.mixin.Observable" rel="Ext.mixin.Observable" class="docClass">observable</a>, changes to the
 * <code>dirty</code> config will fire the <code>dirtychange</code> event.
 */
public class Dirty extends ext.Base implements ext.mixin.IDirty {
  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * This config property describes the modified state of this object. In most
   * cases this config's value is maintained by the object and should be considered
   * readonly. The class implementor should be the only one to call the setter.
   * @see #getDirty()
   * @see #setDirty()
   */
  public native function get dirty():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set dirty(value:Boolean):void;

  [ExtConfig]
  /**
   * This config property indicates that the <code>dirty</code> state of this object should be
   * ignored. Because this capability is mixed in at a class level, this config can
   * be helpful when some instances do not participate in dirty state tracking.
   *
   * <p>This option should be set at construction time. When set to <code>true</code>, the object
   * will always have <code>dirty</code> value of <code>false</code>.</p>
   * @default false
   */
  public native function get ignoreDirty():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set ignoreDirty(value:Boolean):void;

  [ExtConfig]
  /**
   * Set this config at construction time (or on the class body) to automatically set
   * the <code>dirty</code> state based on the records passed to <code>trackRecordState</code>.
   *
   * <p>This config defaults to <code>true</code> but only has an effect when the record tracking
   * methods are called (<code>trackRecordState</code>, <code>untrackRecordState</code> and <code>clearRecordStates</code>).</p>
   * @default true
   */
  public native function get recordStateIsDirtyState():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set recordStateIsDirtyState(value:Boolean):void;

  /**
   * Returns the value of <a href="#!/api/Ext.mixin.Dirty-cfg-dirty" rel="Ext.mixin.Dirty-cfg-dirty" class="docClass">dirty</a>.
   */
  public native function getDirty():Boolean;

  /**
   * Returns <code>true</code> if this object is <code>dirty</code>.
   */
  public native function isDirty():void;

  /**
   * Sets the value of <a href="#!/api/Ext.mixin.Dirty-cfg-dirty" rel="Ext.mixin.Dirty-cfg-dirty" class="docClass">dirty</a>.
   * @param dirty The new value.
   */
  public native function setDirty(dirty:Boolean):void;

  /**
   * Clears all record state tracking. This state is maintained by <code>trackRecordState</code>
   * and <code>untrackRecordState</code>.
   */
  protected native function clearRecordStates():void;

  /**
   * This method is called to track a given record in the total number of dirty records
   * (modified, created or dropped). See <code>untrackRecordState</code> and <code>clearRecordStates</code>.
   * @param record The record to track.
   * @param initial Pass <code>true</code> the first time a record is introduced.
   * @return Returns <code>true</code> if the state of dirty records has changed.
   */
  protected native function trackRecordState(record:ext.data.Model, initial:Boolean):Boolean;

  /**
   * This method is called to remove the tracking of a given record from the total number
   * of dirty records (modified, created or dropped). The record passed to this method
   * must have been previously passed to <code>trackRecordState</code>.
   * @param record The record to stop tracking.
   * @return Returns <code>true</code> if the state of dirty records has changed.
   */
  protected native function untrackRecordState(record:ext.data.Model):Boolean;
}
}