package ext.panel {
import ext.util.IFocusableContainer;

[Uses("ext.panel.PanelBar")]
[Uses("ext.util.FocusableContainer")]
[Uses("ext.container.Container")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.Base")]
[Uses("ext.mixin.Container")]
[Uses("ext.Mixin")]
[Uses("ext.Component")]
[Uses("ext.util.Positionable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Animate")]
[Uses("ext.util.Floating")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.state.Stateful")]
[Native("Ext.panel.Header", require)]
/**
 * Fires when the header is clicked. This event will not be fired
 * if the click was on a <a href="#!/api/Ext.panel.Tool" rel="Ext.panel.Tool" class="docClass">Ext.panel.Tool</a>
 * @eventType ext.panel.events.PanelHeader_eEvent.CLICK
 */
[Event(name = 'onClick', type = 'ext.panel.events.PanelHeader_eEvent')]
/**
 * Fires when the header is double clicked. This event will not
 * be fired if the click was on a <a href="#!/api/Ext.panel.Tool" rel="Ext.panel.Tool" class="docClass">Ext.panel.Tool</a>
 * @eventType ext.panel.events.PanelHeader_eEvent.DBLCLICK
 */
[Event(name = 'onDblclick', type = 'ext.panel.events.PanelHeader_eEvent')]
/**
 * Simple header class which is used for on <a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a> and <a href="#!/api/Ext.window.Window" rel="Ext.window.Window" class="docClass">Ext.window.Window</a>.
 */
public class PanelHeader extends ext.panel.PanelBar implements ext.util.IFocusableContainer {
  /**
   * @param config @inheritDoc
   */
  public function PanelHeader(config:ext.panel.PanelHeader = null) {
    super();
  }

  /**
   * The name of the element
   * that FocusableContainer should bind its keyboard handler to. Similar to ariaEl,
   * this name is resolved to the Ext.dom.Element instance after rendering.
   * @default "el"
   */
  public native function get focusableContainerEl():*;

  /**
   * @private
   */
  public native function set focusableContainerEl(value:*):void;

  /**
   * <code>true</code> in this class to identify an object as an instantiated Header, or subclass thereof.
   * @default true
   */
  public native function get isHeader():Boolean;

  /**
   * @private
   */
  public native function set isHeader(value:Boolean):void;

  [ExtConfig]
  /**
   * DOM tabIndex attribute to set on the
   * active Focusable child of this container when using the "Roaming tabindex"
   * technique.
   * @default 0
   */
  public native function get activeChildTabIndex():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set activeChildTabIndex(value:Number):void;

  [ExtConfig]
  /**
   * Set this to <code>true</code> to enable focusing
   * disabled children via keyboard.
   */
  public native function get allowFocusingDisabledChildren():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set allowFocusingDisabledChildren(value:Boolean):void;

  [ExtConfig]
  /**
   * Enable or disable
   * navigation with arrow keys for this FocusableContainer. This option may
   * be useful with nested FocusableContainers such as Grid column headers,
   * when only the root container should handle keyboard events.
   * @default true
   */
  public native function get enableFocusableContainer():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set enableFocusableContainer(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * A numeric unicode character code to use as the icon.  The default font-family
   * for glyphs can be set globally using
   * <a href="#!/api/Ext.app.Application-cfg-glyphFontFamily" rel="Ext.app.Application-cfg-glyphFontFamily" class="docClass">glyphFontFamily</a> application
   * config or the <a href="#!/api/Ext-method-setGlyphFontFamily" rel="Ext-method-setGlyphFontFamily" class="docClass">Ext.setGlyphFontFamily()</a> method.
   * It is initially set to <code>'Pictos'</code>.
   *
   * <p>The following shows how to set the glyph using the font icons provided in the
   * SDK (assuming the font-family has been configured globally):</p>
   *
   * <pre><code>// assumes the glyphFontFamily is "Pictos"
   * glyph: 'x48'       // the "home" icon (H character)
   *
   * // assumes the glyphFontFamily is "Pictos"
   * glyph: 72          // The "home" icon (H character)
   *
   * // assumes the glyphFontFamily is "Pictos"
   * glyph: 'H'         // the "home" icon
   * </code></pre>
   *
   * <p>Alternatively, this config option accepts a string with the charCode and
   * font-family separated by the <code>&#64;</code> symbol.</p>
   *
   * <pre><code>// using Font Awesome
   * glyph: 'xf015&#64;FontAwesome'     // the "home" icon
   *
   * // using Pictos
   * glyph: 'H&#64;Pictos'              // the "home" icon
   * </code></pre>
   *
   * <p>Depending on the theme you're using, you may need include the font icon
   * packages in your application in order to use the icons included in the
   * SDK.  For more information see:</p>
   *
   * <ul>
   * <li><a href="http://fortawesome.github.io/Font-Awesome/cheatsheet/">Font Awesome icons</a></li>
   * <li><a href="http://docs.sencha.com/extjs/6.0/core_concepts/font_ext.html">Pictos icons</a></li>
   * <li><a href="http://docs.sencha.com/extjs/6.0/core_concepts/theming.html">Theming Guide</a></li>
   * </ul>
   * @see #getGlyph()
   * @see #setGlyph()
   */
  public native function get glyph():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set glyph(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Path to an image to use as an icon.
   *
   * <p>For instructions on how you can use icon fonts including those distributed in
   * the SDK see <a href="#!/api/Ext.panel.Header-cfg-iconCls" rel="Ext.panel.Header-cfg-iconCls" class="docClass">iconCls</a>.</p>
   * @see #getIcon()
   * @see #setIcon()
   */
  public native function get icon():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set icon(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The side of the title to render the icon.
   * @default 'left'
   * @see #getIconAlign()
   * @see #setIconAlign()
   */
  public native function get iconAlign():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set iconAlign(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * One or more space separated CSS classes to be applied to the icon element.<br/>
   * The CSS rule(s) applied should specify a background image to be used as the
   * icon.
   *
   * <p>An example of specifying a custom icon class would be something like:</p>
   *
   * <pre><code>// specify the property in the config for the class:
   * iconCls: 'my-home-icon'
   *
   * // css rule specifying the background image to be used as the icon image:
   * .my-home-icon {
   *     background-image: url(../images/my-home-icon.gif) !important;
   * }
   * </code></pre>
   *
   * <p>In addition to specifying your own classes, you can use the font icons
   * provided in the SDK using the following syntax:</p>
   *
   * <pre><code>// using Font Awesome
   * iconCls: 'x-fa fa-home'
   *
   * // using Pictos
   * iconCls: 'pictos pictos-home'
   * </code></pre>
   *
   * <p>Depending on the theme you're using, you may need include the font icon
   * packages in your application in order to use the icons included in the
   * SDK.  For more information see:</p>
   *
   * <ul>
   * <li><a href="http://fortawesome.github.io/Font-Awesome/cheatsheet/">Font Awesome icons</a></li>
   * <li><a href="http://docs.sencha.com/extjs/6.0/core_concepts/font_ext.html">Pictos icons</a></li>
   * <li><a href="http://docs.sencha.com/extjs/6.0/core_concepts/theming.html">Theming Guide</a></li>
   * </ul>
   * @see #getIconCls()
   * @see #setIconCls()
   */
  public native function get iconCls():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set iconCls(value:String):void;

  [ExtConfig]
  /**
   * DOM tabIndex attribute to set on
   * inactive Focusable children of this container when using the "Roaming tabindex"
   * technique. This value rarely needs to be changed from its default.
   * @default -1
   */
  public native function get inactiveChildTabIndex():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set inactiveChildTabIndex(value:Number):void;

  [ExtConfig]
  /**
   * The index at which the any <a href="#!/api/Ext.panel.Header-cfg-items" rel="Ext.panel.Header-cfg-items" class="docClass">items</a> will be inserted into the Header's
   * items collection.  By default this will effectively be the <code>1</code> position
   * placing the items following the panel <a href="#!/api/Ext.panel.Panel-cfg-title" rel="Ext.panel.Panel-cfg-title" class="docClass">title</a>.
   *
   * <p>Set to <code>0</code> to have the items <a href="#!/api/Ext.panel.Header-method-insert" rel="Ext.panel.Header-method-insert" class="docClass">inserted</a> before the panel title.</p>
   *
   * <pre><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a>', {
   *     title: 'Hello',
   *     width: 200,
   *     html: '&lt;p&gt;World!&lt;/p&gt;',
   *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
   *     tools: [{
   *         type: 'pin'
   *     }],
   *     header: {
   *         //itemPosition: 0,  // before panel title
   *         //itemPosition: 1, // after panel title
   *         //itemPosition: 2, // after pin tool
   *         items: [{
   *             xtype: 'button',
   *             text: 'Header Button'
   *         }]
   *     }
   * });
   * </code></pre>
   */
  public native function get itemPosition():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set itemPosition(value:Number):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The title text or config object for the <a href="#!/api/Ext.panel.Title" rel="Ext.panel.Title" class="docClass">Title</a> component.
   * @see #getTitle()
   * @see #setTitle()
   */
  public native function get title():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set title(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The alignment of the title text within the available space between the
   * icon and the tools.
   * @default 'left'
   * @see #getTitleAlign()
   * @see #setTitleAlign()
   */
  public native function get titleAlign():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set titleAlign(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The ordinal position among the header items (tools and other components specified using the <a href="#!/api/Ext.panel.Header-cfg-items" rel="Ext.panel.Header-cfg-items" class="docClass">items</a> config)
   * at which the title component is inserted. See <a href="#!/api/Ext.panel.Panel-cfg-header" rel="Ext.panel.Panel-cfg-header" class="docClass">Panel's header config</a>.
   *
   * <p>If not specified, the title is inserted after any <a href="#!/api/Ext.panel.Header-cfg-items" rel="Ext.panel.Header-cfg-items" class="docClass">items</a>, but <em>before</em> any <a href="#!/api/Ext.panel.Panel-cfg-tools" rel="Ext.panel.Panel-cfg-tools" class="docClass">Ext.panel.Panel.tools</a>.</p>
   *
   * <p>Note that if an <a href="#!/api/Ext.panel.Header-cfg-icon" rel="Ext.panel.Header-cfg-icon" class="docClass">icon</a> or <a href="#!/api/Ext.panel.Header-cfg-iconCls" rel="Ext.panel.Header-cfg-iconCls" class="docClass">iconCls</a> has been configured, then the icon component will be the
   * first item before all specified tools or <a href="#!/api/Ext.panel.Header-cfg-items" rel="Ext.panel.Header-cfg-items" class="docClass">items</a>. This configuration does not include the icon.</p>
   * @default 0
   * @see #getTitlePosition()
   * @see #setTitlePosition()
   */
  public native function get titlePosition():Number;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set titlePosition(value:Number):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The rotation of the header's title text.  Can be one of the following values:
   *
   * <ul>
   * <li><code>'default'</code> - use the default rotation, depending on the dock position of the header</li>
   * <li><code>0</code> - no rotation</li>
   * <li><code>1</code> - rotate 90deg clockwise</li>
   * <li><code>2</code> - rotate 90deg counter-clockwise</li>
   * </ul>
   *
   *
   * <p>The default behavior of this config depends on the dock position of the header:</p>
   *
   * <ul>
   * <li><code>'top'</code> or <code>'bottom'</code> - <code>0</code></li>
   * <li><code>'right'</code> - <code>1</code></li>
   * <li><code>'left'</code> - <code>1</code></li>
   * </ul>
   * @default 'default'
   * @see #getTitleRotation()
   * @see #setTitleRotation()
   */
  public native function get titleRotation():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set titleRotation(value:*):void;

  /**
   * Add a tool to the header
   */
  public native function addTool(tool:Object):void;

  /**
   * Returns the value of <a href="#!/api/Ext.panel.Header-cfg-glyph" rel="Ext.panel.Header-cfg-glyph" class="docClass">glyph</a>.
   */
  public native function getGlyph():*;

  /**
   * Returns the value of <a href="#!/api/Ext.panel.Header-cfg-icon" rel="Ext.panel.Header-cfg-icon" class="docClass">icon</a>.
   */
  public native function getIcon():String;

  /**
   * Returns the value of <a href="#!/api/Ext.panel.Header-cfg-iconAlign" rel="Ext.panel.Header-cfg-iconAlign" class="docClass">iconAlign</a>.
   */
  public native function getIconAlign():String;

  /**
   * Returns the value of <a href="#!/api/Ext.panel.Header-cfg-iconCls" rel="Ext.panel.Header-cfg-iconCls" class="docClass">iconCls</a>.
   */
  public native function getIconCls():String;

  /**
   * Returns the value of <a href="#!/api/Ext.panel.Header-cfg-title" rel="Ext.panel.Header-cfg-title" class="docClass">title</a>.
   */
  public native function getTitle():*;

  /**
   * Returns the value of <a href="#!/api/Ext.panel.Header-cfg-titleAlign" rel="Ext.panel.Header-cfg-titleAlign" class="docClass">titleAlign</a>.
   */
  public native function getTitleAlign():String;

  /**
   * Returns the value of <a href="#!/api/Ext.panel.Header-cfg-titlePosition" rel="Ext.panel.Header-cfg-titlePosition" class="docClass">titlePosition</a>.
   */
  public native function getTitlePosition():Number;

  /**
   * Returns the value of <a href="#!/api/Ext.panel.Header-cfg-titleRotation" rel="Ext.panel.Header-cfg-titleRotation" class="docClass">titleRotation</a>.
   */
  public native function getTitleRotation():*;

  /**
   * Gets the tools for this header.
   * @return The tools
   */
  public native function getTools():Array;

  /**
   * Sets the value of <a href="#!/api/Ext.panel.Header-cfg-glyph" rel="Ext.panel.Header-cfg-glyph" class="docClass">glyph</a>.
   * @param glyph The new value.
   */
  public native function setGlyph(glyph:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.panel.Header-cfg-icon" rel="Ext.panel.Header-cfg-icon" class="docClass">icon</a>.
   * @param icon The new value.
   */
  public native function setIcon(icon:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.panel.Header-cfg-iconAlign" rel="Ext.panel.Header-cfg-iconAlign" class="docClass">iconAlign</a>.
   * @param iconAlign The new value.
   */
  public native function setIconAlign(iconAlign:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.panel.Header-cfg-iconCls" rel="Ext.panel.Header-cfg-iconCls" class="docClass">iconCls</a>.
   * @param iconCls The new value.
   */
  public native function setIconCls(iconCls:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.panel.Header-cfg-title" rel="Ext.panel.Header-cfg-title" class="docClass">title</a>.
   * @param title The new value.
   */
  public native function setTitle(title:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.panel.Header-cfg-titleAlign" rel="Ext.panel.Header-cfg-titleAlign" class="docClass">titleAlign</a>.
   * @param titleAlign The new value.
   */
  public native function setTitleAlign(titleAlign:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.panel.Header-cfg-titlePosition" rel="Ext.panel.Header-cfg-titlePosition" class="docClass">titlePosition</a>.
   * @param titlePosition The new value.
   */
  public native function setTitlePosition(titlePosition:Number):void;

  /**
   * Sets the value of <a href="#!/api/Ext.panel.Header-cfg-titleRotation" rel="Ext.panel.Header-cfg-titleRotation" class="docClass">titleRotation</a>.
   * @param titleRotation The new value.
   */
  public native function setTitleRotation(titleRotation:*):void;
}
}