package ext.panel {
import ext.Base;
import ext.dom.Element;
import js.HTMLElement;

[Uses("ext.Base")]
[Native("Ext.panel.Proxy", require)]
[ExcludeClass]
/**
 * A custom drag proxy implementation specific to <a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a>s. This class
 * is primarily used internally for the Panel's drag drop implementation, and
 * should never need to be created directly.
 */
public class PanelProxy extends ext.Base {
  /**
   * Creates new panel proxy.
   * @param panel The <a href="#!/api/Ext.panel.Panel" rel="Ext.panel.Panel" class="docClass">Ext.panel.Panel</a> to proxy for
   * @param config Config object
   */
  public function PanelProxy(panel:ext.panel.Panel, config:ext.panel.PanelProxy = null) {
    super();
  }

  public native function get panel():ext.panel.Panel;

  /**
   * @private
   */
  public native function set panel(value:ext.panel.Panel):void;

  [ExtConfig]
  /**
   * True to insert a placeholder proxy element while dragging the panel, false to drag with no proxy.
   * Most Panels are not absolute positioned and therefore we need to reserve this space.
   * @default true
   */
  public native function get insertProxy():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set insertProxy(value:Boolean):void;

  [ExtConfig]
  /**
   * True to move the panel to the dragged position when dropped
   * @default true
   */
  public native function get moveOnDrag():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set moveOnDrag(value:Boolean):void;

  /**
   * Gets the proxy's element
   * @return The proxy's element
   */
  public native function getEl():ext.dom.Element;

  /**
   * Gets the proxy's ghost Panel
   * @return The proxy's ghost Panel
   */
  public native function getGhost():ext.panel.Panel;

  /**
   * Gets the proxy element. This is the element that represents where the
   * Panel was before we started the drag operation.
   * @return The proxy's element
   */
  public native function getProxy():ext.dom.Element;

  /**
   * Hides the proxy
   */
  public native function hide():void;

  /**
   * Moves the proxy to a different position in the DOM.  This is typically
   * called while dragging the Panel to keep the proxy sync'd to the Panel's
   * location.
   * @param parentNode The proxy's parent DOM node
   * @param before The sibling node before which the
   * proxy should be inserted. Defaults to the parent's last child if not
   * specified.
   */
  public native function moveProxy(parentNode:js.HTMLElement, before:js.HTMLElement = null):void;

  /**
   * Shows the proxy
   */
  public native function show():void;
}
}