package ext.panel {
import ext.Component;

[Uses("ext.Component")]
[Uses("ext.util.Positionable")]
[Uses("ext.Base")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Animate")]
[Uses("ext.util.Floating")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.state.Stateful")]
[Native("Ext.panel.Title", require)]
/**
 * A basic title component for a Panel Header
 */
public class PanelTitle extends ext.Component {
  /**
   * @param config @inheritDoc
   */
  public function PanelTitle(config:ext.panel.PanelTitle = null) {
    super();
  }

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * A numeric unicode character code to use as the icon.  The default font-family
   * for glyphs can be set globally using
   * <a href="#!/api/Ext.app.Application-cfg-glyphFontFamily" rel="Ext.app.Application-cfg-glyphFontFamily" class="docClass">glyphFontFamily</a> application
   * config or the <a href="#!/api/Ext-method-setGlyphFontFamily" rel="Ext-method-setGlyphFontFamily" class="docClass">Ext.setGlyphFontFamily()</a> method.
   * It is initially set to <code>'Pictos'</code>.
   *
   * <p>The following shows how to set the glyph using the font icons provided in the
   * SDK (assuming the font-family has been configured globally):</p>
   *
   * <pre><code>// assumes the glyphFontFamily is "Pictos"
   * glyph: 'x48'       // the "home" icon (H character)
   *
   * // assumes the glyphFontFamily is "Pictos"
   * glyph: 72          // The "home" icon (H character)
   *
   * // assumes the glyphFontFamily is "Pictos"
   * glyph: 'H'         // the "home" icon
   * </code></pre>
   *
   * <p>Alternatively, this config option accepts a string with the charCode and
   * font-family separated by the <code>&#64;</code> symbol.</p>
   *
   * <pre><code>// using Font Awesome
   * glyph: 'xf015&#64;FontAwesome'     // the "home" icon
   *
   * // using Pictos
   * glyph: 'H&#64;Pictos'              // the "home" icon
   * </code></pre>
   *
   * <p>Depending on the theme you're using, you may need include the font icon
   * packages in your application in order to use the icons included in the
   * SDK.  For more information see:</p>
   *
   * <ul>
   * <li><a href="http://fortawesome.github.io/Font-Awesome/cheatsheet/">Font Awesome icons</a></li>
   * <li><a href="http://docs.sencha.com/extjs/6.0/core_concepts/font_ext.html">Pictos icons</a></li>
   * <li><a href="http://docs.sencha.com/extjs/6.0/core_concepts/theming.html">Theming Guide</a></li>
   * </ul>
   * @see #getGlyph()
   * @see #setGlyph()
   */
  public native function get glyph():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set glyph(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * Path to an image to use as an icon.
   *
   * <p>For instructions on how you can use icon fonts including those distributed in
   * the SDK see <a href="#!/api/Ext.panel.Title-cfg-iconCls" rel="Ext.panel.Title-cfg-iconCls" class="docClass">iconCls</a>.</p>
   * @see #getIcon()
   * @see #setIcon()
   */
  public native function get icon():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set icon(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * alignment of the icon
   * @default 'left'
   * @see #getIconAlign()
   * @see #setIconAlign()
   */
  public native function get iconAlign():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set iconAlign(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * One or more space separated CSS classes to be applied to the icon element.<br/>
   * The CSS rule(s) applied should specify a background image to be used as the
   * icon.
   *
   * <p>An example of specifying a custom icon class would be something like:</p>
   *
   * <pre><code>// specify the property in the config for the class:
   * iconCls: 'my-home-icon'
   *
   * // css rule specifying the background image to be used as the icon image:
   * .my-home-icon {
   *     background-image: url(../images/my-home-icon.gif) !important;
   * }
   * </code></pre>
   *
   * <p>In addition to specifying your own classes, you can use the font icons
   * provided in the SDK using the following syntax:</p>
   *
   * <pre><code>// using Font Awesome
   * iconCls: 'x-fa fa-home'
   *
   * // using Pictos
   * iconCls: 'pictos pictos-home'
   * </code></pre>
   *
   * <p>Depending on the theme you're using, you may need include the font icon
   * packages in your application in order to use the icons included in the
   * SDK.  For more information see:</p>
   *
   * <ul>
   * <li><a href="http://fortawesome.github.io/Font-Awesome/cheatsheet/">Font Awesome icons</a></li>
   * <li><a href="http://docs.sencha.com/extjs/6.0/core_concepts/font_ext.html">Pictos icons</a></li>
   * <li><a href="http://docs.sencha.com/extjs/6.0/core_concepts/theming.html">Theming Guide</a></li>
   * </ul>
   * @see #getIconCls()
   * @see #setIconCls()
   */
  public native function get iconCls():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set iconCls(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The rotation of the header's title text.  Can be one of the following values:
   *
   * <ul>
   * <li><code>'default'</code> - use the default rotation, depending on the dock position of the header</li>
   * <li><code>0</code> - no rotation</li>
   * <li><code>1</code> - rotate 90deg clockwise</li>
   * <li><code>2</code> - rotate 90deg counter-clockwise</li>
   * </ul>
   *
   *
   * <p>The default behavior of this config depends on the dock position of the header:</p>
   *
   * <ul>
   * <li><code>'top'</code> or <code>'bottom'</code> - <code>0</code></li>
   * <li><code>'right'</code> - <code>1</code></li>
   * <li><code>'left'</code> - <code>1</code></li>
   * </ul>
   * @see #getRotation()
   * @see #setRotation()
   */
  public native function get rotation():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set rotation(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The title's text (can contain html tags/entities)
   * @see #getText()
   * @see #setText()
   */
  public native function get text():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set text(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The alignment of the title text within the available space between the
   * icon and the tools.
   * @default 'left'
   * @see #getTextAlign()
   * @see #setTextAlign()
   */
  public native function get textAlign():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set textAlign(value:String):void;

  /**
   * Returns the value of <a href="#!/api/Ext.panel.Title-cfg-glyph" rel="Ext.panel.Title-cfg-glyph" class="docClass">glyph</a>.
   */
  public native function getGlyph():Object;

  /**
   * Returns the value of <a href="#!/api/Ext.panel.Title-cfg-icon" rel="Ext.panel.Title-cfg-icon" class="docClass">icon</a>.
   */
  public native function getIcon():Object;

  /**
   * Returns the value of <a href="#!/api/Ext.panel.Title-cfg-iconAlign" rel="Ext.panel.Title-cfg-iconAlign" class="docClass">iconAlign</a>.
   */
  public native function getIconAlign():String;

  /**
   * Returns the value of <a href="#!/api/Ext.panel.Title-cfg-iconCls" rel="Ext.panel.Title-cfg-iconCls" class="docClass">iconCls</a>.
   */
  public native function getIconCls():Object;

  /**
   * Returns the value of <a href="#!/api/Ext.panel.Title-cfg-rotation" rel="Ext.panel.Title-cfg-rotation" class="docClass">rotation</a>.
   */
  public native function getRotation():Object;

  /**
   * Returns the value of <a href="#!/api/Ext.panel.Title-cfg-text" rel="Ext.panel.Title-cfg-text" class="docClass">text</a>.
   */
  public native function getText():String;

  /**
   * Returns the value of <a href="#!/api/Ext.panel.Title-cfg-textAlign" rel="Ext.panel.Title-cfg-textAlign" class="docClass">textAlign</a>.
   */
  public native function getTextAlign():Object;

  /**
   * Sets the value of <a href="#!/api/Ext.panel.Title-cfg-glyph" rel="Ext.panel.Title-cfg-glyph" class="docClass">glyph</a>.
   * @param glyph The new value.
   */
  public native function setGlyph(glyph:Object):void;

  /**
   * Sets the value of <a href="#!/api/Ext.panel.Title-cfg-icon" rel="Ext.panel.Title-cfg-icon" class="docClass">icon</a>.
   * @param icon The new value.
   */
  public native function setIcon(icon:Object):void;

  /**
   * Sets the value of <a href="#!/api/Ext.panel.Title-cfg-iconAlign" rel="Ext.panel.Title-cfg-iconAlign" class="docClass">iconAlign</a>.
   * @param iconAlign The new value.
   */
  public native function setIconAlign(iconAlign:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.panel.Title-cfg-iconCls" rel="Ext.panel.Title-cfg-iconCls" class="docClass">iconCls</a>.
   * @param iconCls The new value.
   */
  public native function setIconCls(iconCls:Object):void;

  /**
   * Sets the value of <a href="#!/api/Ext.panel.Title-cfg-rotation" rel="Ext.panel.Title-cfg-rotation" class="docClass">rotation</a>.
   * @param rotation The new value.
   */
  public native function setRotation(rotation:Object):void;

  /**
   * Sets the value of <a href="#!/api/Ext.panel.Title-cfg-text" rel="Ext.panel.Title-cfg-text" class="docClass">text</a>.
   * @param text The new value.
   */
  public native function setText(text:String):void;

  /**
   * Sets the value of <a href="#!/api/Ext.panel.Title-cfg-textAlign" rel="Ext.panel.Title-cfg-textAlign" class="docClass">textAlign</a>.
   * @param textAlign The new value.
   */
  public native function setTextAlign(textAlign:Object):void;
}
}