package ext.picker {
import ext.Component;

[Uses("ext.Component")]
[Uses("ext.util.Positionable")]
[Uses("ext.Base")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Animate")]
[Uses("ext.util.Floating")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.state.Stateful")]
[Native("Ext.picker.Month", require)]
/**
 * Fires when the cancel button is pressed.
 * @eventType ext.picker.events.MonthPickerEvent.CANCEL_CLICK
 */
[Event(name = 'onCancelClick', type = 'ext.picker.events.MonthPickerEvent')]
/**
 * Fires when a month is clicked.
 * @eventType ext.picker.events.MonthPicker_valueEvent.MONTH_CLICK
 */
[Event(name = 'onMonthClick', type = 'ext.picker.events.MonthPicker_valueEvent')]
/**
 * Fires when a month is clicked.
 * @eventType ext.picker.events.MonthPicker_valueEvent.MONTH_DBLCLICK
 */
[Event(name = 'onMonthDblclick', type = 'ext.picker.events.MonthPicker_valueEvent')]
/**
 * Fires when the ok button is pressed.
 * @eventType ext.picker.events.MonthPicker_valueEvent.OK_CLICK
 */
[Event(name = 'onOkClick', type = 'ext.picker.events.MonthPicker_valueEvent')]
/**
 * Fires when a month/year is selected.
 * @eventType ext.picker.events.MonthPicker_valueEvent.SELECT
 */
[Event(name = 'onSelect', type = 'ext.picker.events.MonthPicker_valueEvent')]
/**
 * Fires when a year is clicked.
 * @eventType ext.picker.events.MonthPicker_valueEvent.YEAR_CLICK
 */
[Event(name = 'onYearClick', type = 'ext.picker.events.MonthPicker_valueEvent')]
/**
 * Fires when a year is clicked.
 * @eventType ext.picker.events.MonthPicker_valueEvent.YEAR_DBLCLICK
 */
[Event(name = 'onYearDblclick', type = 'ext.picker.events.MonthPicker_valueEvent')]
/**
 * A month / year picker component. This class is used by the
 * <a href="#!/api/Ext.picker.Date" rel="Ext.picker.Date" class="docClass">Date picker</a> to allow browsing and selection of year and
 * months combinations, but may also be used as a standalone component.
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>({
 *     xtype: 'monthpicker',
 *     renderTo: document.body,
 *     value: new Date(),
 *     onSelect: function() {
 *         <a href="#!/api/Ext.MessageBox-method-alert" rel="Ext.MessageBox-method-alert" class="docClass">Ext.Msg.alert</a>('Selected', this.getValue());
 *     },
 *     listeners: {
 *         okclick: 'onSelect',
 *         monthdblclick: 'onSelect',
 *         yeardblclick: 'onSelect',
 *         cancelclick: function () {
 *             this.setValue(new Date());
 *         }
 *     }
 * });
 * </code></pre>
 */
public class MonthPicker extends ext.Component {
  /**
   * @param config @inheritDoc
   */
  public function MonthPicker(config:ext.picker.MonthPicker = null) {
    super();
  }

  [ExtConfig]
  /**
   * The base CSS class to apply to the picker element.
   * @default 'x-monthpicker'
   */
  override public native function get baseCls():String;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set baseCls(value:String):void;

  [ExtConfig]
  /**
   * The text to display on the cancel button.
   * @default 'Cancel'
   */
  public native function get cancelText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set cancelText(value:String):void;

  [ExtConfig]
  /**
   * The <a href="#!/api/Ext.button.Button-cfg-ui" rel="Ext.button.Button-cfg-ui" class="docClass">Ext.button.Button.ui</a> to use for the month picker's footer buttons.
   * @default 'default'
   */
  public native function get footerButtonUI():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set footerButtonUI(value:String):void;

  [ExtConfig]
  /**
   * The text to display on the ok button.
   * @default 'OK'
   */
  public native function get okText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set okText(value:String):void;

  [ExtConfig]
  /**
   * The class to be added to selected items in the picker.
   * @default 'x-monthpicker-selected'
   */
  public native function get selectedCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set selectedCls(value:String):void;

  [ExtConfig]
  /**
   * True to show ok and cancel buttons below the picker.
   * @default true
   */
  public native function get showButtons():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set showButtons(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The default value to set. See <a href="#!/api/Ext.picker.Month-method-setValue" rel="Ext.picker.Month-method-setValue" class="docClass">setValue</a>
   * @see #getValue()
   * @see #setValue()
   */
  public native function get value():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set value(value:*):void;

  /**
   * Modify the year display by passing an offset.
   * @param offset The offset to move by.
   * @default 10
   */
  public native function adjustYear(offset:Number = 10):void;

  /**
   * Gets the selected value. It is returned as an array [month, year]. It may
   * be a partial value, for example [null, 2010]. The month is returned as
   * 0 based.
   * @return The selected value
   */
  public native function getValue():Array;

  /**
   * Checks whether the picker has a selection
   * @return Returns true if both a month and year have been selected
   */
  public native function hasSelection():Boolean;

  /**
   * Set the value for the picker.
   * @param value The value to set. It can be a Date object, where the month/year will be extracted, or
   * it can be an array, with the month as the first index and the year as the second.
   * @return this
   */
  public native function setValue(value:*):ext.picker.MonthPicker;
}
}