package ext.picker {
import ext.view.BoundListView;

[Uses("ext.view.BoundListView")]
[Uses("ext.view.DataView")]
[Uses("ext.view.AbstractView")]
[Uses("ext.util.StoreHolder")]
[Uses("ext.Base")]
[Uses("ext.Component")]
[Uses("ext.util.Positionable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Animate")]
[Uses("ext.util.Floating")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.state.Stateful")]
[Uses("ext.mixin.Queryable")]
[Native("Ext.picker.Time", require)]
/**
 * A time picker which provides a list of times from which to choose. This is used by the <a href="#!/api/Ext.form.field.Time" rel="Ext.form.field.Time" class="docClass">Ext.form.field.Time</a>
 * class to allow browsing and selection of valid times, but could also be used with other components.
 *
 * <p>By default, all times starting at midnight and incrementing every 15 minutes will be presented. This list of
 * available times can be controlled using the <a href="#!/api/Ext.picker.Time-cfg-minValue" rel="Ext.picker.Time-cfg-minValue" class="docClass">minValue</a>, <a href="#!/api/Ext.picker.Time-cfg-maxValue" rel="Ext.picker.Time-cfg-maxValue" class="docClass">maxValue</a>, and <a href="#!/api/Ext.picker.Time-cfg-increment" rel="Ext.picker.Time-cfg-increment" class="docClass">increment</a>
 * configuration properties. The format of the times presented in the list can be customized with the <a href="#!/api/Ext.picker.Time-cfg-format" rel="Ext.picker.Time-cfg-format" class="docClass">format</a>
 * config.</p>
 *
 * <p>To handle when the user selects a time from the list, you can subscribe to the <a href="#!/api/Ext.picker.Time-event-selectionchange" rel="Ext.picker.Time-event-selectionchange" class="docClass">selectionchange</a> event.</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.picker.Time" rel="Ext.picker.Time" class="docClass">Ext.picker.Time</a>', {
 *    width: 60,
 *    minValue: <a href="#!/api/Ext.Date-method-parse" rel="Ext.Date-method-parse" class="docClass">Ext.Date.parse</a>('04:30:00 AM', 'h:i:s A'),
 *    maxValue: <a href="#!/api/Ext.Date-method-parse" rel="Ext.Date-method-parse" class="docClass">Ext.Date.parse</a>('08:00:00 AM', 'h:i:s A'),
 *    renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 * });
 * </code></pre>
 */
public class TimePicker extends ext.view.BoundListView {
  /**
   * @param config @inheritDoc
   */
  public function TimePicker(config:ext.picker.TimePicker = null) {
    super();
  }

  [ExtConfig]
  /**
   * The default time format string which can be overriden for localization support. The format must be valid
   * according to <a href="#!/api/Ext.Date-method-parse" rel="Ext.Date-method-parse" class="docClass">Ext.Date.parse</a>.
   *
   * <p>Defaults to <code>'g:i A'</code>, e.g., <code>'3:15 PM'</code>. For 24-hour time format try <code>'H:i'</code> instead.</p>
   * @default undefined
   */
  public native function get format():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set format(value:String):void;

  [ExtConfig]
  /**
   * The number of minutes between each time value in the list.
   * @default 15
   */
  public native function get increment():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set increment(value:Number):void;

  [ExtConfig]
  /**
   * The maximum time to be shown in the list of times. This must be a Date object (only the time fields will be
   * used); no parsing of String values will be done.
   * @see #setMaxValue()
   */
  public native function get maxValue():Date;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set maxValue(value:Date):void;

  [ExtConfig]
  /**
   * The minimum time to be shown in the list of times. This must be a Date object (only the time fields will be
   * used); no parsing of String values will be done.
   * @see #setMinValue()
   */
  public native function get minValue():Date;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set minValue(value:Date):void;

  /**
   * Set the <a href="#!/api/Ext.picker.Time-cfg-maxValue" rel="Ext.picker.Time-cfg-maxValue" class="docClass">maxValue</a> and update the list of available times. This must be a Date object (only the time
   * fields will be used); no parsing of String values will be done.
   */
  public native function setMaxValue(value:Date):void;

  /**
   * Set the <a href="#!/api/Ext.picker.Time-cfg-minValue" rel="Ext.picker.Time-cfg-minValue" class="docClass">minValue</a> and update the list of available times. This must be a Date object (only the time
   * fields will be used); no parsing of String values will be done.
   */
  public native function setMinValue(value:Date):void;

  /**
   * Update the list of available times in the list to be constrained within the <a href="#!/api/Ext.picker.Time-cfg-minValue" rel="Ext.picker.Time-cfg-minValue" class="docClass">minValue</a>
   * and <a href="#!/api/Ext.picker.Time-cfg-maxValue" rel="Ext.picker.Time-cfg-maxValue" class="docClass">maxValue</a>.
   */
  public native function updateList():void;
}
}