package ext.plugin {

[Uses("ext.plugin.ResponsivePlugin")]
[Uses("ext.mixin.Responsive")]
[Uses("ext.Base")]
[Native("Ext.plugin.Viewport", require)]
/**
 * This plugin can be applied to any <code>Component</code> (although almost always to a <code>Container</code>)
 * to make it fill the browser viewport. This plugin is used internally by the more familiar
 * <code><a href="#!/api/Ext.container.Viewport" rel="Ext.container.Viewport" class="docClass">Ext.container.Viewport</a></code> class.
 *
 * <p>The <code>Viewport</code> container is commonly used but it can be an issue if you need to fill the
 * viewport with a container that derives from another class (e.g., <code><a href="#!/api/Ext.tab.Panel" rel="Ext.tab.Panel" class="docClass">Ext.tab.Panel</a></code>). Prior
 * to this plugin, you would have to do this:</p>
 *
 * <pre><code> <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.container.Viewport" rel="Ext.container.Viewport" class="docClass">Ext.container.Viewport</a>', {
 *      layout: 'fit', // full the viewport with the tab panel
 *
 *      items: [{
 *          xtype: 'tabpanel',
 *          items: [{
 *              ...
 *          }]
 *      }]
 *  });
 * </code></pre>
 *
 * <p>With this plugin you can create the <code>tabpanel</code> as the viewport:</p>
 *
 * <pre><code> <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.tab.Panel" rel="Ext.tab.Panel" class="docClass">Ext.tab.Panel</a>', {
 *      plugins: 'viewport',
 *
 *      items: [{
 *          ...
 *      }]
 *  });
 * </code></pre>
 *
 * <p>More importantly perhaps is that as a plugin, the view class can be reused in other
 * contexts such as the content of a <code><a href="#!/api/Ext.window.Window" rel="Ext.window.Window" class="docClass">window</a></code>.</p>
 *
 * <p>The Viewport renders itself to the document body, and automatically sizes itself to the size of
 * the browser viewport and manages window resizing. There may only be one Viewport created
 * in a page.</p>
 *
 * <h2>Responsive Design</h2>
 *
 * <p>This plugin enables <a href="#!/api/Ext.mixin.Responsive-cfg-responsiveConfig" rel="Ext.mixin.Responsive-cfg-responsiveConfig" class="docClass">Ext.mixin.Responsive.responsiveConfig</a> for the target component.</p>
 */
public class ViewportPlugin extends ext.plugin.ResponsivePlugin {
  /**
   * @param config @inheritDoc
   */
  public function ViewportPlugin(config:ext.plugin.ViewportPlugin = null) {
    super();
  }
}
}