package ext.slider {
import ext.form.field.IField;

[Uses("ext.slider.MultiSlider")]
[Uses("ext.form.field.BaseField")]
[Uses("ext.Component")]
[Uses("ext.util.Positionable")]
[Uses("ext.Base")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Animate")]
[Uses("ext.util.Floating")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.state.Stateful")]
[Uses("ext.form.Labelable")]
[Uses("ext.form.field.Field")]
[Native("Ext.slider.Single", require)]
/**
 * Slider which supports vertical or horizontal orientation, keyboard adjustments, configurable snapping, axis clicking
 * and animation. Can be added as an item to any container.
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.slider.Single" rel="Ext.slider.Single" class="docClass">Ext.slider.Single</a>', {
 *     width: 200,
 *     value: 50,
 *     increment: 10,
 *     minValue: 0,
 *     maxValue: 100,
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 * });
 * </code></pre>
 *
 * <p>The class <a href="#!/api/Ext.slider.Single" rel="Ext.slider.Single" class="docClass">Ext.slider.Single</a> is aliased to <a href="#!/api/Ext.slider.Single" rel="Ext.slider.Single" class="docClass">Ext.Slider</a> for backwards compatibility.</p>
 */
public class SliderField extends ext.slider.MultiSlider {
  /**
   * @param config @inheritDoc
   */
  public function SliderField(config:ext.slider.SliderField = null) {
    super();
  }

  [ExtConfig]
  /**
   * This controls when the value of the slider is published to the <code>ViewModel</code>. By
   * default this is done only when the thumb is released (the change is complete). To
   * cause this to happen on every change of the thumb position, specify <code>false</code>. This
   * setting is <code>true</code> by default for improved performance on slower devices (such as
   * older browsers or tablets).
   * @default true
   */
  public native function get publishOnComplete():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set publishOnComplete(value:Boolean):void;

  /**
   * Programmatically sets the value of the Slider. Ensures that the value is constrained within the minValue and
   * maxValue.
   * @param value The value to set
   */
  override public native function setValue(value:*):ext.form.field.IField;
}
}