package ext.state {

[Uses("ext.state.Provider")]
[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Native("Ext.state.CookieProvider", require)]
/**
 * A Provider implementation which saves and retrieves state via cookies. The CookieProvider supports the usual cookie
 * options, such as:
 *
 * <ul>
 * <li><a href="#!/api/Ext.state.CookieProvider-cfg-path" rel="Ext.state.CookieProvider-cfg-path" class="docClass">path</a></li>
 * <li><a href="#!/api/Ext.state.CookieProvider-cfg-expires" rel="Ext.state.CookieProvider-cfg-expires" class="docClass">expires</a></li>
 * <li><a href="#!/api/Ext.state.CookieProvider-cfg-domain" rel="Ext.state.CookieProvider-cfg-domain" class="docClass">domain</a></li>
 * <li><a href="#!/api/Ext.state.CookieProvider-cfg-secure" rel="Ext.state.CookieProvider-cfg-secure" class="docClass">secure</a></li>
 * </ul>
 *
 *
 * <p>Example:</p>
 *
 * <pre><code>var cp = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.state.CookieProvider" rel="Ext.state.CookieProvider" class="docClass">Ext.state.CookieProvider</a>', {
 *     path: "/cgi-bin/",
 *     expires: new Date(new Date().getTime()+(1000*60*60*24*30)), //30 days
 *     domain: "sencha.com"
 * });
 *
 * <a href="#!/api/Ext.state.Manager-method-setProvider" rel="Ext.state.Manager-method-setProvider" class="docClass">Ext.state.Manager.setProvider</a>(cp);
 * </code></pre>
 */
public class CookieProvider extends ext.state.Provider {
  /**
   * Creates a new CookieProvider.
   * @param config Config object.
   */
  public function CookieProvider(config:ext.state.CookieProvider = null) {
    super();
  }

  [ExtConfig]
  /**
   * The domain to save the cookie for. Note that you cannot specify a different domain than your page is on, but you can
   * specify a sub-domain, or simply the domain itself like 'sencha.com' to include all sub-domains if you need to access
   * cookies across different sub-domains. Defaults to null which uses the same domain the page is running on including
   * the 'www' like 'www.sencha.com'.
   */
  public native function get domain():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set domain(value:String):void;

  [ExtConfig]
  /**
   * The cookie expiration date. Defaults to 7 days from now.
   */
  public native function get expires():Date;

  [ExtConfig]
  /**
   * @private
   */
  public native function set expires(value:Date):void;

  [ExtConfig]
  /**
   * The path for which the cookie is active. Defaults to root '/' which makes it active for all pages in the site.
   */
  public native function get path():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set path(value:String):void;

  [ExtConfig]
  /**
   * True if the site is using SSL
   * @default false
   */
  public native function get secure():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set secure(value:Boolean):void;
}
}