package ext.state {
import ext.Base;

[Uses("ext.Base")]
[Native]
/**
 * This is the global state manager. By default all components that are "state aware" check this class
 * for state information if you don't pass them a custom state provider. In order for this class
 * to be useful, it must be initialized with a provider when your application initializes. Example usage:
 *
 * <pre><code> // in your initialization function
 *  init: function() {
 *      <a href="#!/api/Ext.state.Manager-method-setProvider" rel="Ext.state.Manager-method-setProvider" class="docClass">Ext.state.Manager.setProvider</a>(new <a href="#!/api/Ext.state.CookieProvider" rel="Ext.state.CookieProvider" class="docClass">Ext.state.CookieProvider</a>());
 *  }
 * </code></pre>
 *
 * <p>This class passes on calls from components to the underlying <a href="#!/api/Ext.state.Provider" rel="Ext.state.Provider" class="docClass">Ext.state.Provider</a> so that
 * there is a common interface that can be used without needing to refer to a specific provider instance
 * in every component.</p>
 * <p>Type of singleton StateManager.</p>
 * @see ext.state.#StateManager ext.state.StateManager
 */
public class SStateManager extends ext.Base {
  /**
   * Clears a value from the state
   * @param key The key name
   */
  public native function clear(key:String):void;

  /**
   * Returns the current value for a key
   * @param key The key name
   * @param defaultValue The default value to return if the key lookup does not match
   * @return The state data
   */
  public native function get(key:String, defaultValue:Object):Object;

  /**
   * Gets the currently configured state provider
   * @return The state provider
   */
  public native function getProvider():ext.state.Provider;

  /**
   * Sets the value for a key
   * @param key The key name
   * @param value The state data
   */
  public native function set(key:String, value:Object):void;

  /**
   * Configures the default state provider for your application
   * @param stateProvider The state provider to set
   */
  public native function setProvider(stateProvider:ext.state.Provider):void;
}
}