package ext.tab {
import ext.Component;
import ext.button.Button;

[Uses("ext.button.Button")]
[Uses("ext.Component")]
[Uses("ext.util.Positionable")]
[Uses("ext.Base")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Animate")]
[Uses("ext.util.Floating")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.state.Stateful")]
[Uses("ext.mixin.Queryable")]
[Native("Ext.tab.Tab", require)]
/**
 * Fired when the tab is activated.
 * @eventType ext.tab.events.TabEvent.ACTIVATE
 */
[Event(name = 'onActivate', type = 'ext.tab.events.TabEvent')]
/**
 * Fires if the user clicks on the Tab's close button, but before the <a href="#!/api/Ext.tab.Tab-event-close" rel="Ext.tab.Tab-event-close" class="docClass">close</a> event is fired. Return
 * false from any listener to stop the close event being fired
 * @eventType ext.tab.events.TabEvent.BEFORE_CLOSE
 */
[Event(name = 'onBeforeClose', type = 'ext.tab.events.TabEvent')]
/**
 * Fires to indicate that the tab is to be closed, usually because the user has clicked the close button.
 * @eventType ext.tab.events.TabEvent.CLOSE
 */
[Event(name = 'onClose', type = 'ext.tab.events.TabEvent')]
/**
 * Fired when the tab is deactivated.
 * @eventType ext.tab.events.TabEvent.DEACTIVATE
 */
[Event(name = 'onDeactivate', type = 'ext.tab.events.TabEvent')]
/**
 * Represents a single Tab in a <a href="#!/api/Ext.tab.Panel" rel="Ext.tab.Panel" class="docClass">TabPanel</a>. A Tab is simply a slightly customized <a href="#!/api/Ext.button.Button" rel="Ext.button.Button" class="docClass">Button</a>,
 * styled to look like a tab. Tabs are optionally closable, and can also be disabled. 99% of the time you will not
 * need to create Tabs manually as the framework does so automatically when you use a <a href="#!/api/Ext.tab.Panel" rel="Ext.tab.Panel" class="docClass">TabPanel</a>
 */
public class Tab extends ext.button.Button {
  /**
   * @param config @inheritDoc
   */
  public function Tab(config:ext.tab.Tab = null) {
    super();
  }

  /**
   * Indicates that this tab is currently active. This is NOT a public configuration.
   * @default false
   */
  public native function get active():Boolean;

  /**
   * <code>true</code> in this class to identify an object as an instantiated Tab, or subclass thereof.
   * @default true
   */
  public native function get isTab():Boolean;

  /**
   * @private
   */
  public native function set isTab(value:Boolean):void;

  [ExtConfig]
  /**
   * True if the tab is currently closable
   * @see #setClosable()
   */
  public native function get closable():Boolean;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set closable(value:Boolean):void;

  [ExtConfig]
  /**
   * The accessible text label for the close button link to be announced by screen readers
   * when the tab is focused. This text does not appear visually and is only used when
   * <a href="#!/api/Ext.tab.Tab-cfg-closable" rel="Ext.tab.Tab-cfg-closable" class="docClass">closable</a> is <code>true</code>.
   * @default "removable"
   */
  public native function get closeText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set closeText(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The rotation of the tab.  Can be one of the following values:
   *
   * <ul>
   * <li><code>null</code> - use the default rotation, depending on the dock position of the tabbar</li>
   * <li><code>0</code> - no rotation</li>
   * <li><code>1</code> - rotate 90deg clockwise</li>
   * <li><code>2</code> - rotate 90deg counter-clockwise</li>
   * </ul>
   *
   *
   * <p>The default behavior of this config depends on the dock position of the tabbar:</p>
   *
   * <ul>
   * <li><code>'top'</code> or <code>'bottom'</code> - <code>0</code></li>
   * <li><code>'right'</code> - <code>1</code></li>
   * <li><code>'left'</code> - <code>2</code></li>
   * </ul>
   * @default 'default'
   * @see #getRotation()
   * @see #setRotation()
   */
  public native function get rotation():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set rotation(value:*):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The tab's position.  Users should not typically need to set this, as it is
   * configured automatically by the tab bar
   * @default 'top'
   * @see #getTabPosition()
   * @see #setTabPosition()
   */
  public native function get tabPosition():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set tabPosition(value:String):void;

  /**
   * Returns the value of <a href="#!/api/Ext.tab.Tab-cfg-rotation" rel="Ext.tab.Tab-cfg-rotation" class="docClass">rotation</a>.
   */
  public native function getRotation():*;

  /**
   * Returns the value of <a href="#!/api/Ext.tab.Tab-cfg-tabPosition" rel="Ext.tab.Tab-cfg-tabPosition" class="docClass">tabPosition</a>.
   */
  public native function getTabPosition():String;

  /**
   * Sets this tab's attached card. Usually this is handled automatically by the <a href="#!/api/Ext.tab.Panel" rel="Ext.tab.Panel" class="docClass">Ext.tab.Panel</a> that this Tab
   * belongs to and would not need to be done by the developer
   * @param card The card to set
   */
  public native function setCard(card:ext.Component):void;

  /**
   * Sets the tab as either closable or not.
   * @param closable Pass false to make the tab not closable. Otherwise the tab will be made closable (eg a
   * close button will appear on the tab)
   */
  public native function setClosable(closable:Boolean):void;

  /**
   * Sets the value of <a href="#!/api/Ext.tab.Tab-cfg-rotation" rel="Ext.tab.Tab-cfg-rotation" class="docClass">rotation</a>.
   * @param rotation The new value.
   */
  public native function setRotation(rotation:*):void;

  /**
   * Sets the value of <a href="#!/api/Ext.tab.Tab-cfg-tabPosition" rel="Ext.tab.Tab-cfg-tabPosition" class="docClass">tabPosition</a>.
   * @param tabPosition The new value.
   */
  public native function setTabPosition(tabPosition:String):void;
}
}