package ext.tree {
import ext.data.Model;
import ext.data.TreeModel;
import ext.panel.TablePanel;

[Uses("ext.panel.TablePanel")]
[Uses("ext.grid.locking.Lockable")]
[Uses("ext.Base")]
[Uses("ext.panel.Panel")]
[Uses("ext.container.DockingContainer")]
[Uses("ext.container.Container")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.mixin.Container")]
[Uses("ext.Mixin")]
[Uses("ext.Component")]
[Uses("ext.util.Positionable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Animate")]
[Uses("ext.util.Floating")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.state.Stateful")]
[Native("Ext.tree.Panel", require)]
/**
 * Fires after an item has been visually collapsed and is no longer visible in the tree.
 * @eventType ext.tree.events.TreePanel_node_index_itemEvent.AFTER_ITEM_COLLAPSE
 */
[Event(name = 'onAfterItemCollapse', type = 'ext.tree.events.TreePanel_node_index_itemEvent')]
/**
 * Fires after an item has been visually expanded and is visible in the tree.
 * @eventType ext.tree.events.TreePanel_node_index_itemEvent.AFTER_ITEM_EXPAND
 */
[Event(name = 'onAfterItemExpand', type = 'ext.tree.events.TreePanel_node_index_itemEvent')]
/**
 * Fires when a node with a checkbox's checked property changes.
 * @eventType ext.tree.events.TreePanel_node_checked_eEvent.BEFORE_CHECK_CHANGE
 */
[Event(name = 'onBeforeCheckChange', type = 'ext.tree.events.TreePanel_node_checked_eEvent')]
/**
 * Fires before a new child is appended, return false to cancel the append.
 * @eventType ext.tree.events.TreePanel_nodeEvent.BEFORE_ITEM_APPEND
 */
[Event(name = 'onBeforeItemAppend', type = 'ext.tree.events.TreePanel_nodeEvent')]
/**
 * Fires before this node is collapsed.
 * @eventType ext.tree.events.TreePanelEvent.BEFORE_ITEM_COLLAPSE
 */
[Event(name = 'onBeforeItemCollapse', type = 'ext.tree.events.TreePanelEvent')]
/**
 * Fires before this node is expanded.
 * @eventType ext.tree.events.TreePanelEvent.BEFORE_ITEM_EXPAND
 */
[Event(name = 'onBeforeItemExpand', type = 'ext.tree.events.TreePanelEvent')]
/**
 * Fires before a new child is inserted, return false to cancel the insert.
 * @eventType ext.tree.events.TreePanel_node_refNodeEvent.BEFORE_ITEM_INSERT
 */
[Event(name = 'onBeforeItemInsert', type = 'ext.tree.events.TreePanel_node_refNodeEvent')]
/**
 * Fires before this node is moved to a new location in the tree. Return false to cancel the move.
 * @eventType ext.tree.events.TreePanel_oldParent_newParent_indexEvent.BEFORE_ITEM_MOVE
 */
[Event(name = 'onBeforeItemMove', type = 'ext.tree.events.TreePanel_oldParent_newParent_indexEvent')]
/**
 * Fires before a child is removed, return false to cancel the remove.
 * @eventType ext.tree.events.TreePanel_node_isMoveEvent.BEFORE_ITEM_REMOVE
 */
[Event(name = 'onBeforeItemRemove', type = 'ext.tree.events.TreePanel_node_isMoveEvent')]
/**
 * Fires before a request is made for a new data object. If the beforeload handler returns false the load
 * action will be canceled.
 * @eventType ext.tree.events.TreePanel_store_operationEvent.BEFORE_LOAD
 */
[Event(name = 'onBeforeLoad', type = 'ext.tree.events.TreePanel_store_operationEvent')]
/**
 * Fires when a node with a checkbox's checked property changes.
 * @eventType ext.tree.events.TreePanel_node_checked_eEvent.CHECK_CHANGE
 */
[Event(name = 'onCheckChange', type = 'ext.tree.events.TreePanel_node_checked_eEvent')]
/**
 * Fires when a new child node is appended
 * @eventType ext.tree.events.TreePanel_node_indexEvent.ITEM_APPEND
 */
[Event(name = 'onItemAppend', type = 'ext.tree.events.TreePanel_node_indexEvent')]
/**
 * Fires when this node is collapsed.
 * @eventType ext.tree.events.TreePanelEvent.ITEM_COLLAPSE
 */
[Event(name = 'onItemCollapse', type = 'ext.tree.events.TreePanelEvent')]
/**
 * Fires when this node is expanded.
 * @eventType ext.tree.events.TreePanelEvent.ITEM_EXPAND
 */
[Event(name = 'onItemExpand', type = 'ext.tree.events.TreePanelEvent')]
/**
 * Fires when a new child node is inserted.
 * @eventType ext.tree.events.TreePanel_node_refNodeEvent.ITEM_INSERT
 */
[Event(name = 'onItemInsert', type = 'ext.tree.events.TreePanel_node_refNodeEvent')]
/**
 * Fires when this node is moved to a new location in the tree
 * @eventType ext.tree.events.TreePanel_oldParent_newParent_indexEvent.ITEM_MOVE
 */
[Event(name = 'onItemMove', type = 'ext.tree.events.TreePanel_oldParent_newParent_indexEvent')]
/**
 * Fires when a child node is removed
 * @eventType ext.tree.events.TreePanel_node_isMove_contextEvent.ITEM_REMOVE
 */
[Event(name = 'onItemRemove', type = 'ext.tree.events.TreePanel_node_isMove_contextEvent')]
/**
 * Fires whenever the store reads data from a remote data source.
 * @eventType ext.tree.events.TreePanel_records_successful_operation_nodeEvent.LOAD
 */
[Event(name = 'onLoad', type = 'ext.tree.events.TreePanel_records_successful_operation_nodeEvent')]
/**
 * The TreePanel provides tree-structured UI representation of tree-structured data.
 * A TreePanel must be bound to a <a href="#!/api/Ext.data.TreeStore" rel="Ext.data.TreeStore" class="docClass">Ext.data.TreeStore</a>.
 *
 * <p>TreePanels support multiple columns through the <a href="#!/api/Ext.tree.Panel-cfg-columns" rel="Ext.tree.Panel-cfg-columns" class="docClass">columns</a> configuration.</p>
 *
 * <p>By default a TreePanel contains a single column which uses the <code>text</code> Field of
 * the store's nodes.</p>
 *
 * <p>Simple TreePanel using inline data:</p>
 *
 * <pre class='inline-example '><code>var store = <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.TreeStore" rel="Ext.data.TreeStore" class="docClass">Ext.data.TreeStore</a>', {
 *     root: {
 *         expanded: true,
 *         children: [
 *             { text: 'detention', leaf: true },
 *             { text: 'homework', expanded: true, children: [
 *                 { text: 'book report', leaf: true },
 *                 { text: 'algebra', leaf: true}
 *             ] },
 *             { text: 'buy lottery tickets', leaf: true }
 *         ]
 *     }
 * });
 *
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.tree.Panel" rel="Ext.tree.Panel" class="docClass">Ext.tree.Panel</a>', {
 *     title: 'Simple Tree',
 *     width: 200,
 *     height: 200,
 *     store: store,
 *     rootVisible: false,
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
 * });
 * </code></pre>
 *
 * <p>For the tree node config options (like <code>text</code>, <code>leaf</code>, <code>expanded</code>), see the documentation of
 * <a href="#!/api/Ext.data.NodeInterface" rel="Ext.data.NodeInterface" class="docClass">NodeInterface</a> config options.</p>
 *
 * <p>Unless the TreeStore is configured with a <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">model</a> of your choosing, nodes in the <a href="#!/api/Ext.data.TreeStore" rel="Ext.data.TreeStore" class="docClass">Ext.data.TreeStore</a> are by default, instances of <a href="#!/api/Ext.data.TreeModel" rel="Ext.data.TreeModel" class="docClass">Ext.data.TreeModel</a>.</p>
 *
 * <h1>Heterogeneous node types.</h1>
 *
 * <p>If the tree needs to use different data model classes at different levels there is much flexibility in how to specify this.</p>
 *
 * <h3>Configuring the Reader.</h3>
 *
 * <p>If you configure the proxy's reader with a <a href="#!/api/Ext.data.reader.Reader-cfg-typeProperty" rel="Ext.data.reader.Reader-cfg-typeProperty" class="docClass">typeProperty</a>, then the server is in control of which data model
 * types are created. A discriminator field is used in the raw data to decide which class to instantiate.
 * <strong>If this is configured, then the data from the server is prioritized over other ways of determining node class</strong>.</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('myApp.Territory', {
 *     extend: '<a href="#!/api/Ext.data.TreeModel" rel="Ext.data.TreeModel" class="docClass">Ext.data.TreeModel</a>',
 *     fields: [{
 *         name: 'text',
 *         mapping: 'name'
 *     }]
 * });
 * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('myApp.Country', {
 *     extend: '<a href="#!/api/Ext.data.TreeModel" rel="Ext.data.TreeModel" class="docClass">Ext.data.TreeModel</a>',
 *     fields: [{
 *         name: 'text',
 *         mapping: 'name'
 *     }]
 * });
 * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('myApp.City', {
 *     extend: '<a href="#!/api/Ext.data.TreeModel" rel="Ext.data.TreeModel" class="docClass">Ext.data.TreeModel</a>',
 *     fields: [{
 *         name: 'text',
 *         mapping: 'name'
 *     }]
 * });
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.tree.Panel" rel="Ext.tree.Panel" class="docClass">Ext.tree.Panel</a>', {
 *     renderTo: document.body,
 *     height: 200,
 *     width: 400,
 *     title: 'Sales Areas - using typeProperty',
 *     rootVisible: false,
 *     store: {
 *         // Child types use namespace of store's model by default
 *         model: 'myApp.Territory',
 *         proxy: {
 *             type: 'memory',
 *             reader: {
 *                 typeProperty: 'mtype'
 *             }
 *         },
 *         root: {
 *             children: [{
 *                 name: 'Europe, ME, Africa',
 *                 mtype: 'Territory',
 *                 children: [{
 *                     name: 'UK of GB &amp; NI',
 *                     mtype: 'Country',
 *                     children: [{
 *                         name: 'London',
 *                         mtype: 'City',
 *                         leaf: true
 *                     }]
 *                 }]
 *             }, {
 *                 name: 'North America',
 *                 mtype: 'Territory',
 *                 children: [{
 *                     name: 'USA',
 *                     mtype: 'Country',
 *                     children: [{
 *                         name: 'Redwood City',
 *                         mtype: 'City',
 *                         leaf: true
 *                     }]
 *                 }]
 *             }]
 *         }
 *     }
 * });
 * </code></pre>
 *
 * <h3>Node being loaded decides.</h3>
 *
 * <p>You can declare your TreeModel subclasses with a <a href="#!/api/Ext.data.TreeModel-cfg-childType" rel="Ext.data.TreeModel-cfg-childType" class="docClass">childType</a> which means that the node being loaded decides the
 * class to instantiate for all of its child nodes.</p>
 *
 * <p>It is important to note that if the root node is <a href="#!/api/Ext.tree.Panel-cfg-rootVisible" rel="Ext.tree.Panel-cfg-rootVisible" class="docClass">hidden</a>, its type will default to the store's model type, and if left
 * as the default (<code><a href="#!/api/Ext.data.TreeModel" rel="Ext.data.TreeModel" class="docClass">Ext.data.TreeModel</a></code>) this will have no knowledge of creation of special child node types. So be sure to specify a store model in this case:</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('myApp.TerritoryRoot', {
 *     extend: '<a href="#!/api/Ext.data.TreeModel" rel="Ext.data.TreeModel" class="docClass">Ext.data.TreeModel</a>',
 *     childType: 'myApp.Territory',
 *     fields: [{
 *         name: 'text',
 *         mapping: 'name'
 *     }]
 * });
 * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('myApp.Territory', {
 *     extend: '<a href="#!/api/Ext.data.TreeModel" rel="Ext.data.TreeModel" class="docClass">Ext.data.TreeModel</a>',
 *     childType: 'myApp.Country',
 *     fields: [{
 *         name: 'text',
 *         mapping: 'name'
 *     }]
 * });
 * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('myApp.Country', {
 *     extend: '<a href="#!/api/Ext.data.TreeModel" rel="Ext.data.TreeModel" class="docClass">Ext.data.TreeModel</a>',
 *     childType: 'myApp.City',
 *     fields: [{
 *         name: 'text',
 *         mapping: 'name'
 *     }]
 * });
 * <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('myApp.City', {
 *     extend: '<a href="#!/api/Ext.data.TreeModel" rel="Ext.data.TreeModel" class="docClass">Ext.data.TreeModel</a>',
 *     fields: [{
 *         name: 'text',
 *         mapping: 'name'
 *     }]
 * });
 * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.tree.Panel" rel="Ext.tree.Panel" class="docClass">Ext.tree.Panel</a>', {
 *     renderTo: document.body,
 *     height: 200,
 *     width: 400,
 *     title: 'Sales Areas',
 *     rootVisible: false,
 *     store: {
 *         model: 'myApp.TerritoryRoot', // Needs to be this so it knows to create 'Country' child nodes
 *         root: {
 *             children: [{
 *                 name: 'Europe, ME, Africa',
 *                 children: [{
 *                     name: 'UK of GB &amp; NI',
 *                     children: [{
 *                         name: 'London',
 *                         leaf: true
 *                     }]
 *                 }]
 *             }, {
 *                 name: 'North America',
 *                 children: [{
 *                     name: 'USA',
 *                     children: [{
 *                         name: 'Redwood City',
 *                         leaf: true
 *                     }]
 *                 }]
 *             }]
 *         }
 *     }
 * });
 * </code></pre>
 *
 * <h1>Data structure</h1>
 *
 * <p>The <a href="#!/api/Ext.data.TreeStore" rel="Ext.data.TreeStore" class="docClass">TreeStore</a> maintains a <a href="#!/api/Ext.data.TreeStore-method-getRoot" rel="Ext.data.TreeStore-method-getRoot" class="docClass">root node</a> and a hierarchical structure of <a href="#!/api/Ext.data.TreeModel" rel="Ext.data.TreeModel" class="docClass">node</a>s.</p>
 *
 * <p>The <a href="#!/api/Ext.tree.View" rel="Ext.tree.View" class="docClass">UI</a> of the tree is driven by a {<a href="#!/api/Ext.data.NodeStore" rel="Ext.data.NodeStore" class="docClass">Ext.data.NodeStore</a> NodeStore} which is a flattened view of <em>visible</em> nodes.
 * The NodeStore is dynamically updated to reflect the visibility state of nodes as nodes are added, removed or expanded. The UI
 * responds to mutation events fire by the NodeStore.</p>
 *
 * <p>Note that nodes have several more <a href="#!/api/Ext.data.Model-cfg-fields" rel="Ext.data.Model-cfg-fields" class="docClass">fields</a> in order to describe their state within the hierarchy.</p>
 *
 * <p>If you add store listeners to the <a href="#!/api/Ext.data.Store-event-update" rel="Ext.data.Store-event-update" class="docClass">update</a> event, then you will receive notification when any of this state changes.
 * You should check the array of modified field names passed to the listener to decide whether the listener should take action or ignore the event.</p>
 *
 * <h1>Tree Grid</h1>
 *
 * <p>Trees may be configured using the <a href="#!/api/Ext.tree.Panel-cfg-columns" rel="Ext.tree.Panel-cfg-columns" class="docClass">columns</a> config including a
 * <a href="#!/api/Ext.tree.Column" rel="Ext.tree.Column" class="docClass">treecolumn</a> to give the tree panel a hybrid tree /
 * <a href="#!/api/Ext.grid.Panel" rel="Ext.grid.Panel" class="docClass">grid</a> structure.</p>
 *
 * <pre class='inline-example '><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>({
 *     xtype: 'treepanel',
 *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
 *     height: 200,
 *     width: 300,
 *     rootVisible: false,
 *     store: <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.data.TreeStore" rel="Ext.data.TreeStore" class="docClass">Ext.data.TreeStore</a>', {
 *         fields: ['text', 'duration', 'isLayover'],
 *         root: {
 *             expanded: true,
 *             children: [{
 *                 text: 'SFO  &amp;nbsp;✈&amp;nbsp; DFW',
 *                 duration: '6h 55m',
 *                 expanded: true,
 *                 children: [{
 *                     text: 'SFO &amp;nbsp;✈&amp;nbsp; PHX',
 *                     duration: '2h 04m',
 *                     leaf: true
 *                 }, {
 *                     text: 'PHX layover',
 *                     duration: '2h 36m',
 *                     isLayover: true,
 *                     leaf: true
 *                 }, {
 *                     text: 'PHX &amp;nbsp;✈&amp;nbsp; DFW',
 *                     duration: '2h 15m',
 *                     leaf: true
 *                 }]
 *             }]
 *         }
 *     }),
 *     columns: [{
 *         xtype: 'treecolumn',
 *         text: 'Flight Endpoints',
 *         dataIndex: 'text',
 *         flex: 1,
 *         renderer: function (val, meta, rec) {
 *             if (rec.get('isLayover')) {
 *                 meta.tdStyle = 'color: gray; font-style: italic;';
 *             }
 *             return val;
 *         }
 *     }, {
 *         text: 'Duration',
 *         dataIndex: 'duration',
 *         width: 100
 *     }]
 * });
 * </code></pre>
 */
public class TreePanel extends ext.panel.TablePanel {
  /**
   * @param config @inheritDoc
   */
  public function TreePanel(config:ext.tree.TreePanel = null) {
    super();
  }

  [ExtConfig]
  /**
   * Configure as true to separate rows with visible horizontal lines (depends on theme).
   * @default false
   */
  override public native function get rowLines():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set rowLines(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The <a href="#!/api/Ext.data.TreeStore" rel="Ext.data.TreeStore" class="docClass">Store</a> the tree should use as its data source.
   * @see #getStore()
   * @see #setStore()
   */
  override public native function get store():*;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  override public native function set store(value:*):void;

  [ExtConfig]
  /**
   * True to enable animated expand/collapse. Defaults to the value of <a href="#!/api/Ext-property-enableFx" rel="Ext-property-enableFx" class="docClass">Ext.enableFx</a>.
   */
  public native function get animated():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set animated(value:Boolean):void;

  [ExtConfig]
  /**
   * This configuration controls whether, and how checkbox click gestures are propagated to
   * child nodes, or to a parent node.
   *
   * <p>Valid values are</p>
   *
   * <pre><code> - `'none'` Checking a check node does not affect any other nodes.
   *  - `'up'` Checking a check node synchronizes the value of its parent node with the state of its children.
   *  - `'down'` Checking a check node propagates the value to its child nodes.
   *  - `'both'` Checking a check node updates its child nodes, and syncs its parent node.
   * </code></pre>
   * @default none
   */
  public native function get checkPropagation():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set checkPropagation(value:String):void;

  [ExtConfig]
  /**
   * The field inside the model that will be used as the node's text.
   * @default text
   */
  public native function get displayField():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set displayField(value:String):void;

  [ExtConfig]
  /**
   * True to automatically prepend a leaf sorter to the store.
   */
  public native function get folderSort():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set folderSort(value:Boolean):void;

  [ExtConfig]
  /**
   * False to disable tree lines.
   * @default true
   */
  public native function get lines():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set lines(value:Boolean):void;

  [ExtConfig]
  /**
   * Allows you to not specify a store on this TreePanel. This is useful for creating a simple tree with preloaded
   * data without having to specify a TreeStore and Model. A store and model will be created and root will be passed
   * to that store. For example:
   *
   * <pre><code><a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>('<a href="#!/api/Ext.tree.Panel" rel="Ext.tree.Panel" class="docClass">Ext.tree.Panel</a>', {
   *     title: 'Simple Tree',
   *     root: {
   *         text: "Root node",
   *         expanded: true,
   *         children: [
   *             { text: "Child 1", leaf: true },
   *             { text: "Child 2", leaf: true }
   *         ]
   *     },
   *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>()
   * });
   * </code></pre>
   */
  public native function get root():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set root(value:*):void;

  [ExtConfig]
  /**
   * False to hide the root node.
   *
   * <p>Note that trees <em>always</em> have a root node. If you do not specify a <a href="#!/api/Ext.tree.Panel-cfg-root" rel="Ext.tree.Panel-cfg-root" class="docClass">root</a> node, one will be created.</p>
   *
   * <p>If the root node is not visible, then in order for a tree to appear to the end user, the root node is autoloaded with its child nodes.</p>
   * @default true
   */
  public native function get rootVisible():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set rootVisible(value:Boolean):void;

  [ExtConfig]
  /**
   * True if only 1 node per branch may be expanded.
   * @default false
   */
  public native function get singleExpand():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set singleExpand(value:Boolean):void;

  [ExtConfig]
  /**
   * True to use Vista-style arrows in the tree.
   * @default false
   */
  public native function get useArrows():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set useArrows(value:Boolean):void;

  /**
   * Expand the tree to the path of a particular node, then scroll it into view.
   * @param record The record, record id,  or the zero-based position in the dataset to scroll to.
   * @param options An object containing options to modify the operation.
   * @param {Boolean} [options.animate] Pass <code>true</code> to animate the row into view.
   * @param {Boolean} [options.highlight] Pass <code>true</code> to highlight the row with a glow animation when it is in view.
   * @param {Boolean} [options.select] Pass as <code>true</code> to select the specified row.
   * @param {Boolean} [options.focus] Pass as <code>true</code> to focus the specified row.
   * @param {Function} [options.callback] A function to execute when the record is in view. This may be necessary if the
   *                          first parameter is a record index and the view is backed by a <a href="#!/api/Ext.data.BufferedStore" rel="Ext.data.BufferedStore" class="docClass">buffered store</a>
   *                          which does not contain that record.
   * @param {Boolean} options.callback.success <code>true</code> if acquiring the record's view node was successful.
   * @param {ext.data.Model} options.callback.record If successful, the target record.
   * @param {js.HTMLElement} options.callback.node If successful, the record's view node.
   * @param {Object} [options.scope] The scope (<code>this</code> reference) in which the callback function is executed.
   */
  override public native function ensureVisible(record:*, options:Object = null):void;

  /**
   * Collapse all nodes
   * @param callback A function to execute when the collapse finishes.
   * @param scope The scope of the callback function
   */
  public native function collapseAll(callback:Function = null, scope:Object = null):void;

  /**
   * Collapses a record that is loaded in the tree.
   * @param record The record to collapse
   * @param deep True to collapse nodes all the way up the tree hierarchy.
   * @param callback The function to run after the collapse is completed
   * @param scope The scope of the callback function.
   */
  public native function collapseNode(record:ext.data.Model, deep:Boolean = false, callback:Function = null, scope:Object = null):void;

  /**
   * Expand all nodes
   * @param callback A function to execute when the expand finishes.
   * @param scope The scope of the callback function
   */
  public native function expandAll(callback:Function = null, scope:Object = null):void;

  /**
   * Expands a record that is loaded in the tree.
   * @param record The record to expand
   * @param deep True to expand nodes all the way down the tree hierarchy.
   * @param callback The function to run after the expand is completed
   * @param scope The scope of the callback function.
   */
  public native function expandNode(record:ext.data.Model, deep:Boolean = false, callback:Function = null, scope:Object = null):void;

  /**
   * Expand the tree to the path of a particular node. This is the way to expand a known path
   * when the intervening nodes are not yet loaded.
   *
   * <p>The path may be an absolute path (beginning with a <code>'/'</code> character) from the root, eg:</p>
   *
   * <pre><code>'/rootId/nodeA/nodeB/nodeC'
   * </code></pre>
   *
   * <p>Or, the path may be relative, starting from an <strong>existing</strong> node in the tree:</p>
   *
   * <pre><code>'nodeC/nodeD'
   * </code></pre>
   * @param path The path to expand. The path may be absolute, including a leading separator and starting
   *                          from the root node id, or relative with no leading separator, starting from an <em>existing</em>
   *                          node in the tree.
   * @param options An object containing options to modify the operation.
   * @param {String} [options.field] The field to get the data from. Defaults to the model idProperty.
   * @param {String} [options.separator] A separator to use.
   * @default '/'
   * @param {Boolean} [options.select] Pass as <code>true</code> to select the specified row.
   * @param {Boolean} [options.focus] Pass as <code>true</code> to focus the specified row.
   * @param {Function} [options.callback] A function to execute when the expand finishes.
   * @param {Boolean} options.callback.success <code>true</code> if the node expansion was successful.
   * @param {ext.data.Model} options.callback.record If successful, the target record.
   * @param {js.HTMLElement} options.callback.node If successful, the record's view node. If unsuccessful, the
   *                          last view node encountered while expanding the path.
   * @param {Object} [options.scope] The scope (<code>this</code> reference) in which the callback function is executed.
   */
  public native function expandPath(path:String, options:Object = null):void;

  /**
   * Retrieve an array of checked records.
   * @return An array containing the checked records
   */
  public native function getChecked():Array;

  /**
   * Returns the root node for this tree.
   */
  public native function getRootNode():ext.data.TreeModel;

  /**
   * Expand the tree to the path of a particular node, then select it.
   * @param path The path to expand. The path may be absolute, including a leading separator and
   *                                  starting from the root node id, or relative with no leading separator, starting from
   *                                  an <em>existing</em> node in the tree.
   * @param field The field to get the data from. Defaults to the model idProperty.
   * @param separator A separator to use.
   * @default '/'
   * @param callback A function to execute when the select finishes.
   * @param {Boolean} callback.success <code>true</code> if the node expansion was successful.
   * @param {ext.data.NodeInterface} callback.lastNode If successful, the target node. If unsuccessful, the
   *                                  last tree node encountered while expanding the path.
   * @param {js.HTMLElement} callback.node If successful, the record's view node.
   * @param scope The scope of the callback function
   */
  public native function selectPath(path:String, field:String = null, separator:String = '/', callback:Function = null, scope:Object = null):void;

  /**
   * Sets root node of this tree. All trees <em>always</em> have a root node. It may be <a href="#!/api/Ext.tree.Panel-cfg-rootVisible" rel="Ext.tree.Panel-cfg-rootVisible" class="docClass">hidden</a>.
   *
   * <p>If the passed node has not already been loaded with child nodes, and has its expanded field set, this triggers the <a href="#!/api/Ext.tree.Panel-cfg-store" rel="Ext.tree.Panel-cfg-store" class="docClass">store</a> to load the child nodes of the root.</p>
   * @return The new root
   */
  public native function setRootNode(root:*):ext.data.TreeModel;
}
}