package ext.tree {
import ext.data.Model;
import ext.view.TableView;

[Uses("ext.view.TableView")]
[Uses("ext.view.DataView")]
[Uses("ext.view.AbstractView")]
[Uses("ext.util.StoreHolder")]
[Uses("ext.Base")]
[Uses("ext.Component")]
[Uses("ext.util.Positionable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Animate")]
[Uses("ext.util.Floating")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.state.Stateful")]
[Uses("ext.mixin.Queryable")]
[Native("Ext.tree.View", require)]
/**
 * Fires after an item has been visually collapsed and is no longer visible in the tree.
 * @eventType ext.tree.events.TreeView_node_index_itemEvent.AFTER_ITEM_COLLAPSE
 */
[Event(name = 'onAfterItemCollapse', type = 'ext.tree.events.TreeView_node_index_itemEvent')]
/**
 * Fires after an item has been visually expanded and is visible in the tree.
 * @eventType ext.tree.events.TreeView_node_index_itemEvent.AFTER_ITEM_EXPAND
 */
[Event(name = 'onAfterItemExpand', type = 'ext.tree.events.TreeView_node_index_itemEvent')]
/**
 * Fires when a tree node is being targeted for a drag drop, return false to signal drop not allowed.
 * @eventType ext.tree.events.TreeView_targetNode_position_dragData_eEvent.NODE_DRAG_OVER
 */
[Event(name = 'onNodeDragOver', type = 'ext.tree.events.TreeView_targetNode_position_dragData_eEvent')]
/**
 * Used as a view by <a href="#!/api/Ext.tree.Panel" rel="Ext.tree.Panel" class="docClass">TreePanel</a>.
 */
public class TreeView extends ext.view.TableView {
  /**
   * @param config @inheritDoc
   */
  public function TreeView(config:ext.tree.TreeView = null) {
    super();
  }

  /**
   * <code>true</code> in this class to identify an object as an instantiated TreeView, or subclass thereof.
   * @default true
   */
  public native function get isTreeView():Boolean;

  /**
   * @private
   */
  public native function set isTreeView(value:Boolean):void;

  [ExtConfig]
  /**
   * True to enable animated expand/collapse (defaults to the value of <a href="#!/api/Ext-property-enableFx" rel="Ext-property-enableFx" class="docClass">Ext.enableFx</a>)
   */
  public native function get animated():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set animated(value:Boolean):void;

  [ExtConfig]
  /**
   * False to hide the root node.
   * @default true
   */
  public native function get rootVisible():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set rootVisible(value:Boolean):void;

  /**
   * Collapses a record that is loaded in the view.
   *
   * <p>If an animated collapse or expand of the record is in progress, this call will be ignored.</p>
   * @param record The record to collapse
   * @param deep True to collapse nodes all the way up the tree hierarchy.
   * @param callback The function to run after the collapse is completed
   * @param scope The scope of the callback function.
   */
  public native function collapse(record:ext.data.Model, deep:Boolean = false, callback:Function = null, scope:Object = null):void;

  /**
   * Expands a record that is loaded in the view.
   *
   * <p>If an animated collapse or expand of the record is in progress, this call will be ignored.</p>
   * @param record The record to expand
   * @param deep True to expand nodes all the way down the tree hierarchy.
   * @param callback The function to run after the expand is completed
   * @param scope The scope of the callback function.
   */
  public native function expand(record:ext.data.Model, deep:Boolean = false, callback:Function = null, scope:Object = null):void;

  /**
   * Toggles a record between expanded and collapsed.
   *
   * <p>If an animated collapse or expand of the record is in progress, this call will be ignored.</p>
   * @param deep True to collapse nodes all the way up the tree hierarchy.
   * @param callback The function to run after the expand/collapse is completed
   * @param scope The scope of the callback function.
   */
  public native function toggle(record:ext.data.Model, deep:Boolean = false, callback:Function = null, scope:Object = null):void;
}
}