package ext.util {
import ext.mixin.Observable;

[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Native("Ext.util.AbstractMixedCollection", require)]
[ExcludeClass]
/**
 * Fires when an item is added to the collection.
 * @since 1.1.0
 * @eventType ext.util.events.AbstractMixedCollection_index_o_keyEvent.ADD
 */
[Event(name = 'onAdd', type = 'ext.util.events.AbstractMixedCollection_index_o_keyEvent')]
/**
 * Fires when the collection is cleared.
 * @since 1.1.0
 * @eventType ext.util.events.AbstractMixedCollectionEvent.CLEAR
 */
[Event(name = 'onClear', type = 'ext.util.events.AbstractMixedCollectionEvent')]
/**
 * Fires when an item is removed from the collection.
 * @since 1.1.0
 * @eventType ext.util.events.AbstractMixedCollection_o_keyEvent.REMOVE
 */
[Event(name = 'onRemove', type = 'ext.util.events.AbstractMixedCollection_o_keyEvent')]
/**
 * Fires when an item is replaced in the collection.
 * @since 1.1.0
 * @eventType ext.util.events.AbstractMixedCollection_key_old_newEvent.REPLACE
 */
[Event(name = 'onReplace', type = 'ext.util.events.AbstractMixedCollection_key_old_newEvent')]
public class AbstractMixedCollection extends ext.mixin.Observable {
  /**
   * @param config @inheritDoc
   */
  public function AbstractMixedCollection(config:ext.util.AbstractMixedCollection = null) {
    super();
  }

  /**
   * <code>true</code> in this class to identify an object as an instantiated MixedCollection, or subclass thereof.
   * @default true
   */
  public native function get isMixedCollection():Boolean;

  /**
   * @private
   */
  public native function set isMixedCollection(value:Boolean):void;

  [ExtConfig]
  /**
   * Specify <code>true</code> if the <a href="#!/api/Ext.util.AbstractMixedCollection-method-addAll" rel="Ext.util.AbstractMixedCollection-method-addAll" class="docClass">addAll</a>
   * function should add function references to the collection. Defaults to
   * <code>false</code>.
   * @default false
   * @since 3.4.0
   */
  public native function get allowFunctions():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set allowFunctions(value:Boolean):void;

  /**
   * Returns the number of items in the collection.
   * Better use <code>getCount()</code>.
   *
   * @see #getCount()
   */
  [Deprecated(replacement = "getCount()")]
  public native function get length():uint;

  /**
   * Adds an item to the collection. Fires the <a href="#!/api/Ext.util.AbstractMixedCollection-event-add" rel="Ext.util.AbstractMixedCollection-event-add" class="docClass">add</a> event when complete.
   * @since 1.1.0
   * @param key The key to associate with the item, or the new item.
   * <p>If a <a href="#!/api/Ext.util.AbstractMixedCollection-method-getKey" rel="Ext.util.AbstractMixedCollection-method-getKey" class="docClass">getKey</a> implementation was specified for this MixedCollection,
   * or if the key of the stored items is in a property called <code>id</code>,
   * the MixedCollection will be able to <em>derive</em> the key for the new item.
   * In this case just pass the new item in this parameter.</p>
   * @param obj The item to add.
   * <p>Note that when adding a value that is iterable, it must be wrapped in brackets, i.e.:</p>
   * <pre><code>c.add([[1, 2]]);
   * </code></pre>
   * <p>This will be needed for any value that is iterable, i.e., an array, arguments object,
   * HTML collections, etc.</p>
   * @return The item added.
   */
  public native function add(key:*, obj:Object = null):Object;

  /**
   * Adds all elements of an Array or an Object to the collection.
   * @since 1.1.0
   * @param objs An Object containing properties which will be added
   * to the collection, or an Array of values, each of which are added to the collection.
   * Functions references will be added to the collection if <code><a href="#!/api/Ext.util.AbstractMixedCollection-cfg-allowFunctions" rel="Ext.util.AbstractMixedCollection-cfg-allowFunctions" class="docClass">allowFunctions</a></code>
   * has been set to <code>true</code>.
   */
  public native function addAll(objs:*):void;

  /**
   * Removes all items from the collection.  Fires the <a href="#!/api/Ext.util.AbstractMixedCollection-event-clear" rel="Ext.util.AbstractMixedCollection-event-clear" class="docClass">clear</a> event when complete.
   * @since 1.1.0
   */
  public native function clear():void;

  /**
   * Creates a shallow copy of this collection
   * @since 1.1.0
   */
  public native function clone():ext.util.MixedCollection;

  /**
   * Collects unique values of a particular property in this MixedCollection
   * @param property The property to collect on
   * @param root 'root' property to extract the first argument from. This is used mainly when
   * summing fields in records, where the fields are all stored inside the 'data' object
   * @param allowBlank Pass true to allow null, undefined or empty string values
   * @return The unique values
   */
  public native function collect(property:String, root:String = null, allowBlank:Boolean = false):Array;

  /**
   * Returns true if the collection contains the passed Object as an item.
   * @since 1.1.0
   * @param o The Object to look for in the collection.
   * @return True if the collection contains the Object as an item.
   */
  public native function contains(o:Object):Boolean;

  /**
   * Returns true if the collection contains the passed Object as a key.
   * @since 1.1.0
   * @param key The key to look for in the collection.
   * @return True if the collection contains the Object as a key.
   */
  public native function containsKey(key:String):Boolean;

  /**
   * Executes the specified function once for every item in the collection.
   * The function should return a boolean value.
   * Returning false from the function will stop the iteration.
   * @since 1.1.0
   * @param fn The function to execute for each item.
   * @param fn.item The collection item.
   * @param {Number} fn.index The index of item.
   * @param {Number} fn.len Total length of collection.
   * @param scope The scope (<code>this</code> reference)
   * in which the function is executed. Defaults to the current item in the iteration.
   */
  public native function each(fn:Function, scope:Object = null):void;

  /**
   * Executes the specified function once for every key in the collection, passing each
   * key, and its associated item as the first two parameters.
   * @since 1.1.0
   * @param fn The function to execute for each item.
   * @param {String} fn.key The key of collection item.
   * @param fn.item The collection item.
   * @param {Number} fn.index The index of item.
   * @param {Number} fn.len Total length of collection.
   * @param scope The scope (<code>this</code> reference) in which the
   * function is executed. Defaults to the browser window.
   */
  public native function eachKey(fn:Function, scope:Object = null):void;

  /**
   * Filters the objects in this collection by a set of <a href="#!/api/Ext.util.Filter" rel="Ext.util.Filter" class="docClass">Filter</a>s, or by a single
   * property/value pair with optional parameters for substring matching and case sensitivity. See
   * <a href="#!/api/Ext.util.Filter" rel="Ext.util.Filter" class="docClass">Filter</a> for an example of using Filter objects (preferred). Alternatively,
   * MixedCollection can be easily filtered by property like this:
   *
   *
   * <pre><code>//create a simple store with a few people defined
   * var people = new <a href="#!/api/Ext.util.MixedCollection" rel="Ext.util.MixedCollection" class="docClass">Ext.util.MixedCollection</a>();
   * people.addAll([
   *     {id: 1, age: 25, name: 'Ed'},
   *     {id: 2, age: 24, name: 'Tommy'},
   *     {id: 3, age: 24, name: 'Arne'},
   *     {id: 4, age: 26, name: 'Aaron'}
   * ]);
   *
   * //a new MixedCollection containing only the items where age == 24
   * var middleAged = people.filter('age', 24);
   * </code></pre>
   * @since 1.1.0
   * @param property A property on your objects, or an array of <a href="#!/api/Ext.util.Filter" rel="Ext.util.Filter" class="docClass">Filter</a> objects
   * @param value Either string that the property values
   * should start with or a RegExp to test against the property
   * @param anyMatch True to match any part of the string, not just the beginning
   * @default false
   * @param caseSensitive True for case sensitive comparison.
   * @default false
   * @return The new filtered collection
   */
  public native function filter(property:String, value:*, anyMatch:Boolean = false, caseSensitive:Boolean = false):ext.util.MixedCollection;

  /**
   * Filter by a function. Returns a <i>new</i> collection that has been filtered.
   * The passed function will be called with each object in the collection.
   * If the function returns true, the value is included otherwise it is filtered.
   * @since 1.1.0
   * @param fn The function to be called.
   * @param fn.item The collection item.
   * @param {String} fn.key The key of collection item.
   * @param scope The scope (<code>this</code> reference) in
   * which the function is executed. Defaults to this MixedCollection.
   * @return The new filtered collection
   */
  public native function filterBy(fn:Function, scope:Object = null):ext.util.MixedCollection;

  [Deprecated(replacement = "findBy", since = "4.0")]
  /**
   * Returns the first item in the collection which elicits a true return value from the passed selection function.
   * @param fn The selection function to execute for each item.
   * @param scope The scope (<code>this</code> reference) in which the
   * function is executed. Defaults to the browser window.
   * @return The first item in the collection which returned true from the selection
   * function, or null if none was found.
   * @since 1.1.0
   */
  public native function find(fn:Function, scope:Object = null):Object;

  /**
   * Returns the first item in the collection which elicits a true return value from the
   * passed selection function.
   * @param fn The selection function to execute for each item.
   * @param fn.item The collection item.
   * @param {String} fn.key The key of collection item.
   * @param scope The scope (<code>this</code> reference) in which the
   * function is executed. Defaults to the browser window.
   * @return The first item in the collection which returned true from the selection
   * function, or null if none was found.
   */
  public native function findBy(fn:Function, scope:Object = null):Object;

  /**
   * Finds the index of the first matching object in this collection by a specific property/value.
   * @since 2.3.0
   * @param property The name of a property on your objects.
   * @param value A string that the property values
   * should start with or a RegExp to test against the property.
   * @param start The index to start searching at.
   * @default 0
   * @param anyMatch True to match any part of the string, not just the beginning.
   * @default false
   * @param caseSensitive True for case sensitive comparison.
   * @default false
   * @return The matched index or -1
   */
  public native function findIndex(property:String, value:*, start:Number = 0, anyMatch:Boolean = false, caseSensitive:Boolean = false):Number;

  /**
   * Find the index of the first matching object in this collection by a function.
   * If the function returns <i>true</i> it is considered a match.
   * @since 2.3.0
   * @param fn The function to be called.
   * @param fn.item The collection item.
   * @param {String} fn.key The key of collection item.
   * @param scope The scope (<code>this</code> reference) in which the function is executed. Defaults to this MixedCollection.
   * @param start The index to start searching at.
   * @default 0
   * @return The matched index or -1
   */
  public native function findIndexBy(fn:Function, scope:Object = null, start:Number = 0):Number;

  /**
   * Returns the first item in the collection.
   * @since 1.1.0
   * @return the first item in the collection..
   */
  public native function first():Object;

  /**
   * Returns the item associated with the passed key OR index.
   * Key has priority over index.  This is the equivalent
   * of calling <a href="#!/api/Ext.util.AbstractMixedCollection-method-getByKey" rel="Ext.util.AbstractMixedCollection-method-getByKey" class="docClass">getByKey</a> first, then if nothing matched calling <a href="#!/api/Ext.util.AbstractMixedCollection-method-getAt" rel="Ext.util.AbstractMixedCollection-method-getAt" class="docClass">getAt</a>.
   * @since 1.1.0
   * @param key The key or index of the item.
   * @return If the item is found, returns the item.  If the item was not found, returns <code>undefined</code>.
   * If an item was found, but is a Class, returns <code>null</code>.
   */
  public native function get(key:*):Object;

  /**
   * Returns the item at the specified index.
   * @param index The index of the item.
   * @return The item at the specified index.
   */
  public native function getAt(index:Number):Object;

  /**
   * Returns the item associated with the passed key.
   * @param key The key of the item.
   * @return The item associated with the passed key.
   */
  public native function getByKey(key:*):Object;

  /**
   * Returns the number of items in the collection.
   * @since 1.1.0
   * @return the number of items in the collection.
   */
  public native function getCount():Number;

  /**
   * A function which will be called, passing a newly added object
   * when the object is added without a separate id.  The function
   * should yield the key by which that object will be indexed.
   *
   * <p>If no key is yielded, then the object will be added, but it
   * cannot be accessed or removed quickly. Finding it in this
   * collection for interrogation or removal will require a linear
   * scan of this collection's items.</p>
   *
   * <p>The default implementation simply returns <code>item.id</code> but you can
   * provide your own implementation to return a different value as
   * in the following examples:</p>
   *
   * <pre><code>// normal way
   * var mc = new <a href="#!/api/Ext.util.MixedCollection" rel="Ext.util.MixedCollection" class="docClass">Ext.util.MixedCollection</a>();
   * mc.add(someEl.dom.id, someEl);
   * mc.add(otherEl.dom.id, otherEl);
   * //and so on
   *
   * // using getKey
   * var mc = new <a href="#!/api/Ext.util.MixedCollection" rel="Ext.util.MixedCollection" class="docClass">Ext.util.MixedCollection</a>({
   *     getKey: function(el){
   *         return el.dom.id;
   *     }
   * });
   * mc.add(someEl);
   * mc.add(otherEl);
   * </code></pre>
   * @since 1.1.0
   * @param item The item for which to find the key.
   * @return The key for the passed item.
   */
  public native function getKey(item:Object):Object;

  /**
   * Returns a range of items in this collection
   * @since 1.1.0
   * @param startIndex The starting index. Defaults to 0.
   * @param endIndex The ending index. Defaults to the last item.
   * @return An array of items
   */
  public native function getRange(startIndex:Number = 0, endIndex:Number = NaN):Array;

  /**
   * Returns index within the collection of the passed Object.
   * @since 1.1.0
   * @param o The item to find the index of.
   * @return index of the item. Returns -1 if not found.
   */
  public native function indexOf(o:Object):Number;

  /**
   * Returns index within the collection of the passed key.
   * @since 1.1.0
   * @param key The key to find the index of.
   * @return index of the key.
   */
  public native function indexOfKey(key:String):Number;

  /**
   * Inserts an item at the specified index in the collection. Fires the <a href="#!/api/Ext.util.AbstractMixedCollection-event-add" rel="Ext.util.AbstractMixedCollection-event-add" class="docClass">add</a> event when complete.
   * @since 1.1.0
   * @param index The index to insert the item at.
   * @param key The key to associate with the new item, or the item itself.
   * May also be an array of either to insert multiple items at once.
   * @param o If the second parameter was a key, the new item.
   * May also be an array to insert multiple items at once.
   * @return The item inserted or an array of items inserted.
   */
  public native function insert(index:Number, key:String, o:Object = undefined):Object;

  /**
   * Returns the last item in the collection.
   * @since 1.1.0
   * @return the last item in the collection..
   */
  public native function last():Object;

  /**
   * Remove an item from the collection.
   * @since 1.1.0
   * @param o The item to remove.
   * @return The item removed or false if no item was removed.
   */
  public native function remove(o:Object):Object;

  /**
   * Remove all items in the collection. Can also be used
   * to remove only the items in the passed array.
   * @param items An array of items to be removed.
   * @return this object
   */
  public native function removeAll(items:Array = null):ext.util.MixedCollection;

  /**
   * Remove an item from a specified index in the collection. Fires the <a href="#!/api/Ext.util.AbstractMixedCollection-event-remove" rel="Ext.util.AbstractMixedCollection-event-remove" class="docClass">remove</a> event when complete.
   * @since 1.1.0
   * @param index The index within the collection of the item to remove.
   * @return The item removed or false if no item was removed.
   */
  public native function removeAt(index:Number):Object;

  /**
   * Removes an item associated with the passed key fom the collection.
   * @param key The key of the item to remove. If <code>null</code> is passed,
   * all objects which yielded no key from the configured <a href="#!/api/Ext.util.AbstractMixedCollection-method-getKey" rel="Ext.util.AbstractMixedCollection-method-getKey" class="docClass">getKey</a> function are removed.
   * @return Only returned if removing at a specified key. The item removed or false if no item was removed.
   */
  public native function removeAtKey(key:String):Object;

  /**
   * Remove a range of items starting at a specified index in the collection.
   * Does not fire the remove event.
   * @param index The index within the collection of the item to remove.
   * @param removeCount The nuber of items to remove beginning at the specified index.
   * @default 1
   * @return The last item removed or false if no item was removed.
   */
  public native function removeRange(index:Number, removeCount:Number = 1):Object;

  /**
   * Reorders each of the items based on a mapping from old index to new index. Internally this
   * just translates into a sort. The 'sort' event is fired whenever reordering has
   * occurred.
   * @param mapping Mapping from old item index to new item index
   * <pre><code>// example of moving the last of 4 items to the front of the collection
   * // and moving each one before it forward one
   * collection.reorder({
   *     0: 1,
   *     1: 2,
   *     2: 3,
   *     3: 0,
   * });
   * </code></pre>
   */
  public native function reorder(mapping:Object):void;

  /**
   * Replaces an item in the collection. Fires the <a href="#!/api/Ext.util.AbstractMixedCollection-event-replace" rel="Ext.util.AbstractMixedCollection-event-replace" class="docClass">replace</a> event when complete.
   * @since 1.1.0
   * @param key The key associated with the item to replace, or the replacement item.
   * <p>If you supplied a <a href="#!/api/Ext.util.AbstractMixedCollection-method-getKey" rel="Ext.util.AbstractMixedCollection-method-getKey" class="docClass">getKey</a> implementation for this MixedCollection, or if the key
   * of your stored items is in a property called <em><code>id</code></em>, then the MixedCollection
   * will be able to <i>derive</i> the key of the replacement item. If you want to replace an item
   * with one having the same key value, then just pass the replacement item in this parameter.</p>
   * @param o o (optional) If the first parameter passed was a key, the item to associate
   * with that key.
   * @return The new item.
   */
  public native function replace(key:String, o:Object):Object;

  /**
   * Collects all of the values of the given property and returns their sum
   * @param property The property to sum by
   * @param root 'root' property to extract the first argument from. This is used mainly when
   * summing fields in records, where the fields are all stored inside the 'data' object
   * @param start The record index to start at
   * @default 0
   * @param end The record index to end at
   * @default -1
   * @return The total
   */
  public native function sum(property:String, root:String = null, start:Number = 0, end:Number = -1):Number;

  /**
   * Change the key for an existing item in the collection. If the old key
   * does not exist this is a no-op.
   * @param oldKey The old key
   * @param newKey The new key
   */
  public native function updateKey(oldKey:Object, newKey:Object):void;
}
}