package ext.util {
import ext.IMixin;

[Uses("ext.Mixin")]
[Uses("ext.util.FocusableContainer")]
[Uses("ext.Base")]
[Mixin("ext.util.FocusableContainer")]
/**
 * A mixin for groups of Focusable things (Components, Widgets, etc) that
 * should respond to arrow keys to navigate among the peers, but keep only
 * one of the peers tabbable by default (tabIndex=0)
 *
 * <p>Some examples: Toolbars, Tab bars, Panel headers, Menus</p>
 * @see ext.util.FocusableContainer
 */
public interface IFocusableContainer extends ext.IMixin {
  /**
   * The name of the element
   * that FocusableContainer should bind its keyboard handler to. Similar to ariaEl,
   * this name is resolved to the Ext.dom.Element instance after rendering.
   * @default "el"
   */
  function get focusableContainerEl():*;

  function set focusableContainerEl(value:*):void;

  [ExtConfig]
  /**
   * DOM tabIndex attribute to set on the
   * active Focusable child of this container when using the "Roaming tabindex"
   * technique.
   * @default 0
   */
  function get activeChildTabIndex():Number;

  [ExtConfig]
  function set activeChildTabIndex(value:Number):void;

  [ExtConfig]
  /**
   * Set this to <code>true</code> to enable focusing
   * disabled children via keyboard.
   */
  function get allowFocusingDisabledChildren():Boolean;

  [ExtConfig]
  function set allowFocusingDisabledChildren(value:Boolean):void;

  [ExtConfig]
  /**
   * Enable or disable
   * navigation with arrow keys for this FocusableContainer. This option may
   * be useful with nested FocusableContainers such as Grid column headers,
   * when only the root container should handle keyboard events.
   * @default true
   */
  function get enableFocusableContainer():Boolean;

  [ExtConfig]
  function set enableFocusableContainer(value:Boolean):void;

  [ExtConfig]
  /**
   * DOM tabIndex attribute to set on
   * inactive Focusable children of this container when using the "Roaming tabindex"
   * technique. This value rarely needs to be changed from its default.
   * @default -1
   */
  function get inactiveChildTabIndex():Number;

  [ExtConfig]
  function set inactiveChildTabIndex(value:Number):void;
}
}