package ext.util {
import ext.Base;

[Uses("ext.Base")]
[Native("Ext.util.Memento", require)]
/**
 * This class manages a set of captured properties from an object. These captured properties
 * can later be restored to an object.
 */
public class Memento extends ext.Base {
  /**
   * Creates a new memento and optionally captures properties from the target object.
   * @param target The target from which to capture properties. If specified in the
   * constructor, this target becomes the default target for all other operations.
   * @param props The property or array of properties to capture.
   */
  public function Memento(target:Object, props:*) {
    super();
  }

  /**
   * The default target object for capture/restore (passed to the constructor).
   */
  public native function get target():Object;

  /**
   * @private
   */
  public native function set target(value:Object):void;

  /**
   * Captures the specified properties from the target object in this memento.
   * @param props The property or array of properties to capture.
   * @param target The object from which to capture properties.
   */
  public native function capture(props:*, target:Object):void;

  /**
   * Removes the specified properties from this memento. These properties will not be
   * restored later without re-capturing their values.
   * @param props The property or array of properties to remove.
   */
  public native function remove(props:*):void;

  /**
   * Restores the specified properties from this memento to the target object.
   * @param props The property or array of properties to restore.
   * @param clear True to remove the restored properties from this memento or
   * false to keep them (default is true).
   * @param target The object to which to restore properties.
   */
  public native function restore(props:*, clear:Boolean, target:Object):void;

  /**
   * Restores all captured properties in this memento to the target object.
   * @param clear True to remove the restored properties from this memento or
   * false to keep them (default is true).
   * @param target The object to which to restore properties.
   */
  public native function restoreAll(clear:Boolean, target:Object):void;
}
}