package ext.util {
import ext.mixin.IObservable;
import ext.mixin.Observable;

[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Native("Ext.util.Observable", require)]
/**
 * A <a href="#!/api/Ext.mixin.Observable" rel="Ext.mixin.Observable" class="docClass">Ext.mixin.Observable</a> subclass that is provided for backward compatibility.
 * Applications should avoid using this class, and use <a href="#!/api/Ext.mixin.Observable" rel="Ext.mixin.Observable" class="docClass">Ext.mixin.Observable</a> instead.
 */
public class Observable extends ext.mixin.Observable {
  /**
   * Starts capture on the specified Observable. All events will be passed to the supplied function with the event
   * name + standard signature of the event <strong>before</strong> the event is fired. If the supplied function returns false,
   * the event will not fire.
   * @param o The Observable to capture events from.
   * @param fn The function to call when an event is fired.
   * @param scope The scope (<code>this</code> reference) in which the function is executed. Defaults to
   * the Observable firing the event.
   */
  public static native function capture(o:ext.mixin.IObservable, fn:Function, scope:Object = null):void;

  /**
   * Sets observability on the passed class constructor.
   *
   * <p>This makes any event fired on any instance of the passed class also fire a single event through
   * the <strong>class</strong> allowing for central handling of events on many instances at once.</p>
   *
   * <p>Usage:</p>
   *
   * <pre><code><a href="#!/api/Ext.util.Observable-static-method-observe" rel="Ext.util.Observable-static-method-observe" class="docClass">Ext.util.Observable.observe</a>(<a href="#!/api/Ext.data.Connection" rel="Ext.data.Connection" class="docClass">Ext.data.Connection</a>);
   * <a href="#!/api/Ext.data.Connection-method-on" rel="Ext.data.Connection-method-on" class="docClass">Ext.data.Connection.on</a>('beforerequest', function(con, options) {
   *     console.log('Ajax request made to ' + options.url);
   * });
   * </code></pre>
   * @param c The class constructor to make observable.
   * @param listeners An object containing a series of listeners to
   * add. See <a href="#!/api/Ext.util.Observable-method-addListener" rel="Ext.util.Observable-method-addListener" class="docClass">addListener</a>.
   */
  public static native function observe(c:Class, listeners:Object = null):void;

  /**
   * Removes <strong>all</strong> added captures from the Observable.
   * @param o The Observable to release
   */
  public static native function releaseCapture(o:ext.mixin.IObservable):void;

  /**
   * @inheritDoc
   */
  public function Observable(config:ext.mixin.Observable = null) {
    super();
  }
}
}