package ext.util {
import js.Event;

[Uses("ext.util.Region")]
[Uses("ext.Base")]
[Native("Ext.util.Point", require)]
/**
 * Represents a 2D point with x and y properties, useful for comparison and instantiation
 * from an event:
 *
 * <pre><code>var point = <a href="#!/api/Ext.util.Point-static-method-fromEvent" rel="Ext.util.Point-static-method-fromEvent" class="docClass">Ext.util.Point.fromEvent</a>(e);
 * </code></pre>
 */
public class Point extends ext.util.Region {
  /**
   * Returns a new point from an object that has <code>x</code> and <code>y</code> properties, if that object is not an instance
   * of <a href="#!/api/Ext.util.Point" rel="Ext.util.Point" class="docClass">Ext.util.Point</a>. Otherwise, returns the given point itself.
   */
  public static native function from(object:Object):ext.util.Point;

  /**
   * Returns a new instance of <a href="#!/api/Ext.util.Point" rel="Ext.util.Point" class="docClass">Ext.util.Point</a> based on the <code>pageX</code> / <code>pageY</code> values of the given event.
   * @param e The event.
   */
  public static native function fromEvent(e:js.Event):ext.util.Point;

  /**
   * Returns a new instance of <a href="#!/api/Ext.util.Point" rel="Ext.util.Point" class="docClass">Ext.util.Point</a> based on the <code>pageX</code> / <code>pageY</code> values of the given touch.
   */
  public static native function fromTouch(touch:js.Event):ext.util.Point;

  /**
   * Creates point on 2D plane.
   * @param x X coordinate.
   * @default 0
   * @param y Y coordinate.
   * @default 0
   */
  public function Point(x:Number = 0, y:Number = 0) {
    super(NaN, NaN, NaN, NaN);
  }

  [Deprecated(replacement = "clone", since = "2.0.0")]
  /**
   * Clones this Point.
   * @return The new point.
   */
  override public native function copy():ext.util.Region;

  /**
   * Copy the <code>x</code> and <code>y</code> values of another point / object to this point itself.
   * @param p The region to copy from.
   * @return This point.
   */
  override public native function copyFrom(p:ext.util.Region):ext.util.Region;

  /**
   * Compare this point and another point.
   * @param region The region to compare with
   * @return Returns whether they are equivalent.
   */
  override public native function equals(region:ext.util.Region):Boolean;

  /**
   * Copy a new instance of this point.
   * @return The new point.
   */
  public native function clone():ext.util.Point;

  /**
   * Returns <code>true</code> if the passed point is within a certain distance of this point.
   * @param point The point to check with, either an instance
   * of <a href="#!/api/Ext.util.Point" rel="Ext.util.Point" class="docClass">Ext.util.Point</a> or an object with <code>x</code> and <code>y</code> properties.
   * @param threshold Can be either an object with <code>x</code> and <code>y</code> properties or a number.
   */
  public native function isCloseTo(point:*, threshold:*):Boolean;

  /**
   * Determins whether this Point contained by the passed Region, Component or element.
   * @param region The rectangle to check that this Point is within.
   */
  public native function isContainedBy(region:*):Boolean;

  [Deprecated(replacement = "isCloseTo", since = "2.0.0")]
  /**
   * Returns <code>true</code> if this point is close to another one.
   */
  public native function isWithin():Boolean;

  /**
   * Compare this point with another point when the <code>x</code> and <code>y</code> values of both points are rounded. For example:
   * [100.3,199.8] will equals to [100, 200].
   * @param point The point to compare with, either an instance
   * of <a href="#!/api/Ext.util.Point" rel="Ext.util.Point" class="docClass">Ext.util.Point</a> or an object with <code>x</code> and <code>y</code> properties.
   */
  public native function roundedEquals(point:*):Boolean;

  /**
   * Returns a human-eye-friendly string that represents this point,
   * useful for debugging.
   * @return For example <code>Point[12,8]</code>.
   */
  public native function toString():String;

  /**
   * Alias for <a href="#!/api/Ext.util.Point-method-translateBy" rel="Ext.util.Point-method-translateBy" class="docClass">translateBy</a>
   * @param x Object containing the <code>x</code> and <code>y</code> properties.
   * Or the x value is using the two argument form.
   * @param y The y value unless using an Offset object.
   * @return this This Region
   */
  public native function translate(x:*, y:Number):ext.util.Region;
}
}