package ext.util {
import ext.mixin.Observable;

[Uses("ext.mixin.Observable")]
[Uses("ext.Base")]
[Native]
/**
 * Fires when navigation back or forwards within the local page's history occurs.
 * @eventType ext.util.events.History_tokenEvent.CHANGE
 */
[Event(name = 'onChange', type = 'ext.util.events.History_tokenEvent')]
/**
 * Fires when the <a href="#!/api/Ext.util.History" rel="Ext.util.History" class="docClass">Ext.util.History</a> singleton has been initialized and is ready for use.
 * @eventType ext.util.events.HistoryEvent.READY
 */
[Event(name = 'onReady', type = 'ext.util.events.HistoryEvent')]
/**
 * History management component that allows you to register arbitrary tokens that signify application
 * history state on navigation actions.  You can then handle the history <a href="#!/api/Ext.util.History-event-change" rel="Ext.util.History-event-change" class="docClass">change</a> event in order
 * to reset your application UI to the appropriate state when the user navigates forward or backward through
 * the browser history stack.
 *
 * <h2>Initializing</h2>
 *
 * <p>The <a href="#!/api/Ext.util.History-method-init" rel="Ext.util.History-method-init" class="docClass">init</a> method of the History object must be called before using History. This sets up the internal
 * state and must be the first thing called before using History.</p>
 * <p>Type of singleton History.</p>
 * @see ext.util.#History ext.util.History
 */
public class SHistory extends ext.mixin.Observable {
  /**
   * @inheritDoc
   */
  public function SHistory(config:ext.mixin.Observable = null) {
    super();
  }

  /**
   * True to use <code>window.top.location.hash</code> or false to use <code>window.location.hash</code>. Must be set before <a href="#!/api/Ext.util.History-method-init" rel="Ext.util.History-method-init" class="docClass">init</a> is called
   * because the <code>hashchange</code> event listener is added to the window at initialization time.
   * @default false
   */
  public native function get useTopWindow():Boolean;

  /**
   * @private
   */
  public native function set useTopWindow(value:Boolean):void;

  /**
   * Add a new token to the history stack. This can be any arbitrary value, although it would
   * commonly be the concatenation of a component id and another id marking the specific history
   * state of that component. Example usage:
   *
   * <pre><code>// Handle tab changes on a TabPanel
   * tabPanel.on('tabchange', function(tabPanel, tab){
   *      <a href="#!/api/Ext.util.History-method-add" rel="Ext.util.History-method-add" class="docClass">Ext.History.add</a>(tabPanel.id + ':' + tab.id);
   * });
   * </code></pre>
   * @param token The value that defines a particular application-specific history state
   * @param preventDuplicates When true, if the passed token matches the current token
   * it will not save a new history step. Set to false if the same state can be saved more than once
   * at the same history stack location.
   * @default true
   */
  public native function add(token:String, preventDuplicates:Boolean = true):void;

  /**
   * Programmatically steps back one step in browser history (equivalent to the user pressing the Back button).
   */
  public native function back():void;

  /**
   * Programmatically steps forward one step in browser history (equivalent to the user pressing the Forward button).
   */
  public native function forward():void;

  /**
   * Retrieves the currently-active history token.
   * @return The token
   */
  public native function getToken():String;

  /**
   * Initializes the global History instance.
   * @param onReady A callback function that will be called once the history
   * component is fully initialized.
   * @param scope The scope (<code>this</code> reference) in which the callback is executed.
   * Defaults to the browser window.
   */
  public native function init(onReady:Function = null, scope:Object = null):void;
}
}