package ext.util {
import ext.Base;
import ext.data.AbstractStore;
import ext.data.Store;

[Uses("ext.Base")]
[Native("Ext.util.StoreHolder", require)]
[ExcludeClass]
/**
 * This class is used as a mixin.
 *
 * <p>This class is to be used to provide basic methods for binding/unbinding stores to other
 * classes.</p>
 *
 * <p>This class is not intended for direct use but rather internally by those classes that
 * manage a Store.</p>
 */
public class StoreHolder extends ext.Base implements ext.util.IStoreHolder {
  /**
   * This property allows the object
   * to destroy bound stores that have #Ext.data.AbstractStore.autoDestroy
   * option set to <code>true</code>.
   * @default false
   */
  public native function get autoDestroyBoundStore():Boolean;

  /**
   * @private
   */
  public native function set autoDestroyBoundStore(value:Boolean):void;

  /**
   * Binds a store to this instance.
   * @param store The store to bind or ID of the store.
   * When no store given (or when <code>null</code> or <code>undefined</code> passed), unbinds the existing store.
   */
  public native function bindStore(store:ext.data.Store = undefined):void;

  /**
   * Gets the current store instance.
   * @return The store, null if one does not exist.
   */
  public native function getStore():ext.data.AbstractStore;

  /**
   * Sets the store to the specified store.
   * @since 5.0.0
   */
  public native function setStore(store:Object):void;

  /**
   * Binds listeners for this component to the store. By default it will add
   * anything bound by the getStoreListeners method, however it can be overridden
   * in a subclass to provide any more complicated handling.
   * @param store The store to bind to
   */
  protected native function bindStoreListeners(store:ext.data.AbstractStore):void;

  /**
   * Gets the listeners to bind to a new store.
   * @param store The Store which is being bound to for which a listeners object should be returned.
   * @return The listeners to be bound to the store in object literal form. The scope
   * may be omitted, it is assumed to be the current instance.
   */
  protected native function getStoreListeners(store:ext.data.Store):Object;

  /**
   * Template method, it is called when a new store is bound
   * to the current instance.
   * @param store The store being bound
   * @param initial True if this store is being bound as initialization of the instance.
   */
  protected native function onBindStore(store:ext.data.AbstractStore, initial:Boolean):void;

  /**
   * Template method, it is called when an existing store is unbound
   * from the current instance.
   * @param store The store being unbound
   * @param initial True if this store is being bound as initialization of the instance.
   */
  protected native function onUnbindStore(store:ext.data.AbstractStore, initial:Boolean):void;

  /**
   * Unbinds listeners from this component to the store. By default it will remove
   * anything bound by the bindStoreListeners method, however it can be overridden
   * in a subclass to provide any more complicated handling.
   * @param store The store to unbind from
   */
  protected native function unbindStoreListeners(store:ext.data.AbstractStore):void;
}
}