package ext.view {
import ext.Component;
import ext.data.AbstractStore;
import ext.data.Model;
import ext.data.Store;
import ext.event.Event;
import ext.selection.SelectionModel;
import ext.util.IStoreHolder;
import js.HTMLElement;

[Uses("ext.Component")]
[Uses("ext.util.StoreHolder")]
[Uses("ext.Base")]
[Uses("ext.util.Positionable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Animate")]
[Uses("ext.util.Floating")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.Mixin")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.state.Stateful")]
[Native("Ext.view.AbstractView", require)]
[ExcludeClass]
/**
 * Fires before the view is refreshed
 * @eventType ext.view.events.AbstractViewEvent.BEFORE_REFRESH
 */
[Event(name = 'onBeforeRefresh', type = 'ext.view.events.AbstractViewEvent')]
/**
 * Fires when the nodes associated with an recordset have been added to the underlying store
 * @eventType ext.view.events.AbstractView_records_index_node_viewEvent.ITEM_ADD
 */
[Event(name = 'onItemAdd', type = 'ext.view.events.AbstractView_records_index_node_viewEvent')]
/**
 * Fires when the node associated with an individual record is removed
 * @eventType ext.view.events.AbstractView_records_index_item_viewEvent.ITEM_REMOVE
 */
[Event(name = 'onItemRemove', type = 'ext.view.events.AbstractView_records_index_item_viewEvent')]
/**
 * Fires when the node associated with an individual record is updated
 * @eventType ext.view.events.AbstractView_record_index_node_viewEvent.ITEM_UPDATE
 */
[Event(name = 'onItemUpdate', type = 'ext.view.events.AbstractView_record_index_node_viewEvent')]
/**
 * Fires when the view is refreshed
 * @eventType ext.view.events.AbstractViewEvent.REFRESH
 */
[Event(name = 'onRefresh', type = 'ext.view.events.AbstractViewEvent')]
/**
 * Fires when the View's item elements representing Store items has been rendered. No items will be available
 * for selection until this event fires.
 * @eventType ext.view.events.AbstractViewEvent.VIEW_READY
 */
[Event(name = 'onViewReady', type = 'ext.view.events.AbstractViewEvent')]
/**
 * This is an abstract superclass and should not be used directly. Please see <a href="#!/api/Ext.view.View" rel="Ext.view.View" class="docClass">Ext.view.View</a>.
 */
public class AbstractView extends ext.Component implements ext.util.IStoreHolder {
  /**
   * @param config @inheritDoc
   */
  public function AbstractView(config:ext.view.AbstractView = null) {
    super();
  }

  [ExtConfig]
  /**
   * The HTML fragment or an array of fragments that will make up the template used by this DataView.  This should
   * be specified in the same format expected by the constructor of <a href="#!/api/Ext.XTemplate" rel="Ext.XTemplate" class="docClass">Ext.XTemplate</a>. When a <code>tpl</code> is specified,
   * this class assumes that records are rendered in the order they appear in the <code><a href="#!/api/Ext.view.AbstractView-cfg-store" rel="Ext.view.AbstractView-cfg-store" class="docClass">store</a></code>. If a custom <code>tpl</code>
   * does not conform to this assumption, index values will be incorrect which may cause the view to misbehave.
   * @since 2.3.0
   */
  override public native function get tpl():*;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set tpl(value:*):void;

  [ExtConfig]
  /**
   * Set this to true to ignore refresh events on the bound store. This is useful if
   * you wish to provide custom transition animations via a plugin
   * @default false
   * @since 3.4.0
   */
  public native function get blockRefresh():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set blockRefresh(value:Boolean):void;

  [ExtConfig]
  /**
   * True to defer emptyText being applied until the store's first load.
   * @default true
   * @since 2.3.0
   */
  public native function get deferEmptyText():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set deferEmptyText(value:Boolean):void;

  [ExtConfig]
  /**
   * Configure as 'true` to defer the initial refresh of the view.
   *
   * <p>This allows the View to execute its render and initial layout more quickly because the process will not be encumbered
   * by the update of the view structure.</p>
   * @default false
   */
  public native function get deferInitialRefresh():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set deferInitialRefresh(value:Boolean):void;

  [ExtConfig]
  /**
   * True to disable selection within the DataView. This configuration will lock the selection model
   * that the DataView uses.
   * @default false
   */
  public native function get disableSelection():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set disableSelection(value:Boolean):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The text to display in the view when there is no data to display.
   * Note that when using local data the emptyText will not be displayed unless you set
   * the <a href="#!/api/Ext.view.AbstractView-cfg-deferEmptyText" rel="Ext.view.AbstractView-cfg-deferEmptyText" class="docClass">deferEmptyText</a> option to false.
   * @default ""
   * @since 2.3.0
   */
  public native function get emptyText():String;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set emptyText(value:String):void;

  [ExtConfig]
  /**
   * Specifies the class to be assigned to each element in the view when used in conjunction with the
   * <a href="#!/api/Ext.view.AbstractView-cfg-itemTpl" rel="Ext.view.AbstractView-cfg-itemTpl" class="docClass">itemTpl</a> configuration.
   * @default Ext.baseCSSPrefix + 'dataview-item'
   * @since 2.3.0
   */
  public native function get itemCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set itemCls(value:String):void;

  [ExtConfig]
  /**
   * <b>This is a required setting</b>. A simple CSS selector (e.g. <code>div.some-class</code> or
   * <code>span:first-child</code>) that will be used to determine what nodes this DataView will be
   * working with. The itemSelector is used to map DOM nodes to records. As such, there should
   * only be one root level element that matches the selector for each record. The itemSelector
   * will be automatically configured if the <a href="#!/api/Ext.view.AbstractView-cfg-itemTpl" rel="Ext.view.AbstractView-cfg-itemTpl" class="docClass">itemTpl</a> config is used.
   *
   * <pre><code>new <a href="#!/api/Ext.view.View" rel="Ext.view.View" class="docClass">Ext.view.View</a>({
   *     renderTo: <a href="#!/api/Ext-method-getBody" rel="Ext-method-getBody" class="docClass">Ext.getBody</a>(),
   *     store: {
   *         fields: ['name'],
   *         data: [
   *             {name: 'Item 1'},
   *             {name: 'Item 2'}
   *         ]
   *     },
   *     tpl: [
   *         '&lt;ul&gt;',
   *         '&lt;tpl for="."&gt;',
   *             '&lt;li&gt;{name}&lt;/li&gt;',
   *         '&lt;/tpl&gt;',
   *         '&lt;/ul&gt;'
   *     ],
   *     // Match the li, since each one maps to a record
   *     itemSelector: 'li'
   * });
   * </code></pre>
   * @since 2.3.0
   */
  public native function get itemSelector():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set itemSelector(value:String):void;

  [ExtConfig]
  /**
   * The inner portion of the item template to be rendered. Follows an XTemplate
   * structure and will be placed inside of a tpl.
   */
  public native function get itemTpl():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set itemTpl(value:*):void;

  [ExtConfig]
  /**
   * False to disable a load mask from displaying while the view is loading. This can also be a
   * <a href="#!/api/Ext.LoadMask" rel="Ext.LoadMask" class="docClass">Ext.LoadMask</a> configuration object.
   * @default true
   */
  public native function get loadMask():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set loadMask(value:*):void;

  [ExtConfig]
  /**
   * The CSS class to apply to the loading message element. Defaults to Ext.LoadMask.prototype.msgCls "x-mask-loading".
   */
  public native function get loadingCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set loadingCls(value:String):void;

  [ExtConfig]
  /**
   * If specified, gives an explicit height for the data view when it is showing the <a href="#!/api/Ext.view.AbstractView-cfg-loadingText" rel="Ext.view.AbstractView-cfg-loadingText" class="docClass">loadingText</a>,
   * if that is specified. This is useful to prevent the view's height from collapsing to zero when the
   * loading mask is applied and there are no other contents in the data view.
   */
  public native function get loadingHeight():Number;

  [ExtConfig]
  /**
   * @private
   */
  public native function set loadingHeight(value:Number):void;

  [ExtConfig]
  /**
   * A string to display during data load operations.  If specified, this text will be
   * displayed in a loading div and the view's contents will be cleared while loading, otherwise the view's
   * contents will continue to display normally until the new data is loaded and the contents are replaced.
   * @default 'Loading...'
   * @since 2.3.0
   */
  public native function get loadingText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set loadingText(value:String):void;

  [Deprecated(replacement = "mode", since = "4.0")]
  [ExtConfig]
  /**
   * True to allow selection of more than one item at a time, false to allow selection of only a single item
   * at a time or no selection at all, depending on the value of <a href="#!/api/Ext.view.AbstractView-cfg-singleSelect" rel="Ext.view.AbstractView-cfg-singleSelect" class="docClass">singleSelect</a>.
   * @default false
   * @since 2.3.0
   */
  public native function get multiSelect():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set multiSelect(value:Boolean):void;

  [ExtConfig]
  /**
   * A CSS class to apply to each item in the view on mouseover.
   * Setting this will automatically set <a href="#!/api/Ext.view.AbstractView-cfg-trackOver" rel="Ext.view.AbstractView-cfg-trackOver" class="docClass">trackOver</a> to <code>true</code>.
   */
  public native function get overItemCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set overItemCls(value:String):void;

  [ExtConfig]
  /**
   * True to preserve scroll position across refresh operations.
   * @default false
   */
  public native function get preserveScrollOnRefresh():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set preserveScrollOnRefresh(value:Boolean):void;

  [ExtConfig]
  /**
   * True to preserve scroll position when the store is reloaded.
   *
   * <p>You may want to configure this as <code>true</code> if you are using a <a href="#!/api/Ext.data.BufferedStore" rel="Ext.data.BufferedStore" class="docClass">buffered store</a>
   * and you require refreshes of the client side data state not to disturb the state of the UI.</p>
   * @default false
   * @since 5.1.1
   */
  public native function get preserveScrollOnReload():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set preserveScrollOnReload(value:Boolean):void;

  [ExtConfig]
  /**
   * A CSS class to apply to each selected item in the view.
   * @default Ext.baseCSSPrefix + 'item-selected'
   */
  public native function get selectedItemCls():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set selectedItemCls(value:String):void;

  [ExtConfig]
  /**
   * The selected model. Typically used with <a href="#!/api/Ext.view.AbstractView-cfg-bind" rel="Ext.view.AbstractView-cfg-bind" class="docClass">binding</a>.
   * @see #setSelection()
   */
  public native function get selection():ext.data.Model;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set selection(value:ext.data.Model):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * The <a href="#!/api/Ext.selection.Model" rel="Ext.selection.Model" class="docClass">selection model</a> [dataviewmodel] config or alias to use.
   * @default {type: 'dataviewmodel'}
   * @since 5.1.0
   * @see #getSelectionModel()
   * @see #setSelectionModel()
   */
  public native function get selectionModel():ext.selection.SelectionModel;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set selectionModel(value:ext.selection.SelectionModel):void;

  [Deprecated(replacement = "mode", since = "4.0")]
  [ExtConfig]
  /**
   * True to enable multiselection by clicking on multiple items without requiring the user to hold Shift or Ctrl,
   * false to force the user to hold Ctrl or Shift to select more than on item.
   * @default false
   * @since 2.3.0
   */
  public native function get simpleSelect():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set simpleSelect(value:Boolean):void;

  [ExtConfig]
  /**
   * Allows selection of exactly one item at a time. As this is the default selection mode anyway, this config
   * is completely ignored.
   * @since 2.3.0
   */
  public native function get singleSelect():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set singleSelect(value:Boolean):void;

  [ExtConfig]
  /**
   * The <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a> to bind this DataView to.
   * @default 'ext-empty-store'
   * @since 2.3.0
   * @see #setStore()
   */
  public native function get store():ext.data.Store;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set store(value:ext.data.Store):void;

  [ExtConfig]
  /**
   * Configure as <code>true</code> to have this view participate in the global throttled update queue which flushes store changes to the UI at a maximum rate
   * determined by the <a href="#!/api/Ext.view.AbstractView-static-cfg-updateDelay" rel="Ext.view.AbstractView-static-cfg-updateDelay" class="docClass">updateDelay</a> setting.
   * @default false
   */
  public native function get throttledUpdate():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set throttledUpdate(value:Boolean):void;

  [ExtConfig]
  /**
   * When <code>true</code> the <a href="#!/api/Ext.view.AbstractView-cfg-overItemCls" rel="Ext.view.AbstractView-cfg-overItemCls" class="docClass">overItemCls</a> will be applied to items when hovered over.
   * This in return will also cause <a href="#!/api/Ext.view.View-event-highlightitem" rel="Ext.view.View-event-highlightitem" class="docClass">highlightitem</a> and
   * <a href="#!/api/Ext.view.View-event-unhighlightitem" rel="Ext.view.View-event-unhighlightitem" class="docClass">unhighlightitem</a> events to be fired.
   *
   * <p>Enabled automatically when the <a href="#!/api/Ext.view.AbstractView-cfg-overItemCls" rel="Ext.view.AbstractView-cfg-overItemCls" class="docClass">overItemCls</a> config is set.</p>
   * @default false
   * @since 2.3.0
   */
  public native function get trackOver():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set trackOver(value:Boolean):void;

  /**
   * @inheritDoc
   */
  public native function get autoDestroyBoundStore():Boolean;

  /**
   * @inheritDoc
   */
  public native function set autoDestroyBoundStore(value:Boolean):void;

  /**
   * Adds a CSS Class to a specific item.
   * @param itemInfo An HTMLElement, index or instance of a model
   * representing this item
   */
  public native function addItemCls(itemInfo:*, cls:String):void;

  /**
   * Changes the data store bound to this view and refreshes it.
   * @since 3.4.0
   * @param store The store to bind or ID of the store.
   * When no store given (or when <code>null</code> or <code>undefined</code> passed), unbinds the existing store.
   */
  public native function bindStore(store:ext.data.Store = undefined):void;

  [Deprecated(replacement = "deselectAll", since = "4.0")]
  /**
   * Deselects all selected records.
   * @since 2.3.0
   */
  public native function clearSelections():void;

  /**
   * Function which can be overridden which returns the data object passed to this
   * DataView's <a href="#!/api/Ext.view.AbstractView-cfg-tpl" rel="Ext.view.AbstractView-cfg-tpl" class="docClass">template</a> to render the whole DataView.
   *
   * <p>This is usually an Array of data objects, each element of which is processed by an
   * <a href="#!/api/Ext.XTemplate" rel="Ext.XTemplate" class="docClass">XTemplate</a> which uses <code>'&amp;lt;tpl for="."&amp;gt;'</code> to iterate over its supplied
   * data object as an Array. However, <i>named</i> properties may be placed into the data object to
   * provide non-repeating data such as headings, totals etc.</p>
   * @since 2.3.0
   * @param records An Array of <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a>s to be rendered into the DataView.
   * @param startIndex the index number of the Record being prepared for rendering.
   * @return An Array of data objects to be processed by a repeating XTemplate. May also
   * contain <i>named</i> properties.
   */
  public native function collectData(records:Array, startIndex:Number):Array;

  /**
   * Deselects a record instance by record instance or index.
   * @since 2.3.0
   * @param records An array of records or an index
   * @param suppressEvent Set to false to not fire a deselect event
   */
  public native function deselect(records:*, suppressEvent:Boolean):void;

  /**
   * Returns the template node the passed child belongs to, or null if it doesn't belong to one.
   * @return The template node
   */
  public native function findItemByChild(node:js.HTMLElement):js.HTMLElement;

  /**
   * Returns the template node by the <a href="#!/api/Ext.event.Event" rel="Ext.event.Event" class="docClass">Ext.event.Event</a> or null if it is not found.
   */
  public native function findTargetByEvent(e:ext.event.Event):void;

  /**
   * Gets a template node.
   * @since 2.3.0
   * @param nodeInfo An HTMLElement template node, index of a template node,
   * the id of a template node or the record associated with the node.
   * @return The node or null if it wasn't found
   */
  public native function getNode(nodeInfo:*):js.HTMLElement;

  /**
   * Returns a CSS selector which selects the element which contains record nodes.
   */
  public native function getNodeContainerSelector():String;

  /**
   * Gets a range nodes.
   * @since 2.3.0
   * @param start The index of the first node in the range
   * @param end The index of the last node in the range
   * @return An array of nodes
   */
  public native function getNodes(start:Number = NaN, end:Number = NaN):Array;

  /**
   * Gets a record from a node
   * @since 2.3.0
   * @param node The node to evaluate
   * @return record The <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a> object
   */
  public native function getRecord(node:js.HTMLElement):ext.data.Model;

  /**
   * Gets an array of the records from an array of nodes
   * @since 2.3.0
   * @param nodes The nodes to evaluate
   * @return records The <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a> objects
   */
  public native function getRecords(nodes:Array):Array;

  /**
   * Gets the currently selected nodes.
   * @since 2.3.0
   * @return An array of HTMLElements
   */
  public native function getSelectedNodes():Array;

  [Deprecated(replacement = "getSelection", since = "4.0")]
  /**
   * Gets an array of the selected records
   * @since 2.3.0
   * @return An array of <a href="#!/api/Ext.data.Model" rel="Ext.data.Model" class="docClass">Ext.data.Model</a> objects
   */
  public native function getSelectedRecords():Array;

  [Deprecated(replacement = "getCount", since = "4.0")]
  /**
   * Gets the number of selected nodes.
   * @since 2.3.0
   * @return The node count
   */
  public native function getSelectionCount():Number;

  /**
   * Returns the value of <a href="#!/api/Ext.view.AbstractView-cfg-selectionModel" rel="Ext.view.AbstractView-cfg-selectionModel" class="docClass">selectionModel</a>.
   * @since 5.1.0
   */
  public native function getSelectionModel():ext.selection.SelectionModel;

  /**
   * Returns the value of <a href="#!/api/Ext.view.AbstractView-cfg-store" rel="Ext.view.AbstractView-cfg-store" class="docClass">store</a>.
   * @since 2.3.0
   */
  public native function getStore():ext.data.AbstractStore;

  /**
   * Finds the index of the passed node.
   * @since 2.3.0
   * @param nodeInfo An HTMLElement template node, index of a template node, the id of a template node
   * or a record associated with a node.
   * @return The index of the node or -1
   */
  public native function indexOf(nodeInfo:*):Number;

  /**
   * Returns true if the passed node is selected, else false.
   * @since 2.3.0
   * @param node The node, node index or record to check
   * @return True if selected, else false
   */
  public native function isSelected(node:*):Boolean;

  /**
   * Function which can be overridden to provide custom formatting for each Record that is used by this
   * DataView's <a href="#!/api/Ext.view.AbstractView-cfg-tpl" rel="Ext.view.AbstractView-cfg-tpl" class="docClass">template</a> to render each node.
   * @since 2.3.0
   * @param data The raw data object that was used to create the Record.
   * @param recordIndex the index number of the Record being prepared for rendering.
   * @param record The Record being prepared for rendering.
   * @return The formatted data in a format expected by the internal <a href="#!/api/Ext.view.AbstractView-cfg-tpl" rel="Ext.view.AbstractView-cfg-tpl" class="docClass">template</a>'s overwrite() method.
   * (either an array if your params are numeric (i.e. {0}) or an object (i.e. {foo: 'bar'}))
   */
  public native function prepareData(data:*, recordIndex:Number, record:ext.data.Model):*;

  /**
   * Refreshes the view by reloading the data from the store and re-rendering the template.
   *
   * <p><strong>Note:</strong> This method should only be used when <code>bufferedRenderer</code> is set to <code>false</code>.  BufferedRender
   * has its own methods for managing its data's state.</p>
   * @since 2.3.0
   */
  public native function refresh():void;

  /**
   * Refreshes an individual node's data from the store.
   * @since 2.3.0
   * @param record The record or index of the record to update.
   */
  public native function refreshNode(record:Number):void;

  /**
   * Removes a CSS Class from a specific item.
   * @param itemInfo An HTMLElement, index or instance of a model
   * representing this item
   */
  public native function removeItemCls(itemInfo:*, cls:String):void;

  [Deprecated(replacement = "select", since = "4.0")]
  /**
   * Selects a record instance by record instance or index.
   * @since 2.3.0
   * @param records An array of records or an index
   * @param suppressEvent Set to false to not fire a select event
   */
  public native function select(records:*, keepExisting:Boolean = false, suppressEvent:Boolean = false):void;

  /**
   * Sets the value of <a href="#!/api/Ext.view.AbstractView-cfg-selection" rel="Ext.view.AbstractView-cfg-selection" class="docClass">selection</a>.
   * @param selection The new value.
   */
  public native function setSelection(selection:ext.data.Model):void;

  /**
   * Sets the value of <a href="#!/api/Ext.view.AbstractView-cfg-selectionModel" rel="Ext.view.AbstractView-cfg-selectionModel" class="docClass">selectionModel</a>.
   * @since 5.1.0
   * @param selectionModel The new value.
   */
  public native function setSelectionModel(selectionModel:*):void;

  /**
   * @inheritDoc
   */
  public native function setStore(store:Object):void;

  /**
   * Binds listeners for this component to the store. By default it will add
   * anything bound by the getStoreListeners method, however it can be overridden
   * in a subclass to provide any more complicated handling.
   * @param store The store to bind to
   */
  protected native function bindStoreListeners(store:ext.data.AbstractStore):void;

  /**
   * Unbinds listeners from this component to the store. By default it will remove
   * anything bound by the bindStoreListeners method, however it can be overridden
   * in a subclass to provide any more complicated handling.
   * @param store The store to unbind from
   */
  protected native function unbindStoreListeners(store:ext.data.AbstractStore):void;
}
}