package ext.view {
import ext.data.Store;
import ext.panel.Panel;

[Uses("ext.panel.Panel")]
[Uses("ext.container.DockingContainer")]
[Uses("ext.Base")]
[Uses("ext.container.Container")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.mixin.Container")]
[Uses("ext.Mixin")]
[Uses("ext.Component")]
[Uses("ext.util.Positionable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Animate")]
[Uses("ext.util.Floating")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.state.Stateful")]
[Native("Ext.view.MultiSelectorSearch", require)]
/**
 * This component provides a grid holding selected items from a second store of potential
 * members. The <code>store</code> of this component represents the selected items. The <code>searchStore</code>
 * represents the potentially selected items.
 *
 * <p>The default view defined by this class is intended to be easily replaced by deriving a
 * new class and overriding the appropriate methods. For example, the following is a very
 * different view that uses a date range and a data view:</p>
 *
 * <pre><code> <a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('App.view.DateBoundSearch', {
 *      extend: '<a href="#!/api/Ext.view.MultiSelectorSearch" rel="Ext.view.MultiSelectorSearch" class="docClass">Ext.view.MultiSelectorSearch</a>',
 *
 *      makeDockedItems: function () {
 *          return {
 *              xtype: 'toolbar',
 *              items: [{
 *                  xtype: 'datefield',
 *                  emptyText: 'Start date...',
 *                  flex: 1
 *              },{
 *                  xtype: 'datefield',
 *                  emptyText: 'End date...',
 *                  flex: 1
 *              }]
 *          };
 *      },
 *
 *      makeItems: function () {
 *          return [{
 *              xtype: 'dataview',
 *              itemSelector: '.search-item',
 *              selModel: 'rowselection',
 *              store: this.store,
 *              scrollable: true,
 *              tpl:
 *                  '&lt;tpl for="."&gt;' +
 *                      '&lt;div class="search-item"&gt;' +
 *                          '&lt;img src="{icon}"&gt;' +
 *                          '&lt;div&gt;{name}&lt;/div&gt;' +
 *                      '&lt;/div&gt;' +
 *                  '&lt;/tpl&gt;'
 *          }];
 *      },
 *
 *      getSearchStore: function () {
 *          return this.items.getAt(0).getStore();
 *      },
 *
 *      selectRecords: function (records) {
 *          var view = this.items.getAt(0);
 *          return view.getSelectionModel().select(records);
 *      }
 *  });
 * </code></pre>
 *
 * <p><strong>Important</strong>: This class assumes there are two components with specific <code>reference</code>
 * names assigned to them. These are <code>"searchField"</code> and <code>"searchGrid"</code>. These components
 * are produced by the <code>makeDockedItems</code> and <code>makeItems</code> method, respectively. When
 * overriding these it is important to remember to place these <code>reference</code> values on the
 * appropriate components.</p>
 */
public class MultiSelectorSearchView extends ext.panel.Panel {
  /**
   * @param config @inheritDoc
   */
  public function MultiSelectorSearchView(config:ext.view.MultiSelectorSearchView = null) {
    super();
  }

  [ExtConfig]
  /**
   * A field from your grid's store that will be used for filtering your search results.
   */
  public native function get field():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set field(value:String):void;

  [ExtConfig]
  /**
   * This text is displayed as the "emptyText" of the search <code>textfield</code>.
   * @default 'Search...'
   */
  public native function get searchText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set searchText(value:String):void;

  [ExtConfig]
  /**
   * The data source to which the grid / tree is bound. Acceptable values for this
   * property are:
   *
   * <ul>
   * <li><strong>any <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Store</a> class / subclass</strong></li>
   * <li><strong>an <a href="#!/api/Ext.data.Store-cfg-storeId" rel="Ext.data.Store-cfg-storeId" class="docClass">ID of a store</a></strong></li>
   * <li><strong>a <a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Store</a> config object</strong>.  When passing a config you can
   * specify the store type by alias.  Passing a config object with a store type will
   * dynamically create a new store of that type when the grid / tree is instantiated.</li>
   * </ul>
   *
   *
   * <p>For example:</p>
   *
   * <pre><code><a href="#!/api/Ext-method-define" rel="Ext-method-define" class="docClass">Ext.define</a>('MyApp.store.Customers', {
   *     extend: '<a href="#!/api/Ext.data.Store" rel="Ext.data.Store" class="docClass">Ext.data.Store</a>',
   *     alias: 'store.customerstore',
   *     fields: ['name']
   * });
   *
   * <a href="#!/api/Ext-method-create" rel="Ext-method-create" class="docClass">Ext.create</a>({
   *     xtype: 'gridpanel',
   *     renderTo: document.body,
   *     store: {
   *         type: 'customerstore',
   *         data: [{
   *             name: 'Foo'
   *         }]
   *     },
   *     columns: [{
   *         text: 'Name',
   *         dataIndex: 'name'
   *     }]
   * });
   * </code></pre>
   */
  public native function get store():*;

  [ExtConfig]
  /**
   * @private
   */
  public native function set store(value:*):void;

  /**
   * Returns the store that holds search results. By default this comes from the
   * "search grid". If this aspect of the view is changed sufficiently so that the
   * search grid cannot be found, this method should be overridden to return the proper
   * store.
   */
  public native function getSearchStore():ext.data.Store;
}
}