package ext.view {
import ext.grid.GridPanel;

[Uses("ext.grid.GridPanel")]
[Uses("ext.panel.TablePanel")]
[Uses("ext.grid.locking.Lockable")]
[Uses("ext.Base")]
[Uses("ext.panel.Panel")]
[Uses("ext.container.DockingContainer")]
[Uses("ext.container.Container")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.mixin.Container")]
[Uses("ext.Mixin")]
[Uses("ext.Component")]
[Uses("ext.util.Positionable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Animate")]
[Uses("ext.util.Floating")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.state.Stateful")]
[Native("Ext.view.MultiSelector", require)]
/**
 * This component provides a grid holding selected items from a second store of potential
 * members. The <code>store</code> of this component represents the selected items. The "search store"
 * represents the potentially selected items.
 *
 * <p>While this component is a grid and so you can configure <code>columns</code>, it is best to leave
 * that to this class in its <code>initComponent</code> method. That allows this class to create the
 * extra column that allows the user to remove rows. Instead use <code><a href="#!/api/Ext.view.MultiSelector-cfg-fieldName" rel="Ext.view.MultiSelector-cfg-fieldName" class="docClass">fieldName</a></code> and
 * <code><a href="#!/api/Ext.view.MultiSelector-cfg-fieldTitle" rel="Ext.view.MultiSelector-cfg-fieldTitle" class="docClass">fieldTitle</a></code> to configure the primary column's <code>dataIndex</code> and column <code>text</code>,
 * respectively.</p>
 */
public class MultiSelectorView extends ext.grid.GridPanel {
  /**
   * @param config @inheritDoc
   */
  public function MultiSelectorView(config:ext.view.MultiSelectorView = null) {
    super();
  }

  [ExtConfig]
  /**
   * The tooltip to display when the user hovers over the "+" tool in the panel header.
   * @default 'Search for items to add'
   * @since 5.0.0
   */
  public native function get addToolText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set addToolText(value:String):void;

  [ExtConfig]
  /**
   * The name of the data field to display in the primary column of the grid.
   * @default "name"
   * @since 5.0.0
   */
  public native function get fieldName():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set fieldName(value:String):void;

  [ExtConfig]
  /**
   * The text to display in the column header for the primary column of the grid.
   * @since 5.0.0
   */
  public native function get fieldTitle():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set fieldTitle(value:String):void;

  [ExtConfig]
  /**
   * The text to display in the "remove this row" column. By default this is a Unicode
   * "X" looking glyph.
   * @default '\u2716'
   * @since 5.0.0
   */
  public native function get removeRowText():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set removeRowText(value:String):void;

  [ExtConfig]
  /**
   * The tooltip to display when the user hovers over the remove cell.
   * @default 'Remove this item'
   * @since 5.0.0
   */
  public native function get removeRowTip():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set removeRowTip(value:String):void;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * This object configures the search popup component. By default this contains the
   * <code>xtype</code> for a <code><a href="#!/api/Ext.view.MultiSelectorSearch" rel="Ext.view.MultiSelectorSearch" class="docClass">Ext.view.MultiSelectorSearch</a></code> component and specifies <code>autoLoad</code>
   * for its <code>store</code>.
   * @default {xtype: 'multiselector-search', width: 200, height: 200, store: {autoLoad: true}}
   * @see #getSearch()
   * @see #setSearch()
   */
  public native function get search():Object;

  [ExtConfig]
  [Bindable(event = "DUMMY")]
  /**
   * @private
   */
  public native function set search(value:Object):void;

  /**
   * Returns the value of <a href="#!/api/Ext.view.MultiSelector-cfg-search" rel="Ext.view.MultiSelector-cfg-search" class="docClass">search</a>.
   */
  public native function getSearch():Object;

  /**
   * Sets the value of <a href="#!/api/Ext.view.MultiSelector-cfg-search" rel="Ext.view.MultiSelector-cfg-search" class="docClass">search</a>.
   * @param search The new value.
   */
  public native function setSearch(search:Object):void;
}
}