package ext.window {

[Uses("ext.window.Window")]
[Uses("ext.panel.Panel")]
[Uses("ext.container.DockingContainer")]
[Uses("ext.Base")]
[Uses("ext.container.Container")]
[Uses("ext.mixin.Queryable")]
[Uses("ext.mixin.Container")]
[Uses("ext.Mixin")]
[Uses("ext.Component")]
[Uses("ext.util.Positionable")]
[Uses("ext.util.Focusable")]
[Uses("ext.util.ElementContainer")]
[Uses("ext.mixin.Bindable")]
[Uses("ext.mixin.Observable")]
[Uses("ext.util.Animate")]
[Uses("ext.util.Floating")]
[Uses("ext.mixin.Inheritable")]
[Uses("ext.mixin.ComponentDelegation")]
[Uses("ext.mixin.Keyboard")]
[Uses("ext.util.Renderable")]
[Uses("ext.mixin.Accessible")]
[Uses("ext.state.Stateful")]
[Native("Ext.window.Toast", require)]
/**
 * This class provides for lightweight, auto-dismissing pop-up notifications called "toasts".
 * At the base level, you can display a toast message by calling <code>Ext.toast</code> like so:
 *
 * <pre><code> Ext.toast('Data saved');
 * </code></pre>
 *
 * <p>This will result in a toast message, which displays in the default location of bottom right in your viewport.</p>
 *
 * <p>You may expand upon this simple example with the following parameters:</p>
 *
 * <pre><code> Ext.toast(message, title, align, iconCls);
 * </code></pre>
 *
 * <p>For example, the following toast will appear top-middle in your viewport.  It will display
 * the 'Data Saved' message with a title of 'Title'</p>
 *
 * <pre><code> Ext.toast('Data Saved', 'Title', 't')
 * </code></pre>
 *
 * <p>It should be noted that the toast's width is determined by the message's width.
 * If you need to set a specific width, or any of the other available configurations for your toast,
 * you can create the toast object as seen below:</p>
 *
 * <pre><code> Ext.toast({
 *      html: 'Data Saved',
 *      title: 'My Title',
 *      width: 200,
 *      align: 't'
 *  });
 * </code></pre>
 *
 * <p>This component is derived from the excellent work of a Sencha community member, Eirik
 * Lorentsen.</p>
 */
public class Toast extends ext.window.Window {
  /**
   * @param config @inheritDoc
   */
  public function Toast(config:ext.window.Toast = null) {
    super();
  }

  [ExtConfig]
  /**
   * The component or the <code>id</code> of the component to which the <code>toast</code> will be anchored.
   * The default behavior is to anchor a <code>toast</code> to the document body (no component).
   */
  override public native function get anchor():String;

  [ExtConfig]
  /**
   * @private
   */
  override public native function set anchor(value:String):void;

  [ExtConfig]
  /**
   * Specifies the basic alignment of the toast message with its <a href="#!/api/Ext.window.Toast-cfg-anchor" rel="Ext.window.Toast-cfg-anchor" class="docClass">anchor</a>. This
   * controls many aspects of the toast animation as well. For fine grain control of
   * the final placement of the toast and its <code>anchor</code> you may set
   * <a href="#!/api/Ext.window.Toast-cfg-anchorAlign" rel="Ext.window.Toast-cfg-anchorAlign" class="docClass">anchorAlign</a> as well.
   *
   * <p>Possible values:</p>
   *
   * <ul>
   * <li>br - bottom-right</li>
   * <li>bl - bottom-left</li>
   * <li>tr - top-right</li>
   * <li>tl - top-left</li>
   * <li>t  - top</li>
   * <li>l  - left</li>
   * <li>b  - bottom</li>
   * <li>r  - right</li>
   * </ul>
   * @default 't'
   */
  public native function get align():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set align(value:String):void;

  [ExtConfig]
  /**
   * This string is a full specification of how to position the toast with respect to
   * its <code>anchor</code>. This is set to a reasonable value based on <code>align</code> but the <code>align</code>
   * also sets defaults for various other properties. This config controls only the
   * final position of the toast.
   */
  public native function get anchorAlign():String;

  [ExtConfig]
  /**
   * @private
   */
  public native function set anchorAlign(value:String):void;

  [ExtConfig]
  /**
   * Set this to <code>false</code> to make toasts appear and disappear without animation.
   * This is helpful with applications' unit and integration testing.
   * @default true
   */
  public native function get animated():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set animated(value:Boolean):void;

  [ExtConfig]
  /**
   * This config will prevent the Toast from closing when you click on it. If this is set to <code>true</code>,
   * closing the Toast will have to be handled some other way (e.g., Setting <code>closable: true</code>).
   * @default false
   */
  public native function get stickOnClick():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set stickOnClick(value:Boolean):void;

  [ExtConfig]
  /**
   * Directs the toast message to animate on the x-axis (if <code>true</code>) or y-axis (if <code>false</code>).
   * This value defaults to a value based on the <code>align</code> config.
   * @default false
   */
  public native function get useXAxis():Boolean;

  [ExtConfig]
  /**
   * @private
   */
  public native function set useXAxis(value:Boolean):void;
}
}